/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.autoqualitycontrol;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.TreeSet;
import javax.swing.JFrame;
import org.spectrim.model.GlobalModel;

/**
 *
 * @author Giezi
 */
public class LabellingHandler {

    private GlobalModel model = null;
    private TreeMap<Integer, String> unlabelledVoxels;
    private LabellingPopupInterface labellingFrame;

    public LabellingHandler(GlobalModel model) {
        this.model = model;
    }

    public void labelSpectra(LabellingListener listener, File startFolder, File referenceFile, int mode, JFrame parent) {
        HashMap<String, ArrayList<Double>> features = new HashMap<>();
        unlabelledVoxels = getVoxelIndex(model.getSelectedVoxels());

//        String coord;
//        String[] coords;
        String coord = null;
        if (mode != GlobalModel.LABELLING_ITERATIVE) {
            for (String coordIter : unlabelledVoxels.values()) {
                //            coord = unlabelledVoxels.get(i);
                if (mode == GlobalModel.LABELLING_QUAL) {
                    features.put(coordIter, model.getSpectra().getSpectralQualityFeatures(coordIter));
                }
            }
            coord = unlabelledVoxels.firstEntry().getValue();
            String[] coords = coord.split("_");
            model.selectVoxel(Integer.valueOf(coords[0]), Integer.valueOf(coords[1]), Integer.valueOf(coords[2]), GlobalModel.SELECTION_SINGLE);
            model.getImageViewer().setMouseSelectionActive(false);
        }

        if (labellingFrame != null) {
            labellingFrame.dispose();
        }

        if (mode == GlobalModel.LABELLING_QUAL) {
            labellingFrame = new PopupLabelling(listener, features, model.getPatientName() + "\\" + model.getStudyDate() + "\\" + model.getSeriesNumber() + "-" + model.getSeriesDescription(), unlabelledVoxels.size(), startFolder);
            labellingFrame.initialization();
        } else if (mode == GlobalModel.LABELLING_CONSENSUS) {
            labellingFrame = new ConsensusLabellingAssistant(listener, startFolder, referenceFile, model);
            labellingFrame.initialization();
        } else if (mode == GlobalModel.LABELLING_ITERATIVE) {
            labellingFrame = new MismatchLabellingAssistant(listener, startFolder, referenceFile, model, parent);
            labellingFrame.initialization();
        }
        if (mode != GlobalModel.LABELLING_ITERATIVE) {
            labellingFrame.askLabel(coord);
            unlabelledVoxels.remove(unlabelledVoxels.firstKey());
            labellingFrame.setLocationRelTo(parent);
        }

    }

    public void nextLabel() {
        String coord = unlabelledVoxels.firstEntry().getValue();
        String[] coords = coord.split("_");
        model.selectVoxel(Integer.valueOf(coords[0]), Integer.valueOf(coords[1]), Integer.valueOf(coords[2]), GlobalModel.SELECTION_SINGLE);
        labellingFrame.askLabel(coord);
        unlabelledVoxels.remove(unlabelledVoxels.firstKey());
    }

    public void finish() {
        unlabelledVoxels.clear();
        model.getImageViewer().setMouseSelectionActive(true);
    }

    private TreeMap<Integer, String> getVoxelIndex(ArrayList<String> vxls) {
        TreeMap<Integer, String> indexedVoxels = new TreeMap<>();
        for (String v : vxls) {
            String[] c = v.split("_");
            indexedVoxels.put(Integer.parseInt(c[0]) * 10000 + Integer.parseInt(c[1]) * 100 + Integer.parseInt(c[2]), v);
        }
        return indexedVoxels;
    }

    public void setUnlabelledVoxels(TreeSet<String> set) {
        this.unlabelledVoxels.clear();
        this.unlabelledVoxels = getVoxelIndex(new ArrayList<>(set));
        String coord;
        String[] coords;
        model.getImageViewer().setMouseSelectionActive(false);
        coord = unlabelledVoxels.firstEntry().getValue();
        coords = coord.split("_");
        model.selectVoxel(Integer.valueOf(coords[0]), Integer.valueOf(coords[1]), Integer.valueOf(coords[2]), GlobalModel.SELECTION_SINGLE);
        labellingFrame.askLabel(coord);
        unlabelledVoxels.remove(unlabelledVoxels.firstKey());
    }
}
