/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.autoqualitycontrol;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import rcaller.RCaller;
import rcaller.RCode;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.GlobalSettings;

/**
 *
 * @author nunopbarros
 */
public class QualityClassifier {

    private ArrayList<String> voxels = new ArrayList<>();
    private GlobalModel model;
    private String classifier;
    private HashMap<String, Double> qscores = new HashMap<>();

    public QualityClassifier(GlobalModel model) {
        this.model = model;
    }

    public void setVoxels(ArrayList<String> voxels) {
        this.voxels = (ArrayList<String>) voxels.clone();
    }

    public void setClassifier(String classifier) {
        this.classifier = classifier;
    }

    public void execute() {
        String coord;
        String text = "";

        ArrayList<String> names = model.getSpectra().getSpectralQualityFeaturesNames();
        text = "\"" + names.get(0) + "\"";
        for (int j = 1; j < names.size(); j++) {
            text = text + ";\"" + names.get(j) + "\"";
        }
        text = text + "\n";
        for (int i = 0; i < voxels.size(); i++) {
            coord = voxels.get(i);
            ArrayList<Double> vals = model.getSpectra().getSpectralQualityFeatures(coord);
            text = text + Double.toString(vals.get(0));
            for (int j = 1; j < vals.size(); j++) {
                text = text + ";" + vals.get(j);
            }
            text = text + "\n";
        }
        try {
            PrintWriter out = new PrintWriter("features_TEMP.csv");
            out.println(text);
            out.close();

            RCaller r_caller = new RCaller();

            // Define system to make program usable on linux
            boolean isLinux = System.getProperty("os.name").compareTo("Linux") == 0 ? true : false;
            r_caller.setRscriptExecutable(GlobalSettings.getInstance().getRExecutable());
            String separator = isLinux ? "/" : "\\\\";
            r_caller.cleanRCode();
            RCode r_code = new RCode();
             r_code.addRCode("if(!require(randomForest)){install.packages(\"randomForest\"); library(randomForest)}");
            r_code.addRCode("if(!require(Runiversal)){install.packages(\"Runiversal\"); library(Runiversal)}");
            r_code.addRCode("trainedforest <- load(\"" + classifier.replace("\\", separator) + "\")");
            r_code.addRCode("setwd(\"" + System.getProperty("user.dir").replace("\\", separator) + "\")");
            r_code.addRCode("data <- read.csv( \"features_TEMP.csv\", head=TRUE, sep=\";\")");
            r_code.addRCode("results <- predict(get(trainedforest),data,type=\"prob\")[,2]");
            r_caller.setRCode(r_code);
            r_caller.runAndReturnResult("results");
            double[] results = r_caller.getParser().getAsDoubleArray("results");
            for (int i = 0; i < results.length; i++) {
                qscores.put(voxels.get(i), results[i]);
            }
            File aux = new File("features_TEMP.csv");
            try {
                Files.delete(aux.toPath());
            } catch (IOException ex) {
                Logger.getLogger(QualityClassifier.class.getName()).log(Level.SEVERE, null, ex);
            }

        } catch (FileNotFoundException ex) {
            File aux = new File("features_TEMP.csv");
            try {
                Files.delete(aux.toPath());
            } catch (IOException ex2) {
                Logger.getLogger(QualityClassifier.class.getName()).log(Level.SEVERE, null, ex);
            }
            Logger.getLogger(QualityClassifier.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public HashMap<String, Double> getQualityScores() {
        return qscores;
    }
}
