/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.batchprocessing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;
import org.spectrim.settings.processing.QuantificationAction;

/**
 *
 * @author Nuno Barros // Johannes Slotboom
 */
public class BatchProcessing 
{

    private static BatchProcessingWorker worker;
    private static BatchProcessingTaskExecutor taskExecutor;
    public static final int APO_EXP = 0;
    public static final int APO_GAUSS = 1;
    public static final int SELECT_ALL = 0;
    public static final int SELECT_EXCITEDVOL = 1;
    public static final int SELECT_INNEREXCITEDVOL = 2;
    public static final int REPORT_PNG = 0;
    public static final int REPORT_DICOM = 1;

    public static String m_file;
    private static final Vector<ActionListener> m_actionListeners = new Vector<ActionListener>();

    public final static int THREAD_STARTED = 0;
    public final static int THREAD_FINISHED = 1;

    public final static String THREAD_STARTED_CMD = "<html><font color=\"green\">Processing started ..</font></html>";
    public final static String THREAD_FINISHED_CMD = "<html><font color=\"green\"><b>Processing finished successfully</b></font></html>";
    
    /**
     * Singleton class: the one and only BatchProcessing!
     */
    private BatchProcessing( ) 
    {
    }

    public static void setTaskExecutor( BatchProcessingTaskExecutor inst ) 
    {
        taskExecutor = inst;
    }

    public static boolean isBusy( ) 
    {
        return ( worker != null && !worker.isCancelled( ) && !worker.isDone( ) );
    }

    public static void resume( ) 
    {
        if ( worker != null ) 
        {
            worker.resume( );
        }
    }

    public static void run( ) 
    {
        if ( worker != null ) 
        {
            worker.execute( );
            // System.out.println( "##### ##### Swing worker started working on " + m_file );
            notifyActionListeners( BatchProcessing.class, THREAD_STARTED, THREAD_STARTED_CMD );
        }
    }

    public static void cancel( ) 
    {
        if ( worker != null ) 
        {
            worker.cancel( true );
        }
    }

    public static void workerFinished( ) 
    {
        worker = null;
        taskExecutor.processingFinished( );
        // System.out.println( "##### ##### Swing worker finished working on " + m_file );
        notifyActionListeners( BatchProcessing.class, THREAD_FINISHED, THREAD_FINISHED_CMD );
    }

    public static void addOpenImageFileCommand( String file ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addOpenImageFileCommand( file );
    }

    public static void addOpenSpectroscopyFileCommand( String file ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addOpenSpectroscopyFileCommand( file );
        m_file = file;
    }

    public static void addOpenReferencesCommand( String file1, String file2, String file3 ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addOpenReferencesCommand( file1, file2, file3 );
    }

    public static void addCreateReportCommand( String outputFile, 
                                               int outputFileType, 
                                               String customReportFile) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addCreateReportCommand( outputFile, outputFileType, customReportFile );
    }

    public static void addSaveResultsCommand( String outputFolder ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addSaveResultsCommand( outputFolder );
    }

    public static void addSaveMetaboliteMapsCommand( String outputFolder, boolean profileCorrection ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addSaveMetaboliteMapsCommand( outputFolder, profileCorrection );
    }

    public static void addApodization( int apoMethd, double factor ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addApodization( apoMethd, factor );
    }

    public static void addFrequencyShiftCorrection( double[] minvals, 
                                                    double[] maxvals, 
                                                    double[] targets, 
                                                    int[] multiplicities, 
                                                    double SNRLimit, 
                                                    double minPPM, 
                                                    double maxPPM, 
                                                    double maxLocalShift ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addFrequencyShiftCorrection( minvals, maxvals, 
                                            targets, multiplicities, 
                                            SNRLimit, minPPM, 
                                            maxPPM, maxLocalShift);
    }

    public static void addAutoPhasing( float minppm, float maxppm ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addAutoPhasing( minppm, maxppm );
    }

    public static void addAmplitudeOffsetRemovalFD( float minAmplitudeOffsetFD, 
                                                    float maxAmplitudeOffsetFD ) 
    {
        
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        
        worker.addAmplitudeOffsetRemovalFD( minAmplitudeOffsetFD, 
                                            maxAmplitudeOffsetFD );
    }
    
    public static void addAmplitudeOffsetRemovalTD( float minAmplitudeOffsetTD, 
                                                    float maxAmplitudeOffsetTD ) 
    {
        
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addAmplitudeOffsetRemovalTD( minAmplitudeOffsetTD, 
                                            maxAmplitudeOffsetTD );    
    }
    
    public static void addWaterPeakRemovalHLSVDPro( double min, double max, int nrpeaks ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addWaterPeakRemovalHLSVDPro( min, max, nrpeaks );
    }

    public static void addWaterPeakRemovalHLSVD64( double min, double max, int nrpeaks ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addWaterPeakRemovalHLSVD64( min, max, nrpeaks );
    }

    public static void addWaterPeakRemovalTDFDFit( String tdfdfitWaterModelName ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addWaterPeakRemovalTDFDFit( tdfdfitWaterModelName );
    }
    
    public static void _obsolete_addQuantification( ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addQuantification( );
    }

    public static void addQuantification(  QuantificationAction quantAction ) 
    {
        if ( quantAction != null )
        {
            if ( worker == null ) 
            {
                worker = new BatchProcessingWorker( taskExecutor );
            }
            if ( quantAction.getQuantificationType( ).compareTo( QuantificationAction.QUANTIFICATION_QUEST ) == 0 )
            {
                worker.addQuantification( );
            }
            
            if ( quantAction.getQuantificationType( ).compareTo( QuantificationAction.QUANTIFICATION_TDFDFIT ) == 0 )
            {
                worker.addQuantificationTDFDFit( quantAction.getTDFDFitQuantificationModelName( ), quantAction.getTDFDFitQuantificationModelPathName( ) );
            }
            
        }
    }
    
    
    
    public static void addQualityFiltering( String file, double threshold ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addQualityFiltering( file, threshold );
    }

    public static void addBaselineCorrection( int nrPoints ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addBaselineCorrection( nrPoints );
    }

    public static void addSelectionCommand( int selectMode ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addSelectionCommand( selectMode );
    }

    public static void addAutoPreprocessingCommand( double B0, double TE ) 
    {
        if ( worker == null ) 
        {
            worker = new BatchProcessingWorker( taskExecutor );
        }
        worker.addAutoPreprocessingCommand( B0, TE );
    }

    /**
     * Notifies actionEvent to all registered listeners.
     */
    private static void notifyActionListeners( Object obj, 
                                               int eventType, 
                                               String actionCommand ) 
    {
        Iterator it = m_actionListeners.iterator( );
        while ( it.hasNext( ) ) 
        {
            ( (ActionListener) it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    }

    /**
     * Adds an ActionListener to the list of listeners.
     *
     * @param listener
     */
    public static void addActionListener( ActionListener listener ) 
    {
        m_actionListeners.add( listener );
    }

    public static String getFile( ) 
    {
        return m_file;
    }
}
