/*
 * File: $Id: ClinicalSpectroscopyWorklistFrame.java,v 1.1.1.1 $
 * History
 * Date: 12.12.2016  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2016 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.clinicalcasehandling;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.JFrame;
import org.spectrim.automaticprocessing.AutomaticProcessingFileHandler;

/**
 *
 * @author slotboom
 */
public class ClinicalSpectroscopyWorklistFrame extends javax.swing.JFrame implements ActionListener
{

    /** 
     * Creates new form DicomSpectroWorklistFrame
     */
    private ClinicalSpectroscopyWorklistFrame( JFrame parent )
    {
        initComponents( );

        // Link the events of the AutomaticProcessingFileHandler to
        // the ClinicalSpectroscopyWorklistControl
        AutomaticProcessingFileHandler.getInstance( ).addActionListener( m_spectroscopyWorklistPanel );
        
        setTitle( "Spectroscopy Processing Worklist" );

        pack( );
        setLocationRelativeTo(null);
        setVisible(false);
    }

    @Override
    public void setVisible(boolean b) {
        
        m_spectroscopyWorklistPanel.syncTableContent( );
        
        super.setVisible(b); //To change body of generated methods, choose Tools | Templates.
    }
    /**
     * Returns the one and only instance of MRFittingDefaults
     * @return the one and only instance
     */
    public static ClinicalSpectroscopyWorklistFrame getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new ClinicalSpectroscopyWorklistFrame( null );
        }
        return theInstance;
    }
    
    
    /**
     * Sets the DicomSpectroWorklistDataElements of the
     * DicomSpectroIncomingWorklistControl
     * @param fileReferences
     */
    public synchronized void changeWorkListDataElement( ClinicalSpectroscopyCaseWorklistDataElement worklistElement )
    {
        m_spectroscopyWorklistPanel.changeSpectroWorklistDataElement( worklistElement );
    }
 

    /**
     * Returns the list DicomSpectroWorklistDataElements of the
     * DicomSpectroIncomingWorklistControl
     * @param
     */
    public Vector<ClinicalSpectroscopyCaseWorklistDataElement> getWorkListDataElements( )
    {
        return m_spectroscopyWorklistPanel.getAllSpectroWorklistDataElements( );
    }

    /**
     * Sets all elements of the table according to the Vector
     * @param worklistElements
     */
    public void addWorkListElement( ClinicalSpectroscopyCaseWorklistDataElement worklistElement )
    {
        m_spectroscopyWorklistPanel.addWorkListDataElement( worklistElement );
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_controlPanel = new javax.swing.JPanel();
        m_spectroscopyWorklistPanel = new org.spectrim.clinicalcasehandling.ClinicalSpectroscopyWorklistPanel();

        javax.swing.GroupLayout m_controlPanelLayout = new javax.swing.GroupLayout(m_controlPanel);
        m_controlPanel.setLayout(m_controlPanelLayout);
        m_controlPanelLayout.setHorizontalGroup(
            m_controlPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 1805, Short.MAX_VALUE)
        );
        m_controlPanelLayout.setVerticalGroup(
            m_controlPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(m_controlPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(m_spectroscopyWorklistPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(m_spectroscopyWorklistPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 397, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_controlPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Adds an actionlistener to the list of ActionListeners
     * @param 
     */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }

    /**
     * Notify all listeners that a change has taken place in this object
     */
    protected void notifyActionListeners( Object obj, int evType, String cmd )
    {
        Iterator it = m_actionListeners.iterator( );
        // notify registered listeners
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, evType, cmd ) );
        }
    }

    /**
     * Is called whenever a registered component raises an ActionEvent
     * @param arg0
     */
    public void actionPerformed( ActionEvent e )
    {
//        if ( e.getSource( ) == m_spectroscopyLoadingHandler )
//        {
//            if ( e.getID( ) == DicomSpectroIncomingWorklistControl.RECEIVED_DICOM_SPECTRO_FILE_TABLE_UPDATED_ID )
//            {
//                System.out.println( "DicomSpectroWorklistFrame :: incoming DICOM spectrum event" );
//                notifyActionListeners( this,
//                                       INCOMING_DICOM_SPECTRUM_FILE_EVENT,
//                                       INCOMING_DICOM_SPECTRUM_FILE_EVENT_COMMAND );
//            }
//            if ( e.getID( ) == DicomSpectroIncomingWorklistControl.RECEIVED_DICOM_IMAGE_FILE_TABLE_UPDATED_ID )
//            {
//                System.out.println( "DicomSpectroWorklistFrame :: incoming DICOM image event" );
//                notifyActionListeners( this,
//                                       INCOMING_DICOM_IMAGE_FILE_EVENT,
//                                       INCOMING_DICOM_IMAGE_FILE_EVENT_COMMAND );
//            }
//        }
//
//        if ( e.getActionCommand( ).compareTo( SiemensSpectroscopyLoadingHandler.UPDATE_WORK_TABLE_REQUEST_COMMAND ) == 0 )
//        {
//            if ( e.getID( ) == SiemensSpectroscopyLoadingHandler.UPDATE_WORK_TABLE_REQUEST_ID )
//            {
//                ClinicalSpectroscopyCaseIncomingWorklistDataElement elem = (ClinicalSpectroscopyCaseIncomingWorklistDataElement)e.getSource( );
//                changeWorkListDataElement( elem  );
//            }
//        }

        if ( e.getSource( ) instanceof ClinicalSpectroscopyWorklistPanel )
        {
            if ( e.getActionCommand( ).compareTo( ClinicalSpectroscopyWorklistPanel.RECEIVED_CSI_SPECTROSCOPY_DATAFILE_COMMAND ) == 0 )
            {
                
                 notifyActionListeners( e.getSource( ),
                                        ClinicalSpectroscopyWorklistPanel.RECEIVED_CSI_SPECTROSCOPY_DATAFILE_ID,
                                        ClinicalSpectroscopyWorklistPanel.RECEIVED_CSI_SPECTROSCOPY_DATAFILE_COMMAND);

            }
            
            if ( e.getActionCommand( ).compareTo( ClinicalSpectroscopyWorklistPanel.RECEIVED_SVS_SPECTROSCOPY_DATAFILE_COMMAND ) == 0 )
            {
                
                 notifyActionListeners( e.getSource( ),
                                        ClinicalSpectroscopyWorklistPanel.RECEIVED_SVS_SPECTROSCOPY_DATAFILE_ID,
                                        ClinicalSpectroscopyWorklistPanel.RECEIVED_SVS_SPECTROSCOPY_DATAFILE_COMMAND);

            }
        }

    }

    // Added attributes
    
    private static  ClinicalSpectroscopyWorklistFrame theInstance = null;
    
    private Vector<ActionListener> m_actionListeners = new Vector<ActionListener>( );

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel m_controlPanel;
    private org.spectrim.clinicalcasehandling.ClinicalSpectroscopyWorklistPanel m_spectroscopyWorklistPanel;
    // End of variables declaration//GEN-END:variables

    public final static int    INCOMING_DICOM_SPECTRUM_FILE_EVENT               = 0;
    public final static String INCOMING_DICOM_SPECTRUM_FILE_EVENT_COMMAND       = "INCOMING_DICOMSPECTRUM_FILE";

    public final static int    INCOMING_DICOM_IMAGE_FILE_EVENT                  = 1;
    public final static String INCOMING_DICOM_IMAGE_FILE_EVENT_COMMAND          = "INCOMING_DICOMIMAGE_FILE";

}