/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.data;

import org.spectrim.model.GlobalModel;

/**
 *
 * @author nunopbarros
 */
public class MRSDataType {

    public final double TE;
    public final double TR;
    public final double bandwidth;
    public final double B0;
    public final String seriesDescription;
    public final String vendor;
    private static final double TEMaxErr = 1;
    private static final double TRMaxErr = 5;
    private static final double bandwidthMaxErr = 5;
    private static final double B0MaxErr = 0.05;

    public MRSDataType(GlobalModel model) {
        this.TE = model.getTE();
        this.TR = model.getTR();
        this.bandwidth = model.getBandwidth();
        this.B0 = model.getB0();
        this.seriesDescription = model.getSeriesDescription();
        this.vendor = model.getManufacturer();
    }

    public MRSDataType(double B0, String seriesDescription, double TE, String vendor, double bandwidth, double TR) {
        this.TE = TE;
        this.TR = TR;
        this.bandwidth = bandwidth;
        this.B0 = B0;
        this.seriesDescription = seriesDescription;
        this.vendor = vendor;
    }

    public boolean isCompatible(GlobalModel model) {
        if (model.getTE() > 0 && Math.abs(model.getTE() - TE) > TEMaxErr) {
            return false;
        }
        if (model.getTR() > 0 && Math.abs(model.getTR() - TR) > TRMaxErr) {
            return false;
        }
        if (model.getBandwidth() > 0 && Math.abs(model.getBandwidth() - bandwidth) > bandwidthMaxErr) {
            return false;
        }
        if (model.getB0() > 0 && Math.abs(model.getB0() - B0) > B0MaxErr) {
            return false;
        }
        if (model.getSeriesDescription() != null && !model.getSeriesDescription().trim().matches("") && !model.getSeriesDescription().trim().toLowerCase().matches(seriesDescription.trim().toLowerCase())) {
            return false;
        }
        if (model.getManufacturer() != null && !model.getManufacturer().trim().matches("") && !model.getManufacturer().trim().toLowerCase().matches(vendor.trim().toLowerCase())) {
            return false;
        }
        return true;
    }

    public boolean isCompatible(MRSDataType otherType) {
        if (otherType.TE > 0 && Math.abs(otherType.TE - TE) > TEMaxErr) {
            return false;
        }
        if (otherType.TR > 0 && Math.abs(otherType.TR - TR) > TRMaxErr) {
            return false;
        }
        if (otherType.bandwidth > 0 && Math.abs(otherType.bandwidth - bandwidth) > bandwidthMaxErr) {
            return false;
        }
        if (otherType.B0 > 0 && Math.abs(otherType.B0 - B0) > B0MaxErr) {
            return false;
        }
        if (otherType.seriesDescription != null && !otherType.seriesDescription.trim().matches("") && !otherType.seriesDescription.trim().toLowerCase().matches(seriesDescription.trim().toLowerCase())) {
            return false;
        }
        if (otherType.vendor != null && !otherType.vendor.trim().matches("") && !otherType.vendor.trim().toLowerCase().matches(vendor.trim().toLowerCase())) {
            return false;
        }
        return true;
    }

}
