/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.decisionforest.results;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.decisionforest.data.DataSet;

/**
 *
 * @author nunopbarros
 */
public class ResultSet {

    private TreeMap<Integer, ResultMap> data = new TreeMap<>();

    public ResultMap get(Integer i) {
        return data.get(i);
    }

    public void put(Integer i, ResultMap map) {
        data.put(i, map);
    }

    public boolean containsKey(Integer i) {
        return data.containsKey(i);
    }

    public void clear() {
        data.clear();
    }

    public int size() {
        return data.size();
    }

    public void writeResults(String filename) throws FileNotFoundException, UnsupportedEncodingException {
        Set keys;
        keys = data.keySet();
        Iterator iter = keys.iterator();
        Integer indexaux;
        String klasseaux;
        Double meanval;
        Double regval;
        Double prob0;
        Double prob1;

        try (PrintWriter writer = new PrintWriter(filename, "UTF-8")) {
            writer.println("*Index" + "\t" + "Classe" + "\t" + "Mean Value" + "\t" + "Regression" + "\t" + "Prob 0" + "\t" + "Prob 1");
            while (iter.hasNext()) {
                indexaux = (Integer) iter.next();
                klasseaux = data.get(indexaux).getMax();
                meanval = data.get(indexaux).getMeanValue();
                regval = data.get(indexaux).getRegressionResult();
                prob0 = data.get(indexaux).getProbability("0");
                prob1 = data.get(indexaux).getProbability("1");
                writer.println(indexaux + "\t" + klasseaux + "\t" + meanval
                        + "\t" + regval + "\t" + prob0 + "\t" + prob1);
            }
        }
    }

    public double getErrorRate(DataSet data) {
        double res = 0;
        Iterator<Integer> iter = this.data.keySet().iterator();
        while (iter.hasNext()) {
            int pos = iter.next();
            if (!this.get(pos).getMax().matches(data.get(pos).klasse)) {
                res++;
            }
        }
        return (res / size());

    }

    public double getMSE(DataSet data) {
        double res = 0;
        Iterator<Integer> iter = this.data.keySet().iterator();
        while (iter.hasNext()) {
            int pos = iter.next();
            res += Math.pow(this.get(pos).getMeanValue() - Double.valueOf(data.get(pos).klasse), 2);
        }
        return (res / size());

    }

    public static void concatenateResultFiles(String[] files, String newfile) {
        String line;
        int linenr = 0;

        try {
            File file = new File(newfile);
            if (file.exists()) {
                file.delete();
            }
            BufferedWriter writer = new BufferedWriter(new FileWriter(newfile, true));
            for (int j = 0; j < files.length; j++) {
                BufferedReader reader = new BufferedReader(new FileReader(files[j]));
                while ((line = reader.readLine()) != null) {
                    String[] parts = line.split("\\s");
                    if (parts.length > 0) {
                        if (!parts[0].startsWith("*")) {
                            line = String.valueOf(linenr);
                            for (int k = 1; k < parts.length; k++) {
                                line += "\t" + parts[k];
                            }
                            linenr++;
                        }
                        writer.append(line + System.getProperty("line.separator"));
                    }
                }
                reader.close();
            }
            writer.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public static void evaluateResults(String results, String groundtruth, int positiveClass) {
        String line;
        ArrayList<Integer> valuesRes = new ArrayList<>();
        ArrayList<Integer> valuesGT = new ArrayList<>();
        double TP = 0;
        double TN = 0;
        double FN = 0;
        double FP = 0;

        try {
            BufferedReader reader = new BufferedReader(new FileReader(results));
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split("\\s");
                if (parts.length > 0) {
                    if (!parts[0].startsWith("*")) {
                        valuesRes.add(Integer.valueOf(parts[1]));
                    }
                }
            }
            reader.close();

            reader = new BufferedReader(new FileReader(groundtruth));
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split("\\s");
                if (parts.length > 0) {
                    if (!parts[0].startsWith("*")) {
                        valuesGT.add(Integer.valueOf(parts[1]));
                    }
                }
            }
            reader.close();

            if (valuesRes.size() == valuesGT.size()) {
                for (int i = 0; i < valuesRes.size(); i++) {
                    if (valuesRes.get(i) == positiveClass && valuesGT.get(i) == positiveClass) {
                        TP++;
                    }
                    if (valuesRes.get(i) != positiveClass && valuesGT.get(i) != positiveClass) {
                        TN++;
                    }
                    if (valuesRes.get(i) == positiveClass && valuesGT.get(i) != positiveClass) {
                        FP++;
                    }
                    if (valuesRes.get(i) != positiveClass && valuesGT.get(i) == positiveClass) {
                        FN++;
                    }
                }
                double errorRate = (FP + FN) / valuesRes.size();
                double sensitivity = TP / (TP + FN);
                double specificity = TN / (TN + FP);
                double ppv = TP / (TP + FP);
                double npv = TN / (TN + FN);
                double MCC = (TP * TN - FP * FN) / Math.sqrt((TP + FP) * (TP + FN) * (TN + FP) * (TN + FN));
                System.out.println("ErrorRate\tSensitivity\tSpecificity\tPPV\tNPV\tMCC");
                System.out.println(errorRate + "\t" + sensitivity + "\t" + specificity + "\t" + ppv + "\t" + npv + "\t" + MCC);
            } else {
                System.out.println("Files with different number of entries!");
            }

        } catch (FileNotFoundException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void mergeWith(ResultSet results2) {
        Iterator<Integer> iter = results2.data.keySet().iterator();
        while (iter.hasNext()) {
            int key = iter.next();
            ResultMap mapaux2 = results2.get(key);
            if (this.containsKey(key)) {
                ResultMap mapaux = this.get(key);
                Iterator<String> iter2 = mapaux2.keySet().iterator();
                while (iter2.hasNext()) {
                    String klasse = iter2.next();
                    if (mapaux.containsKey(klasse)) {
                        ResultVect vect = mapaux.get(klasse);
                        ResultVect vect2 = mapaux2.get(klasse);
                        vect.nr += vect2.nr;
                        vect.probability += vect2.probability;
                        mapaux.put(klasse, vect);
                    } else {
                        ResultVect vect2 = mapaux2.get(klasse);
                        mapaux.put(klasse, vect2);
                    }
                }
                this.put(key, mapaux);
            } else {
                this.put(key, mapaux2);
            }
        }
    }
}
