/*
 * File: $Id: DicomImageData.java,v 1.2 2006/11/09 10:09:01 hans Exp $
 * History
 * Date: 6. Februar 2003, 09:40,    Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.dicom;

/**
 * Class to handle a Dicom image
 * The two main attributes of this class are;
 *
 *      1. FileInfo object
 *      2. Pixeldata object
 *
 * @author  slotboom
 */

import java.util.LinkedHashMap;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;

import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.lib.Convert;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.FileUtils;

public class DicomImageData 
{
    
    /** 
     * Creates a new instance of DicomImageData. 
     */
    public DicomImageData( ) 
    {
    }

    /** 
     * Creates a new instance of DicomImageData and loads the file. 
     * @param fileName
     */
    public DicomImageData( String fileName ) 
    {
        loadImage( fileName );
    }
    
    /** 
     * Creates a new instance of DicomImageData and loads the file. 
     * @param inputStream
     */
    public DicomImageData( InputStream inputStream ) 
    {
        loadImage( inputStream );
    }
    
    /**
     * Loads a Dicom image from a medium.
     * @param fileName
     * @return 
     */
    private boolean loadImage( String fileName ) 
    {
	
        m_fileName = fileName;
        
        int i = m_fileName.lastIndexOf( System.getProperty( "file.separator" ) );
        String dirName;
        String fName;
	DicomDecoder dd;
         
        if ( i == -1 )
        {
            dirName = "." + System.getProperty( "file.separator" );
            fName   = m_fileName;
        }
        else
        {
            dirName = m_fileName.substring( 0, i + 1 );
            fName   = m_fileName.substring( i + 1 );
        }

        try
        {
            dd = new DicomDecoder( dirName, fName );
            m_headerInfo = dd.getFileInfo( );
            m_pixelData  = dd.getPixelData( );
            
            
//            if ( ( m_headerInfo.canDisplayImagePixelData( ) == true ) && ( m_headerInfo.getJMRUIInfo( ) == null ) )
//            // if ( m_headerInfo.canDisplayImagePixelData( ) == true  )
//            {
//                int bDataSize   = m_headerInfo.getBytesPerPixel( ) * m_headerInfo.getImageWidth( ) * m_headerInfo.getImageHeight( );
//                int imageSize   = m_headerInfo.getImageWidth( ) * m_headerInfo.getImageHeight( );
//                byte[] bData    = new byte[ bDataSize ];
//            
//                int hdrSize     = m_headerInfo.getFilePointerOffsetOfPixelData( );
//
//                FileInputStream fis = new FileInputStream( fileName );
//
//                if ( hdrSize != fis.skip( hdrSize ) )
//                {
//                    fis.close( );
//                    return false;
//                }
//
//                if ( bDataSize != fis.read( bData, 0, bDataSize ) )
//                {
//                    fis.close( );
//                    return false;
//                }
//
//                fis.close( );
//            
//                m_pixelData = new int[ imageSize ];
//                for ( int n = 0; n < imageSize; n++ )
//                {
//                    // System.out.println( "Number " + n );
//                    m_pixelData[ n ] = SCANalyze.Lib.Convert.byte2intl( bData, n << 1 );
//                }
//            }
//            else
//            {
//                generateDefaultInfoMessageImage( );
//            }
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return false;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while opening " + fileName );
            return false;
        }
        catch ( OutOfMemoryError e )
        {
            System.out.println( "Out of memory." );
            return false;
        }

        // makeQuadratic( );
        
        if ( ( dd.getDicomImageDataRead( ) == true ) && 
             ( dd.isSpectroscopyData( ) == false   ) )
        {
            makeQuadraticOn2ToThePowerNGrid( );
        }
        
        m_dicomImageDataLoaded = true;
        
        return m_dicomImageDataLoaded;
    }
    
    /**
     * Loads a Dicom image from an input stream. 
     * ( e.g. from a database )
     * @param inputStream
     * @return 
     */
    public final boolean loadImage( InputStream inputStream ) 
    {
	
        m_fileName = null;
        DicomDecoder dd;
        try
        {
            dd = new DicomDecoder( inputStream );
            
            m_headerInfo = dd.getFileInfo( );
            m_pixelData  = dd.getPixelData( );
            
//            if ( m_headerInfo.canDisplayImagePixelData( ) == true )
//            {
//                int bDataSize = m_headerInfo.getBytesPerPixel( ) * m_headerInfo.getImageWidth( ) * m_headerInfo.getImageHeight( );
//                int imageSize = m_headerInfo.getImageWidth( ) * m_headerInfo.getImageHeight( );
//                byte[] bData = new byte[ bDataSize ];
//            
//                int hdrSize = m_headerInfo.getFilePointerOffsetOfPixelData( );
//            
//                inputStream.reset( );
//	    
//                if ( hdrSize != inputStream.skip( hdrSize ) )
//                {
//                    inputStream.close( );
//                    return false;
//                }
//                if ( bDataSize != inputStream.read( bData, 0, bDataSize ) )
//                {
//                    inputStream.close( );
//                    return false;
//                }
//                inputStream.close( );
//            
//                m_pixelData = new int[imageSize];
//                for ( int n = 0; n < imageSize; n++ )
//                {
//                    m_pixelData[ n ] = SCANalyze.Lib.Convert.byte2intl( bData, n << 1 );
//                }
//            }
//            else
//            {
//                generateDefaultInfoMessageImage( );
//            }
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while reading InputStream " );
            return false;
        }
        catch ( OutOfMemoryError e )
        {
            System.out.println( "Out of memory." );
            return false;
        }
        
        if ( m_pixelData == null )
        {
            return false;
        }
        
        m_dicomImageDataLoaded = true;
        
        return m_dicomImageDataLoaded;
    }
    
    /**
     * Returns whether the file has been parsed and loaded.
     * @return 
     */
    public boolean isImageDataLoaded( )
    {
        return m_dicomImageDataLoaded;
    }

     /**
     * Converts a BufferedImage to GrayScale image
     * @param source
     * @return
     */
    public static BufferedImage convertToGrayscale( BufferedImage source )
    {
        BufferedImageOp op = new ColorConvertOp( ColorSpace.getInstance( ColorSpace.CS_GRAY ), null );
        return op.filter( source, null );
    }


    /**
     * Generates an image object.
     */
    private void generateDefaultInfoMessageImage( )
    {
       
        System.out.println( "Generating default info image!" );
        int[]  pixelData =
                       {

                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,1,1,0,0,0,1,1,0,0,1,0,0,1,0,1,0,0,1,0,0,1,1,0,0,1,1,1,1,1,0,
                       0,1,0,0,1,0,1,0,0,1,0,1,1,0,1,0,1,1,0,1,0,1,0,0,1,0,0,0,1,0,0,0,
                       0,1,0,0,0,0,1,1,1,1,0,1,0,1,1,0,1,0,1,1,0,1,0,0,1,0,0,0,1,0,0,0,
                       0,1,0,0,0,0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,0,0,1,0,0,0,
                       0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,0,0,1,0,0,0,
                       0,0,1,1,0,0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,0,1,1,0,0,0,0,1,0,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,1,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,
                       0,1,0,0,0,0,1,0,0,1,0,1,0,0,1,0,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,1,1,0,0,1,1,1,1,0,1,0,0,1,0,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,0,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,0,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,1,1,0,0,1,0,0,1,0,0,1,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,1,1,0,0,0,1,1,0,0,1,0,0,0,1,0,1,1,1,0,0,1,1,1,0,0,0,0,0,0,0,
                       0,1,0,0,1,0,1,0,0,1,0,1,1,0,1,1,0,1,0,0,1,0,1,0,0,1,0,0,0,0,0,0,
                       0,1,0,0,0,0,1,0,0,1,0,1,0,1,0,1,0,1,1,1,0,0,1,1,1,0,0,0,0,0,0,0,
                       0,1,0,0,0,0,1,0,0,1,0,1,0,0,0,1,0,1,0,0,0,0,1,1,0,0,0,0,0,0,0,0,
                       0,1,0,0,1,0,1,0,0,1,0,1,0,0,0,1,0,1,0,0,0,0,1,0,1,0,0,0,0,0,0,0,
                       0,0,1,1,0,0,0,1,1,0,0,1,0,0,0,1,0,1,0,0,0,0,1,0,0,1,0,1,0,0,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,1,0,1,0,0,0,1,0,0,1,1,0,0,0,1,1,1,0,1,1,1,1,0,0,1,1,0,0,1,0,0,
                       0,1,0,1,1,0,1,1,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,1,0,0,1,0,1,0,0,
                       0,1,0,1,0,1,0,1,0,1,1,1,1,0,1,0,0,0,0,1,1,1,0,0,0,1,1,0,0,1,0,0,
                       0,1,0,1,0,0,0,1,0,1,0,0,1,0,1,0,1,1,0,1,0,0,0,0,0,0,0,1,0,1,0,0,
                       0,1,0,1,0,0,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,0,0,1,0,0,1,0,0,0,0,
                       0,1,0,1,0,0,0,1,0,1,0,0,1,0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0,1,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                    };
            
            m_pixelData = new int[pixelData.length];
            for ( int i=0; i < pixelData.length; i++ )
            {
                m_pixelData[i] = 1 + 255 * pixelData[i];
            }
            
            m_headerInfo.setImageWidth( 32 );
            m_headerInfo.setImageHeight( 32 );

    }
    /**
     * Returns the filename and directory selected by the user.
     */
    private String getNewDicomFilePathAndName( String defaultString, String buttonText )
    {

        FileNameExtensionFilter dcmFilter = new FileNameExtensionFilter( "DCM file", "dcm", "DCM" );


        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        // only image file can be choosen!
        fileChooser.addChoosableFileFilter( dcmFilter );

        // Create a File object containing the canonical path of the
        // desired directory

        String filePath = System.getProperty( "user.dir" );

        File f = new File( filePath );

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        //fileChooser.setSelectedFile( new File( patientName+ "_" + examinationDate + "_" + defaultString ) );

        result = fileChooser.showDialog( null, defaultString );

        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }

    /**
     * Routine allowing the user to write changed pixel data to new DICOM-file.
     * @param imagePixels
     * @param pathAndName
     */
    public void saveDicomImageWithNewPixelData( int[] imagePixels, String pathAndName )
    {
        // check whether a image was loaded
        if ( m_dicomImageDataLoaded == true )
        {
            // routine may not change the image dimension!!
            if ( imagePixels.length == m_pixelData.length )
            {
                if ( pathAndName == null )
                {
                    pathAndName = getNewDicomFilePathAndName( "Write", m_fileName );
                }
                try
                {
                    FileOutputStream fout = new FileOutputStream( pathAndName );
                    try
                    {
                        writeDicomHeaderAsByteArray( fout, getDicomHeaderAsByteArray( ) );

                        writeNewDicomImagePixelDataAsByteArray( fout, imagePixels, true );

                    } 
                    catch ( IOException ex )
                    {
                        Logger.getLogger( DicomImageData.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }

                }
                catch ( FileNotFoundException ex )
                {
                    Logger.getLogger( DicomImageData.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
        }

    }
    
    /**
     * Returns the header info of the loaded file.
     * @return 
     */
    public FileInfo getHeader( )
    {
        if ( m_dicomImageDataLoaded == true )
        {
            return m_headerInfo;
        }
        else
        {
            return null;
        }
    }

    /**
     * Returns the Dicom header as a byte array.
     * @return 
     */
    public byte[] getDicomHeaderAsByteArray( )
    {
        byte[] hdr;
        
        try
        {
            //Dimension d = this.m_headerInfo.get;
            int bDataSize = m_headerInfo.getBytesPerPixel( ) * m_headerInfo.getImageWidth( ) * m_headerInfo.getImageHeight( );
            int fSize = (int)FileUtils.getUncompressedFileSize( m_fileName );
            int hdrSize = fSize - bDataSize;
	    
            if ( hdrSize <= 0 )
            {
                Dialogs.messageBox( null, "Wrong image dimensions: file size = " + fSize + ", data size = " + bDataSize, "Load error" );
                return null;
            }
	    
            hdr = new byte[hdrSize];
	    
            FileInputStream fis = new FileInputStream( m_fileName );
            if ( FileUtils.fileIsZipped( m_fileName ) )
            {
                GZIPInputStream zip = new GZIPInputStream( fis, hdrSize );
                if ( hdrSize != zip.read( hdr, 0, hdrSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return null;
                }
                zip.close( );
            }
            else
            {
                if ( hdrSize != fis.read( hdr, 0, hdrSize ) )
                {
                    fis.close( );
                    return null;
                }
                fis.close( );
            }
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + m_fileName + " not found." );
            return null;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while reading " + m_fileName );
            return null;
        }
	
        return hdr;
    }


    /**
     * Writes the Dicom header as a byte array.
     * @param fout
     * @param hdr
     * @throws java.io.IOException
     */
    public void writeDicomHeaderAsByteArray( FileOutputStream fout, byte[] hdr  ) throws IOException
    {
        fout.write( hdr );
    }

    /**
     * Writes the Dicom header as a byte array.
     * @param fout
     * @param newImagePixelData
     * @param closeOutputstream
     * @throws java.io.IOException
     */
    public void writeNewDicomImagePixelDataAsByteArray( FileOutputStream fout, int[] newImagePixelData, boolean closeOutputstream ) throws IOException
    {

        byte[] outputImageData = new byte[ m_headerInfo.getBytesPerPixel( ) * newImagePixelData.length ];
        
         // loop through all pixels
        for ( int y = 0; y < m_headerInfo.getImageWidth( ); y++ )
        {
            for ( int x = 0; x < m_headerInfo.getImageHeight( ); x++ )
            {
                int offset = ( y * m_headerInfo.getImageWidth( ) + x ) << 1;
                
                Convert.int2wordl( outputImageData, offset, newImagePixelData[ y * m_headerInfo.getImageWidth( ) + x ] );
                
            }           /* x - loop */
        }               /* y - loop */

        // write the header
        // write the image pixel data
        fout.write( outputImageData );

        if ( closeOutputstream == true )
        {
            fout.close( );
        }
    }


    /**
     * Returns the pixel data of the loaded file (integers).
     * @return 
     */
    public int[] getPixelData( )
    {
        if ( m_dicomImageDataLoaded == true )
        {
            return m_pixelData;
        }
        else
        {
            return null;
        }
    }

     /**
      * Sets the pixel data of the already loaded file (integers)
      * This method is needed for those case where the pixel data 
      * are processed, and one is interested in showing the processed
      * data, while maintaining the Dicom header information.
      * @param pixData
      */
    public void setPixelData( int[] pixData )
    {
        if ( m_dicomImageDataLoaded == true )
        {
            if ( m_pixelData.length == pixData.length )
            {
                System.arraycopy( pixData, 0, m_pixelData, 0, pixData.length ); 
            }
            else
            {
                System.out.println( "Error -> DicomImageData::setPixelData : Cannot set pixel data matrix unequal sizes" );
            }
        }
        else
        {
            System.out.println( "Error -> DicomImageData::setPixelData : Cannot set pixel data matrix unequal sizes, No file loaded" );
        }
    }
    
    /**
     * Returns the Siemens specific file header information object.
     * @return 
     */
    public LinkedHashMap getSiemensSpecificDataHeaderInfo( )
    {
        if ( m_siemensDicomData != null )
        {
            return m_siemensDicomData.getSiemensAttributes( );
        }
        else
        {
            return null;
        }
    }
    
    /**
     * If the instance exists the Siemens specific data can be determined.
     * @return 
     */
    public LinkedHashMap getSiemensSpecificDataFromFile( )
    {
        if ( m_siemensDicomData == null )
        {
           m_siemensDicomData = new DicomFileSiemensSpecificData( m_fileName );
        }
        return m_siemensDicomData.getSiemensAttributes( );
    }
    
    /**
     * If the instance exists the Siemens specific data can be determined
     * @param sopInstanceUID
     * @param is
     * @return 
     */
    public LinkedHashMap getSiemensSpecificDataFromInputStream( String sopInstanceUID, InputStream is )
    {
        if ( m_siemensDicomData == null )
        {
           m_siemensDicomData = new DicomFileSiemensSpecificData( sopInstanceUID, is );
        }
        return m_siemensDicomData.getSiemensAttributes( );
    }

    /**
     * Makes the image quadratic, i.e. if the image width is not equal to the
     * image height, this routines adds zeroes to the image in order to make
     * it quadratic
     */
    private void makeQuadratic( )
    {
        int oldImageWidth = m_headerInfo.getImageWidth( );
        int oldImageHeight = m_headerInfo.getImageHeight( );

        int newImageWidth = m_headerInfo.getImageWidth( );
        int newImageHeight = m_headerInfo.getImageHeight( );

        if ( m_headerInfo.getImageHeight( ) != m_headerInfo.getImageWidth( ) )
        {
            if ( m_headerInfo.getImageHeight( ) > m_headerInfo.getImageWidth( ) )
            {
                m_headerInfo.setImageWidth( m_headerInfo.getImageHeight( ) );
                newImageWidth = m_headerInfo.getImageHeight( );


                System.out.println( "makeQuadratic changes input pixel data matrix" );
                System.out.println( "old width = " + oldImageWidth + "old height = " + oldImageHeight );
                System.out.println( "new width = " + newImageWidth + "new height = " + newImageHeight );

                // declare more space for the zero-filled dataset
                int[] enlargedImageData = new int[ newImageHeight * newImageWidth ];
                for ( int i = 0; i < oldImageHeight; i++ )
                {
                    for ( int j = 0; j < oldImageWidth; j++ )
                    {

                        enlargedImageData[ i * newImageWidth + j + ( ( newImageWidth - oldImageWidth ) / 2 ) ] = m_pixelData[ i*oldImageWidth + j ];
                    }
                }

                m_pixelData = new int[ newImageHeight * newImageWidth ];

                System.arraycopy( enlargedImageData, 0, m_pixelData, 0, enlargedImageData.length );
                
            }
            else if ( m_headerInfo.getImageHeight( ) < m_headerInfo.getImageWidth( ) )
            {
                System.out.print( "DicomImageData::MakeQuadratic : image height < image width -> non handled case image is not made quadratic!" );
                System.out.print( "Contact developer, who is interested in this case!" );
            }
        }
    }

    /**
     * Makes the image quadratic, i.e. if the image width is not equal to the
     * image height, this routines adds zeroes to the image in order to make
     * it quadratic
     */
    private void makeQuadraticOn2ToThePowerNGrid( )
    {
        int newImageWidth  = m_headerInfo.getImageWidth( );
        int newImageHeight = m_headerInfo.getImageHeight( );
        
        int oldImageWidth  = m_headerInfo.getImageWidth( );
        int oldImageHeight = m_headerInfo.getImageHeight( );

        if ( ( oldImageWidth == 0 ) || ( oldImageHeight == 0 ) )
        {
            System.out.println( "Image width or image height is 0" );
            return;
        }
        
        if ( ( oldImageWidth < 8 ) || ( oldImageWidth < 8 ) )
        {
            System.out.println( "Image too small to make quadratic!" );
            newImageWidth  = -1;
            oldImageHeight = -1;
            return;
        }
        
        if ( ( oldImageWidth > 8 ) && ( oldImageWidth <= 16 ) )
        {
            newImageWidth = 16;
        }
        
        if ( ( oldImageWidth > 16 ) && ( oldImageWidth <= 32 ) )
        {
            newImageWidth = 32;
        }
        
        if ( ( oldImageWidth > 32 ) && ( oldImageWidth <= 64 ) )
        {
            newImageWidth = 64;
        }
        
        if ( ( oldImageWidth > 64 ) && ( oldImageWidth <= 128 ) )
        {
            newImageWidth = 128;
        }
          
        if ( ( oldImageWidth > 128 ) && ( oldImageWidth <= 256 ) )
        {
            newImageWidth = 256;
        }
        
        if ( ( oldImageWidth > 256 ) && ( oldImageWidth <= 512 ) )
        {
            newImageWidth = 512;
        }

        if ( ( oldImageWidth > 512 ) && ( oldImageWidth <= 1024 ) )
        {
            newImageWidth = 1024;
        }
        
        // do the same tests for the hight
        if ( ( oldImageHeight > 8 ) && ( oldImageHeight <= 16 ) )
        {
            newImageHeight = 16;
        }
        
        if ( ( oldImageHeight > 16 ) && ( oldImageHeight <= 32 ) )
        {
            newImageHeight = 32;
        }
        
        if ( ( oldImageHeight > 32 ) && ( oldImageHeight <= 64 ) )
        {
            newImageHeight = 64;
        }
        
        if ( ( oldImageHeight > 64 ) && ( oldImageHeight <= 128 ) )
        {
            newImageHeight = 128;
        }
          
        if ( ( oldImageHeight > 128 ) && ( oldImageHeight <= 256 ) )
        {
            newImageHeight = 256;
        }
        
        if ( ( oldImageHeight > 256 ) && ( oldImageHeight <= 512 ) )
        {
            newImageHeight = 512;
        }

        if ( ( oldImageHeight > 512 ) && ( oldImageHeight <= 1024 ) )
        {
            newImageHeight = 1024;
        }
        
        if ( newImageHeight > newImageWidth )
        {
            newImageWidth = newImageHeight;
        }
        else
        {
            newImageHeight = newImageWidth;
        }
        
        
        if ( ( newImageWidth > 0 ) && ( newImageHeight > 0 ) )
        {
            m_headerInfo.setImageWidth( newImageWidth );
            m_headerInfo.setImageHeight( newImageHeight );

            //System.out.println( "makeQuadraticOn2ToThePowerNGrid changes input pixel data matrix" );
            //System.out.println( "old width = " + oldImageWidth + "old height = " + oldImageHeight );
            //System.out.println( "new width = " + newImageWidth + "new height = " + newImageHeight );
            
            
            
            // Declare more space for the zero-filled dataset
            int[] enlargedImageData = new int[ newImageHeight * newImageWidth ];
            int i, j;
            try
            {
                for ( i = 0; i < oldImageHeight; i++ )
                {
                    for ( j = 0; j < oldImageWidth; j++ )
                    {
                        enlargedImageData[ ( i + ( ( newImageHeight - oldImageHeight ) / 2 ) ) * newImageWidth + j + ( ( newImageWidth - oldImageWidth ) / 2 ) ] = m_pixelData[ i*oldImageWidth + j ];
                    }
                    if ( i > oldImageHeight-2 )
                    {
                        // System.out.println( "Element offset = " + ( i + ( ( newImageHeight - oldImageHeight ) / 2 ) ) * newImageWidth + j + ( ( newImageWidth - oldImageWidth ) / 2 ) );
                    }
                }   
            }
            catch ( ArrayIndexOutOfBoundsException aioobe )
            {
                System.out.println( "Array index out of bound." );
            }
            
            m_pixelData = new int[ newImageHeight * newImageWidth ];

            System.arraycopy( enlargedImageData, 0, m_pixelData, 0, enlargedImageData.length );
        }          
    }

    /**
     * 
     * Returns the reference of the image file in the file system.
     * 
     * @return
     */
    public String getFileNameAndPath( )
    {
       return m_fileName;
    }

    // Attributes
    private String      m_fileName = "";
    private int         m_pixelData[];
    private FileInfo    m_headerInfo;
    private DicomFileSiemensSpecificData m_siemensDicomData;
    private boolean     m_dicomImageDataLoaded = false;
}