/*
 * File: $Id: DicomNetworkNodeDefaults.java,v 1.6 2009/15/05 13:28:39 hans Exp $
 * History
 * Date: 15.05.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

/**
 *
 * @author slotboom
 */
public class DicomNetworkNodeDefaults extends Properties
{

     /**
        Creates a new instance of DicomNetworkNodeDefaults
     */
    private DicomNetworkNodeDefaults( )
    {

        load( );

    }

    /**
     * Returns the one and only instance of DicomNetworkNodeDefaults.
     * @return 
     */
    public static DicomNetworkNodeDefaults getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new DicomNetworkNodeDefaults( );
        }
        return theInstance;
    }

    /**
     * Loads the user preferences file
     */
    public void load( )
    {
        String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "DicomNetworkNodes.ini";
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );

            // now the ini file must exist with default contents, so load it

            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            try
            {
                // file does not exist, so create one with default settings..

                setProperty( SELECTED_REMOTE_DICOM_HOST_NUMBER, "-1" );

                setProperty( CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY, System.getProperty( "user.dir" ) );
                setProperty( CURRENT_RENAME_INPUT_PATH, System.getProperty( "user.dir" ) );
                setProperty( CURRENT_RENAME_TO_DICOM_DIRECTORY, System.getProperty( "user.dir" ) );

                setProperty( LOCAL_HOST_NAME, "NOT DEFINED" );
                setProperty( LOCAL_HOST_IP_ADDRESS, "NOT DEFINED" );
                setProperty( LOCAL_HOST_DICOM_RECEIVE_AETITLE, "NOT DEFINED" );
                setProperty( LOCAL_HOST_DICOM_TRANSMIT_AETITLE, "NOT DEFINED" );
                setProperty( LOCAL_HOST_DICOM_RECEIVE_PORT, "-1" );
                setProperty( LOCAL_HOST_DICOM_TRANSMIT_PORT, "-1" );

                setProperty( REMOTE_DICOM_HOST_0_NAME , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_0_IP_ADDRESS , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_0_DICOM_RECEIVE_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_0_DICOM_TRANSMIT_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_0_DICOM_RECEIVE_PORT , "-1" );
                setProperty( REMOTE_DICOM_HOST_0_DICOM_TRANSMIT_PORT , "-1" );

                setProperty( REMOTE_DICOM_HOST_1_NAME , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_1_IP_ADDRESS , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_1_DICOM_RECEIVE_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_1_DICOM_TRANSMIT_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_1_DICOM_RECEIVE_PORT , "-1" );
                setProperty( REMOTE_DICOM_HOST_1_DICOM_TRANSMIT_PORT , "-1" );

                setProperty( REMOTE_DICOM_HOST_2_NAME , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_2_IP_ADDRESS , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_2_DICOM_RECEIVE_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_2_DICOM_TRANSMIT_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_2_DICOM_RECEIVE_PORT , "-1" );
                setProperty( REMOTE_DICOM_HOST_2_DICOM_TRANSMIT_PORT , "-1" );

                setProperty( REMOTE_DICOM_HOST_3_NAME , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_3_IP_ADDRESS , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_3_DICOM_RECEIVE_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_3_DICOM_TRANSMIT_AETITLE , "NOT DEFINED" );
                setProperty( REMOTE_DICOM_HOST_3_DICOM_RECEIVE_PORT , "-1" );
                setProperty( REMOTE_DICOM_HOST_3_DICOM_TRANSMIT_PORT , "-1" );

                OutputStream os = new BufferedOutputStream( new FileOutputStream(  userPreferenceFileName ) );

                this.save( os, "// This is a generated file: do not edit!!" );
                os.close( );
            }
            catch ( IOException ioe )
            {
                // should hopefully never occur
            }
        }

        // now the ini file must exist with default contents, so load it
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            // should never occur
        }
    }

   /**
     * Saves the user preferences file
     */
    public void save( )
    {
        try
        {
            String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "DicomNetworkNodes.ini";
            OutputStream out = new BufferedOutputStream( new FileOutputStream( userPreferenceFileName ) );
            this.save( out, "//This is a generated file : do not edit" );
        }
        catch ( IOException e )
        {
            // should not occur
        }
    }

    // static private attribute is the instance itself (Singleton!)
    private static  DicomNetworkNodeDefaults theInstance = null;

    // needed for dicomization!
    public final static String CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY = "CURRENT_INTERMEDIATE_DICOM_DIRECTORY";
    
    // renamed dicom files will land in this directory
    public final static String CURRENT_RENAME_TO_DICOM_DIRECTORY = "CURRENT_RENAME_TO_DICOM_DIRECTORY";

    // files in this directory will be renamed and moved to CURRENT_RENAME_TO_DICOM_DIRECTORY-path
    public final static String CURRENT_RENAME_INPUT_PATH = "CURRENT_RENAME_INPUT_PATH";

    public final static String SELECTED_REMOTE_DICOM_HOST_NUMBER = "SELECTED_REMOTE_DICOM_HOST_NUMBER";
    
    public final static String LOCAL_HOST_NAME = "LOCAL_HOST_NAME";
    public final static String LOCAL_HOST_IP_ADDRESS = "LOCAL_HOST_IP_ADDRESS";
    public final static String LOCAL_HOST_DICOM_RECEIVE_AETITLE = "LOCAL_HOST_DICOM_RECEIVE_AETITLE";
    public final static String LOCAL_HOST_DICOM_TRANSMIT_AETITLE = "LOCAL_HOST_DICOM_TRANSMIT_AETITLE";
    public final static String LOCAL_HOST_DICOM_RECEIVE_PORT = "LOCAL_HOST_DICOM_RECEIVE_PORT";
    public final static String LOCAL_HOST_DICOM_TRANSMIT_PORT = "LOCAL_HOST_DICOM_TRANSMIT_PORT";

    public final static String REMOTE_DICOM_HOST_0_NAME = "REMOTE_DICOM_HOST_0_NAME";
    public final static String REMOTE_DICOM_HOST_0_IP_ADDRESS = "REMOTE_DICOM_HOST_0_IP_ADDRESS";
    public final static String REMOTE_DICOM_HOST_0_DICOM_RECEIVE_AETITLE = "REMOTE_DICOM_HOST_0_DICOM_RECEIVE_AETITLE";
    public final static String REMOTE_DICOM_HOST_0_DICOM_TRANSMIT_AETITLE = "REMOTE_DICOM_HOST_0_DICOM_TRANSMIT_AETITLE";
    public final static String REMOTE_DICOM_HOST_0_DICOM_RECEIVE_PORT = "REMOTE_DICOM_HOST_0_DICOM_RECEIVE_PORT";
    public final static String REMOTE_DICOM_HOST_0_DICOM_TRANSMIT_PORT = "REMOTE_DICOM_HOST_0_DICOM_TRANSMIT_PORT";


    public final static String REMOTE_DICOM_HOST_1_NAME = "REMOTE_DICOM_HOST_1_NAME";
    public final static String REMOTE_DICOM_HOST_1_IP_ADDRESS = "REMOTE_DICOM_HOST_1_IP_ADDRESS";
    public final static String REMOTE_DICOM_HOST_1_DICOM_RECEIVE_AETITLE = "REMOTE_DICOM_HOST_1_DICOM_RECEIVE_AETITLE";
    public final static String REMOTE_DICOM_HOST_1_DICOM_TRANSMIT_AETITLE = "REMOTE_DICOM_HOST_1_DICOM_TRANSMIT_AETITLE";
    public final static String REMOTE_DICOM_HOST_1_DICOM_RECEIVE_PORT = "REMOTE_DICOM_HOST_1_RECEIVE_DICOM_PORT";
    public final static String REMOTE_DICOM_HOST_1_DICOM_TRANSMIT_PORT = "REMOTE_DICOM_HOST_1_TRANSMIT_DICOM_PORT";

    public final static String REMOTE_DICOM_HOST_2_NAME = "REMOTE_DICOM_HOST_2_NAME";
    public final static String REMOTE_DICOM_HOST_2_IP_ADDRESS = "REMOTE_DICOM_HOST_2_IP_ADDRESS";
    public final static String REMOTE_DICOM_HOST_2_DICOM_RECEIVE_AETITLE = "REMOTE_DICOM_HOST_2_DICOM_RECEIVE_AETITLE";
    public final static String REMOTE_DICOM_HOST_2_DICOM_TRANSMIT_AETITLE = "REMOTE_DICOM_HOST_2_DICOM_TRANSMIT_AETITLE";
    public final static String REMOTE_DICOM_HOST_2_DICOM_RECEIVE_PORT = "REMOTE_DICOM_HOST_2_DICOM_RECEIVE_PORT";
    public final static String REMOTE_DICOM_HOST_2_DICOM_TRANSMIT_PORT = "REMOTE_DICOM_HOST_1_TRANSMIT_DICOM_PORT";

    public final static String REMOTE_DICOM_HOST_3_NAME = "REMOTE_DICOM_HOST_3_NAME";
    public final static String REMOTE_DICOM_HOST_3_IP_ADDRESS = "REMOTE_DICOM_HOST_3_IP_ADDRESS";
    public final static String REMOTE_DICOM_HOST_3_DICOM_RECEIVE_AETITLE = "REMOTE_DICOM_HOST_3_DICOM_RECEIVE_AETITLE";
    public final static String REMOTE_DICOM_HOST_3_DICOM_TRANSMIT_AETITLE = "REMOTE_DICOM_HOST_3_DICOM_TRANSMIT_AETITLE";
    public final static String REMOTE_DICOM_HOST_3_DICOM_RECEIVE_PORT = "REMOTE_DICOM_HOST_3_DICOM_RECEIVE_PORT";
    public final static String REMOTE_DICOM_HOST_3_DICOM_TRANSMIT_PORT = "REMOTE_DICOM_HOST_3_DICOM_TRANSMIT_PORT";

    // Must be declader somehow
    private static final long serialVersionUID = 3487495895819393L;

}
