 /*
 * File: $Id: DicomSeriesSelectorFrame.java,v 1.5 2009/05/27 10:11:00 hans Exp $
 * History
 * Date: 27.05.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003-2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * DicomSeriesSelectorFrame.java
 *
 * Created on May 3, 2006, 2:43 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.dicom;

import org.spectrim.lib.Dialogs;
import org.spectrim.lib.DualProgressBar;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Cursor;

import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;

import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JRadioButton;
import javax.swing.JLabel;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Class handing Dicom series selections.
 * @author  hans
 */
public class DicomSeriesSelectorFrame extends JFrame implements ActionListener
{
    private final JFrame m_parent;
    private DualProgressBar m_dualProgressBar;
    
    /** 
     * Creates a new instance of DicomDataLoaderFrame. 
     * @param parent
     */
    public DicomSeriesSelectorFrame( JFrame parent ) 
    {
        m_parent = parent;
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_dicomDataLoaderPanel = new DicomDataLoaderPanel( parent, null );
        
        // This class listens to changes in the loader
        m_dicomDataLoaderPanel.addActionListener( this );
        
        instantiateCancelApplyPanel( );
        
        m_contentPanel.add( m_dicomDataLoaderPanel, BorderLayout.NORTH );
        
        // Handle window closing
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   System.exit( 0 );
                }
            }
        );
        
        setContentPane( m_contentPanel );
        pack( );
        
        Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) - getWidth( ) ) / 2,
                                        parent.getY( ) + ( parent.getHeight( ) - getHeight( ) ) / 2 );
        
        this.setLocation( centerParent );
        
        setVisible( true );
        
        m_dicomDataLoaderPanel.mountDirectory( );
        
    }
    
   
    /**
     * Creates a panel allowing the user to assign an image series.
     */
    private void createSeriesDetailsPanel( )
    {
        m_imageSeriesDetailsPanel = new JPanel( );
        m_imageSeriesDetailsPanel.setLayout( new BoxLayout( m_imageSeriesDetailsPanel , BoxLayout.Y_AXIS ) );
        
        m_numberOfImagesPanel = new JPanel( );
        m_numberOfImagesPanel.setLayout( new BoxLayout( m_numberOfImagesPanel, BoxLayout.X_AXIS ) );
        
        m_selectedImagesInSeriesLabel = new JLabel( "Number of images in series : " );
        m_selectedImagesInSeriesValue = new JLabel( "--" );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        m_numberOfImagesPanel.add( m_selectedImagesInSeriesLabel );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        m_numberOfImagesPanel.add( m_selectedImagesInSeriesValue );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        
        m_differentSlicePositionsLabel = new JLabel( "Number of different slice positions :" );
        m_differentSlicePositionsValue = new JLabel( " -- " );
        
        m_numberOfImagesPanel.add( m_differentSlicePositionsLabel );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        
        m_numberOfImagesPanel.add( m_differentSlicePositionsValue );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        
        m_differentTimePointsLabel = new JLabel( "Number of time points : " );
        m_differentTimePointsValue = new JLabel( " -- " );
        
        m_numberOfImagesPanel.add( m_differentTimePointsLabel );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        m_numberOfImagesPanel.add( m_differentTimePointsValue );
        m_numberOfImagesPanel.add( Box.createHorizontalGlue( ) );
        
        m_imageSeriesDetailsPanel.add( m_numberOfImagesPanel );
    }
    
    /**
     * Instantiates an apply/cancel panel.
     */
    private void instantiateCancelApplyPanel( )
    {
        // Selection panel definitions
        m_loadButton = new JButton( "Load" );
        m_loadButton.addActionListener( this );
        m_loadButton.setToolTipText( "Loads the currently selected dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without loading" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_loadButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
    }

    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens in this component
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent
     */
    public void actionPerformed( ActionEvent e ) 
    {   

        Object sender = e.getSource( );
       
        if ( sender == this.m_instancesinDifferentSeriesRadioButton )
        {
            if ( m_instancesinDifferentSeriesRadioButton.isSelected( ) )
            {
                this.m_dicomDataLoaderPanel.setImagesIndifferentSeries( true );
            }
            else
            {
                this.m_dicomDataLoaderPanel.setImagesIndifferentSeries( false );
            }
        }
        
        if ( sender == this.m_allInstancesInOneSeriesRadioButton )
        {
            if ( m_allInstancesInOneSeriesRadioButton.isSelected( ) )
            {
                this.m_dicomDataLoaderPanel.setImagesIndifferentSeries( false );
            }
            else
            {
                this.m_dicomDataLoaderPanel.setImagesIndifferentSeries( true );
            }
        }
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
            }
            dispose( );
        }
        
        // handle responses on apply button pressed event
        if ( sender == m_loadButton )
        {
            
            setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
            
            // do the physical loading of the dicom data
            
            m_selectedDicomFileReferences = m_dicomDataLoaderPanel.getSelectedFileReferencesFromTable( );
                        
            // notify all listeners that the data has been loaded
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, LOAD_EVENT, "" ) );
            }
            
            setVisible( false );
            
            setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
        }


         if ( sender == m_dicomRegridImageSeriesPropertiesFrame )
         {
             if ( e.getID( ) == DicomRegridImageSeriesPropertiesFrame.APPLY_REGRIDDING_EVENT )
             {

                int newHorizontal   = m_dicomRegridImageSeriesPropertiesFrame.getNewHorizontalGridSize( );
                int newVertical     = m_dicomRegridImageSeriesPropertiesFrame.getNewVerticalGridSize( );

                System.out.println( "New horizontal number of points : " + newHorizontal );
                System.out.println( "New vertical   number of points : " + newVertical );

                createRegriddedDICOMFilesOfSelectedImageSeries( newHorizontal, newVertical );
             }
             
             if ( e.getID( ) == DicomRegridImageSeriesPropertiesFrame.CANCEL_REGRIDDING_EVENT )
             {
                m_dicomRegridImageSeriesPropertiesFrame.dispose( );
             }

             if ( e.getID( ) == DicomRegridImageSeriesPropertiesFrame.SHOW_DICOM_INFORMATION_EVENT )
             {
                showDicomInformation( );
             }

         }

        if ( sender instanceof DicomDataLoaderPanel )
        {
            if ( e.getID( ) == DicomDataLoaderPanel.MOUNT_DIRECTORY_EVENT )
            {
                // not handled!!
            }
            
            
            if ( e.getID( ) == DicomDataLoaderPanel.COPY_AND_RENAME_MOUNTED_DIRECTORY_EVENT )
            {
                copyAndRenameDicomFilesFromMountedDicomDirInTread( );
            }
            
            if ( e.getID( ) == DicomDataLoaderPanel.CREATE_REGRIDDED_DICOM_FILES_OF_SELECTED_IMAGE_SERIES_EVENT )
            {

                LinkedList<String> filesInSeries = m_dicomDataLoaderPanel.getSelectedSeriesFileReferences( );
                DicomImageData imaDat = new DicomImageData( filesInSeries.getFirst( ) );
                FileInfo finfo = imaDat.getHeader( );

                m_dicomRegridImageSeriesPropertiesFrame = new DicomRegridImageSeriesPropertiesFrame( m_parent,
                                                                                                     finfo.getImageWidth( ),
                                                                                                     finfo.getImageHeight( ),
                                                                                                     finfo.getSeriesDescription( ) );
                m_dicomRegridImageSeriesPropertiesFrame.addActionListener( this );

            }
            
            // The user selects an item of the dicom series tree
            if ( e.getActionCommand( ).compareTo( DicomFileSelectionControl.TREE_ENTRY_SELECTED_CMD ) == 0 )
            {
                //not implemented yet
            }
            
            if ( e.getID( ) == DicomDataLoaderPanel.UNMOUNT_DIRECTORY_EVENT )
            {
                if ( m_selectedImagesInSeriesValue != null )
                {
                    m_selectedImagesInSeriesValue.setText( "--" );
                }
            }
            
        }
    }
    

    /**
     * Returns the DicomDirectory object.
     * @return
     */
    public DicomDirectory getDicomDir( )
    {
        return m_dicomDataLoaderPanel.getDicomDir( );
    }
    /**
     *
     * Performs the renaming in a separate thread to keep the GUI responsive
     *
     */
    private void copyAndRenameDicomFilesFromMountedDicomDirInTread( )
    {

        m_dualProgressBar = new DualProgressBar( this, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Copy and Renaming files .."  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

             @Override
             public Object construct( )
             {
                 copyAndRenameDicomFilesFromMountedDicomDir( );
                 return null;
             }

             // runs in the event dispatching thread
             @Override
             public void finished( )
             {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox(m_parent, "Renaming of mounted directory finished!" );
             }
         };
         worker.start( );
    }


    /**
     * Copies and renames dicom files:
     * Rename from numeric "filename" to 
     *      PatientName_SeriesDescription_SeriesNumber_InstanceNumber.dcm
     * In which the filename parts stem from the dicom header
     * For each SeriesDescription a separate sub directory with name 
     * "SeriesDescription" is created.
     */
    private void copyAndRenameDicomFilesFromMountedDicomDir( )
    {
        File directoryToCopyTo = getDicomDirectory( false );
        LinkedList list = m_dicomDataLoaderPanel.getDicomDir( ).getAllFileRefences( );
        
        LinkedList <String>failedInputFileRefs = new LinkedList<String>( );
        LinkedList <String>failedOutputFileRefs = new LinkedList<String>( );
        
        Iterator it = list.iterator( );
        
        int cntr = 0;

        while ( it.hasNext( ) )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)cntr++ / list.size( ) ) );
            m_dualProgressBar.setLowerLabel( "    Renaming dicom image : " +  cntr + " of " + list.size( )  );
            
            String srcFileName;
            srcFileName = m_dicomDataLoaderPanel.getDicomDir( ).getPath( ) + System.getProperty( "file.separator" ) + (String)it.next( );
                    
            DicomImageData d_ima_data = new DicomImageData( srcFileName );
            FileInfo fi = d_ima_data.getHeader( );
                    
            String instanceNumberString = "";

            int seriesNumber = new Integer( fi.getInstanceNumber( ).trim( ) ).intValue( );


            if ( ( seriesNumber < 10 ) && ( seriesNumber > -1 ) )
            {
                instanceNumberString = "000" + fi.getInstanceNumber( ).trim( );
            }
                    
            if ( ( seriesNumber < 100 ) && ( seriesNumber > 9 ) )
            {
                instanceNumberString = "00" + fi.getInstanceNumber( ).trim( );
            }
                    
            if ( ( seriesNumber < 999 ) && ( seriesNumber > 99 ) )
            {
                instanceNumberString = "0" + fi.getInstanceNumber( ).trim( );
            }

            if ( ( seriesNumber < 9999 ) && ( seriesNumber > 999 ) )
            {
                instanceNumberString = "" + fi.getInstanceNumber( ).trim( );
            }

            String seriesNumberString = "";
            if ( ( seriesNumber < 10 ) && ( seriesNumber > -1 ) )
            {
                seriesNumberString = "000" + fi.getSeriesNumber( ).trim( );
            }
                    
            if ( ( seriesNumber < 100 ) && ( seriesNumber > 9 ) )
            {
                 seriesNumberString = "00" + fi.getSeriesNumber( ).trim( );
            }
                    
            if ( ( seriesNumber < 999 ) && ( seriesNumber > 99 ) )
            {
                seriesNumberString = "0" + fi.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 9999 ) && ( seriesNumber > 999 ) )
            {
                seriesNumberString = "" + fi.getSeriesNumber( ).trim( );
            }
                    
            String dstPath;
            String dstFileName;
            String dstPathCmd;
            
            String reformattedPatientName = fi.getPatientName( ).replace( ' ', '_' );
            reformattedPatientName = reformattedPatientName.replace( '-', '_' );
            reformattedPatientName = reformattedPatientName.replace( '^', '_' );
            reformattedPatientName = reformattedPatientName.replace( ',', '_' );
            reformattedPatientName = reformattedPatientName.replace( '!', '_' );
            reformattedPatientName = reformattedPatientName.replace( ':', '_' );
            reformattedPatientName = reformattedPatientName.replace( ';', '_' );
            reformattedPatientName = reformattedPatientName.replace( '?', '_' );
            
            
            if ( fi.getSeriesDescription( ) != null )
            {
                String reformattedSeriesDescription = fi.getSeriesDescription( );
            
                // eliminate all kinds of unwanted characters which are not accepted by the file system
                reformattedSeriesDescription = reformattedSeriesDescription.replace( ' ', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( '-', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( '^', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( ',', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( '!', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( ':', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( ';', '_' );
                reformattedSeriesDescription = reformattedSeriesDescription.replace( '?', '_' );
                
                dstPath = directoryToCopyTo.getAbsolutePath( ) + 
                          System.getProperty( "file.separator" ) +
                          reformattedSeriesDescription.trim( ) + "-" + 
                          fi.getSeriesDate( ).trim( ) + "-" + fi.getStudyTime( ).trim( );
                    
                dstFileName = dstPath + System.getProperty( "file.separator" ) +
                              reformattedPatientName + "_" +
                              fi.getStudyDate( ).trim( ) + "_" +
                              reformattedSeriesDescription.trim( ) + "-" +
                              fi.getSeriesTime( ).trim( ) + "-" +
                              seriesNumberString + "-" + 
                              instanceNumberString + ".dcm";
                    
                dstPathCmd = directoryToCopyTo.getAbsolutePath( ) + 
                             System.getProperty( "file.separator" ) +
                             reformattedSeriesDescription.trim( )+ "-" + 
                             fi.getSeriesDate( ).trim( ) + "-" + fi.getStudyTime( ).trim( );
                
                // rhese filenames may contain characters that are not allowd by the
                // underlying file system
                dstFileName = dstFileName.replace( '<', '_' );
                dstFileName = dstFileName.replace( '>', '_' );
                dstFileName = dstFileName.replace( '!', '_' );
                
            
                dstPathCmd = dstPathCmd.replace( '<', '_' );
                dstPathCmd = dstPathCmd.replace( '>', '_' );
                dstPathCmd = dstPathCmd.replace( '!', '_' );
                
            }
            else
            {
                dstPath = directoryToCopyTo.getAbsolutePath( ) + 
                System.getProperty( "file.separator" ) +
                "no_series_description";
                    
                dstFileName = dstPath + System.getProperty( "file.separator" ) +
                              reformattedPatientName + "_" +
                              "no_series_description" + "-" + 
                              seriesNumberString + "-" + 
                              instanceNumberString + ".dcm";
                    
                dstPathCmd = directoryToCopyTo.getAbsolutePath( ) + 
                             System.getProperty( "file.separator" ) +
                             "no_series_description";

                dstFileName = dstFileName.replace( '<', '_' );
                dstFileName = dstFileName.replace( '>', '_' );
                
                
                
                dstPathCmd = dstPathCmd.replace( '<', '_' );
                dstPathCmd = dstPathCmd.replace( '>', '_' );
                
                
            }
            // test whether the path exists; if not than create it
            File path = new File( dstPathCmd );
            if ( path.exists( ) == false )
            {
                path.mkdir( );
            }

            try 
            {
                // Create channel on the source
                FileChannel srcChannel = new FileInputStream( srcFileName ).getChannel( );
    
                // Create channel on the destination
                FileChannel dstChannel = new FileOutputStream( dstFileName ).getChannel( );
                
                srcChannel.force( true );
                
                // Copy file contents from source to destination
                dstChannel.transferFrom( srcChannel, 0, srcChannel.size( ) );
                
                dstChannel.force( true );
                
                // Close the channels
                srcChannel.close( );
                dstChannel.close( );
            } 
            catch ( IOException ex ) 
            {
                System.out.println( ">>>>>>>>>>>>>>>>>>> IOException in copying dicom dir" );
                failedInputFileRefs.add( srcFileName );
                failedOutputFileRefs.add( dstFileName );
            }
        }
    }
    

    /**
     * Shows the DICOM information of the first image of the selected series
     */
    public void showDicomInformation( )
    {
        LinkedList<String> filesInSeries = m_dicomDataLoaderPanel.getSelectedSeriesFileReferences( );
        DicomImageData imaDat = new DicomImageData( filesInSeries.getFirst( ) );
        DicomHeaderViewer view = new DicomHeaderViewer( m_parent, imaDat.getHeader( ) );
    }

    /**
     * This routine performs the actual regridding
     */
    private int[] performActualRegriddingByFactorTwo( int imageWidth, int imageHeight, int[] pixelData )
    {
        int[] regridIma = new int[ imageWidth * imageHeight / ( 2 * 2 ) ];
            
        int cntr = 0;

        for ( int i = 0 ; i < ( imageHeight / 2 ) ; i++ )
        {

           for ( int j = 0 ; j < ( imageWidth / 2 ) ; j++ )
           {
                int sum = 0;
                
                int ii = 2 * i;
                int jj = 2 * j;

                sum += pixelData[ ( ii * imageWidth ) + jj ];
                sum += pixelData[ ( ii * imageWidth ) + jj + 1 ];
                sum += pixelData[ ( ( ii + 1 ) * imageWidth ) + jj ];
                sum += pixelData[ ( ( ii + 1 ) * imageWidth ) + jj + 1 ];

                regridIma[ cntr ] = sum / 4;
                
                cntr++;
            }

        }

        return regridIma;
    }

    /**
     * Copies and renames dicom files:
     * Rename from numeric "filename" to 
     *      PatientName_SeriesDescription_SeriesNumber_InstanceNumber.dcm
     * In which the filename parts stem from the dicom header
     * For each SeriesDescription a separate sub directory with name 
     * "SeriesDescription" is created.
     */
    private void createRegriddedDICOMFilesOfSelectedImageSeries( int newHorizontalWidth, int newVerticalHeight )
    {

        int factor = 0;
        // ask user to which directory the data must be regridded and stored
        File directoryToCopyTo = getDicomDirectory( true );

        int seriesNumberToRegrid = m_dicomDataLoaderPanel.getSelectedSeriesNumber( );

        LinkedList list = m_dicomDataLoaderPanel.getDicomDir( ).getAllFileRefences( );
               
        Iterator it = list.iterator( );
        while ( it.hasNext( ) )
        {
            String srcFileName;
            srcFileName = m_dicomDataLoaderPanel.getDicomDir( ).getPath( ) + System.getProperty( "file.separator" ) + (String)it.next( );
                    
            DicomImageData d_ima_data = new DicomImageData( srcFileName );
            FileInfo fi = d_ima_data.getHeader( );

            factor = fi.getImageWidth( ) / newHorizontalWidth;

            int imageWidth = fi.getImageWidth( );
            int imageHeight = fi.getImageHeight( );

            if ( ( new Integer( fi.getSeriesNumber( ).trim( ) ).intValue( ) == seriesNumberToRegrid ) &&
                 ( factor > 1 ) )
            {

                int[] regriddedImage = new int[ d_ima_data.getPixelData( ).length ];

                for ( int i=0;i< regriddedImage.length ; i++ )
                {
                    regriddedImage[i] = d_ima_data.getPixelData( )[i];
                }

                for ( int i = 0 ; i < factor / 2; i++ )
                {
                    regriddedImage = performActualRegriddingByFactorTwo( imageWidth, imageHeight, regriddedImage );

                    // after one time the image is halved!
                    imageWidth  = imageWidth / 2;
                    imageHeight = imageHeight / 2;
                }

                String instanceNumberString = "";
                int instanceNumber = new Integer( fi.getInstanceNumber( ).trim( ) ).intValue( );

                if ( ( instanceNumber < 10 ) && ( instanceNumber > -1 ) )
                {
                    instanceNumberString = "000" + fi.getInstanceNumber( ).trim( );
                }

                if ( ( instanceNumber < 100 ) && ( instanceNumber > 9 ) )
                {
                    instanceNumberString = "00" + fi.getInstanceNumber( ).trim( );
                }

                if ( ( instanceNumber < 999 ) && ( instanceNumber > 99 ) )
                {
                    instanceNumberString = "0" + fi.getInstanceNumber( ).trim( );
                }

                if ( ( instanceNumber < 9999 ) && ( instanceNumber > 999 ) )
                {
                    instanceNumberString = "" + fi.getInstanceNumber( ).trim( );
                }

                String seriesNumberString = "";

                int seriesNumber = new Integer( fi.getSeriesNumber( ).trim( ) ).intValue( );

                if ( ( seriesNumber < 10 ) && ( seriesNumber > -1 ) )
                {
                    seriesNumberString = "000" + fi.getSeriesNumber( ).trim( );
                }

                if ( ( seriesNumber < 100 ) && ( seriesNumber > 9 ) )
                {
                     seriesNumberString = "00" + fi.getSeriesNumber( ).trim( );
                }

                if ( ( seriesNumber < 999 ) && ( seriesNumber > 99 ) )
                {
                    seriesNumberString = "0" + fi.getSeriesNumber( ).trim( );
                }

                if ( ( seriesNumber < 9999 ) && ( seriesNumber > 999 ) )
                {
                    seriesNumberString = "" + fi.getSeriesNumber( ).trim( );
                }

                String dstPath;
                String dstFileName;

                dstPath = directoryToCopyTo.getAbsolutePath( ) +
                          System.getProperty( "file.separator" ) +
                          fi.getSeriesDescription( ).trim( ) + "_regridded" + "_" + newHorizontalWidth + "_" + newVerticalHeight + "_";

                dstFileName = dstPath + System.getProperty( "file.separator" ) +
                              fi.getPatientName( ).replace( ' ', '_' ) + "_" +
                              fi.getSeriesDescription( ).trim( ) + "-" +
                              "regridded" + "_" + newHorizontalWidth + "_" + newVerticalHeight + "_" +
                              seriesNumberString + "-" +
                              instanceNumberString + ".dcm";

                String dstPathCmd = dstPath;

                dstFileName =  eliminateIllegalFileSystemCharacters( dstFileName );
                
                dstPathCmd = eliminateIllegalFileSystemCharacters( dstPathCmd );

                System.out.println( dstPath );

                // test whether the path exists; if not than create it
                File path = new File( dstPathCmd );
                if ( path.exists( ) == false )
                {
                    System.out.println("Creating  path : " + dstPath );
                    path.mkdir( );
                }

                // try to write the new DICOM file

                // first update all DICOM tags for new image series
                fi.setImageHeight( imageWidth );
                fi.setImageWidth( imageHeight );

                fi.setPixelWidth( "" + ( new Float( fi.getPixelWidth( ).trim( ) ) * factor ) );
                fi.setPixelHeight( "" + ( new Float( fi.getPixelHeight( ).trim( ) ) * factor ) );

                fi.setPixelSpacing(  fi.getPixelWidth() + '\\' + fi.getPixelHeight( ) );


                fi.setSeriesNumber( "" + ( (   new Integer( fi.getSeriesNumber( ).trim( ) ).intValue( ) ) + 1000 ) );
                
                // some reformatted image data do not contain a slice position tag
                if ( fi.getSliceLocation( ).compareTo( "N.A." ) == 0 )
                {
                    fi.setSliceLocation( "" + new Float( fi.getInstanceNumber( ) ).floatValue( ) );
                }

                long currentTime  = System.currentTimeMillis( );
                String uid_prefix = "1.3.12.2.1107.5.2";

                DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
                String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );

                String imageInstanceUID = uid_prefix + IP_address + currentTime;

                fi.setSOPInstanceUID( imageInstanceUID );
                fi.setSeriesInstanceUID( uid_prefix + IP_address + ( currentTime - 4 ) );


                fi.setSeriesDescription( "Regridded-" + fi.getImageWidth( ) + "x" + fi.getImageHeight( ) + "of " + fi.getSeriesDescription( ) );

                writeAsTrueDicom( dstFileName, fi, regriddedImage );
                        
            }
        }
    }


    /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters( String cmd )
    {

        // first get rid ofm blancs
        cmd = cmd.trim( );

        cmd = cmd.replace( ' ', '_' );
        cmd = cmd.replace( ',', '_' );
        cmd = cmd.replace( ';', '_' );
        cmd = cmd.replace( '%', '_' );
        //cmd.replace( '*', '_' );

        cmd = cmd.replace( 'é', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'è', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'ö', 'o' ); // problematic on some linux systems
        cmd = cmd.replace( 'ü', 'u' ); // problematic on some linux systems
        cmd = cmd.replace( 'à', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'ä', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ö', 'O' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ü', 'U' ); // problematic on some linux systems

        cmd = cmd.replace( '@', '_' ); // problematic on some linux systems
        cmd = cmd.replace( '#', '_' ); // problematic on some linux systems

        cmd = cmd.replace( '?', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '¢', 'c' ); // problematic on some linux systems
        cmd = cmd.replace( '!', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '§', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '¦', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '$', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '[', '_' );  // problematic on some linux systems
        cmd = cmd.replace( ']', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '^', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems

        cmd = cmd.replace( 'ç', 'c' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '"', '_' );  // problematic on some linux systems

        cmd = cmd.replace(">", "_" );
        cmd = cmd.replace("<", "_" );

        cmd = cmd.replace("____", "_" );
        cmd = cmd.replace("___", "_" );
        cmd = cmd.replace("__", "_" );

        return cmd;
    }



    /**
     * Writes the FileInfo together with the pixel information into a true
     * DICOM file.
     */
    public int writeAsTrueDicom( String fileName, FileInfo finfo, int[] pixelData )
    {
        try
        {
            RandomAccessFile fout = new RandomAccessFile( fileName, "rw" );

            byte[] leadingZeroes = new byte[128];

            fout.write( leadingZeroes );

            String dicm = new String( "DICM" ).trim( );

            fout.write( dicm.getBytes( ) );

            finfo.writePatientRelatedTags( fout );
            finfo.writeStudySeriesInstancesRelatedTags( fout );
            finfo.writeMRRelatedTags( fout );
            finfo.writeGeometryRelatedTags( fout );
            finfo.writeOtherTags( fout );

            writePixelInformation( fout, pixelData );

            fout.close( );

            return IO_OK;
        }
        catch ( IOException ex )
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

        return 0;
    }


    /**
     * Depending on the type of the pixel information.
     * @param fout
     */
    private void writePixelInformation( RandomAccessFile fout, int[] pixelInformation ) throws IOException
    {

        //int outp0 = Integer.rotateLeft( DicomDecoder.PIXEL_DATA, 8 );
        fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.PIXEL_DATA ), 16 ) );

        fout.writeShort( DicomDecoder.OW );

        byte[] pixels2write = new byte[ 2 * pixelInformation.length ];

        for ( int i = 0; i < pixelInformation.length; i++ )
        {
            // System.out.println( "pixel information : " + pixelInformation[ i ] );
            org.spectrim.lib.Convert.int2wordl( pixels2write, 2 * i, pixelInformation[ i ] );
        }

        byte[] byteArr = new byte[2];

        fout.write( byteArr );

        fout.writeInt( Integer.reverseBytes( pixels2write.length ) );

        fout.write( pixels2write );

    }

    
    /**
     * Returns the Dicom directory selected by the user.
     */
    private File getDicomDirectory( boolean setParentAsDefault )
    {   
        DicomDefaults fdef = DicomDefaults.getInstance( );
        
        String currentPath = fdef.getProperty( "Dicom mount path" );

        if ( setParentAsDefault == true )
        {
            currentPath = currentPath.substring( 0, currentPath.lastIndexOf( System.getProperty( "file.separator" ) ) );
        }

        JFileChooser fileChooser = new JFileChooser( currentPath + System.getProperty( "file.separator" ) );
        
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        int result = fileChooser.showOpenDialog( this );
        
        File selected_dir = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                DicomDefaults def = DicomDefaults.getInstance( );
                                                def.setProperty( "Dicom rename mount path", selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null )
                                                {
                                                    m_dicomDirectorySet = true;
                                                }
                                                
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selected_dir;
    } 
    
    
    /**
     * Returns the vector of all Dicom file references of currently 
     * selected case.
     * @return 
     */
    public Vector getDicomFileReferences( )
    {
        return m_dicomDataLoaderPanel.getDataFileReferences( m_selectedSeriesName, m_currentlyLoadedSliceNumber );
    }
    
    /**
     * Returns a representative Dicom header as byte array of 
     * the current dataset. 
     * @return 
     */
    public byte[] getDicomHeaderAsByteArray( )
    {
        Vector refVec = getDicomFileReferences( );
        return m_dicomDataLoaderPanel.getDicomHeaderAsByteArray( (String)refVec.elementAt( 0 ) );
    }
    
    /**
     * Returns the image number.
     * @return 
     */
    public int getCurrentSlice( )
    {
        return m_currentlyLoadedSliceNumber;
    }
    
    /**
      
     * @return 
     */
    public int getImageHeight( )
    {
        return m_imageHeight;
    }
    
    /**
     * Returns the image width.
     * @return 
     */
    public int getImageWidth( )
    {
        return m_imageWidth;
    }
    
    /**
     * Returns the number of images.
     * @return 
     */
    public int getNumberOfSlices( )
    {
        return m_differentSlicePositions;
    }
    
    /**
     * Returns the LinkedList of selected Dicom file references.
     * @return 
     */
    public LinkedList<String> getSelectedDicomFileReferences( )
    {
        return m_selectedDicomFileReferences;
    }
    
    /**
     * Returns the number of time points.
     * @return 
     */
    public int getNumberOfTimePoints( )
    {
        return m_numberOfTimePoints;
    }
    
    /**
     * Returns the series number of the currently selected series.
     * @return 
     */
    public int getCurrentSeriesNumber( )
    {
        return m_dicomDataLoaderPanel.getCurrentlySelectedSeriesNumber( );
    }
    
    /**
     * Returns the patient name attribute.
     * @return 
     */
    public String getPatientName( )
    {
        return m_dicomDataLoaderPanel.getPatientName( );
    }
    
    /**
     * Returns whether the data is stored in one series or in different
     * series.
     * @return 
     */
    public boolean areImagesFilesStoredInOneSeries( )
    {
        return m_allInstancesInOneSeriesRadioButton.isSelected( );
    }
    
    /**
     * Returns whether a image series or spectrum data is selected
     * @return: boolean
     */
    public boolean isImageSeriesSelected( )
    {
    	return m_dicomDataLoaderPanel.isImageSeries( );
    }
    
    
    // Attributes
    private int[] m_data;
    

    private final JPanel m_contentPanel;
    private JPanel m_perfusionImagingParameterPanel;
    private JPanel m_modalityPanel;
    
    private JPanel m_imageSeriesDetailsPanel;
    private JPanel m_numberOfImagesPanel;
    
    private JLabel m_selectedImagesInSeriesLabel;
    private JLabel m_selectedImagesInSeriesValue;
    
    private JLabel m_differentSlicePositionsLabel;
    private JLabel m_differentSlicePositionsValue;

    private JLabel m_differentTimePointsLabel;
    private JLabel m_differentTimePointsValue;
    
    private JPanel m_timeDependencyStorageTypePanel;
    private JLabel m_timeDependencyLabel;
    
    // 
    private JRadioButton m_allInstancesInOneSeriesRadioButton;
    private JRadioButton m_instancesinDifferentSeriesRadioButton;
    
    private String m_selectedSeriesName;
    
    private int m_currentlyLoadedSliceNumber;
    
    private final int m_imageWidth = -1;
    private final int m_imageHeight = -1;
    private final int m_differentSlicePositions = -1;
    private final int m_numberOfTimePoints = -1;
    
    private final DicomDataLoaderPanel m_dicomDataLoaderPanel; 
    
    private JButton m_loadButton;
    private JButton m_cancelButton;
    private JPanel m_selectionPanel;
    
     // Listeners to changes in this class
    private final LinkedList <ActionListener> m_listeners = new LinkedList<>( );
    
    // Indicates whether a valied series has been selected
    
    // events raised by accept and cancel button
    public final static int LOAD_EVENT = 0;
    public final static int CANCEL_EVENT = 1;
    public final static int COPY_AND_RENAME_MOUNTED_DIRECTORY_EVENT = 2;
    
    private boolean m_dicomDirectorySet = false;
    
    private LinkedList<String> m_selectedDicomFileReferences;

    DicomRegridImageSeriesPropertiesFrame m_dicomRegridImageSeriesPropertiesFrame;


    public final static int IO_OK = 0;

    // Must be declarated somehow
    private static final long serialVersionUID = 3487495845819293L;
}
