 /*
 * File: $Id: DicomSliceCanvas.java,v 1.5 2006/11/09 10:11:00 hans Exp $
 * History
 * Date: 31.01.2003  Authors: Martin Buerki / Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;


import java.awt.Rectangle;
import java.awt.Polygon;
import java.awt.Image;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.GraphicsEnvironment;
import java.awt.Transparency;
import java.awt.GraphicsDevice;
import java.awt.GraphicsConfiguration;
import java.awt.HeadlessException;
import java.awt.Font;

import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.PixelGrabber;
import java.awt.image.ImageObserver;
import java.awt.image.MemoryImageSource;
import java.awt.image.ColorModel;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.LinkedHashMap;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.text.NumberFormat;
import java.text.DecimalFormat;

import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.imagegraphicobjects.ImageCrossLines;
import org.spectrim.imagegraphicobjects.Distance;

import org.spectrim.lib.Tiff;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;



public class DicomSliceCanvas extends JPanel implements MouseListener, ActionListener
{

    /**
     * Constructor needs to know its JFrame parent object
     */
    public DicomSliceCanvas( JFrame parent )
    {
        m_parent = parent;
        setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
        
        // set component to be double buffered
        this.setDoubleBuffered( true );
        this.setOpaque( true );
        
        m_zoom = 1f;
        zoomQuality = Image.SCALE_FAST;
        CMapSize = 256;
        m_colorMapType = CMAP_GRAYSCALE;
        drawScale = false;
        m_scaleRectangle = new Rectangle( );
        scalePosition = POSITION_BOTTOMLEFT;
        m_redrawImage = true;
        setBackground( Color.black );
        m_title = new String( "" );
        setAutoWindow( );
    }
    
    
    /**
     *  Returns the parent frame
     * @return
     */
    public JFrame getParentFrame( )
    {
        return m_parent;
    }
    
    /**
     * Defines the image position on the Graphics2D g object
     * You must only set these attributes if you want to print,
     * for normal displaying om the screen the layout manager manages
     * the postioning. 
     */
    public void setImagePosition( int imageLeft, int imageTop )
    {
       m_imageLeft = imageLeft;
       m_imageTop = imageTop;
    }
    
    /**
     * Returns the image top position on the Graphics2D g object
     */
    public int getImagePositionTop( )
    {
       return m_imageTop;
    }
    
    /**
     * Returns the image top position on the Graphics2D g object
     */
    public int getImagePositionLeft( )
    {
       return m_imageLeft;
    }
    
    /**
     * Sets the title string of the SliceCanvas object
     */
    public void setTitle( String title ) 
    {
        m_title = new String( title );
    }
    
    /**
     * Returns the title string of the SliceCanvas object
     */
    public String getTitle( ) 
    { 
        return m_title; 
    }
    
    /**
     * Returns the abbreviated title: if the files are named conform the
     * convention this method returns -tp, -cbv etc
     */
    public String getShortImageTitle( )
    {
        if ( m_title.length( ) < 8 )
        {
            return "Pixel value : ";
        }
        else
        {
            return m_title.substring( m_title.length( ) - 8, m_title.length( ) - 4 );
        }
    }
    
    /**
     * Set the windowing values automatic
     */
    public void setAutoWindow( ) 
    {
        m_brightness = 1;
        m_contrast = CMapSize - 1 - m_brightness;
    }
    
    /**
     * Returns the current brightness value
     */
    public int getBrightness( ) 
    { 
        return m_brightness; 
    }
    
    /**
     * Returns the current contrast value
     */
    public int getContrast( ) 
    { 
        return m_contrast; 
    }

    /**
     * Sets the brightness value only
     */
    public void setBrightness( int b ) 
    {
        m_brightness = b;
        setColormap( b, m_contrast );
    }
    
    /**
     * Sets the contrast value only and re-calculates a colormap
     */
    public void setContrast( int c ) 
    {
        m_contrast = c;
        setColormap( m_brightness, c );
    }
    
    /**
     * Sets the type of colormap
     */
    public void setColormapType( int type ) 
    {
        if ( type < CMAP_GRAYSCALE || type > CMAP_COLOR_INVERSE )
        {
            System.out.println( "SliceCanvas.setColormapType( ) warning: undefined type, using default type" );
            m_colorMapType = CMAP_GRAYSCALE;
        }
        else
        {
            m_colorMapType = type;
        }
        
        setColormap( );
        
        notifyActionListeners( COLOR_MAP_CHANGED_EVENT, COLOR_MAP_CHANGED_EVENT_COMMAND );
        
    }

    /**
     * Sets the type of colormap
     */
    public void setColormapTypeWithoutNotification( int type ) 
    {
        if ( type < CMAP_GRAYSCALE || type > CMAP_COLOR_INVERSE )
        {
            System.out.println( "SliceCanvas.setColormapType( ) warning: undefined type, using default type" );
            m_colorMapType = CMAP_GRAYSCALE;
        }
        else
        {
            m_colorMapType = type;
        }
        
        setColormap( );
        update( true );
    }
    
    
    /**
     * Returns the type of colormap member
     */
    public int getColormapType( ) 
    { 
        return m_colorMapType; 
    }
    
    /**
     * Sets the colormap brightness and contrast attributes
     */
    public void setColormap( int brightness, int contrast ) 
    {

        if ( brightness < 1 )
        { 
            this.m_brightness = 1;
        }
        else if ( brightness > CMapSize - 2 )
        { 
            this.m_brightness = CMapSize - 2;
        }
        else
        {
            this.m_brightness = brightness;
        }
        if ( contrast < 1 )
        { 
            this.m_contrast = 1;
        }
        else
        { 
            this.m_contrast = contrast;
        }
        
        setColormap( );
        
        notifyActionListeners( DicomSliceCanvas.WINDOWING_EVENT,
                               DicomSliceCanvas.WINDOWING_EVENT_COMMAND );
    }

    
     /**
     * Sets the colormap brightness and contrast attributes
     */
    public void setColormapWithoutNotification( int brightness, int contrast ) 
    {

        if ( brightness < 1 )
        { 
            this.m_brightness = 1;
        }
        else if ( brightness > CMapSize - 2 )
        { 
            this.m_brightness = CMapSize - 2;
        }
        else
        {
            this.m_brightness = brightness;
        }
        if ( contrast < 1 )
        { 
            this.m_contrast = 1;
        }
        else
        { 
            this.m_contrast = contrast;
        }
        
        setColormap( );
        update( true );
        
    }
    
    
    /**
     * Calculates a IndexColorMap object to be used to display 
     * the image data
     */
    protected void setColormap( ) 
    {

        // Set colours
        byte a[ ] = new byte[ 3 * CMapSize ];
        int numColors = CMapSize;

            // int firstIndex, lastIndex;
        float R, G, B;
        int q0 = 0, q1 = 0, q2 = 0, q4 = 0;

        m_firstCMapIndex = m_brightness;

        int bc = m_brightness + m_contrast;
        
        if ( bc < numColors )
        {
            m_lastCMapIndex = bc;
        }
        else
        {
            m_lastCMapIndex = numColors - 1;
        }
        
        if ( ( m_raw2displayScale == 0f ) && ( m_fileInfo != null ) )
        {
            m_displayedMinVal = m_rawDataMinVal;
            m_displayedMaxVal = m_rawDataMaxVal;
        }
        else
        {
            if ( m_fileInfo != null )
            {
                m_displayedMinVal = (int)( (float)( m_firstCMapIndex - 1 )  / m_raw2displayScale )
                                   + m_rawDataMinVal;
                m_displayedMaxVal = (int)( (float)( m_lastCMapIndex - 1 ) / m_raw2displayScale )
                                   + m_rawDataMinVal;
            }
        }

        if ( m_colorMapType == CMAP_GRAYSCALE )
        {
            float dR = 255f / (float)m_contrast;
            R = 0f;

            for ( int k = 0; k < 3 * m_firstCMapIndex; k += 3 )
            {
                a[k+0] = (byte)0;
                a[k+1] = (byte)0;
                a[k+2] = (byte)0;
            }
            for ( int k = 3 * m_firstCMapIndex; k < 3 * m_lastCMapIndex; k += 3 )
            {
                a[k+0] = (byte)Math.round(R);
                a[k+1] = (byte)Math.round(R);
                a[k+2] = (byte)Math.round(R);
                R += dR;
            }
            for ( int k = 3 * m_lastCMapIndex; k < 3 * numColors; k += 3 )
            {
                a[k+0] = (byte)255;
                a[k+1] = (byte)255;
                a[k+2] = (byte)255;
                
            }
        } /* colorMapType == CMAP_GRAYSCALE */
        else if ( m_colorMapType == CMAP_COLOR )
        {
            int qlength = m_contrast / 4;
            q0 = m_brightness;
            q1 = m_brightness + qlength;
            if ( q1 >= CMapSize )
            { 
                q1 = CMapSize - 1;
            }
            q2 = m_brightness + 2 * qlength;
	    
            if ( q2 >= CMapSize )
            { 
                q2 = CMapSize - 1;
            }
            q4 = m_brightness + m_contrast;
            if ( q4 >= CMapSize )
            { 
                q4 = CMapSize - 1; 
            }
            float dR, dG, dB;

            //System.out.println(q0 + "  " + q1 + " " + q2 + " " + q4);

            R = 0f; G = 0f; B = 0f;
            dR = 0f;
            dG = 0f;
            dB = 0f; //(q0>0 ? 255f/(float)q0 : 0f);
            for ( int k = 0; k < 3 * q0; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            R = 0f; G = 0f; B = 255f;
            dR = 0f;
            dG = 255f / (float)( q1 - q0 );
            dB = -dG;
            for ( int k = 3 * q0; k < 3 * q1; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            R = 0f; G = 255f; B = 0f;
            dR = 255f/(float)( q2 - q1 );
            dG = 0f;
            dB = 0f;
            for ( int k = 3 * q1; k < 3 * q2; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            R = 255f; G = 255f; B = 0f;
            dR = 0f;
            dG = -255f/(float)(q4-q2);
            dB = 0f;
            for ( int k = 3 * q2; k < 3 * q4; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            for ( int k = 3 * q4; k < 3 * CMapSize; k += 3 )
            {
                a[k+0] = (byte)255;
                a[k+1] = (byte)0;
                a[k+2] = (byte)0;
            }
        } /* colorMapType == CMAP_COLOR */
        else if ( m_colorMapType == CMAP_COLOR_INVERSE )
        {
            int qlength = m_contrast / 4;
            q0 = m_brightness;
            q1 = m_brightness + 2 * qlength;
            if ( q1 >= CMapSize ) q1 = CMapSize - 1;
            q2 = m_brightness + 3 * qlength;
            if ( q2 >= CMapSize ) q2 = CMapSize - 1;
            q4 = m_brightness + m_contrast;
            if ( q4 >= CMapSize ) q4 = CMapSize - 1;
            float dR, dG, dB;

            // System.out.println(q0 + "  " + q1 + " " + q2 + " " + q4);

            R = 0f; G = 0f; B = 0f;
            dR = ( q0 > 0 ? 255f/(float)q0 : 0f );
            dG = 0f;
            dB = 0f;
            for ( int k = 0; k < 3 * q0; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            R = 255f; G = 0f; B = 0f;
            dR = 0f;
            dG = 255f/(float)( q1 - q0 );
            dB = 0f;
            for ( int k = 3 * q0; k < 3 * q1; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            R = 255f; G = 255f; B = 0f;
            dR = -255f/(float)( q2 - q1 );
            dG = 0f;
            dB = 0f;
            for ( int k = 3 * q1; k < 3 * q2; k += 3 )
            {
                a[k+0] = (byte)R;
                a[k+1] = (byte)G;
                a[k+2] = (byte)B;
                R+=dR;
                G+=dG;
                B+=dB;
            }
            R = 0f; G = 255f; B = 0f;
            dR = 0f;
            dG = -255f / (float)( q4 - q2 );
            dB = -dG;
            for ( int k=3 * q2; k < 3 * q4; k += 3 )
            {
            a[k+0] = (byte)R;
            a[k+1] = (byte)G;
            a[k+2] = (byte)B;
            R+=dR;
            G+=dG;
            B+=dB;
            }
            for ( int k = 3 * q4; k < 3 * CMapSize; k += 3 )
            {
                a[k+0] = (byte)0;
                a[k+1] = (byte)0;
                a[k+2] = (byte)255;
            }

        } /* colorMapType == CMAP_COLOR_INVERSE */
	

        // Create colour model.
        int n = CMapSize - 1;
        int nbits = 0;
        while ( n > 0 )
        {
            n >>>= 1;
            nbits++;
        }
        
        cmap = new IndexColorModel( nbits, CMapSize, a, 0, false );
        
    }
  
    /**
     * Sets and displays the processed image data
     */
    public void setProcessedImageData( int[] data )
    {    
        int dataSize = m_imageWidth * m_imageHeight;

        if ( m_rawData == null || m_rawData.length != dataSize )
        {
            m_rawData = new int[dataSize];
        }
        if ( m_scaledData == null || m_scaledData.length != dataSize )
        {
            m_scaledData = new int[dataSize];
        }
        
        System.arraycopy( data, dataSize * m_currentlyDisplayedImageNumber, m_rawData, 0, dataSize );

        // scale rawData to the range 0..CMapSize
        //int minVal= rawData[0];
        m_rawDataMinVal = Integer.MAX_VALUE;
        m_rawDataMaxVal= 0;
        for ( int k = 0; k < dataSize; k++ )
        {
            //if (rawData[k]<30 && rawData[k]>0) System.out.println("Warning: low value (k="+k+", val="+rawData[k]+")");

            if ( m_rawData[k] == 0 ) continue;
            if ( m_rawData[k] > m_rawDataMaxVal )
            {
                m_rawDataMaxVal = m_rawData[k];
            }
            else if ( m_rawData[k] < m_rawDataMinVal )
            {
                m_rawDataMinVal = m_rawData[k];
            }
        }
        
	
        m_displayedMinVal = m_rawDataMinVal;
        m_displayedMaxVal = m_rawDataMaxVal;

        if ( m_rawDataMaxVal - m_rawDataMinVal > 0 )
        {
            m_raw2displayScale = (float)( CMapSize - 1 ) / (float)( m_rawDataMaxVal - m_rawDataMinVal );
        }
        else
        {
            m_raw2displayScale = 0f;
        }        
        
        // System.out.println("min = " + rawDataMinVal + " max = " + rawDataMaxVal + " => m = " + m);
	
        for ( int k = 0; k < dataSize; k++ )
        {
            if ( m_rawData[k] == 0 )
            {
                m_scaledData[k] = 0;
            }
            else
            {
                m_scaledData[k] = (int)( m_raw2displayScale * (float)( m_rawData[k] - m_rawDataMinVal ) );
            }
        }

        setSize( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) );
        
        setPreferredSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMinimumSize( new Dimension( (int)( m_zoom * m_imageWidth ),  (int)( m_zoom * m_imageHeight ) ) );
        setMaximumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        
        calculateImagePixels( );
        
        drawImage( false );
    }
    
    /**
     * Sets the number of images
     *  
     * @param numberOfImages 
     * @return
     */
    public void setNumberOfImages( int numberOfImages )
    {
       m_numberOfImages = numberOfImages; 
    }
    
    /**
     * Returns the number of images
     *  
     * @param numberOfImages 
     * @return int m_numberOfImages
     */
    public int getNumberOfImages( )
    {
       return m_numberOfImages; 
    }
    
    /**
     * Sets the image pixel data
     * @param data
     */
    public void setImageSeriesPixelData( int[] data )
    {
         m_imagePixelData = data;
         m_rawData = m_imagePixelData;
    }
    
    /**
     * Sets the image data and causes slicenumber 'slice' to be displayed
     */
    public void setImageOfSeries( int[] data, int width, int height, int slice, int series_size ) 
    {
        
        m_currentlyDisplayedImageNumber = slice;
        m_currentlyDisplayedDTIOverlayImageNumber = slice;
        m_currentlyDisplayedfMRIOverlayImageNumber = slice;

        int totalDataSize = width * height * series_size;
        int dataSize = width * height;
        
        m_numberOfImages = series_size;
        m_imageWidth = width;
        m_imageHeight = height;
        
        if ( m_rawData == null || m_rawData.length != totalDataSize )
        {
            m_rawData = new int[totalDataSize];
        }
        if ( m_scaledData == null || m_scaledData.length != totalDataSize )
        {
            m_scaledData = new int[totalDataSize];
        }
        if ( this.m_imagePixelData == null || m_imagePixelData.length != totalDataSize )
        {
            m_imagePixelData = new int[totalDataSize];
        }
        
        
        System.arraycopy( data, 0, m_rawData, dataSize * slice, dataSize );
        
        
        
        // scale rawData to the range 0..CMapSize
        //int minVal= rawData[0];
        if ( this.m_forceRawMinMaxPixelValue == false )
        {
            m_rawDataMinVal = Integer.MAX_VALUE;
            m_rawDataMaxVal= 0;
            for ( int k = 0; k < dataSize; k++ ) 
            {
                //if (rawData[k]<30 && rawData[k]>0) System.out.println("Warning: low value (k="+k+", val="+rawData[k]+")");

                if ( m_rawData[k] == 0 ) continue;
                if ( m_rawData[k] > m_rawDataMaxVal ) 
                {
                    m_rawDataMaxVal = m_rawData[k];
                }
                else if ( m_rawData[k] < m_rawDataMinVal ) 
                {
                    m_rawDataMinVal = m_rawData[k];
                }
            }
            
            //displayedMinVal = m_rawDataMinVal;
            //displayedMaxVal = m_rawDataMaxVal;
        }
        else
        {
            //displayedMinVal = m_forceRawDataMinVal;
            //displayedMaxVal = m_forceRawDataMaxVal;
            m_rawDataMinVal = m_forceRawDataMinVal;
            m_rawDataMaxVal = m_forceRawDataMaxVal;
        }

        if ( m_rawDataMaxVal - m_rawDataMinVal > 0 )
        {
            m_raw2displayScale = (float)( CMapSize - 1 ) / (float)( m_rawDataMaxVal - m_rawDataMinVal );
        }
        else
        {
            m_raw2displayScale = 0f;
        }
        
        // System.out.println("min = " + rawDataMinVal + " max = " + rawDataMaxVal + " => m = " + m);
	
        for ( int k = 0; k < dataSize; k++ )
        {
            if ( m_rawData[k] == 0 )
            {
                m_scaledData[k] = 0;
            }
            else
            {
                m_scaledData[k] = (int)( m_raw2displayScale * (float)( m_rawData[k] - m_rawDataMinVal ) );
            }
        }

        setSize( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) );
        
        setPreferredSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMinimumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMaximumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        
        calculateImagePixels( );
        
        setGeometricSliceAttributes( );
        
        // let the change listeners know that the displayed image has changed
        notifyChangeListeners( );
        
        // Defines where the pager 'object' should be positioned
        definePagerLocation( );
        enableImagePaging( true );
    }
    
    /**
     * Sets the image data and causes slicenumber 'slice' to be displayed
     */
    public void setImage( int[] data, int width, int height, int slice ) 
    {
        
        // m_numberOfImages = 1;
        
        m_currentlyDisplayedImageNumber = slice;
        m_currentlyDisplayedDTIOverlayImageNumber = slice;
        m_currentlyDisplayedfMRIOverlayImageNumber = slice;

        int dataSize = width * height;
        
        // Only set the number of images if this number is not defiend yet 
        if ( ( dataSize != 0 ) && ( m_numberOfImages == 0 ) )
        {
            m_numberOfImages = data.length / dataSize;
        }
        
        if ( m_rawData == null || m_rawData.length != dataSize )
        {
            m_rawData = new int[dataSize];
        }
        if ( m_scaledData == null || m_scaledData.length != dataSize )
        {
            m_scaledData = new int[dataSize];
        }
        m_imageWidth = width;
        m_imageHeight = height;

        System.arraycopy( data, dataSize * slice, m_rawData, 0, dataSize );

        // scale rawData to the range 0..CMapSize
        //int minVal= rawData[0];
        if ( this.m_forceRawMinMaxPixelValue == false )
        {
            m_rawDataMinVal = Integer.MAX_VALUE;
            m_rawDataMaxVal= 0;
            for ( int k = 0; k < dataSize; k++ ) 
            {
                //if (rawData[k]<30 && rawData[k]>0) System.out.println("Warning: low value (k="+k+", val="+rawData[k]+")");

                if ( m_rawData[k] == 0 ) continue;
                if ( m_rawData[k] > m_rawDataMaxVal ) 
                {
                    m_rawDataMaxVal = m_rawData[k];
                }
                else if ( m_rawData[k] < m_rawDataMinVal ) 
                {
                    m_rawDataMinVal = m_rawData[k];
                }
            }
            
            //displayedMinVal = m_rawDataMinVal;
            //displayedMaxVal = m_rawDataMaxVal;
        }
        else
        {
            //displayedMinVal = m_forceRawDataMinVal;
            //displayedMaxVal = m_forceRawDataMaxVal;
            m_rawDataMinVal = m_forceRawDataMinVal;
            m_rawDataMaxVal = m_forceRawDataMaxVal;
        }

        if ( m_rawDataMaxVal - m_rawDataMinVal > 0 )
        {
            m_raw2displayScale = (float)( CMapSize - 1 ) / (float)( m_rawDataMaxVal - m_rawDataMinVal );
        }
        else
        {
            m_raw2displayScale = 0f;
        }
        
        // System.out.println("min = " + rawDataMinVal + " max = " + rawDataMaxVal + " => m = " + m);
	
        for ( int k = 0; k < dataSize; k++ )
        {
            if ( m_rawData[k] == 0 )
            {
                m_scaledData[k] = 0;
            }
            else
            {
                m_scaledData[k] = (int)( m_raw2displayScale * (float)( m_rawData[k] - m_rawDataMinVal ) );
            }
        }

        setSize( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) );
        
        setPreferredSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMinimumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMaximumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        
        calculateImagePixels( );
        
        setGeometricSliceAttributes( );
        
        // let the change listeners know that the displayed image has changed
        notifyChangeListeners( );
        
        // Defines where the pager 'object' should be positioned
        definePagerLocation( );
        
    }
    
    /**
     * Sets the image size, pixels values and Dicom file info header information
     */
    public void setImage( int[] data, int width, int height, int slice, FileInfo finfo, LinkedHashMap fSiemensInfo ) 
    {
        // set the dicom file header info
        m_fileInfo = finfo;
        m_siemensSpecificData = fSiemensInfo;
        
        m_currentlyDisplayedImageNumber = slice;
        
//        m_numberOfImages = 1;

        if ( ( finfo.getPixelWidth( ) != null ) &&
             ( finfo.getPixelHeight( ) != null ) &&
             ( finfo.getSliceThickness( ) != null &&
             ( finfo.getPixelWidth( ).compareTo( "N.A." ) != 0 ) &&
             ( finfo.getPixelHeight( ).compareTo( "N.A." ) != 0 ) &&
             ( finfo.getSliceThickness( ).compareTo( "N.A." ) != 0 ) ) )
        {
            m_volumePerPixel = new Double( finfo.getPixelWidth( ).trim( ) ).doubleValue( ) *
                               new Double( finfo.getPixelHeight( ).trim( ) ).doubleValue( ) *
                               new Double( finfo.getSliceThickness( ).trim( ) ).doubleValue( ) ;
        }
        else
        {
            m_volumePerPixel = 0.0;
        }
        
        int dataSize = width * height;

        if ( m_rawData == null || m_rawData.length != dataSize )
        {
            m_rawData = new int[dataSize];
        }
        if ( m_scaledData == null || m_scaledData.length != dataSize )
        {
            m_scaledData = new int[dataSize];
        }
        m_imageWidth = width;
        m_imageHeight = height;

        System.arraycopy( data, dataSize * slice, m_rawData, 0, dataSize );

        // scale rawData to the range 0..CMapSize
        m_rawDataMinVal = Integer.MAX_VALUE;
        m_rawDataMaxVal= 0;
        for ( int k = 0; k < dataSize; k++ ) 
        {
            if ( m_rawData[k] == 0 ) continue;
            if ( m_rawData[k] > m_rawDataMaxVal ) 
            {
                m_rawDataMaxVal = m_rawData[k];
            }
            else if ( m_rawData[k] < m_rawDataMinVal ) 
            {
                m_rawDataMinVal = m_rawData[k];
            }
        }
       
        m_displayedMinVal = m_rawDataMinVal;
        m_displayedMaxVal = m_rawDataMaxVal;
        
        if (  m_rawDataMaxVal - m_rawDataMinVal > 0 )
        {
            m_raw2displayScale = (float)( CMapSize - 1 ) / (float)( m_rawDataMaxVal - m_rawDataMinVal );
        }
        else
        {
            m_raw2displayScale = 0f;
        }
        
        // System.out.println( "min = " + rawDataMinVal + " max = " + rawDataMaxVal + " => m = " + m );
	
        for ( int k = 0; k < dataSize; k++ )
        {
            if ( m_rawData[k] == 0 )
            {
                m_scaledData[k] = 0;
            }
            else
            {
                m_scaledData[k] = (int)( m_raw2displayScale * (float)( m_rawData[k] - m_rawDataMinVal ) );
                
            }
        }

        setSize( (int)(m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) );
        
        setPreferredSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMinimumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        setMaximumSize( new Dimension( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) ) );
        
        calculateImagePixels( );
        
        // Defines where the pager 'object' should be positioned
        definePagerLocation( );
        
        setGeometricSliceAttributes( );
        
        // let the change listeners know that the displayed image has changed
        notifyChangeListeners( );
    }
    
    
    
    /**
     * Sets the Geometric Slice Attributes, like slice orientation and
     * offset etc based on caller provider information. Application objects
     * which are created on the fly
     */
    public void setComputedGeometricSliceAttributes( Point3D dirVecX, Point3D dirVecY,
                                                     Point3D offset,
                                                     int xOffset, int yOffset,
                                                     double fovx, double fovy )
    {
      
        m_dirVecX = dirVecX;
        m_dirVecY = dirVecY;

        // Define the spatial 3D coordinates of the topleft point in the image
        m_topLeftPoint = offset;
        
        // Compute the FOV sizes width and height of the current slice
        m_FOVX = fovx;
        m_FOVY = fovy;
    }
    
    /**
     * Sets the Geometric Slice Attributes, like slice orientation and
     * offset etc
     */
    private void setGeometricSliceAttributes( )
    {
        if ( m_fileInfo != null )
        {
            String str = this.m_fileInfo.getImageOrientationPatient( );

            if ( ( str != null ) && 
                 ( str.compareTo( "N.A." ) != 0 ) &&
                 ( str.compareTo( "" ) != 0 ) )
            {
                // for invalid headers this string is sometimes not set! therefore if
                m_dirVecX = new Point3D( );
                m_dirVecY = new Point3D( );

                ///////////////////
                int seperatorPos = str.lastIndexOf( '\\' );
                String subStr = str.substring( seperatorPos + 1, str.length( ) );

                m_dirVecY.m_x3 = (new Double( subStr )).doubleValue( );

                str = str.substring( 0, seperatorPos );
                ///////////////////
                seperatorPos = str.lastIndexOf( '\\' );
                subStr = str.substring( seperatorPos + 1, str.length( ) );

                m_dirVecY.m_x2 = (new Double( subStr )).doubleValue( );

                str = str.substring( 0, seperatorPos );
                ///////////////////
                seperatorPos = str.lastIndexOf( '\\' );
                subStr = str.substring( seperatorPos + 1, str.length( ) );

                m_dirVecY.m_x1 = (new Double( subStr )).doubleValue( );

                str = str.substring( 0, seperatorPos );
                ///////////////////

                seperatorPos = str.lastIndexOf( '\\' );
                subStr = str.substring( seperatorPos + 1, str.length( ) );

                m_dirVecX.m_x3 = ( new Double( subStr ) ).doubleValue( );

                str = str.substring( 0, seperatorPos );

                ///////////////////

                seperatorPos = str.lastIndexOf( '\\' );
                subStr = str.substring( seperatorPos + 1, str.length( ) );

                m_dirVecX.m_x2 = ( new Double( subStr ) ).doubleValue( );

                str = str.substring( 0, seperatorPos );

                ///////////////////

                m_dirVecX.m_x1 = ( new Double( str ) ).doubleValue( );

                ///////////////////
                // Define the spatial 3D coordinates of the topleft point in the image

                m_topLeftPoint = new Point3D( m_fileInfo.getImagePositionPatientX( ),
                                              m_fileInfo.getImagePositionPatientY( ),
                                              m_fileInfo.getImagePositionPatientZ( )
                                            );

                // Compute the FOV sizes width and height of the current slice
                if ( ( m_fileInfo.getPixelWidth( ).compareTo( "N.A." ) != 0 )  &&
                     ( m_fileInfo.getPixelHeight( ).compareTo( "N.A." ) != 0 ) &&
                     ( m_fileInfo.getImageWidth( ) != 0 ) &&
                     ( m_fileInfo.getImageHeight( ) != 0 ) )
                {
                    m_FOVX = new Double( m_fileInfo.getPixelWidth( ) ).doubleValue( ) * new Double( m_fileInfo.getImageWidth( ) ).doubleValue( );
                    m_FOVY = new Double( m_fileInfo.getPixelHeight( ) ).doubleValue( ) * new Double( m_fileInfo.getImageHeight( ) ).doubleValue( );
                }

                if ( ( m_fileInfo.getPixelWidth( ).compareTo( "N.A." ) == 0 )  &&
                     ( m_fileInfo.getPixelHeight( ).compareTo( "N.A." ) == 0 ) )
                {
                    m_fileInfo.setPixelWidth( "0.0" );
                    m_fileInfo.setPixelHeight( "0.0" );
                }

                // if the image matrix size has been changed, the top left point must be translated
                // in order to get correct coordinates

                if ( ( m_fileInfo.getInitialImageHeight( ) != m_fileInfo.getImageHeight( ) ) ||
                     ( m_fileInfo.getInitialImageWidth( ) != m_fileInfo.getImageWidth( ) ) )
                {
                    int X = ( m_fileInfo.getImageWidth( ) - m_fileInfo.getInitialImageWidth( ) ) / 2;
                    int Y = ( m_fileInfo.getImageHeight( ) - m_fileInfo.getInitialImageHeight( ) ) / 2;

                    double factorX = -( X / ( new Double( m_fileInfo.getInitialImageWidth( ) ).doubleValue( ) ) ) * new Double( m_fileInfo.getPixelWidth( ) ).doubleValue( ) * m_fileInfo.getInitialImageWidth( );
                    double factorY = -( Y / ( new Double( m_fileInfo.getInitialImageHeight( ) ).doubleValue( ) ) ) * new Double( m_fileInfo.getPixelHeight( ) ).doubleValue( )* m_fileInfo.getInitialImageHeight( );

                    Point3D hori = new Point3D( this.m_dirVecX );
                    hori.scalarMultiply( factorX );

                    Point3D verti = new Point3D( this.m_dirVecY );
                    verti.scalarMultiply( factorY );

                    m_topLeftPoint = m_topLeftPoint.translate( hori );
                    m_topLeftPoint = m_topLeftPoint.translate( verti );

                }
            }
        }
    }


    
    /**
     * Sets the pixel data object ; no displaying just setting
     */
    public void setDicomImages( DicomImageSeriesData imageSeriesData )
    {
        m_dicomImageSeriesData    = imageSeriesData;
        m_numberOfImages          = m_dicomImageSeriesData.getNumberOfImages( );
        m_imagePixelData          = imageSeriesData.getSeriesPixelData( );
        m_currentlyDisplayedImageNumber = m_numberOfImages / 2;

        m_currentlyDisplayedDTIOverlayImageNumber  = m_numberOfImages / 2;
        m_currentlyDisplayedfMRIOverlayImageNumber = m_numberOfImages / 2;

        m_imageHeight = imageSeriesData.getFileInfo( 0 ).getImageHeight( );
        m_imageWidth = imageSeriesData.getFileInfo( 0 ).getImageWidth( );
        
        m_fileInfo = imageSeriesData.getFileInfo( m_numberOfImages / 2 );
        
        // defice a local variable to keep track over nordic pixel scaling..
        m_nordicScaleFactor = m_fileInfo.getNordicPixelScaling( );
        
        determineMinMaxValue( );
    }

    /**
     * Returns the image series data object.
     * @return
     */
    public DicomImageSeriesData getImageSeriesData( )
    {
        return m_dicomImageSeriesData;
    }

    /**
     * Sets the pixel data object ; no displaying just setting
     */
    public void setDicomImages( )
    {
        m_imagePixelData = m_dicomImageSeriesData.getSeriesPixelData( );
        
        m_numberOfImages = m_dicomImageSeriesData.getNumberOfImages( );
        
        m_currentlyDisplayedImageNumber = m_numberOfImages/2;
        
        m_currentlyDisplayedDTIOverlayImageNumber = m_numberOfImages / 2;
        m_currentlyDisplayedfMRIOverlayImageNumber = m_numberOfImages / 2;


        determineMinMaxValue( );
    }   

    
    /**
     * Redefine the m_imagePixelData object ; no displaying just setting.
     * This method allows the image pixeldata to be changed by different
     * pixel values. Usefull method when calculated data need to be 
     * permanently stored, after e.g. thressholding etc. Since a necessary
     * precondition is, that the data.length is equal to the m_imagePixelData
     * length, assures that the method redefines  a l l   slices.
     */
    public synchronized void redefineImageData( int[] data  )
    {
        if ( m_imagePixelData.length == data.length )
        {
            System.arraycopy( data, 0, m_imagePixelData, 0, data.length );
            determineMinMaxValue( );
        }
    }
    
    /**
     * determines the minimum and maximum pixel value of the complete
     * dataset. These  an invariant
     */
    public void determineMinMaxValue( )
    {
        
        m_imagePixelDataMinVal = Integer.MAX_VALUE;
        m_imagePixelDataMaxVal = -Integer.MIN_VALUE;
        
        for ( int  i = 0; i < m_imagePixelData.length; i++ )
        {
            if ( m_imagePixelData[i] < m_imagePixelDataMinVal )
            {
                m_imagePixelDataMinVal = m_imagePixelData[i];
            }
            
            if ( m_imagePixelData[i] > m_imagePixelDataMaxVal )
            {
                m_imagePixelDataMaxVal = m_imagePixelData[i];
            }
        }
        
        m_imagePixelDataMinVal = m_imagePixelDataMinVal * m_nordicScaleFactor;
        m_imagePixelDataMaxVal = m_imagePixelDataMaxVal * m_nordicScaleFactor;
        
    }
    
    /**
     * Returns the image offset parameter of image having order number
     * 'imageNumber'
     */
    public double getSlicePosition( int imageNumber )
    {
        FileInfo finfo = m_dicomImageSeriesData.getFileInfo( imageNumber );
        return new Double( finfo.getSliceLocation( ) ).doubleValue( );
    }
    
    /**
     * Writes the DicomImage belonging to imageNumberToWrite to disk
     * @param imageNumberToWrite
     */
    public void writeDicomImage( int imageNumberToWrite, int[] newPixelData, String path, String fileName )
    {
        DicomImageData dicomData = m_dicomImageSeriesData.getDicomImageDataObject( imageNumberToWrite );

        String pathAndName = path + System.getProperty( "file.separator" ) + fileName;

        dicomData.saveDicomImageWithNewPixelData( newPixelData, pathAndName );
    }



    /**
     * Displays a dicom image with number 'displayImageNumber'
     */
    public boolean displayDicomImage( int displayImageNumber )
    {
        // set the attribute which defines which image of the imagestack is displayed
        
        if ( ( m_imagePixelData != null ) && ( displayImageNumber > -1 ) )
        {
            FileInfo finfo = m_dicomImageSeriesData.getFileInfo( displayImageNumber );
            LinkedHashMap fSiemensInfo = m_dicomImageSeriesData.getFileSiemensInfo( displayImageNumber );

            if ( ( finfo.getSliceLocation( ) != null ) && 
                 ( finfo.getSliceLocation( ).compareTo( "N.A." ) != 0 ) &&
                 ( finfo.getSliceLocation( ).compareTo( "" ) != 0 ) )
            {
                m_slicePosition = new Double( finfo.getSliceLocation( ) ).doubleValue( );
            }
            else
            {
                m_slicePosition = 0.0;
            }

            m_currentlyDisplayedImageNumber = displayImageNumber;

            m_currentlyDisplayedDTIOverlayImageNumber = displayImageNumber;
            m_currentlyDisplayedfMRIOverlayImageNumber = displayImageNumber;

            setImage(m_imagePixelData, finfo.getImageWidth( ), finfo.getImageHeight( ),
                      m_currentlyDisplayedImageNumber, finfo, fSiemensInfo );
            return true;
        }
        return false;
    }
    
    /**
     * Returns an Image instance of the currently displayed image
     */
    public Image getImageInstance( ) 
    {
        int w = (int)( m_imageWidth * m_zoom );
        int h = (int)( m_imageHeight * m_zoom );
        
        Image imageCopy = createImage( w, h );
        
        Graphics2D g = ( Graphics2D ) imageCopy.getGraphics( );
        g.drawImage( m_image, m_imageTop, m_imageLeft, this );

        // add the scale to the data ...
        if ( drawScale )
        {
            drawScale( g );
        }
        return imageCopy;
    }
    
    /**
     * Writes the currently visible image in tiff format to 
     * a storage medium
     */
    public int writeAsTiff( String fileName ) 
    {
        int w = (int)( m_imageWidth * m_zoom );
        int h = (int)( m_imageHeight * m_zoom );
        Image imageCopy = getImageInstance( );
        int[] pixels = new int[ w * h ];
        PixelGrabber pg = new PixelGrabber( imageCopy, 0, 0, w, h, pixels, 0, w );
        try
        {
            pg.grabPixels( );
        }
        catch ( InterruptedException e )
        {
            return  IO_INTERRUPTED;
        }
        if ( ( pg.getStatus( ) & ImageObserver.ABORT ) != 0 )
        {
            return IO_ERROR;
        }
        Tiff T = new Tiff( );
        T.setImage( pixels, w, h );
        if ( Tiff.TIFF_OK != T.writeFullRGB( fileName ) )
        {
            return IO_ERROR;
        }

        return IO_OK;

    }
    
    public void scaleSetVisible( boolean vis ) 
    {
        drawScale = vis;
    }

    public void setScalePosition( int position ) 
    {
        scalePosition = position;
    }

    public int getScalePosition( ) 
    { 
        return scalePosition; 
    }

    public boolean scaleIsLeft( ) 
    {
        return ( ( scalePosition & 2 ) == 0 );
    }
    
    public boolean scaleIsTop( ) 
    {
        return ( ( scalePosition & 1 ) == 0 );
    }
    
    
    /**
     * Defines a rectangle in which the colormap will be drawn
     */
    private void setScaleRectangle( ) 
    {
	
        if ( m_imageWidth == 0 || m_imageHeight == 0 ) return;
        
        switch ( scalePosition )
        {
            case POSITION_TOPLEFT:
                m_scaleRectangle.setBounds( (int)( m_imageWidth * m_zoom ) / 20,
                                            (int)(  m_imageHeight * m_zoom ) / 20,
                                            (int)( m_imageWidth * m_zoom ) / 30,
                                            (int)( m_imageHeight * m_zoom ) / 4 );
                break;
            case POSITION_BOTTOMLEFT:
                m_scaleRectangle.setBounds( (int)( m_imageWidth * m_zoom ) / 20 ,
                                            (int)( 14 * m_imageHeight * m_zoom ) / 20,
                                            (int)( m_imageWidth * m_zoom ) / 30,
                                            (int)( m_imageHeight * m_zoom ) / 4 );
                break;
            case POSITION_BOTTOMRIGHT:
                m_scaleRectangle.setBounds( (int)( 18 * m_imageWidth * m_zoom ) / 20,
                                            (int)( 14 * m_imageHeight * m_zoom ) / 20,
                                            (int)( m_imageWidth * m_zoom ) / 30,
                                            (int)( m_imageHeight * m_zoom )/4 );
                break;
            case POSITION_TOPRIGHT:
                m_scaleRectangle.setBounds( (int)( 18 * m_imageWidth * m_zoom ) / 20,
                                            (int)( m_imageHeight * m_zoom ) / 20 ,
                                            (int)( m_imageWidth * m_zoom ) / 30 ,
                                            (int)( m_imageHeight * m_zoom ) / 4 ) ;
                break;
            default:
                break;
        }
    }
    
    /**
     * Defines the position of the pager
     */
    protected void definePagerLocation( )
    {
        
        m_showNextImage = new  Polygon( );
        m_showPreviousImage = new Polygon( );
        
//        int x1 = this.m_imageWidth * (int)m_zoom - 40;
//        int y1 = 0;
//
//        int x2 = this.m_imageWidth * (int)m_zoom - 1;
//        int y2 = 0;
//
//        int x3 = this.m_imageWidth * (int)m_zoom - 1;
//        int y3 = 40;
//
//        int x4 = this.m_imageWidth * (int)m_zoom - 40;
//        int y4 = 40;


        int x1 = (int)(m_imageWidth * m_zoom) - 40;
        int y1 = 0;

        int x2 = (int)(m_imageWidth * m_zoom) - 1;
        int y2 = 0;

        int x3 = (int)(m_imageWidth * m_zoom) - 1;
        int y3 = 40;

        int x4 = (int)(m_imageWidth * m_zoom) - 40;
        int y4 = 40;




        // If the user clicks in the region (x1,y1), (x2,y2), (x3,y3)  
        // the next image will be shown
        m_showNextImage.addPoint( x1, y1 );
        m_showNextImage.addPoint( x2, y2 );
        m_showNextImage.addPoint( x3, y3 );

        // If the user clicks in the region (x1, y1), (x3, y3), (x4, y4)  
        // the next image will be shown

        m_showPreviousImage.addPoint( x1, y1 );
        m_showPreviousImage.addPoint( x3, y3 );
        m_showPreviousImage.addPoint( x4, y4 );
    }
    
    /**
     * Sets the zoom attribute (integer!) and performs a 
     * recalculation of the image
     */
    public void setZoom( float zoom ) 
    {
        m_zoom = zoom;
        definePagerLocation( );
        drawImage( false );
        calculateImagePixels( );
        
        if ( m_imageCrossLines != null )
        {
            m_imageCrossLines.setZoom( zoom );
        }
    }

    /**
     * Sets the zoom attribute (integer!) and performs a 
     * recalculation of the image
     */
    public void setZoomWithoutNotification( float zoom ) 
    {
        m_zoom = zoom;
        definePagerLocation( );
        drawImage( false );
        calculateImagePixels( );
        
        if ( m_imageCrossLines != null )
        {
            m_imageCrossLines.setZoom( zoom );
        }
    }
    
    /**
     * Sets the zooming factor, and updates the control
     */
    public void setZoom( float zoom, boolean updateCm ) 
    {
        m_zoom = zoom;
        definePagerLocation( );
        drawImage( updateCm );
        calculateImagePixels( );
    }

    /**
     * Sets the zooming factor no update of the control
     */
    public void setZoomNoDraw( float zoom ) 
    {
        this.m_zoom = zoom;
        definePagerLocation( );
    }

    /**
     * Returns the current zoom factor
     */
    public float getZoom( ) 
    { 
        return m_zoom; 
    }
    
    /**
     * Returns the image width
     */
    public int getImageWidth( ) 
    { 
        return m_imageWidth; 
    }
    
    /**
     * Returns the image height
     */
    public int getImageHeight( ) 
    { 
        return m_imageHeight; 
    }

    /**
     * Returns the controls preferred size
     */
    public Dimension getPreferredSize( ) 
    {
        int w = (int)( m_imageWidth * m_zoom );
        int h = (int)( m_imageHeight * m_zoom );
        return new Dimension( w, h );
    }
    
    /**
     * Returns the controls maximum size
     */
    public Dimension getMaximumSize( ) 
    {
        return getPreferredSize( );
    }
    
    /**
     * Draws the image
     */
    public void drawImage( boolean updateCMap ) 
    {

        if ( cmap == null || updateCMap ) setColormap( );
        
        if ( m_redrawImage == true )
        {
            if ( ( m_imageWidth > 0 ) && ( m_imageHeight > 0 ) && ( m_zoom != 0 ) )
            {
                Image ima;
                ima = createImage( new MemoryImageSource( m_imageWidth, m_imageHeight, cmap,
                                                          m_scaledData, 0, m_imageWidth ) );
            
                ima = ima.getScaledInstance( (int)( m_zoom * m_imageWidth ), 
                                             (int)( m_zoom * m_imageHeight ), 
                                             zoomQuality );
            
                m_image = toBufferedImage( ima );
            
                setSize( (int)( m_zoom * m_imageWidth ), (int)( m_zoom * m_imageHeight ) );
            }
        }
        
        paintComponent( getGraphics( ) );
    }

   /** 
    *  This method returns a buffered image with the contents of an image
    */
    public static BufferedImage toBufferedImage( Image image ) 
    {
        if ( image instanceof BufferedImage ) 
        {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon( image ).getImage( );
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e665 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha( image );
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment( );
        try
        {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
            if ( hasAlpha ) 
            {
                transparency = Transparency.BITMASK;
            }
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice( );
            GraphicsConfiguration gc = gs.getDefaultConfiguration( );
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency );
        } catch ( HeadlessException e ) 
        {
            // The system does not have a screen
        }
    
        if ( bimage == null ) 
        {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            if ( hasAlpha ) 
            {
                type = BufferedImage.TYPE_INT_ARGB;
            }
            bimage = new BufferedImage( image.getWidth( null ), image.getHeight( null ), type );
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics( );
    
        // Paint the image onto the buffered image
        g.drawImage( image, m_imageLeft, m_imageTop, null );
        g.dispose( );
    
        return bimage;
    }
 
    
    /**
     *   This method returns true if the specified image has transparent pixels
     */
    public static boolean hasAlpha( Image image ) 
    {
        // If buffered image, the color model is readily available
        if ( image instanceof BufferedImage ) 
        {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel( ).hasAlpha( );
        }
    
        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
        PixelGrabber pg = new PixelGrabber( image, 0, 0, 1, 1, false );
        try 
        {
            pg.grabPixels( );
        }
        catch ( InterruptedException e )
        {
        }
    
        // Get the image's color model
        ColorModel cm = pg.getColorModel( );
        return cm.hasAlpha( );
    }

    public void update( int quality ) 
    {
        zoomQuality = quality;
        drawImage( false );
        // update( getGraphics( ) );
    }
    
    public void update( boolean redrawImage ) 
    {
        this.m_redrawImage = redrawImage;
        drawImage( redrawImage );
        // update( getGraphics( ) );
    }
    
    public void update( ) 
    {
        m_redrawImage = true;
        update( getGraphics( ) );
    }
    
    public void setZoomQuality( int quality ) 
    {
        zoomQuality = quality;
    }
    
    public int getZoomQuality( ) 
    {
        return zoomQuality;
    }

    
    /**
     * Draws the scale into the image window
     */
    private void drawScale( Graphics2D g ) 
    {
        setScaleRectangle( );
        int rightEdge = m_imageTop + m_scaleRectangle.x + m_scaleRectangle.width;
        int bottom = m_imageLeft + m_scaleRectangle.y + m_scaleRectangle.height;
        
        int mapSize = m_lastCMapIndex - m_firstCMapIndex + 1;
        float step = (float)mapSize / (float)m_scaleRectangle.height;
        
        int y = bottom;
        for ( int i = 0; i < m_scaleRectangle.height; i++ )
        {
            int k = (int)( i * step ) + m_firstCMapIndex;
            g.setColor( new Color( cmap.getRed( k ), cmap.getGreen( k ), cmap.getBlue( k ) ) );
            g.drawLine( m_imageTop + m_scaleRectangle.x, y, rightEdge, y );
            y--;
        }
        g.setColor( Color.white );
        int textX = ( scaleIsLeft( ) ? (int)( 1.1 * ( m_scaleRectangle.x + m_scaleRectangle.width ) + m_imageTop ) :
                      m_scaleRectangle.x - g.getFontMetrics( ).stringWidth( " " + m_displayedMinVal ) );
	
        NumberFormat format = new DecimalFormat( "###.##" );
        
        if ( m_fileInfo.getNordicPixelScaling( ) == 1.0f )
        {
            g.drawString( "" + (int)m_displayedMinVal , textX, bottom );
        
            if ( ! scaleIsLeft( ) )
            { 
                textX = m_scaleRectangle.x - g.getFontMetrics( ).stringWidth( " " + m_displayedMaxVal );
            }
        
            g.drawString( "" + (int)m_displayedMaxVal, textX, y + g.getFontMetrics( ).getAscent( ) );
        }
        else
        {
             g.drawString( "" + format.format( m_displayedMinVal * m_nordicScaleFactor ) , textX, bottom );
        
            if ( ! scaleIsLeft( ) )
            { 
                textX = m_scaleRectangle.x - g.getFontMetrics( ).stringWidth( " " + m_displayedMaxVal );
            }
        
            g.drawString( "" + format.format( m_displayedMaxVal * m_nordicScaleFactor ), textX, y + g.getFontMetrics( ).getAscent( ) );
        }
        
    }
    
    /**
     * Does the actual painting of the component
     */ 
    public synchronized void paintComponent( Graphics g ) 
    {
        if ( g != null )
        // super.paintComponent( g );
        {
            if ( m_image != null ) 
            {
            

                if ( m_redrawImage )
                {
                    g.setPaintMode( );
		
                    try 
                    {
                        g.drawImage( m_image, m_imageTop, m_imageLeft, this );
                    
                        if ( ( m_title.length( ) > 8 ) && ( m_drawAnnotation == false ) )
                        {
                            Color saveColor = g.getColor( );
                            g.setColor( Color.green );

                            g.drawString( m_title.substring( m_title.length( ) - 8, m_title.length( ) - 4 ) , 
                                          m_imageTop + (int)(m_zoom * getImageWidth( ) ) / 2, m_imageLeft + (int)( m_zoom * getImageHeight( ) ) - 10 );
                        
                            g.setColor( saveColor );
                        }
                    
                    
                        // Paint a image pager in the top right corner 
                        if ( ( m_numberOfImages > 1 ) && ( m_enablePaging == true ) )
                        {
                            g.setColor( Color.green );
                            g.drawRect( (int)m_zoom * getImageWidth( ) - 40, 0, 39, 39 );
                            g.drawLine( (int)m_zoom * getImageWidth( ) - 40, 0, (int)m_zoom * getImageWidth( ), 39 );
                            g.drawString( "S. " + m_fileInfo.getSeriesNumber( ).trim( ) + " / I. " + m_fileInfo.getInstanceNumber( ).trim( ) + " of " + m_numberOfImages, 5, 20 );
                        }
                    
                        if ( m_drawAnnotation == true )
                        {
                            Color saveColor = g.getColor( );
                            g.setColor( Color.green );
                    
                            Font f = g.getFont( );
                            Font newfont = new Font( f.getFamily( ), f.getStyle( ), 9 );
                    
                            g.setFont( newfont );
                            NumberFormat format = new DecimalFormat( "##.##" );
                            String currentSlicePos = format.format( m_slicePosition );
                            g.drawString( m_title, 5, 10 );
                            g.drawString( "Slice pos. " + currentSlicePos, 5, 20 );
                            g.setColor( saveColor );
                            //set font back
                            g.setFont( f );
                        }
                    
                        if ( m_drawCrossLineMode == true )
                        {
                            m_imageCrossLines.paint( g );
                        }
                    
                    }
                    catch ( OutOfMemoryError e ) 
                    {
                        System.out.println( e );
                        System.out.println( "brightness = " + m_brightness + 
                                            ", contrast = " + m_contrast );
                        return;
                    }
		
                    if ( drawScale ) 
                    {
                        drawScale( (Graphics2D)g );
                    }
                }
            }
        }
    }
     
    /**
     * Returs the value of the pixel at ( x_coord, y_coord )
     *
     *      @param x_coord : x coordinate
     *      @param y_coord : y coordinate
     */
    public float getPixelValue( int x_coord, int y_coord )
    {
        float display_value = -1.0f;
        if ( m_imagePixels == null )
        {
            calculateImagePixels( );
        }
        
        if ( ( m_imagePixels != null ) && ( m_imagePixels.length != 0 ) )
        {
            display_value = (float)( m_imagePixels[ x_coord + (int)( m_imageWidth * m_zoom * y_coord ) ] + 16777216.0 ) / (float)16777216.0;
            display_value = m_rawDataMinVal + display_value * ( m_rawDataMaxVal - m_rawDataMinVal );
        }
        
        return display_value;
    }
    
    
    /**
     * Returs the value of the pixel at ( x_coord, y_coord )
     *
     *      @param x_coord : x coordinate
     *      @param y_coord : y coordinate
     */
    public float getRawPixelValue( int x_coord, int y_coord )
    { 
        float display_value;
        if ( ( x_coord < this.m_imageWidth ) && ( y_coord < this.m_imageHeight ) )
        {
            display_value = (float)( m_rawData[ x_coord + ( m_imageWidth * y_coord ) ] ) * m_nordicScaleFactor ;
        }
        else
        {
            display_value = -1f;
        }
        
        return display_value;
    }
    
    /**
     *  Computes the image pixels
     */
    public void calculateImagePixels( )
    {
        
        m_imagePixels = null;
        
        if ( this.m_image != null )
        {
            int w = (int)(m_imageWidth * m_zoom );
            int h = (int)(m_imageHeight * m_zoom );
            Image imageCopy = getImageInstance( );        
            m_imagePixels = new int[ w * h ];
            PixelGrabber pg = new PixelGrabber( imageCopy, 0, 0, w, h, m_imagePixels, 0, w );
            try 
            {
                pg.grabPixels( );
            }
            catch ( InterruptedException ie ) 
            {
                ;
            }
        }
    }
       
    /**
     * returns the minimum value of the raw data file
     */
    public int getMinRawDataValue( ) 
    {
        return m_rawDataMinVal;
    }
    
    /**
     * returns the maximum value of the raw data file
     */
    public int getMaxRawDataValue( ) 
    {
        return m_rawDataMaxVal;
    }
    
     /**
     * returns the minimum value of the raw data file
     */
    public float getMinRawDataValueOfSliceStack( ) 
    {
        return m_imagePixelDataMinVal;
    }
    
    /**
     * returns the maximum value of the raw data file
     */
    public float getMaxRawDataValueOfSliceStack( ) 
    {
        return m_imagePixelDataMaxVal;
    }
    
    
    
    
    /**
     * Returns the integer array conataining the raw data of the currently
     * visible data
     */
    public int[] getRawData( ) 
    {
        return m_rawData;
    }
    
    /**
     * This is an awfull hack to get it working ...
     *
     */
    public void setRegioReference( boolean regioIsParent )
    {
        m_regioIsParent = regioIsParent;
    }
 
    /**
     *  Set drawImage attribute
     */
    public void setRedrawImage( boolean redraw )
    {
        this.m_redrawImage = redraw;
    }
    
    
    /**
     * Sets the FileInfo object, which contains all the DICOM header information
     */
    public void setDicomFileInfo( FileInfo finfo )
    {
        m_fileInfo = finfo;
        if ( ( finfo.getPixelWidth( )     != null ) &&
             ( finfo.getPixelHeight( )    != null ) &&
             ( finfo.getSliceThickness( ) != null ) )
        {
            m_volumePerPixel = new Double( finfo.getPixelWidth( ).trim( ) ).doubleValue( ) *
                               new Double( finfo.getPixelHeight( ).trim( ) ).doubleValue( ) *
                               new Double( finfo.getSliceThickness( ).trim( ) ).doubleValue( ) ;
        }
        else
        {
            m_volumePerPixel = 0.0;
        }
    }
    
    /**
     * Sets the Siemens specific FileInfo object
     */
    public void setDicomSpecificFileInfo( LinkedHashMap siemensSpecificData )
    {
        m_siemensSpecificData = siemensSpecificData;
    }
    
    
    /**
     * Returns a reference to the currently set DicomImageSeriesData object
     */
    public DicomImageSeriesData getDicomImageSeriesData( )
    {
        return m_dicomImageSeriesData;
    }

    /**
     * Sets a new FileInfo object
     * @param m_finfo
     */
    public void setFileInfo( FileInfo finfo )
    {
        m_fileInfo = finfo;
    }

    /**
     * Returns the FileInfo object, containing specific DICOM header information
     */
    public FileInfo getDicomFileInfo( )
    {
        return m_fileInfo;
    }
    
    public void setTooltipTextsVisible( boolean visible )
    {
        m_showToolTipText = visible;
    }
    
    /**
     * Returns the raw image pixel data of the currently shown image
     */
    public int[] getImagePixelData( )
    {
        return m_imagePixelData;
    }
    
    /**
     * Method to allow the registration of change listeners 
     */
    public void addChangeListener( ChangeListener e )
    {
        m_changeListeners.add( e );
    }
    
    /**
     * Notifies change listeners that something in the object has changed
     */
    private void notifyChangeListeners( )
    {
        Iterator it = m_changeListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ChangeListener )it.next( ) ).stateChanged( new ChangeEvent( this ) );
        }
    }
 
    /**
     * Force minimum/maximum pixel data to be set to a specific value in order to be 
     * able to create one colortable for multiple images: this is to enforce
     * that red is one image matches the color red in the other image.
     * N.B. this routine only sets the values but does not do any updating of 
     * the GUI
     */
    public void setForcedMinMaxRawDataValues( int minValue, int maxValue )
    {
        m_forceRawDataMinVal = minValue;
        m_forceRawDataMaxVal = maxValue;
        // backed up raw data values
        m_raw_1 = m_rawData[1];
        m_raw_2 = m_rawData[2];
    }

    /**
     * Sets the boolean attribute to true or false
     */
    public void setForcedMinMaxRawDataValue( boolean force )
    {
        int displayNumber = this.m_currentlyDisplayedImageNumber;
        if ( force == true )
        {    
            m_forceRawMinMaxPixelValue = true;
            m_rawData[2] = m_forceRawDataMinVal;            
            m_rawData[1] = m_forceRawDataMaxVal;
            this.setImage( m_rawData, m_imageWidth, m_imageHeight, 0 );
        }
        else
        {
            m_forceRawMinMaxPixelValue = false;
            m_rawData[1] = m_raw_1;
            m_rawData[2] = m_raw_2;
            this.setImage( m_rawData, m_imageWidth, m_imageHeight, 0 );
        }
        displayDicomImage( displayNumber );
    }
    
    /**
     * Called in response to a mouse click
     */ 
    public void mouseClicked( MouseEvent e ) 
    {
        
        DicomSliceSelectionManager.getInstance( ).setSelectedDicomSliceCanvas( this );
        boolean neglectClick = false;
        
        if ( m_enablePaging == true )
        {
            // Only left mouse button clicks are considered
            int modifiers = e.getModifiers( );
            int x = e.getX( );
            int y = e.getY( );
            if ( ( modifiers & MouseEvent.BUTTON1_MASK ) > 0 ) 
            {
                if ( m_showNextImage.contains( x, y ) )
                {
                    neglectClick = true;
                    // System.out.println( "Next image" );
                    if ( m_currentlyDisplayedImageNumber <= m_numberOfImages - 1 )
                    {
                        m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber + 1;

                        m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
                        m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;

                        displayDicomImage(m_currentlyDisplayedImageNumber );
                        drawImage( false ); 
                        notifyActionListeners( DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT, 
                                               DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT_ACTION_COMMAND );
                        return;
                    }
                }
                
                if ( m_showPreviousImage.contains( x, y ) )
                {
                    neglectClick = true;
                    // System.out.println( "Previous image" );
                    if ( m_currentlyDisplayedImageNumber  > 0 )
                    {
                        m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber - 1;

                        m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
                        m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;

                        displayDicomImage(m_currentlyDisplayedImageNumber );
                        drawImage( false );
                        
                        notifyActionListeners( DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT, 
                                               DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT_ACTION_COMMAND );
                        return;
                    }
                }
            }
        }
        
        if ( ( this.m_drawCrossLineMode == true ) && ( neglectClick == false ) )
        {        
             this.m_lastMouseClickX =  e.getX( );
             this.m_lastMouseClickY =  e.getY( );
             
             this.notifyActionListeners( DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT, DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT_ACTION_COMMAND );
             
             m_imageCrossLines.setCenter( e.getX( ), e.getY( ) );
                 
             update( );
        }
    }

    /**
     * Called in response to the mouse cursor enters the canvas
     */ 
    public void mouseEntered( MouseEvent mouseEvent ) 
    {
    }

    /**
     * Called in response to the mouse cursor exits the canvas
     */ 
    public void mouseExited( MouseEvent mouseEvent ) 
    {
    }
    
    /**
     * Called in response to a pressed mouse button
     */
    public void mousePressed( MouseEvent mouseEvent ) 
    {
        DicomSliceSelectionManager.getInstance( ).setSelectedDicomSliceCanvas( this );
    }

    /**
     * Called in response to a released mouse button
     */
    public void mouseReleased( MouseEvent mouseEvent ) 
    {
    }
    
    /**
     * Enables or disables image paging
     */
    public void enableImagePaging( boolean paging )
    {
        m_enablePaging = paging;
        if ( ( m_enablePaging == true ) && ( m_multiImageMenu == null ) )
        {
            // Menu should also allow operations on slice stacks
            if ( m_isJMRUIRelease == false )
            {
                m_multiImageMenu = new JMenu( "Image stack" );
            
                m_multiImageMenu.addActionListener( this );

                // initialize the image numbers that have focus!
                if ( m_minimumImageNumberToFocus == -1 )
                {
                    m_minimumImageNumberToFocus = 0;
                }

                if ( m_maximumImageNumberToFocus == -1 )
                {
                    m_maximumImageNumberToFocus = getNumberOfImages( );
                }

                m_ImageStackMinMaxImageNumberThatHasFocusDefintionMenu = new JMenu( "Image range focus" );
                m_ImageStackMinMaxImageNumberThatHasFocusDefintionMenu.setToolTipText("Define the current minimum and maximum image number of the stack that has focus (default focus: all images)" );

                m_ImageStackMinimumImageNumberThatHasFocus = new JMenuItem( "Set minimum image number having focus (" + m_minimumImageNumberToFocus + ")" );
                m_ImageStackMinimumImageNumberThatHasFocus.addActionListener( this );

                m_ImageStackMinMaxImageNumberThatHasFocusDefintionMenu.add( m_ImageStackMinimumImageNumberThatHasFocus );

                m_ImageStackMaximumImageNumberThatHasFocus = new JMenuItem( "Set maximum image number having focus (" + m_maximumImageNumberToFocus + ")" );
                m_ImageStackMaximumImageNumberThatHasFocus.addActionListener( this );

                m_ImageStackMinMaxImageNumberThatHasFocusDefintionMenu.add( m_ImageStackMaximumImageNumberThatHasFocus );

                m_multiImageMenu.add( m_ImageStackMinMaxImageNumberThatHasFocusDefintionMenu );


                m_displaySliceNumber = new JMenuItem( "Display slice number .." );
                m_displaySliceNumber.setToolTipText( "Allows the user to set the selected image number" );
                m_displaySliceNumber.addActionListener( this );
            
                m_multiImageMenu.add( m_displaySliceNumber );
            
                m_performSimple3DCut = new JMenuItem( "Perform simple 3D Cut" );
                m_performSimple3DCut.setToolTipText( "Sets all pixels ouside the selected region to zero" );
                m_performSimple3DCut.addActionListener( this );
                m_performSimple3DCut.setEnabled( false );
            
                m_multiImageMenu.add( m_performSimple3DCut );
            
                m_computeRegionPropertiesForAllSlicesMenu = new JMenu( "Regio properties for all slices" );
            
                m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly = new JMenuItem( "For currentlySelectedROI" );
                m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.setToolTipText( "" );
                m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.addActionListener( this );
                m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.setEnabled( false );
            
                m_computePropertiesofSelectedROIForAllSlicesForAllRegions = new JMenuItem( "For all ROIs" );
                m_computePropertiesofSelectedROIForAllSlicesForAllRegions.setToolTipText( "" );
                m_computePropertiesofSelectedROIForAllSlicesForAllRegions.addActionListener( this );
                m_computePropertiesofSelectedROIForAllSlicesForAllRegions.setEnabled( false );
            
                m_computeRegionPropertiesForAllSlicesMenu.add( m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly );
                m_computeRegionPropertiesForAllSlicesMenu.add( m_computePropertiesofSelectedROIForAllSlicesForAllRegions );
            
                m_multiImageMenu.add( m_computeRegionPropertiesForAllSlicesMenu );

                m_reduceImageMatrixAndWriteAsDICOM = new JMenuItem( "Reduce image matrix size and export as DICOM .." );
                m_reduceImageMatrixAndWriteAsDICOM.setToolTipText( "Reduce image matrix size and write as DICOM" );
                m_reduceImageMatrixAndWriteAsDICOM.addActionListener( this );
                m_reduceImageMatrixAndWriteAsDICOM.setEnabled( true );

                m_multiImageMenu.add( m_reduceImageMatrixAndWriteAsDICOM );

                m_popupmenu.add( m_multiImageMenu );
            }
        }
        if ( ( m_enablePaging == false ) && ( m_multiImageMenu != null ) )
        {
            m_popupmenu.remove( m_multiImageMenu );
        }
    }
    
    /**
     * Returns the horizontal direction vector of the image
     */
    public Point3D getImageOrientationUnitVectorX( )
    {
        return this.m_dirVecX;
    }

    /**
     * Returns the vertical direction vector of the image
     */
    public Point3D getImageOrientationUnitVectorY( )
    {
        return this.m_dirVecY;
    }

    /**
     * Returns the horizontal direction vector of the image
     */
    public Point3D getTopLeftImageCoordinate( )
    {
        return this.m_topLeftPoint;
    }

    /**
     * Sets the crossline object
     */
    public void displayCrossLines( Color horizontalDisplayColor,  Color verticalDisplayColor )
    {
        
        m_imageCrossLines = new ImageCrossLines( m_imageWidth, m_imageHeight, 
                                                 m_imageWidth/2, m_imageHeight/2, 
                                                 horizontalDisplayColor, verticalDisplayColor );
        
        m_drawCrossLineMode = true;
        
        this.update( );
    }
    
    /**
     * Sets the boolean 
     * @param drawCrosLineMode
     */
    public void setDisplayCrossLineMode( boolean drawCrosLineMode )
    {
        m_drawCrossLineMode = drawCrosLineMode;
    }

    /**
     * Method to allow the registration of listeners 
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    /**
     * Notify listeners that something in the object has changed
     */
    protected void notifyActionListeners( int eventType, String actionCommand )
    {
        
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, actionCommand ) );
        }
        
    }
    
    /**
     * Notify listeners that something in the object has changed
     */
    protected void notifyActionListeners( int eventType )
    {
        
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
        
    }
    
    /**
     * Returns the X-value of the point that was clicked by the mouse pointer
     */
    public int getLastMouseClickXPosition( )
    {
        return m_lastMouseClickX;
    }
    
    /**
     * Returns the Y-value of the point that was clicked by the mouse pointer
     */
    public int getLastMouseClickYPosition( )
    {
        return m_lastMouseClickY;
    }
    
    
    /**
     * Updates the cross line x coordinate
     */
    public void updateCrossLineX( int Xcoord )
    {
        m_imageCrossLines.setCenterX( Xcoord );
    }
    
    /**
     * Updates the cross line x coordinate
     */
    public void updateCrossLineY( int Ycoord )
    {
        m_imageCrossLines.setCenterY( Ycoord );
    }
    /**
     *  Returns the currently displayed image
     * @return
     */
    public int getCurrentlyDisplayedImageNumber( )
    {
        return m_currentlyDisplayedImageNumber;
    }
    
    /**
     * 
     * Called whenever a registerend component raises an ActionEvent
     * 
     * @param actionEvent
     */
    public void actionPerformed( ActionEvent e )
    {
       // no further implementation needed uptil now
        if ( e.getSource( ) == m_ImageStackMinimumImageNumberThatHasFocus )
        {
            handleMinimumFocusImageNumberIsSetByUser( );
            //notifyActionListeners( NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );
        }

        if ( e.getSource( ) == m_ImageStackMaximumImageNumberThatHasFocus )
        {
            handleMaximumFocusImageNumberIsSetByUser( );
            //notifyActionListeners( NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );
        }

        if ( ( e.getSource( ) instanceof DicomSliceCanvas ) && e.getSource( ) != this )
        {
            if ( e.getID( ) == DicomSliceCanvas.NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT )
            {
                 handleMinimumFocusImageNumberIsSetByUser( ( (DicomSliceCanvas)e.getSource( ) ).getMinimumImageNumberToFocus( ) );
            }

            if ( e.getID( ) == DicomSliceCanvas.NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT )
            {
                 handleMaximumFocusImageNumberIsSetByUser( ( (DicomSliceCanvas)e.getSource( ) ).getMaximumImageNumberToFocus( ) );
            }

        }

    }

    /**
     * User defined the maximum image number that should have focus
     */
    private void handleMinimumFocusImageNumberIsSetByUser( )
    {
        int selectedImageNumber = getCurrentlyDisplayedImageNumber( );

        if ( selectedImageNumber <= m_minimumImageNumberToFocus )
        {
            m_minimumImageNumberToFocus = selectedImageNumber;
        }
        else if ( ( selectedImageNumber >= m_minimumImageNumberToFocus ) && ( selectedImageNumber <= m_maximumImageNumberToFocus ) )
        {
            // interchange minimum and maximum image number
            m_minimumImageNumberToFocus = selectedImageNumber;
        }
        else if ( ( selectedImageNumber >= m_minimumImageNumberToFocus ) && ( selectedImageNumber >= m_maximumImageNumberToFocus ) )
        {
            // interchange minimum and maximum image number
            m_minimumImageNumberToFocus = selectedImageNumber;
            m_maximumImageNumberToFocus = selectedImageNumber;
        }

        m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
        m_ImageStackMaximumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );

        notifyActionListeners( NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );

    }

    /**
     * User defined the maximum image number that should have focus
     */
    private void handleMaximumFocusImageNumberIsSetByUser( )
    {
        int selectedImageNumber = getCurrentlyDisplayedImageNumber( );

        // make tests
        if ( selectedImageNumber >= m_maximumImageNumberToFocus )
        {
            m_maximumImageNumberToFocus = selectedImageNumber;
        }
        else if ( ( selectedImageNumber >= m_minimumImageNumberToFocus ) && ( selectedImageNumber <= m_maximumImageNumberToFocus ) )
        {
            m_maximumImageNumberToFocus = selectedImageNumber;
        }
        else if ( selectedImageNumber <= m_minimumImageNumberToFocus )
        {
            m_maximumImageNumberToFocus = selectedImageNumber;
            m_minimumImageNumberToFocus = selectedImageNumber;
        }

        m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
        m_ImageStackMaximumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );
        
        notifyActionListeners( NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );

    }

    /**
     * User defined the maximum image number that should have focus
     */
    private void handleMinimumFocusImageNumberIsSetByUser( int minimumImageFocusNumber )
    {
        int selectedImageNumber = minimumImageFocusNumber;

        // make tests
        if ( selectedImageNumber <= m_minimumImageNumberToFocus )
        {
            m_minimumImageNumberToFocus = selectedImageNumber;
        }
        else if ( ( selectedImageNumber >= m_minimumImageNumberToFocus ) && ( selectedImageNumber <= m_maximumImageNumberToFocus ) )
        {
            // interchange minimum and maximum image number
            m_minimumImageNumberToFocus = selectedImageNumber;
        }
        else if ( ( selectedImageNumber >= m_minimumImageNumberToFocus ) && ( selectedImageNumber >= m_maximumImageNumberToFocus ) )
        {
            // interchange minimum and maximum image number
            m_minimumImageNumberToFocus = selectedImageNumber;
            m_maximumImageNumberToFocus = selectedImageNumber;
        }

        m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
        m_ImageStackMaximumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );
    }

    /**
     * User defined the maximum image number that should have focus
     */
    private void handleMaximumFocusImageNumberIsSetByUser( int maximumImageFocusNumber )
    {
        int selectedImageNumber = maximumImageFocusNumber;
   
        // make tests
        if ( selectedImageNumber >= m_maximumImageNumberToFocus )
        {
            m_maximumImageNumberToFocus = selectedImageNumber;
        }
        else if ( ( selectedImageNumber >= m_minimumImageNumberToFocus ) && ( selectedImageNumber <= m_maximumImageNumberToFocus ) )
        {
                m_maximumImageNumberToFocus = selectedImageNumber;
        }
        else if ( selectedImageNumber <= m_minimumImageNumberToFocus )
        {
            m_maximumImageNumberToFocus = selectedImageNumber;
            m_minimumImageNumberToFocus = selectedImageNumber;
        }

        m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
        m_ImageStackMaximumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );
    }


    /**
     * User defined the maximum image number that should have focus
     */
    public void setMinimumMaximumFocusImageNumberRange( int minimumImageFocusNumber, int  maximumImageFocusNumber )
    {
        m_minimumImageNumberToFocus = minimumImageFocusNumber;
        m_maximumImageNumberToFocus = maximumImageFocusNumber;

        m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
        m_ImageStackMaximumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );
    }


    /**
     *
     * Only sets the selected imagenumber, further nothing, no updating etcetera
     *
     * @param selectedImageNumber
     */
    public void setCurrentlySelectedImageDTIOverlayNumber( int selectedImageNumber )
    {
        m_currentlyDisplayedDTIOverlayImageNumber = selectedImageNumber;
    }

    /**
     *
     * Only sets the selected fmri overlay imagenumber, further nothing, no updating etcetera
     *
     * @param selectedImageNumber
     */
    public void setCurrentlySelectedImagefMRIOverlayNumber( int selectedImageNumber )
    {
        m_currentlyDisplayedfMRIOverlayImageNumber = selectedImageNumber;
    }



        /**¨
     * Sets the minimum image number of the slice stack which has stack focus
     * @param minimumImageNumberToFocus
     */
     public void setMinimumImageNumberToFocus ( int minimumImageNumberToFocus )
     {
         m_minimumImageNumberToFocus = minimumImageNumberToFocus;

         if (m_ImageStackMinimumImageNumberThatHasFocus != null )
         {
            m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
         }
         
         notifyActionListeners( NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );
     }

     /**¨
     * Sets the minimum image number of the slice stack which has stack focus
     * @param minimumImageNumberToFocus
     */
     public void setMaximumImageNumberToFocus ( int maximumImageNumberToFocus )
     {
         m_maximumImageNumberToFocus = maximumImageNumberToFocus;

         if ( m_ImageStackMinimumImageNumberThatHasFocus != null )
         {
            m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );
         }

         notifyActionListeners( NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );
     }

    /**¨
     * Sets the minimum image number of the slice stack which has stack focus
     * @param minimumImageNumberToFocus
     */
     public int getMinimumImageNumberToFocus( )
     {
         return m_minimumImageNumberToFocus;
     }

     /**¨
     * Sets the minimum image number of the slice stack which has stack focus
     * @param minimumImageNumberToFocus
     */
     public int getMaximumImageNumberToFocus( )
     {
         return m_maximumImageNumberToFocus;
     }

     /**
      * Returns the nordic scaling factor
      * @return 
      */
     public float getNordicScaleFactor( )
     {
         return m_nordicScaleFactor;
     }
     
     /**
      * Sets the nordic scaling factor
      * @param scale 
      */
     public void setNordicScaleFactor( float scale )
     {
         m_nordicScaleFactor = scale;
     }
     
     
    // Attributes 
    public final static String[] Colormaps = { "Grayscale", "Color", "Color inverse" };
    public final static int CMAP_GRAYSCALE = 0;
    public final static int CMAP_COLOR = 1;
    public final static int CMAP_COLOR_INVERSE = 2;

    public final static int IO_OK = 0;
    public final static int IO_INTERRUPTED = 1;
    public final static int IO_ERROR = 2;
    
    protected int m_brightness;
    protected int m_contrast;
    protected int m_startBrightness;
    protected int m_startContrast;
    
    // Member accounting for the number of images of the imageseries to be displayed
    protected int m_numberOfImages = 0;
    
    public final static int POSITION_TOPLEFT = 0;
    public final static int POSITION_BOTTOMLEFT = 1;
    public final static int POSITION_TOPRIGHT = 2;
    public final static int POSITION_BOTTOMRIGHT = 3;

    // the followin g two variables are the minimum and maximum raw data value
    // for the  currently diusplayed image
    protected int m_rawDataMinVal, m_rawDataMaxVal;
    
    // these attributes are necessary to enforce a 
    protected int m_forceRawDataMinVal, m_forceRawDataMaxVal;
    protected boolean m_forceRawMinMaxPixelValue = false;
    
    private float   m_displayedMinVal, m_displayedMaxVal;
    private float   m_raw2displayScale;
    private int     m_firstCMapIndex, m_lastCMapIndex;
    
    public JFrame m_parent;
    
    // a hack!! 
    protected boolean m_regioIsParent;
    
    private BufferedImage m_image = null;
    
    protected int[] m_imagePixels = null;
        
    protected   int[] m_rawData;
    private     int[] m_scaledData;
    protected int m_imageWidth, m_imageHeight;
    protected float m_zoom;
    private int zoomQuality;
    private IndexColorModel cmap = null;
    private int CMapSize;
    private int m_colorMapType;
    
    boolean m_redrawImage;

    private Rectangle m_scaleRectangle;
    private int scalePosition;
    private boolean drawScale;

    private String m_title;
        
    protected FileInfo m_fileInfo;
    protected LinkedHashMap m_siemensSpecificData;
    
    public double m_volumePerPixel = 0.0;
    
    protected boolean m_showToolTipText = true;

    // sets the number of the overlay which needs to be displayed
    protected int m_currentlyDisplayedDTIOverlayImageNumber;
    protected int m_currentlyDisplayedfMRIOverlayImageNumber;


    // Member that indicates which image number of the image series is currently displayed 
    protected int m_currentlyDisplayedImageNumber = 0;
    // Member that stores the dicom filedata information
    protected DicomImageSeriesData m_dicomImageSeriesData;
    
    // Stores all the dicom images pixeldata
    protected int[] m_imagePixelData;
    
    // The following two variables are invariant for a loaded dataset
    // changed to float to allow scaled values to be displayed
    protected float m_imagePixelDataMinVal = Float.MAX_VALUE;
    protected float m_imagePixelDataMaxVal = -Float.MIN_VALUE;
    
    // backup members for m_rawData[1] and m_rawData[2]
    protected int m_raw_1, m_raw_2;
    
    double m_slicePosition;
    
    protected boolean m_drawAnnotation = false;
    
    // Listeners for change events in SliceCanvasPolyDraw
    private LinkedList<ChangeListener> m_changeListeners = new LinkedList<ChangeListener>( );
    
    
    // Listeners for action events in SliceCanvasPolyDraw
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );
    
    
    // Integers indicating the top left position 
    protected static int m_imageLeft = 0;
    protected static int m_imageTop = 0;
    
    // Polygons for indication that next or previous image has to be
    // displayed
    protected Polygon m_showNextImage;
    protected Polygon m_showPreviousImage;
    protected boolean m_enablePaging = false;

    private static int COLORMAPSIZE = 256;
    
    // Attributes from dicom header to indicate position and orientation of slice
    protected Point3D m_dirVecX;
    protected Point3D m_dirVecY;
    protected Point3D m_topLeftPoint;
    
    // Horizontal and vertical FOV of display window
    protected double m_FOVX;
    protected double m_FOVY;
 
    private boolean m_drawCrossLineMode = false;
    protected ImageCrossLines m_imageCrossLines;
    
    private int m_lastMouseClickX;
    private int m_lastMouseClickY;
    
    public final static int WINDOWING_EVENT  = 10;
    public final static String WINDOWING_EVENT_COMMAND  = "WINDOWING_EVENT";
    
    public final static int COLOR_MAP_CHANGED_EVENT  = 11;
    public final static String COLOR_MAP_CHANGED_EVENT_COMMAND  = "COLOR_MAP_CHANGED";
    
    
    public final static int CROSS_HAIR_CHANGE_EVENT  = 9;
    public final static String CROSS_HAIR_CHANGE_EVENT_ACTION_COMMAND  = "CHANGE_CROSS_HAIR_POSITION";
    
    public final static int DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT = 100;
    public final static String DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT_ACTION_COMMAND  = "DISPLAY_NEXT_IMAGE_OF_SLICE_STACK";
    
    public final static int DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT = 101;
    public final static String DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT_ACTION_COMMAND  = "DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK";

    public final static int NOTIFY_MOUSE_MOVED_IN_INTERACTIVE_MODE_ENABLED_EVENT = 102;
    public final static String NOTIFY_MOUSE_MOVED_IN_INTERACTIVE_MODE_ENABLED_COMMAND  = "NOTIFY_MOUSE_MOVED_IN_INTERACTIVE_MODE_ENABLED_EVENT";

    public final static int MOUSE_DOUBLE_CLICKED_EVENT = 103;
    public final static String MOUSE_DOUBLE_CLICKED_COMMAND = "MOUSE_DOUBLE_CLICKED_COMMAND";

            
    private boolean         m_updatedColorMapOnce = false;
    
    protected JPopupMenu    m_popupmenu;
    protected JMenu         m_multiImageMenu;
    protected JMenuItem     m_displaySliceNumber;
    protected JMenuItem     m_performSimple3DCut;
    protected JMenuItem     m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly;
    protected JMenuItem     m_computePropertiesofSelectedROIForAllSlicesForAllRegions;
    protected JMenu         m_computeRegionPropertiesForAllSlicesMenu;
    protected JMenuItem     m_reduceImageMatrixAndWriteAsDICOM;
    
    protected JMenu         m_ImageStackMinMaxImageNumberThatHasFocusDefintionMenu = new JMenu( "Image range focus" );

    protected JMenuItem     m_ImageStackMinimumImageNumberThatHasFocus;
    protected JMenuItem     m_ImageStackMaximumImageNumberThatHasFocus;

    protected int           m_minimumImageNumberToFocus = -1;
    protected int           m_maximumImageNumberToFocus = -1;

    public final static int NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT = 66;
    public final static int NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT = 67;

    // For proper serialization
    private static final long serialVersionUID = 1482324591289393L;
    
    public Vector< LinkedList <Distance> > m_distancesVector = null;

    // code logic for the #jMRUI-release#
    protected boolean m_isJMRUIRelease = false;
    
    protected float   m_nordicScaleFactor = 1.0f;
}
