/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.geometricobjects;

import org.spectrim.imagegraphicobjects.Point3D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import org.spectrim.registration.DisplacementField;

/**
 *
 * @author nunopbarros
 */
public class MeshCube3D {

    private HashMap<String, Point3D> extPoints = new HashMap<>();
    private HashMap<String, Point3D> intPoints = new HashMap<>();
    private Cube3D cube;
    private int divisionFact;

    public MeshCube3D(Cube3D cube, int divisionFact) {
        this.cube = cube;
        this.divisionFact = divisionFact;
        Point3D v1 = cube.p2.subtractPoint(cube.p1);
        Point3D v2 = cube.p1.subtractPoint(cube.p5);
        Point3D v3 = cube.p4.subtractPoint(cube.p1);


//      faces 1 and 3 
        for (int i = 0; i <= divisionFact; i++) {
            for (int j = 0; j <= divisionFact; j++) {
                Point3D pFace1 = Point3D.vectorCombination(v1, v3, v2, (double) i / (double) divisionFact, (double) j / (double) divisionFact, 0).translate(cube.p5);
                Point3D pFace3 = Point3D.vectorCombination(v1, v3, v2, (double) i / (double) divisionFact, (double) j / (double) divisionFact, 1).translate(cube.p5);
                extPoints.put(1 + "_" + i + "_" + j, pFace1);
                extPoints.put(3 + "_" + i + "_" + j, pFace3);
            }
        }

//      faces 2 and 4 
        for (int i = 0; i <= divisionFact; i++) {
            for (int j = 1; j < divisionFact; j++) {
                Point3D pFace2 = Point3D.vectorCombination(v1, v2, v3, (double) i / (double) divisionFact, (double) j / (double) divisionFact, 0).translate(cube.p5);
                Point3D pFace4 = Point3D.vectorCombination(v1, v2, v3, (double) i / (double) divisionFact, (double) j / (double) divisionFact, 1).translate(cube.p5);
                extPoints.put(2 + "_" + i + "_" + j, pFace2);
                extPoints.put(4 + "_" + i + "_" + j, pFace4);
            }
        }

//      faces 5 and 6 
        for (int i = 1; i < divisionFact; i++) {
            for (int j = 1; j < divisionFact; j++) {
                Point3D pFace5 = Point3D.vectorCombination(v2, v3, v1, (double) i / (double) divisionFact, (double) j / (double) divisionFact, 0).translate(cube.p5);
                Point3D pFace6 = Point3D.vectorCombination(v2, v3, v1, (double) i / (double) divisionFact, (double) j / (double) divisionFact, 1).translate(cube.p5);
                extPoints.put(5 + "_" + i + "_" + j, pFace5);
                extPoints.put(6 + "_" + i + "_" + j, pFace6);
            }
        }


        //     internal points
        for (int i = 1; i < divisionFact; i++) {
            for (int j = 1; j < divisionFact; j++) {
                for (int k = 1; k < divisionFact; k++) {
                    Point3D pint = Point3D.vectorCombination(v1, v2, v3, (double) i / (double) divisionFact, (double) j / (double) divisionFact, (double) k / (double) divisionFact).translate(cube.p5);
                    intPoints.put(i + "_" + j + "_" + k, pint);
                }
            }
        }

    }

    public HashMap<String, Point3D> getExternalPoints() {
        return extPoints;
    }

    public HashMap<String, Point3D> getInternalPoints() {
        return intPoints;
    }

//    public ArrayList<Line> getLines() {
//        ArrayList<Line> lines = new ArrayList<>();
//        for (int f = 1; f <= 6; f++) {
//            for (int i = 0; i <= divisionFact; i++) {
//                for (int j = 0; j <= divisionFact; j++) {
//                    Point3D p = extPoints.get(f + "_" + i + "_" + j);
//                    Point3D p1 = extPoints.get(f + "_" + (i + 1) + "_" + j);
//                    Point3D p2 = extPoints.get(f + "_" + i + "_" + (j + 1));
//                    if (p1 != null && p != null) {
//                        lines.add(new Line(p, p1));
//                    }
//                    if (p2 != null && p != null) {
//                        lines.add(new Line(p, p2));
//                    }
//                }
//            }
//        }
//
//        for (int i = 0; i <= divisionFact; i++) {
//            Point3D p = extPoints.get(3 + "_" + i + "_" + j);
//            Point3D p1 = extPoints.get(f + "_" + (i + 1) + "_" + j);
//            Point3D p2 = extPoints.get(f + "_" + i + "_" + (j + 1));
//        }
//
//        return lines;
//    }

    public void applyDisplacements(DisplacementField disp) {
        Iterator<String> iter = extPoints.keySet().iterator();

        while (iter.hasNext()) {
            String key = iter.next();
            Point3D p = extPoints.get(key);
            Point3D disp_p = disp.getDisplacementForPoint(p);
            if (disp_p != null) {
                p = p.addPoint(disp_p);
            } else {
                System.out.println("Displacement not defined for point " + p.asString());
            }
            extPoints.put(key, p);
        }
        iter = intPoints.keySet().iterator();

        while (iter.hasNext()) {
            String key = iter.next();
            Point3D p = intPoints.get(key);
            Point3D disp_p = disp.getDisplacementForPoint(p);
            if (disp_p != null) {
                p = p.addPoint(disp_p);
            } else {
                System.out.println("Displacement not defined for point " + p.asString());
            }
            intPoints.put(key, p);
        }
    }

    public Cube3D getCube() {
        return cube;
    }
}
