/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;
import javax.swing.JFrame;
import javax.swing.table.DefaultTableModel;
import org.spectrim.tdfdfit.MRFittingParameters;
import org.spectrim.tdfdfit.MRLineModelSpectralResponse;
import org.spectrim.tdfdfit.MRSignalProperties;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 * Instance enabling the users to control the HLSVD.
 * @author Johannes Slotboom
 */
public class HLSVDControlPanel extends javax.swing.JPanel
{

    /**
     * Creates new form HLSVDControlPanel.
     * It enables the user to launch HLSVD and severaL filtering options
     * can be applied.
     */
    public HLSVDControlPanel( ) 
    {
        initComponents( );
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
         
        m_numberOfHSLVDCOmponentsTextField.setText( defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS ) );
        m_minimumWaterRemovalFrequencyRangeValueTextField.setText( defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY ) );
        m_maximumWaterRemovalFrequencyRangeValueTextField.setText( defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY ) );
        m_maximumTolerableDampingTextField.setText( defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD ) );
             
    }

    /**
     * Sets the parent frame (needed to close the JFrame).
     * @param frame 
     */
    public void setParentFrame( JFrame frame )
    {
        m_parentFrame = frame; 
    }
    
    /**
     * Adds an ActionListener to the list of listeners.
     * @param listener
    */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }
    
    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( Object obj, int eventType, String actionCommand ) 
    {
        Iterator it = m_actionListeners.iterator( );
        while ( it.hasNext( ) ) 
        {
            ( (ActionListener) it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        m_performHLSVDButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        m_numberOfHSLVDCOmponentsTextField = new javax.swing.JTextField();
        m_removeWaterCheckbox = new java.awt.Checkbox();
        jLabel2 = new javax.swing.JLabel();
        m_minimumWaterRemovalFrequencyRangeValueTextField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        m_maximumWaterRemovalFrequencyRangeValueTextField = new javax.swing.JTextField();
        m_filterOutBroadComponentsCheckbox = new java.awt.Checkbox();
        jLabel4 = new javax.swing.JLabel();
        m_maximumTolerableDampingTextField = new javax.swing.JTextField();
        m_mrParameterSpectrumModelViewer = new org.spectrim.tdfdfit.MRParameterSpectrumModelViewer();
        m_filterOutSelectedTableComponentsButton = new javax.swing.JButton();
        m_setHLSVDDefaultsButton = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        m_spectralComponentsTable = new javax.swing.JTable();
        jPanel4 = new javax.swing.JPanel();
        m_ApplyButton = new javax.swing.JButton();
        m_quitjButton = new javax.swing.JButton();

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("HLSVD & Filter settings"));

        m_performHLSVDButton.setText("Perform HLSVD");
        m_performHLSVDButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_performHLSVDButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Number of components HLSVD ");

        m_numberOfHSLVDCOmponentsTextField.setText("30");

        m_removeWaterCheckbox.setLabel("Remove water");
        m_removeWaterCheckbox.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                m_removeWaterCheckboxMouseClicked(evt);
            }
        });

        jLabel2.setText("Minimum water range [ppm]");

        m_minimumWaterRemovalFrequencyRangeValueTextField.setText("482");
        m_minimumWaterRemovalFrequencyRangeValueTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_minimumWaterRemovalFrequencyRangeValueTextFieldActionPerformed(evt);
            }
        });

        jLabel3.setText("Maximum water range [ppm]");

        m_maximumWaterRemovalFrequencyRangeValueTextField.setText("542");
        m_maximumWaterRemovalFrequencyRangeValueTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_maximumWaterRemovalFrequencyRangeValueTextFieldActionPerformed(evt);
            }
        });

        m_filterOutBroadComponentsCheckbox.setLabel("Remove broad components");
        m_filterOutBroadComponentsCheckbox.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                m_filterOutBroadComponentsCheckboxMouseClicked(evt);
            }
        });

        jLabel4.setText("Linewidths larger than [Hz]");

        m_maximumTolerableDampingTextField.setText("20");

        m_filterOutSelectedTableComponentsButton.setText("Filter out selected table components");
        m_filterOutSelectedTableComponentsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_filterOutSelectedTableComponentsButtonActionPerformed(evt);
            }
        });

        m_setHLSVDDefaultsButton.setText("Set HLSVD Defaults");
        m_setHLSVDDefaultsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_setHLSVDDefaultsButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(20, 20, 20)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel2)
                                    .addComponent(jLabel3))
                                .addGap(16, 16, 16)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(m_minimumWaterRemovalFrequencyRangeValueTextField)
                                    .addComponent(m_maximumWaterRemovalFrequencyRangeValueTextField)))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jLabel4)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_maximumTolerableDampingTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 29, Short.MAX_VALUE)
                                .addComponent(m_numberOfHSLVDCOmponentsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 99, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(m_filterOutBroadComponentsCheckbox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_removeWaterCheckbox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(42, 42, 42)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(m_performHLSVDButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_filterOutSelectedTableComponentsButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_setHLSVDDefaultsButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGap(0, 0, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_mrParameterSpectrumModelViewer, javax.swing.GroupLayout.PREFERRED_SIZE, 800, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(m_mrParameterSpectrumModelViewer, javax.swing.GroupLayout.DEFAULT_SIZE, 637, Short.MAX_VALUE)
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addGap(78, 78, 78)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(m_numberOfHSLVDCOmponentsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(37, 37, 37)
                .addComponent(m_removeWaterCheckbox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(23, 23, 23)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(m_minimumWaterRemovalFrequencyRangeValueTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(m_maximumWaterRemovalFrequencyRangeValueTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(20, 20, 20)
                .addComponent(m_filterOutBroadComponentsCheckbox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(m_maximumTolerableDampingTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_performHLSVDButton)
                .addGap(18, 18, 18)
                .addComponent(m_filterOutSelectedTableComponentsButton)
                .addGap(77, 77, 77)
                .addComponent(m_setHLSVDDefaultsButton)
                .addGap(21, 21, 21))
        );

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 671, Short.MAX_VALUE)
        );

        m_spectralComponentsTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null}
            },
            new String [] {
                "Number", "Amplitude", "% Amplitude", "Frequency", "Damping", "Phase"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.Double.class, java.lang.Double.class, java.lang.Double.class, java.lang.Double.class, java.lang.Double.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jScrollPane1.setViewportView(m_spectralComponentsTable);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1)
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 139, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_ApplyButton.setText("Apply and Quit");
        m_ApplyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_ApplyButtonActionPerformed(evt);
            }
        });

        m_quitjButton.setText("Quit");
        m_quitjButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_quitjButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGap(231, 231, 231)
                .addComponent(m_ApplyButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_quitjButton, javax.swing.GroupLayout.PREFERRED_SIZE, 102, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(175, 175, 175))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                .addContainerGap(30, Short.MAX_VALUE)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_ApplyButton)
                    .addComponent(m_quitjButton))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void m_minimumWaterRemovalFrequencyRangeValueTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_minimumWaterRemovalFrequencyRangeValueTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_minimumWaterRemovalFrequencyRangeValueTextFieldActionPerformed

    private void m_removeWaterCheckboxMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_m_removeWaterCheckboxMouseClicked
        
        if ( m_removeWaterCheckbox.getState( ) == true )
        {
            m_performHLSVDButtonActionPerformed( null );
            performDefaultHLSVDWaterLineRemovalOnAllTableItems( );
        }
        else
        {
            m_mrParameterSpectrumModelViewer.deleteAllCurves( );

        }
        
    }//GEN-LAST:event_m_removeWaterCheckboxMouseClicked

    private void m_performHLSVDButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_performHLSVDButtonActionPerformed
        
        // Arguments to be used to start HLSVD. (According to Ron de Beer)
        String[] pars = new String[]{ "testsignal", 
                                      "1024", 
                                      "1", 
                                      "1", 
                                      "x",
                                      "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits", 
                                      "1024", 
                                      "1024", 
                                      "300", 
                                      "30", 
                                      "4.0", 
                                      "6.0" };
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String tdSampleStepSize             = "" + ( 1.0 / MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) );
        
        String larmorFrequency              = "" + MRSignalProperties.getInstance( ).getResonanceFrequency( );
        
        String numberOfPointsInHLSVD        = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS );
        String hankelMatrixSize             = defaults.getProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE );
        String numberOfHLSVDComponents      = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS );
        String hlsvdMinimumFilterFrequency  = defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY );
        String hlsvdMaximumFilterFrequency  = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
        String extremeLineWidthThresshold   = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD ); 
        
        String[] arguments = new String[12];
        
        pars[0] = "null";
        pars[1] = "" + m_workingResponse.getAbsorptionChannelTD( ).length;
        pars[2] = "" + tdSampleStepSize; // overwrite "tdSampleStepSize" with 1: we want point units! 
        pars[3] = "" + larmorFrequency; // overwrite "larmorFrequency" with 1: we want point units! 
        pars[4] = "x";
        pars[5] = "null";
        pars[6] = "" + m_workingResponse.getAbsorptionChannelTD( ).length;
        pars[7] = "" + numberOfPointsInHLSVD;
        pars[8] = "" + hankelMatrixSize;
        pars[9] = "" + numberOfHLSVDComponents;
        pars[10] = "" + hlsvdMinimumFilterFrequency;
        pars[11] = "" + hlsvdMaximumFilterFrequency;
        
        
        SingleVoxelMeasuredSpectralData signal = new SingleVoxelMeasuredSpectralData( m_workingResponse );
        
        double amplitude_re_td_0 = signal.getAbsorptionChannelTD( )[0];
        double amplitude_im_td_0 = signal.getDispersionChannelTD( )[0];

        
        m_workNumberOfPoints = m_workingResponse.getAbsorptionChannelTD( ).length;
        
        // truncate to max 1024
        double[] abs = new double[m_workNumberOfPoints];
        double[] disp = new double[m_workNumberOfPoints];
       
        for ( int i = 0; i < m_workNumberOfPoints; i++ )
        {
            abs[i] = (double)signal.getAbsorptionChannelTD( )[i];
            disp[i] = (double)signal.getDispersionChannelTD( )[i];
            pars[1] = "" + m_workNumberOfPoints;
            pars[6] = "" + m_workNumberOfPoints;
        }

        double magnitude_td_sig_0 = Math.sqrt(amplitude_re_td_0 * amplitude_re_td_0  + amplitude_im_td_0 * amplitude_im_td_0 );
        
        // Try HLSVD with initial 
        Compute_Native_HLSVD hlsvd = new Compute_Native_HLSVD( pars, abs, disp );
        hlsvd.computeHLSVD( );
        m_hlsvdResults = hlsvd.getHLSVDResults( );

        double[] amps = m_hlsvdResults.getAmplitudesHLSVD( );
        double[] freq = m_hlsvdResults.getFrequenciesHLSVD( );
        double[] phas = m_hlsvdResults.getPhasesHLSVD( );
        double[] damp = m_hlsvdResults.getDampingsHLSVD( );

        // determine the sum of the amps
        double sumamps = 0.0;
        for ( int i = 0;i < amps.length; i++ )
        {
            sumamps += amps[i];
        }

        // test whether standard HLSVD failed..
        if ( sumamps < 0.01 * magnitude_td_sig_0 )
        {

            System.out.println( "Straight HLSVD Failed!" );
            // HLSVD failed apparently ...
            Random ran = new Random( );

            for ( int i = 0; i < m_workNumberOfPoints; i++ )
            {
                abs[i] = (double)signal.getAbsorptionChannelTD( )[i] + 0.018 * magnitude_td_sig_0 * ( ran.nextDouble( ) - 0.5 );
                disp[i] = (double)signal.getDispersionChannelTD( )[i] + 0.018 * magnitude_td_sig_0 * ( ran.nextDouble( ) - 0.5 );
            }

            hlsvd = new Compute_Native_HLSVD( pars, abs, disp );
            hlsvd.computeHLSVD( );
            m_hlsvdResults = hlsvd.getHLSVDResults( );

            amps = m_hlsvdResults.getAmplitudesHLSVD( );
            freq = m_hlsvdResults.getFrequenciesHLSVD( );
            phas = m_hlsvdResults.getPhasesHLSVD( );
            damp = m_hlsvdResults.getDampingsHLSVD( );  
        }

        double[] percentageAmps = new double[amps.length];
        double sum = 0.0;

        for ( int i = 0; i < hlsvd.getHLSVDResults( ).getNumberOfResonlineLines( ); i++ )
        {
            sum += amps[i];    
        }

        if ( sum != 0.0 )
        {
            for ( int i = 0; i < amps.length; i++ )
            {
                percentageAmps[i] = amps[i]*100.0/sum;    
            }
        }

        for ( int i = 0; i < freq.length; i++ )
        {
            freq[i] = freq[i] * m_workNumberOfPoints;
        }

        Object[][] data = new Object[amps.length][6];
        for ( int i = 0; i < amps.length; i++ )
        {
            data[i][0] = i;
            data[i][1] = amps[i];
            data[i][2] = percentageAmps[i];
            data[i][3] = freq[i];
            data[i][4] = phas[i] * 180.0 / Math.PI;
            data[i][5] = damp[i] * 1000.0f / Math.PI;
        }

        String[] columnNames = { "Number", "Amplitude", "%-Amplitude", 
                                 "Frequency", "Phase", "Damping" };
        DefaultTableModel  model = new DefaultTableModel( data, columnNames );
        this.m_spectralComponentsTable.setModel( model );

    }//GEN-LAST:event_m_performHLSVDButtonActionPerformed

    
    
    
    /**
     * This routine performs the HLSVD filtering on the selected items of
     * the JTable.
     */
    public void performHLSVDBroadBaseLineRemovalOnAllTableItems( )
    {
        m_spectralComponentsTable.selectAll( );
        
        m_broadBaselineResponse = new MRLineModelSpectralResponse( m_workNumberOfPoints );
        m_broadBaselineResponse.setCurveColor( Color.ORANGE );
        
        int[] selectedComponents = m_spectralComponentsTable.getSelectedRows( );
        
        float[] sumTDReal = new float[m_workNumberOfPoints];
        float[] sumTDImag = new float[m_workNumberOfPoints];
        
        double[] amps = m_hlsvdResults.getAmplitudesHLSVD( );
        double[] freqs = m_hlsvdResults.getFrequenciesHLSVD( );
        double[] phass = m_hlsvdResults.getPhasesHLSVD( );
        double[] damps = m_hlsvdResults.getDampingsHLSVD( );
        
        for ( int i = 0; i < amps.length; i++ )
        {
            freqs[i] = freqs[i] * m_workNumberOfPoints;
            phass[i] = phass[i] * 180.0 / Math.PI;
            damps[i] = damps[i] * 1000.0 / Math.PI;
        }
        
        for ( int i = 0; i < amps.length; i++ )
        {
            double ampl = amps[i];
            double freq = freqs[i];
            double phas = phass[i];
            double damp = damps[i];
            
            if ( Math.abs( damp ) > new Integer ( m_maximumTolerableDampingTextField.getText( ) ).doubleValue( ) )
            {
//                System.out.println( "ampl[" + i + "] = " + ampl + "  freq[" + i + "] = " + freq  + 
//                                    "  phas[" + i + "] = " + phas + "  damp[" + i + "] = " + damp );
            
                MRFittingParameters parameters = new MRFittingParameters( );
                parameters.setArea( (float)ampl );
                parameters.setOffset( (float)freq );
                parameters.setPhase( (float)phas );
                parameters.setLorentzianWidth( (float)Math.abs( damp ) );
                parameters.setGaussianWidth( 0.0f );
            
                MRLineModelSpectralResponse response = new MRLineModelSpectralResponse( parameters, m_workNumberOfPoints );
            
                float[] partReal = new float[m_workNumberOfPoints];
                partReal = response.getAbsorptionChannelTD( );
            
                float[] partImag = new float[m_workNumberOfPoints];
                partImag = response.getDispersionChannelTD( );
            
                partReal[0] = 2.0f * partReal[0];
                partImag[0] = 2.0f * partImag[0];
                
                m_broadBaselineResponse.addTimeDomainResponse( partReal, partImag );
            }
        }
        
        // Call copy contructor...
        SingleVoxelMeasuredSpectralData filteredSignal = new SingleVoxelMeasuredSpectralData( m_workingResponse );
        
        filteredSignal.subtractSingleVoxelSpectralDataObject( m_broadBaselineResponse );
        filteredSignal.setCurveColor( Color.RED );        
        
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Broad baseline response", m_broadBaselineResponse, true );
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Difference response", filteredSignal, true );
        
        m_hlsvdFilteredSignalList.add( filteredSignal );
    }
    
    /**
     * This routine performs the HLSVD filtering on the selected items of
     * the JTable.
     */
    public void performDefaultHLSVDWaterLineRemovalOnAllTableItems( )
    {
        
        m_hlsvdFilteredSignalList.clear( );
        m_spectralComponentsTable.selectAll( );
        
        m_residualWaterResonanceSignal = new SingleVoxelMeasuredSpectralData( );
        m_residualWaterResonanceSignal.setCurveColor( Color.GREEN );
        
        int[] selectedComponents = m_spectralComponentsTable.getSelectedRows( );
        
        float[] sumTDReal = new float[m_workNumberOfPoints];
        float[] sumTDImag = new float[m_workNumberOfPoints];
        
        double[] amps  = m_hlsvdResults.getAmplitudesHLSVD( );
        double[] freqs = m_hlsvdResults.getFrequenciesHLSVD( );
        double[] phass = m_hlsvdResults.getPhasesHLSVD( );
        double[] damps = m_hlsvdResults.getDampingsHLSVD( );
        
        for ( int i = 0; i < amps.length; i++ )
        {   
            freqs[i] = freqs[i];
            phass[i] = phass[i] * 180.0 / Math.PI;
            damps[i] = damps[i] * 1000.0 / Math.PI;
        }
        
        for ( int i = 0; i < selectedComponents.length; i++ )
        {            
            double ampl = amps[selectedComponents[i]];
            double freq = freqs[selectedComponents[i]];
            double phas = phass[selectedComponents[i]];
            double damp = damps[selectedComponents[i]];
            
             // Make frequency units to be expressed in Hz
             
            float minFreqStopInPPM = new Float ( m_minimumWaterRemovalFrequencyRangeValueTextField.getText( ) );  
            float maxFreqStopInPPM = new Float ( m_maximumWaterRemovalFrequencyRangeValueTextField.getText( ) ); 
            float totalSpectralWidthPPM = (float)( MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) * 1.0E6 / MRSignalProperties.getInstance( ).getResonanceFrequency( ) );
            float minFreqStopInHz = ( minFreqStopInPPM - MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( ) ) * MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) / totalSpectralWidthPPM  ;
            float maxFreqStopInHz = ( maxFreqStopInPPM - MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( ) ) * MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) / totalSpectralWidthPPM  ;
            
            System.out.println( "ampl[" + i + "] = " + ampl + "  freq[" + i + "] = " + freq  + 
                                "  phas[" + i + "] = " + phas + "  damp[" + i + "] = " + damp );
            
            if ( ( freq > minFreqStopInHz ) && ( freq < maxFreqStopInHz ) )
            {
                            
                MRFittingParameters parameters = new MRFittingParameters( );
                parameters.setArea( (float)ampl );
                parameters.setOffset( (float)freq );
                parameters.setPhase( (float)phas );
                parameters.setLorentzianWidth( (float)Math.abs( damp ) );
                parameters.setGaussianWidth( 0.0f );
                
                MRLineModelSpectralResponse response = new MRLineModelSpectralResponse( parameters, m_workNumberOfPoints );
            
                float[] partReal = new float[m_workNumberOfPoints];
                partReal = response.getAbsorptionChannelTD( );
            
                float[] partImag = new float[m_workNumberOfPoints];
                partImag = response.getDispersionChannelTD( );
            
                partReal[0] = 2.0f * partReal[0];
                partImag[0] = 2.0f * partImag[0];
                
                m_residualWaterResonanceSignal.addTimeDomainResponse( partReal, partImag );              
            }
        }
        
        // Call copy contructor...
        SingleVoxelMeasuredSpectralData filteredSignal = new SingleVoxelMeasuredSpectralData( m_workNumberOfPoints );
        
        float[] realWRR = new float[m_workNumberOfPoints];
        float[] imagWRR = new float[m_workNumberOfPoints];
        float[] realRS = new float[m_workNumberOfPoints];
        float[] imagRS = new float[m_workNumberOfPoints];
        
        float[] realWR = m_workingResponse.getAbsorptionChannelTD( );
        float[] imagWR = m_workingResponse.getDispersionChannelTD( );
        
        for ( int i = 0; i < realRS.length; i++ )
        {
            realWRR[i] = realWR[i];
            imagWRR[i] = imagWR[i];
        }
        
        realRS = m_residualWaterResonanceSignal.getAbsorptionChannelTD( );
        imagRS = m_residualWaterResonanceSignal.getDispersionChannelTD( );
        
        
        float factorHLSVD = m_workNumberOfPoints/1024f;
        
        for ( int i = 0; i < realRS.length; i++ )
        {
            realRS[i] = realWRR[i] - factorHLSVD * realRS[i];
            imagRS[i] = imagWRR[i] - factorHLSVD * imagRS[i];
        }
        
        filteredSignal.setNewTimeDomainResponse( realRS, imagRS );
        
        // filteredSignal.subtractSingleVoxelSpectralDataObject( m_residualWaterResonanceSignal );
        // filteredSignal.setCurveColor( Color.RED );  
        m_mrParameterSpectrumModelViewer.deleteAllCurves( );
        
        m_workingResponse.setCurveColor(Color.RED);
        m_residualWaterResonanceSignal.setCurveColor(Color.BLUE);
        filteredSignal.setCurveColor(Color.BLACK); 
        
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Working response", m_workingResponse, true );
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Residual water response", m_residualWaterResonanceSignal, true );
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Difference response", filteredSignal, true );
        
        m_hlsvdFilteredSignalList.add( filteredSignal );
 
    }
    
    
    /**
     * Methods that filters out the water signals based on the selections the
     * user made.
     * @param evt 
     */
    private void m_filterOutSelectedTableComponentsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_filterOutSelectedTableComponentsButtonActionPerformed
        
        
        m_residualWaterResonanceSignal = new SingleVoxelMeasuredSpectralData( m_workNumberOfPoints );
        m_residualWaterResonanceSignal.autoSetXaxis( );
        m_residualWaterResonanceSignal.setCurveColor( Color.BLACK );
        
        m_waterSignalToFilterOut = new MRLineModelSpectralResponse( m_workNumberOfPoints );
        m_waterSignalToFilterOut.autoSetXaxis( );
        m_waterSignalToFilterOut.setCurveColor( Color.GREEN );
                
        int[] selectedComponents = m_spectralComponentsTable.getSelectedRows( );
        
        float[] sumTDReal = new float[m_workNumberOfPoints];
        float[] sumTDImag = new float[m_workNumberOfPoints];
        
        double[] amps = m_hlsvdResults.getAmplitudesHLSVD( );
        double[] freqs = m_hlsvdResults.getFrequenciesHLSVD( );
        double[] phass = m_hlsvdResults.getPhasesHLSVD( );
        double[] damps = m_hlsvdResults.getDampingsHLSVD( );
        
        for ( int i = 0; i < amps.length; i++ )
        {
            freqs[i] = freqs[i] * m_workNumberOfPoints;
            phass[i] = phass[i] * 180.0 / Math.PI;
            damps[i] = damps[i] * 1000.0 / Math.PI;
        }
        
        for ( int i = 0; i < selectedComponents.length; i++ )
        {
            double ampl = amps[selectedComponents[i]];
            double freq = freqs[selectedComponents[i]];
            double phas = phass[selectedComponents[i]];
            double damp = damps[selectedComponents[i]];
            
            float minFreqStopInPPM = new Float ( m_minimumWaterRemovalFrequencyRangeValueTextField.getText( ) );  
            float maxFreqStopInPPM = new Float ( m_maximumWaterRemovalFrequencyRangeValueTextField.getText( ) ); 
            float totalSpectralWidthPPM = (float)( MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) * 1.0E6 / MRSignalProperties.getInstance( ).getResonanceFrequency( ) );
            float minFreqStopInHz = ( minFreqStopInPPM - MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( ) ) * MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) / totalSpectralWidthPPM  ;
            float maxFreqStopInHz = ( maxFreqStopInPPM - MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( ) ) * MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) / totalSpectralWidthPPM  ;
            
            System.out.println( "ampl[" + i + "] = " + ampl + "  freq[" + i + "] = " + freq  + 
                                "  phas[" + i + "] = " + phas + "  damp[" + i + "] = " + damp );
            
            if ( ( freq  > minFreqStopInHz ) && ( freq  < maxFreqStopInHz ) )            
            {
//                System.out.println( "ampl[" + i + "] = " + ampl + "  freq[" + i + "] = " + freq  + 
//                                    "  phas[" + i + "] = " + phas + "  damp[" + i + "] = " + damp );
            
                MRFittingParameters parameters = new MRFittingParameters( );
                parameters.setArea( (float)ampl );
                parameters.setOffset( (float)freq );
                parameters.setPhase( (float)phas );
                parameters.setLorentzianWidth( (float)Math.abs( damp ) );
                parameters.setGaussianWidth( 0.0f );
            
                MRLineModelSpectralResponse response = new MRLineModelSpectralResponse( parameters, m_workNumberOfPoints );
            
                float[] partReal = new float[m_workingResponse.getAbsorptionChannelTD( ).length];
                partReal = response.getAbsorptionChannelTD( );
            
                float[] partImag = new float[m_workingResponse.getAbsorptionChannelTD( ).length];
                partImag = response.getDispersionChannelTD( );
            
                partReal[0] = 2.0f * partReal[0];
                partImag[0] = 2.0f * partImag[0];
                
                m_waterSignalToFilterOut.addTimeDomainResponse( partReal, partImag );
            }
        }
        
        // Call copy contructor...
        SingleVoxelMeasuredSpectralData filteredSignal = new SingleVoxelMeasuredSpectralData( m_workingResponse );
        
        
        filteredSignal.subtractSingleVoxelSpectralDataObject( m_waterSignalToFilterOut );
        filteredSignal.setCurveColor( Color.RED );        
        
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Residual water response", m_waterSignalToFilterOut, true );
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Difference response", filteredSignal, true );
        
        m_hlsvdFilteredSignalList.add( filteredSignal );

        
    }//GEN-LAST:event_m_filterOutSelectedTableComponentsButtonActionPerformed

    private void m_maximumWaterRemovalFrequencyRangeValueTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_maximumWaterRemovalFrequencyRangeValueTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_maximumWaterRemovalFrequencyRangeValueTextFieldActionPerformed

    private void m_ApplyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_ApplyButtonActionPerformed
        notifyActionListeners( this, RETURN_HLSVD_FILTERED_SIGNAL_ID, RETURN_HLSVD_FILTERED_SIGNAL_CMD );
        m_parentFrame.dispose( );
    }//GEN-LAST:event_m_ApplyButtonActionPerformed

    private void m_filterOutBroadComponentsCheckboxMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_m_filterOutBroadComponentsCheckboxMouseClicked
        
        if ( m_filterOutBroadComponentsCheckbox.getState( ) == true )
        {
            m_performHLSVDButtonActionPerformed( null );
            performHLSVDBroadBaseLineRemovalOnAllTableItems( );
        }
        else
        {
            m_mrParameterSpectrumModelViewer.deleteAllCurves( );
            m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Signal", m_workingResponse, true );
        }
    }//GEN-LAST:event_m_filterOutBroadComponentsCheckboxMouseClicked

    private void m_setHLSVDDefaultsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_setHLSVDDefaultsButtonActionPerformed
        
        HLSVDDefaultsDefinerFrame frame = new HLSVDDefaultsDefinerFrame( );
        
    }//GEN-LAST:event_m_setHLSVDDefaultsButtonActionPerformed

    private void m_quitjButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_quitjButtonActionPerformed
        // do nothing: the user want to stop without saving. 
        m_parentFrame.dispose( );
    }//GEN-LAST:event_m_quitjButtonActionPerformed

    
    
    

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton m_ApplyButton;
    private java.awt.Checkbox m_filterOutBroadComponentsCheckbox;
    private javax.swing.JButton m_filterOutSelectedTableComponentsButton;
    private javax.swing.JTextField m_maximumTolerableDampingTextField;
    private javax.swing.JTextField m_maximumWaterRemovalFrequencyRangeValueTextField;
    private javax.swing.JTextField m_minimumWaterRemovalFrequencyRangeValueTextField;
    private org.spectrim.tdfdfit.MRParameterSpectrumModelViewer m_mrParameterSpectrumModelViewer;
    private javax.swing.JTextField m_numberOfHSLVDCOmponentsTextField;
    private javax.swing.JButton m_performHLSVDButton;
    private javax.swing.JButton m_quitjButton;
    private java.awt.Checkbox m_removeWaterCheckbox;
    private javax.swing.JButton m_setHLSVDDefaultsButton;
    private javax.swing.JTable m_spectralComponentsTable;
    // End of variables declaration//GEN-END:variables
    
    private double[]    m_signalReal;
    private double[]    m_signalImag;
    private HLSVDResults m_hlsvdResults;
    int     m_workNumberOfPoints;
    private JFrame      m_parentFrame;
    
    // Signals that result from filtering
    private SingleVoxelMeasuredSpectralData m_broadBaselineResponse = null;
    private SingleVoxelMeasuredSpectralData m_residualWaterResonanceSignal = null;
    private SingleVoxelMeasuredSpectralData m_waterSignalToFilterOut = null;
    
    private SingleVoxelMeasuredSpectralData m_workingResponse =  null;
    private LinkedList<SingleVoxelMeasuredSpectralData> m_hlsvdFilteredSignalList = new LinkedList<>( );
    public final static int RETURN_HLSVD_FILTERED_SIGNAL_ID = 0;
    public final static String RETURN_HLSVD_FILTERED_SIGNAL_CMD = "Return HLSVD filtered signal";
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    
    /**
     * Sets the response on which the HLSVD should be performed.
     * @param signal
     */
    void setSVSResponseToFilter( SingleVoxelMeasuredSpectralData signal ) 
    {
        // m_inputResponse = new SingleVoxelMeasuredSpectralData( signal ); 
        
        m_mrParameterSpectrumModelViewer.setTitle( "Model spectrum" );
        m_workingResponse = new SingleVoxelMeasuredSpectralData( signal );
        m_workingResponse.autoSetXaxis( );
        // m_workingResponse.doComplexConjuation( );
        
        m_mrParameterSpectrumModelViewer.addSpectralDataSet( "Signal", m_workingResponse, true );      
        // m_mrParameterSpectrumModelViewer.setTitle( "Model spectrum" );
        // m_mrParameterSpectrumModelViewer.addCurve( signal.getAbsorptionCurveFDPoints( ), true );
    }

    /**
     * Returns the user defined filtered version of m_signal.
     * @return 
     */
    public LinkedList<SingleVoxelMeasuredSpectralData> getHLSVDFilteredSignal( ) 
    {
        return m_hlsvdFilteredSignalList;
    }

}
