/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

/**
 * 
 * Container class containing the results of an HSLVD processing step
 * 
 * @author johan
 */
public class HLSVDResults 
{
    /**
     * Constructs a HLSVD container class.
     * @param n_time_points 
     * @param numberOfResonanceLines 
     */
    public HLSVDResults( int n_time_points, int numberOfResonanceLines )
    {
        m_ndp = n_time_points;
        m_numberOfResonanceLines = numberOfResonanceLines;
    }
            
    //////////////////////////////////////////////////////////////////////////        
    double[] m_signal_re;
    double[] m_signal_im;
    
    private double[] m_ampl;
    private double[] m_freq;
    private double[] m_damp;
    private double[] m_phase;

    private double[] m_recon_re; 
    private double[] m_recon_im; 
    private double[] m_filtered_re; 
    private double[] m_filtered_im; 
    private double[] m_res_water_model_re;
    private double[] m_res_water_model_im;
    private double[] m_broad_baseline_model_re;
    private double[] m_broad_baseline_model_im;
    
    private int      m_ndp;
    private int      m_numberOfResonanceLines;
    private double   m_step;
    private double   m_larmor;
    private String[] m_str_str;
    private String[] m_str_int;
    private String[] m_str_dbl;
    private String   m_signal_name;

    
    /**
     * Sets the real part of the signal on which an HLSVD is performed.
     * @param signal_re 
     */
    public void setSignalRealPart( double[] signal_re ) 
    {
        m_signal_re = new double[signal_re.length];
        System.arraycopy(signal_re, 0, m_signal_re, 0, signal_re.length);
    }

    /**
     * Sets the imaginary part of the signal on which an HLSVD is performed.
     * @param signal_im 
     */
    public void setSignalImaginaryPart( double[] signal_im ) 
    {
        m_signal_im = new double[signal_im.length];
        System.arraycopy(signal_im, 0, m_signal_im, 0, signal_im.length);
    }

    /**
     * Sets the signal name of the signal to be HLSVD processed.
     * @param signal_name 
     */
    public void setSignalName( String signal_name ) 
    {
        m_signal_name = signal_name;
    }

    /**
     * Sets the number of data samples of the time domain signal of which
     * HLSVD is computed.
     * @param ndp 
     */
    public void setNumberOfDataPoints( int ndp )
    {
        m_ndp = ndp;
    }

    /**
     * Sets the step attribute.
     * @param step 
     */
    public void setStepSize( double step ) 
    {
        m_step = step;
    }

    /**
     * Sets the larmor frequency attribute.
     * @param larmor 
     */
    public void setLarmorFrequency( double larmor ) 
    {
        m_larmor = larmor;
    }

    /**
     * Set the amplitudes of the components found.
     * @param ampl 
     */
    public void setAmplitudesHLSVD( double[] ampl ) 
    {
        m_ampl = new double[ampl.length];
        System.arraycopy( ampl, 0, m_ampl, 0, ampl.length );
    }

    /**
     * Set the frequencies of the components found.
     * @param freq
     */
    public void setFrequenciesHLSVD( double[] freq ) 
    {
        m_freq = new double[freq.length];
        System.arraycopy( freq, 0, m_freq, 0, freq.length );
    }

    /**
     * Set the Lorentzian damping constants of the components found.
     * @param damp 
     */
    public void setDampingsHLSVD( double[] damp ) 
    {
        m_damp = new double[damp.length];
        System.arraycopy( damp, 0, m_damp, 0, damp.length );
    }

    
    /**
     * * Set the phases of the components found.
     * @param phase 
     */
    public void setPhasesHLSVD(double[] phase) 
    {
       m_phase = new double[phase.length];
       System.arraycopy( phase, 0, m_phase, 0, phase.length );
    }
    
    /**
     * Set the recon_re signal resulting from HLSVD.
     * @param recon_re
     */
    public void setReconstructedSignalRealPart( double[] recon_re ) 
    {
        m_recon_re = new double[ recon_re.length ];
        System.arraycopy( recon_re, 0, m_recon_re, 0, recon_re.length ); 
    }
    
    /**
    * Set the recon_im signal resulting from HLSVD. 
    * @param recon_im 
    */
    public void setReconstructedSignalImagenaryPart( double[] recon_im ) 
    {
        m_recon_im = new double[ recon_im.length ];
        System.arraycopy( recon_im, 0, m_recon_im, 0, recon_im.length ); 
    }

    /**
     * Set the filtered_re signal resulting from HLSVD.
     * @param filtered_re 
     */
    public void setFilteredSignalRealPart( double[] filtered_re ) 
    {
        m_filtered_re = new double[ filtered_re.length ];
        System.arraycopy( filtered_re, 0, m_filtered_re, 0, filtered_re.length ); 
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public void setFilteredSignalImagenaryPart( double[] filtered_im ) 
    {
        m_filtered_im = new double[ filtered_im.length ];
        System.arraycopy( filtered_im, 0, m_filtered_im, 0, filtered_im.length ); 
    }
    
            
   /**
     * Set the filtered_im signal resulting from HLSVD. 
     * @param broad_baseline_model_im
     */
    public void setBroadBaselineModelSignalImaginaryPart( double[] broad_baseline_model_im ) 
    {
        m_broad_baseline_model_im = new double[ broad_baseline_model_im.length ];
        System.arraycopy( broad_baseline_model_im, 0, m_broad_baseline_model_im, 0, broad_baseline_model_im.length ); 
    }
    
    /**
     * Set the filtered_re signal resulting from HLSVD. 
     * @param broad_baseline_model_re
     */
    public void setBroadBaselineModelSignalRealPart( double[] broad_baseline_model_re ) 
    {
        m_broad_baseline_model_re = new double[ broad_baseline_model_re.length ];
        System.arraycopy( broad_baseline_model_re, 0, m_broad_baseline_model_re, 0, broad_baseline_model_re.length ); 
    } 
     
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public double[] getBroadBaselineModelSignalRealPart( ) 
    {
        return m_broad_baseline_model_re;
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public double[] getBroadBaselineModelSignalImaginaryPart( ) 
    {
        return m_broad_baseline_model_im;
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD. 
     * @param res_water_model_im
     */
    public void setResidualWaterModelSignalImaginaryPart( double[] res_water_model_im ) 
    {
        m_res_water_model_im = new double[ res_water_model_im.length ];
        System.arraycopy( res_water_model_im, 0, m_res_water_model_im, 0, res_water_model_im.length ); 
    }
    
    /**
     * Set the filtered_re signal resulting from HLSVD.
     * @param filtered_re 
     */
    public void setResidualWaterModelSignalRealPart( double[] res_water_model_re ) 
    {
        m_res_water_model_re = new double[ res_water_model_re.length ];
        System.arraycopy( res_water_model_re, 0, m_res_water_model_re, 0, res_water_model_re.length ); 
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public double[] getResidualWaterModelSignalRealPart( ) 
    {
        return m_res_water_model_re;
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public double[] getResidualWaterModelSignalImaginaryPart( ) 
    {
        return m_res_water_model_im;
    }
    
    
    
    
    
    /**
     * Sets the number of resonance lines found by the algorithm.
     * @param numberOfResonanceLines 
     */
    void setNumberOfResonlineLines( int numberOfResonanceLines ) 
    {
        m_numberOfResonanceLines = numberOfResonanceLines;
    }
    
    // Now the getter methods ...
    
    /**
     * Returns the real part of the signal on which an HLSVD is performed. 
     * @return  
     */
    public double[] getSignalRealPart( ) 
    {
        return m_signal_re;
    }

    /**
     * Returns the imaginary part of the signal on which an HLSVD is performed. 
     * @return  
     */
    public double[] getSignalImaginaryPart( ) 
    {
        return m_signal_im;
    }

    /**
     * Returns the signal name of the signal to be HLSVD processed. 
     * @return 
     */
    public String getSignalName(  ) 
    {
        return m_signal_name;
    }

    /**
     * Returns the number of data samples of the time domain signal of which
     * HLSVD is computed. 
     * @return  
     */
    public int getNumberOfDataPoints(  )
    {
        return m_ndp;
    }

    /**
     * Returns the step attribute. 
     * @return  
     */
    public double getStepSize( ) 
    {
        return m_step;
    }

    /**
     * Returns the larmor frequency attribute. 
     * @return  
     */
    public double getLarmorFrequency( ) 
    {
        return m_larmor;
    }

    /**
     * Returns the amplitudes of the components found. 
     * @return  
     */
    public double[] getAmplitudesHLSVD( ) 
    {
       return m_ampl;
    }

    /**
     * Returns the frequencies of the components found.
     * @return 
     */
    public double[] getFrequenciesHLSVD( ) 
    {
        return m_freq;
    }

    /**
     * Returns the Lorentzian damping constants of the components found. 
     * @return 
     */
    public double[] getDampingsHLSVD( ) 
    {
        return m_damp;
    }

    
    /**
     * Returns the phases of the components found. 
     * @return  
     */
    public double[] getPhasesHLSVD( ) 
    {
       return m_phase;
    }
    
    /**
     * Returns the recon_re signal resulting from HLSVD.
     * @return 
     */
    public double[] getReconstructedSignalRealPart( ) 
    {
       return m_recon_re; 
    }
    
    /**
    * Returns the recon_im signal resulting from HLSVD. 
     * @return  
    */
    public double[] getReconstructedSignalImagenaryPart( ) 
    {
        return m_recon_im; 
    }

    /**
     * Returns the filtered_re signal resulting from HLSVD. 
     * @return  
     */
    public double[] getFilteredSignalRealPart( ) 
    {
        return m_filtered_re; 
    }
    
    /**
     * Returns the filtered_im signal resulting from HLSVD. 
     * @return _re 
     */
    public double[] getFilteredSignalImagenaryPart( ) 
    {
        return m_filtered_im; 
    }
    
    /**
     * Sets the number of resonance lines found by the algorithm. 
     * @return 
     */
    public int getNumberOfResonlineLines( ) 
    {
        return m_numberOfResonanceLines;
    }
    
    
}
