/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

/**
 * 
 * Container class containing the results of an HSLVD processing step
 * 
 * @author johan
 */
public class HLSVDResults_batch 
{
    /**
     * Constructs a HLSVDPro_batch container class.
     * @param n_time_points 
     * @param numberOfResonanceLines 
     * @param batch_size
     */
    public HLSVDResults_batch( int n_time_points, int numberOfResonanceLines, int batch_size )
    {
        m_batch_size = batch_size;
        m_ndp = n_time_points;
        m_numberOfResonanceLines = numberOfResonanceLines;
    }
            
    ////////////////////////////////////////////////////////////////////////// 
    int m_batch_size;
    double[][] m_signal_re;
    double[][] m_signal_im;
    
    private double[][] m_ampl;
    private double[][] m_freq;
    private double[][] m_damp;
    private double[][] m_phase;

    private double[][] m_recon_re; 
    private double[][] m_recon_im; 
    private double[][] m_filtered_re; 
    private double[][] m_filtered_im; 
    private double[] m_res_water_model_re;
    private double[] m_res_water_model_im;
    
    private int      m_ndp;
    private int      m_numberOfResonanceLines;
    private double   m_step;
    private double   m_larmor;
    private String   m_signal_name;

    
    /**
     * Set a matrix of the original spectrum batch on which HLSVDPro will be applied.
     * The generated matrix size is [m_batch_size, m_ndp]
     * @param signal_re 
     */
    public void setSignalRealPart( double[] signal_re )
    {
        m_signal_re = new double[m_batch_size][m_ndp];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(signal_re, i*m_ndp, m_signal_re[i], 0,m_ndp);
        }
    }

    /**
     * Sets the imaginary part of the signal on which an HLSVD is performed.
     * @param signal_im 
     */
    public void setSignalImaginaryPart( double[] signal_im ) 
    {
        m_signal_im = new double[m_batch_size][m_ndp];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(signal_im, i*m_ndp, m_signal_im[i], 0,m_ndp);
        }
    }

    /**
     * Sets the signal name of the signal to be HLSVD processed.
     * @param signal_name 
     */
    public void setSignalName( String signal_name ) 
    {
        m_signal_name = signal_name;
    }

    /**
     * Sets the number of data samples of the time domain signal of which
     * HLSVD is computed.
     * @param ndp 
     */
    public void setNumberOfDataPoints( int ndp )
    {
        m_ndp = ndp;
    }

    /**
     * Sets the step attribute.
     * @param step 
     */
    public void setStepSize( double step ) 
    {
        m_step = step;
    }

    /**
     * Sets the larmor frequency attribute.
     * @param larmor 
     */
    public void setLarmorFrequency( double larmor ) 
    {
        m_larmor = larmor;
    }

    /**
     * Set the amplitudes of the components found.
     * @param ampl 
     */
    public void setAmplitudesHLSVD( double[] ampl ) 
    {
        m_ampl = new double[m_batch_size][m_numberOfResonanceLines];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(ampl, i*m_numberOfResonanceLines, m_ampl[i], 0,m_numberOfResonanceLines);
        }
    }

    /**
     * Set the frequencies of the components found.
     * @param freq
     */
    public void setFrequenciesHLSVD( double[] freq ) 
    {
        m_freq = new double[m_batch_size][m_numberOfResonanceLines];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(freq, i*m_numberOfResonanceLines, m_freq[i], 0,m_numberOfResonanceLines);
        }
    }

    /**
     * Set the Lorentzian damping constants of the components found.
     * @param damp 
     */
    public void setDampingsHLSVD( double[] damp ) 
    {
        m_damp = new double[m_batch_size][m_numberOfResonanceLines];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(damp, i*m_numberOfResonanceLines, m_damp[i], 0,m_numberOfResonanceLines);
        }
    }

    
    /**
     * * Set the phases of the components found.
     * @param phase 
     */
    public void setPhasesHLSVD(double[] phase) 
    {
        m_phase = new double[m_batch_size][m_numberOfResonanceLines];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(phase, i*m_numberOfResonanceLines, m_phase[i], 0,m_numberOfResonanceLines);
        }
    }
    
    /**
     * Set the recon_re signal resulting from HLSVD.
     * @param recon_re
     */
    public void setReconstructedSignalRealPart( double[] recon_re ) 
    {        
        m_recon_re = new double[m_batch_size][m_ndp];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(recon_re, i*m_ndp, m_recon_re[i], 0,m_ndp);
        }
    }
    
    /**
    * Set the recon_im signal resulting from HLSVD. 
    * @param recon_im 
    */
    public void setReconstructedSignalImagenaryPart( double[] recon_im ) 
    {
        m_recon_im = new double[m_batch_size][m_ndp];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(recon_im, i*m_ndp, m_recon_im[i], 0,m_ndp);
        }
    }

    /**
     * Set the filtered_re signal resulting from HLSVD.
     * @param filtered_re 
     */
    public void setFilteredSignalRealPart( double[] filtered_re ) 
    {
        m_filtered_re = new double[m_batch_size][m_ndp];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(filtered_re, i*m_ndp, m_filtered_re[i], 0,m_ndp);
        }
    }
    
    public void setFilteredSignalRealPart( double[] filtered_re, int index ) 
    {
        if ( m_filtered_re==null ){
            m_filtered_re = new double[m_batch_size][m_ndp];
        }
        if ( index>=m_batch_size ){
            m_filtered_re[index] = null;
        }
        System.arraycopy(filtered_re, index*m_ndp, m_filtered_re[index], 0,m_ndp);
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public void setFilteredSignalImagenaryPart( double[] filtered_im ) 
    {
        m_filtered_im = new double[m_batch_size][m_ndp];
        for ( int i=0; i<m_batch_size; i++ ){
            System.arraycopy(filtered_im, i*m_ndp, m_filtered_im[i], 0,m_ndp);
        }
    }
    
    public void setFilteredSignalImagenaryPart( double[] filtered_im, int index ) 
    {
        if ( m_filtered_im==null ){
            m_filtered_im = new double[m_batch_size][m_ndp];
        }
        if ( index>=m_batch_size ){
            m_filtered_im[index] = null;
        }
        System.arraycopy(filtered_im, index*m_ndp, m_filtered_im[index], 0,m_ndp);
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD. 
     * @param res_water_model_im
     */
    public void setResidualWaterModelSignalImaginaryPart( double[] res_water_model_im ) 
    {
        m_res_water_model_im = new double[ res_water_model_im.length ];
        System.arraycopy( res_water_model_im, 0, m_res_water_model_im, 0, res_water_model_im.length ); 
    }
    
    /**
     * Set the filtered_re signal resulting from HLSVD.
     * @param filtered_re 
     */
    public void setResidualWaterModelSignalRealPart( double[] res_water_model_re ) 
    {
        m_res_water_model_re = new double[ res_water_model_re.length ];
        System.arraycopy( res_water_model_re, 0, m_res_water_model_re, 0, res_water_model_re.length ); 
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public double[] getResidualWaterModelSignalRealPart( ) 
    {
        return m_res_water_model_re;
    }
    
    /**
     * Set the filtered_im signal resulting from HLSVD.
     * @param filtered_im 
     */
    public double[] getResidualWaterModelSignalImaginaryPart( ) 
    {
        return m_res_water_model_im;
    }
    
    
    
    
    
    /**
     * Sets the number of resonance lines found by the algorithm.
     * @param numberOfResonanceLines 
     */
    void setNumberOfResonlineLines( int numberOfResonanceLines ) 
    {
        m_numberOfResonanceLines = numberOfResonanceLines;
    }
    
    // Now the getter methods ...
    
    /**
     * Returns the real part of the signal on which an HLSVD is performed. 
     * @param index
     * @return  
     */
    public double[] getSignalRealPart( int index )
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_signal_re[index];
    }
    
    /**
     * Returns the real part of the signal on which an HLSVD is performed. 
     * @return  
     */
    public double[][] getSignalRealPart( ) 
    {
        return m_signal_re;
    }

    /**
     * Returns the imaginary part of the signal on which an HLSVD is performed. 
     * @param index
     * @return  
     */
    public double[] getSignalImaginaryPart( int index )
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_signal_im[index];
    }
    
    /**
     * Returns the imaginary part of the signal on which an HLSVD is performed. 
     * @return  
     */
    public double[][] getSignalImaginaryPart( )
    {
        return m_signal_im;
    }

    /**
     * Returns the signal name of the signal to be HLSVD processed. 
     * @return 
     */
    public String getSignalName(  ) 
    {
        return m_signal_name;
    }

    /**
     * Returns the number of data samples of the time domain signal of which
     * HLSVD is computed. 
     * @return  
     */
    public int getNumberOfDataPoints(  )
    {
        return m_ndp;
    }

    /**
     * Returns the step attribute. 
     * @return  
     */
    public double getStepSize( ) 
    {
        return m_step;
    }

    /**
     * Returns the larmor frequency attribute. 
     * @return  
     */
    public double getLarmorFrequency( ) 
    {
        return m_larmor;
    }

    /**
     * Returns the amplitudes of the components found. 
     * @return  
     */
    public double[] getAmplitudesHLSVD( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_ampl[index];
    }

    /**
     * Returns the frequencies of the components found.
     * @return 
     */
    public double[] getFrequenciesHLSVD( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_freq[index];
    }

    /**
     * Returns the Lorentzian damping constants of the components found. 
     * @return 
     */
    public double[] getDampingsHLSVD( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_damp[index];
    }

    
    /**
     * Returns the phases of the components found. 
     * @return  
     */
    public double[] getPhasesHLSVD( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_phase[index];
    }
    
    /**
     * Returns the recon_re signal resulting from HLSVD.
     * @return 
     */
    public double[] getReconstructedSignalRealPart( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_recon_re[index]; 
    }
    
    /**
    * Returns the recon_im signal resulting from HLSVD. 
     * @return  
    */
    public double[] getReconstructedSignalImagenaryPart( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_recon_im[index]; 
    }

    /**
     * Returns the filtered_re signal resulting from HLSVD. 
     * @return  
     */
    public double[] getFilteredSignalRealPart( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_filtered_re[index];
    }
    
    /**
     * Returns the filtered_im signal resulting from HLSVD. 
     * @return _re 
     */
    public double[] getFilteredSignalImagenaryPart( int index ) 
    {
        if ( index>=m_batch_size ){
            return null;
        }
        return m_filtered_im[index]; 
    }
    
    /**
     * Sets the number of resonance lines found by the algorithm. 
     * @return 
     */
    public int getNumberOfResonlineLines( ) 
    {
        return m_numberOfResonanceLines;
    }
    
    
}
