/*
 * MirrorAxis.java
 *
 * Created on March 5, 2004, 9:05 AM
 */

package org.spectrim.imagegraphicobjects;

import java.util.Iterator;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.JFrame;
import java.util.LinkedList;
import java.awt.geom.Point2D;


/**
 * Class that defines a mirror axis for images; Objects of this class can 
 * be used eg axial and coronal brain images.
 * @author  hans
 */
public class MirrorAxis implements ActionListener
{
    
    /**
     * Creates a new instance of MirrorAxis 
     * @param parent
     * @param imageWidth
     * @param imageHeight
     * @param zoom
     */
    public MirrorAxis( JFrame parent, int imageWidth, int imageHeight, float zoom )  
    {
        m_x = (int)( zoom * imageWidth / 2 );
        m_y = (int)( zoom * imageHeight / 2 );
        m_dir_x = 0.0;
        m_dir_y = 1.0;
        m_imageWidth = (int)( zoom * imageWidth );
        m_imageHeight = (int)( zoom * imageHeight );
        m_parent = parent;
        m_zoom = zoom;
    }
    
    /**
     * Creates a new instance of MirrorAxis 
     * @param parent
     * @param origenX
     * @param origenY
     * @param dir_x
     * @param dir_y
     * @param imageWidth
     * @param imageHeight
     * @param zoom
     */    
    public MirrorAxis( JFrame parent, 
                       int origenX, int origenY, int dir_x, int dir_y, 
                       int imageWidth, int imageHeight, float zoom ) 
    {
        m_x = origenX;
        m_y = origenY;
        m_dir_x = dir_x;
        m_dir_y = dir_y;
        m_imageWidth = (int)( zoom * imageWidth );
        m_imageHeight = (int)( zoom * imageHeight );
        m_parent = parent;
        m_zoom = zoom;
    }
    
    /**
     * Sets the value of the x-coordinate of a point on the mirror axis
     * @param origenX
     */
    public void setX( int origenX )
    {
        m_x = origenX;
    }
    
    /**
     * Returns the value of the x-coordinate of a point on the mirror axis.
     * @return 
     */
    public int getX(  )
    {
        return m_x;
    }

    /**
     * Sets the value of the y-coordinate of a point on the mirror axis.
     * @param origenY
     */
    public void setY( int origenY )
    {
        m_y = origenY;
    }
    
    /**
     * Returns the value of the x-coordinate of a point on the mirror axis.
     * @return 
     */
    public int getY(  )
    {
        return m_y;
    }

    /**
     * Sets the value of the x-coordinate of a direction vector.
     * @param dirX
     */
    public void setDirX( double dirX )
    {
        m_dir_x = dirX;
    }
    
    /**
     * Returns the value of the x-coordinate of a point on the mirror axis.
     * @return 
     */
    public double getDirX(  )
    {
        return m_dir_x;
    }

    /**
     * Sets the value of the y-coordinate of a direction vector.
     * @param dirY
     */
    public void setDirY( double dirY )
    {
        m_dir_y = dirY;
    }
    
    /**
     * Returns the value of the y-coordinate of a direction vector.
     * @return 
     */
    public double getDirY(  )
    {
        return m_dir_y;
    }
    

    /**
     * Sets the image width.
     * @param width
     */
    public void setImageWidth( int width )
    {
        m_imageWidth = width;
    }
    
    /**
     * Returns the image width.
     * @return 
     */
    public int getImageWidth(  )
    {
        return m_imageWidth;
    }

    /**
     * Sets the image height.
     * @param height
     */
    public void setImageHeight( int height )
    {
        m_imageHeight = height;
    }
    
    /**
     * Returns the image height.
     * @return 
     */
    public int getImageHeight(  )
    {
        return m_imageHeight;
    }
    
    
    /**
     * Draws the mirror axis on the Graphics 
     */
    public void draw( )
    {
        draw( m_g );
    }
    
    /**
     * Draws the mirror axis on the Graphics 
     * @param g
     */
    public void draw( Graphics g )
    {
        
        if ( g != null )
        {
            m_g = g;
            // Calculate two points of rotation axis:
            // Calculate whether mirror line crosses the y = 0
            // this is the case if the calcutated x1 value
        
            // default a vertical line 
            double x1 = m_x;
            double y1 = 0;
            double x2  = m_x;
            double y2 = m_imageHeight;
        
            if ( m_dir_y != 0.0 )
            {
                // PRIMARY TASK TO FIND X1,Y1
                x1 = m_x - ( m_y * m_dir_x ) / m_dir_y;
                if ( x1 > m_imageWidth )
                {
                    // calculate the y-coord of the point were the mirror axis crosses the line x = getWidth()
                    x1 = m_imageWidth;
                    if ( m_dir_x != 0 )
                    {
                        y1 = m_y + ( m_imageWidth - m_x ) * m_dir_y / m_dir_x;
                    }
                    else
                    {
                        // so were dealing with a pure vertical line
                        x1 = m_x;
                        y1 = 0;
                        x2 = m_x;
                        y2 = m_imageHeight;
                    }
                }
                if ( x1 < 0 )
                {
                    // calculate the y-coord of the point were the mirror axis crosses the line x = 0
                    x1 = 0;
                    if ( m_dir_x != 0 )
                    {
                        y1 = m_y - ( m_x * m_dir_y / m_dir_x );
                    }
                    else
                    {
                        // so were dealing with a pure vertical line
                        x1 = m_x;
                        y1 = 0;
                        x2 = m_x;
                        y2 = m_imageHeight;
                    }
                }
            
                // PRIMARY TASK TO FIND X2,Y2; IN SOME case X1 AND Y1 FOLLOW
                // The P2 = (x2,y2) place where mirror axis crosses the line y=getHeigth()
                x2 = m_x + ( m_imageHeight - m_y ) * m_dir_x / m_dir_y;
            
                if ( x2 > m_imageWidth )
                {
                    // calculate the y-coord of the point were the mirror axis crosses the line x = getWidth()
                    x2 = m_imageWidth;
                    if ( m_dir_x != 0 )
                    {
                        y2 = m_y + ( m_imageWidth - m_x ) * m_dir_y / m_dir_x;
                    }
                    else
                    {
                        // so were dealing with a pure vertical line
                        x1 = m_x;
                        y1 = 0;
                        x2 = m_x;
                        y2 = m_imageHeight;
                    }
                }
                if ( x2 < 0 )
                {
                    // calculate the y-coord of the point were the mirror axis crosses the line x = 0
                    x2 = 0;
                    if ( m_dir_x != 0 )
                    {
                        y2 = m_y - ( m_x * m_dir_y / m_dir_x );
                    }
                    else
                    {
                        // so were dealing with a pure vertical line
                        x1 = m_x;
                        y1 = 0;
                        x2 = m_x;
                        y2 = m_imageHeight; 
                    }
                }  
            }
            else
            {
                // so were are dealing with a pure horizontal line
                x1 = 0;
                y1 = m_y;
                x2 = m_imageWidth;
                y2 = m_y;
            }
            
            g.setColor( Color.yellow );
            // draw the line itself
            g.drawLine( (int)x1, (int)y1, (int)x2, (int)y2 );
        }
    }
    
    /**
     * Reflect a point P in the current mirror axis
     * @param px
     * @param py
     * @return 
     */
    public Point2D.Double getMirrorPoint( double px, double py )
    {
        Point2D mP;
        
        double x = -px + 2.0 * m_x + 2.0 * ( ( px - m_x ) * m_dir_x + ( py - m_y ) * m_dir_y ) * m_dir_x; 
        double y = -py + 2.0 * m_y + 2.0 * ( ( px - m_x ) * m_dir_x + ( py - m_y ) * m_dir_y ) * m_dir_y;
        
        return new Point2D.Double( x, y );
    }
    
    /**
     * Allows the user to change the position of the mirror axis.
     */
    public void changeMirrorAxis( )
    {
        if ( m_axisControl == null )
        {
            m_x_old = m_x;
            m_y_old = m_y;
            m_dir_x_old = m_dir_x;
            m_dir_y_old = m_dir_y;
            
            m_axisControl = new MirrorAxisPositioningControl( "Change Position of Mirror Axis", m_parent, m_imageWidth, m_imageHeight );
            m_axisControl.addActionListener( this );
        }
    }
    
    /**
     * Called whenever a registered component raises an actionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender instanceof MirrorAxisPositioningControl )
        {
            
            if ( e.getID( ) == MirrorAxisPositioningControl.ANGLE_CHANGE_EVENT )
            {
                int angle_change = ((MirrorAxisPositioningControl)sender).getRotationAngle( );
                m_dir_x = Math.sin( angle_change * Math.PI / 180.0 );
                m_dir_y = Math.cos( angle_change * Math.PI / 180.0 );
                
                Iterator it = m_actionListeners.iterator( );
                
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, 0, "" ) );
                }
            }
            
            if ( e.getID( ) == MirrorAxisPositioningControl.HORIZONTAL_POSITION_CHANGE_EVENT )
            {
                m_x = ((MirrorAxisPositioningControl)sender).getHorizontalTranslation( );
                
                Iterator it = m_actionListeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, 0, "" ) );
                }
            }
            
            if ( e.getID( ) == MirrorAxisPositioningControl.VERTICAL_POSITION_CHANGE_EVENT )
            {
                m_y = ((MirrorAxisPositioningControl)sender).getVerticalTranslation( );
                
                Iterator it = m_actionListeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, 0, "" ) );
                }
            }
            if ( e.getID( ) == MirrorAxisPositioningControl.CANCEL_EVENT )
            {
                m_x = m_x_old;
                m_y = m_y_old;
                m_dir_x = m_dir_x_old;
                m_dir_y = m_dir_y_old;
                
                Iterator it = m_actionListeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, 0, "" ) );
                }
                m_axisControl.dispose( );
                m_axisControl = null;
            }
            if ( e.getID( ) == MirrorAxisPositioningControl.ACCEPT_EVENT )
            {
                m_axisControl.dispose( );
                m_axisControl = null;
            }

        }
    }    
     
    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens within this object
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    // Attributes
    private JFrame m_parent;
    
    private int m_x, m_y;
    private double m_dir_x, m_dir_y;
    
    private int m_x_old, m_y_old;
    private double m_dir_x_old, m_dir_y_old;
    
    
    private int m_imageWidth, m_imageHeight;
   
    private MirrorAxisPositioningControl m_axisControl;
    private Graphics m_g;
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );
    private float m_zoom = 1.0f;
}
