/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imagegraphicobjects;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * 
 * Tool bar that allows easy ROI editing mode handling.
 * 
 * @author slotboom
 */
public class RegionModeToolBar extends JPanel implements ActionListener
{
    
    /**
     * Constructor RegionModeToolBar.
     */
    public RegionModeToolBar( )
    {
        super( new BorderLayout( ) );

        //Create the toolbar.
        JToolBar toolBar = new JToolBar( "ROI-mode ToolBar" );
        addButtons( toolBar );

        add( toolBar, BorderLayout.PAGE_START );
        
        setSelectedButton( SELECT_ROI_ID );
        
        setVisible( true );
    }

    /**
     * Sets one of the toggle buttons selected.
     * @param id
     */
    public void setSelectedButton( int id ) 
    {
        switch( id )
        {
            case DELETE_ROI_ID:             m_deleteROIButton.setSelected( true );
                                            break;
            case SELECT_ROI_ID:             m_selectROIButton.setSelected( true );
                                            break;
            case ADD_ROI_ID:                m_addROIButton.setSelected( true );
                                            break;
            case ROI_CUT_ID:                m_cutROIButton.setSelected( true );
                                            break;            
            case EDIT_ROI_ID:               m_editROIButton.setSelected( true );
                                            break;          
            case AFFINE_TRANSFORM_ROI_ID:   m_affineTransformROIButton.setSelected( true );
                                            break;
            case DRAG_ROI_ID:               m_dragROIButton.setSelected( true );
                                            break;
        }
    }

    /**
     * Add the buttons to the tool bar.
     * @param toolBar
     */
    protected void addButtons( JToolBar toolBar )
    {
        JToggleButton button = null;
        
        //first button
        m_selectROIButton = makeDicomToolBarButton( "Roi_select24", SELECT_ROI,
                                                    "Activates ROI selection mode",
                                                    "SelectROI" );
        toolBar.add( m_selectROIButton );

        //second button
        m_addROIButton = makeDicomToolBarButton( "Roi_add24", ADD_ROI,
                                                 "Activates add ROI mode",
                                                 "AddROI");
        toolBar.add( m_addROIButton );

        //third button
        m_deleteROIButton = makeDicomToolBarButton( "Roi_delete24", DELETE_ROI,
                                                    "Activates delete ROI mode",
                                                    "DeleteROI" );
        toolBar.add( m_deleteROIButton );


        //fourth button
        m_cutROIButton = makeDicomToolBarButton( "Roi_cut24", ROI_CUT,
                                                 "Activates ROI cut mode",
                                                 "CutROI" );
        toolBar.add( m_cutROIButton );

        //fifth button
        m_editROIButton = makeDicomToolBarButton( "Roi_edit24", EDIT_ROI,
                                                  "Activates ROI-node editing mode",
                                                  "EditRoiNodes" );

        toolBar.add( m_editROIButton );
        
        
        //sixth button
        m_dragROIButton = makeDicomToolBarButton( "Roi_drag24", DRAG_ROI,
                                                  "Activates ROI dragging editing mode",
                                                  "DragROI" );

        toolBar.add( m_dragROIButton );   
        
        
        //seventh button
        m_affineTransformROIButton = makeDicomToolBarButton( "Roi_affine24", AFFINE_TRANSFORM_ROI,
                                                             "Activates affine transformation editing mode",
                                                             "AffineTransformROI" );

        toolBar.add( m_affineTransformROIButton  );
        
    }


    /**
     *
     * Creates all buttons of the tool bar.
     *
     * @param imageName
     * @param actionCommand
     * @param toolTipText
     * @param altText
     * @return the button to add
     */
    protected JToggleButton makeDicomToolBarButton( String imageName,
                                                    String actionCommand,
                                                    String toolTipText,
                                                    String altText )
    {
        //Look for the image.
        String workingDir = System.getProperty( "user.dir" );
        String imgLocation = workingDir + System.getProperty( "file.separator" )
                                        + "images"
                                        + System.getProperty( "file.separator" )
                                        + imageName
                                        + ".gif";
        File imageFile = new File( imgLocation );

        //Create and initialize the button.
        JToggleButton button = new JToggleButton( );
        button.setActionCommand( actionCommand );
        button.setToolTipText( toolTipText );
        button.addActionListener( this );

        if ( imageFile.exists( ) )
        {                      //image found
            button.setIcon( new ImageIcon( imgLocation ) );
        }
        else
        {                        //no image found
            button.setText( altText );
            System.err.println( "Resource not found: "
                               + imgLocation );
        }
        
        m_buttonGroup.add( button );
        
        return button;
    }


    /**
     *
     * Is called whenever a registered component raises an ActionEvent.
     *
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        String cmd = e.getActionCommand( );
        String description = null;

        // Handle each button.
        if ( DELETE_ROI.equals( cmd ) )
        { //first button clicked
           notifyActionListeners( DELETE_ROI, DELETE_ROI_ID );
        }
        else if ( SELECT_ROI.equals( cmd ) )
        {   // second button clicked
            notifyActionListeners( SELECT_ROI, SELECT_ROI_ID );
        }
        else if ( ADD_ROI.equals( cmd ) )
        {   // third button clicked
            notifyActionListeners( ADD_ROI, ADD_ROI_ID );
        }
        else if ( ROI_CUT.equals( cmd ) )
        {   // fourth button clicked
            notifyActionListeners( ROI_CUT, ROI_CUT_ID );
        }
        else if ( EDIT_ROI.equals( cmd ) )
        {   // fifth button clicked
            notifyActionListeners( EDIT_ROI, EDIT_ROI_ID );
        }
        else if ( DRAG_ROI.equals( cmd ) )
        {   // fifth button clicked
            notifyActionListeners( DRAG_ROI, DRAG_ROI_ID );
        }
        else if ( AFFINE_TRANSFORM_ROI.equals( cmd ) )
        {   // sixth button clicked
            notifyActionListeners( AFFINE_TRANSFORM_ROI, AFFINE_TRANSFORM_ROI_ID );
        }
    }

    /**
     * Create the GUI and show it;  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    private static void createAndShowGUI( )
    {
        //Create and set up the window.
        JFrame frame = new JFrame( "ROIModeToolBar" );
        frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

        //Add content to the window.
        frame.add( new RegionModeToolBar( )  );

        //Display the window.
        frame.pack( );
        frame.setVisible( true );
    }

    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( String actionCommand, int actionID )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionID, actionCommand ) );
        }
    }

    /**
     * Adds an action listener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }


    public static void main( String[] args )
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater( new Runnable( )
        {
            @Override
            public void run( )
            {
                //Turn off metal's use of bold fonts
                UIManager.put( "swing.boldMetal", Boolean.FALSE );
                createAndShowGUI( );
            }
        });
    }

    static final public String SELECT_ROI               = "Activates ROI selection mode";
    static final public String ADD_ROI                  = "Activates add ROI mode";
    static final public String DELETE_ROI               = "Activates delete ROI mode";
    static final public String ROI_CUT                  = "Activates ROI cut mode";
    static final public String EDIT_ROI                 = "Activates ROI-node editing mode";
    static final public String AFFINE_TRANSFORM_ROI     = "Activates affine transformation editing mode";
    static final public String DRAG_ROI                 = "Activates the drag ROI editing mode";
    
    static final public int SELECT_ROI_ID               = 0;
    static final public int ADD_ROI_ID                  = 1;
    static final public int DELETE_ROI_ID               = 2;
    static final public int ROI_CUT_ID                  = 3;
    static final public int EDIT_ROI_ID                 = 4;
    static final public int AFFINE_TRANSFORM_ROI_ID     = 5;
    static final public int DRAG_ROI_ID                 = 6;
    
    private JToggleButton m_selectROIButton;
    private JToggleButton m_addROIButton;
    private JToggleButton m_deleteROIButton;
    private JToggleButton m_cutROIButton;
    private JToggleButton m_editROIButton;
    private JToggleButton m_affineTransformROIButton;
    private JToggleButton m_dragROIButton;
    
    LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    ButtonGroup m_buttonGroup = new ButtonGroup( );
}
