/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomDefaults;
import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.dicom.FileInfo;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * ExtendedRegionPropertiesTableFrame
 * @author slotboom
 */
public class ExtendedRegionPropertiesTableFrame extends JFrame implements ActionListener
{

    /**
     * Default constructor.
     * @param parent
     * @param dicomSliceCanvasPolydraw
     * @param title
     */
    public ExtendedRegionPropertiesTableFrame( JFrame parent, 
                                               DicomSliceCanvasPolyDraw dicomSliceCanvasPolydraw,
                                               String title )
    {

        setTitle( title );

        m_parent = parent;
        m_dicomSliceCanvasPolydraw = dicomSliceCanvasPolydraw;
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        createExtendedROIPropertiesPanel( );
        m_collectionPanel.add( m_ROIPropertiesScrollPane, BorderLayout.CENTER );
        createApplyOrCancelPanel( );
        m_collectionPanel.add( m_cancelApplyPanel, BorderLayout.SOUTH );

        setContentPane( m_collectionPanel );
        
         // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        
        setVisible( true );
    }

    /**
     * Creates the panel on which the properties of the regions are drawn.
     */
    private void createExtendedROIPropertiesPanel( )
    {
        m_extendedROIPropertiesTable = new ExtendedRegionPropertiesTable( this, m_dicomSliceCanvasPolydraw, false );
        // m_extendedROIPropertiesTable.addActionListener( this );
        // JList does not provide a scroll list automatically so we have to
        // add it ourselves.
        m_ROIPropertiesScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS , JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        m_ROIPropertiesScrollPane.getViewport( ).setView( m_extendedROIPropertiesTable );
        m_ROIPropertiesScrollPane.setPreferredSize( new Dimension( 1024, 128 ) );
        //m_ROIPropertiesScrollPane.setMaximumSize( new Dimension( 512, 128 ) );
        m_ROIPropertiesScrollPane.setMinimumSize( new Dimension( 1024, 128 ) );

        m_extendedROIPropertiesTable.addActionListener( m_dicomSliceCanvasPolydraw );
    }

    /**
     * Creates the cancel/apply panel.
     */
    private void createApplyOrCancelPanel( )
    {

        m_createReportFileButton = new JButton( "Create report file .." );
        m_createReportFileButton.addActionListener( this );
        m_createReportFileButton.setToolTipText( "Ask the user to specify a file name to store the ROI information into" );

        m_okButton = new JButton( "OK" );
        m_okButton.addActionListener( this );
        m_okButton.setToolTipText( "Closes the frame and disposes" );

        m_cancelApplyPanel = new JPanel( );

        m_cancelApplyPanel.setLayout( new BoxLayout( m_cancelApplyPanel, BoxLayout.X_AXIS ) );
        m_cancelApplyPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_cancelApplyPanel.add( Box.createGlue( ) );
        m_cancelApplyPanel.add( m_createReportFileButton );
        m_cancelApplyPanel.add( Box.createGlue( ) );
        m_cancelApplyPanel.add( m_okButton );
        m_cancelApplyPanel.add( Box.createGlue( ) );
    }

    /**
     * Is called whenever a registered component raises an action event.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        Object sender = e.getSource( );

        if ( sender == m_okButton )
        {
            dispose( );
        }

        if ( sender == m_createReportFileButton )
        {
            saveInformationInReportFile( );
        }
        
    }

    /**
     * Creates a report file with all statistical information of all regions. 
     */
    private void saveInformationInReportFile( )
    {

        String path = DicomDefaults.getInstance( ).getProperty( DicomDefaults.DICOM_MOUNT_PATH );

        FileInfo finfo = m_dicomSliceCanvasPolydraw.getDicomFileInfo( );

        String fileDefaultName = finfo.getPatientName( ).trim( ) + "roi-statististics";

        String outputPathAndFile = getFilePathAndName( path,
                                                       "statistics-of-roi",
                                                       fileDefaultName +
                                                       ".txt", "Save" );

        if ( outputPathAndFile != null )
        {

            try
            {
                // try to create create and open a RandomAccessfile
                FileWriter writer = new FileWriter( new File( outputPathAndFile  ) );

                writer.write( "Region no." + "\t" );
                writer.write( "Image no. " + "\t");
                writer.write( "ROI name  " + "\t");
                writer.write( "Pixels    " + "\t");
                writer.write( "Volume    " + "\t");

                writer.write( "Mean      " + "\t");
                writer.write( "St. dev.  " + "\t");

                writer.write( "Variance  " + "\t" );
                writer.write( "Var[var]  " + "\t" );

                writer.write( "Skewness  " + "\t");
                writer.write( "Var[skew.]" + "\t");

                writer.write( "Kurtosis  " + "\t");
                writer.write( "Var[kurt.]" + "\n" );

                for ( int j = 0; j < m_extendedROIPropertiesTable.getRowCount( ); j++ )
                {
                    for ( int i = 0; i <  m_extendedROIPropertiesTable.getColumnCount( ); i++ )
                    {
                        String outputString = new String( "" + m_extendedROIPropertiesTable.getModel( ).getValueAt( j, i ) );
                        writer.write( outputString );
                        writer.write( "\t" );
                    }
                    writer.write( "\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                ioe.printStackTrace( );
            }
        }
    }

    /**
     * Returns the filename and directory selected by the user.
     * @param path
     * @param subDirString
     * @param fileDefaultName
     * @param buttonText
     * @return 
     */
    protected String getFilePathAndName( String path, 
                                         String subDirString, 
                                         String fileDefaultName, 
                                         String buttonText )
    {
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        String outputPath = new String( path + System.getProperty( "file.separator" ) + subDirString );

        File f = new File( outputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        fileChooser.setSelectedFile( new File( fileDefaultName ) );

        result = fileChooser.showDialog( this, buttonText );


        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }


    // Attributes
    private final JPanel                    m_collectionPanel;
    private ExtendedRegionPropertiesTable   m_extendedROIPropertiesTable;
    private final DicomSliceCanvasPolyDraw  m_dicomSliceCanvasPolydraw;
    private JScrollPane                     m_ROIPropertiesScrollPane;
    private JButton                         m_createReportFileButton;
    private JButton                         m_okButton;
    private JPanel                          m_cancelApplyPanel;
    private final JFrame                    m_parent;

}