/*
 * File: $Id: ImageRegionProperties.java,v 1.1.1.1 2008/10/10 09:38:47 hans Exp $
 * History
 * Date: 28.10.2008 Hans Slotboom  Changes:
 *
 * Copyright (C) 2008 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imageprocessing;

/**
 * Container class that stores allowing the setting and the reading of
 * image region properties.
 * @author  hans
 */
public class ImageRegionProperties 
{
    
    /** 
     * Creates a new instance of ImageRegionProperties:
     *  
     * Assumed are pixelWidth, pixelHeight and sliceThickness in mm.
     * 
     * @param pixelWidth
     * @param pixelHeight
     * @param sliceThickness
     */
    public ImageRegionProperties( double pixelWidth, double pixelHeight, double sliceThickness )
    {
        m_volumePerPixel = pixelWidth * pixelHeight * sliceThickness / 1000.0f; // return volume in cm^3
    }
    
    /**
     * Sets the name of the ROI to which this belongs.
     * @param name
     */
    public void setImageRegionName( String name )
    {
        m_nameOfImageRegion = name;
    }

    /**
     * Returns the name of the ROI to which this belongs.
     * @return 
     */
    public String getImageRegionName( )
    {
        return m_nameOfImageRegion;
    }

    /**
     * Sets the  number of pixels within ROI. 
     * @param number
     */
    public void setNumberOfPixels( int number )
    {
        m_numberOfPixelsWithinRegion = number;
    }

    /**
     * Returns the  number of pixels within ROI.
     * @return 
     */
    public int getNumberOfPixels( )
    {
        return m_numberOfPixelsWithinRegion;
    }

    /**
     * Sets the volume of the ROI.
     * @param volume
     */
    public void setImageRegionVolume( double volume )
    {
        m_imageSegmentVolume = volume;
    }

    /**
     * Sets the image position of the slice in which the ROI is positioned. 
     * @param position
     */
    public void setImagePosition( float position )
    {
        m_imagePosition = position;
    }
    
    
    /**
     * Returns the volume of the ROI.
     * @return 
     */
    public double getImageRegionVolume( )
    {
        return m_imageSegmentVolume;
    }    
    
    /**
     * Sets the mean pixel value of the ROI.
     * @param mean
     */
    public void setMeanPixelValue( float mean )
    {
        m_meanOfPixelValues = mean;
    }

    /**
     * Returns the mean pixel value of the ROI. 
     * @return 
     */
    public double getMeanPixelValue( )
    {
        return m_meanOfPixelValues;
    }    
    
    /**
     * Sets the standard deviation pixel value of the ROI. 
     * @param stdev
     */
    public void setStandardDeviationOfPixelValues( float stdev )
    {
        m_standardDeviationOfPixelValues = stdev;
    }

    /**
     * Returns the standard deviation pixel value of the ROI. 
     * @return 
     */
    public double getStandardDeviationOfPixelValues( )
    {
        return m_standardDeviationOfPixelValues;
    }
    
    /**
     * Returns the image position of the slice in which the ROI is positioned. 
     * @return 
     */
    public double getImagePosition( )
    {
        return m_imagePosition;
    }
    
    /**
     * Returns the volume enclosed by the polygon
     * @return 
     */
    public double getVolumeEnclosedByRegio( )
    {
        return ( m_volumePerPixel * m_numberOfPixelsWithinRegion );
    }
    
    // Attributes
    private String  m_nameOfImageRegion;
    private int     m_numberOfPixelsWithinRegion;
    private double  m_imageSegmentVolume;
    
    private double   m_meanOfPixelValues;
    private double   m_standardDeviationOfPixelValues;
    private double   m_imagePosition;
    
    private final double   m_volumePerPixel;   //in cm^3
}

