/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.lib.StringDefinitionControl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.TableColumn;

/**
 * A utility class that shows the measured distance values of a 
 * DicomSliceCanvasPolydraw object. 
 * @author slotboom
 */
public class MeasuredDistancesTable extends JTable implements 
                                            ActionListener, MouseListener
{
    /**
     * Constructor that creates a MeasuredDistancesTable object.
     */
    public MeasuredDistancesTable( )
    {
        super( );

        setModel( new MeasuredDistancesTableModel( ) );

        setAutoCreateColumnsFromModel( false );
       
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );

        addPopupMenu( );

        // Disable auto resizing
        setAutoResizeMode( JTable.AUTO_RESIZE_OFF );

        // Set the first visible column to 55 pixels wide
        int vColIndex = 0;
        TableColumn col1 = getColumnModel( ).getColumn( vColIndex );
        int width = 55;
        col1.setPreferredWidth( width );


        // Set the second visible column to 100 pixels wide
        vColIndex = 1;
        TableColumn col2 = getColumnModel( ).getColumn( vColIndex );
        width = 80;
        col2.setPreferredWidth( width );

        // Set the second visible column to 100 pixels wide
        vColIndex = 2;
        TableColumn col3 = getColumnModel( ).getColumn( vColIndex );
        width = 110;
        col3.setPreferredWidth( width );

        // Set the second visible column to 100 pixels wide
        vColIndex = 3;
        TableColumn col4 = getColumnModel( ).getColumn( vColIndex );
        width = 80;
        col4.setPreferredWidth( width );

        // Set the second visible column to 100 pixels wide
        vColIndex = 4;
        TableColumn col5 = getColumnModel( ).getColumn( vColIndex );
        width = 80;
        col5.setPreferredWidth( width );

        // Set the second visible column to 100 pixels wide
        vColIndex = 5;
        TableColumn col6 = getColumnModel( ).getColumn( vColIndex );
        width = 180;
        col6.setPreferredWidth( width );


        // Set the second visible column to 100 pixels wide
        vColIndex = 6;
        TableColumn col7 = getColumnModel( ).getColumn( vColIndex );
        width = 80;
        col7.setPreferredWidth( width );

        // Set the second visible column to 100 pixels wide
        vColIndex = 7;
        TableColumn col8 = getColumnModel( ).getColumn( vColIndex );
        width = 80;
        col8.setPreferredWidth( width );

        // Set the second visible column to 100 pixels wide
        vColIndex = 8;
        TableColumn col9 = getColumnModel( ).getColumn( vColIndex );
        width = 180;
        col9.setPreferredWidth( width );

        // Set the 10th visible column to 100 pixels wide
        vColIndex = 9;
        TableColumn col10 = getColumnModel( ).getColumn( vColIndex );
        width = 100;
        col10.setPreferredWidth( width );

        // Listen to mouse events and react to them!
        addMouseListener( this );

    }

    /**
     * Does the rest of the object construction work.
     */
    private void addPopupMenu( )
    {
        m_popup = new JPopupMenu( );

        m_renameDistanceName = new JMenuItem( "Rename " );
        m_renameDistanceName.addActionListener( this );
        m_renameDistanceName.setEnabled( true );

        m_popup.add( m_renameDistanceName );

        add( m_popup );
    }

    /**
     * Notifies all registered components interested in selection changes in the table
     * @param eventType
     */
    public void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            if ( eventType == OverlayPropertiesTableSelectionListener.ROW_SELECTION_CHANGED )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, OverlayPropertiesTableSelectionListener.ROW_SELECTION_CHANGED, "" ) );
            }
            if ( eventType == OverlayPropertiesTableSelectionListener.COLUMN_SELECTION_CHANGED )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, OverlayPropertiesTableSelectionListener.COLUMN_SELECTION_CHANGED, "" ) );
            }
        }
    }

    /**
     * Notifies all other registered components interested in selection
     * changes in the table.
     * @param eventType
     */
    public void notifyOtherActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
           ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }


    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens with the object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }


    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        Object sender = e.getSource( );

        if ( sender == m_renameDistanceName )
        {
            notifyOtherActionListeners( DISTANCE_RENAMING_EVENT );
            getModel( );
            
            String name = (String)((MeasuredDistancesTableModel)getModel( ) ).getValueAt( getSelectedRow( ), 2 );

            m_renameControl = new StringDefinitionControl( "Rename", "New name", name, null );
            m_renameControl.addActionListener( this );
        }

        if ( sender == m_renameControl )
        {
            if ( e.getID( ) == StringDefinitionControl.ACCEPT_EVENT )
            {
                m_renamedString = m_renameControl.getRenamedString( );
                ((MeasuredDistancesTableModel)getModel( ) ).setValueAt( m_renameControl.getRenamedString( ), getSelectedRow( ), 2 );
                notifyActionListeners( RENAMING_EVENT );
            }
        }

    }


    /**
     * Returns the renamed string
     * @return
     */
    public String getRenamedString( )
    {
        return m_renamedString;
    }

    /**
     * Is called whenever a registered component raises mouseClicked event.
     * @param e
     */
    @Override
    public void mouseClicked( MouseEvent e )
    {

    }

    /**
     * The user pressed the table, this may be a change in the selected
     * row. Sent a notification to the listeners to this table
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e )
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }

        notifySelectionChange( );
    }
    
    /**
     * The user released the mouse button.
     * @param e 
     */
    @Override
    public void mouseReleased( MouseEvent e )
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }

        notifySelectionChange( );
    }
    
    /**
     * The user entered with the mouse pointer.
     * @param e 
     */
    @Override
    public void mouseEntered( MouseEvent e )
    {
        // not implemented
    }
    /**
     * The user exited with the mouse pointer.
     * @param e 
     */
    @Override
    public void mouseExited( MouseEvent e )
    {
        // not implemented
    }

    /**
     * Notifies listeners that a change in table selection was performed.
     */
    private void notifySelectionChange( )
    {
         notifyActionListeners( TABLE_SELECTED_EVENT );
    }

    // Attributes of the class

    private static final long serialVersionUID = 99822446327598273L;

    // Listeners to action events in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<>( );

    public final static int TABLE_SELECTED_EVENT = 0;
    public final static int RENAMING_EVENT = 1;

    private JPopupMenu m_popup;

    private JMenu m_operationsMenu;

    private JMenuItem m_renameDistanceName;
    
    // possible action events raised by this
    public final static int DISTANCE_RENAMING_EVENT = 10;

    public StringDefinitionControl m_renameControl;
    
    private String m_renamedString;
}