/*
 * File: $Id: MinimumMask.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 20.07.2004 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imageprocessing;

/**
 * Every pixel of and image is set to maskedValue if 
 * smaller than minimumValue.
 * @author  hans
 */
public class MinimumMask 
{  
    /** 
     * Creates a new instance of MinimumMask. 
     * @param minimumValue
     * @param maskedValue
     */
    public MinimumMask( int minimumValue, int maskedValue ) 
    {
        m_minValue = minimumValue;
        m_maskValue = maskedValue;
    }
    
    /**
     * Performs the masking of the int[] image. 
     * @param maskedImage
     * @return 
     */
    public int[] applyMinimumMask( int[] maskedImage )
    {   
        for ( int i = 0; i < maskedImage.length; i++ )
        {
            if ( maskedImage[i] < m_minValue )
            {
                maskedImage[i] = m_maskValue;
            }
        }
        return maskedImage;
    }
    
    /**
     * Performs the masking of the int[] image. 
     * @param maskedImage
     * @return 
     */
    public float[] applyMinimumMask( float[] maskedImage )
    {   
        for ( int i = 0; i < maskedImage.length; i++ )
        {
            if ( maskedImage[i] < m_minValue )
            {
                maskedImage[i] = m_maskValue;
            }
        }
        return maskedImage;
    }
    
    /**
     * Performs the masking of the int[] image. 
     * @param inputMask
     * @param image
     * @return 
     */
    public int[] applyMinimumMask( int[] inputMask, int[] image )
    {   
        for ( int i = 0; i < image.length; i++ )
        {
            if ( image[i] < m_minValue )
            {
                inputMask[i] = m_maskValue;
            }
        }
        return inputMask;
    }
    
    /**
     * Performs the masking of the float[] image .
     * @param inputMask
     * @param image
     * @return 
     */
    public int[] applyMinimumMask( int[] inputMask, float[] image )
    {
        
        for ( int i = 0; i < image.length; i++ )
        {
            if ( image[i] < m_minValue )
            {
                inputMask[i] = m_maskValue;
            }
        }
        return inputMask;
    }
    
    
    // attributes
    int m_minValue;
    int m_maskValue;
}
