/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.imagegraphicobjects.RegioPolygon;
import java.awt.Polygon;
import java.awt.Rectangle;

/**
 * This class computes the first four moments about the mean and
 * their corresponding variances.
 * @author slotboom
 */
public class MomentsAboutTheMeanOfRegionCalculator
{

    /**
     * 
     * Constructor
     * 
     * @param p
     * @param image
     * @param imageWidth
     * @param imageHeight
     * @param zoom
     */
    public MomentsAboutTheMeanOfRegionCalculator( RegioPolygon p, int[] image, int imageWidth, int imageHeight, float zoom, float nordicScalingFactor )
    {
        m_polygon       = p;
        m_zoom          = zoom;
        m_image         = image;
        m_imageWidth    = imageWidth;
        m_imageHeight   = imageHeight;
        m_nordicScalingFactor = nordicScalingFactor;
    }

    /**
     * Does the actual computation of the moments about the mean.
     * @return 
     */
    public MomentsAboutTheMean computeStatistics( )
    {
        
        m_roiSize = 0;
        MomentsAboutTheMean momentAboutTheMean;
        
        double m_minimumValue = Double.MAX_VALUE;
        double m_maximumValue = -Double.MAX_VALUE;

        if ( m_polygon != null )
        {
            float zoom = 1f / m_zoom;
            Polygon roi = (Polygon)m_polygon.scaled( zoom );

            if ( roi != null )
            {
                Rectangle bb = roi.getBounds( );
                if ( bb.x < 0 )
                {
                    bb.x = 0;
                }
                if ( bb.x + bb.width >= m_imageWidth )
                {
                    bb.width = m_imageWidth - bb.x - 1;
                }
                if ( bb.y < 0 )
                {
                    bb.y = 0;
                }

                if ( bb.y + bb.height >= m_imageHeight )
                {
                    bb.height = m_imageHeight - bb.y - 1;
                }

                int imageSize = m_imageWidth * m_imageHeight;

                // Calculate the average of the ROI
                for ( int y = bb.y; y <= bb.y + bb.height; y++ )
                {
                    int yoffset = y * m_imageWidth;
                    for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                    {
                        if ( roi.contains( x, y ) )
                        {
                            m_roiSize++;
                        }
                    }
                }

                float[] signalOfRegion = new float[m_roiSize];
                int cntr = 0;

                for ( int y = bb.y; y <= bb.y + bb.height; y++ )
                {
                    int yoffset = y * m_imageWidth;
                    for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                    {
                        if ( roi.contains( x, y ) )
                        {
                            signalOfRegion[cntr] = m_image[ x + m_imageWidth * y ] * m_nordicScalingFactor;
                            cntr++;
                        }
                    }
                }

                // 1. compute first moment: sample mean
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    m_average += signalOfRegion[i];
                }

                m_average = m_average / signalOfRegion.length;

                // 2. compute sample variance of the
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    m_variance += ( signalOfRegion[i] - m_average ) * ( signalOfRegion[i] - m_average );
                }

                m_variance = m_variance / ( signalOfRegion.length - 1 );

                // 3. variance of sample variance
                double mhu_2 = 0.0f;
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    mhu_2 += signalOfRegion[i] * signalOfRegion[i];
                }
                mhu_2 = mhu_2 / signalOfRegion.length;

                double mhu_4 = 0.0f;
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    mhu_4 += signalOfRegion[i] * signalOfRegion[i] * signalOfRegion[i] * signalOfRegion[i];
                }
                mhu_4 = mhu_4 / signalOfRegion.length;

                double M = (double)signalOfRegion.length;
                double n = (double)( signalOfRegion.length - 1 );

                double a1 = M * ( M - n ) * ( M * n - M - n - 1 ) / ( ( M - 3 ) * ( M - 2 ) * ( M - 1 ) * ( n - 1 ) * n );
                double a2 = M * ( M - n ) * ( M * M * n - 3 * n - 3 * M * M + 6 * M - 3 ) / ( ( M - 3 ) * ( M - 2 ) * ( M - 1 ) * ( n - 1 ) * n );

                m_varianceOfVariance = Math.log( a1 * mhu_4 + a2 * mhu_2 );

                // 4. Sample skewness
                double numerator = 0.0;
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    numerator += ( signalOfRegion[i] - m_average ) * ( signalOfRegion[i] - m_average ) * ( signalOfRegion[i] - m_average );
                }
                double dominator = 0.0;
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    dominator += ( signalOfRegion[i] - m_average ) * ( signalOfRegion[i] - m_average );
                }
                double factor = Math.sqrt( M * ( M - 1 ) )/( M - 2 );

                m_skewness = factor * Math.sqrt( M ) * dominator / ( dominator * Math.sqrt( dominator ) );

                // 5. variance of sample skewness (assume Gaussian distribution)
                m_varianceOfSkewness = 6 * M / ( ( M + 1 ) * ( M + 3 ) );

                // 6. Sample kurtosis
                double numerator_kur = 0.0;
                for ( int i = 0; i < signalOfRegion.length; i++ )
                {
                    double factor_kur = ( signalOfRegion[i] - m_average );
                    numerator_kur += factor_kur * factor_kur * factor_kur * factor_kur;
                }

                double G = ( M + 1 ) * M / ( ( M - 1 ) * ( M - 2 ) * ( M - 3 ) );
                double H = ( M - 1 ) * ( M - 1 ) / ( ( M - 2 ) * ( M - 3 ) );

                m_kurtosis = ( G * numerator_kur/ ( m_variance * m_variance ) ) - 3 * H;

                // 7. variance of sample kurtosis
                m_varianceOfKurtosis = 24 * M * ( M - 2 ) * ( M - 3 ) / ( ( M + 1 ) * ( M + 1 ) * (M + 3) * ( M + 5 ) );

                // 8. compute minimum and maximum
                
                
                for ( int kk = 0; kk < signalOfRegion.length; kk++ )
                {
                    if ( signalOfRegion[kk] > m_maximumValue )
                    {
                        m_maximumValue = signalOfRegion[kk];
                    }
                    
                    if ( signalOfRegion[kk] < m_minimumValue )
                    {
                        m_minimumValue = signalOfRegion[kk];
                    }   
                }
                
//                System.out.println( "m_average               : " + m_average );
//                System.out.println( "m_variance              : " + m_variance );
//                System.out.println( "m_stdev                 : " + Math.sqrt( m_variance ) );
//                System.out.println( "m_varianceOfVariance    : " + m_varianceOfVariance );
//                System.out.println( "m_skewness              : " + m_skewness );
//                System.out.println( "m_varianceOfSkewness    : " + m_varianceOfSkewness );
//                System.out.println( "m_kurtosis              : " + m_kurtosis );
//                System.out.println( "m_varianceOfKurtosis    : " + m_varianceOfKurtosis );
            }

            momentAboutTheMean = new MomentsAboutTheMean( );

            momentAboutTheMean.setMean( m_average );
            momentAboutTheMean.setStDev( Math.sqrt( m_variance ) );
            momentAboutTheMean.setVariance( m_variance );
            momentAboutTheMean.setVarianceOfVariance( m_varianceOfVariance );
            momentAboutTheMean.setSkewness( m_skewness );
            momentAboutTheMean.setVarianceOfSkewness( m_varianceOfSkewness );
            momentAboutTheMean.setKurtosis( m_kurtosis );
            momentAboutTheMean.setVarianceOfKurtosis( m_varianceOfKurtosis );
            momentAboutTheMean.setMinimum( m_minimumValue );
            momentAboutTheMean.setMaximum( m_maximumValue );
            
        }
        else
        {
            momentAboutTheMean = new MomentsAboutTheMean( );
        }

        return momentAboutTheMean;
        
    }


    // Attributes
    private final RegioPolygon m_polygon;
    private int                m_roiSize;
    private final int[]        m_image;
    private final int          m_imageWidth;
    private final int          m_imageHeight;
    private final float        m_zoom;
    private final float        m_nordicScalingFactor;

    // Attributes than contain the numeric values of the computation
    private double        m_average              = 0.0;
    private double        m_variance             = 0.0;
    private double        m_varianceOfVariance   = 0.0;
    private double        m_skewness             = 0.0;
    private double        m_varianceOfSkewness   = 0.0;
    private double        m_kurtosis             = 0.0;
    private double        m_varianceOfKurtosis   = 0.0;
    private final double  m_minimumValue         = 0.0;
    private final double  m_maximumValue         = 0.0;
    
}
