/*
 * File: $Id: RegioContoursByMorphologicFiltering.java,v 1.1.1.1 2009/06/30 09:38:48 hans Exp $
 * History
 * Date: 30.06.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.imageprocessing;

/**
 *
 * @author slotboom
 */
public class RegioContoursByMorphologicFiltering
{

    /**
     * Constructor with integer image (containing zeroes / non zero elements.
     * @param intImage
     * @param width
     * @param height
     */
    public RegioContoursByMorphologicFiltering( int[] intImage, int width, int height  )
    {
        m_intImage = intImage;
        m_imageWidth = width;
        m_imageHeight = height;
    }
    
    /**
     * Constructor with integer image (containing zeroes / non zero elements.
     * @param boolImage
     * @param width
     * @param height
     */
    public RegioContoursByMorphologicFiltering( boolean[][] boolImage, 
                                                int width, int height )
    {
        m_booleanImage = boolImage;
        m_imageWidth = width;
        m_imageHeight = height;
    }
    
    /**
     * Returns the int image only containing contours.
     * @return
     */
    public int[] getContoursAsIntImage( )
    {
        int[] imageOut;
        imageOut = new int[ m_intImage.length ];

        Morphologic1DKernalOperator horiOper = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.HORIZONTAL_OPERATOR );
        Morphologic1DKernalOperator vertiOper = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.VERTICAL_OPERATOR );

        imageOut = vertiOper.dilate( m_imageWidth, m_imageHeight, m_intImage );
        imageOut = horiOper.dilate( m_imageWidth, m_imageHeight, imageOut );

        imageOut = horiOper.performExorOperation( m_intImage, imageOut );

        return imageOut;
    }


    /**
     * Returns the int image only containing contours
     * @return
     */
    public boolean[][] getContoursAsBooleanImage( )
    {
        boolean[][] imageOut = new boolean[m_imageWidth][m_imageHeight];

        Morphologic1DKernalOperator horiOper = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.HORIZONTAL_OPERATOR );
        Morphologic1DKernalOperator vertiOper = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.VERTICAL_OPERATOR );

        imageOut = vertiOper.dilate( m_imageWidth, m_imageHeight, m_booleanImage );
        imageOut = horiOper.dilate( m_imageWidth, m_imageHeight, imageOut );

        imageOut = horiOper.performExorOperation( m_booleanImage, imageOut, m_imageWidth, m_imageHeight );

        return imageOut;
    }

    // Attribrutes
    private int[] m_intImage;
    private boolean[][] m_booleanImage;

    private int m_imageWidth;
    private int m_imageHeight;
}
