/*
 * File: $Id: BrainVoyagerFBRFileParserAndLoader.java,v 1.2 2006/10/25 08:27:01 hans Exp $
 * History
 * Date: 03.06.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003-2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.awt.Color;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 *
 * The code in this class allows the parsing of a .voi file of Brainvoyager
 * The caller can get access to the 3D overlay array of boolean type
 *
 * @author slotboom
 */
public class BrainVoyagerFBRFileParserAndLoader
{
    /**
     * Constructor of BrainVoyagerVOIFileParserAndLoader.
     * @param fbrFileReference
     * @param numberOfSlices
     * @param imageWidth
     * @param imageHeight
     */
    public BrainVoyagerFBRFileParserAndLoader( String fbrFileReference, int numberOfSlices, int imageWidth, int imageHeight )
    {
        m_fbrFileReference = fbrFileReference;
        m_brainVoyagerOverlayPositions = new boolean[numberOfSlices][imageWidth][imageHeight];
        m_brainOverlayColors = new Color[numberOfSlices][imageWidth][imageHeight];
    }



    /**
     * Returns true if the string can be parsed as Integer, else false
     * @param str
     * @return
     */
    public boolean isParsableToInt( String str )
    {
        try
        {
            Integer.parseInt( str );
            return true;
        }
        catch( NumberFormatException nfe )
        {
            return false;
        }
    }

     /**
     * Returns true if the string can be parsed as Integer, else false
     * @param str
     * @return
     */
    public boolean isParsableToFloat( String str )
    {
        try
        {
            Float.parseFloat( str );
            return true;
        }
        catch( NumberFormatException nfe )
        {
            return false;
        }
    }



    /**
     * Loads the voi file
     * @return the contence of the file as a 3D array of boolean-type
     */
    public boolean[][][] loadBrainVoyagerFBRFile( )
    {
        try
        {
            m_fbrFile = new RandomAccessFile( m_fbrFileReference, "r" );

            String str = m_fbrFile.readLine( );
            //str = m_fbrFile.readLine( );

            // check whether the file is a valid .voi-file
            // if the second line contains the string  "FileVersion:"
            // we assume the file to be valid
            if ( str.contains( "FileVersion:" ) == true )
            {
                // read and ignore following lines
                for ( int i=1; i < 17; i++ )
                {
                    str = m_fbrFile.readLine( );
                    // System.out.println( "Skipping > " + str );
                }

                m_brainVoyagerOverlayPositions = parseRestOfFile( );
            }
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
            System.out.println( "IOException while reading " + m_fbrFileReference );
            return null;
        }

        return m_brainVoyagerOverlayPositions;

    }


    /**
     * Does the factual parsing of the file.
     * @return
     * @throws java.io.IOException
     */
    public boolean[][][] parseRestOfFile( ) throws IOException
    {

        String str = m_fbrFile.readLine( );
        int cntr = 0;
        
        int x = 0, y = 0, z = 0;
        float fx = 0f, fy = 0f, fz = 0f;

        String dummy1, dummy2, dummy3, dummy4, dummy5, dummy6;

        while ( str != null )
        {

            // initialize dummies
            dummy1 = "";
            dummy2 = "";
            dummy3 = "";

            dummy4 = "";
            dummy5 = "";
            dummy6 = "";

            if ( str.length( ) > 0 )
            {
                if ( str.lastIndexOf( " " ) > 0 )
                {
                    dummy3 = str.substring( str.lastIndexOf( " " ) + 1, str.length( ) );
                    dummy3 = dummy3.trim( );

                    // make the string shorter: remove last part
                    str = str.substring( 0, str.lastIndexOf( " " ) );
                }

                if ( str.lastIndexOf( " " ) > 0 )
                {

                    dummy2 = str.substring( str.lastIndexOf( " " ) + 1, str.length( ) );
                    dummy2.trim( );

                    // make the string shorter: remove last part
                    str = str.substring( 0, str.lastIndexOf( " " ) );
                }

                if ( str.lastIndexOf( " " ) > 0 )
                {
                    dummy1 = str.substring( str.lastIndexOf( " " ) + 1, str.length( ) );
                    dummy1 = dummy1.trim( );

                    // make the string shorter: remove last part
                    str = str.substring( 0, str.lastIndexOf( " " ) );
                    str = str.trim( );
                }

                if ( str.lastIndexOf( " " ) > 0 )
                {
                    dummy6 = str.substring( str.lastIndexOf( " " ) + 1, str.length( ) );
                    dummy6 = dummy6.trim( );

                    // make the string shorter: remove last part
                    str = str.substring( 0, str.lastIndexOf( " " ) - 1 );
                    str = str.trim( );
                }

                if ( str.lastIndexOf( " " ) > 0 )
                {
                    dummy5 = str.substring( str.lastIndexOf( " " ) + 1, str.length( ) );
                    dummy5 = dummy5.trim( );

                    // make the string shorter: remove last part
                    str = str.substring( 0, str.lastIndexOf( " " ) - 1 );
                    str = str.trim( );
               
                    dummy4 = str.substring( 0, str.length( ) );
                    dummy4 = dummy4.trim( );

                }

                // System.out.println( "dummy 1  " + dummy1 + " dummy 2 " + dummy2 + " dummy 3 " + dummy3 );
                // System.out.println( "dummy 4  " + dummy4 + " dummy 5 " + dummy5 + " dummy 6 " + dummy6 );

                boolean cond1 = false;
                boolean cond2 = false;
                Color   color = new Color( 0, 0, 0 );

                // only if all three strings have non zero length we can try to convert them to integer ...
                if ( ( isParsableToInt( dummy1 ) == true ) && ( isParsableToInt( dummy2 ) == true ) && ( isParsableToInt( dummy3 ) == true ) )
                {
                    x = new Integer( dummy1 ).intValue( );
                    y = new Integer( dummy2 ).intValue( );
                    z = new Integer( dummy3 ).intValue( );
                   
                    // define an opaque color for the time being ...
                    color = new Color( x, y, z );

                    cond1 = true;
                }

                // only if all three strings have non zero length we can try to convert them to integer ...
                if ( ( isParsableToFloat( dummy4 ) == true ) && ( isParsableToFloat( dummy5 ) == true ) && ( isParsableToFloat( dummy6 ) == true ) )
                {
                    fx = new Float( dummy4 ).floatValue( );
                    fy = new Float( dummy5 ).floatValue( );
                    fz = new Float( dummy6 ).floatValue( );
                    
                    // System.out.println( "pixel " + (cntr++) + "  x = " + fx +" y =" + fy + "z = " + fz );
                    
                    cond2 = true;
                }

                // both positoin and color could be defined.
                if ( ( cond1 == true ) && ( cond2 == true ) )
                {
                    m_brainVoyagerOverlayPositions[Math.round(fz)][Math.round(fx)][Math.round(fy)] = true;
                    m_brainOverlayColors[Math.round(fz)][Math.round(fx)][Math.round(fy)] = color;
                }

            }

            // read the next line from the voi file..
            str = m_fbrFile.readLine( );
        }

        // return the values contained in the file..
        return m_brainVoyagerOverlayPositions;
    }



    /**
     * Returns the overlay color fibers.
     * @return
     */
    public Color[][][] getOverlayColors( )
    {
        return m_brainOverlayColors;
    }

    // Attributes
    private final String  m_fbrFileReference;
    private RandomAccessFile  m_fbrFile;

    private boolean[][][] m_brainVoyagerOverlayPositions;
    private final Color[][][] m_brainOverlayColors;
}
