/*
 * File: $Id: ImageUtils.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 30. November 2002, 16:32  Authors: Martin Buerki
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;
 
import java.awt.Dimension;

import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.util.zip.GZIPInputStream;

import org.spectrim.dicom.DicomDecoder;
import org.spectrim.dicom.FileInfo;


public class ImageUtils 
{
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static Dimension getImageDimensionsNUMARIS( String fileName, ImageFormatInfo p ) 
    {
        Dimension d = null;

        byte[] hdr = readHeaderNUMARIS( fileName, p );
        if ( hdr == null ) return new Dimension( 0, 0 );

        if ( p.byteSwap )
            d = new Dimension( Convert.byte2intl( hdr, 4996 ), Convert.byte2intl( hdr, 4994 ) );
        else
            d = new Dimension( Convert.byte2intb( hdr, 4996 ), Convert.byte2intb( hdr, 4994 ) );

        return d;
    }

    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static Dimension getImageDimensionsACR( String fileName, ImageFormatInfo p ) 
    {
        System.out.println( "Warning: getImageDimensions() not implemented. Taking the defined values." );
        return new Dimension( p.getWidth( ), p.getHeight( ) );
    }

    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static Dimension getImageDimensionsDICOM( String fileName, ImageFormatInfo p ) 
    {

        int n = fileName.lastIndexOf( System.getProperty( "file.separator" ) );
        String dirName;
        String fName;
	DicomDecoder dd = null;
        
        if ( n == -1 )
        {
            dirName = "." + System.getProperty( "file.separator" );
            fName = fileName;
        }
        else
        {
            dirName = fileName.substring( 0, n + 1 );
            fName = fileName.substring( n + 1 );
        }

        try
        {
            dd = new DicomDecoder( dirName, fName );
            FileInfo fi = dd.getFileInfo( );
            
            return new Dimension( fi.getImageWidth( ), fi.getImageHeight( ) );
        }
        catch ( IOException e )
        {
            dd.closeInputStream( );
            return null;
        }
    }
    
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static Dimension getImageDimensions( String fileName, ImageFormatInfo p ) 
    {

        Globals globs = Globals.getInstance( );

        // rows are read into d.width
        // cols are read into d.height

        Dimension d = null;
        int fileFormat = p.getFormat( );

        switch ( fileFormat )
        {
            case Globals.NUMARIS:
                                    d = getImageDimensionsNUMARIS( fileName, p );
                                    break;
            case Globals.ACR_NEMA:
                                    d = getImageDimensionsACR( fileName, p );
                                    break;
            case Globals.DICOM:
                                    d = getImageDimensionsDICOM( fileName, p );
                                    break;
            case Globals.CUSTOM:
                                    d = new Dimension( p.getWidth( ), p.getHeight( ) );
                                    break;
            default:
                                    System.out.println( "Error in ImageUtils.getImageDimensions(): unknown file format" );
        }

        return d;
    }
    
    /**
     * 
     * @param fileName
     * @param data
     * @param offset
     * @param p
     * @return 
     */
    public static boolean loadImageNUMARIS( String fileName, int[] data, int offset, 
                                            ImageFormatInfo p )
    {
        GZIPInputStream zip;
        FileInputStream fIn;
        byte[] hdr;

        Dimension d = getImageDimensionsNUMARIS( fileName, p );

        int bDataSize = p.bytesPerPixel * d.width * d.height;
        int imageSize = d.width * d.height;
        byte[] bData = new byte[bDataSize];

        try
        {
            int hdrSize = p.headerSize;
            hdr = new byte[hdrSize];
            if ( FileUtils.fileIsZipped( fileName ) )
            {
            	zip = new GZIPInputStream( new FileInputStream( fileName ), bDataSize );
                zip.read( hdr, 0, hdrSize );
                if (bDataSize != zip.read( bData, 0, bDataSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return false;
                }
                zip.close( );
            }
            else
            {
                FileInputStream fis = new FileInputStream( fileName );
                if ( hdrSize != fis.skip( hdrSize ) )
                {
                    fis.close( );
                    return false;
                }
                if ( bDataSize != fis.read( bData, 0, bDataSize ) )
                {
                    fis.close( );
                    return false;
                }
                fis.close( );
            }
            if ( p.byteSwap )
            {
                for ( int n = 0; n < imageSize; n++ )
                {
                    data[offset+n] = Convert.byte2intl( bData, n << 1 );
                }
            }
            else
            {
                for ( int n = 0; n < imageSize; n++ )
                {
                    data[offset+n] = Convert.byte2intb( bData, n << 1 );
                }
            }
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return false;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while opening " + fileName );
            return false;
        }
        catch ( OutOfMemoryError e )
        {
            System.out.println( "Out of memory." );
            return false;
        }
	
        return true;
    }
    
    /**
     * 
     * @param fileName
     * @param data
     * @param offset
     * @param p
     * @return 
     */
    public static boolean loadImageACR( String fileName, int[] data, 
                                        int offset, ImageFormatInfo p ) 
    {

        GZIPInputStream zip;
        FileInputStream fIn;
        byte[] hdr;

        Dimension d = getImageDimensionsACR( fileName, p );
        int bDataSize = p.bytesPerPixel * d.width*d.height;
        int imageSize = d.width * d.height;
        byte[] bData = new byte[ bDataSize ];

        try
        {
            int fSize = (int)FileUtils.getUncompressedFileSize( fileName );
            int hdrSize = fSize - bDataSize;
            if ( hdrSize <= 0 )
            {
                return false;
            }
            hdr = new byte[hdrSize];
            if ( FileUtils.fileIsZipped( fileName ) )
            {
                zip = new GZIPInputStream( new FileInputStream( fileName ), bDataSize );
                zip.read( hdr, 0, hdrSize );
                if ( bDataSize != zip.read( bData, 0, bDataSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return false;
                }
                zip.close( );
            }
            else
            {
                FileInputStream fis = new FileInputStream( fileName );
                if ( hdrSize != fis.skip( hdrSize ) )
                {
                    fis.close( );
                    return false;
                }
                if ( bDataSize != fis.read( bData, 0, bDataSize ) )
                {
                    fis.close( );
                    return false;
                }
                fis.close( );
            }
            if ( p.byteSwap )
            {
                for ( int n = 0; n < imageSize; n++ )
                {
                    data[offset+n] = Convert.byte2intl( bData, n << 1 );
                }
            }
            else
                for ( int n = 0; n < imageSize; n++ )
                {
                    data[offset+n] = Convert.byte2intb( bData, n << 1 );
                }
            }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return false;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while opening " + fileName );
            return false;
        }
        catch ( OutOfMemoryError e )
        {
            System.out.println( "Out of memory." );
            return false;
        }
	
        return true;
    }
    
    /**
     * 
     * @param fileName
     * @param data
     * @param offset
     * @param p
     * @return 
     */
    public static boolean loadImageDICOM( String fileName, int[] data, 
                                          int offset, ImageFormatInfo p ) 
    {
	
	int i = fileName.lastIndexOf( System.getProperty( "file.separator" ) );
	String dirName;
	String fName;
	
        DicomDecoder dd = null;
        
	if ( i == -1 ) 
        {
	    dirName = "." + System.getProperty( "file.separator" );
	    fName = fileName;
	}
	else 
        {
	    dirName = fileName.substring( 0, i + 1 );
	    fName = fileName.substring( i + 1 );
	}

	try 
        {
	    
            dd = new DicomDecoder( dirName, fName );
            
	    FileInfo fi = dd.getFileInfo( );
	    
            //Sets the image modality attribute
            m_imageModality = fi.getImageModality( );
	    
            int bDataSize = fi.getBytesPerPixel( ) * fi.getImageWidth( ) * fi.getImageHeight( );
	    int imageSize = fi.getImageWidth( ) * fi.getImageHeight( );
	    byte[] bData = new byte[ bDataSize ];

	    int hdrSize = fi.getFilePointerOffsetOfPixelData( );
	    FileInputStream fis = new FileInputStream( fileName );
	    if ( hdrSize != fis.skip( hdrSize ) ) 
            {
		fis.close( );
		return false;
	    }
	    if ( bDataSize != fis.read( bData, 0, bDataSize ) ) 
            {
		fis.close( );
		return false;
	    }
	    fis.close( );

	    for ( int n = 0; n < imageSize; n++ )
            {
               data[ offset + n ] = Convert.byte2intl( bData, n << 1 );
            }
            
	    // version for byte swap turned off
	    // for (int n=0; n<imageSize; n++) data[offset+n] = Convert.byte2intb(bData, n<<1);
	}
	catch ( FileNotFoundException e ) 
        {
	    System.out.println( "File " + fileName + " not found." );
	    return false;
	}
	catch ( IOException e ) 
        {
            dd.closeInputStream( );
	    System.out.println( "IOException while opening " + fileName );
	    return false;
	}
	catch ( OutOfMemoryError e ) 
        {
	    System.out.println( "Out of memory." );
	    return false;
	}

	return true;

	/*	
		// temporarly convert to NUMARIS
		String tempDir = System.getProperty("user.home");
		String tempFileName = "tmpNUMA" + System.currentTimeMillis();
		String resultingTmpName = tempDir + 
		System.getProperty("file.separator") +
		tempFileName + "-00001.ima";
		int r = dcm2numaris(fileName, tempDir, tempFileName);
		if (r != 0) {
		System.out.println(dicomErrorMessage(r));
		}
		p.rememberFormatValues();
		p.setFormatValues(Globals.NUMARIS);
		boolean status = loadImageNUMARIS(resultingTmpName, data, offset, p);
		// remove the temporary NUMARIS file
		File f = new File(resultingTmpName);
		f.delete();
		p.restoreFormatValues();
		return status;
	*/	
	
	
    }
    
    /**
     * 
     * @param fileName
     * @param data
     * @param offset
     * @param p
     * @return 
     */
    public static boolean loadImageCustom( String fileName, int[] data, int offset, 
                                           ImageFormatInfo p )
    {
        GZIPInputStream zip;
        FileInputStream fIn;
        byte[] hdr;

        try
        {
            int hdrSize = p.headerSize;
            int fileSize = (int)FileUtils.getUncompressedFileSize( fileName );
            int imageSize = fileSize - hdrSize;
            int bDataSize = p.bytesPerPixel * imageSize;
	    
            byte[] bData = new byte[bDataSize];
	    
            hdr = new byte[hdrSize];
            if ( FileUtils.fileIsZipped( fileName ) )
            {
                zip = new GZIPInputStream( new FileInputStream( fileName ),
                                           bDataSize );
                zip.read( hdr, 0, hdrSize );
                if ( bDataSize != zip.read( bData, 0, bDataSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return false;
                }
                zip.close( );
            }
            else
            {
                FileInputStream fis = new FileInputStream( fileName );
                if ( hdrSize != fis.skip( hdrSize ) )
                {
                    fis.close( );
                    return false;
                }
                if ( bDataSize != fis.read( bData, 0, bDataSize ) )
                {
                    fis.close( );
                    return false;
                }
                fis.close( );
            }
            if ( p.byteSwap )
            {
                for (int n=0; n < imageSize; n++ )
                {
                    data[offset+n] = Convert.byte2intl( bData, n << 1 );
                }
            }
            else
            {
                for ( int n = 0; n < imageSize; n++ )
                {
                    data[offset+n] = Convert.byte2intb( bData, n<<1 );
                }
            }
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found.");
            return false;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while opening " + fileName );
            return false;
        }
        catch ( OutOfMemoryError e )
        {
            System.out.println( "Out of memory." );
            return false;
        }
        
        return true;
    }

    /**
     * 
     * @param fileName
     * @param data
     * @param offset
     * @param p
     * @return 
     */
    public static boolean loadImage( String fileName, int[] data, int offset, 
                                     ImageFormatInfo p ) 
    {
        boolean status = false;
        int fileFormat = p.getFormat( );

        Globals globs = Globals.getInstance( );

        switch ( fileFormat )
        {
            case Globals.NUMARIS:
                status = loadImageNUMARIS( fileName, data, offset, p );
                break;
            case Globals.ACR_NEMA:
                status = loadImageACR( fileName, data, offset, p );
                break;
            case Globals.DICOM:
                status = loadImageDICOM( fileName, data, offset, p );
                break;
            case Globals.CUSTOM:
                status = loadImageCustom( fileName, data, offset, p );
                break;
            default:
                System.out.println( "ImageUtils.loadImage() error: unknown file format " 
                                    + fileFormat );
        }

        return status;
    }

    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static byte[] readHeaderNUMARIS( String fileName, ImageFormatInfo p ) 
    {
        byte[] hdr;
        int hdrSize = p.headerSize;
	
        hdr = new byte[hdrSize];

        try
        {
            hdr = new byte[hdrSize];
	    
            FileInputStream fis = new FileInputStream( fileName );
            if ( FileUtils.fileIsZipped( fileName ) )
            {
                GZIPInputStream zip = new GZIPInputStream( fis, hdrSize );
                if ( hdrSize != zip.read( hdr, 0, hdrSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return null;
                }
                zip.close( );
            }
            else
            {
                if ( hdrSize != fis.read( hdr, 0, hdrSize ) )
                {
                    fis.close( );
                    return null;
                }
            }
            fis.close( );
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return null;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while reading " + fileName );
            return null;
        }
	
        return hdr;
    }
    
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static byte[] readHeaderACR( String fileName, ImageFormatInfo p ) 
    {

        byte[] hdr;

        try
        {
            Dimension d = getImageDimensionsACR( fileName, p );
            int bDataSize = p.bytesPerPixel * d.width * d.height;
            int fSize = (int)FileUtils.getUncompressedFileSize( fileName );
            int hdrSize = fSize - bDataSize;
	    
            hdr = new byte[ hdrSize ];
	    
            if ( hdrSize <= 0 )
            {
                //Dialogs.messageBox(this, "Wrong image dimensions: file size = " + fSize + ", data size = " + bDataSize, "Load error");
            return null;
            }
	    
            FileInputStream fis = new FileInputStream( fileName );
            if ( FileUtils.fileIsZipped( fileName ) )
            {
                GZIPInputStream zip = new GZIPInputStream( fis, hdrSize );
                if ( hdrSize != zip.read( hdr, 0, hdrSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return null;
                }
                zip.close( );
            }
            else
            {
                if ( hdrSize != fis.read( hdr, 0, hdrSize ) )
                {
                    fis.close( );
                    return null;
                }
                fis.close( );
            }
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return null;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while reading " + fileName );
            return null;
        }
	
        return hdr;
    }
    
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static byte[] readHeaderDICOM( String fileName, ImageFormatInfo p ) 
    {
        byte[] hdr;

        try
        {
            Dimension d = getImageDimensionsDICOM( fileName, p );
            int bDataSize = p.bytesPerPixel * d.width * d.height;
            int fSize = (int)FileUtils.getUncompressedFileSize( fileName );
            int hdrSize = fSize - bDataSize;

            if ( hdrSize <= 0 )
            {
                //Dialogs.messageBox(this, "Wrong image dimensions: file size = " + fSize + ", data size = " + bDataSize, "Load error");
                return null;
            }
	    
            hdr = new byte[hdrSize];

            FileInputStream fis = new FileInputStream( fileName );
            if ( FileUtils.fileIsZipped( fileName ) )
            {
                GZIPInputStream zip = new GZIPInputStream( fis, hdrSize );
                if ( hdrSize != zip.read( hdr, 0, hdrSize ) )
                {
                    // something is wrong with the file
                    zip.close( );
                    return null;
                }
                zip.close( );
            }
            else
            {
                if ( hdrSize != fis.read( hdr, 0, hdrSize ) )
                {
                    fis.close( );
                    return null;
                }
            fis.close( );
            }
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return null;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while reading " + fileName );
            return null;
        }
	
        return hdr;
    }
    
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static byte[] readHeaderCustom( String fileName, ImageFormatInfo p ) 
    {

	byte[] hdr;
	int hdrSize = p.headerSize;

	hdr = new byte[hdrSize];

	try 
        {
	    FileInputStream fis = new FileInputStream( fileName );
	    if (FileUtils.fileIsZipped(fileName)) 
            {
		GZIPInputStream zip = new GZIPInputStream( fis, hdrSize );
		if (hdrSize != zip.read(hdr, 0, hdrSize)) 
                {
		    // something is wrong with the file
		    zip.close( );
		    return null;
		}
		zip.close( );
	    }
	    else 
            {
		if ( hdrSize != fis.read(hdr, 0, hdrSize ) ) 
                {
		    fis.close( );
		    return null;
		}
		fis.close( );
	    }
	}
	catch ( FileNotFoundException e )
        {
	    System.out.println( "File " + fileName + " not found." );
	    return null;
	}
	catch ( IOException e ) 
        {
	    System.out.println( "IOException while reading " + fileName );
	    return null;
	}
	
	return hdr;
    }
    
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static byte[] readHeader( String fileName, ImageFormatInfo p ) 
    {
        
        Globals globs = Globals.getInstance( );

        byte[] hdr;
        int fileFormat = p.getFormat( );

        switch ( fileFormat )
        {
            case Globals.NUMARIS:
                    hdr = readHeaderNUMARIS( fileName, p );
                    break;
            case Globals.ACR_NEMA:
                    hdr = readHeaderACR( fileName, p );
                    break;
            case Globals.DICOM:
                    hdr = readHeaderDICOM( fileName, p );
                    break;
            case Globals.CUSTOM:
                    hdr = readHeaderCustom( fileName, p );
                    break;
            default:
                    hdr = null;
                    System.out.println( "ImageUtils.readHeader( ) error: unknown file format "
                                        + fileFormat );
        }

        return hdr;
    }
    
    /**
     * 
     * @param dcmFileName
     * @param destinationPath
     * @param numarisFileName
     * @return 
     */
    public static int dcm2numaris( String dcmFileName, String destinationPath, 
                                   String numarisFileName ) 
    {

        int retval = 99;

        Globals globs = Globals.getInstance( );

        String cmd = globs.dcm2imaCommand + " -d "
	    + destinationPath + " -o "
	    + numarisFileName + " "
	    + dcmFileName;

        try
        {
            Process p =	Runtime.getRuntime( ).exec( cmd );
            retval = p.waitFor( );
            //System.out.println(" exit code is " + retval);
        }
        catch ( IOException ioX )
        {
            //Dialogs.messageBox(this, "Error in command '" + cmd + "'");
            System.out.println( ioX.toString( ) );
            //ioX.printStackTrace();
        }
        catch ( InterruptedException intX )
        {
            System.out.println( intX.toString( ) );
        }

        return retval;
    }
    
    /**
     * 
     * @param fileName
     * @param p
     * @return 
     */
    public static double[] getPixelDimensions( String fileName, ImageFormatInfo p ) 
    {

        double[] pixdims = { 1.0, 1.0, 1.0 };

        int fileFormat = p.getFormat( );

        Globals globs = Globals.getInstance( );
        
        if ( fileFormat == globs.NUMARIS )
        {
            byte[] hdr = readHeaderNUMARIS( fileName, p );
            pixdims = getPixelDimensions( hdr, p );
        }
        else
        {
            System.out.println( "Warning: getPixelDimensions() not implemented for this file format. Returning default values (1, 1, 1)" );
        }
	
        return pixdims;
    }
    
    /**
     * 
     * @param hdr
     * @param p
     * @return 
     */
    public static double[] getPixelDimensions( byte[] hdr, ImageFormatInfo p ) 
    {

        double[] pixdims = { 1.0, 1.0, 1.0 };

        Globals globs = Globals.getInstance( );

        if ( hdr == null )
        {
            System.out.println( "Warning in getPixelDimensions(): header is null. Returning default values (1, 1, 1)" );
            return pixdims;
        }

        int fileFormat = p.getFormat( );

        if ( fileFormat == globs.NUMARIS )
        {
            pixdims[0] = Convert.byte2double( hdr, 5000 );
            pixdims[1] = Convert.byte2double( hdr, 5008 );
            pixdims[2] = Convert.byte2double( hdr, 1544 ) * ( 1.0 + Convert.byte2double( hdr, 4136 ) );
        }
        else
        {
            System.out.println( "Warning: getPixelDimensions( ) not implemented for this file format. Returning default values (1, 1, 1)" );
        }
	
        return pixdims;
    }
    
    /**
     * 
     * @param code
     * @return 
     */
    public static String dicomErrorMessage( int code ) 
    {
        String msg;

        switch ( code )
        {
            case 0:
                    msg = "OK";
                    break;
            case -1:
                    msg = "Not a dicom file";
                    break;
            case -2:
                    msg = "File not found";
                    break;
            case -3:
                    msg = "Not enough memory";
                    break;
            case -4:
                    msg = "File corrupt";
                    break;
            case -5:
                    msg = "File create error";
                    break;
            case -6:
                    msg = "Symbolic link create error";
                    break;
            case -7:
                    msg = "Directory create error";
                    break;
            case 1:
                    msg = "Empty image";
                    break;
            case 99:
                    msg = "Invalid command";
                    break;
            default:
                    msg = "unknown error code";
        }
        return msg;
    }
    
    /**
     * Returns the image modality of the images (e.g. CT/MR)
     * @return 
     */
    static public String getImageModality( )
    {
        return m_imageModality;
    }
    
    // Private attributes
    static String m_imageModality = "MR";
}
