/*
 * File: $Id: MessageBox.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 30.11.2002  Authors: Martin Buerki  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.Point;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.BorderFactory;

public class MessageBox extends JDialog 
{

    // constants defining the set of buttons
    public static int MB_OK = 1;
    public static int MB_CANCEL = 2;
    public static int MB_YES = 4;
    public static int MB_NO = 8;
    public static int MB_OKCANCEL = 3;
    public static int MB_YESNO = 12;

    // modal results
    public static int MR_NONE = 0;
    public static int MR_OK = 1;
    public static int MR_CANCEL = 2;
    public static int MR_YES = 4;
    public static int MR_NO = 8;

    private JLabel text;
    private int answer;
    private JFrame parent;
    private buttonPanel bPanel;

    public MessageBox( JFrame parent ) 
    {
        super( parent, "MessageBox", true );
        setSize( 200, 100 );
        this.parent = parent;
        text = new JLabel( );
        text.setAlignmentX( JLabel.CENTER );
        text.setAlignmentY( JLabel.CENTER );
        JPanel contentPanel = new JPanel( );
        
        contentPanel.setLayout( new BorderLayout( ) );
        contentPanel.add( text, BorderLayout.CENTER );
        contentPanel.add( bPanel = new buttonPanel( this ),
                                    BorderLayout.SOUTH );
        contentPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        this.getContentPane( ).add( contentPanel );
    }

    public int showModal( String msg, String title, int buttons ) 
    {
        text.setText( msg );
        setTitle( title );
        bPanel.setButtons( buttons );
        /*
        FontMetrics fm = text.getFontMetrics( text.getFont( ) );
        int w = fm.stringWidth( msg ) + 50;
        int h = 100;
        Rectangle r = parent.getBounds( );
        setBounds( r.x + ( r.width - w ) / 2, r.y + ( r.height - h ) / 2, w, h );
         */
        Toolkit.getDefaultToolkit( ).beep( );
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        if ( parent != null )
        {
            Point parentLoc = parent.getLocation( );
            Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        parent.getY( ) + ( parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
            setLocation( centerParent );
        }
        
        setVisible( true );
	
        return answer;
    }

    class buttonPanel extends JPanel implements ActionListener 
    {
        MessageBox dlg;
        private JButton bOK;
        private JButton bCancel;
        private JButton bYes;
        private JButton bNo;

        public buttonPanel( MessageBox owner )
        {
            dlg = owner;
            bOK = new JButton( "OK" );
            bOK.addActionListener( this );
            add( bOK );
            bYes = new JButton( "Yes" );
            bYes.addActionListener( this );
            add( bYes );
            bNo = new JButton( "No" );
            bNo.addActionListener( this );
            add( bNo );
            bCancel = new JButton( "Cancel" );
            bCancel.addActionListener( this );
            add( bCancel );
        }
        
        public void setButtons( int buttons )
        {
            bOK.setVisible( ( buttons & MB_OK ) != 0 );
            bCancel.setVisible( ( buttons & MB_CANCEL ) != 0 );
            bYes.setVisible( ( buttons & MB_YES ) != 0 );
            bNo.setVisible( ( buttons & MB_NO ) != 0 );
        }

        public void actionPerformed( ActionEvent e )
        {
            if ( e.getSource( ) == bOK )
            {
                dlg.answer = MR_OK;
            }
            else if ( e.getSource( ) == bCancel )
            {
                dlg.answer = MR_CANCEL;
            }
            else if ( e.getSource( ) == bYes )
            {
                dlg.answer = MR_YES;
            }
            else if ( e.getSource( ) == bNo )
            {
                dlg.answer = MR_NO;
            }
            else
            {
                dlg.answer = MR_NONE;
            }
            dlg.dispose( );
        }
        // For proper serialization
        private static final long serialVersionUID = 1481295122349393L;
    }
    
    
    // For proper serialization
    private static final long serialVersionUID = 3381295812349393L;
}
