/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.model;

import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.dcm4che2.data.DicomObject;
import org.spectrim.autoqualitycontrol.QualityClassifier;
import org.spectrim.data.CSIData;
import org.spectrim.geometricobjects.Cube3D;
import org.spectrim.geometricobjects.ImagePlane;
import org.spectrim.geometricobjects.IntersectionLine;
import org.spectrim.grid.CSIGrid;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.registration.AffineTransformation;
import org.spectrim.registration.DisplacementField;
import org.spectrim.registration.ImageTools;
import org.spectrim.spectra.FittedSpectra;
import org.spectrim.spectra.Spectra;
import org.spectrim.colormap.ColorMap;
import org.spectrim.colormap.ColorMapUI;
import org.spectrim.autoqualitycontrol.MismatchLabellingAssistant;
import org.spectrim.main.MetaboliteFeatures;
import org.spectrim.spectra.Spectrum;
import org.spectrim.tdfdfit.NumericTDFDFitResults;
import org.spectrim.tdfdfit.TDFDFitResultsStore;
import org.spectrim.view.ImageSlice;
import org.spectrim.view.ImageStack;
import org.spectrim.view.ImageViewerPanel;
import org.spectrim.view.LocalizerPanel;
import org.spectrim.view.OverlaySlice;
import org.spectrim.view.OverlayStack;
import org.spectrim.view.SpectraPanel;

/**
 *
 * @author nunopbarros
 */
public class GlobalModel {

    // State Variables
    private int slice;
    private ArrayList<String> selectedVoxels = new ArrayList<>();
    private int[] lastSelectedVoxel = new int[]{0, 0, 0};
    private ArrayList<SelectionGroup> selectedVoxelsGroup = new ArrayList<>();
    private TDFDFitResultsStore m_tdfdfitResultsStore = new TDFDFitResultsStore( this );
    
    private boolean fidCSIDataLoaded = false;
    private HashMap<String, Double> m_allFittedWaterReferenceResonanceFrequencies;
    private HashMap<String, Double> m_TDFDFitQualityNumbers;
    private float m_gmSubVolume = -1f;
    private float m_wmSubVolume = -1f;
    private float m_csfSubVolume = -1f;
    
    private int m_setMinimalGMSegmentationPixelValue = -1;
    private int m_setMinimalWMSegmentationPixelValue = -1;
    private int m_setMinimalCSFSegmentationPixelValue = -1;
    private int m_setMaximalGMSegmentationPixelValue = -1;
    private int m_setMaximalWMSegmentationPixelValue = -1;
    private int m_setMaximalCSFSegmentationPixelValue = -1;
    
    private boolean isMidasEPSIImportedDataMode = false;
    
    /**
     * 
     * Defines whether an segmentation overlay is available.
     * 
     * @param overlayAvailable
     */
    public void setSegmentationOverlayAvailable( boolean overlayAvailable )
    {
        m_overlayAvailable = overlayAvailable;
    }
    
    public boolean getSegmentationOverlayAvailable( )
    {
        return m_overlayAvailable;
    }
    
    /**
     * Returns the CSIData object as used by TDFDFit.
     * @return 
     */
    public CSIData getCSIData( ) 
    {
        return csidata;
    }
    
    /**
     * Utility function getStudyDescription
     * @return 
     */
    public String getStudyDescription( ) 
    {
        return csidata.getStudyDescription( );
    }
    
    /**
     * Utility function getAccessionNumber
     * @return 
     */
    public String getAccessionNumber( ) 
    {
        return csidata.getAccessionNumber( );
    }

    public String getPatientBirthDate( ) 
    {
        return csidata.getPatientBirthDate( );
    }

    public String getPatientSex( ) 
    {
       return csidata.getPatientSex( );
    }

    public String getStudyTime( ) 
    {
        return csidata.getStudyTime( );
    }

    public String getSeriesTime( ) 
    {
        return csidata.getSeriesTime( );
    }
    
    /**
     * Sets the variable necessary to identify the case of FIDCSI data: 
     * In this case there is excited PRESS volume: the whole FOV is excited.
     * @param fidCSILoaded
     */
    public void setFidCSIDataLoaded( boolean fidCSILoaded )
    {
        fidCSIDataLoaded = fidCSILoaded;
    }
    
    /**
     * Variable necessary to identify the case of FIDCSI data: 
     * In this case there is excited PRESS volume: the whole FOV is excited.
     * @return fidCSIDataLoaded
     */
    public boolean getFidCSIDataLoaded( )
    {
        return fidCSIDataLoaded;
    }
    
    /**
     * Returns the scanner type, eg TRIO etc.
     * @return
     */
    public String getScannerModelName( ) 
    {
        return csidata.getScannerName( );
    }

    /**
     * After TDFDFit has finished on the dataset.
     * @param allTDFDFitData
     * @param errorMap
     * @param noiseStdMap 
     */
    public void setTDFDFitResultsStore( HashMap<String, NumericTDFDFitResults> allTDFDFitData,
                                        HashMap<String, float[]> errorMap, 
                                        HashMap<String, Float> noiseStdMap
                                       ) 
    {
        this.setLastQuantificationMethod(GlobalModel.TDFDFIT );
        m_tdfdfitResultsStore.addTDFDFitResultsData( allTDFDFitData, errorMap, noiseStdMap );
    }
    
    /**
     * Returns the TDFDFit store.
     * @return 
     */
    public TDFDFitResultsStore getTDFDFitStore( ) 
    {
        return m_tdfdfitResultsStore;
    }

    public void setWaterResonanceFrequenciesForTemperatureMapping( HashMap<String, Double> allFittedWaterReferenceResonanceFrequencies ) 
    {
        m_allFittedWaterReferenceResonanceFrequencies = allFittedWaterReferenceResonanceFrequencies;
    }
    
    
    public void setFitQualityNumbersForMapping( HashMap<String, Double> TDFDFitQualityNumbers ) 
    {
        m_TDFDFitQualityNumbers = TDFDFitQualityNumbers;
    }
    
    
    public HashMap<String, Double> getTDFDFitQualityNumbers( )
    {
        return m_TDFDFitQualityNumbers;
    }
    
    /**
     * 
     * This function returns the list of temperatures. 
     * 
     * @return 
     */
    public HashMap<String, Double> getTemperatures( )
    {
       // Temperature mapping.
       // ====================
       // NAA: T = -97.134 ( ± 0.726 ) * ( delta_W_NAA - 2.677 ) + 36.0 (± 0.138)
       // Cr : T = -94.768 ( ± 0.741 ) * ( delta_W_CR -  1.660 ) + 36.0 (± 0.123) 
       // Cho: T = -100.813 ( ± 0.986 ) * ( delta_W_CHO - 1.473 ) + 36.0 (± 0.156)
       // Values taken from Covaciu et al. 
       // JOURNAL OF MAGNETIC RESONANCE IMAGING 31:807–814 (2010) 
        
        HashMap<String, Double> tissueTemperaturesNAA = new HashMap<>( );
        HashMap<String, Double> tissueTemperaturesCr = new HashMap<>( );
        HashMap<String, Double> tissueTemperaturesCho = new HashMap<>( );
        
        HashMap<String, Double> meanTissueTemperatures = new HashMap<>( );
        
        HashMap<String, Double> naaFrequencies = new HashMap<>( );
        HashMap<String, Double> crFrequencies = new HashMap<>( );
        HashMap<String, Double> choFrequencies = new HashMap<>( );
        
        m_tdfdfitResultsStore.getVoxelsMetabolites( );
        
        String[] metabNames = m_tdfdfitResultsStore.getMetabolitesNames( );
        for ( int i = 0 ; i < metabNames.length; i++ )
        {
            
            if ( metabNames[i].contains( "naa" ) || metabNames[i].contains( "NAA" ) )
            {
                naaFrequencies = m_tdfdfitResultsStore.getFeatureMap( metabNames[i], MetaboliteFeatures.FRQSHIFT );
            }
            
            if ( metabNames[i].contains( "cr" ) || metabNames[i].contains( "Cr" ) || metabNames[i].contains( "CR" ) )
            {
                crFrequencies = m_tdfdfitResultsStore.getFeatureMap( metabNames[i], MetaboliteFeatures.FRQSHIFT );
            }
            
            if ( metabNames[i].contains( "Cho" ) || metabNames[i].contains( "CHO" ) || metabNames[i].contains( "cho" ) )
            {
                choFrequencies = m_tdfdfitResultsStore.getFeatureMap( metabNames[i], MetaboliteFeatures.FRQSHIFT );
            }
        }
        
        // Temperatures using NAA position.
        Iterator it = m_allFittedWaterReferenceResonanceFrequencies.entrySet( ).iterator( );

        Iterator it2 = naaFrequencies.entrySet( ).iterator( );
        int cntr = 0;
        
        while ( it2.hasNext( ) ) 
        {
            Map.Entry pair = (Map.Entry)it2.next( );
            double naaFrequency = (Double)pair.getValue( );
            double waterFrequency = m_allFittedWaterReferenceResonanceFrequencies.get( pair.getKey( ) );
            
//            System.out.println( "naaResonanceIn [pnts]    : " + ( naaFrequency - 512.0 ) );
            
            double naaResonanceInPPM   = 4.7 - ( ( ( naaFrequency - 512.0 ) * getBandwidth( ) ) * 1.0e6 ) / ( 1024 * getResonanceFrequency( ) ); 
            double waterResonanceInPPM = 4.7 - ( ( ( waterFrequency - 512.0 ) * getBandwidth( ) ) * 1.0e6 ) / ( 1024 * getResonanceFrequency( ) ); 
            double delta1 = waterResonanceInPPM - naaResonanceInPPM;
            double temperature = ( -97.134 * ( delta1 - 2.677 ) )  + 36.0; 
            
            tissueTemperaturesNAA.put( (String)pair.getKey( ), new Double( temperature ) );
            
//            System.out.println( "Temperature          : " + temperature );
//            System.out.println( "naaResonanceInPPM    : " + naaResonanceInPPM );
//            System.out.println( "waterResonanceInPPM  : " + waterResonanceInPPM );
//            System.out.println( "deltaInPPM           : " + ( waterResonanceInPPM - naaResonanceInPPM ) );
            
        }
        
        ////////////////////////////////////////////////////////////////////////////////////////////
        // Temperatures using Cr position.
        Iterator it3 = crFrequencies.entrySet( ).iterator( );
        cntr = 0;
        
        while ( it3.hasNext( ) ) 
        {
            Map.Entry pair = (Map.Entry)it3.next( );
            double crFrequency = (Double)pair.getValue( );
            double waterFrequency = m_allFittedWaterReferenceResonanceFrequencies.get( pair.getKey( ) );
            
//           System.out.println( "crResonanceIn [pnts]    : " + ( crFrequency - 512.0 ) );
           
            double crResonanceInPPM   = 4.7 - ( ( ( crFrequency - 512.0 ) * getBandwidth( ) ) * 1.0e6 ) / ( 1024 * getResonanceFrequency( ) ); 
            double waterResonanceInPPM = 4.7 - ( ( ( waterFrequency - 512.0 ) * getBandwidth( )  ) * 1.0e6 ) / ( 1024 * getResonanceFrequency( ) ); 
            
            double delta2 = waterResonanceInPPM - crResonanceInPPM;
            double temperature =  -94.768 * ( delta2 - 1.660 )  + 36.0; 
            
            tissueTemperaturesCr.put( (String)pair.getKey( ), new Double( temperature ) );
            
//            System.out.println( "Temperature          : " + temperature );
//            System.out.println( "crResonanceInPPM    : " + crResonanceInPPM );
//            System.out.println( "waterResonanceInPPM  : " + waterResonanceInPPM );
//            System.out.println( "deltaInPPM           : " + ( waterResonanceInPPM - crResonanceInPPM ) );
            
        }
        
        ////////////////////////////////////////////////////////////////////////////////////////////
        // Temperatures using Cho position.
        Iterator it4 = choFrequencies.entrySet( ).iterator( );
        cntr = 0;
        
        while ( it4.hasNext( ) ) 
        {
            Map.Entry pair = (Map.Entry)it4.next( );
            double choFrequency = (Double)pair.getValue( );
            double waterFrequency = m_allFittedWaterReferenceResonanceFrequencies.get( pair.getKey( ) );
            
//            System.out.println( "choResonanceIn [pnts]    : " + ( choFrequency - 512.0 ) );
           
            double choResonanceInPPM   = 4.7 - ( ( ( choFrequency - 512.0 ) * getBandwidth( ) ) * 1.0e6 ) / ( 1024 * getResonanceFrequency( ) ); 
            double waterResonanceInPPM = 4.7 - ( ( ( waterFrequency - 512.0 ) * getBandwidth( )  ) * 1.0e6 ) / ( 1024 * getResonanceFrequency( ) ); 
            
            double delta3 = waterResonanceInPPM - choResonanceInPPM;
            
            double temperature =  -100.813 * ( delta3 - 1.473 )  + 36.0; 
            
            tissueTemperaturesCho.put( (String)pair.getKey( ), new Double( temperature ) );
            
//            System.out.println( "Temperature          : " + temperature );
//            System.out.println( "choResonanceInPPM    : " + choResonanceInPPM );
//            System.out.println( "waterResonanceInPPM  : " + waterResonanceInPPM );
//            System.out.println( "deltaInPPM           : " + ( waterResonanceInPPM - choResonanceInPPM ) );
            
        }
        
        Iterator it5 = choFrequencies.entrySet( ).iterator( );
        while ( it5.hasNext( ) ) 
        {
            Map.Entry pair = (Map.Entry)it5.next( );
            double naaTemp = tissueTemperaturesNAA.get( pair.getKey( ) );
            double crTemp = tissueTemperaturesCr.get( pair.getKey( ) );
            double choTemp = tissueTemperaturesCho.get( pair.getKey( ) );
            double average = ( naaTemp + crTemp + choTemp ) / 3.0;
            
            meanTissueTemperatures.put( (String)pair.getKey( ) , new Double( average ) );
            //System.out.println( "Average NAA Temperature = " + naaTemp );
            //System.out.println( "Average Cr Temperature  = " + crTemp );
            //System.out.println( "Average Cho Temperature = " + choTemp );
        }
        
        
        return meanTissueTemperatures;
    }

    public void setSVSGMSegmentVolume(float gmGMVolumeValue ) 
    {
        m_gmSubVolume = gmGMVolumeValue;
    }

    public void setSVSWMSegmentVolume( float csfWMVolumeValue ) 
    {
        m_wmSubVolume = csfWMVolumeValue;
    }

    public void setSVSCSFSegmentVolume( float csfSVSVolumeValue ) 
    {
        m_csfSubVolume = csfSVSVolumeValue;
    }
  
     public float getSVSGMSegmentVolume( ) 
    {
        return m_gmSubVolume;
    }

    public float getSVSWMSegmentVolume( ) {
        return m_wmSubVolume;
    }

    public float getSVSCSFSegmentVolume( ) 
    {
        return m_csfSubVolume;
    }

    public void setMinimalGMSegmentationPixelValue(int value) 
    {
        m_setMinimalGMSegmentationPixelValue = value;
    }

    public void setMinimalWMSegmentationPixelValue(int value) 
    {
        m_setMinimalWMSegmentationPixelValue = value;
    }

    public void setMinimalCSFSegmentationPixelValue(int value) 
    {
        m_setMinimalCSFSegmentationPixelValue = value;
    }

    public void setMaximalGMSegmentationPixelValue(int value) 
    {
        m_setMaximalGMSegmentationPixelValue = value;
    }

    public void setMaximalWMSegmentationPixelValue(int value) 
    {
        m_setMaximalWMSegmentationPixelValue = value;
    }

    public void setMaximalCSFSegmentationPixelValue(int value) 
    {
        m_setMaximalCSFSegmentationPixelValue = value;
    }
    
    public int getMinimalGMSegmentationPixelValue( ) 
    {
        return m_setMinimalGMSegmentationPixelValue;
    }

    public int getMinimalWMSegmentationPixelValue( ) 
    {
        return m_setMinimalWMSegmentationPixelValue;
    }

    public int getMinimalCSFSegmentationPixelValue( ) 
    {
        return m_setMinimalCSFSegmentationPixelValue;
    }

    public int getMaximalGMSegmentationPixelValue( ) 
    {
        return m_setMaximalGMSegmentationPixelValue;
    }

    public int getMaximalWMSegmentationPixelValue( ) 
    {
        return m_setMaximalWMSegmentationPixelValue;
    }

    public int getMaximalCSFSegmentationPixelValue( ) 
    {
        return m_setMaximalCSFSegmentationPixelValue;
    }
    
        /**
     * Returns the maximum ppm value in the center point of the spectrum.
     * in ppm.
     * @return 
     */
    public double getMaximalPPMValueOfSpectrum( ) 
    {
        
        double bandwithInPPM = ( getBandwidth( ) / this.getResonanceFrequency( ) ) * 1000000f;
        double maxPPMValue = getPPMReference( ) + bandwithInPPM / 2f;
        return maxPPMValue;
    }
    
     /**
     * Returns the minimum ppm value in the center point of the spectrum.
     * in ppm.
     * @return 
     */
    public double getMinimalPPMValueOfSpectrum( ) 
    {
        double bandwithInPPM = ( getBandwidth( ) / this.getResonanceFrequency( ) ) * 1000000f;
        double minPPMValue = getPPMReference() - bandwithInPPM / 2f;
        return minPPMValue;
    }
    
     /**
     * Returns the ppm range value in the center point of the spectrum.
     * in ppm.
     * @return 
     */
    public double getPPMRangeOfSpectrum( ) 
    {
        double bandwithInPPM = ( getBandwidth( ) / this.getResonanceFrequency( ) ) * 1000000f;
        return bandwithInPPM;
    }

    
    public void loadExternalCSVOverlayLabelFile( String externalCSVOverlayLabelFileName )
    {
        BufferedReader br;
        int linecounter = 0;
        int tokenCounter = 0;
        int responsecounter = 0;
        int verificationResponsesCounter = 0;
        String voxel_ID;
        int totalNumberOfLines = 0;
        int numberOfColumns = 0;
        
        BufferedReader reader;
        try 
        {
            reader = new BufferedReader( new FileReader( externalCSVOverlayLabelFileName ) );
            
            while ( reader.readLine( ) != null )
            {
                totalNumberOfLines++;   
            }
            reader.close( );
        }
        catch ( FileNotFoundException ex ) 
        {
            Logger.getLogger( GlobalModel.class.getName( ) ).log( Level.SEVERE, null, ex );
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( GlobalModel.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        try 
        {
            br = new BufferedReader( new FileReader( externalCSVOverlayLabelFileName ) );
            String line;
            
            // Read the header description of the csv file
            line = br.readLine( );
            StringTokenizer labelNamesTokenizer = new StringTokenizer( line, "," );
              
            int cntr=0;
            
            numberOfColumns = labelNamesTokenizer.countTokens( );
            
            m_overLayDataHeader = new String[ labelNamesTokenizer.countTokens( ) ];
            
            while ( labelNamesTokenizer.hasMoreElements( ) )
            {
                m_overLayDataHeader[cntr] = labelNamesTokenizer.nextToken( );
//                System.out.print( "Header label [" + cntr + "] -> " + m_overLayDataHeader[cntr] + "; " );
                cntr++;
            }
            
            System.out.println( );
            
            // Read the header description of the csv file
            line = br.readLine( );
            StringTokenizer columnDataTypeTokenizer = new StringTokenizer( line, "," );
            
            m_overlayColumnDataType = new String[ columnDataTypeTokenizer.countTokens( ) ];
            
            cntr = 0;
            while ( columnDataTypeTokenizer.hasMoreElements( ) )
            {
                m_overlayColumnDataType[cntr] = columnDataTypeTokenizer.nextToken( );
//                System.out.print( "Column[" + cntr + "] data type -> " + m_overlayColumnDataType[cntr] + "; " );
                cntr++;
            }
            
            // Now the data size of the objects in the CSV file is known.
            m_overLayData = new Object[totalNumberOfLines-2][numberOfColumns];
            
            System.out.println( );
                       
            while ( ( line = br.readLine( ) ) != null ) 
            {
                // Write line to stout
                // System.out.println( line );
                
                if ( line.trim( ).equals( "" ) )
                {
                    System.out.println( "End of file reached!" );
                    break;
                }    
                
                // tokenize the line ...
                StringTokenizer lineTokenizer = new StringTokenizer( line, "," );
                
                for ( int i = 0; i < numberOfColumns; i++ )
                {
 //                   System.out.print( "*" );
                    m_overLayData[linecounter][i] = lineTokenizer.nextElement( );
                }
                linecounter++;
            }
            m_isColoredOverlayDisplayOfCSVDataFileModeActive = true;
            br.close( );
            System.out.println( "CSV formatted overlay label file :\n" +  externalCSVOverlayLabelFileName + " sucessfully loaded !" );
            
        }
        catch ( Exception ex ) 
        {
            ex.printStackTrace( );
        }
    }
    
    /**
     * Returns the overlay data matrix of an external csv file related to the
     * current MRSI dataset.
     * @return 
     */
    public Object[][] getOverlayCSVTypedDataMatrix( )
    {
        return m_overLayData;
    }
    
    /**
     * Returns the overlay data matrix header names of a previous loaded external 
     * .csv file related to the current MRSI dataset.
     * @return 
     */
    public String[] getOverlayDataMatrixHeaderColumnDataDescription( )
    {
        return m_overLayDataHeader;
    }
    
    /**
     * Returns the overlay data matrix header names of a previous loaded external 
     * .csv file related to the current MRSI dataset.
     * @return 
     */
    public String[] getOverlayDataMatrixHeaderColumnDataType( )
    {
        return m_overlayColumnDataType;
    }
    
    /**
     * Returns true if overlay file was read correctly.
     * @return 
     */
    public boolean isColoredOverlayDisplayOfCSVDataFileModeActive( )
    {
        return m_isColoredOverlayDisplayOfCSVDataFileModeActive;
    }

    public void complexConjugateAllSpectra( ) 
    {
        if ( spectra != null )
        {
            spectra.complexConjugateAllSpectra( );
            updateSpectraPanel( );
        }
    }
    
    public enum SelectionGroup 
    {
        RED, BLUE
    };
    
    private SelectionGroup defaultSelectionGroup = SelectionGroup.BLUE;
    
    private boolean m_overlayAvailable = false;
    
    private boolean blankImage = false;
    public static final int SELECTION_SINGLE = 0;
    public static final int SELECTION_ADD = 1;
    public static final int SELECTION_REMOVE = 2;
    public static final int SELECTION_RANGE_SINGLE = 3;
    public static final int SELECTION_RANGE_ADD = 4;
    public static final int SELECTION_RANGE_REMOVE = 5;
    public static final int STANDARD = 0;
    public static final int TRI_CHANNEL = 1;
    public static final int LABELLING_QUAL = 0;
    public static final int LABELLING_ITERATIVE = 1;
    public static final int LABELLING_CONSENSUS = 2;
    private int mapMode = GlobalModel.STANDARD;
    private boolean showAbsorption;
    private boolean showDispersion;
    private boolean showMagnitude;
    private boolean showDifference;
    private boolean isSpectraNormalized;
    private ColorMap colormap;
    private ArrayList<IntersectionLine> localizerIntersectionLines = new ArrayList<>( );
//    Data Variables
    private CSIData csidata;
    private CSIGrid csigrid;
    private Spectra spectra;
    private FittedSpectra spectraFitted;
    private ImageStack imageStack = new ImageStack( );
    private OverlayStack overlayStack = new OverlayStack( );
//    Viewers
    private SpectraPanel spectrapanel;
    private ImageViewerPanel viewerpanel;
    private ArrayList<LocalizerPanel> localizerList = new ArrayList<>( );
    private JSlider sliceSlider;
    private JTextArea txtarea;
    private boolean copy2Clipboard;
    private boolean showMapValues = true;
    private boolean showFittedData = true;
    private boolean showOriginalData = true;
    private JLabel labelXVal;
    private JLabel labelYVal;
    private JLabel labelPixelVal;
    private XYPlotFrame plotframe;
    private MismatchLabellingAssistant iterativeLabellingFrame;
    private ColorMapUI colormapUI;
    private int lastQuantificationMethodUsed;
    
    private final static int jMRUI   = 0;
    private final static int TDFDFIT = 1;
    
    // Three data structures to store the features of the csv overlay file...
    private String[]   m_overLayDataHeader;
    private Object[][] m_overLayData;
    private String[]   m_overlayColumnDataType;
    private boolean    m_isColoredOverlayDisplayOfCSVDataFileModeActive = false;
    private  HashMap<String, Color> m_csvFileColorMapValues;
    
    /**
     * The global model: a kind of global variable used in spectrIm to connect
     * all parts together.
     */
    public GlobalModel( ) 
    {
    }
    
    
    /**
     * The copy constructor of the GlobalModel. 
     * @param copiedModel 
     */
    public GlobalModel( GlobalModel copiedModel ) 
    {
        this.slice = copiedModel.slice;
        this.viewerpanel = copiedModel.viewerpanel;
        this.imageStack = copiedModel.imageStack;
        this.overlayStack = copiedModel.overlayStack;
        this.csidata = copiedModel.csidata;
        if ( copiedModel.csigrid != null ) 
        {
            this.csigrid = new CSIGrid( copiedModel.csigrid );
            this.csigrid.setModel( this );
        }
        this.spectra = copiedModel.spectra;
        this.spectraFitted = copiedModel.spectraFitted;
        this.mapMode = copiedModel.mapMode;
        this.colormap = copiedModel.colormap;
        this.selectedVoxels = new ArrayList<>(copiedModel.selectedVoxels);
        this.selectedVoxelsGroup = new ArrayList<>(copiedModel.selectedVoxelsGroup);
        this.spectrapanel = copiedModel.spectrapanel;
        if ( copiedModel.colormap != null ) 
        {
            this.colormap = new ColorMap( copiedModel.colormap, this );
        }
    }

   
    
    public GlobalModel createCombinedModel(GlobalModel model2, int mode) {
        ImageStack ims = new ImageStack();
        ImageStack im2 = model2.imageStack;
        if (this.getImageStackSize() == im2.size()) {
            if (mode == 0) {
                ims = imageStack.getDifference(im2);
            } else if (mode == 1) {
                ims = imageStack.getCheckboard(im2);
            }

            GlobalModel newmodel = new GlobalModel();
            newmodel.setImageStack(ims);
            newmodel.setCSIData(this.csidata);
            newmodel.setCSIGrid(this.csigrid);
            newmodel.setSpectra(this.spectra);
            return newmodel;
        } else {
            return null;
        }
    }

    public GlobalModel createCombinedModel(ArrayList<Raster> im2, int mode) {
        ImageStack ims = new ImageStack();
        if (this.getImageStackSize() == im2.size()) {
            if (mode == 0) {
                ims = imageStack.getDifference(im2);
            } else if (mode == 1) {
                ims = imageStack.getCheckboard(im2);
            }

            GlobalModel newmodel = new GlobalModel();
            newmodel.setImageStack(ims);
            newmodel.setCSIData(this.csidata);
            newmodel.setCSIGrid(this.csigrid);
            newmodel.setSpectra(this.spectra);
            return newmodel;
        } else {
            return null;
        }
    }

    public void setCSIData(CSIData csidata) {
        this.csidata = csidata;
    }

    public boolean isCSIDataNull() {
        return (csidata == null);
    }

    public boolean isSpectraNull() {
        return (spectra == null);
    }

    public boolean isImageBlank() {
        return blankImage;
    }

    public void setCSIGrid(CSIGrid csigrid) {
        this.csigrid = csigrid;
        if (csigrid != null) {
            csigrid.setModel(this);
            this.selectedVoxels.clear();
            this.selectedVoxelsGroup.clear();
            this.selectedVoxels.add("0_0_0");
            this.selectedVoxelsGroup.add(SelectionGroup.BLUE);
            if (viewerpanel != null) {
                String csiInfo = csidata.getStudyDate();
                viewerpanel.setGridInfo(csiInfo.substring(6, 8) + "/" + csiInfo.substring(4, 6) + "/" + csiInfo.substring(0, 4) + " - " + csidata.getSeriesNumber() + " - " + csidata.getSeriesDescription());
            }
            this.colormap = new ColorMap(this);
        }
    }

    public CSIGrid getCSIGrid() {
        return csigrid;
    }

    public void setSpectra(Spectra spectra) {
        this.spectra = spectra;
        if (spectra != null) {
            spectra.setModel(this);
        }
    }

    public void setImageStack(ImageStack imStack) {
        this.imageStack = imStack;
        if (imageStack != null) {
            if (imageStack.size() > 0 && viewerpanel != null) {
                String file = imageStack.getFiles().get(0);
                DicomObject object = DicomHandler.getDicomObject(file);
                if (object != null) {
                    String imageInfo = DicomHandler.getStudyDate(object);
                    imageInfo = imageInfo.substring(6, 8) + "/" + imageInfo.substring(4, 6) + "/" + imageInfo.substring(0, 4) + " - " + DicomHandler.getSeriesNumber(object) + " - " + DicomHandler.getSeriesDescription(object);
                    viewerpanel.setImageInfo(imageInfo);
                    viewerpanel.setPatientName(DicomHandler.getPatientName(object));
                }
            }
        }
        calcGridIntersections();
    }

    public void setImageViewer(ImageViewerPanel viewerpanel) {
        this.viewerpanel = viewerpanel;
        viewerpanel.setModel(this);
        if (imageStack != null) {
            if (imageStack.size() > 0) {
                String file = imageStack.getFiles().get(0);
                DicomObject object = DicomHandler.getDicomObject(file);
                if (object != null) {
                    String imageInfo = DicomHandler.getStudyDate(object);
                    imageInfo = imageInfo.substring(6, 8) + "/" + imageInfo.substring(4, 6) + "/" + imageInfo.substring(0, 4) + " - " + DicomHandler.getSeriesNumber(object) + " - " + DicomHandler.getSeriesDescription(object);
                    viewerpanel.setImageInfo(imageInfo);
                    viewerpanel.setPatientName(DicomHandler.getPatientName(object));
                }
            }

        }
        if (csigrid != null) {
            csigrid.setModel(this);
            String csiInfo = csidata.getStudyDate();
            viewerpanel.setGridInfo(csiInfo.substring(6, 8) + "/" + csiInfo.substring(4, 6) + "/" + csiInfo.substring(0, 4) + " - " + csidata.getSeriesNumber() + " - " + csidata.getSeriesDescription());
        }
    }

    public ImageViewerPanel getImageViewer() {
        return viewerpanel;
    }

    public void setSpectraPanel(SpectraPanel spectrapanel) {
        this.spectrapanel = spectrapanel;
        spectrapanel.setModel(this);
    }

    public void setSliceSlider(JSlider slider) {
        this.sliceSlider = slider;
    }

    public void setTextArea(JTextArea component) {
        this.txtarea = component;
    }

    public void setTextLabelXVal(JLabel label) {
        this.labelXVal = label;
    }

    public void setTextLabelYVal(JLabel label) {
        this.labelYVal = label;
    }

    public void setTextLabelPixelVal(JLabel label) {
        this.labelPixelVal = label;
    }

    public void addLocalizerPanel(LocalizerPanel loc) {
        LocalizerPanel locaux;
        loc.setModel(this);
        switch (loc.getImageInfo().getOrientation()) {
            case TRANS:
                loc.setColor(Color.green);
                break;
            case COR:
                loc.setColor(Color.yellow);
                break;
            case SAG:
                loc.setColor(Color.red);
                break;
        }
        //Remove Localizers with the same orientation
        for (int i = (localizerList.size() - 1); i >= 0; i--) {
            locaux = localizerList.get(i);
            if (locaux.getImageInfo().getOrientation() == loc.getImageInfo().getOrientation()) {
                localizerList.remove(locaux);
            }
        }
        localizerList.add(loc);
        loc.setBackground(Color.black);
        loc.computeProjections();
        for (int i = 0; i < localizerList.size(); i++) {
            locaux = localizerList.get(i);
            loc.addReferencePlane(locaux.getColor(), locaux.getImagePlane());
            locaux.addReferencePlane(loc.getColor(), loc.getImagePlane());
        }
    }

    public void removeLocalizers() {
        localizerList.clear();
        this.localizerIntersectionLines.clear();
    }

    public ArrayList<LocalizerPanel> getLocalizerList() {
        return localizerList;
    }

    public void setPatientNameVisible(boolean bool) {
        if (viewerpanel != null) {
            viewerpanel.setNameVisible(bool);
        }
    }

    public void setSeriesInfoVisible(boolean bool) {
        if (viewerpanel != null) {
            viewerpanel.setInfoSeriesVisible(bool);
        }
    }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
//   STATE CHANGING/READING METHODS
    public void setSlice(int slice) 
    {
        if ( this.slice != slice ) 
        {
            this.slice = slice;
            updateViewSliceChange( );
        }
    }

    public void showMiddleSliceWithIntersection( ) 
    {
        if ( csigrid != null ) 
        {
            setSlice( csigrid.getMiddleSliceWithIntersection( ) );
        }
    }

    public int getSlice() {
        return slice;
    }

    public void setAutoCopyClipboard(boolean state) {
        copy2Clipboard = state;
    }

    public void printNumeric(String txt, boolean clipboard) {
        String[] lines = txt.split("\n");
        String txt2 = "";
        for (int l = 0; l < lines.length; l++) {
            String[] parts = lines[l].split("\t");
            DecimalFormat formatter = new DecimalFormat("#.#####");
            for (int i = 0; i < parts.length; i++) {
                try {
                    txt2 = txt2 + formatter.format(Double.valueOf(parts[i]));
                } catch (NumberFormatException ex) {
                    txt2 = txt2 + parts[i];
                }

                if (i != (parts.length - 1)) {
                    txt2 = txt2 + "\t";
                }
            }
            txt2 = txt2 + "\n";
        }
        printTxt(txt2);
        try {
            if (copy2Clipboard && clipboard) {
                Thread.sleep(50);
                StringSelection stringSelection = new StringSelection(txt);
                Clipboard clpbrd = Toolkit.getDefaultToolkit().getSystemClipboard();
                clpbrd.setContents(stringSelection, null);
            }
        } catch (InterruptedException ex) {
            Logger.getLogger(GlobalModel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void printTxt(String txt) {
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss.SSS");
        Date now = new Date();
        String strDate = sdf.format(now);

        if (this.txtarea != null) {
            txtarea.setCaretPosition(txtarea.getDocument().getLength());
            txtarea.append("#" + strDate + ">> " + txt + "\n");
        } else {
            System.out.println("#" + strDate + ">> " + txt);
        }
    }

    public void setSpectraDomain(Spectra.Domain domain) {
        if (spectra != null) {
            spectra.setDefaultDomain(domain);
            if (spectraFitted != null) {
                spectraFitted.setDefaultDomain(domain);
            }
            if (domain == Spectra.Domain.TD) {
                spectrapanel.setInvertedAxis(false);
            } else {
                spectrapanel.setInvertedAxis(true);
            }
            spectrapanel.setxAxisText(domain.txt);
            spectrapanel.restoreZoom();
            updateSpectraPanel();
        }
    }

    public void setSpectraCombinationFunction(Spectra.CombinationFunction function) {
        if (spectra != null) {
            spectra.setDefaultCombinationFunction(function);
            if (spectraFitted != null) {
                spectraFitted.setDefaultCombinationFunction(function);
            }
            updateSpectraPanel();
        }
    }

    public int getValueMapMode() {
        return mapMode;
    }

    public void setColorMode(int mode) {
        if (viewerpanel != null) {
            this.viewerpanel.setColorMode(mode);
            this.setLocalizersColorMode(mode);
        }
    }
    
    public void setLastQuantificationMethod( int method )
    {
        lastQuantificationMethodUsed = method;
    }
    
    public int setLastQuantificationMethod( )
    {
        return lastQuantificationMethodUsed;
    }
   
    
    public void autoWindowing() {
        if (viewerpanel != null) {
            this.viewerpanel.autoWindowing();
            this.autoWindowingLocalizers();
        }
    }

    public int getSpectraVectorSize() {
        if (spectra != null) {
            return spectra.getVectorSize();
        } else {
            return 0;
        }
    }

    public void setAbsorptionVisible(boolean bool) {
        showAbsorption = bool;
//        if (spectrapanel != null) {
//            if (bool) {
//                spectrapanel.addAbsorptionData();
//                if (this.spectraFit != null) {
//                    spectrapanel.addAbsorptionFitData();
//                }
//                updateSpectraPanel();
//            } else {
//                spectrapanel.removeAbsorptionData();
//                spectrapanel.removeAbsorptionFitData();
//            }
//        }
        updateSpectraPanel();
    }

    public void setDispersionVisible(boolean bool) {
        showDispersion = bool;
//        if (spectrapanel != null) {
//            if (bool) {
//                spectrapanel.addDispersionData();
//                if (this.spectraFit != null) {
//                    spectrapanel.addDispersionFitData();
//                }
//                updateSpectraPanel();
//            } else {
//                spectrapanel.removeDispersionData();
//                spectrapanel.removeDispersionFitData();
//            }
//        }
        updateSpectraPanel();
    }

    public void SetShowDifference(boolean value){
        showDifference = value;
    }
    
    public boolean getShowDifference(){
        return showDifference;
    }
    
    public void setMagnitudeVisible(boolean bool) {
        showMagnitude = bool;
//        if (spectrapanel != null) {
//            if (bool) {
//                spectrapanel.addMagnitudeData();
//                if (this.spectraFit != null) {
//                    spectrapanel.addMagnitudeFitData();
//                }
//                updateSpectraPanel();
//            } else {
//                spectrapanel.removeMagnitudeData();
//                spectrapanel.removeMagnitudeFitData();
//            }
//        }
        updateSpectraPanel();
    }

    public boolean isSpectraNormalized() {
        return isSpectraNormalized;
    }

    public boolean isAbsorptionVisible() {
        return showAbsorption;
    }

    public boolean isDispersionVisible() {
        return showDispersion;
    }

    public boolean isMagnitudeVisible() {
        return showMagnitude;
    }

    public boolean isFIttedSpectraVisible() {
        return showFittedData;
    }

    public void setSpectraAxisVisible(boolean bool) {
        if (spectrapanel != null) {
            spectrapanel.setAxisVisible(bool);
        }
    }

    public String getPatientName() {
        return csidata.getPatientName().replaceAll( "�","_" );
    }

    public String getPatientID() {

        return csidata.getPatientID();
    }

    public String getStudyDate() {
        return csidata.getStudyDate();
    }

    public String getSeriesDescription() {
        if (csidata != null) {
            return csidata.getSeriesDescription();
        } else {
            return "";
        }
    }

    
    public String getSeriesNumber() {
        return csidata.getSeriesNumber();
    }

    public String getSOPInstanceUID() {
        return csidata.getSOPInstanceUID();
    }

    public String getManufacturer() {
        return csidata.getManufacturer();
    }

    public String getModelName() {
        return csidata.getModelName();
    }

    public double getB0() {
        return csidata.getB0();
    }

    public double getTE() {
        if (csidata == null) {
            return -1;
        } else {
            return csidata.getTE();
        }
    }

    public double getTR() {
        if (csidata == null) {
            return -1;
        } else {
            return csidata.getTR();
        }
    }

    public double getBandwidth() {
        if (csidata == null) {
            return -1;
        } else {
            return csidata.getBandwidth();
        }
    }

    public String getSpectroscopyFile() {
        if (csidata == null) {
            return "";
        } else {
            return csidata.getFile();
        }
    }

    public String getNucleus() {
        return csidata.getNucleus();
    }
    
       // Jana
     public String getModality() {
        return csidata.getModality();
    }
    
      public String getStudyID() {
        return csidata.getStudyID();
    }
       
     public String getInstanceNumber() {
        return csidata.getInstanceNumber();
    }  

    public void setXLabelText(String txt) {
        if (labelXVal != null) {
            this.labelXVal.setText(txt);
        }
    }

    public void setYLabelText(String txt) {
        if (labelYVal != null) {
            this.labelYVal.setText(txt);
        }
    }

    public void setPixelValLabelText(String txt) {
        if (labelPixelVal != null) {
            this.labelPixelVal.setText(txt);
        }
    }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
//  CSI DATA METHODS
    public String getReferenceImage(int nr) {
        if (csidata != null) {
            switch (nr) {
                case 1:
                    return csidata.m_ref_imageUID1;
                case 2:
                    return csidata.m_ref_imageUID2;
                case 3:
                    return csidata.m_ref_imageUID3;
                default:
                    return "";
            }
        }
        return "";
    }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
//  CSI GRID METHODS
    public void calcGridIntersections() {
        if (csigrid != null) {
            csigrid.calcIntersections(imageStack);
        }
    }

    public void setGridVisible(boolean bool) {
        if (csigrid != null && viewerpanel != null) {
            viewerpanel.setGridVisible(bool);
            viewerpanel.repaint();
        }
    }

    public void setSaturationBandsVisible(boolean bool) {
        if (csigrid != null && viewerpanel != null) {
            viewerpanel.setSaturationBandsVisible(bool);
            viewerpanel.repaint();
        }
    }

    public void updateGridAmplificationFactor() {
        ImageSlice imageaux = getCurrentImage();
        csigrid.setAmplificationFact( (double) viewerpanel.getWidth( ) / ( (double) imageaux.sizeY * (double) imageaux.pixelSpacingY ), 
                                      (double) viewerpanel.getHeight( ) / ( (double) imageaux.sizeX * (double) imageaux.pixelSpacingX ) );
    }

    public void clearGrid() {
        if (csigrid != null) {
            csigrid = null;
            spectra = null;
            this.selectedVoxels.clear();
            this.selectedVoxelsGroup.clear();
            this.viewerpanel.clearGridProjection();
            updateViewImagePanels();
            this.spectrapanel.clearData();
            clearLocalizerGridProjections();
        }
    }

    public int getGridSize() {
        if (csigrid != null) {
            return csigrid.size();
        } else {
            return 0;
        }
    }

    public int[] getGridDimensions() {
        if (csigrid != null) {
            int[] res = {csigrid.m_gridSizeDir1, csigrid.m_gridSizeDir2, csigrid.m_gridSizeDir3};
            return res;
        } else {
            return new int[]{0, 0, 0};
        }
    }

    public int getGridDim1() {
        if (csigrid != null) {
            return csigrid.m_gridSizeDir1;
        } else {
            return 0;
        }
    }

    public int getGridDim2() {
        if (csigrid != null) {
            return csigrid.m_gridSizeDir2;
        } else {
            return 0;
        }
    }

    public int getGridDim3() {
        if (csigrid != null) {
            return csigrid.m_gridSizeDir3;
        } else {
            return 0;
        }
    }

    public void drawIntersections(Graphics2D g, boolean drawMidPoint, boolean drawSatBands) {
        if (csigrid != null) {
            csigrid.drawIntersections(g, drawMidPoint, drawSatBands);
        }
    }
    
    public void drawSegmentationOverlays( Graphics2D g )
    {
        if ( csigrid != null ) 
        {
            int[] dims = imageStack.getDimensions( );
            
            float physPixelWidth = imageStack.get(0).pixelSpacingX;
            float physPixelHeight = imageStack.get(0).pixelSpacingY;
            float physPixelDepth = imageStack.get(0).sliceThickness;
            
            int imageXPixels = dims[0];
            int imageYPixels = dims[1];
            int translationX = 0;
            int pixelScaleFactor = 1;
            // some stupid hard coding to get it working in most cases
            if ( imageXPixels < imageYPixels )
            {
                if ( ( imageXPixels > 32 ) && ( imageXPixels < 64 ) )
                {
                    translationX = ( 64 - dims[0] )/2;
                    pixelScaleFactor = 4;
                }
                if ( ( imageXPixels > 64 ) && ( imageXPixels < 128 ) )
                {
                    translationX = ( 128 - dims[0] )/2;
                    pixelScaleFactor = 3;
                }
                if ( ( imageXPixels > 128 ) && ( imageXPixels < 256 ) )
                {
                    translationX = ( 256 - dims[0] )/2;
                    pixelScaleFactor = 2;
                }
                if ( ( imageXPixels > 256 ) && ( imageXPixels < 512 ) )
                {
                    translationX = ( 512 - dims[0] )/2;
                    pixelScaleFactor = 2;
                }
                if ( ( imageXPixels > 512 ) && ( imageXPixels < 1024 ) )
                {
                    translationX = ( 1024 - dims[0] )/2;
                    pixelScaleFactor = 1;
                }
            }
            if ( imageXPixels == 64 )
            {
                pixelScaleFactor = 4;
            }
            if ( imageXPixels == 128 )
            {
                pixelScaleFactor = 3;
            }
            if ( imageXPixels == 256 )
            {
                pixelScaleFactor = 2;
            }
            
            if ( imageXPixels == 512 )
            {
                pixelScaleFactor = 2;
            }
            
            if ( imageXPixels == 1024 )
            {
                pixelScaleFactor = 1;
            }
            
            Rectangle bounds = this.viewerpanel.getBounds( );
            
            float scaleX = (float)( bounds.width - 2 * translationX ) / dims[0];
            float scaleY = (float)bounds.height/dims[1];
            
            csigrid.drawSegmentationOverlay( g, scaleX, scaleY, translationX, pixelScaleFactor );
            
        }
    }
    
    
    
    public void setAmplificationFact(double d1, double d2) {
        if (csigrid != null) {
            csigrid.setAmplificationFact(d1, d2);
        }
    }

    public double[] analyseSelectedVoxelsSubImage() {
        double[] res = new double[]{};
        if (csigrid != null) {
            res = csigrid.analyseSelectedVoxelsSubImage(false);
        }
        return res;
    }

    public double[] analyseSelectedVoxelsSubImage(double referenceVal) {
        double[] res = new double[]{};
        if (csigrid != null) {
            res = csigrid.analyseSelectedVoxelsSubImage(false, referenceVal);
        }
        return res;
    }

    public double[] analyseVoxelSubImage(String coord) {
        double[] res = new double[]{};
        if (csigrid != null) {
            res = csigrid.analyseVoxelSubImage(coord, false);
        }
        return res;
    }

    public void analyseSelectedVoxelsOverlaidImage() {
        if (csigrid != null) {
            if (this.hasOverlay()) {
                csigrid.analyseSelectedVoxelsSubImage(true);
            } else {
                this.printTxt("The image contains no overlay!");
            }
        }
    }

    public void setGridTransparency(float trnsp) {
        if (csigrid != null) {
            csigrid.setGridAlpha(trnsp);
        }
    }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
//   UPDATE METHODS
    public void updateViewSliceChange() 
    {
        updateViewImagePanels( );
        if ( sliceSlider != null ) 
        {
            sliceSlider.setValue( this.slice );
        }
    }

    public void updateViewSelectionChange() {
        updateViewImagePanels();
        updateSpectraPanel();
    }

    public void updateViewImagePanels() 
    {
        if ( viewerpanel != null ) 
        {
            viewerpanel.setView( );
            viewerpanel.repaint( );
        }
        setViewLocalizers();
        repaintLocalizers();
    }

    public void updateSpectraPanel( ) 
    {
        if (spectrapanel != null && spectra != null) 
        {
            spectrapanel.setSeriesCollectionNotify(false);
            spectrapanel.setXAxis( getxAxis( ) );
            if ( this.showAbsorption && this.showOriginalData ) 
            {
                spectrapanel.setData( getSpectrumData( Spectra.Mode.ABS, false, SelectionGroup.BLUE ), SpectraPanel.DataType.ABS_BLUE );
                spectrapanel.setData( getSpectrumData( Spectra.Mode.ABS, false, SelectionGroup.RED ), SpectraPanel.DataType.ABS_RED );
            } 
            else 
            {
                spectrapanel.clearData( SpectraPanel.DataType.ABS_RED );
                spectrapanel.clearData( SpectraPanel.DataType.ABS_BLUE );
            }
            if ( spectraFitted != null ) 
            {
                if ( spectraFitted.containsID(selectedVoxels) && this.showAbsorption && this.showFittedData ) 
                {
                    spectrapanel.setData( getSpectrumData( Spectra.Mode.ABS, true, SelectionGroup.BLUE ), SpectraPanel.DataType.ABS_BLUE_FIT );
                    spectrapanel.setData( getSpectrumData( Spectra.Mode.ABS, true, SelectionGroup.RED ), SpectraPanel.DataType.ABS_RED_FIT );
                } 
                else 
                {
                    spectrapanel.clearData( SpectraPanel.DataType.ABS_RED_FIT );
                    spectrapanel.clearData( SpectraPanel.DataType.ABS_BLUE_FIT );
                }
            } 
            else 
            {
                spectrapanel.clearData( SpectraPanel.DataType.ABS_RED_FIT );
                spectrapanel.clearData( SpectraPanel.DataType.ABS_BLUE_FIT );
            }

            if ( this.showDispersion && this.showOriginalData ) 
            {
                spectrapanel.setData( getSpectrumData( Spectra.Mode.DISP, false, SelectionGroup.BLUE ), SpectraPanel.DataType.DISP_BLUE );
                spectrapanel.setData( getSpectrumData( Spectra.Mode.DISP, false, SelectionGroup.RED ), SpectraPanel.DataType.DISP_RED );
            } 
            else 
            {
                spectrapanel.clearData( SpectraPanel.DataType.DISP_RED );
                spectrapanel.clearData( SpectraPanel.DataType.DISP_BLUE );
            }
            if ( spectraFitted != null ) 
            {
                if ( spectraFitted.containsID( selectedVoxels ) && this.showDispersion && this.showFittedData ) 
                {
                     spectrapanel.setData( getSpectrumData( Spectra.Mode.DISP, true, SelectionGroup.BLUE ), SpectraPanel.DataType.DISP_BLUE_FIT );
                    spectrapanel.setData(getSpectrumData(Spectra.Mode.DISP, true, SelectionGroup.RED), SpectraPanel.DataType.DISP_RED_FIT);
                } 
                else 
                {
                    spectrapanel.clearData( SpectraPanel.DataType.DISP_RED_FIT );
                    spectrapanel.clearData( SpectraPanel.DataType.DISP_BLUE_FIT );
                }
            } 
            else 
            {
                spectrapanel.clearData( SpectraPanel.DataType.DISP_RED_FIT );
                spectrapanel.clearData( SpectraPanel.DataType.DISP_BLUE_FIT );
            }

            if ( this.showMagnitude && this.showOriginalData ) 
            {
                spectrapanel.setData( getSpectrumData( Spectra.Mode.MAG, false, SelectionGroup.BLUE ), SpectraPanel.DataType.MAG_BLUE );
                spectrapanel.setData( getSpectrumData( Spectra.Mode.MAG, false, SelectionGroup.RED ), SpectraPanel.DataType.MAG_RED );
            } 
            else 
            {
                spectrapanel.clearData( SpectraPanel.DataType.MAG_RED );
                spectrapanel.clearData( SpectraPanel.DataType.MAG_BLUE );
            }
            if ( spectraFitted != null ) 
            {
                if ( this.showMagnitude && spectraFitted.containsID( selectedVoxels ) && this.showFittedData ) 
                {
                    spectrapanel.setData( getSpectrumData( Spectra.Mode.MAG, true, SelectionGroup.BLUE ), SpectraPanel.DataType.MAG_BLUE_FIT );
                    spectrapanel.setData( getSpectrumData( Spectra.Mode.MAG, true, SelectionGroup.RED ), SpectraPanel.DataType.MAG_RED_FIT );
                } 
                else 
                {
                    spectrapanel.clearData( SpectraPanel.DataType.MAG_RED_FIT );
                    spectrapanel.clearData( SpectraPanel.DataType.MAG_BLUE_FIT );
                }
            } 
            else 
            {
                spectrapanel.clearData( SpectraPanel.DataType.MAG_RED_FIT );
                spectrapanel.clearData( SpectraPanel.DataType.MAG_BLUE_FIT );
            }
            if (this.showDifference)
            {
                if (this.showAbsorption){
                    ArrayList<float[]> final_array = new ArrayList<float[]>();
                    final_array = computeDifference(Spectra.Mode.ABS);
                    spectrapanel.setData( final_array, SpectraPanel.DataType.ABS_BLUE );
                    spectrapanel.setData( final_array, SpectraPanel.DataType.ABS_RED );
                }
                else{
                    spectrapanel.clearData( SpectraPanel.DataType.ABS_RED );
                    spectrapanel.clearData( SpectraPanel.DataType.ABS_BLUE );
                }
                if (this.showDispersion){
                    ArrayList<float[]> final_array = new ArrayList<float[]>();
                    final_array = computeDifference(Spectra.Mode.DISP);
                    spectrapanel.setData( final_array, SpectraPanel.DataType.DISP_BLUE );
                    spectrapanel.setData( final_array, SpectraPanel.DataType.DISP_RED );
                }
                else{
                    spectrapanel.clearData( SpectraPanel.DataType.DISP_RED );
                    spectrapanel.clearData( SpectraPanel.DataType.DISP_BLUE );
                }
                if (this.showMagnitude){
                    ArrayList<float[]> final_array = new ArrayList<float[]>();
                    final_array = computeDifference(Spectra.Mode.MAG);
                    spectrapanel.setData( final_array, SpectraPanel.DataType.MAG_BLUE );
                    spectrapanel.setData( final_array, SpectraPanel.DataType.MAG_RED );
                }
                else{
                    spectrapanel.clearData( SpectraPanel.DataType.MAG_RED );
                    spectrapanel.clearData( SpectraPanel.DataType.MAG_BLUE );
                }
            }
            spectrapanel.updateSeriesColor( );
        }
    }
    
    public void restoreOriginalPanel(){
        spectrapanel.setData(spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.BLUE), Spectra.Mode.ABS), SpectraPanel.DataType.ABS_BLUE);
        spectrapanel.setData(spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.RED), Spectra.Mode.ABS), SpectraPanel.DataType.ABS_RED);
    }
    
    public ArrayList<float[]> computeDifference(Spectra.Mode mode){
        ArrayList<float[]> final_array = new ArrayList<float[]>();
        if(mode==Spectra.Mode.MAG){
            ArrayList<float[]> mean_blue_abs = spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.BLUE), Spectra.Mode.ABS);
            ArrayList<float[]> mean_red_abs = spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.RED), Spectra.Mode.ABS);
            ArrayList<float[]> mean_blue_disp = spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.BLUE), Spectra.Mode.DISP);
            ArrayList<float[]> mean_red_disp = spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.RED), Spectra.Mode.DISP);
            float[] mean_final = new float[mean_blue_abs.get(0).length];
            for (int i=0; i<mean_blue_abs.get(0).length; i++){
                mean_final[i] = (float) Math.sqrt(Math.pow(mean_blue_abs.get(0)[i] - mean_red_abs.get(0)[i],2) + Math.pow(mean_blue_disp.get(0)[i] - mean_red_disp.get(0)[i], 2));
            }
            final_array.add(mean_final);
        }
        else{
            ArrayList<float[]> mean_blue = spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.BLUE), mode);
            ArrayList<float[]> mean_red = spectra.getSpectrumData(getSelectedVoxels(GlobalModel.SelectionGroup.RED), mode);
            float[] mean_final = new float[mean_blue.get(0).length];
            for (int i=0; i<mean_blue.get(0).length; i++){
                mean_final[i] = mean_blue.get(0)[i] - mean_red.get(0)[i];
            }
            final_array.add(mean_final);
        }
    return final_array;
    }
    
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    public int getCurrentlyDisplayedImageSliceNumber( )
    {
        return slice;
    }
    
    
//   VOXEL SELECTION METHODS
    public void setCurrentSelectionGroup(SelectionGroup group) {
        this.defaultSelectionGroup = group;
    }

    public SelectionGroup getCurrentSelectionGroup() {
        return defaultSelectionGroup;
    }

    public SelectionGroup getSelectedVoxelGroup(int pos) {
        return selectedVoxelsGroup.get(pos);
    }

    public ArrayList<String> getSelectedVoxels(SelectionGroup group) {
        ArrayList<String> groupVoxels = new ArrayList<>();
        for (int i = 0; i < selectedVoxels.size(); i++) {
            if (selectedVoxelsGroup.get(i) == group) {
                groupVoxels.add(selectedVoxels.get(i));
            }
        }
        return groupVoxels;
    }

    public void setSelectedVoxels(ArrayList<String> selection) {
        selectedVoxels.clear();
        selectedVoxelsGroup.clear();
        this.selectedVoxels = (ArrayList<String>) selection.clone();
        for (int i = 0; i < selectedVoxels.size(); i++) {
            this.selectedVoxelsGroup.add(this.defaultSelectionGroup);
        }
        this.updateViewSelectionChange();
    }

    public void selectVoxel(int x, int y, int mode) 
    {
        if ( csigrid != null && imageStack.size() > 0 ) 
        {
            int[] dim = csigrid.getVoxelIndex( x, y, imageStack.get(slice).imagePosition, 
                                                     imageStack.get(slice).imageOrientationX, 
                                                     imageStack.get(slice).imageOrientationY);
            if ( dim[0] != -1 ) 
            {
                selectVoxel( dim[0], dim[1], dim[2], mode );
            }
        }
    }

    public int[] getVoxelIndex( int x, int y ) 
    {
        if ( csigrid != null && imageStack.size( ) > 0 ) 
        {
            int[] dim = csigrid.getVoxelIndex( x, y, 
                                              imageStack.get( slice ).imagePosition, 
                                              imageStack.get( slice ).imageOrientationX, 
                                              imageStack.get( slice ).imageOrientationY );
            
            // dirty programming! just to get it working for tilded slices ...
//            if ( isMidasEPSIImportedDataMode == true )
//            {
//                dim[2] = slice;
//            }
            
            if ( dim[0] != -1 ) 
            {
                return dim;
            }
        }
        return new int[]{-1, -1, -1};
    }

    public void setIsManualMidasEPSIImported( boolean midasManualImportedData )
    {
        isMidasEPSIImportedDataMode = midasManualImportedData;
    }
    
    public boolean getIsManualMidasEPSIImported( )
    {
        return isMidasEPSIImportedDataMode;
    } 
    
    
    public void selectVoxel(int dim1, int dim2, int dim3, int mode) {
        if (csigrid != null) {
            boolean update = false;
            int[] lastVoxel = lastSelectedVoxel;
            
            if ( mode != SELECTION_RANGE_SINGLE && 
                 mode != SELECTION_RANGE_ADD && 
                 mode != SELECTION_RANGE_REMOVE && 
                 dim1 < csigrid.m_gridSizeDir1 && 
                 dim2 < csigrid.m_gridSizeDir2 && 
                 dim3 < csigrid.m_gridSizeDir3 && 
                 dim1 >= 0 && 
                 dim2 >= 0 && 
                 dim3 >= 0 ) 
            {
                lastSelectedVoxel = new int[]{ dim1, dim2, dim3 };
            }

            if ( ( mode == SELECTION_SINGLE || mode == SELECTION_RANGE_SINGLE ) && 
                   !selectedVoxels.isEmpty( ) ) 
            {
                selectedVoxels.clear( );
                selectedVoxelsGroup.clear( );
                update = true;
            }
            
            int dim1_2 = dim1;
            int dim2_2 = dim2;
            int dim3_2 = dim3;
            
            if ( lastSelectedVoxel.length == 3 && 
                 ( mode == SELECTION_RANGE_SINGLE || 
                   mode == SELECTION_RANGE_ADD || 
                   mode == SELECTION_RANGE_REMOVE ) ) 
            {
                if ( dim1 < lastVoxel[0] ) 
                {
                    dim1_2 = lastVoxel[0];
                } 
                else 
                {
                    dim1 = lastVoxel[0];
                };
                if ( dim2 < lastVoxel[1] ) 
                {
                    dim2_2 = lastVoxel[1];
                } 
                else 
                {
                    dim2 = lastVoxel[1];
                };
                if ( dim3 < lastVoxel[2] ) 
                {
                    dim3_2 = lastVoxel[2];
                } 
                else 
                {
                    dim3 = lastVoxel[2];
                };
            }
            if ( mode == SELECTION_SINGLE || 
                 mode == SELECTION_RANGE_SINGLE || 
                 mode == SELECTION_RANGE_ADD ) 
            {
                mode = SELECTION_ADD;
            } 
            else if ( mode == SELECTION_RANGE_REMOVE ) 
            {
                mode = SELECTION_REMOVE;
            }
            
            for ( int i = dim1; i <= dim1_2; i++ ) 
            {
                for ( int j = dim2; j <= dim2_2; j++ ) 
                {
                    for ( int k = dim3; k <= dim3_2; k++ ) 
                    {
                        if (    i < csigrid.m_gridSizeDir1 && j < csigrid.m_gridSizeDir2 && k < csigrid.m_gridSizeDir3
                             && i >= 0 && j >= 0 && k >= 0 ) 
                        {
                            String aux = i + "_" + j + "_" + k;
                            if ( !selectedVoxels.contains( aux ) && mode == SELECTION_ADD ) 
                            {
                                selectedVoxels.add( aux );
                                selectedVoxelsGroup.add( this.defaultSelectionGroup );
                                update = true;
                            } 
                            else if ( selectedVoxels.contains( aux ) && 
                                      selectedVoxelsGroup.get( selectedVoxels.indexOf( aux ) ) != this.defaultSelectionGroup && 
                                      mode == SELECTION_ADD ) 
                            {
                                selectedVoxelsGroup.set( selectedVoxels.indexOf( aux ), defaultSelectionGroup );
                                update = true;
                            } 
                            else if ( selectedVoxels.contains( aux ) && mode == SELECTION_REMOVE ) 
                            {
                                selectedVoxelsGroup.remove( selectedVoxels.indexOf( aux ) );
                                selectedVoxels.remove( aux );
                                update = true;
                            }
                        }
                    }
                }
            }
            if ( update ) 
            {
                updateViewSelectionChange( );
            }
        }
    }

    public void selectVoxel(String coord, int mode) {
        String[] coords = coord.split("_");
        if (coords.length == 3) {
            selectVoxel(Integer.valueOf(coords[0]), Integer.valueOf(coords[1]), Integer.valueOf(coords[2]), mode);
        }
    }

    public void selectVoxel(Polygon pol, int mode) {
        if (csigrid != null && imageStack.size() > 0) {
            ArrayList<int[]> voxels = csigrid.getVoxelsInside(pol);
            if (mode == SELECTION_SINGLE) {
                selectedVoxels.clear();
                selectedVoxelsGroup.clear();
                mode = SELECTION_ADD;
            }
            for (int i = 0; i < voxels.size(); i++) {
                int[] dim = voxels.get(i);
                selectVoxel(dim[0], dim[1], dim[2], mode);
            }
        }
    }

    public void selectAllVoxels( ) 
    {
        if ( csigrid != null ) 
        {
            selectedVoxels.clear( );
            selectedVoxelsGroup.clear( );
            for ( int i = 0; i < csigrid.m_gridSizeDir1; i++ ) 
            {
                for ( int j = 0; j < csigrid.m_gridSizeDir2; j++ ) 
                {
                    for ( int k = 0; k < csigrid.m_gridSizeDir3; k++ ) 
                    {
                        selectedVoxels.add( i + "_" + j + "_" + k );
                        selectedVoxelsGroup.add( this.defaultSelectionGroup );
                    }
                }
            }
            updateViewSelectionChange( );
        }
    }

    
    /**
     * Return the ArrayList with excited voxels. 
     * Add exception handling in case the linear algebra fails!
     * @return 
     */
    public ArrayList<String> getExcitedVoxels( ) 
    {
        ArrayList<String> excitedVoxels = new ArrayList<>( );
        if ( csigrid != null ) 
        {
            Cube3D specaux;
            boolean ans;
            for ( int i = 0; i < csigrid.m_gridSizeDir1; i++ ) 
            {
                for ( int j = 0; j < csigrid.m_gridSizeDir2; j++ ) 
                {
                    for ( int k = 0; k < csigrid.m_gridSizeDir3; k++ ) 
                    {
                        specaux = csigrid.getVoxel( i, j, k );
                        try
                        {    
                            ans = csigrid.excitationVoxel.isCenterInside( specaux );
                        }
                        catch ( org.apache.commons.math3.linear.SingularMatrixException e )
                        {
                            ans = false;
                        }
                        
                        if ( ans ) 
                        {
                            excitedVoxels.add( i + "_" + j + "_" + k );
                        }
                    }
                }
            }
            if ( excitedVoxels.isEmpty( ) )
            {
                fidCSIDataLoaded = true;
                selectAllVoxels( );
                excitedVoxels = selectedVoxels; 
            }
        }
        return excitedVoxels;
    }

    
    /**
     * In case of FID-MRSI this routine does not work! 
     * All voxels in the FOV are in fact excited voxels!!
     */
    public void selectExcitedVoxels( ) 
    {
        if ( csigrid != null ) 
        {
            selectedVoxels.clear( );
            selectedVoxelsGroup.clear( );
            Cube3D specaux;
            boolean ans;
            for ( int i = 0; i < csigrid.m_gridSizeDir1; i++ ) 
            {
                for ( int j = 0; j < csigrid.m_gridSizeDir2; j++ ) 
                {
                    for ( int k = 0; k < csigrid.m_gridSizeDir3; k++ ) 
                    {
                        specaux = csigrid.getVoxel( i, j, k );
                        ans = csigrid.excitationVoxel.isCenterInside( specaux );
                        if ( ans ) 
                        {
                            selectedVoxels.add( i + "_" + j + "_" + k );
                            selectedVoxelsGroup.add( this.defaultSelectionGroup );
                        }
                    }
                }
            }
            updateViewSelectionChange( );
        }
    }

    public void removedSaturatedVoxels() {
        if (csigrid != null) {
            Cube3D specaux;
            boolean ans;

            for (int i = (selectedVoxels.size() - 1); i >= 0; i--) {
                String coord = selectedVoxels.get(i);
                specaux = csigrid.getVoxel(coord);
                ans = false;
                for (Cube3D satband : csigrid.saturationBands) {
                    if (!ans) {
                        ans = satband.isCenterInside(specaux);
                    }
                }
                if (ans) {
                    selectedVoxels.remove(coord);
                    selectedVoxelsGroup.remove(i);
                }
            }
            updateViewSelectionChange();
        }
    }

    public void selectInnerExcitedVoxels() {
        if (csigrid != null) {
            selectedVoxels.clear();
            selectedVoxelsGroup.clear();
            Cube3D specaux;
            boolean ans;
            int maxi = Integer.MIN_VALUE;
            int maxj = Integer.MIN_VALUE;
            int mini = Integer.MAX_VALUE;
            int minj = Integer.MAX_VALUE;

            for (int i = 0; i < csigrid.m_gridSizeDir1; i++) {
                for (int j = 0; j < csigrid.m_gridSizeDir2; j++) {
                    for (int k = 0; k < csigrid.m_gridSizeDir3; k++) {
                        specaux = csigrid.getVoxel(i, j, k);
                        ans = csigrid.excitationVoxel.isCenterInside(specaux);
                        if (ans) {
                            selectedVoxels.add(i + "_" + j + "_" + k);
                            selectedVoxelsGroup.add(this.defaultSelectionGroup);
                            if (i < mini) {
                                mini = i;
                            }
                            if (j < minj) {
                                minj = j;
                            }
                            if (i > maxi) {
                                maxi = i;
                            }
                            if (j > maxj) {
                                maxj = j;
                            }
                        }
                    }
                }
            }
            for (int n = (this.selectedVoxels.size() - 1); n >= 0; n--) {
                String[] parts = selectedVoxels.get(n).split("_");
                int i = Integer.valueOf(parts[0]);
                int j = Integer.valueOf(parts[1]);
                if (i == maxi || i == mini || j == maxj || j == minj) {
                    selectedVoxels.remove(n);
                    selectedVoxelsGroup.remove(n);
                }
            }
            updateViewSelectionChange();
        }
    }

    public void clearVoxelSelection() {
        if (csigrid != null) {
            if (getGridSize() > 1) {
                if (spectrapanel != null) {
                    spectrapanel.clearData();
                }
                selectedVoxels.clear();
                selectedVoxelsGroup.clear();
                updateViewImagePanels();
            }
        }
    }

    public ArrayList<String> getSelectedVoxels() {
        return (ArrayList<String>) selectedVoxels.clone();
    }

    public int[] getLastSelectedVoxel() {
        if (this.getGridSize() == 1) {
            return new int[]{0, 0, 0};
        }
        return lastSelectedVoxel;
    }

    public boolean isVoxelSelected() {
        return (!selectedVoxels.isEmpty());
    }

    public Iterator getSelectedVoxelsIterator() {
        return selectedVoxels.iterator();
    }

    public void activateFreeHandSelection() {
        this.viewerpanel.newFreeHandSelection();
    }

    public void mirrorSelection() {
        for (int i = selectedVoxels.size() - 1; i >= 0; i--) {
            if (selectedVoxelsGroup.get(i) != this.defaultSelectionGroup) {
                selectedVoxelsGroup.remove(i);
                selectedVoxels.remove(i);
            }
        }
        if (defaultSelectionGroup == SelectionGroup.BLUE) {
            defaultSelectionGroup = SelectionGroup.RED;
        } else {
            defaultSelectionGroup = SelectionGroup.BLUE;
        }

        for (int i = selectedVoxels.size() - 1; i >= 0; i--) {
            String aux = selectedVoxels.get(i);
            String[] parts = aux.split("_");
            int[] coord = {Integer.valueOf(parts[0]), Integer.valueOf(parts[1]), Integer.valueOf(parts[2])};
            int mirrC1 = Math.abs(coord[0] - csigrid.m_gridSizeDir1) - 1;
            aux = mirrC1 + "_" + parts[1] + "_" + parts[2];
            if (!selectedVoxels.contains(aux)) {
                selectedVoxels.add(aux);
                selectedVoxelsGroup.add(this.defaultSelectionGroup);
            } else if (selectedVoxels.contains(aux) && selectedVoxelsGroup.get(selectedVoxels.indexOf(aux)) != this.defaultSelectionGroup) {
                selectedVoxelsGroup.set(selectedVoxels.indexOf(aux), defaultSelectionGroup);
            }
        }
        if (defaultSelectionGroup == SelectionGroup.BLUE) {
            defaultSelectionGroup = SelectionGroup.RED;
        } else {
            defaultSelectionGroup = SelectionGroup.BLUE;
        }
        this.updateViewSelectionChange();
    }

    public void setDisplacementField(String file, boolean update) {
        DisplacementField dispF = new DisplacementField(file);
        this.imageStack.setDisplacements(dispF);
        if (csigrid != null) {
            this.csigrid.applyDisplacements(update);
            if (update) {
                this.updateViewImagePanels();
            }
        }
    }

    public void setAffineTransform(String file, boolean update) {
        try {
            AffineTransformation affT = new AffineTransformation(file);
            this.imageStack.setAffineTransform(affT);
            if (csigrid != null) {
                this.csigrid.applyAffine(update);
                if (update) {
                    this.updateViewImagePanels();
                }
            }
        } catch (Exception e) {

        }
    }
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

//   IMAGE STACK METHODS
    public void setBlankImageStack() {
        imageStack.clear();
        blankImage = true;
        if (csigrid != null) {
            Point3D fovSpectroscopy = csigrid.getFOV();
            Point3D topLeftImg;
            Point3D centerGrid = csigrid.getCenterPoint().Clone();

            Point3D e1 = csigrid.getDir1().Clone().normalize();
            Point3D e2 = csigrid.getDir2().Clone().normalize();
            Point3D e3 = csigrid.getDir3().Clone();
            int fovImg = Math.max((int) fovSpectroscopy.m_x1 * 2, (int) fovSpectroscopy.m_x2 * 2);
            topLeftImg = centerGrid.subtractPoint(e1.scalarMultiplication(fovImg / 2))
                    .subtractPoint(e2.scalarMultiplication(fovImg / 2))
                    .subtractPoint(e3.scalarMultiplication(fovSpectroscopy.m_x3 / (2 * e3.length())));
            for (int i = 0; i < csidata.nrVoxelsDir3; i++) {
                if (i == 0) {
                    topLeftImg = topLeftImg.addPoint(e3.scalarMultiplication(0.5d));
                } else {
                    topLeftImg = topLeftImg.addPoint(e3.scalarMultiplication(1d));
                }
                imageStack.add(new ImageSlice(e1, e2, topLeftImg, fovImg, fovImg, 1f, 1f));
            }
            removeLocalizers();

            this.addLocalizerPanel(this.getBlankLocalizerTransversal());
            this.addLocalizerPanel(this.getBlankLocalizerCoronal());
            this.addLocalizerPanel(this.getBlankLocalizerSagital());

            calcLocalizersIntersections();
            this.calcGridIntersections();
        } else {
            imageStack.add(new ImageSlice(new Point3D(1, 0, 0), new Point3D(0, 1, 0),
                    new Point3D(),
                    1, 1, 1, 1));
            for (LocalizerPanel loc : this.localizerList) {
                loc.setImageInfo(new ImageSlice(new Point3D(1, 0, 0), new Point3D(0, 1, 0),
                        new Point3D(),
                        1, 1, 1, 1));
            }
            removeLocalizers();
        }

        if (sliceSlider != null) {
            this.sliceSlider.setMinimum(0);
            this.sliceSlider.setMaximum(imageStack.size() - 1);
            this.sliceSlider.setValue(0);
        }
        this.updateViewImagePanels();
    }

    public LocalizerPanel getBlankLocalizerTransversal() {
        Point3D fovSpectroscopy = csigrid.getFOV();
        Point3D centerGrid = csigrid.getCenterPoint().Clone();
        Point3D e1 = csigrid.getDir1().Clone().normalize();
        Point3D e2 = csigrid.getDir2().Clone().normalize();
        int fovImg = Math.max((int) fovSpectroscopy.m_x1 * 2, (int) fovSpectroscopy.m_x2 * 2);
        Point3D topLeftImg = centerGrid.subtractPoint(e1.scalarMultiplication(fovImg / 2))
                .subtractPoint(e2.scalarMultiplication(fovImg / 2));
        LocalizerPanel loc = new LocalizerPanel();
        loc.setImageInfo(new ImageSlice(e1, e2, topLeftImg, fovImg, fovImg, 1f, 1f));
        return loc;
    }

    public LocalizerPanel getBlankLocalizerCoronal() {
        Point3D fovSpectroscopy = csigrid.getFOV();
        Point3D centerGrid = csigrid.getCenterPoint().Clone();
        Point3D e1 = csigrid.getDir1().Clone().normalize();
        Point3D e3 = csigrid.getDir3().Clone().normalize().scalarMultiplication(-1d);
        int fovImg = Math.max((int) fovSpectroscopy.m_x1 * 2, (int) fovSpectroscopy.m_x3 * 2);
        Point3D topLeftImg = centerGrid.subtractPoint(e1.scalarMultiplication(fovImg / 2))
                .subtractPoint(e3.scalarMultiplication(fovImg / 2));
        LocalizerPanel loc = new LocalizerPanel();
        loc.setImageInfo(new ImageSlice(e1, e3, topLeftImg, fovImg, fovImg, 1f, 1f));
        return loc;
    }

    public LocalizerPanel getBlankLocalizerSagital() {
        Point3D fovSpectroscopy = csigrid.getFOV();
        Point3D centerGrid = csigrid.getCenterPoint().Clone();
        Point3D e2 = csigrid.getDir2().Clone().normalize();
        Point3D e3 = csigrid.getDir3().Clone().normalize().scalarMultiplication(-1d);
        int fovImg = Math.max((int) fovSpectroscopy.m_x2 * 2, (int) fovSpectroscopy.m_x3 * 2);
        Point3D topLeftImg = centerGrid.subtractPoint(e2.scalarMultiplication(fovImg / 2))
                .subtractPoint(e3.scalarMultiplication(fovImg / 2));
        LocalizerPanel loc = new LocalizerPanel();
        loc.setImageInfo(new ImageSlice(e2, e3, topLeftImg, fovImg, fovImg, 1f, 1f));
        return loc;
    }

    public void clearImageStack() {
        if (viewerpanel != null) {
            viewerpanel.setImageInfo("");
            viewerpanel.setPatientName("");
        }
        this.imageStack.clear();
        if (csigrid != null) {
            this.setBlankImageStack();
        }
    }

    public ImageStack getImageStack() {
        return this.imageStack;
    }

    public int getImageStackSize() {
        return this.imageStack.size();
    }

    public ImageSlice getCurrentImage() {
        if (imageStack != null) {
            if (slice >= imageStack.size()) {
                slice = imageStack.size() - 1;
            } else if (slice < 0) {
                slice = 0;
            }
            return this.imageStack.get(slice);
        } else {
            return null;
        }
    }

    public ImageSlice getImage(int i) {
        if (imageStack != null) {
            if (imageStack.size() > i) {
                return this.imageStack.get(i);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    public void updateImageStack(ArrayList<WritableRaster> rasters) {
        System.out.println("update image stack");
        if (rasters.size() == imageStack.size()) {
            for (int i = 0; i < imageStack.size(); i++) {
                imageStack.get(i).image = rasters.get(i);
            }
        }
        this.updateViewImagePanels();
    }

    public void normalizeImageStack() {
        if (imageStack != null) {
            this.setImageStack(ImageTools.normalizeImageStack(imageStack, 64, 0.01));
            this.viewerpanel.autoWindowing();
            this.updateViewImagePanels();
        }
    }
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

//   SPECTRA METHODS
    public double[] getMeanPhase() {
        if (spectra != null) {
            return this.spectra.getMeanPhase(selectedVoxels);
        } else {
            return new double[]{0, 0};
        }
    }

    public double[] getStdDevPhase() {
        if (spectra != null) {
            return this.spectra.getStdDevPhase(selectedVoxels);
        } else {
            return new double[]{0, 0};
        }
    }

    public void addPhase(double[] phase, boolean... update) {
        if (spectra != null) {
            this.spectra.addPhase(selectedVoxels, phase);
            if (spectraFitted != null) {
                this.spectraFitted.addPhase(selectedVoxels, phase);
            }
        }
        if (update.length > 0 && update[0]) {
            updateSpectraPanel();
        }
    }

    public void setPhase(double[] phase, boolean... update) {
        if (spectra != null) {
            this.spectra.setPhase(selectedVoxels, phase);
            if (spectraFitted != null) {
                this.spectraFitted.setPhase(selectedVoxels, phase);
            }
        }
        if (update.length > 0 && update[0]) {
            updateSpectraPanel();
        }
    }

    public void setPhase(double[] phase, String id, boolean... update) {
        if (spectra != null) {
            this.spectra.setPhase(id, phase);
            if (spectraFitted != null) {
                this.spectraFitted.setPhase(id, phase);
            }
        }
        if (update.length > 0 && update[0]) {
            updateSpectraPanel();
        }
    }

    public void setPhase(double[] phase, int i, int j, int k, boolean... update) {
        if (spectra != null) {
            this.spectra.setPhase(phase, i, j, k);
            if (spectraFitted != null) {
                this.spectraFitted.setPhase(phase, i, j, k);
            }
        }
        if (update.length > 0 && update[0]) {
            updateSpectraPanel();
        }
    }

    public double[] getxAxis() {
        return spectra.getXAxis();
    }

    public ArrayList<float[]> getSpectrumData(Spectra.Mode mode, boolean fit) {
        if (fit) {
            return spectraFitted.getSpectrumData(selectedVoxels, mode);
        } else {
            return spectra.getSpectrumData(selectedVoxels, mode);
        }
    }

    public ArrayList<float[]> getSpectrumData(Spectra.Mode mode, boolean fit, SelectionGroup group) {
        if (fit) {
            return spectraFitted.getSpectrumData(getSelectedVoxels(group), mode);
        } else {
            return spectra.getSpectrumData(getSelectedVoxels(group), mode);
        }
    }

    public HashMap<String, float[]> getDiffObsFitt(ArrayList<String> v) {
        return spectra.getDiff(spectraFitted, v);
    }

    public HashMap<String, float[]> getDiffObsFitt(ArrayList<String> v, double minPPM, double maxPPM) {
        return spectra.getDiff(spectraFitted, v, minPPM, maxPPM);
    }

    public HashMap<String, Float> getNoiseStdMap(ArrayList<String> v) {
        return spectra.getNoiseStdMap(v);
    }

    public void zeroFilling(int newsize, boolean update) {
        if (spectra != null) {
            spectra.zerofilling(newsize - spectra.getVectorSize());
            if (update) {
                updateSpectraPanel();
            }
        }
    }

    public void truncateTDModel( int new_tds_data_size, boolean update ) 
    {
        
        if ( spectra != null ) 
        {
            spectra.truncate_td_signal( new_tds_data_size );
            if ( update ) 
            {
                updateSpectraPanel( );
            }
        }
    }
    
    public int getVectorSize( ) 
    {
        if ( spectra != null )
        {
            return spectra.getVectorSize( );
        }
        else
        {
            return 1024;
        }
    }
    
    public void setVectorSize( int vectorSize ) 
    {
        spectra.setVectorSize( vectorSize );
    }
    
    public int getZeroFillingSize( ) 
    {
        return spectra.getZeroFillingSize( );
    }

    public void normalizeSpectra( boolean val, boolean update ) 
    {
        if ( spectra != null ) 
        {
            isSpectraNormalized = val;
            if ( update ) 
            {
                updateSpectraPanel( );
            }
        }
    }

    public void autoPhase(float minppm, float maxppm, boolean update) {
        spectra.autoPhase(minppm, maxppm);
        if (this.spectraFitted != null) {
            spectraFitted.setPhase(spectra.getPhase());
        }
        if (update) {
            updateSpectraPanel();
        }
    }

    public void autoPhaseSelection(float minppm, float maxppm, boolean update) {
        spectra.autoPhase(selectedVoxels, minppm, maxppm);
        if (this.spectraFitted != null) {
            spectraFitted.setPhase(spectra.getPhase(this.selectedVoxels));
        }
        if (update) {
            updateSpectraPanel();
        }
    }

    
    /**
     * This routine applies the zero and first order phasing to be applied to
     * the selected signals.
     * @param zeroOrder
     * @param firstOrder
     * @param update 
     */
    public void manualPhaseSelection( float zeroOrder, float firstOrder, boolean update) {
        
        spectra.manualPhase( selectedVoxels, zeroOrder, firstOrder );
        
        if ( this.spectraFitted != null ) 
        {
            spectraFitted.setPhase( spectra.getPhase( this.selectedVoxels ) );
        }
        if ( update ) 
        {
            updateSpectraPanel( );
        }
    }
    
    
    public void apodizeExponential(double a, boolean update) {
        spectra.apodizeExponential(a);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void apodizeExponentialSelection(double a, boolean update) {
        spectra.apodizeExponential(selectedVoxels, a);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void apodizeGaussian(double a, boolean update) {
        spectra.apodizeGaussian(a);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void apodizeGaussianSelection(double a, boolean update) {
        spectra.apodizeGaussian(selectedVoxels, a);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void calcPPMshift(double[] minvals, double[] maxvals, double[] targets, int[] multiplicities, double SNRLimit, double minPPM, double maxPPM, double maxLocalShift, boolean update) {
        spectra.calcPPMShift(minvals, maxvals, targets, multiplicities, SNRLimit, minPPM, maxPPM, maxLocalShift);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void calcPPMshiftSelection(double[] minvals, double[] maxvals, double[] targets, int[] multiplicities, double SNRLimit, double minPPM, double maxPPM, double maxLocalShift, boolean update) {
        spectra.calcPPMShift(new HashSet<String>(selectedVoxels), minvals, maxvals, targets, multiplicities, SNRLimit, minPPM, maxPPM, maxLocalShift);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void resetShifts(boolean update) {
        spectra.resetShift();
        if (update) {
            updateSpectraPanel();
        }
    }

    public void resetShiftsSelection(boolean update) {
        spectra.resetShift(selectedVoxels);
        if (update) {
            updateSpectraPanel();
        }
    }

    public void saveSelectedSpectraToMAT(String folder) {
        spectra.saveSpectraToMAT(selectedVoxels, folder);
    }

    public void saveAllSpectraToMAT(String folder) {
        spectra.saveSpectraToMAT(folder);
    }

    public void savePeakAnalysisAll(String folder) {
        spectra.savePeakAnalysis(folder);
    }

    public void savePeakAnalysisSelection(String folder) {
        spectra.savePeakAnalysis(folder, this.selectedVoxels);
    }

    public void printPeakAnalysisAll() {
        spectra.printPeakAnalysis();
    }

    public void printPeakAnalysisSelection() {
        spectra.printPeakAnalysis(this.selectedVoxels);
    }

//    public String addLorentzian(double x0, double y0, double w, double A) {
//        double[] xaxis = spectra.getXAxis();
//        float[] linepoints = new float[xaxis.length];
//        for (int i = 0; i < xaxis.length; i++) {
//            linepoints[i] = (float) (y0 + 2 * A * w / (Math.PI * (4 * Math.pow(xaxis[i] - x0, 2) + Math.pow(w, 2))));
//        }
//        return spectrapanel.addLine(linepoints, "Lorentzian");
//    }
//
//    public void updateLorentzian(String id, double x0, double y0, double w, double A) {
//        double[] xaxis = spectra.getAxisPPM();
//        float[] linepoints = new float[xaxis.length];
//        for (int i = 0; i < xaxis.length; i++) {
//            linepoints[i] = (float) (y0 + 2 * A * w / (Math.PI * (4 * Math.pow(xaxis[i] - x0, 2) + Math.pow(w, 2))));
//        }
//        spectrapanel.updateLine(linepoints, id);
//    }
    public void removeLine(String id) {
        spectrapanel.removeLine(id);
    }

//    public void fitExponential() {
//        if (spectra != null) {
//            Exponential curve = new Exponential(0, 0, 0);
//            int lengthfit = spectra.getVectorSize() / 4;
//            float[] subspectra = new float[lengthfit];
//            double[] subaxis = new double[lengthfit];
//            float[] abstd = getMagnitude();
//            double[] axis = spectra.getAxisTD();
//            for (int i = 0; i < lengthfit; i++) {
//                subspectra[i] = abstd[i];
//                subaxis[i] = axis[i];
//            }
//            ExponentialFit fit = new ExponentialFit(curve, subspectra, subaxis);
//            float valA = (abstd[0] + abstd[1] + abstd[2] + abstd[3] + abstd[4] + abstd[5] + abstd[6] + abstd[7] + abstd[8] + abstd[9]) / 10;
//            fit.setStartParameters(new double[]{0, 0.01, valA});
//            DoubleRange[] ranges = new DoubleRange[3];
////        ranges[0] = new DoubleRange(-spectra.getAxisTD()[spectra.getVectorSize()/10], spectra.getAxisTD()[spectra.getVectorSize()/10]);
//            ranges[0] = new DoubleRange(0, 0);
//            ranges[1] = new DoubleRange(0.001, 1);
//            ranges[2] = new DoubleRange(valA / 10, valA * 10);
//            fit.setParameterRanges(ranges);
//            fit.performRandomMinimization(32, 25, true);
//            double par[] = fit.getMinimizingParameters();
//            curve.setParameters(par[0], par[1], par[2]);
//            spectrapanel.addLine(curve.getY(spectra.getAxisTD()), "Exponential - T2=" + par[1]);
//            printTxt("\n" + ">> Exponential fit parameters (A*exp(-(t-t0)/w))");
//            printTxt("A\tw\tError");
//            String outputtxt = par[2] + "\t" + par[1] + "\t" + fit.getMinimumFunctionValue();
//            printNumeric(outputtxt, true);
//        }
//    }
//
//    public void fitLorentzian(int minx, int maxx) {
//        float[] subAbs = Arrays.copyOfRange(getAbsorption(), minx, maxx);
//        float max = 0;
//        for (int i = 0; i < subAbs.length; i++) {
//            if (subAbs[i] > max) {
//                max = subAbs[i];
//            }
//        }
//        Lorentzian curve = new Lorentzian(0, 0, 0, 0);
//        LorentzianFit fit = new LorentzianFit(curve, subAbs, Arrays.copyOfRange(getxAxis(), minx, maxx));
//
//        fit.setStartParameters(new double[]{(spectra.getAxisPPM(minx) + spectra.getAxisPPM(maxx)) / 2, 0, 0.1, 0.1 * Math.PI * 0.5 * max});
//        DoubleRange[] ranges = new DoubleRange[4];
//        ranges[0] = new DoubleRange(spectra.getAxisPPM(minx), spectra.getAxisPPM(maxx));
//        ranges[1] = new DoubleRange(0, 0);
//        ranges[2] = new DoubleRange(0.001, 1);
//        ranges[3] = new DoubleRange(0.001 * Math.PI * 0.5 * max, Math.PI * 0.5 * max);
//        fit.setParameterRanges(ranges);
//        fit.performRandomMinimization(20, 20, true);
//        double par[] = fit.getMinimizingParameters();
//        curve.setParameters(par[0], par[1], par[2], par[3]);
//        spectrapanel.addLine(curve.getY(spectra.getAxisPPM()), "Lorentzian");
//    }
    public void addLorentzianfit() {
        if (spectrapanel != null) {
            spectrapanel.listenLimitsSingle(SpectraPanel.ADD_LORENTZIAN);
        }
    }

    public void setSpectrum(float[] absFID, float[] dispFID, int i, int j, int k) {
        if (spectra != null) {
            spectra.setSpectrum(absFID, dispFID, i + "_" + j + "_" + k);
        }
    }

    public void setSpectrum(float[] absFID, float[] dispFID, String id) {
        if (spectra != null) {
            spectra.setSpectrum(absFID, dispFID, id);
        }
    }

    public double getResonanceFrequency() {
        return spectra.getResonanceFrequency();
    }

    public double getPPMReference() {
        return spectra.getPPMReference();
    }

    public double getTDstep() {
        return spectra.getDwellTime();
    }

    public double getFDstep() {
        return spectra.getFDstep();
    }

    public double[] getPhase(int i, int j, int k) {
        return spectra.getPhase(i, j, k);
    }

    public SpectraPanel getSpectraPanel() {
        return spectrapanel;
    }

    public Spectra getSpectra() {
        return spectra;
    }

    public void setFittedSpectra(FittedSpectra spectra) {
        spectraFitted = spectra;
        if (spectraFitted != null) {
            spectraFitted.setModel(this);
        }
    }

    public boolean hasFittedSpectra() {
        return (spectraFitted != null);
    }

    public FittedSpectra getFittedSpectra() {
        return spectraFitted;
    }

    public void setFittedSpectraVisible(boolean bool) {
        this.showFittedData = bool;
        this.updateSpectraPanel();
    }

    public void setOriginalSpectraVisible(boolean bool) {
        this.showOriginalData = bool;
        this.updateSpectraPanel();
    }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
//   COLOR MAP METHODS    
    public ColorMap getColorMap() {
        return colormap;
    }

    public void setColorMapUI(ColorMapUI obj) {
        this.colormapUI = obj;
    }

    public void setColorMapUIRangesTxt(double min1, double max1, double min2, double max2) {
        if (this.colormapUI != null) {
            this.colormapUI.setRangesText(min1, max1, min2, max2);
        }
    }

    public void setColorMapUIRangesTxt(double min1, double max1) {
        if (this.colormapUI != null) {
            this.colormapUI.setRangesText(min1, max1);
        }
    }

    public void initiate1RangeSelection() {
        if (csigrid != null) {
            spectrapanel.listenLimitsSingle(SpectraPanel.RANGES_1);
        }
    }

    public void initiate2RangesSelection() {
        if (csigrid != null) {
            spectrapanel.listenLimitsDouble(SpectraPanel.RANGES_2);
        }
    }

    public void addDomainMarker(double pos, int color) {
        this.spectrapanel.addDomainMarker(pos, color);
    }

    public void createColoredMap(int type, double... limits) {
        if (colormap != null) {
            colormap.createMap(type, limits);
        }
    }

    public boolean isColorMapActive() {
        if (colormap != null) {
            return colormap.isActive();
        } else {
            return false;
        }
    }

    public void setColorBarVisible(boolean val) {
        if (colormap != null) {
            colormap.setColorBarVisible(val);
        }
    }

    public boolean isColorBarVisible() {
        if (colormap != null) {
            return colormap.isColorBarVisible();
        } else {
            return false;
        }
    }

    public void clearColorMap() {
        if (colormap != null) {
            colormap.clear();
        }
        if (spectrapanel != null) {
            spectrapanel.clearDomainMarkers();
        }
        if (viewerpanel != null) {
            viewerpanel.repaint();
        }
    }

    public void clearDomainMarkers() {
        spectrapanel.clearDomainMarkers();
    }

    public void setValuesVisible(boolean bool) {
        this.showMapValues = bool;
        viewerpanel.repaint();
    }

    public boolean isValuesVisible() {
        return showMapValues;
    }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
//   LOCALIZER METHODS
    public ArrayList<IntersectionLine> getLocalizersIntersections() {
        return localizerIntersectionLines;
    }

    public void setViewLocalizers() {
        for (int i = 0; i < this.localizerList.size(); i++) {
            this.localizerList.get(i).setView();
        }
    }

    public void clearLocalizerGridProjections() {
        for (int i = 0; i < this.localizerList.size(); i++) {
            this.localizerList.get(i).clearGridProjections();
        }
    }

    public void repaintLocalizers() {
        for (int i = 0; i < this.localizerList.size(); i++) {
            this.localizerList.get(i).repaint();
        }
    }

    public void setLocalizersVisible(boolean bool) {
        for (int i = 0; i < this.localizerList.size(); i++) {
            this.localizerList.get(i).setVisible(bool);
        }
    }

    public void autoWindowingLocalizers() {
        for (int i = 0; i < this.localizerList.size(); i++) {
            this.localizerList.get(i).autoWindowing();
        }
    }

    public void setLocalizersColorMode(int mode) {
        for (int i = 0; i < this.localizerList.size(); i++) {
            this.localizerList.get(i).setColorMode(mode);
        }
    }

    public void calcLocalizersIntersections() {
        localizerIntersectionLines.clear();
        double[] aux;
        ImagePlane plane;
        IntersectionLine line;
        for (int i = 0; i < this.localizerList.size(); i++) {
            plane = this.localizerList.get(i).getImagePlane();
            aux = getIntersection(plane.p, plane.v1, plane.v2);
            line = new IntersectionLine(this.localizerList.get(i).getColor(), (int) Math.round(aux[0]), (int) Math.round(aux[1]), (int) Math.round(aux[2]), (int) Math.round(aux[3]));
            localizerIntersectionLines.add(line);
        }
    }

    public double[] getIntersection(Point3D sliceP, Point3D sliceV1, Point3D sliceV2) {
        ArrayList<Point3D> intersections = new ArrayList<>();
        ImageSlice imageInfo = getCurrentImage();
        Point3D p1 = imageInfo.imagePosition;
        Point3D p2 = imageInfo.imageOrientationX.scalarMultiplication(imageInfo.sizeX * imageInfo.pixelSpacingX).translate(p1);
        Point3D p3 = imageInfo.imageOrientationY.scalarMultiplication(imageInfo.sizeY * imageInfo.pixelSpacingY).translate(p1);
        Point3D p4 = p3.Clone().translate(p2.subtractPoint(p1));
        PlaneLineIntersection(p1, p2, intersections, sliceP, sliceV1, sliceV2);
        PlaneLineIntersection(p2, p4, intersections, sliceP, sliceV1, sliceV2);
        PlaneLineIntersection(p4, p3, intersections, sliceP, sliceV1, sliceV2);
        PlaneLineIntersection(p3, p1, intersections, sliceP, sliceV1, sliceV2);

        double[] point2Daux1;
        double[] point2Daux2;
        double[] res = new double[4];
        if (intersections.size() == 2) {
            point2Daux1 = coordTransform(intersections.get(0), imageInfo.imagePosition, imageInfo.imageOrientationX, imageInfo.imageOrientationY);
            point2Daux2 = coordTransform(intersections.get(1), imageInfo.imagePosition, imageInfo.imageOrientationX, imageInfo.imageOrientationY);
            res[0] = point2Daux1[0];
            res[1] = point2Daux1[1];
            res[2] = point2Daux2[0];
            res[3] = point2Daux2[1];
        }
        return res;
    }

    public double[] coordTransform(Point3D c1, Point3D p, Point3D r1, Point3D r2) {
        ImageSlice imageInfo = getCurrentImage();
        Point3D point = new Point3D();
        point.m_x1 = (c1.m_x1 - p.m_x1);
        point.m_x2 = (c1.m_x2 - p.m_x2);
        point.m_x3 = (c1.m_x3 - p.m_x3);
        double[][] matrixData = {
            {r1.m_x1, r2.m_x1},
            {r1.m_x2, r2.m_x2},
            {r1.m_x3, r2.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{point.m_x1, point.m_x2, point.m_x3}, false);
        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new QRDecomposition(n).getSolver();
        RealVector solution = solver.solve(constants);
        double[] res = new double[2];
        res[0] = solution.getEntry(0) * viewerpanel.getVirtualSize() / (imageInfo.sizeX * imageInfo.pixelSpacingX);
        res[1] = solution.getEntry(1) * viewerpanel.getVirtualSize() / (imageInfo.sizeY * imageInfo.pixelSpacingY);
        return res;
    }

    //Intersection of segment AB with the Plane defined in the class constructor 
    public void PlaneLineIntersection(Point3D pA, Point3D pB, ArrayList<Point3D> intersections, Point3D p, Point3D r1, Point3D r2) {
        Point3D res = new Point3D();
        double[][] matrixData = {
            {r1.m_x1, r2.m_x1, pA.m_x1 - pB.m_x1},
            {r1.m_x2, r2.m_x2, pA.m_x2 - pB.m_x2},
            {r1.m_x3, r2.m_x3, pA.m_x3 - pB.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{pA.m_x1 - p.m_x1, pA.m_x2 - p.m_x2, pA.m_x3 - p.m_x3}, false);

        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new LUDecomposition(n).getSolver();

        try {
            RealVector solution = solver.solve(constants);
            double[] sol;
            sol = solution.toArray();
            if (sol[2] >= 0d && sol[2] <= 1d) {
                res.m_x1 = pA.m_x1 + sol[2] * (pB.m_x1 - pA.m_x1);
                res.m_x2 = pA.m_x2 + sol[2] * (pB.m_x2 - pA.m_x2);
                res.m_x3 = pA.m_x3 + sol[2] * (pB.m_x3 - pA.m_x3);
                intersections.add(res);
            }
        } catch (Exception ex) {
        }
    }

    public ArrayList<double[]> getGridProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY) {
        if (csigrid != null) {
            return csigrid.getGridProjection(p, r1, r2, sizeX, sizeY, pixelSpacingX, pixelSpacingY);
        } else {
            return new ArrayList<>();
        }
    }

    public ArrayList<double[]> getSelectedVoxelProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY, SelectionGroup group) {
        if (csigrid != null) {
            return csigrid.getSelectedVoxelProjection(p, r1, r2, sizeX, sizeY, pixelSpacingX, pixelSpacingY, group);
        } else {
            return new ArrayList<>();
        }
    }

    public ArrayList<double[]> getExcitationVoxelProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY) {
        if (csigrid != null) {
            return csigrid.getExcitationVoxelProjection(p, r1, r2, sizeX, sizeY, pixelSpacingX, pixelSpacingY);
        } else {
            return new ArrayList<>();
        }
    }

    public ArrayList<double[]> getExcitationVoxelProjection() {
        if (csigrid != null) {
            ImageSlice currIm = getCurrentImage();
            return csigrid.getExcitationVoxelProjection(currIm.imagePosition,
                    currIm.imageOrientationX,
                    currIm.imageOrientationY,
                    currIm.sizeX,
                    currIm.sizeY,
                    currIm.pixelSpacingX,
                    currIm.pixelSpacingY);
        } else {
            ArrayList<double[]> res = new ArrayList<>();
            return res;
        }
    }

    public ArrayList<double[]> getGridLinerProjection() {
        if (csigrid != null) {
            ImageSlice currIm = getCurrentImage();
            return csigrid.getGridLinerProjection(currIm.imagePosition,
                    currIm.imageOrientationX,
                    currIm.imageOrientationY,
                    currIm.sizeX,
                    currIm.sizeY,
                    currIm.pixelSpacingX,
                    currIm.pixelSpacingY);
        } else {
            ArrayList<double[]> res = new ArrayList<>();
            return res;
        }
    }

    public ArrayList<double[]> getGridLinerProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY) {
        if (csigrid != null) {
            return csigrid.getGridLinerProjection(p, r1, r2, sizeX, sizeY, pixelSpacingX, pixelSpacingY);
        } else {
            ArrayList<double[]> res = new ArrayList<>();
            return res;
        }
    }

    public void showIntersectionsInMain(boolean bool) {
        this.viewerpanel.setIntersectionsVisible(bool);
    }
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

//   OVERLAYING IMAGE METHODS
    public void setOverlayStack(OverlayStack imStack) {
        this.overlayStack = imStack;
    }

    public boolean hasOverlay() {
        return overlayStack.size() > 0;
    }

    public double getOverlayMax() {
        return overlayStack.getMaxIntensity()[0];
    }

    public double getOverlayMin() {
        return overlayStack.getMinIntensity()[0];
    }

    public void setOverlayThresholdMin(float val, boolean update) {
        this.overlayStack.setMinThreshold(val, update);
        if (update) {
            this.updateViewImagePanels();
        }
    }

    public void setOverlayThresholdMax(float val, boolean update) {
        this.overlayStack.setMaxThreshold(val, update);
        if (update) {
            this.updateViewImagePanels();
        }
    }

    public OverlaySlice getOverlaySlice(int nr) {
        return overlayStack.get(nr);
    }

    public OverlaySlice getCurrentOverlay() {
        return overlayStack.get(slice);
    }
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

//   RERENCE SPECTRA (NWS) METHODS
    public void setReferenceSpectra(Spectra ref) {
        int[] gridsize = spectra.getGridSize();
        int[] gridsizeRef = ref.getGridSize();

        //Include another checks like localization...
        if (gridsize[0] == gridsizeRef[0] && gridsize[1] == gridsizeRef[1] && gridsize[2] == gridsizeRef[2]) {
            this.spectra.setReference(ref);
            updateSpectraPanel();
        } else {
            printTxt("The spectra provided as reference has different dimensions than the one in focus!");
        }
    }
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

//   PLOTFRAME METHODS    
    public void openPlotFrame(String title) {
        if (plotframe != null) {
            plotframe.dispose();
        }
        this.plotframe = new XYPlotFrame(title, null);
        plotframe.getPlotArea().forcedPlotRanges(500, 2500);
    }

    public void addCurve(float[] x, float[] y) {
        Curve curve = new Curve(x, y);
        plotframe.addCurve(curve, true);
    }

    public void addCurve(float[] x, float[] y, Color color) {
        Curve curve = new Curve(x, y, color);
        plotframe.addCurve(curve, true);
    }

    public void addCurve(double[] x, double[] y) {
        Curve curve = new Curve(x, y);
        plotframe.addCurve(curve, true);
    }

    public void addCurve(double[] x, double[] y, Color color) {
        Curve curve = new Curve(x, y, color);
        plotframe.addCurve(curve, true);
    }

    public void addCurve(float[] x, float[] y, String label) {
        Curve curve = new Curve(x, y, label);
        plotframe.addCurve(curve, true);
    }

    public void addCurve(float[] x, float[] y, Color color, String label) {
        Curve curve = new Curve(x, y, color, label);
        plotframe.addCurve(curve, true);
    }

    public void addCurve(double[] x, double[] y, Color color, String label) {
        Curve curve = new Curve(x, y, color, label);
        plotframe.addCurve(curve, true);
    }

    public void clearPlotFrame() {
        plotframe.deleteAllCurves();
    }
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

//   CLASSIFY SPECTRA
    public void classifySpectralQuality(String file, boolean mapResults) {
        QualityClassifier classifier = new QualityClassifier(this);
        if (this.getGridSize() == 1) {
            selectVoxel(0, 0, 0, SELECTION_SINGLE);
        }
        classifier.setVoxels(selectedVoxels);
        int nrSelected = selectedVoxels.size();
        classifier.setClassifier(file);
        classifier.execute();
        spectra.setQualityScores(classifier.getQualityScores());

        double meanQScore = 0;
        double stdQScore = 0;
        Iterator<String> iter = classifier.getQualityScores().keySet().iterator();
        while (iter.hasNext()) {
            meanQScore += classifier.getQualityScores().get(iter.next());
        }
        meanQScore = meanQScore / classifier.getQualityScores().size();
        iter = classifier.getQualityScores().keySet().iterator();
        while (iter.hasNext()) {
            stdQScore += Math.pow(classifier.getQualityScores().get(iter.next()) - meanQScore, 2);
        }
        stdQScore = Math.sqrt(stdQScore / classifier.getQualityScores().size());
        this.printTxt("Mean QScore\tStd.dev QScore\t#Selected\t#Good");
        int nrGood = 0;

        nrGood = this.spectra.getQualitySelection(0.5).size();
        this.printNumeric(meanQScore + "\t" + stdQScore + "\t" + nrSelected + "\t" + nrGood, true);
        if ( mapResults && nrSelected > 1) 
        {
            mapClassificationResults( );
        }
    }

    /**
     * Maps the values of data loaded from a CSV file ... 
     * @param csvFileValues 
     */
    public void mapExternalCSVFileNumericalValues( HashMap<String, Double> csvFileValues ) 
    {
        if ( spectrapanel != null ) 
        {
            m_isColoredOverlayDisplayOfCSVDataFileModeActive = false;
            if ( !csvFileValues.isEmpty( ) ) 
            {
                colormap.setColorMapValues( csvFileValues, false );
                // colormap.setLimits( 0, 1 );
            }
            viewerpanel.repaint( );
        }
    }
    
    /**
     * Maps the values of data loaded from a CSV file ... 
     * @param csvFileColorMapValues 
     */
    public void mapExternalCSVFileColorValues( HashMap<String, Color> csvFileColorMapValues ) 
    {  
        if ( spectrapanel != null ) 
        {
            if ( !csvFileColorMapValues.isEmpty( ) ) 
            {
                m_csvFileColorMapValues = csvFileColorMapValues;
                m_isColoredOverlayDisplayOfCSVDataFileModeActive = true;
            }
            viewerpanel.repaint( );
        }
    }
    
    public HashMap<String, Color> getExternalCSVColorOverlayMap( )
    {
        return m_csvFileColorMapValues;
    }
    
    public void mapClassificationResults( ) 
    {
        if ( spectrapanel != null ) 
        {
            if ( !spectra.getQualityScores().isEmpty( ) ) 
            {
                colormap.setColorMapValues( ( HashMap<String, Double> ) spectra.getQualityScores( ).clone( ), false );
                colormap.setLimits( 0, 1 );
            }
            this.viewerpanel.repaint( );
        }
    }

    public void selectGoodQualityVoxels(double val, boolean update) {
        this.selectedVoxels = spectra.getQualitySelection(val);
        selectedVoxelsGroup.clear();
        for (int i = 0; i < selectedVoxels.size(); i++) {
            this.selectedVoxelsGroup.add(this.defaultSelectionGroup);
        }
        if (update) {
            this.updateViewSelectionChange();
        }
    }

    public boolean isQScoresEmpty() {
        return spectra.isQScoresEmpty();
    }

    public boolean isBlueGroupSelected() {
        for (SelectionGroup grp : selectedVoxelsGroup) {
            if (grp == SelectionGroup.BLUE) {
                return true;
            }
        }
        return false;
    }

    public boolean isRedGroupSelected() {
        for (SelectionGroup grp : selectedVoxelsGroup) {
            if (grp == SelectionGroup.RED) {
                return true;
            }
        }
        return false;
    }

}
