/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.patientbrowser;

import java.util.regex.Pattern;
import org.dcm4che2.data.DicomObject;
import org.spectrim.io.dicom.DicomHandler;

/**
 *
 * @author nunopbarros
 */
public class ModalityIdentifier {

    public static final int UNKNOWN = 0;
    public static final int LOCALIZER = 1;
    public static final int TRUFI = 2;
    public static final int T1 = 3;
    public static final int T1C = 4;
    public static final int T2 = 5;
    public static final int FLAIR = 6;
    public static final int ADC = 7;
    public static final int DWI = 8;
    public static final int SWI = 9;
    public static final int SWI_MIP = 10;
    public static final int CSI_135 = 11;
    public static final int CSI_30 = 12;
    public static final int SVS_30 = 13;
    public static final int SVS_135 = 14;
    public static final int SVS_TCM_DUMMY = 15;
    public static final int SVS_TCM_30 = 16;
    public static final int SVS_TCM_40 = 17;
    public static final int SVS_TCM_50 = 18;
    public static final int SVS_TCM_80 = 19;
    public static final int SVS_TCM_100 = 20;
    public static final int SVS_TCM_200 = 21;
    public static final int SVS_TCM_500 = 22;
    public static final int SVS_TCM_1500 = 23;
    public static final int B1_MAP = 24; 
    public static final int VELOCITY_MAP = 25; 
    public static final int CSI = 26; 
    public static final int SURVEY = 27;
    public static final int SV = 28; 
    public static final int FFE = 29; 
    public static final int SI = 30 ; 
    public static final int MPR = 31; 
    public static final int PROJECTION_IMG = 32; 

    public static final String[] names = {
        "Unknown", //0
        "Localizer", //1
        "Trufi", //2
        "T1", //3
        "T1c", //4
        "T2", //5
        "FLAIR", //6
        "ADC", //7
        "DWI", //8
        "SWI", //9
        "SWI mIP", //10
        "CSI 135", //11
        "CSI 30", //12
        "SVS 30", //13
        "SVS 135", //14
        "SVS TCM DUMMY", //15
        "SVS TCM 30", //16
        "SVS TCM 40", //17
        "SVS TCM 50", //18
        "SVS TCM 80", //19
        "SVS TCM 100", //20
        "SVS TCM 200", //21
        "SVS TCM 500", //22
        "SVS TCM 1500",//23
        "T2_MAP",//24
        "VELOCITY_MAP", //25
        "CSI", //26
        "Survey",//27
        "SVS", //28
        "FLASH",//29
        "SI", //30
        "MPR",//31
        "Projection"}; //32


    public static String getModalityName(DicomObject object) {
        return names[getModality(object)];
    }

    public static String getModalityName(String protocol, String series, String contrast, String imageType) {
        return names[getModality(protocol, series, contrast, imageType)];
    }

    public static int getModality(DicomObject object) {
        String protocol = DicomHandler.getProtocolName(object);
        String series = DicomHandler.getSeriesDescription(object);
        String contrast = DicomHandler.getContrast(object);
        String imageType = DicomHandler.getImageType(object);
        return getModality(protocol, series, contrast, imageType);
    }

    public static int getModality(String protocol, String series, String contrast, String imageType) {
        if (contrast != null) {
            if (contrast.matches("N.A.")) {
                contrast = null;
            }
        }
        //MJ for Philips
        if(imageType.toLowerCase().contains("spectroscopy")){
            if(protocol.toLowerCase().contains("sv"))
                return SV; 
            else if(protocol.toLowerCase().contains("csi"))
                return CSI; 
            else if(protocol.toLowerCase().contains("2d"))
                return CSI; 
        }
             
        if(imageType.toLowerCase().contains("m\\ffe"))
            return FFE; 
        
        if(imageType.toLowerCase().contains("m_si"))
            return SI; 
        
        if(imageType.toLowerCase().contains("t2 map"))
            return B1_MAP; 
        
        if(imageType.toLowerCase().contains("velocity map"))
            return VELOCITY_MAP; 
        
        if(imageType.toLowerCase().contains("mpr"))
            return MPR; 
        
        if(imageType.toLowerCase().contains("PROJECTION IMAG"))
            return PROJECTION_IMG; 
        //MJ
        
        if (contains(protocol, "dti") || contains(protocol, "NoProtocolNameFound")) {
            return UNKNOWN;
        } else if (contains(protocol, "localizer")) {
            return LOCALIZER;
        } else if (contains(protocol, "trufi")) {
            return TRUFI;
        } else if (contains(protocol, "t1")) {
            if (contrast == null && !contains(protocol, "km") && !contains(protocol, "we")) {
                return T1;
            } else {
                return T1C;
            }
        } else if (contains(series, "SWI")) {
            return SWI;
        } else if (contains(protocol, "SWI") && contains(series, "mip")) {
            return SWI_MIP;
        } else if (contains(protocol, "t2")) {
            return T2;
        } else if (contains(protocol, "tirm") || contains(protocol, "dark_fluid")) {
            return FLAIR;
        } else if (contains(protocol, "3scan")) {
            if (contains(series, "adc")) {
                return ADC;
            } else {
                return DWI;
            }
        } else if (contains(protocol, "csi")) {
            if (contains(imageType, "secondary")) {
                return UNKNOWN;
            } else if (contains(protocol, "135")) {
                return CSI_135;
            } else if (contains(protocol, "30")) {
                return CSI_30;
            }
        } else if (contains(protocol, "svs")) {
            if (contains(imageType, "secondary")) {
                return UNKNOWN;
            } else if (contains(protocol, "tcm")) {
                if (contains(protocol, "1500")) {
                    return SVS_TCM_1500;
                } else if (contains(protocol, "500")) {
                    return SVS_TCM_500;
                } else if (contains(protocol, "50")) {
                    return SVS_TCM_50;
                } else if (contains(protocol, "200")) {
                    return SVS_TCM_200;
                } else if (contains(protocol, "100")) {
                    return SVS_TCM_100;
                } else if (contains(protocol, "80")) {
                    return SVS_TCM_80;
                } else if (contains(protocol, "40")) {
                    return SVS_TCM_40;
                } else if (contains(protocol, "30")) {
                    return SVS_TCM_30;
                } else if (contains(protocol, "dummy")) {
                    return SVS_TCM_DUMMY;
                }
            } else if (contains(protocol, "30")) {
                return SVS_30;
            } else if (contains(protocol, "135")) {
                return SVS_135;
            } else if (contains(protocol, "dummy")) {
                return SVS_TCM_DUMMY;
            }
        }
        return UNKNOWN;
    }

    public static int getModality(String modality) {
        for (int i = 0; i < names.length; i++) {
            if (names[i].matches(modality.trim())) {
                return i;
            }
        }
        return UNKNOWN;
    }

    private static boolean contains(String s1, String s2) {
        if (s1 == null) {
            return false;
        }
        return Pattern.compile(Pattern.quote(s2), Pattern.CASE_INSENSITIVE).matcher(s1).find();
    }
}
