/*
 * File: $Id: MRPatientSpectrumDataBaseItemViewer.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 08.02.2004, 21:37  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 -2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.patientdatabase;

import java.awt.Dimension;
import java.awt.BorderLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.LinkedHashMap;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.spectrim.dicom.FileInfo;

import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;

/**
 * Class the displays data base items and allows editing and storage of 
 * an MRPatientSpectrumDataBaseItem
 *
 * @author  hans
 */

public class MRPatientSpectrumDataBaseItemViewer extends JPanel implements ActionListener
{
    
    /** 
     * Creates a new instance of MRPatientDataBaseItemGUIControl 
     */
    public MRPatientSpectrumDataBaseItemViewer( )
    {
        m_contentPanel = new JPanel( );
        
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        constructPatientAttributesPanel( );
        constructPathologyPanel( );
        
        m_contentPanel.add( m_allPatientDataCollectionPanel, BorderLayout.NORTH );
        
        constructSpectrumAttributesPanel( );
        
        constructOptionsPanel();
        
        setLayout( new BorderLayout( ) );
        
        add( m_contentPanel, BorderLayout.CENTER );
        
    }
    
    
    /**
     * Contructs the panel containing all patient data
     **/
    private void  constructPatientAttributesPanel( )
    {
        m_allPatientDataCollectionPanel = new JPanel( );
        
        m_allPatientDataCollectionPanel.setLayout( new BoxLayout( m_allPatientDataCollectionPanel, BoxLayout.Y_AXIS ) );
        
        
        
        
        m_patientAttributesPanel = new JPanel( );
        m_patientAttributesPanel.setLayout( new BoxLayout( m_patientAttributesPanel, BoxLayout.Y_AXIS ) );
        
        m_patientAttributesPanel.setBorder( BorderFactory.createCompoundBorder( 
                                            BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                            BorderFactory.createTitledBorder( "Patient Information" ) ) );
        
        m_nameAndIDPanel = new JPanel( );
        
        m_nameAndIDPanel.setLayout( new BoxLayout( m_nameAndIDPanel, BoxLayout.X_AXIS ) );
        m_nameAndIDPanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        m_patientNameLabel = new JLabel( "Patient name " );
        m_patientNameTextField = new JTextField( );
        m_patientNameTextField.setPreferredSize( new Dimension( 250, 20 ) );
        m_patientNameTextField.setMinimumSize( new Dimension( 250, 20 ) );
        m_patientNameTextField.setMaximumSize( new Dimension( 250, 20 ) );
        
        m_patientIDLabel = new JLabel( "  Patient ID " );
        m_patientIDTextField = new JTextField( );
        m_patientIDTextField.setPreferredSize( new Dimension( 100, 20 ) );
        m_patientIDTextField.setMinimumSize( new Dimension( 100, 20 ) );
        m_patientIDTextField.setMaximumSize( new Dimension( 100, 20 ) );
        
        m_nameAndIDPanel.add( m_patientNameLabel );
        m_nameAndIDPanel.add( m_patientNameTextField );
        m_nameAndIDPanel.add( Box.createHorizontalGlue( ) );
        m_nameAndIDPanel.add( m_patientIDLabel );
        m_nameAndIDPanel.add( m_patientIDTextField );
        
        m_patientAttributesPanel.add( m_nameAndIDPanel );
        
        m_patientDOBAgeAndSexPanel = new JPanel( );
        
        m_patientDOBAgeAndSexPanel.setLayout( new BoxLayout( m_patientDOBAgeAndSexPanel, BoxLayout.X_AXIS ) );
        m_patientDOBAgeAndSexPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 2, 5 ) );
        m_sexLabel = new JLabel( "Sex " ); 
        
        m_patientDOBAgeAndSexPanel.add( m_sexLabel );
        
        m_patientDOBAgeAndSexPanel.add( Box.createHorizontalGlue( ) );
        
        m_male = new JRadioButton( "Male" );
        m_female = new JRadioButton( "Female" );
        m_other = new JRadioButton( "Other" );
        
        m_sexButtonGroup = new ButtonGroup( );
        
        m_sexButtonGroup.add( m_male );
        m_sexButtonGroup.add( m_female );
        m_sexButtonGroup.add( m_other );
        
        m_patientDOBAgeAndSexPanel.add( m_male );
        m_patientDOBAgeAndSexPanel.add( m_female );
        m_patientDOBAgeAndSexPanel.add( m_other );
        
        m_patientDOBAgeAndSexPanel.add( Box.createHorizontalGlue( ) );
        
        m_DOBLabel = new JLabel( "Birthdate " );
        m_DOBTextField = new JTextField( "--" );
        m_DOBTextField.setPreferredSize( new Dimension( 130, 20 ) );
        m_DOBTextField.setMinimumSize( new Dimension( 130, 20 ) );
        m_DOBTextField.setMaximumSize( new Dimension( 130, 20 ) );
        
        m_patientDOBAgeAndSexPanel.add( m_DOBLabel );
        m_patientDOBAgeAndSexPanel.add( m_DOBTextField );
        
        m_patientDOBAgeAndSexPanel.add( Box.createHorizontalGlue( ) );
        m_patientAgeLabel = new JLabel( "Age " );
        m_patientAgeTextField = new JTextField( "--" );
        
        m_patientAgeTextField.setPreferredSize( new Dimension( 50, 20 ) );
        m_patientAgeTextField.setMinimumSize( new Dimension( 50, 20 ) );
        m_patientAgeTextField.setMaximumSize( new Dimension( 50, 20 ) );
        
        m_patientDOBAgeAndSexPanel.add( m_patientAgeLabel );
        m_patientDOBAgeAndSexPanel.add( m_patientAgeTextField );
        
        m_patientAttributesPanel.add( m_patientDOBAgeAndSexPanel );
        
        
        m_weightAndPositionPanel = new JPanel( );
        m_weightAndPositionPanel.setLayout( new BoxLayout( m_weightAndPositionPanel, BoxLayout.X_AXIS ) );
        m_weightAndPositionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5) );
        
        m_weightAndPositionPanel.add( Box.createHorizontalGlue( ) );
        
        m_weightLabel = new JLabel( " Weight " );
        m_weightUnitLabel = new JLabel( " [kg]" );
        
        m_weightTextField = new JTextField( "--" ) ;
        
        m_weightTextField.setPreferredSize( new Dimension( 40, 20 ) );
        m_weightTextField.setMinimumSize( new Dimension( 40, 20 ) );
        m_weightTextField.setMaximumSize( new Dimension( 40, 20 ) );
        
        m_weightAndPositionPanel.add( m_weightLabel );
        m_weightAndPositionPanel.add( m_weightTextField );
        m_weightAndPositionPanel.add( m_weightUnitLabel );
        

        m_patientPositionLabel = new JLabel( " Patient position ");
        m_patientPostionTextField = new JTextField( "--" );
        
        m_patientPostionTextField.setPreferredSize( new Dimension( 40, 20 ) );
        m_patientPostionTextField.setMaximumSize( new Dimension( 40, 20 ) );
        m_patientPostionTextField.setMinimumSize( new Dimension( 40, 20 ) );
        
        m_weightAndPositionPanel.add( Box.createHorizontalGlue( ) );
        
        m_weightAndPositionPanel.add( m_patientPositionLabel );
        m_weightAndPositionPanel.add( m_patientPostionTextField );
        
        m_weightAndPositionPanel.add( Box.createHorizontalGlue( ) );     
        m_patientAttributesPanel.add(  m_weightAndPositionPanel );
        
        m_allPatientDataCollectionPanel.add( m_patientAttributesPanel ); 
    }
    
    /**
     * Sets the gui components with dicom file information retrieved from a
     * dicom file and creates a current object of MRPatientSpectrumDataBaseItem
     * that contains all information entered by the user afterwards
     */
    public void setPatientSpectrumDataBaseItem( SingleVoxelMeasuredSpectralData measuredPatientSpectrum )
    {
        FileInfo finfo = measuredPatientSpectrum.getDicomHeaderInfo( );
        
        MRPatientSpectrumDataBaseItem m_currentPatientDBItem = new MRPatientSpectrumDataBaseItem( );
        
        /* patient name */
        m_currentPatientDBItem.setPatientName( finfo.getPatientName( ) );
        
        /* patient ID */
        m_currentPatientDBItem.setPatientID( finfo.getPatientID( ) );
        
        /* birthdate */
        m_currentPatientDBItem.setPatientDayOfBirth( finfo.getPatientBirthDate( ) );
        
        /* sex */
        if ( finfo.getPatientSex( ) == "M" )
        {
            m_male.setSelected( true );
        }
        else if ( finfo.getPatientSex( ) == "F" )
        {
            m_female.setSelected( true );
        }
        else
        {
            m_other.setSelected( true );
        }
        
        m_currentPatientDBItem.setPatientSex( finfo.getPatientSex( ) );
        
        /* age */
        m_currentPatientDBItem.setPatientAge( finfo.getPatientAge( ) );
        
        /* weight */
        m_currentPatientDBItem.setPatientWeight( finfo.getPatientWeight( ) );
        
        /* patient position */
        m_currentPatientDBItem.setPatientPosition( finfo.getPatientPosition( ) );
    }
    

    
    
    
    /**
     * updateForm updates the form with info's from the Dicom header
     * Preventing the user lots of typing work. The info-fields are
     * also fields of the database that allow querying
     */
    public void updateForm( FileInfo fi,  LinkedHashMap ssd )
    {
        m_patientNameTextField.setText( fi.getPatientName( ) );
        
        m_DOBTextField.setText( fi.getPatientBirthDate( ) );
        
        if ( fi.getPatientSex( ).compareTo( "M " ) == 0 )
        {
            m_male.setSelected( true );
        }
        else if ( fi.getPatientSex( ).compareTo( "F " ) == 0 )
        {
            m_female.setSelected( true );
        }
        else
        {
            this.m_other.setSelected( true );
        }
        
        m_patientAgeTextField.setText( fi.getPatientAge( ) );
        
        m_patientIDTextField.setText( fi.getPatientID( ) );
        
        m_weightTextField.setText( fi.getPatientWeight( ) );
        
        m_patientPostionTextField.setText( fi.getPatientPosition( ) );
        
        m_pulseSequenceTextField.setText( fi.getProtocolName( ) + " - " +fi.getSeriesDescription( ) );
        m_pulseSequenceAveragesTextField.setText( "" );
        
        
        // retrieve elements from Siemens specific header
        // e.q. get the pulse sequence number of averages her from the LinkedHashMap
        String name;
        
        if ( ssd != null )
        {
            Iterator it = ssd.keySet( ).iterator( );
        
            while ( it.hasNext( ) )
            {
                Object key = it.next( );
                
                // System.out.println( "" + (String)key );
                
                
                if ( ((String)key).compareTo( "lAverages" ) == 0 )    
                {
                    m_pulseSequenceAveragesTextField.setText( (String)ssd.get( key ) );
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.dThickness" ) == 0 )
                {
                    m_thickVoxelSizeTextField.setText( (String)ssd.get( key ) );
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.dReadoutFOV" ) == 0 )
                {
                    m_readOutVoxelSizeTextField.setText( (String)ssd.get( key ) );
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.dPhaseFOV" ) == 0 )
                {
                    m_phaseVoxelSizeTextField.setText( (String)ssd.get( key ) );
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.sPosition.dSag" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dSagVoxelPositionTextField.setText( (  "" + Math.rint ( d.doubleValue() *100.)/100. ) );
                    }
                    else
                    {
                        m_dSagVoxelPositionTextField.setText( "0.00" );
                    }
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.sPosition.dCor" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dCorVoxelPositionTextField.setText( (  "" + Math.rint ( d.doubleValue() *100.)/100. ) );
                    }
                    else
                    {
                        m_dCorVoxelPositionTextField.setText( "0.00" );
                    }
                    
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.sPosition.dTra" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dTraVoxelPositionTextField.setText( (  "" + Math.rint ( d.doubleValue() *100.)/100. ) );
                    }
                    else
                    {
                        m_dTraVoxelPositionTextField.setText( "0.00" );
                    }
                }
             
                if ( ((String)key).compareTo( "sTXSPEC.asNucleusInfo[0].tNucleus" ) == 0 )
                {
                    m_nucleusTypeTextField.setText( ( (String)ssd.get( key ) ) );
                    
                }
                
                if ( ((String)key).compareTo( "sTXSPEC.asNucleusInfo[0].lFrequency" ) == 0 )
                {
                    m_resonanceFrequencyTextField.setText( (String)ssd.get( key ) );
                }
                
                if ( ((String)key).compareTo( "sSpecPara.sVoI.sNormal.dSag" ) == 0 )
                { 
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dSagVoxelOrientationTextField.setText( (  "" + Math.rint ( d.doubleValue() *10000.)/10000. ) );
                    }
                    else
                    {
                        m_dSagVoxelOrientationTextField.setText( "0.00" );
                    }
                
                }
                if ( ((String)key).compareTo( "sSpecPara.sVoI.sNormal.dCor" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dCorVoxelOrientationTextField.setText( (  "" + Math.rint ( d.doubleValue() *10000.)/10000. ) );
                    }
                    else
                    {
                        m_dCorVoxelOrientationTextField.setText( "0.00" );
                    }
                    
                }
                if ( ((String)key).compareTo( "sSpecPara.sVoI.sNormal.dTra" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dTraVoxelOrientationTextField.setText( (  "" + Math.rint ( d.doubleValue() *10000.)/10000. ) );
                    }
                    else
                    {
                        m_dTraVoxelOrientationTextField.setText( "0.00" );
                    }
                }
                
                
            }
        }
        
    }
     
    
    /**
     * Adds the panel containing spectroscopy attributes to the main panel
     */
    private void constructSpectrumAttributesPanel( )
    {
        m_spectroAttributePanel = new JPanel( );
        m_spectroAttributePanel.setLayout( new BoxLayout( m_spectroAttributePanel, BoxLayout.Y_AXIS ) );
        
        m_spectroAttributePanel.setBorder( BorderFactory.createCompoundBorder( 
                                           BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                           BorderFactory.createTitledBorder( "Pulse sequence Information" ) ) );
        // Pulse sequence panel
        m_pulseSequencePanel = new JPanel( );
        
        m_pulseSequencePanel.setLayout( new BoxLayout( m_pulseSequencePanel, BoxLayout.X_AXIS ) );
        m_pulseSequencePanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        
        m_pulseSequenceNameLabel = new JLabel( "Pulse sequence name " );
        m_pulseSequenceTextField = new JTextField( );
        m_pulseSequenceTextField.setPreferredSize( new Dimension( 200, 20 ) );
        m_pulseSequenceTextField.setMinimumSize( new Dimension( 200, 20 ) );
        m_pulseSequenceTextField.setMaximumSize( new Dimension( 200, 20 ) );
        
        m_pulseSequenceAveragesLabel = new JLabel( "  # of averages " );
        m_pulseSequenceAveragesTextField = new JTextField( );
        m_pulseSequenceAveragesTextField.setPreferredSize( new Dimension( 100, 20 ) );
        m_pulseSequenceAveragesTextField.setMinimumSize( new Dimension( 100, 20 ) );
        m_pulseSequenceAveragesTextField.setMaximumSize( new Dimension( 100, 20 ) );
        
        m_pulseSequencePanel.add( m_pulseSequenceNameLabel );
        m_pulseSequencePanel.add( m_pulseSequenceTextField );
        m_pulseSequencePanel.add( Box.createHorizontalGlue( ) );
        m_pulseSequencePanel.add( m_pulseSequenceAveragesLabel );
        m_pulseSequencePanel.add( m_pulseSequenceAveragesTextField );
        
        // Voxel size panel
        m_voxelSizePanel = new JPanel( );
        m_voxelSizePanel.setLayout( new BoxLayout( m_voxelSizePanel, BoxLayout.X_AXIS ) );
        m_voxelSizePanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        
        m_thickVoxelSize = new JLabel( "Voxel size (thickness) " );
        m_thickVoxelSizeTextField = new JTextField( );
        m_thickVoxelSizeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_thickVoxelSizeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_thickVoxelSizeTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_phaseVoxelSize = new JLabel( "Voxel size (phase) " );
        m_phaseVoxelSizeTextField = new JTextField( );
        m_phaseVoxelSizeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_phaseVoxelSizeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_phaseVoxelSizeTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_readOutVoxelSize = new JLabel( "Voxel size (read-out) " );
        m_readOutVoxelSizeTextField = new JTextField( );
        m_readOutVoxelSizeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_readOutVoxelSizeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_readOutVoxelSizeTextField.setMaximumSize( new Dimension( 60, 20 ) );

        m_voxelSizePanel.add( m_thickVoxelSize );
        m_voxelSizePanel.add( m_thickVoxelSizeTextField );
        m_voxelSizePanel.add( Box.createHorizontalGlue( ) );
        m_voxelSizePanel.add( m_phaseVoxelSize );
        m_voxelSizePanel.add( m_phaseVoxelSizeTextField );
        m_voxelSizePanel.add( Box.createHorizontalGlue( ) );
        m_voxelSizePanel.add( m_readOutVoxelSize );
        m_voxelSizePanel.add( m_readOutVoxelSizeTextField );
        
        
        // Voxel size panel
        m_voxelSizePanel = new JPanel( );
        m_voxelSizePanel.setLayout( new BoxLayout( m_voxelSizePanel, BoxLayout.X_AXIS ) );
        m_voxelSizePanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        
        m_thickVoxelSize = new JLabel( "Size (thickness) " );
        m_thickVoxelSizeTextField = new JTextField( "-1");
        m_thickVoxelSizeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_thickVoxelSizeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_thickVoxelSizeTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_phaseVoxelSize = new JLabel( "Size (phase) " );
        m_phaseVoxelSizeTextField = new JTextField( "-1" );
        m_phaseVoxelSizeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_phaseVoxelSizeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_phaseVoxelSizeTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_readOutVoxelSize = new JLabel( "Size (read-out) " );
        m_readOutVoxelSizeTextField = new JTextField( "-1" );
        m_readOutVoxelSizeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_readOutVoxelSizeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_readOutVoxelSizeTextField.setMaximumSize( new Dimension( 60, 20 ) );

        m_voxelSizePanel.add( m_thickVoxelSize );
        m_voxelSizePanel.add( m_thickVoxelSizeTextField );
        m_voxelSizePanel.add( Box.createHorizontalGlue( ) );
        m_voxelSizePanel.add( m_phaseVoxelSize );
        m_voxelSizePanel.add( m_phaseVoxelSizeTextField );
        m_voxelSizePanel.add( Box.createHorizontalGlue( ) );
        m_voxelSizePanel.add( m_readOutVoxelSize );
        m_voxelSizePanel.add( m_readOutVoxelSizeTextField );
        
        
        m_voxelCharacteristicsPanel = new JPanel( );
        m_voxelCharacteristicsPanel.setLayout( new BoxLayout(m_voxelCharacteristicsPanel, BoxLayout.Y_AXIS ) );
        m_voxelCharacteristicsPanel.setBorder( BorderFactory.createCompoundBorder(
                                    BorderFactory.createEmptyBorder(3,3,3,3), 
                                    BorderFactory.createTitledBorder( "MRS Voxel properties" ) ) );
       
        
        // Voxel position panel
        m_voxelPositionPanel = new JPanel( );
        m_voxelPositionPanel.setLayout( new BoxLayout( m_voxelPositionPanel, BoxLayout.X_AXIS ) );
        m_voxelPositionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        
        m_dSagVoxelPosition = new JLabel( "Sag. position       " );
        m_dSagVoxelPositionTextField = new JTextField( "0.00");
        m_dSagVoxelPositionTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_dSagVoxelPositionTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_dSagVoxelPositionTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_dCorVoxelPosition = new JLabel( "Cor. position " );
        m_dCorVoxelPositionTextField = new JTextField( "0.00" );
        m_dCorVoxelPositionTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_dCorVoxelPositionTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_dCorVoxelPositionTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_dTraVoxelPosition = new JLabel( "Trans. position " );
        m_dTraVoxelPositionTextField = new JTextField( "0.00" );
        m_dTraVoxelPositionTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_dTraVoxelPositionTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_dTraVoxelPositionTextField.setMaximumSize( new Dimension( 60, 20 ) );

        m_voxelPositionPanel.add( m_dSagVoxelPosition );
        m_voxelPositionPanel.add( m_dSagVoxelPositionTextField );
        m_voxelPositionPanel.add( Box.createHorizontalGlue( ) );
        m_voxelPositionPanel.add( m_dCorVoxelPosition );
        m_voxelPositionPanel.add( m_dCorVoxelPositionTextField );
        m_voxelPositionPanel.add( Box.createHorizontalGlue( ) );
        m_voxelPositionPanel.add( m_dTraVoxelPosition );
        m_voxelPositionPanel.add( m_dTraVoxelPositionTextField );
        
        // Voxel orientation panel
        m_voxelOrientationPanel = new JPanel( );
        m_voxelOrientationPanel.setLayout( new BoxLayout( m_voxelOrientationPanel, BoxLayout.X_AXIS ) );
        m_voxelOrientationPanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        
        m_dSagVoxelOrientation = new JLabel( "Sag. orientation " );
        m_dSagVoxelOrientationTextField = new JTextField( "0.0000");
        m_dSagVoxelOrientationTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_dSagVoxelOrientationTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_dSagVoxelOrientationTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_dCorVoxelOrientation = new JLabel( "Cor. orientation " );
        m_dCorVoxelOrientationTextField = new JTextField( "0.000" );
        m_dCorVoxelOrientationTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_dCorVoxelOrientationTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_dCorVoxelOrientationTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_dTraVoxelOrientation = new JLabel( "Trans. orientation " );
        m_dTraVoxelOrientationTextField = new JTextField( "0.000" );
        m_dTraVoxelOrientationTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_dTraVoxelOrientationTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_dTraVoxelOrientationTextField.setMaximumSize( new Dimension( 60, 20 ) );

        m_voxelOrientationPanel.add( m_dSagVoxelOrientation );
        m_voxelOrientationPanel.add( m_dSagVoxelOrientationTextField );
        m_voxelOrientationPanel.add( Box.createHorizontalGlue( ) );
        m_voxelOrientationPanel.add( m_dCorVoxelOrientation );
        m_voxelOrientationPanel.add( m_dCorVoxelOrientationTextField );
        m_voxelOrientationPanel.add( Box.createHorizontalGlue( ) );
        m_voxelOrientationPanel.add( m_dTraVoxelOrientation );
        m_voxelOrientationPanel.add( m_dTraVoxelOrientationTextField );
        
        // Basic MRS parameters panel
        m_basicMRSParameterPanel = new JPanel( );
        m_basicMRSParameterPanel.setLayout( new BoxLayout( m_basicMRSParameterPanel, BoxLayout.X_AXIS ) );
        m_basicMRSParameterPanel.setBorder( BorderFactory.createEmptyBorder( 5, 3, 5, 5 ) );
        
        m_nucleusType = new JLabel( "Nucleus " );
        m_nucleusTypeTextField = new JTextField( );
        m_nucleusTypeTextField.setPreferredSize( new Dimension( 60, 20 ) );
        m_nucleusTypeTextField.setMinimumSize( new Dimension( 60, 20 ) );
        m_nucleusTypeTextField.setMaximumSize( new Dimension( 60, 20 ) );
        
        m_resonanceFrequency = new JLabel( "Resonance Frequency " );
        m_resonanceFrequencyTextField = new JTextField( );
        m_resonanceFrequencyTextField.setPreferredSize( new Dimension( 80, 20 ) );
        m_resonanceFrequencyTextField.setMinimumSize( new Dimension( 80, 20 ) );
        m_resonanceFrequencyTextField.setMaximumSize( new Dimension( 80, 20 ) );
        
        m_basicMRSParameterPanel.add( Box.createHorizontalGlue( ) );
        m_basicMRSParameterPanel.add( m_nucleusType );
        m_basicMRSParameterPanel.add( m_nucleusTypeTextField );
        m_basicMRSParameterPanel.add( Box.createHorizontalGlue( ) );
        m_basicMRSParameterPanel.add( m_resonanceFrequency );
        m_basicMRSParameterPanel.add( m_resonanceFrequencyTextField );
        m_basicMRSParameterPanel.add( Box.createHorizontalGlue( ) );
        
        // add all sub panels
        m_spectroAttributePanel.add( m_basicMRSParameterPanel );
        m_spectroAttributePanel.add( m_pulseSequencePanel );
        
        m_voxelCharacteristicsPanel.add( m_voxelSizePanel );
        m_voxelCharacteristicsPanel.add( m_voxelPositionPanel );
        m_voxelCharacteristicsPanel.add( m_voxelOrientationPanel );
        m_spectroAttributePanel.add ( m_voxelCharacteristicsPanel );
        m_contentPanel.add( m_spectroAttributePanel, BorderLayout.CENTER );
    }
    
    /**
     * Creates the options panel: user can add this item to the database, or
     * remove it from it
     */
    private void constructOptionsPanel( )
    {
        m_optionsPanel = new JPanel( );
        
        m_optionsPanel.setLayout( new BoxLayout( m_optionsPanel, BoxLayout.X_AXIS ) );
        
        m_optionsPanel.setBorder( BorderFactory.createCompoundBorder( 
                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                  BorderFactory.createCompoundBorder( BorderFactory.createTitledBorder( "DatabaseItem Options" ),  
                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) ) );
        
        m_insertToDBaseButton = new JButton( "Insert Item" );
        m_insertToDBaseButton.addActionListener( this );
        m_insertToDBaseButton.setToolTipText( "Add the current item to the database" );
        
        m_removeFromDBaseButton = new JButton( "Remove item" );
        m_insertToDBaseButton.setToolTipText( "Removes the current item from the database" );
        m_removeFromDBaseButton.addActionListener( this );
        
        m_optionsPanel.add( Box.createHorizontalGlue( ) );
        m_optionsPanel.add( m_removeFromDBaseButton );
        m_optionsPanel.add( Box.createHorizontalGlue( ) );
        m_optionsPanel.add( m_insertToDBaseButton );
        m_optionsPanel.add( Box.createHorizontalGlue( ) );
        
        m_contentPanel.add( m_optionsPanel, BorderLayout.SOUTH );
        
    }
    
    /**
     * Is called whenever a registered component raises an ActionEvent
     */ 
    public void actionPerformed( ActionEvent actionEvent ) 
    {
        
    }
    
    
    /**
     * Constructs the pathology panel, where the user can set the diagnosis
     * of the mrs/mri separately
     */
    public void constructPathologyPanel( )
    {
        m_pathologyPanel = new JPanel( );
        m_pathologyPanel.setLayout( new BoxLayout( m_pathologyPanel, BoxLayout.Y_AXIS ) );
        
        m_pathologyPanel.setBorder( BorderFactory.createCompoundBorder( 
                                    BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                    BorderFactory.createCompoundBorder( BorderFactory.createTitledBorder( "Anatomy and Pathology" ),  
                                    BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) ) );
        
        
        m_anatomySelectionPanel = new JPanel( );
        m_anatomySelectionPanel.setBorder( BorderFactory.createEmptyBorder(2,0,2,0 ) );
        m_anatomySelectionPanel.setLayout( new BoxLayout( m_anatomySelectionPanel, BoxLayout.X_AXIS ) );
        
        m_examinedMRIAnatomy = new JLabel( "Main anatomy  ");
        m_examinedMRIPathologyCombobox = new AnatomyOfStudyComboBox( );
        
        m_examinedMRIPathologyCombobox.setPreferredSize( new Dimension( 170, 20 ) );
        m_examinedMRIPathologyCombobox.setMinimumSize( new Dimension( 170, 20 ) );
        m_examinedMRIPathologyCombobox.setMaximumSize( new Dimension( 170, 20 ) );
        
        m_mrsVoxelLocalizationLabel = new JLabel( " Voxel Localization " );
        
        m_mrsVoxelPositionCombobox = new AnatomicNameOfMRSLocalizationComboBox( );
        m_mrsVoxelPositionCombobox.setPreferredSize( new Dimension( 170, 20 ) );
        m_mrsVoxelPositionCombobox.setMinimumSize( new Dimension( 170, 20 ) );
        m_mrsVoxelPositionCombobox.setMaximumSize( new Dimension( 170, 20 ) );
        
        m_anatomySelectionPanel.add( m_examinedMRIAnatomy );
        m_anatomySelectionPanel.add( m_examinedMRIPathologyCombobox );
        m_anatomySelectionPanel.add( Box.createHorizontalGlue( ) );
        m_anatomySelectionPanel.add( m_mrsVoxelLocalizationLabel );
        m_anatomySelectionPanel.add( m_mrsVoxelPositionCombobox );
        
        m_diagnosisSelectionPanel = new JPanel( );
        m_diagnosisSelectionPanel.setLayout( new BoxLayout( m_diagnosisSelectionPanel, BoxLayout.X_AXIS ) );
        
        m_mriPathologyLabel = new JLabel( "Diagnosis MRI  " );
        m_mriPathologyCombobox = new MRIPathologyComboBox( );
        
        m_mriPathologyCombobox.setPreferredSize( new Dimension( 170, 20 ) );
        m_mriPathologyCombobox.setMinimumSize( new Dimension( 170, 20 ) );
        m_mriPathologyCombobox.setMaximumSize( new Dimension( 170, 20 ) );
        
        m_mrsPathologyLabel = new JLabel( " Diagnosis MRS ");
        m_mrsPathologyCombobox = new MRSPathologyComboBox( );

        m_mrsPathologyCombobox.setPreferredSize( new Dimension( 170, 20 ) );
        m_mrsPathologyCombobox.setMinimumSize( new Dimension( 170, 20 ) );
        m_mrsPathologyCombobox.setMaximumSize( new Dimension( 170, 20 ) );
        
        // m_diagnosisSelectionPanel.add( Box.createHorizontalGlue( ) );
        m_diagnosisSelectionPanel.add( m_mriPathologyLabel );
        m_diagnosisSelectionPanel.add( m_mriPathologyCombobox );
        
        m_diagnosisSelectionPanel.add( Box.createHorizontalGlue( ) );
        
        m_diagnosisSelectionPanel.add( m_mrsPathologyLabel );
        m_diagnosisSelectionPanel.add( m_mrsPathologyCombobox );
        
        // m_diagnosisSelectionPanel.add( Box.createHorizontalGlue( ) );
        
        m_pathologyPanel.add( m_anatomySelectionPanel );
        m_pathologyPanel.add( m_diagnosisSelectionPanel );
        
        m_diagnosticTextAreaPanel = new JPanel( );
        m_diagnosticTextAreaPanel.setLayout( new BoxLayout( m_diagnosticTextAreaPanel, BoxLayout.X_AXIS ) );
        m_diagnosticTextAreaPanel.setBorder( BorderFactory.createCompoundBorder( 
                                             BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                             BorderFactory.createCompoundBorder( BorderFactory.createTitledBorder( "Medical diagnosis (description)" ),  
                                             BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) ) );
        m_diagnosticTextAreaPanel.setLayout( new BorderLayout( ) );
        
        m_diagnosticTextAreaPanel.setMinimumSize( new Dimension( 250, 125 ) );
        m_diagnosticTextAreaPanel.setPreferredSize( new Dimension( 250, 125 ) );
        m_diagnosticTextAreaPanel.setMinimumSize( new Dimension( 250, 125 ) );
        
        m_diagnosticTextArea = new JTextArea( );
        m_scrollableTextArea = new JScrollPane( m_diagnosticTextArea );
        m_scrollableTextArea.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );

        m_diagnosticTextAreaPanel.add( m_scrollableTextArea );
        
        m_pathologyPanel.add( m_diagnosticTextAreaPanel );
        
        m_allPatientDataCollectionPanel.add( m_pathologyPanel );
        
        
    }
    
    private JPanel m_optionsPanel;
    
    private JPanel m_spectroAttributePanel;
    
    private JPanel m_pulseSequencePanel;
    
    private JLabel     m_pulseSequenceNameLabel;
    private JTextField m_pulseSequenceTextField;
    
    private JLabel     m_pulseSequenceAveragesLabel;
    private JTextField m_pulseSequenceAveragesTextField;
    
    private MRPatientSpectrumDataBaseItem m_currentPatientDBItem;
    
    private JPanel m_allPatientDataCollectionPanel;
    
    private JPanel m_contentPanel;
    private JPanel m_patientAttributesPanel;
    
    private JLabel m_patientNameLabel;
    private JTextField m_patientNameTextField;
    
    private JLabel m_patientIDLabel;
    private JTextField m_patientIDTextField;
    
    private JPanel m_nameAndIDPanel;
    
    private JPanel m_patientDOBAgeAndSexPanel;
    
    private JLabel m_sexLabel;
    private JRadioButton m_male;
    private JRadioButton m_female;
    private JRadioButton m_other;
    private ButtonGroup m_sexButtonGroup;
    
    private JLabel m_DOBLabel;
    private JTextField m_DOBTextField;
    
    private JLabel m_patientAgeLabel;
    private JTextField m_patientAgeTextField;
    
    private JPanel m_weightAndPositionPanel;
    
    private JLabel m_weightLabel;
    private JLabel m_weightUnitLabel;
    private JTextField m_weightTextField;

    private JLabel m_patientPositionLabel;
    private JTextField m_patientPostionTextField;
    
    // panel on which all voxel charaistiscs subpanels are positioned
    private JPanel m_voxelCharacteristicsPanel;
    
    // voxel size panel
    private JPanel m_voxelSizePanel;
    
    private JLabel m_thickVoxelSize;
    private JTextField m_thickVoxelSizeTextField;
    
    private JLabel m_phaseVoxelSize;
    private JTextField m_phaseVoxelSizeTextField;
    
    private JLabel m_readOutVoxelSize;
    private JTextField m_readOutVoxelSizeTextField;
    
    // voxel position panel
    private JPanel m_voxelPositionPanel;
    
    private JLabel m_dSagVoxelPosition;
    private JTextField m_dSagVoxelPositionTextField;
    
    private JLabel m_dCorVoxelPosition;
    private JTextField m_dCorVoxelPositionTextField;
    
    private JLabel m_dTraVoxelPosition;
    private JTextField m_dTraVoxelPositionTextField;
    
    
    // voxel orientation panel
    private JPanel m_voxelOrientationPanel;
    
    private JLabel m_dSagVoxelOrientation;
    private JTextField m_dSagVoxelOrientationTextField;
    
    private JLabel m_dCorVoxelOrientation;
    private JTextField m_dCorVoxelOrientationTextField;
    
    private JLabel m_dTraVoxelOrientation;
    private JTextField m_dTraVoxelOrientationTextField;
    
    // basic MRS parameter panel
    private JPanel m_basicMRSParameterPanel;
    
    private JLabel m_nucleusType;
    private JTextField m_nucleusTypeTextField;
    
    private JLabel m_resonanceFrequency;
    private JTextField m_resonanceFrequencyTextField;
    
    private JButton m_insertToDBaseButton;
    private JButton m_removeFromDBaseButton;
    
    // Pathology & diagnosis panel
    private JPanel m_pathologyPanel;
   
    private JPanel m_diagnosisSelectionPanel;
    
    private JLabel m_mriPathologyLabel;
    private MRIPathologyComboBox m_mriPathologyCombobox;
    
    private JLabel m_mrsPathologyLabel;
    private MRSPathologyComboBox m_mrsPathologyCombobox;
   
    private JPanel m_diagnosticTextAreaPanel;
    private JTextArea m_diagnosticTextArea;
    private JScrollPane m_scrollableTextArea;
    
    private JPanel m_anatomySelectionPanel;
    
    private JLabel  m_examinedMRIAnatomy;
    private AnatomyOfStudyComboBox m_examinedMRIPathologyCombobox;
    
    private JLabel  m_mrsVoxelLocalizationLabel;
    private AnatomicNameOfMRSLocalizationComboBox m_mrsVoxelPositionCombobox;
    
    
    // For proper serialization
    private static final long serialVersionUID = 9982244491119213L;
}
