/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.randomforestregression;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Vector;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.spectra.FittedSpectra;
import org.spectrim.tdfdfit.MRFitStrategy;
import org.spectrim.tdfdfit.MRLineModelParameters;
import org.spectrim.tdfdfit.MRLineModelSpectralResponse;
import org.spectrim.tdfdfit.MRSignalProperties;
import org.spectrim.tdfdfit.OrderedBaseLinePointsDataModel;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.TDFDFitPrototypeModelSpectrumParameters;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import org.spectrim.tdfdfit.NonInteractiveTDFDFitExecuter;
import org.spectrim.tdfdfit.NumericTDFDFitResults;
import org.spectrim.tdfdfit.TDFDFitExecutorSwingWorker;
import org.spectrim.lib.TextFileWriter;
import org.spectrim.tdfdfit.TDFDFittingParameters;

/**
 *
 * @author Brigitte
 */
public class DataGeneratorMain implements ActionListener {

    private static SimulatedSpectralResponse response = null;

    private static MRSignalProperties m_MRSignalProperties;
    private static File m_TDFDfitPrototypeModelFileReference;
    private static MRFitStrategy m_fitStrategy;

    private static SingleVoxelMeasuredSpectralData m_singleVoxelMeasuredData;
    private static SingleVoxelMeasuredSpectralData m_singleVoxelMeasuredDataLoaded;
    private static FittedSpectra m_fittedSpectra;
    private static FittedSpectra fittedSpectra;

    private static final OrderedBaseLinePointsDataModel m_baseLineModel = OrderedBaseLinePointsDataModel.getInstance();
    protected static int numberOfSamplePoints = 0;
    private static boolean quantificationFinished = false;
    private static Random rnd = new Random();

    private static final NonInteractiveTDFDFitExecuter executor = new NonInteractiveTDFDFitExecuter();
    private static final NumericTDFDFitResults m_numericTDFDFit = new NumericTDFDFitResults();
    private static Vector<TDFDFittingParameters> m_fittedElem = new Vector<TDFDFittingParameters>();

    private static Vector<TDFDFitExecutorSwingWorker> m_workers = new Vector<TDFDFitExecutorSwingWorker>();
    private DualProgressBar m_dualProgressBar;

    private String m_fileNameBase;

    private int m_threadCntr = 0;
    
    // modify the number of simulations in order to have more or less simulations
    private int m_totalSimulations = 20;
    private int n_cores = 0;
    private int m_numberOfThreadsActive;

    public DataGeneratorMain() {

        m_TDFDfitPrototypeModelFileReference = new File("C:\\Users\\Brigitte\\.SpectrImClinicalQMRS\\TDFDFit\\TDFDFit_FitModelsDir\\RFR3\\RFR3.tfm");

        boolean interactive = false;
        loadTDFDFitPrototypeModel(interactive);

        n_cores = Runtime.getRuntime().availableProcessors();

        System.out.println("Number of spectra to fit : " + m_totalSimulations);

        m_numberOfThreadsActive = 0;

//        Date date = new Date();
//        System.out.println("Timestamp : " + new Timestamp(date.getTime()));

        m_dualProgressBar = new DualProgressBar(null,
                n_cores,
                m_totalSimulations,
                new Dimension(900, 150), true);

        m_dualProgressBar.setTitle("Fitting spectra using tdfdfit ..");
        m_dualProgressBar.setUpperLabel("<HTML><BR>     Number of threads active : " + m_numberOfThreadsActive + " (maximal number of threads : " + n_cores + ")</HTML>");
        m_dualProgressBar.setUpperProgress(0);
        //m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible(true);
        m_dualProgressBar.setLowerLabel("<HTML><BR>     Finished nmber of spectra : " + 0 + "</HTML>");

        m_dualProgressBar.addActionListener(this);

        while (m_threadCntr < m_totalSimulations) {  
               m_numberOfThreadsActive++;
               m_threadCntr++;
               doDataGenerationAndFitting(m_threadCntr);
        }
    }

    public static void main(String[] args) {
        DataGeneratorMain dataGenerator = new DataGeneratorMain();
    }

    public void doDataGenerationAndFitting(int threadID) {

        LinkedList<MRLineModelParameters> myLines = TDFDFitPrototypeModelSpectrumParameters.getInstance().getSimpleMRLines();
        MRLineModelParameters linePars = myLines.get(0);
        MRLineModelParameters lineParsVar = new MRLineModelParameters(linePars);

        lineParsVar.setArea(lineParsVar.getArea() + 0.4f * (0.5f - rnd.nextFloat()) * lineParsVar.getArea());
        lineParsVar.setPhase(lineParsVar.getPhase() + 10.0f * (0.5f - rnd.nextFloat()) * lineParsVar.getPhase());
        lineParsVar.setOffset(lineParsVar.getOffset() + 0.05f * (0.5f - rnd.nextFloat()) * lineParsVar.getOffset());
        lineParsVar.setLorentzianWidth(lineParsVar.getLorentzianLineWidth() + 0.2f * (0.5f - rnd.nextFloat()) * lineParsVar.getLorentzianLineWidth());
        lineParsVar.setGaussianWidth( lineParsVar.getGaussianWidth( ) + 0.1f * rnd.nextFloat() );

        MRLineModelSpectralResponse myResponse = new MRLineModelSpectralResponse(lineParsVar.getMRFittingParameters(),
                MRSignalProperties.getInstance());

        Curve myCurve = myResponse.getAbsorptionCurveFDPPM();

//        XYPlotFrame pFrame = new XYPlotFrame("One line model - Plot data", null);

//      pFrame.addCurve(myCurve, false);

//      add noise to the model
        float[] abs = myResponse.getAbsorptionChannelTD();
        float[] disp = myResponse.getDispersionChannelTD();

        for (int i = 0; i < abs.length; i++) {
            abs[i] = abs[i] + (0.5f - rnd.nextFloat()) * 100000;
            disp[i] = disp[i] + (0.5f - rnd.nextFloat()) * 100000;
        }

//      add the noise on the top of the other frame using another curve
        Curve myCurve2;

        myResponse.setSpectralDataPointsTD(abs, disp);
        myCurve2 = myResponse.getAbsorptionCurveFDPPM();
        myCurve2.setColor(Color.red);      
        
//        pFrame.addCurve(myCurve2, false);

//      fitting 
        SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData(abs, disp);

        m_fileNameBase = "Simulated_" + threadID;
        System.err.println(" Simulated + threadID : " + m_fileNameBase );

        TDFDFitExecutorSwingWorker tdfdfitExecutionWorker = new TDFDFitExecutorSwingWorker(threadID, dataToFit, false, m_fileNameBase);

        tdfdfitExecutionWorker.addActionListener(this);
        tdfdfitExecutionWorker.execute();

        m_workers.add(tdfdfitExecutionWorker);

//        n_threadCntr ++;
//        System.out.println( "Done." );
        m_dualProgressBar.setUpperProgress(m_numberOfThreadsActive);
        System.out.println("Number of threads ++ : " + m_numberOfThreadsActive);

        saveSignalAsText(threadID);
    }

    private static boolean loadTDFDFitPrototypeModel(boolean interactive) {
        // contains reference to model m_signalPrototypeModelSpectrumFileReference
        if (interactive == true) { // here is some error...
            m_TDFDfitPrototypeModelFileReference = getLoadTDFDFitPrototypeModelFileReference();
        } else {
            // m_TDFDfitModelFileReference is already defined in main.
            // m_TDFDfitPrototypeModelFileReference = getLoadTDFDFitPrototypeModelFileReference( );
        }

        loadTDFDFitPrototypeModelSpectrumFile();

        return true;
    }

    private static void loadTDFDFitPrototypeModelSpectrumFile() {

        if (m_TDFDfitPrototypeModelFileReference != null) {
            try {

                RandomAccessFile reader = new RandomAccessFile(m_TDFDfitPrototypeModelFileReference, "rw");

                m_MRSignalProperties = MRSignalProperties.getInstance();

                // Load all essential object attritubutes to a file.
                m_MRSignalProperties.read(reader);

                // Load the fitting strategy; if not defined you will get the 
                // default strategy.
                m_fitStrategy = MRFitStrategy.getInstance();
                m_fitStrategy.read(reader);  

                // Read the TDFDFitPrototypeModelSpectrum object from file.
                TDFDFitPrototypeModelSpectrumParameters.getInstance().read(reader);

                m_singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData();

                // Read the single voxel spectral data response
                m_singleVoxelMeasuredData.read(reader, false);

                if (m_singleVoxelMeasuredDataLoaded != null) {
                    m_singleVoxelMeasuredData.autoSetXaxis();
                    m_singleVoxelMeasuredDataLoaded = m_singleVoxelMeasuredData;
                    m_singleVoxelMeasuredDataLoaded.autoSetXaxis();
                    //xyplot m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredDataLoaded );
                } else {
                    m_singleVoxelMeasuredDataLoaded = m_singleVoxelMeasuredData;
                    m_singleVoxelMeasuredData.autoSetXaxis();
                    m_singleVoxelMeasuredDataLoaded.autoSetXaxis();
                    // xyplot m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredData );
                }

                // The model spectrum must know its m_MRSignalProperties object
                TDFDFitPrototypeModelSpectrumParameters.getInstance().setMRSignalProperties(m_MRSignalProperties);
                TDFDFitPrototypeModelSpectrumParameters.getInstance().setMeasuredSpectrum(m_singleVoxelMeasuredData);
//             
                // Reads the baseline points from file
                m_baseLineModel.read(reader);

                try {
                    // Load the patient database, and get a reference to it
                    // m_mrSpectrumDataBase = MRPatientSpectrumDataBase.getInstance( );
                    reader.close();
                } catch (IOException ex) {
                    Logger.getLogger(DataGeneratorMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            } catch (FileNotFoundException e) {
                // Not handled yet.
            }

            TDFDFitPrototypeModelSpectrumParameters protoType = TDFDFitPrototypeModelSpectrumParameters.getInstance();
            Curve plotCurve = protoType.getMeasuredSpectrum().getAbsorptionCurveFDPPM();

            XYPlotFrame pFrame = new XYPlotFrame("Test - Plot data", null);

            System.out.println("End of program 3.");

            Curve cur = new Curve(plotCurve.getXvalues(),
                    plotCurve.getYvalues(),
                    Color.BLACK);

            cur.setStyle(Curve.FAT_SOLID_LINE);

            pFrame.addCurve(cur, false);

        }
    }

    private static File getLoadTDFDFitPrototypeModelFileReference() 
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String modelRef = fdef.getProperty(TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME);
        String modelPath = fdef.getProperty(TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH);

        File file = new File( modelPath + System.getProperty( "file.separator" ) + modelRef );

        if (file.exists() == false) {
            file = new File(fdef.getProperty(TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH));
        }

        if (file.exists() == false) {
            file = new File(System.getProperty("user.home"));
        }

        JFileChooser fileChooser = new JFileChooser(file.getAbsolutePath());

        int result = fileChooser.showOpenDialog(null);

        m_TDFDfitPrototypeModelFileReference = new File("");

        switch (result) {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                m_TDFDfitPrototypeModelFileReference = fileChooser.getSelectedFile();
                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                def.load( );
                
                def.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, 
                                 m_TDFDfitPrototypeModelFileReference.getName( ) );
                def.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, 
                                 m_TDFDfitPrototypeModelFileReference.getParent( ) );
                def.save( );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( m_TDFDfitPrototypeModelFileReference.getName( ) );

                break;
            case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                break;
            case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                break;
        }

        return m_TDFDfitPrototypeModelFileReference;
    }
    
    public static void quantifySpectra(String m_TDFDfitPrototypeModelFileReference) 
    {

        if (m_TDFDfitPrototypeModelFileReference.endsWith(".tfm") == false) 
        {
            m_TDFDfitPrototypeModelFileReference = m_TDFDfitPrototypeModelFileReference + ".tfm";
        }

        File defaultModelFile = new File(m_TDFDfitPrototypeModelFileReference);

        if (defaultModelFile.exists() == true) {

            executor.executeTDFDFitting(true);
            System.out.println("test TDFDFit fitting");

            //TODO : when set to false, it used to cause an infinite loop. 
            boolean quantificationFinished = false;

            while (quantificationFinished == false) {
                quantificationFinished = isQuantificationFinished();
                try {
                    TimeUnit.SECONDS.sleep(1);
                } catch (InterruptedException ex) {
                    Logger.getLogger("Error fitting.");
                }
            }

            executor.putFittedSpectraIntoGlobalModel();
            fittedSpectra = getFittedSpectra();
        }
    }

    public static boolean isQuantificationFinished() {
        return quantificationFinished;
    }

    public static FittedSpectra getFittedSpectra() {
        return m_fittedSpectra;
    }

    public void collectAllDataAfterAllFittingIsPerformed() throws IOException {

        int number_of_points_TD = 1024;
        double number_of_points_FD_min = 4.75;
        double number_of_points_FD_max = 5.50;

        System.out.println("All fitting done !");

        String resultPath = TDFDFittingDefaults.getInstance().getProperty(TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH);

        List<NumericTDFDFitResults> numFitResultArray = new ArrayList<>();

        String outputPath = System.getProperty("user.home");

        String outputFileReference = outputPath + System.getProperty("file.separator") + "testOutput.csv"; // asv 

        int samplePoints = m_numericTDFDFit.getNumberOfPoints();
        float bandwidth = samplePoints;

        TextFileWriter writer = null;

        try {
            writer = new TextFileWriter(new FileWriter(outputFileReference));

            writer.setNumberFormat("0.00000000E00");
            writer.writeStringLn("start_pattern");
            writer.writeString("number_of_simulations: \t");
            writer.writeInteger(m_totalSimulations);
            writer.writeStringLn("");
            writer.writeStringLn("parameters");
            writer.writeString("name,line_type,area,frequency,gauss_width,zero_phase,lorentz_width,FQN,");
            
            for(int z = 502; z<= 522; z++){  //  int z= 0; z < number_of_points_TD ; z++
            writer.writeString("absFitFD[" + z + "],");
            }
            
            for(int z = 502; z <= 522; z++){  // z = 502; z<= 522
            writer.writeString("dispFitFD[" + z + "],");
            }
           
            for(int z = 0; z<= 10; z++){ // z = 0; z<= 10
            writer.writeString("absFitTD[" + z + "],");
            }
                
            for(int z = 0; z<= 10; z++){  //z = 0; z<= 10
            writer.writeString("dispFitTD[" + z + "],");
            }    

            writer.writeString("\n");
            
        } catch (Exception ex) {
            Logger.getLogger(DataGeneratorMain.class.getName()).log(Level.SEVERE, null, ex);
        }
  
        for (int i = 0; i < m_totalSimulations; i++) {

            String exlfileNameBase = "Simulated_" + (i + 1);
            m_numericTDFDFit.setResultFileName(resultPath + System.getProperty("file.separator") + exlfileNameBase + ".exl");
            m_numericTDFDFit.setComponentNames(TDFDFitPrototypeModelSpectrumParameters.getInstance().getComponentNameVector());
            m_numericTDFDFit.readFile();

            m_numericTDFDFit.collectSignalInformationFromFile(number_of_points_TD, ".bin");
            System.out.println("File reader loop: " + i);

            numFitResultArray.add(m_numericTDFDFit);   
            System.out.println(numFitResultArray.get(i).getM_fileReference());

            
             // output - file text writer
            m_fittedElem = m_numericTDFDFit.getFittedElements();
            TDFDFittingParameters fitParameters = m_fittedElem.get(i);
            double m_FQN = m_numericTDFDFit.getFitQualityNumber();
            
            float[] absFitFD = m_numericTDFDFit.getFittedSpectrumRealFD();
            float[] dispFitFD = m_numericTDFDFit.getFittedSpectrumImagFD();
           
            float[] absFitTD = m_numericTDFDFit.getFittedSpectrumRealTD();
            float[] dispFitTD = m_numericTDFDFit.getFittedSpectrumImagTD();
          
            
            System.out.println(m_numericTDFDFit.getM_fileReference());

            writer.writeString(fitParameters.getComponentName() + ",");

            writer.writeString("voigt,");
            writer.writeDouble(fitParameters.getAmplitude());
            writer.writeString(",");
            writer.writeDouble((fitParameters.getOffset() + bandwidth / 2f));
            writer.writeString(",");
            writer.writeDouble(fitParameters.getGwidth());
            writer.writeString(",");
            writer.writeDouble(fitParameters.getPhase());
            writer.writeString(",");
            writer.writeDouble(fitParameters.getLwidth());     
            writer.writeString(",");
            writer.writeDouble(m_FQN);
          
            
//            writer.writeArray(absFitFD);
//            writer.writeString(",");
//            writer.writeArray(dispFitFD);
//            writer.writeString(",");
//            writer.writeArray(absFitTD);
//            writer.writeString(",");
//            writer.writeArray(dispFitTD);

            for(int j = 502; j<= 522; j++){
                writer.writeString(",");
                writer.writeFloat(absFitFD[j]);
            }
            
            for(int j = 502; j<= 522; j++){
                writer.writeString(",");
                writer.writeFloat(dispFitFD[j]);
            }
            
            for(int j = 0; j<= 10; j++){
                writer.writeString(",");
                writer.writeFloat(absFitTD[j]);
            }
            
            for(int j = 0; j<= 10; j++){
                writer.writeString(",");
                writer.writeFloat(dispFitTD[j]);
            }
            
            writer.writeStringLn("");
        }

        try {
          
            writer.writeStringLn("end_of_pattern");
            writer.writeStringLn(" ");
            writer.flush();
            writer.close();
        } 
        catch (IOException ex) {
            Logger.getLogger(DataGeneratorMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    private void saveSignalAsText( int threadID ) 
    {
        org.spectrim.tdfdfit.TextFileWriter writer = null;
        try 
        {     
            String txtFile = System.getProperty("user.home") + System.getProperty("file.separator") + "savedSignal_" + threadID + ".txt"; ;
            System.err.println(" Saved signal file: " + txtFile);
            
            writer = new org.spectrim.tdfdfit.TextFileWriter( new FileWriter( txtFile ) );
            
            float[] absSigTD    = m_singleVoxelMeasuredDataLoaded.getAbsorptionChannelTD( );
            float[] dispSigTD   = m_singleVoxelMeasuredDataLoaded.getDispersionChannelTD( );
            float[] absFitTD    = m_singleVoxelMeasuredDataLoaded.getAbsorptionChannelTD( );
            float[] dispFitTD   = m_singleVoxelMeasuredDataLoaded.getDispersionChannelTD( );
            float[] absResTD    = m_singleVoxelMeasuredDataLoaded.getAbsorptionChannelTD( );
            float[] dispResTD   = m_singleVoxelMeasuredDataLoaded.getDispersionChannelTD( );
            writer.writeStringLn( "Number" + "," + "abs_sig_TD" + "," + "disp_sig_TD" + "," + "abs_fit_TD" + "," + "disp_fit_TD" + "," + "abs_res_TD" + "," + "disp_res_TD" );
            for ( int i = 0; i < absSigTD.length; i++ )
            {
                writer.writeInteger( i );
                writer.writeString( "," );
                writer.writeFloat( absSigTD[i] );
                writer.writeString( "," );
                writer.writeFloat( dispSigTD[i] );
                writer.writeString( "," );
                writer.writeFloat( absFitTD[i] );
                writer.writeString( "," );
                writer.writeFloat( dispFitTD[i] );
                writer.writeString( "," );
                writer.writeFloat( absResTD[i] );
                writer.writeString( "," );
                writer.writeFloat( dispResTD[i] );
                writer.writeString( "\n" );
            }
            
            writer.flush( );
            writer.close( );
            
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( DataGeneratorMain.class.getName( ) ).log( Level.SEVERE, null, ex );
        } 
        finally 
        {
            try 
            {
                writer.close( );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( DataGeneratorMain.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {

        Object sender = e.getSource();
        if (sender instanceof TDFDFitExecutorSwingWorker) 
        {
            if (e.getID() == TDFDFitExecutorSwingWorker.IS_PERFORMED) {
//                if ( ( m_threadCntr == m_totalSimulations ) && ( this.m_numberOfThreadsActive == 0 ) )
//                {
////                    System.out.println( "Action performed called" );
////                    collectAllDataAfterAllFittingIsPerformed( );
//                }
//                else
//                {
                // decrement the number of active threads
                m_numberOfThreadsActive--;
                m_dualProgressBar.setUpperProgress(m_numberOfThreadsActive);
                System.out.println("Number of threads -- : " + m_numberOfThreadsActive);
                System.err.println(m_threadCntr + "****" + this.m_numberOfThreadsActive);
//                if ( ( m_threadCntr != m_totalSimulations ) && ( this.m_numberOfThreadsActive == 0 ) ){
                if ((this.m_numberOfThreadsActive == 0)) {
                    System.out.println("Action performed called");
                    try {                     
                        collectAllDataAfterAllFittingIsPerformed();
                    } catch (IOException ex) {
                        Logger.getLogger(DataGeneratorMain.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }
        }

    }
}
