/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.registration;

import org.spectrim.imagegraphicobjects.Point3D;
import java.io.BufferedInputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;
import niftijio.LEDataInputStream;
import niftijio.NiftiHeader;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.spectrim.io.nifti.NiftiHandler;

/**
 *
 * @author nuno
 */
public class DisplacementField {

    private double[][][][] disp;
    private NiftiHeader header;

    public DisplacementField(String filename) {
        try {
            header = NiftiHeader.read(filename);

            InputStream is = new FileInputStream(header.filename);

            if (header.filename.endsWith(".gz")) {
                is = new GZIPInputStream(is);
            }

            is = new BufferedInputStream(is);

            // skip header
            is.skip((long) header.vox_offset);

            int nx = header.dim[1];
            int ny = header.dim[2];
            int nz = header.dim[3];

            if (header.dim[0] == 2) {
                nz = 1;
            }
            this.disp = new double[nx][ny][nz][3];
            DataInput di = header.little_endian ? new LEDataInputStream(is) : new DataInputStream(is);

            for (int k = 0; k < nz; k++) {
                for (int j = 0; j < ny; j++) {
                    for (int i = 0; i < nx; i++) {
                        for (int e = 0; e < 3; e++) {
                            double v;
                            switch (header.datatype) {
                                case NiftiHeader.NIFTI_TYPE_INT8:
                                case NiftiHeader.NIFTI_TYPE_UINT8:
                                    v = di.readByte();

                                    if ((header.datatype == NiftiHeader.NIFTI_TYPE_UINT8) && v < 0) {
                                        v = v + 256d;
                                    }
                                    if (header.scl_slope != 0) {
                                        v = v * header.scl_slope + header.scl_inter;
                                    }
                                    break;
                                case NiftiHeader.NIFTI_TYPE_INT16:
                                case NiftiHeader.NIFTI_TYPE_UINT16:
                                    v = (double) (di.readShort());

                                    if ((header.datatype == NiftiHeader.NIFTI_TYPE_UINT16) && (v < 0)) {
                                        v = Math.abs(v) + (double) (1 << 15);
                                    }
                                    if (header.scl_slope != 0) {
                                        v = v * header.scl_slope + header.scl_inter;
                                    }
                                    break;
                                case NiftiHeader.NIFTI_TYPE_INT32:
                                case NiftiHeader.NIFTI_TYPE_UINT32:
                                    v = (double) (di.readInt());
                                    if ((header.datatype == NiftiHeader.NIFTI_TYPE_UINT32) && (v < 0)) {
                                        v = Math.abs(v) + (double) (1 << 31);
                                    }
                                    if (header.scl_slope != 0) {
                                        v = v * header.scl_slope + header.scl_inter;
                                    }
                                    break;
                                case NiftiHeader.NIFTI_TYPE_INT64:
                                case NiftiHeader.NIFTI_TYPE_UINT64:
                                    v = (double) (di.readLong());
                                    if ((header.datatype == NiftiHeader.NIFTI_TYPE_UINT64) && (v < 0)) {
                                        v = Math.abs(v) + (double) (1 << 63);
                                    }
                                    if (header.scl_slope != 0) {
                                        v = v * header.scl_slope + header.scl_inter;
                                    }
                                    break;
                                case NiftiHeader.NIFTI_TYPE_FLOAT32:
                                    v = (double) (di.readFloat());
                                    if (header.scl_slope != 0) {
                                        v = v * header.scl_slope + header.scl_inter;
                                    }
                                    break;
                                case NiftiHeader.NIFTI_TYPE_FLOAT64:
                                    v = (double) (di.readDouble());
                                    if (header.scl_slope != 0) {
                                        v = v * header.scl_slope + header.scl_inter;
                                    }
                                    break;
                                case NiftiHeader.DT_NONE:
                                case NiftiHeader.DT_BINARY:
                                case NiftiHeader.NIFTI_TYPE_COMPLEX64:
                                case NiftiHeader.NIFTI_TYPE_FLOAT128:
                                case NiftiHeader.NIFTI_TYPE_RGB24:
                                case NiftiHeader.NIFTI_TYPE_COMPLEX128:
                                case NiftiHeader.NIFTI_TYPE_COMPLEX256:
                                case NiftiHeader.DT_ALL:
                                default:
                                    throw new IOException("Sorry, cannot yet read nifti-1 datatype " + NiftiHeader.decodeDatatype(header.datatype));
                            }
                            if(e==2){
                                v=-v;
                            }
                            this.disp[i][j][k][e] = v;
                        }
                    }
                }
            }

        } catch (IOException ex) {
            Logger.getLogger(DisplacementField.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public double[] getDisplacement(int i, int j, int k) {
        return disp[i][j][k];
    }

    public Point3D getDisplacementForPoint(Point3D p) {
        double[] pos = NiftiHandler.getPosition(header, 0);
        double[] v1v2 = NiftiHandler.getOrientation(header);
        float[] spacing = NiftiHandler.getSpacing(header);
        double[] v3 = new double[]{v1v2[1] * v1v2[5] - v1v2[2] * v1v2[4], v1v2[2] * v1v2[3] - v1v2[0] * v1v2[5], v1v2[0] * v1v2[4] - v1v2[1] * v1v2[3]};

        double[][] matrixData = {
            {v1v2[0], v1v2[3], v3[0]},
            {v1v2[1], v1v2[4], v3[1]},
            {v1v2[2], v1v2[5], v3[2]}};
        RealVector constants = new ArrayRealVector(new double[]{p.m_x1 - pos[0], p.m_x2 - pos[1], p.m_x3 - pos[2]}, false);
        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new QRDecomposition(n).getSolver();
        Point3D disp = null;
        try {
            RealVector solution = solver.solve(constants);
            int[] res = new int[3];
            res[0] = (int) Math.round(solution.getEntry(0)/spacing[0]);
            res[1] = (int) Math.round(solution.getEntry(1)/spacing[1]);
            res[2] = (int) Math.round(solution.getEntry(2)/spacing[2]);
            if (res[0] < this.disp.length && res[0] > 0 && res[1] < this.disp[0].length && res[1] > 0 && res[2] < this.disp[0][0].length && res[2] > 0) {
                double[] dx = this.disp[res[0]][res[1]][res[2]];
                disp = new Point3D(dx[0], dx[1], dx[2]);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return disp;
    }

    public static void main(String args[]) throws IOException {
        DisplacementField dips = new DisplacementField("C:\\Users\\nuno\\Desktop\\teste\\dispField.nii");
        System.out.println(dips.getDisplacement(0, 20, 0)[0] + " " + dips.getDisplacement(0, 40, 0)[1] + " " + dips.getDisplacement(0, 0, 0)[2]);
    }
}
