/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.registration;

import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;
import static org.spectrim.io.nifti.NiftiHandler.getImagesFromNifti;
import org.spectrim.view.ImageStack;

/**
 *
 * @author nuno
 */
public class ImageTools {

    public static double entropy(ImageStack ims, int nrbins, double outlierRate) {
        ims = normalizeImageStack(ims, nrbins, outlierRate);
        return entropy(ims);
    }

    public static double entropy(ImageStack ims) {
        TreeMap<Integer, Integer> map = new TreeMap<>();

        int nr = 0;
        for (int n = 0; n < ims.size(); n++) {
            Raster rast = ims.get(n).image;
            for (int i = 0; i < rast.getWidth(); i++) {
                for (int j = 0; j < rast.getHeight(); j++) {
                    int val = rast.getPixel(i, j, new int[]{0, 0, 0})[0];
                    if (map.containsKey(val)) {
                        map.put(val, map.get(val) + 1);
                    } else {
                        map.put(val, 1);
                    }
                    nr++;
                }
            }
        }

        double res = 0;
        ArrayList<Integer> values = new ArrayList(map.values());
        for (int i = 0; i < values.size(); i++) {
            double prob = (double) values.get(i) / (double) nr;
            res -= prob * Math.log(prob);
        }
        return res;
    }
//
//    private static TreeMap<Integer, Integer> hist(TreeMap<Integer, Integer> map, int nr, int nrbins, double outlierRate) {
//        ArrayList<Integer> listaux = new ArrayList<>(map.keySet());
//        Collections.reverse(listaux);
//        int removed = 0;
//        int i = 0;
//        int max = map.get(listaux.get(i));
//        while (((double) removed / (double) nr) < outlierRate) {
//            max = listaux.get(i);
//            if (i >= listaux.size()) {
//                removed = (int) outlierRate * nr;
//            } else {
//                removed += map.get(listaux.get(i));
//                map.remove(listaux.get(i));
//            }
//            i++;
//        }
//        map.put(listaux.get(i), map.get(listaux.get(i)) + removed);
//        TreeMap<Integer, Integer> hist = new TreeMap<>();
//        Iterator<Integer> iter = map.keySet().iterator();
//        while (iter.hasNext()) {
//            int intens = iter.next();
//            int bin = (int) Math.round(((double) intens / (double) max) * ((double) nrbins - 1));
//            if (hist.containsKey(bin)) {
//                hist.put(bin, hist.get(bin) + map.get(intens));
//            } else {
//                hist.put(bin, map.get(intens));
//            }
//        }
//        return hist;
//    }

    public static ImageStack normalizeImageStack(ImageStack ims, int nrbins, double outlierRate) {
//      REMOVES OUTLIERS AND RESETS SCALE ACORDING TO THE NR OF BINS  
        TreeMap<Integer, Integer> map = new TreeMap<>();
        int nr = 0;
        for (int n = 0; n < ims.size(); n++) {
            Raster rast = ims.get(n).image;
            for (int i = 0; i < rast.getWidth(); i++) {
                for (int j = 0; j < rast.getHeight(); j++) {
                    double val1 = rast.getPixel(i, j, new int[]{0, 0, 0})[0];
                    double val2 = rast.getPixel(i, j, new int[]{0, 0, 0})[1];
                    double val3 = rast.getPixel(i, j, new int[]{0, 0, 0})[2];
                    int val = (int) Math.round((val1 + val2 + val3) / 3);
                    if (map.containsKey(val)) {
                        map.put(val, map.get(val) + 1);
                    } else {
                        map.put(val, 1);
                    }
                    nr++;
                }
            }
        }
        ArrayList<Integer> listaux = new ArrayList<>(map.keySet());
        Collections.reverse(listaux);
        ImageStack res = new ImageStack();
        int removed = 0;
        int m = 0;
        int max = map.get(listaux.get(m));
        while (((double) removed / (double) nr) < outlierRate) {
            max = listaux.get(m);
            if (m >= listaux.size()) {
                removed = (int) nr;
            } else {
                removed += map.get(listaux.get(m));
            }
            m++;
        }
        for (int n = 0; n < ims.size(); n++) {
            Raster rast = ims.get(n).image;
            WritableRaster newRast = ims.get(n).image.createCompatibleWritableRaster();
            for (int i = 0; i < rast.getWidth(); i++) {
                for (int j = 0; j < rast.getHeight(); j++) {
                    double val1 = rast.getPixel(i, j, new int[]{0, 0, 0})[0];
                    double val2 = rast.getPixel(i, j, new int[]{0, 0, 0})[1];
                    double val3 = rast.getPixel(i, j, new int[]{0, 0, 0})[2];
                    int val = (int) Math.round((val1 + val2 + val3) / 3);
                    if (val > max) {
                        val = max;
                    }
                    val = (int) Math.round(((double) val / (double) max) * ((double) nrbins - 1));
                    newRast.setPixel(i, j, new int[]{val, val, val});
                }
            }
            res.add(ims.get(n).clone(newRast));
        }
        return res;
    }

    public static TreeMap<Integer, Double> histogram(ArrayList<Integer> intensities, int nrbins, double outlierRate, boolean normalize) {
//      REMOVES OUTLIERS AND RESETS SCALE ACORDING TO THE NR OF BINS  
        TreeMap<Integer, Integer> map = new TreeMap<>();
        TreeMap<Integer, Double> map2 = new TreeMap<>();
        int nr = 0;
        for (int n = 0; n < intensities.size(); n++) {
            if (map.containsKey(intensities.get(n))) {
                map.put(intensities.get(n), map.get(intensities.get(n)) + 1);
            } else {
                map.put(intensities.get(n), 1);
            }
            nr++;
        }
        ArrayList<Integer> listaux = new ArrayList<>(map.keySet());
        Collections.reverse(listaux);

        int removed = 0;
        int m = 0;
        int max = map.lastKey();
        int min = map.firstKey();
        
        if (outlierRate > 0) {
            while (((double) removed / (double) nr) < outlierRate) {
                max = listaux.get(m);
                if (m >= listaux.size()) {
                    removed = (int) nr;
                } else {
                    removed += map.get(listaux.get(m));
                }
                m++;
            }
        }

        for (int n = 0; n < intensities.size(); n++) {
            int val = intensities.get(n);
            if (val > max) {
                val = max;
            }
            val = (int) Math.round( Math.round(((double) (val-min) / ((double) max-min)) * ((double) nrbins - 1))*((max-min)/(double) nrbins)+min);
            if (normalize) {
                double area = (double) intensities.size()*((double)(max-min))/(double) nrbins;
                if (map2.containsKey(val)) {
                    map2.put(val, map2.get(val) + 1d / area);
                } else {
                    map2.put(val, 1d / area);
                }
            } else {
                if (map2.containsKey(val)) {
                    map2.put(val, map2.get(val) + 1d);
                } else {
                    map2.put(val, 1d);
                }
            }
        }
        return map2;
    }

    public static double jointEntropy(ImageStack ims1, ImageStack ims2, int nrbins, double outlierRate) {
        ims1 = normalizeImageStack(ims1, nrbins, outlierRate);
        ims2 = normalizeImageStack(ims2, nrbins, outlierRate);
        return jointEntropy(ims1, ims2);
    }

    public static double jointEntropy(ImageStack ims1, ImageStack ims2) {
        HashMap<Integer[], Integer> map = new HashMap<>();
        int nr = 0;
        for (int n = 0; n < ims1.size(); n++) {
            Raster rast1 = ims1.get(n).image;
            Raster rast2 = ims2.get(n).image;
            for (int i = 0; i < rast1.getWidth(); i++) {
                for (int j = 0; j < rast1.getHeight(); j++) {
                    Integer val[] = {rast1.getPixel(i, j, new int[]{0, 0, 0})[0], rast2.getPixel(i, j, new int[]{0, 0, 0})[0]};
                    if (map.containsKey(val)) {
                        map.put(val, map.get(val) + 1);
                    } else {
                        map.put(val, 1);
                    }
                    nr++;
                }
            }
        }
        double res = 0;
        Iterator iter = map.keySet().iterator();
        while (iter.hasNext()) {
            double prob = (double) map.get(iter.next()) / (double) nr;
            res -= prob * Math.log(prob);
        }
        return res;
    }

    public static double MI(ImageStack ims1, ImageStack ims2, int nrbins, double outlierRate) {
        ims1 = normalizeImageStack(ims1, nrbins, outlierRate);
        ims2 = normalizeImageStack(ims2, nrbins, outlierRate);
        return MI(ims1, ims2);
    }

    public static double MI(ImageStack ims1, ImageStack ims2) {
        double[] entrops = calcEntropies(ims1, ims2);
        return (entrops[0] + entrops[1] - entrops[2]);
    }

    public static double NMI(ImageStack ims1, ImageStack ims2, int nrbins, double outlierRate) {
        ims1 = normalizeImageStack(ims1, nrbins, outlierRate);
        ims2 = normalizeImageStack(ims2, nrbins, outlierRate);
        return NMI(ims1, ims2);
    }

    public static double NMI(ImageStack ims1, ImageStack ims2) {
        double[] entrops = calcEntropies(ims1, ims2);
        return (entrops[0] + entrops[1]) / entrops[2];
    }

    public static double[] calcEntropies(ImageStack ims1, ImageStack ims2, int nrbins, double outlierRate) {
        ims1 = normalizeImageStack(ims1, nrbins, outlierRate);
        ims2 = normalizeImageStack(ims2, nrbins, outlierRate);
        return calcEntropies(ims1, ims2);
    }

    public static double[] calcEntropies(ImageStack ims1, ImageStack ims2) {
        HashMap<Integer, Integer> map1 = new HashMap<>();
        HashMap<Integer, Integer> map2 = new HashMap<>();
        HashMap<String, Integer> mapJ = new HashMap<>();
        int nr = 0;
        for (int n = 0; n < ims1.size(); n++) {
            Raster rast1 = ims1.get(n).image;
            Raster rast2 = ims2.get(n).image;
            for (int i = 0; i < rast1.getWidth(); i++) {
                for (int j = 0; j < rast1.getHeight(); j++) {
                    Integer val1 = rast1.getPixel(i, j, new int[]{0, 0, 0})[0];
                    Integer val2 = rast2.getPixel(i, j, new int[]{0, 0, 0})[0];
                    String valJ = rast1.getPixel(i, j, new int[]{0, 0, 0})[0] + "_" + rast2.getPixel(i, j, new int[]{0, 0, 0})[0];
                    if (mapJ.containsKey(valJ)) {
                        mapJ.put(valJ, mapJ.get(valJ) + 1);
                    } else {
                        mapJ.put(valJ, 1);
                    }
                    if (map1.containsKey(val1)) {
                        map1.put(val1, map1.get(val1) + 1);
                    } else {
                        map1.put(val1, 1);
                    }
                    if (map2.containsKey(val2)) {
                        map2.put(val2, map2.get(val2) + 1);
                    } else {
                        map2.put(val2, 1);
                    }
                    nr++;
                }
            }
        }
        double ent1 = 0;
        double ent2 = 0;
        double entJ = 0;
        Iterator iter = map1.keySet().iterator();
        while (iter.hasNext()) {
            double prob = (double) map1.get(iter.next()) / (double) nr;
            ent1 -= prob * Math.log(prob);
        }
        iter = map2.keySet().iterator();
        while (iter.hasNext()) {
            double prob = (double) map2.get(iter.next()) / (double) nr;
            ent2 -= prob * Math.log(prob);
        }
        iter = mapJ.keySet().iterator();
        while (iter.hasNext()) {
            double prob = (double) mapJ.get(iter.next()) / (double) nr;
            entJ -= prob * Math.log(prob);
        }

        return new double[]{ent1, ent2, entJ};
    }

    public static double corrCoeff(ImageStack ims1, ImageStack ims2) {
        double mean1 = 0;
        double mean2 = 0;
        double var1 = 0;
        double var2 = 0;
        int nr = 0;
        double res = 0;
        for (int n = 0; n < ims1.size(); n++) {
            Raster rast1 = ims1.get(n).image;
            Raster rast2 = ims2.get(n).image;
            for (int i = 0; i < rast1.getWidth(); i++) {
                for (int j = 0; j < rast1.getHeight(); j++) {
                    Integer val1 = rast1.getPixel(i, j, new int[]{0, 0, 0})[0];
                    Integer val2 = rast2.getPixel(i, j, new int[]{0, 0, 0})[0];
                    mean1 += val1;
                    mean2 += val2;
                    nr++;
                }
            }
        }
        mean1 = mean1 / (double) nr;
        mean2 = mean2 / (double) nr;
        for (int n = 0; n < ims1.size(); n++) {
            Raster rast1 = ims1.get(n).image;
            Raster rast2 = ims2.get(n).image;
            for (int i = 0; i < rast1.getWidth(); i++) {
                for (int j = 0; j < rast1.getHeight(); j++) {
                    Integer val1 = rast1.getPixel(i, j, new int[]{0, 0, 0})[0];
                    Integer val2 = rast2.getPixel(i, j, new int[]{0, 0, 0})[0];
                    var1 += Math.pow((val1 - mean1), 2);
                    var2 += Math.pow((val2 - mean2), 2);
                }
            }
        }
        var1 = Math.sqrt(var1 / nr);
        var2 = Math.sqrt(var2 / nr);
        for (int n = 0; n < ims1.size(); n++) {
            Raster rast1 = ims1.get(n).image;
            Raster rast2 = ims2.get(n).image;
            for (int i = 0; i < rast1.getWidth(); i++) {
                for (int j = 0; j < rast1.getHeight(); j++) {
                    Integer val1 = rast1.getPixel(i, j, new int[]{0, 0, 0})[0];
                    Integer val2 = rast2.getPixel(i, j, new int[]{0, 0, 0})[0];
                    res += (val1 - mean1) * (val2 - mean2) / (nr * var1 * var2);
                }
            }
        }
        return res;
    }

    public static double corrCoeff(ImageStack ims1, ImageStack ims2, int nrbins, double outlierRate) {
        ims1 = normalizeImageStack(ims1, nrbins, outlierRate);
        ims2 = normalizeImageStack(ims2, nrbins, outlierRate);
        return corrCoeff(ims1, ims2);
    }

    public static double meanDiff(ImageStack ims1, ImageStack ims2) {
        int nr = 0;
        double res = 0;
        for (int n = 0; n < ims1.size(); n++) {
            Raster rast1 = ims1.get(n).image;
            Raster rast2 = ims2.get(n).image;
            for (int i = 0; i < rast1.getWidth(); i++) {
                for (int j = 0; j < rast1.getHeight(); j++) {
                    int[] val1 = rast1.getPixel(i, j, new int[]{0, 0, 0});
                    int[] val2 = rast2.getPixel(i, j, new int[]{0, 0, 0});
                    res += ((double) Math.abs(val1[0] - val2[0]) + (double) Math.abs(val1[1] - val2[1]) + (double) Math.abs(val1[2] - val2[2])) / 3;
                    nr++;
                }
            }
        }
        res = res / nr;
        return res;
    }

    public static double[] getSimilarityMeasures(ImageStack ims1, ImageStack ims2, int nrbins, double outlierRate) {
        int[] dims1 = ims1.getDimensions();
        int[] dims2 = ims2.getDimensions();
        if (dims1[0] == dims2[0] && dims1[1] == dims2[1] && dims1[2] == dims2[2]) {
            ims1 = normalizeImageStack(ims1, nrbins, outlierRate);
            ims2 = normalizeImageStack(ims2, nrbins, outlierRate);
            return getSimilarityMeasures(ims1, ims2);
        } else {
            return new double[]{0, 0, 0, 0};
        }
    }

    public static double[] getSimilarityMeasures(ImageStack ims1, ImageStack ims2) {
        double[] entrops = calcEntropies(ims1, ims2);
        double MI = entrops[0] + entrops[1] - entrops[2];
        double NMI = (entrops[0] + entrops[1]) / entrops[2];
        double corrcoeff = corrCoeff(ims1, ims2);
        double meanDiff = meanDiff(ims1, ims2);
        return new double[]{MI, NMI, corrcoeff, meanDiff};
    }

    public static void main(String args[]) throws IOException {
        double[] res = getSimilarityMeasures(getImagesFromNifti("C:\\Users\\nuno\\Desktop\\teste\\20130215_130940ep2ddiff3scanp3m128s003a1001.nii"), getImagesFromNifti("C:\\Users\\nuno\\Desktop\\teste\\20130215_130940ep2ddiff3scanp3m128s003a1001.nii"), 256, 0.0001);
        System.out.println(res[0] + " " + res[1] + " " + res[2] + " " + res[3]);
    }
}
