/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.report;

import com.pixelmed.dicom.DicomException;
import com.pixelmed.dicom.ImageToDicom;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import java.util.logging.Logger;
import java.util.logging.Level;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.model.GlobalModel;
import org.spectrim.popups.jMRUIResultsMapListener;

/**
 * Main graphical class to handle the MRS-reporting.
 * @author Giezi, JSlotboom
 */
public class StdReportFrame extends javax.swing.JFrame 
{
    public static final int STDMETAB    = 0;
    public static final int STDCOMP     = 1;
    public static final int CUSTOM      = 2;
    
    GlobalModel csimodel;
    
    //Jana
    private String ISP_OutFileName = null;

    /**
     * Creates new form stdReportFrame
     */
    public StdReportFrame( int panelType, GlobalModel csimodel, 
                           jMRUIResultsMapListener mapListener, 
                           boolean... showSpectraSeparatly ) 
    {
        initComponents( );
        this.setSize( 1000, 1020 );
        this.csimodel = csimodel;
        ISP_OutFileName = mapListener.getISPOutputFileName( );
        switch( panelType )
        {
            case STDMETAB: 
                            fileExportName = "CSI_REPORT_MAIN_METABOLITES";
                            jPanelContainer = new StdReportMainMetabolitesPanel( csimodel, mapListener );
                            break;
            case STDCOMP:
                            fileExportName = "CSI_REPORT_COMPARISON_CONTROL";
                            jPanelContainer = new StdReportComparisonControlPanel( csimodel, mapListener, showSpectraSeparatly[0] );
                            break;
            case CUSTOM:
                            fileExportName = "CUSTOM_REPORT";
                            jPanelContainer = new CstmReportPanel( csimodel, mapListener, "0", "00" );
                            break;
            default: 
                            System.out.println( "Not an option at the moment." );
                            dispose( );
                            break;
        }
        
        jPanelContainer.setSize( 1000,1000 );
        this.add( jPanelContainer );
        this.revalidate( );
        this.pack( );
        this.repaint( );
        this.setVisible( true );
    }

    public JPanel getContainerPanel( ) 
    {
        return jPanelContainer;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jMenuBar1 = new javax.swing.JMenuBar();
        jMenuFile = new javax.swing.JMenu();
        jMenuItemPng = new javax.swing.JMenuItem();
        jMenuItemISP = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("SpectrIm - Report");
        setMinimumSize(new java.awt.Dimension(1000, 1020));
        setResizable(false);

        jMenuBar1.setMaximumSize(new java.awt.Dimension(20, 5));
        jMenuBar1.setPreferredSize(new java.awt.Dimension(20, 20));

        jMenuFile.setText("File");
        jMenuFile.setMaximumSize(new java.awt.Dimension(30, 32767));

        jMenuItemPng.setText("Export to PNG...");
        jMenuItemPng.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemPngActionPerformed(evt);
            }
        });
        jMenuFile.add(jMenuItemPng);

        jMenuItemISP.setText("Export to ISP");
        jMenuItemISP.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemISPActionPerformed(evt);
            }
        });
        jMenuFile.add(jMenuItemISP);

        jMenuBar1.add(jMenuFile);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 280, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    public StdReportFrame( JMenu jMenuFile, 
                           JMenuItem jMenuItemPng, 
                           JMenuItem jMenuItemISP ) 
    {
        this.jMenuFile      = jMenuFile;
        this.jMenuItemPng   = jMenuItemPng;
        this.jMenuItemISP   = jMenuItemISP;
    }
    
    private void jMenuItemPngActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItemPngActionPerformed
        JFileChooser jFileChooserSaveJpg = new JFileChooser( );
        jFileChooserSaveJpg.setSelectedFile( new File( fileExportName + ".png" ) );
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "PNG FILES", "png", "png" );
        jFileChooserSaveJpg.setFileFilter( filter );
        int returnVal = jFileChooserSaveJpg.showSaveDialog( this );
        if ( returnVal == JFileChooser.APPROVE_OPTION ) 
        {
            File imageToSave = jFileChooserSaveJpg.getSelectedFile( );
            BufferedImage bi = new BufferedImage( jPanelContainer.getWidth( ), jPanelContainer.getHeight( ), BufferedImage.TYPE_INT_RGB );
            Graphics g = bi.createGraphics( );
            jPanelContainer.print( g );
            g.dispose( );
            try 
            {
                ImageIO.write( bi, "PNG", imageToSave );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( Report.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        } 
        else 
        {
            //log.append("Open command cancelled by user." + newline);
        }
    }//GEN-LAST:event_jMenuItemPngActionPerformed

    private void jMenuItemISPActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItemISPActionPerformed
        if( ISP_OutFileName != null )
        {    
         
            JFileChooser jFileChooserSaveJpg = new JFileChooser( );
            jFileChooserSaveJpg.setSelectedFile( new File(ISP_OutFileName+ File.separator + "results" + ".dcm" ) );
            FileNameExtensionFilter filter = new FileNameExtensionFilter( "DCM FILES", "dcm", "dcm" );
            jFileChooserSaveJpg.setFileFilter( filter );
     //   int returnVal = jFileChooserSaveJpg.showSaveDialog(this);
       // if (returnVal == JFileChooser.APPROVE_OPTION)
            {
                fileExportName = System.getProperty( "user.home" ) + File.separator + ".eMrui" + File.separator + "temp.png";  
                File imageToSave = new File( fileExportName );
                BufferedImage bi = new BufferedImage( jPanelContainer.getWidth( ), 
                                                      jPanelContainer.getHeight( ), 
                                                      BufferedImage.TYPE_INT_RGB );
                Graphics g = bi.createGraphics( );
                jPanelContainer.print( g );
                g.dispose( );
                try 
                {
                    ImageIO.write( bi, "PNG", imageToSave );
                    try 
                    {
                        //String inputFile,String outputFile,String patientName,String patientID,String studyID,String seriesNumber,String instanceNumber,String modality,String sopClass)
                        ImageToDicom imToSDicom = new ImageToDicom( imageToSave.getAbsolutePath( ),
                                                                    ISP_OutFileName + File.separator + "results" + ".dcm", //jFileChooserSaveJpg.getSelectedFile().getAbsolutePath(),
                                                                    csimodel.getPatientName( ),
                                                                    csimodel.getPatientID( ),
                                                                    csimodel.getStudyID( ),
                                                                    csimodel.getSeriesNumber( ),
                                                                    csimodel.getInstanceNumber( ),
                                                                    csimodel.getModality( ),
                                                                    null );    // String sopClass
                        File procCompl = new File( ISP_OutFileName + File.separator + "ProcessingComplete" );
                        try 
                        {
                            procCompl.createNewFile( );
                        } 
                        catch ( IOException e   ) 
                        {
                            // TODO Auto-generated catch block
                            Logger.getLogger( "Cannot write into " + ISP_OutFileName +":"+ e.getMessage( ) );  

                        }
                    } 
                    catch ( DicomException ex ) 
                    {
                        Logger.getLogger( StdReportFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( Report.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
        } 
    }//GEN-LAST:event_jMenuItemISPActionPerformed

   

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenu jMenuFile;
    private javax.swing.JMenuItem jMenuItemISP;
    private javax.swing.JMenuItem jMenuItemPng;
    // End of variables declaration//GEN-END:variables
    private String fileExportName;
    private JPanel jPanelContainer;
}
