/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings.processing;

import java.io.File;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.TreeMap;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.batchprocessing.BatchProcessing;
import org.spectrim.settings.SettingsGUI;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 * Class that allow users to define preprocessing actions to be performed in
 * batch-mode.
 * Updated 11.2018 JS.
 * @author nunopbarros
 */
public class PreprocessingActionUI extends javax.swing.JDialog 
{
   
    public final static int APODIZATION              = 0;
    public final static int QUALITY_FILTERING        = 1;
    public final static int BASELINE_REMOVAL         = 2;
    public final static int WATER_REMOVAL_HLSVD64    = 4;
    public final static int WATER_REMOVAL_TDFDFIT    = 5;
    public final static int AUTO_PHASING             = 3;
    public final static int FREQUENCY_SHIFT          = 6;
    public final static int ZEROFILLING              = 7;
    public final static int OFFSET_CORRECTION_FD     = 8;
    public final static int OFFSET_CORRECTION_TD     = 9;
    public final static int WATER_REMOVAL_HLSVDPRO  = 10;
    
    PreprocessingAction preprocessingActionObject;
    SettingsGUI parent;
    int number;

    // Panel order
    private int[] panelOrder;
    TreeMap<Integer, JPanel[]> hierarchyTree = new TreeMap<>( );

    // Frequency Shift
    private ArrayList<ReferencePeak> freqShiftPeaks;
    private double freqShiftMinPPM;
    private double freqShiftMaxPPM;
    private double freqShiftSNR;
    private double freqShiftMaxLocalShift;
    private String tdfdfitWaterReferenceModel;
    
    /**
     * Creates new form PreprocessingRuleUI.
     * @param parent
     * @param object
     * @param number
     */
    public PreprocessingActionUI( SettingsGUI parent, 
                                  PreprocessingAction object, 
                                  int number ) 
    {
        super( parent, true );

        this.parent = parent;
        this.preprocessingActionObject = object;
        this.number = number;
        initComponents( );

        // Panel order
        panelOrder = object.getPanelOrder( );

        // headerVariables
        this.jTextFieldName.setText( object.getName( ) );

        // Apodization
        this.jCheckBoxUseApodization.setSelected( object.isApodisationUse( ) );
        this.jRadioButtonApodisationGaussian.setSelected( object.getApodisationType( ) == BatchProcessing.APO_GAUSS );
        this.jRadioButtonApodisationExponential.setSelected( object.getApodisationType( ) == BatchProcessing.APO_EXP );
        this.jFormattedTextFieldApodisationLineBroadning.setValue( object.getApodisationLineBroadning( ) );

        // Quality
        this.jCheckBoxUseQuality.setSelected( object.isQualityUse( ) );
        this.jTextFieldQualityClassifier.setText( object.getQualityRFile( ) );
        this.jFormattedTextFieldQualityThreshold.setValue( object.getQualityThreshold( ) );

        // Baseline
        this.jCheckBoxUseBaseline.setSelected( object.isBaselineUse( ) );
        this.jFormattedTextFieldBaseline.setValue( object.getBaselineNR( ) );

        // Water Removal HLSVD64
        this.jCheckBoxUseHLSVD64WaterRemoval.setSelected( object.isHLSVD64WaterRemovalUse( ) );
        this.jFormattedTextFieldMaxWaterRemoval.setValue( object.getHLSVD64WaterRemovalMax( ) );
        this.jFormattedTextFieldMinWaterRemoval.setValue( object.getHLSVD64WaterRemovalMin( ) );
        this.jFormattedTextFieldNumberPeaksWaterRemoval.setValue( object.getHLSVD64WaterRemovalNumberOfPeaks( ) );
        
        // Water Removal HLSVDPro
        this.jCheckBoxUseHLSVDProWaterRemoval.setSelected( object.isHLSVDProWaterRemovalUse( ) );
        this.jFormattedTextFieldMaxWaterRemovalHLSVDPro.setValue( object.getHLSVDProWaterRemovalMax( ) );
        this.jFormattedTextFieldMinWaterRemovalHLSVDPro.setValue( object.getHLSVDProWaterRemovalMin( ) );
        this.jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro.setValue( object.getHLSVDProWaterRemovalNumberOfPeaks( ) );

        
        // Auto Phasing
        this.jCheckBoxUseAutoPhasing.setSelected( object.isAutoPhasingUse( ) );
        this.jFormattedTextFieldMaxAutoPhasing.setValue( object.getAutoPhasingMaxPPM( ) );
        this.jFormattedTextFieldMinAutoPhasing.setValue( object.getAutoPhasingMinPPM( ) );

        // Remove amplitude offset FD
        this.jCheckBoxUseRemoveOffsetFD.setSelected( object.isUseRemoveAmplitudeOffsetFD( ) );
        this.jFormattedTextFieldMinFDOffsetRange.setValue( object.getMinFDOffsetRange( ) );
        this.jFormattedTextFieldMaxFDOffsetRange.setValue( object.getMaxFDOffsetRange( ) );
        DecimalFormat fd_formatter = new DecimalFormat( "###.###" );
        if ( ( object.getminFDValue( ) == 0.0 ) && ( object.getminFDValue( ) == 0.0 ) )
        {
            jLabelFDOffsetRange.setText( "" );
        }
        else
        {
            jLabelFDOffsetRange.setText( "Possible range: [" + fd_formatter.format( object.getminFDValue( ) ) + ", " + fd_formatter.format( object.getmaxFDValue( ) ) + "]" );
        }
        
        // Remove amplitude offset TD
        this.jCheckBoxUseRemoveOffsetTD.setSelected( object.isUseRemoveAmplitudeOffsetTD( ) );
        this.jFormattedTextFieldMinTDOffsetRange.setValue( object.getMinTDOffsetRange( ) * 1000.0 );
        this.jFormattedTextFieldMaxTDOffsetRange.setValue( object.getMaxTDOffsetRange( ) * 1000.0 );
        
        DecimalFormat td_formatter = new DecimalFormat( "#####.#" );
        if ( ( object.getminTDValue( ) == 0.0 ) && ( object.getmaxTDValue( ) == 0.0 ) )
        {
            this.jLabelTDOffsetRange.setText( "" );
        }
        else
        {
            this.jLabelTDOffsetRange.setText( "Possible range: [" + td_formatter.format( object.getminTDValue( ) * 1000.0 ) + ", " + td_formatter.format( object.getmaxTDValue( ) * 1000.0 ) + "]" );
        }
        
        // Frequency Shift
        this.jCheckBoxUseFrequencyShift.setSelected( object.isFreqShiftUse( ) );
        freqShiftPeaks          = object.getFreqShiftPeaks( );
        freqShiftMinPPM         = object.getFreqShiftMinPPM( );
        freqShiftMaxPPM         = object.getFreqShiftMaxPPM( );
        freqShiftSNR            = object.getFreqShiftSNR( );
        freqShiftMaxLocalShift  = object.getFreqShiftMaxLocalShift( );

        // Zero Filling
        this.jCheckBoxUseZeroFilling.setSelected( object.isZeroFillingUse( ) );

        
        // TDFDFit water removal
        // Default model is always the currently loaded model!
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String waterModelBaseName = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME );
        String waterModelBaseNamePlusExtension = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH );
        
        jTDFDFitWaterRemovalModelNameTextField.setText( waterModelBaseName );
        
        tdfdfitWaterReferenceModel = waterModelBaseName;
        jCheckBoxUseTDFDFitWaterRemoval.setSelected( object.isTDFDFitWaterRemovalUse( ) );
        
        initUpDownButtons( );
        initTreeTables( );
        applyHierarchycalChanges( );

        pack( );
        setLocationRelativeTo( parent );
        setVisible( true );
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroup1 = new javax.swing.ButtonGroup();
        jPanelContainer = new javax.swing.JPanel();
        jPanelLabelContainer = new javax.swing.JPanel();
        jPanelLabels = new javax.swing.JPanel();
        jPanelApodizationLabel = new javax.swing.JPanel();
        jCheckBoxUseApodization = new javax.swing.JCheckBox();
        jPanelQualityLabel = new javax.swing.JPanel();
        jCheckBoxUseQuality = new javax.swing.JCheckBox();
        jPanelBaselineLabel = new javax.swing.JPanel();
        jCheckBoxUseBaseline = new javax.swing.JCheckBox();
        jPanelWaterRemovalLabelHLSVD64 = new javax.swing.JPanel();
        jCheckBoxUseHLSVD64WaterRemoval = new javax.swing.JCheckBox();
        jPanelWaterRemovalLabelTDFDFit = new javax.swing.JPanel();
        jCheckBoxUseTDFDFitWaterRemoval = new javax.swing.JCheckBox();
        jPanelAutoPhasingLabel = new javax.swing.JPanel();
        jCheckBoxUseAutoPhasing = new javax.swing.JCheckBox();
        jPanelFrqShiftLabel = new javax.swing.JPanel();
        jCheckBoxUseFrequencyShift = new javax.swing.JCheckBox();
        jPanelZeroFillingLabel = new javax.swing.JPanel();
        jCheckBoxUseZeroFilling = new javax.swing.JCheckBox();
        jPanelRemoveOffsetFDLabel = new javax.swing.JPanel();
        jCheckBoxUseRemoveOffsetFD = new javax.swing.JCheckBox();
        jPanelRemoveOffsetTDLabel = new javax.swing.JPanel();
        jCheckBoxUseRemoveOffsetTD = new javax.swing.JCheckBox();
        jPanelWaterRemovalLabelHLSVDPro = new javax.swing.JPanel();
        jCheckBoxUseHLSVDProWaterRemoval = new javax.swing.JCheckBox();
        jPanelUpDownButtons = new javax.swing.JPanel();
        jSeparator1 = new javax.swing.JSeparator();
        jPanelVariables = new javax.swing.JPanel();
        jPanelApodization = new javax.swing.JPanel();
        jLabelAppodisationType = new javax.swing.JLabel();
        jRadioButtonApodisationGaussian = new javax.swing.JRadioButton();
        jRadioButtonApodisationExponential = new javax.swing.JRadioButton();
        jLabelAppodisationLineBroadning = new javax.swing.JLabel();
        jFormattedTextFieldApodisationLineBroadning = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jPanel4 = new javax.swing.JPanel();
        jPanelQuality = new javax.swing.JPanel();
        jLabelQualityClassifier = new javax.swing.JLabel();
        jTextFieldQualityClassifier = new javax.swing.JTextField();
        jLabelQualityThreshold = new javax.swing.JLabel();
        jFormattedTextFieldQualityThreshold = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jButtonQualityClassifier = new javax.swing.JButton();
        jPanelBaseline = new javax.swing.JPanel();
        jLabelBaseline = new javax.swing.JLabel();
        jFormattedTextFieldBaseline = new javax.swing.JFormattedTextField(NumberFormat.getIntegerInstance());
        jPanel5 = new javax.swing.JPanel();
        jPanelWaterRemovalHLSVD64 = new javax.swing.JPanel();
        jLabelMinWaterRemoval = new javax.swing.JLabel();
        jFormattedTextFieldMinWaterRemoval = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelMaxWaterRemovalHLSVD64 = new javax.swing.JLabel();
        jFormattedTextFieldMaxWaterRemoval = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelNumberPeaksWaterRemoval = new javax.swing.JLabel();
        jFormattedTextFieldNumberPeaksWaterRemoval = new javax.swing.JFormattedTextField(NumberFormat.getIntegerInstance());
        jButtonWaterRemovalResetToDefault = new javax.swing.JButton();
        jPanel6 = new javax.swing.JPanel();
        jPanelWaterRemovalTDFDFit = new javax.swing.JPanel();
        jLabelWaterModelName = new javax.swing.JLabel();
        jTDFDFitWaterRemovalModelNameTextField = new javax.swing.JTextField();
        jButtonTDFDFitWaterRemovaModelNameDefiner = new javax.swing.JButton();
        jPanel8 = new javax.swing.JPanel();
        jPanelAutoPhasing = new javax.swing.JPanel();
        jLabelMinAutoPhasing = new javax.swing.JLabel();
        jFormattedTextFieldMinAutoPhasing = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelMaxAutoPhasing = new javax.swing.JLabel();
        jFormattedTextFieldMaxAutoPhasing = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jPanel7 = new javax.swing.JPanel();
        jPanelFrequencyShift = new javax.swing.JPanel();
        jButtonSetFrqShift = new javax.swing.JButton();
        jPanelZeroFilling = new javax.swing.JPanel();
        jPanelRemoveOffsetFD = new javax.swing.JPanel();
        jLabelMinFDOffsetRange = new javax.swing.JLabel();
        jFormattedTextFieldMinFDOffsetRange = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelMaxFDOffsetRange = new javax.swing.JLabel();
        jFormattedTextFieldMaxFDOffsetRange = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jspaceFillerPanel_FDP = new javax.swing.JPanel();
        jLabelFDOffsetRange = new javax.swing.JLabel();
        jPanelRemoveOffsetTD = new javax.swing.JPanel();
        jLabelMinTDOffsetRange = new javax.swing.JLabel();
        jFormattedTextFieldMinTDOffsetRange = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelMaxTDOffsetRange = new javax.swing.JLabel();
        jFormattedTextFieldMaxTDOffsetRange = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jspaceFillerPanel_TDP = new javax.swing.JPanel();
        jLabelTDOffsetRange = new javax.swing.JLabel();
        jLabelMaxTDOffsetRange1 = new javax.swing.JLabel();
        jPanelWaterRemovalHLSVDPro = new javax.swing.JPanel();
        jLabelMinWaterRemovalHLSVDPro = new javax.swing.JLabel();
        jFormattedTextFieldMinWaterRemovalHLSVDPro = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelMaxWaterRemovalHLSVDPro = new javax.swing.JLabel();
        jFormattedTextFieldMaxWaterRemovalHLSVDPro = new javax.swing.JFormattedTextField(NumberFormat.getInstance());
        jLabelNumberPeaksWaterRemovalHLSVDPro = new javax.swing.JLabel();
        jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro = new javax.swing.JFormattedTextField(NumberFormat.getIntegerInstance());
        jButtonWaterRemovalResetToDefaultHLSVDPro = new javax.swing.JButton();
        jPanel9 = new javax.swing.JPanel();
        jSeparator2 = new javax.swing.JSeparator();
        jPanel1 = new javax.swing.JPanel();
        jButtonSave = new javax.swing.JButton();
        jButtonCancel = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jTextFieldName = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Pre-processing Action Editor");
        setResizable(false);
        getContentPane().setLayout(new java.awt.GridBagLayout());

        jPanelContainer.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 1));
        jPanelContainer.setLayout(new java.awt.GridBagLayout());

        jPanelLabelContainer.setLayout(new javax.swing.BoxLayout(jPanelLabelContainer, javax.swing.BoxLayout.LINE_AXIS));

        jPanelLabels.setLayout(new java.awt.GridLayout(0, 1));

        jPanelApodizationLabel.setLayout(new javax.swing.BoxLayout(jPanelApodizationLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseApodization.setText("Apodization");
        jPanelApodizationLabel.add(jCheckBoxUseApodization);

        jPanelLabels.add(jPanelApodizationLabel);

        jPanelQualityLabel.setLayout(new javax.swing.BoxLayout(jPanelQualityLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseQuality.setText("Quality Filtering ");
        jPanelQualityLabel.add(jCheckBoxUseQuality);

        jPanelLabels.add(jPanelQualityLabel);

        jPanelBaselineLabel.setLayout(new javax.swing.BoxLayout(jPanelBaselineLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseBaseline.setText("Baseline");
        jPanelBaselineLabel.add(jCheckBoxUseBaseline);

        jPanelLabels.add(jPanelBaselineLabel);

        jPanelWaterRemovalLabelHLSVD64.setLayout(new javax.swing.BoxLayout(jPanelWaterRemovalLabelHLSVD64, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseHLSVD64WaterRemoval.setText("Water Removal (HLSVD-64)");
        jPanelWaterRemovalLabelHLSVD64.add(jCheckBoxUseHLSVD64WaterRemoval);

        jPanelLabels.add(jPanelWaterRemovalLabelHLSVD64);

        jPanelWaterRemovalLabelTDFDFit.setLayout(new javax.swing.BoxLayout(jPanelWaterRemovalLabelTDFDFit, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseTDFDFitWaterRemoval.setText("Water Removal (TDFDFit)");
        jPanelWaterRemovalLabelTDFDFit.add(jCheckBoxUseTDFDFitWaterRemoval);

        jPanelLabels.add(jPanelWaterRemovalLabelTDFDFit);

        jPanelAutoPhasingLabel.setLayout(new javax.swing.BoxLayout(jPanelAutoPhasingLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseAutoPhasing.setText("Auto Phasing");
        jPanelAutoPhasingLabel.add(jCheckBoxUseAutoPhasing);

        jPanelLabels.add(jPanelAutoPhasingLabel);

        jPanelFrqShiftLabel.setLayout(new javax.swing.BoxLayout(jPanelFrqShiftLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseFrequencyShift.setText("Frequency Shift");
        jPanelFrqShiftLabel.add(jCheckBoxUseFrequencyShift);

        jPanelLabels.add(jPanelFrqShiftLabel);

        jPanelZeroFillingLabel.setLayout(new javax.swing.BoxLayout(jPanelZeroFillingLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseZeroFilling.setText("Zero Filling");
        jCheckBoxUseZeroFilling.setEnabled(false);
        jPanelZeroFillingLabel.add(jCheckBoxUseZeroFilling);

        jPanelLabels.add(jPanelZeroFillingLabel);

        jPanelRemoveOffsetFDLabel.setLayout(new javax.swing.BoxLayout(jPanelRemoveOffsetFDLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseRemoveOffsetFD.setText("Remove offset frequency domain");
        jPanelRemoveOffsetFDLabel.add(jCheckBoxUseRemoveOffsetFD);

        jPanelLabels.add(jPanelRemoveOffsetFDLabel);

        jPanelRemoveOffsetTDLabel.setLayout(new javax.swing.BoxLayout(jPanelRemoveOffsetTDLabel, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseRemoveOffsetTD.setText("Remove offset time domain");
        jPanelRemoveOffsetTDLabel.add(jCheckBoxUseRemoveOffsetTD);

        jPanelLabels.add(jPanelRemoveOffsetTDLabel);

        jPanelWaterRemovalLabelHLSVDPro.setLayout(new javax.swing.BoxLayout(jPanelWaterRemovalLabelHLSVDPro, javax.swing.BoxLayout.LINE_AXIS));

        jCheckBoxUseHLSVDProWaterRemoval.setText("Water Removal (HLSVDPro)");
        jPanelWaterRemovalLabelHLSVDPro.add(jCheckBoxUseHLSVDProWaterRemoval);

        jPanelLabels.add(jPanelWaterRemovalLabelHLSVDPro);

        jPanelLabelContainer.add(jPanelLabels);

        jPanelUpDownButtons.setLayout(new java.awt.GridLayout(0, 2));
        jPanelLabelContainer.add(jPanelUpDownButtons);

        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jPanelLabelContainer.add(jSeparator1);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        jPanelContainer.add(jPanelLabelContainer, gridBagConstraints);

        jPanelVariables.setLayout(new java.awt.GridLayout(0, 1));

        jPanelApodization.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelApodization.setLayout(new java.awt.GridBagLayout());

        jLabelAppodisationType.setText("Type: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelApodization.add(jLabelAppodisationType, gridBagConstraints);

        buttonGroup1.add(jRadioButtonApodisationGaussian);
        jRadioButtonApodisationGaussian.setSelected(true);
        jRadioButtonApodisationGaussian.setText("Gaussian");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        jPanelApodization.add(jRadioButtonApodisationGaussian, gridBagConstraints);

        buttonGroup1.add(jRadioButtonApodisationExponential);
        jRadioButtonApodisationExponential.setText("Exponential");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        jPanelApodization.add(jRadioButtonApodisationExponential, gridBagConstraints);

        jLabelAppodisationLineBroadning.setText("Line Broadning Factor (Hz): ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 19, 0, 0);
        jPanelApodization.add(jLabelAppodisationLineBroadning, gridBagConstraints);

        jFormattedTextFieldApodisationLineBroadning.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldApodisationLineBroadning.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldApodisationLineBroadning.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelApodization.add(jFormattedTextFieldApodisationLineBroadning, gridBagConstraints);

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 288, Short.MAX_VALUE)
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelApodization.add(jPanel4, gridBagConstraints);

        jPanelVariables.add(jPanelApodization);

        jPanelQuality.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelQuality.setLayout(new java.awt.GridBagLayout());

        jLabelQualityClassifier.setText("Classifier File: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanelQuality.add(jLabelQualityClassifier, gridBagConstraints);

        jTextFieldQualityClassifier.setEditable(false);
        jTextFieldQualityClassifier.setMaximumSize(new java.awt.Dimension(2147483647, 20));
        jTextFieldQualityClassifier.setMinimumSize(new java.awt.Dimension(50, 20));
        jTextFieldQualityClassifier.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelQuality.add(jTextFieldQualityClassifier, gridBagConstraints);

        jLabelQualityThreshold.setText("Probability Threshold: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelQuality.add(jLabelQualityThreshold, gridBagConstraints);

        jFormattedTextFieldQualityThreshold.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldQualityThreshold.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldQualityThreshold.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        jPanelQuality.add(jFormattedTextFieldQualityThreshold, gridBagConstraints);

        jButtonQualityClassifier.setText("Change...");
        jButtonQualityClassifier.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonQualityClassifierActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelQuality.add(jButtonQualityClassifier, gridBagConstraints);

        jPanelVariables.add(jPanelQuality);

        jPanelBaseline.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelBaseline.setLayout(new java.awt.GridBagLayout());

        jLabelBaseline.setText("Number of Points: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelBaseline.add(jLabelBaseline, gridBagConstraints);

        jFormattedTextFieldBaseline.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldBaseline.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldBaseline.setName(""); // NOI18N
        jFormattedTextFieldBaseline.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelBaseline.add(jFormattedTextFieldBaseline, gridBagConstraints);

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 566, Short.MAX_VALUE)
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelBaseline.add(jPanel5, gridBagConstraints);

        jPanelVariables.add(jPanelBaseline);

        jPanelWaterRemovalHLSVD64.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelWaterRemovalHLSVD64.setLayout(new java.awt.GridBagLayout());

        jLabelMinWaterRemoval.setText("Min PPM: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelWaterRemovalHLSVD64.add(jLabelMinWaterRemoval, gridBagConstraints);

        jFormattedTextFieldMinWaterRemoval.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinWaterRemoval.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinWaterRemoval.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVD64.add(jFormattedTextFieldMinWaterRemoval, gridBagConstraints);

        jLabelMaxWaterRemovalHLSVD64.setText("Max PPM: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 9, 0, 0);
        jPanelWaterRemovalHLSVD64.add(jLabelMaxWaterRemovalHLSVD64, gridBagConstraints);

        jFormattedTextFieldMaxWaterRemoval.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxWaterRemoval.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxWaterRemoval.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVD64.add(jFormattedTextFieldMaxWaterRemoval, gridBagConstraints);

        jLabelNumberPeaksWaterRemoval.setText("#Components: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanelWaterRemovalHLSVD64.add(jLabelNumberPeaksWaterRemoval, gridBagConstraints);

        jFormattedTextFieldNumberPeaksWaterRemoval.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jFormattedTextFieldNumberPeaksWaterRemoval.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldNumberPeaksWaterRemoval.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVD64.add(jFormattedTextFieldNumberPeaksWaterRemoval, gridBagConstraints);

        jButtonWaterRemovalResetToDefault.setText("Reset to Default");
        jButtonWaterRemovalResetToDefault.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonWaterRemovalResetToDefaultActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanelWaterRemovalHLSVD64.add(jButtonWaterRemovalResetToDefault, gridBagConstraints);

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 161, Short.MAX_VALUE)
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVD64.add(jPanel6, gridBagConstraints);

        jPanelVariables.add(jPanelWaterRemovalHLSVD64);

        jPanelWaterRemovalTDFDFit.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelWaterRemovalTDFDFit.setLayout(new java.awt.GridBagLayout());

        jLabelWaterModelName.setText("Model name:  ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelWaterRemovalTDFDFit.add(jLabelWaterModelName, gridBagConstraints);

        jTDFDFitWaterRemovalModelNameTextField.setText("no model defined");
        jTDFDFitWaterRemovalModelNameTextField.setMinimumSize(new java.awt.Dimension(200, 30));
        jTDFDFitWaterRemovalModelNameTextField.setName(""); // NOI18N
        jTDFDFitWaterRemovalModelNameTextField.setPreferredSize(new java.awt.Dimension(250, 30));
        jTDFDFitWaterRemovalModelNameTextField.setRequestFocusEnabled(false);
        jPanelWaterRemovalTDFDFit.add(jTDFDFitWaterRemovalModelNameTextField, new java.awt.GridBagConstraints());

        jButtonTDFDFitWaterRemovaModelNameDefiner.setText("Define TDFDFit-water removal model");
        jButtonTDFDFitWaterRemovaModelNameDefiner.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonTDFDFitWaterRemovaModelNameDefinerActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanelWaterRemovalTDFDFit.add(jButtonTDFDFitWaterRemovaModelNameDefiner, gridBagConstraints);

        javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 182, Short.MAX_VALUE)
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalTDFDFit.add(jPanel8, gridBagConstraints);

        jPanelVariables.add(jPanelWaterRemovalTDFDFit);

        jPanelAutoPhasing.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelAutoPhasing.setLayout(new java.awt.GridBagLayout());

        jLabelMinAutoPhasing.setText("Min PPM: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelAutoPhasing.add(jLabelMinAutoPhasing, gridBagConstraints);

        jFormattedTextFieldMinAutoPhasing.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinAutoPhasing.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinAutoPhasing.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelAutoPhasing.add(jFormattedTextFieldMinAutoPhasing, gridBagConstraints);

        jLabelMaxAutoPhasing.setText("Max PPM: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 9, 0, 0);
        jPanelAutoPhasing.add(jLabelMaxAutoPhasing, gridBagConstraints);

        jFormattedTextFieldMaxAutoPhasing.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxAutoPhasing.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxAutoPhasing.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelAutoPhasing.add(jFormattedTextFieldMaxAutoPhasing, gridBagConstraints);

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 477, Short.MAX_VALUE)
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelAutoPhasing.add(jPanel7, gridBagConstraints);

        jPanelVariables.add(jPanelAutoPhasing);

        jPanelFrequencyShift.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelFrequencyShift.setLayout(new java.awt.BorderLayout());

        jButtonSetFrqShift.setText("Edit Frequency Shift Settings");
        jButtonSetFrqShift.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSetFrqShiftActionPerformed(evt);
            }
        });
        jPanelFrequencyShift.add(jButtonSetFrqShift, java.awt.BorderLayout.CENTER);

        jPanelVariables.add(jPanelFrequencyShift);

        jPanelZeroFilling.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        javax.swing.GroupLayout jPanelZeroFillingLayout = new javax.swing.GroupLayout(jPanelZeroFilling);
        jPanelZeroFilling.setLayout(jPanelZeroFillingLayout);
        jPanelZeroFillingLayout.setHorizontalGroup(
            jPanelZeroFillingLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 770, Short.MAX_VALUE)
        );
        jPanelZeroFillingLayout.setVerticalGroup(
            jPanelZeroFillingLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        jPanelVariables.add(jPanelZeroFilling);

        jPanelRemoveOffsetFD.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelRemoveOffsetFD.setLayout(new java.awt.GridBagLayout());

        jLabelMinFDOffsetRange.setText("Min [ppm] : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelRemoveOffsetFD.add(jLabelMinFDOffsetRange, gridBagConstraints);

        jFormattedTextFieldMinFDOffsetRange.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinFDOffsetRange.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinFDOffsetRange.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelRemoveOffsetFD.add(jFormattedTextFieldMinFDOffsetRange, gridBagConstraints);

        jLabelMaxFDOffsetRange.setText("Max [ppm] : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 9, 0, 0);
        jPanelRemoveOffsetFD.add(jLabelMaxFDOffsetRange, gridBagConstraints);

        jFormattedTextFieldMaxFDOffsetRange.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxFDOffsetRange.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxFDOffsetRange.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelRemoveOffsetFD.add(jFormattedTextFieldMaxFDOffsetRange, gridBagConstraints);

        jLabelFDOffsetRange.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabelFDOffsetRange.setText("Possible range:");
        jLabelFDOffsetRange.setToolTipText("");

        javax.swing.GroupLayout jspaceFillerPanel_FDPLayout = new javax.swing.GroupLayout(jspaceFillerPanel_FDP);
        jspaceFillerPanel_FDP.setLayout(jspaceFillerPanel_FDPLayout);
        jspaceFillerPanel_FDPLayout.setHorizontalGroup(
            jspaceFillerPanel_FDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 450, Short.MAX_VALUE)
            .addGroup(jspaceFillerPanel_FDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jspaceFillerPanel_FDPLayout.createSequentialGroup()
                    .addGap(0, 181, Short.MAX_VALUE)
                    .addComponent(jLabelFDOffsetRange)
                    .addGap(0, 181, Short.MAX_VALUE)))
        );
        jspaceFillerPanel_FDPLayout.setVerticalGroup(
            jspaceFillerPanel_FDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
            .addGroup(jspaceFillerPanel_FDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jspaceFillerPanel_FDPLayout.createSequentialGroup()
                    .addGap(0, 7, Short.MAX_VALUE)
                    .addComponent(jLabelFDOffsetRange)
                    .addGap(0, 7, Short.MAX_VALUE)))
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelRemoveOffsetFD.add(jspaceFillerPanel_FDP, gridBagConstraints);

        jPanelVariables.add(jPanelRemoveOffsetFD);

        jPanelRemoveOffsetTD.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelRemoveOffsetTD.setLayout(new java.awt.GridBagLayout());

        jLabelMinTDOffsetRange.setText("Min [msec] : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelRemoveOffsetTD.add(jLabelMinTDOffsetRange, gridBagConstraints);

        jFormattedTextFieldMinTDOffsetRange.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinTDOffsetRange.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinTDOffsetRange.setName(""); // NOI18N
        jFormattedTextFieldMinTDOffsetRange.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelRemoveOffsetTD.add(jFormattedTextFieldMinTDOffsetRange, gridBagConstraints);

        jLabelMaxTDOffsetRange.setText("Max [msec]: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 9, 0, 0);
        jPanelRemoveOffsetTD.add(jLabelMaxTDOffsetRange, gridBagConstraints);

        jFormattedTextFieldMaxTDOffsetRange.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxTDOffsetRange.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxTDOffsetRange.setName(""); // NOI18N
        jFormattedTextFieldMaxTDOffsetRange.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelRemoveOffsetTD.add(jFormattedTextFieldMaxTDOffsetRange, gridBagConstraints);

        jLabelTDOffsetRange.setText("Possible range:");

        javax.swing.GroupLayout jspaceFillerPanel_TDPLayout = new javax.swing.GroupLayout(jspaceFillerPanel_TDP);
        jspaceFillerPanel_TDP.setLayout(jspaceFillerPanel_TDPLayout);
        jspaceFillerPanel_TDPLayout.setHorizontalGroup(
            jspaceFillerPanel_TDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 445, Short.MAX_VALUE)
            .addGroup(jspaceFillerPanel_TDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jspaceFillerPanel_TDPLayout.createSequentialGroup()
                    .addGap(0, 178, Short.MAX_VALUE)
                    .addComponent(jLabelTDOffsetRange)
                    .addGap(0, 179, Short.MAX_VALUE)))
        );
        jspaceFillerPanel_TDPLayout.setVerticalGroup(
            jspaceFillerPanel_TDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
            .addGroup(jspaceFillerPanel_TDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jspaceFillerPanel_TDPLayout.createSequentialGroup()
                    .addGap(0, 7, Short.MAX_VALUE)
                    .addComponent(jLabelTDOffsetRange)
                    .addGap(0, 7, Short.MAX_VALUE)))
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelRemoveOffsetTD.add(jspaceFillerPanel_TDP, gridBagConstraints);

        jLabelMaxTDOffsetRange1.setText("Max [msec]: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 9, 0, 0);
        jPanelRemoveOffsetTD.add(jLabelMaxTDOffsetRange1, gridBagConstraints);

        jPanelVariables.add(jPanelRemoveOffsetTD);

        jPanelWaterRemovalHLSVDPro.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelWaterRemovalHLSVDPro.setLayout(new java.awt.GridBagLayout());

        jLabelMinWaterRemovalHLSVDPro.setText("Min PPM: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelWaterRemovalHLSVDPro.add(jLabelMinWaterRemovalHLSVDPro, gridBagConstraints);

        jFormattedTextFieldMinWaterRemovalHLSVDPro.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinWaterRemovalHLSVDPro.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMinWaterRemovalHLSVDPro.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVDPro.add(jFormattedTextFieldMinWaterRemovalHLSVDPro, gridBagConstraints);

        jLabelMaxWaterRemovalHLSVDPro.setText("Max PPM: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 9, 0, 0);
        jPanelWaterRemovalHLSVDPro.add(jLabelMaxWaterRemovalHLSVDPro, gridBagConstraints);

        jFormattedTextFieldMaxWaterRemovalHLSVDPro.setMaximumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxWaterRemovalHLSVDPro.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldMaxWaterRemovalHLSVDPro.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVDPro.add(jFormattedTextFieldMaxWaterRemovalHLSVDPro, gridBagConstraints);

        jLabelNumberPeaksWaterRemovalHLSVDPro.setText("#Components: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanelWaterRemovalHLSVDPro.add(jLabelNumberPeaksWaterRemovalHLSVDPro, gridBagConstraints);

        jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro.setMinimumSize(new java.awt.Dimension(50, 20));
        jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVDPro.add(jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro, gridBagConstraints);

        jButtonWaterRemovalResetToDefaultHLSVDPro.setText("Reset to Default");
        jButtonWaterRemovalResetToDefaultHLSVDPro.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonWaterRemovalResetToDefaultHLSVDProActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanelWaterRemovalHLSVDPro.add(jButtonWaterRemovalResetToDefaultHLSVDPro, gridBagConstraints);

        javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
        jPanel9.setLayout(jPanel9Layout);
        jPanel9Layout.setHorizontalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 161, Short.MAX_VALUE)
        );
        jPanel9Layout.setVerticalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelWaterRemovalHLSVDPro.add(jPanel9, gridBagConstraints);

        jPanelVariables.add(jPanelWaterRemovalHLSVDPro);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.weightx = 1.0;
        jPanelContainer.add(jPanelVariables, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(jPanelContainer, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 1017;
        gridBagConstraints.ipady = 9;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 0);
        getContentPane().add(jSeparator2, gridBagConstraints);

        jPanel1.setLayout(new java.awt.GridBagLayout());

        jButtonSave.setText("Save");
        jButtonSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSaveActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel1.add(jButtonSave, gridBagConstraints);

        jButtonCancel.setText("Cancel");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCancelActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel1.add(jButtonCancel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        getContentPane().add(jPanel1, gridBagConstraints);

        jLabel1.setText("Name  ");
        jLabel1.setToolTipText("");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(jLabel1, gridBagConstraints);

        jTextFieldName.setMaximumSize(new java.awt.Dimension(50, 30));
        jTextFieldName.setMinimumSize(new java.awt.Dimension(50, 30));
        jTextFieldName.setName(""); // NOI18N
        jTextFieldName.setPreferredSize(new java.awt.Dimension(50, 30));
        jTextFieldName.setRequestFocusEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jTextFieldName, gridBagConstraints);

        jLabel2.setForeground(new java.awt.Color(0, 0, 255));
        jLabel2.setText("Note: For help setting up Automatic Quality Filtering, please contact nuno.barros@insel.ch");
        jLabel2.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        getContentPane().add(jLabel2, gridBagConstraints);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonQualityClassifierActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonQualityClassifierActionPerformed
        String file = parent.selectFile(preprocessingActionObject.getQualityRFile(), "RData");
        if ((new File(file)).exists()) {
            jTextFieldQualityClassifier.setText(file);
            preprocessingActionObject.setQualityRFile(file);
        }
    }//GEN-LAST:event_jButtonQualityClassifierActionPerformed

    private void jButtonWaterRemovalResetToDefaultActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonWaterRemovalResetToDefaultActionPerformed
        jFormattedTextFieldMaxWaterRemoval.setValue(9);
        jFormattedTextFieldMinWaterRemoval.setValue(4);
        jFormattedTextFieldNumberPeaksWaterRemoval.setValue(8);
    }//GEN-LAST:event_jButtonWaterRemovalResetToDefaultActionPerformed

    private void jButtonSetFrqShiftActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSetFrqShiftActionPerformed
        ShiftCorrectionOptionsEditor editor = new ShiftCorrectionOptionsEditor(this, freqShiftPeaks, freqShiftSNR,
                freqShiftMinPPM, freqShiftMaxPPM, freqShiftMaxLocalShift);
    }//GEN-LAST:event_jButtonSetFrqShiftActionPerformed

    
    
    /**
     * Routine called after save the preprocessing action is pressed.
     * @param evt 
     */
    private void jButtonSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSaveActionPerformed
        
        
        if ( jTextFieldName.getText( ).trim( ).matches( "" ) ) 
        {
            JOptionPane.showMessageDialog( this, "Please provide a valid name.", "Invalid Name", JOptionPane.WARNING_MESSAGE );
            this.jTextFieldName.requestFocus( );
        }
        else if ( parent.isPreprocessingActionNameRepeated( jTextFieldName.getText( ), number ) ) 
        {
            JOptionPane.showMessageDialog( this, "The chosen name is already in use.", "Invalid Name", JOptionPane.WARNING_MESSAGE );
            this.jTextFieldName.requestFocus( );
        } 
        else if ( ( (Number ) this.jFormattedTextFieldQualityThreshold.getValue( ) ).doubleValue( ) <= 0 || 
                  ( (Number ) this.jFormattedTextFieldQualityThreshold.getValue( ) ).doubleValue( ) >= 1 ) 
        {
            JOptionPane.showMessageDialog(this, "The quality control classifier outputs a estimation of the probability of a rater accepting a given spectrum. Please select a threshold between 0 and 1.", "Invalid Quality Threshold", JOptionPane.WARNING_MESSAGE);
            this.jFormattedTextFieldQualityThreshold.setValue( 0.5d );
            this.jFormattedTextFieldQualityThreshold.requestFocus( );
        } 
        else if( this.jCheckBoxUseAutoPhasing.isSelected( ) && 
                ( ( Number ) this.jFormattedTextFieldMinAutoPhasing.getValue( ) ).doubleValue( ) >= ( ( Number ) this.jFormattedTextFieldMaxAutoPhasing.getValue( ) ).doubleValue( ) )
        {
            JOptionPane.showMessageDialog( this, "Invalid Autophasing PPM range", 
                                           "Invalid Range", JOptionPane.WARNING_MESSAGE );
            this.jFormattedTextFieldMaxAutoPhasing.requestFocus( );
        }
        else if( this.jCheckBoxUseHLSVD64WaterRemoval.isSelected( ) && 
                 ( ( Number ) this.jFormattedTextFieldMinWaterRemoval.getValue( ) ).doubleValue( ) >= 
                 ( ( Number ) this.jFormattedTextFieldMaxWaterRemoval.getValue( ) ).doubleValue( ) )
        {
            JOptionPane.showMessageDialog( this, "Invalid residual-water-removal PPM range", "Invalid Range", JOptionPane.WARNING_MESSAGE );
            this.jFormattedTextFieldMaxWaterRemoval.requestFocus( );
        }
        else
        {
            preprocessingActionObject.setName( jTextFieldName.getText( ).trim( ) );
            preprocessingActionObject.setApodisationLineBroadning(((Number) this.jFormattedTextFieldApodisationLineBroadning.getValue()).doubleValue());
            preprocessingActionObject.setApodisationType( this.jRadioButtonApodisationGaussian.isSelected() ? BatchProcessing.APO_GAUSS : BatchProcessing.APO_EXP);
            preprocessingActionObject.setApodisationUse( this.jCheckBoxUseApodization.isSelected());
            preprocessingActionObject.setAutoPhasingMaxPPM( ( (Number) this.jFormattedTextFieldMaxAutoPhasing.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setAutoPhasingMinPPM( ( (Number) this.jFormattedTextFieldMinAutoPhasing.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setAutoPhasingUse( this.jCheckBoxUseAutoPhasing.isSelected( ) );
            preprocessingActionObject.setBaselineNR( ( (Number) this.jFormattedTextFieldBaseline.getValue( ) ).intValue( ) );
            preprocessingActionObject.setBaselineUse( this.jCheckBoxUseBaseline.isSelected( ) );
            preprocessingActionObject.setPanelOrder( panelOrder );
            preprocessingActionObject.setQualityRFile( this.jTextFieldQualityClassifier.getText( ) );
            preprocessingActionObject.setQualityThreshold( ( (Number) this.jFormattedTextFieldQualityThreshold.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setQualityUse( this.jCheckBoxUseQuality.isSelected( ) );
            
            preprocessingActionObject.setHLSVD64WaterRemovalMax( ( (Number) this.jFormattedTextFieldMaxWaterRemoval.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setHLSVD64WaterRemovalMin( ( (Number) this.jFormattedTextFieldMinWaterRemoval.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setHLSVD64WaterRemovalNumberOfPeaks( ( (Number) this.jFormattedTextFieldNumberPeaksWaterRemoval.getValue( ) ).intValue( ) );
            preprocessingActionObject.setHLSVD64WaterRemovalUse( this.jCheckBoxUseHLSVD64WaterRemoval.isSelected( ) );
            
            preprocessingActionObject.setHLSVDProWaterRemovalMax( ( (Number) this.jFormattedTextFieldMaxWaterRemovalHLSVDPro.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setHLSVDProWaterRemovalMin( ( (Number) this.jFormattedTextFieldMinWaterRemovalHLSVDPro.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setHLSVDProWaterRemovalNumberOfPeaks( ( (Number) this.jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro.getValue( ) ).intValue( ) );
            preprocessingActionObject.setHLSVDProWaterRemovalUse( this.jCheckBoxUseHLSVDProWaterRemoval.isSelected( ) );
            
            preprocessingActionObject.setTDFDFitWaterModelName( tdfdfitWaterReferenceModel );
            preprocessingActionObject.setTDFDFitWaterRemovalUse( jCheckBoxUseTDFDFitWaterRemoval.isSelected( ) );
            
            preprocessingActionObject.setZeroFillingUse( this.jCheckBoxUseZeroFilling.isSelected( ) );
            preprocessingActionObject.setFreqShiftMaxLocalShift( freqShiftMaxLocalShift );
            preprocessingActionObject.setFreqShiftMaxPPM( freqShiftMaxPPM );
            preprocessingActionObject.setFreqShiftMinPPM( freqShiftMinPPM );
            preprocessingActionObject.setFreqShiftPeaks( freqShiftPeaks );
            preprocessingActionObject.setFreqShiftSNR( freqShiftSNR );
            preprocessingActionObject.setFreqShiftUse( this.jCheckBoxUseFrequencyShift.isSelected( ) );

            // Action related to offset removal FD
            preprocessingActionObject.setUseRemoveAmplitudeOffsetFD( this.jCheckBoxUseRemoveOffsetFD.isSelected( ) );
            preprocessingActionObject.setMinFDOffsetRange( ( (Number) this.jFormattedTextFieldMinFDOffsetRange.getValue( ) ).doubleValue( ) );
            preprocessingActionObject.setMaxFDOffsetRange( ( (Number) this.jFormattedTextFieldMaxFDOffsetRange.getValue( ) ).doubleValue( ) );
            
            // Action related to offset removal TD
            preprocessingActionObject.setUseRemoveAmplitudeOffsetTD( this.jCheckBoxUseRemoveOffsetTD.isSelected( ) );
            preprocessingActionObject.setMinTDOffsetRange( ( (Number) this.jFormattedTextFieldMinTDOffsetRange.getValue( ) ).doubleValue( ) / 1000.0 );
            preprocessingActionObject.setMaxTDOffsetRange( ( (Number) this.jFormattedTextFieldMaxTDOffsetRange.getValue( ) ).doubleValue( ) / 1000.0 );
            
            
            parent.addPreprocessingActionUI( preprocessingActionObject, number );
            this.dispose( );
        }
    }//GEN-LAST:event_jButtonSaveActionPerformed

    private void jButtonCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCancelActionPerformed
        this.dispose( );
    }//GEN-LAST:event_jButtonCancelActionPerformed

     /**
     * Returns the file reference to the water model to be loaded and used by default.   
     * @param defaultString
     * @return 
     */
    private String getDefaultTDFDFitWaterModelFileReference( String dialogTitle, String approveButtonText )
    {   
        
        // Default model is always the currently loaded model!
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        String waterModelBaseName = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME );
        String waterModelPath = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH );
        String waterModelBaseNamePlusExtension = "";
        
        
        if ( waterModelBaseName.contains( "." ) == true )
        {
            int position = waterModelBaseName.lastIndexOf( "." );
        
            if ( position > 0 )
            {
                waterModelBaseName = waterModelBaseName.substring( 0, position );
            }
        }
        else
        {
            if ( !waterModelBaseName.isEmpty() )
            {
                waterModelBaseNamePlusExtension = waterModelPath + System.getProperty( "file.separator" ) + waterModelBaseName + ".tfm";
            }
            else
            {
                waterModelBaseNamePlusExtension = waterModelPath + System.getProperty( "file.separator" );
            }
        }
        
        String defaultFilePath = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH ) + 
                                                   System.getProperty( "file.separator" );
        
        JFileChooser fileChooser = new JFileChooser( defaultFilePath );
        
        String _tdfdfitWaterReferenceModel = "";
        fileChooser.setDialogTitle( dialogTitle );
        fileChooser.setApproveButtonText( approveButtonText );
        // Define a file filter to make only file selectable with appropriate
        // file extension.
        FileFilter filter = null;
        filter = new FileNameExtensionFilter( "TDFDFit water model file", "tfm" );
        
        // Set the correct file filter options ...
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        // Create a File object containing the canonical path of the 
        // desired directory
        String lastDefinedModel = defaultFilePath + System.getProperty( "file.separator" ) + waterModelBaseName;    
        if ( ( lastDefinedModel != null ) && ( new File( lastDefinedModel ) ).exists( ) )
        {
            fileChooser.setCurrentDirectory( new File( lastDefinedModel ) );
        }
        else
        {
            fileChooser.setCurrentDirectory( new File( System.getProperty( "user.home" ) ) );
        }
        
        File selected_file = new File( fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME ) + 
                                       System.getProperty( "file.separator" ) + waterModelBaseNamePlusExtension );
        
        if ( selected_file.exists( ) == true )
        {
            fileChooser.setSelectedFile( selected_file );
        }
        
        int result = fileChooser.showOpenDialog( this );
            
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                waterModelBaseName = fileChooser.getSelectedFile( ).getName( );
                                                waterModelPath = fileChooser.getSelectedFile( ).getParent( ) ; 
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // The selected process did not terminate properly
                                                break;
        }
        
        fdef.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME, waterModelBaseName );
        
        fdef.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH, waterModelPath );
        fdef.save( );
        
        return waterModelBaseName; 
    }
    
    
    private void jButtonTDFDFitWaterRemovaModelNameDefinerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonTDFDFitWaterRemovaModelNameDefinerActionPerformed

        tdfdfitWaterReferenceModel = getDefaultTDFDFitWaterModelFileReference( "Define TDFDFit water removel model", "Set selected water reference model" );
        this.jTDFDFitWaterRemovalModelNameTextField.setText( tdfdfitWaterReferenceModel );
    }//GEN-LAST:event_jButtonTDFDFitWaterRemovaModelNameDefinerActionPerformed

    private void jButtonWaterRemovalResetToDefaultHLSVDProActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonWaterRemovalResetToDefaultHLSVDProActionPerformed
        jFormattedTextFieldMaxWaterRemovalHLSVDPro.setValue(9);
        jFormattedTextFieldMinWaterRemovalHLSVDPro.setValue(4);
        jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro.setValue(8);
    }//GEN-LAST:event_jButtonWaterRemovalResetToDefaultHLSVDProActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JButton jButtonCancel;
    private javax.swing.JButton jButtonQualityClassifier;
    private javax.swing.JButton jButtonSave;
    private javax.swing.JButton jButtonSetFrqShift;
    private javax.swing.JButton jButtonTDFDFitWaterRemovaModelNameDefiner;
    private javax.swing.JButton jButtonWaterRemovalResetToDefault;
    private javax.swing.JButton jButtonWaterRemovalResetToDefaultHLSVDPro;
    private javax.swing.JCheckBox jCheckBoxUseApodization;
    private javax.swing.JCheckBox jCheckBoxUseAutoPhasing;
    private javax.swing.JCheckBox jCheckBoxUseBaseline;
    private javax.swing.JCheckBox jCheckBoxUseFrequencyShift;
    private javax.swing.JCheckBox jCheckBoxUseHLSVD64WaterRemoval;
    private javax.swing.JCheckBox jCheckBoxUseHLSVDProWaterRemoval;
    private javax.swing.JCheckBox jCheckBoxUseQuality;
    private javax.swing.JCheckBox jCheckBoxUseRemoveOffsetFD;
    private javax.swing.JCheckBox jCheckBoxUseRemoveOffsetTD;
    private javax.swing.JCheckBox jCheckBoxUseTDFDFitWaterRemoval;
    private javax.swing.JCheckBox jCheckBoxUseZeroFilling;
    private javax.swing.JFormattedTextField jFormattedTextFieldApodisationLineBroadning;
    private javax.swing.JFormattedTextField jFormattedTextFieldBaseline;
    private javax.swing.JFormattedTextField jFormattedTextFieldMaxAutoPhasing;
    private javax.swing.JFormattedTextField jFormattedTextFieldMaxFDOffsetRange;
    private javax.swing.JFormattedTextField jFormattedTextFieldMaxTDOffsetRange;
    private javax.swing.JFormattedTextField jFormattedTextFieldMaxWaterRemoval;
    private javax.swing.JFormattedTextField jFormattedTextFieldMaxWaterRemovalHLSVDPro;
    private javax.swing.JFormattedTextField jFormattedTextFieldMinAutoPhasing;
    private javax.swing.JFormattedTextField jFormattedTextFieldMinFDOffsetRange;
    private javax.swing.JFormattedTextField jFormattedTextFieldMinTDOffsetRange;
    private javax.swing.JFormattedTextField jFormattedTextFieldMinWaterRemoval;
    private javax.swing.JFormattedTextField jFormattedTextFieldMinWaterRemovalHLSVDPro;
    private javax.swing.JFormattedTextField jFormattedTextFieldNumberPeaksWaterRemoval;
    private javax.swing.JFormattedTextField jFormattedTextFieldNumberPeaksWaterRemovalHLSVDPro;
    private javax.swing.JFormattedTextField jFormattedTextFieldQualityThreshold;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabelAppodisationLineBroadning;
    private javax.swing.JLabel jLabelAppodisationType;
    private javax.swing.JLabel jLabelBaseline;
    private javax.swing.JLabel jLabelFDOffsetRange;
    private javax.swing.JLabel jLabelMaxAutoPhasing;
    private javax.swing.JLabel jLabelMaxFDOffsetRange;
    private javax.swing.JLabel jLabelMaxTDOffsetRange;
    private javax.swing.JLabel jLabelMaxTDOffsetRange1;
    private javax.swing.JLabel jLabelMaxWaterRemovalHLSVD64;
    private javax.swing.JLabel jLabelMaxWaterRemovalHLSVDPro;
    private javax.swing.JLabel jLabelMinAutoPhasing;
    private javax.swing.JLabel jLabelMinFDOffsetRange;
    private javax.swing.JLabel jLabelMinTDOffsetRange;
    private javax.swing.JLabel jLabelMinWaterRemoval;
    private javax.swing.JLabel jLabelMinWaterRemovalHLSVDPro;
    private javax.swing.JLabel jLabelNumberPeaksWaterRemoval;
    private javax.swing.JLabel jLabelNumberPeaksWaterRemovalHLSVDPro;
    private javax.swing.JLabel jLabelQualityClassifier;
    private javax.swing.JLabel jLabelQualityThreshold;
    private javax.swing.JLabel jLabelTDOffsetRange;
    private javax.swing.JLabel jLabelWaterModelName;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JPanel jPanelApodization;
    private javax.swing.JPanel jPanelApodizationLabel;
    private javax.swing.JPanel jPanelAutoPhasing;
    private javax.swing.JPanel jPanelAutoPhasingLabel;
    private javax.swing.JPanel jPanelBaseline;
    private javax.swing.JPanel jPanelBaselineLabel;
    private javax.swing.JPanel jPanelContainer;
    private javax.swing.JPanel jPanelFrequencyShift;
    private javax.swing.JPanel jPanelFrqShiftLabel;
    private javax.swing.JPanel jPanelLabelContainer;
    private javax.swing.JPanel jPanelLabels;
    private javax.swing.JPanel jPanelQuality;
    private javax.swing.JPanel jPanelQualityLabel;
    private javax.swing.JPanel jPanelRemoveOffsetFD;
    private javax.swing.JPanel jPanelRemoveOffsetFDLabel;
    private javax.swing.JPanel jPanelRemoveOffsetTD;
    private javax.swing.JPanel jPanelRemoveOffsetTDLabel;
    private javax.swing.JPanel jPanelUpDownButtons;
    private javax.swing.JPanel jPanelVariables;
    private javax.swing.JPanel jPanelWaterRemovalHLSVD64;
    private javax.swing.JPanel jPanelWaterRemovalHLSVDPro;
    private javax.swing.JPanel jPanelWaterRemovalLabelHLSVD64;
    private javax.swing.JPanel jPanelWaterRemovalLabelHLSVDPro;
    private javax.swing.JPanel jPanelWaterRemovalLabelTDFDFit;
    private javax.swing.JPanel jPanelWaterRemovalTDFDFit;
    private javax.swing.JPanel jPanelZeroFilling;
    private javax.swing.JPanel jPanelZeroFillingLabel;
    private javax.swing.JRadioButton jRadioButtonApodisationExponential;
    private javax.swing.JRadioButton jRadioButtonApodisationGaussian;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JTextField jTDFDFitWaterRemovalModelNameTextField;
    private javax.swing.JTextField jTextFieldName;
    private javax.swing.JTextField jTextFieldQualityClassifier;
    private javax.swing.JPanel jspaceFillerPanel_FDP;
    private javax.swing.JPanel jspaceFillerPanel_TDP;
    // End of variables declaration//GEN-END:variables
    
    
    
    private void initUpDownButtons( ) 
    {
        for ( int i = 0; i < jPanelLabels.getComponentCount( ); i++ ) 
        {
            final int entry = i;

            JButton u = new JButton("\u2193");
            u.addActionListener(new java.awt.event.ActionListener( ) 
            {
                public void actionPerformed(java.awt.event.ActionEvent evt) 
                {
                    jButtonUp(entry);
                }
            });
            if ( i == jPanelLabels.getComponentCount( ) - 1 ) 
            {
                u.setEnabled(false);
            }
            JButton d = new JButton("\u2191");
            d.addActionListener( new java.awt.event.ActionListener( ) 
            {
                public void actionPerformed(java.awt.event.ActionEvent evt) 
                {
                    jButtonDown( entry );
                }
            } );
            if ( i == 0 ) 
            {
                d.setEnabled(false);
            }
            jPanelUpDownButtons.add( u );
            jPanelUpDownButtons.add( d );
        }
    }

    
    /**
     * Routine that add a JButton with upward arrow to the panel.
     * @param i 
     */
    private void jButtonUp( int i ) 
    {
        int lowerInd = -1, upperInd = -1;
        for ( int j = 0; j < panelOrder.length; j++ ) 
        {
            if ( panelOrder[j] == i ) 
            {
                lowerInd = j;
            }
            if ( panelOrder[j] == ( i + 1 ) ) 
            {
                upperInd = j;
            }
        }
        panelOrder[lowerInd] = i + 1;
        panelOrder[upperInd] = i;
        JPanel[] upperPnl = hierarchyTree.get( i + 1 );
        JPanel[] intrPnl = hierarchyTree.get( i );
        hierarchyTree.put( i, upperPnl );
        hierarchyTree.put( i + 1, intrPnl );
        applyHierarchycalChanges( );
    }
    
    /**
     * Routine that add a JButton with upward arrow to the panel.
     * @param i 
     */
    private void jButtonDown( int i ) 
    {
        int lowerInd = -1, upperInd = -1;
        for ( int j = 0; j < panelOrder.length; j++ ) 
        {
            if ( panelOrder[j] == i ) 
            {
                lowerInd = j;
            }
            if ( panelOrder[j] == (i - 1) ) 
            {
                upperInd = j;
            }
        }
        panelOrder[lowerInd] = i - 1;
        panelOrder[upperInd] = i;
        JPanel[] lowerPnl = hierarchyTree.get( i - 1 );
        JPanel[] intrPnl = hierarchyTree.get( i );
        hierarchyTree.put( i, lowerPnl );
        hierarchyTree.put( i - 1, intrPnl );
        applyHierarchycalChanges( );
    }

    /**
     * Performs the actual changes in the hierarchical JPanel model.
     */
    private void applyHierarchycalChanges( ) 
    {
        jPanelLabels.removeAll( );
        jPanelVariables.removeAll( );
        for ( JPanel[] pnls : hierarchyTree.values( ) ) 
        {
            jPanelLabels.add( pnls[0] );
            jPanelVariables.add( pnls[1] );
        }
        this.revalidate( );
        this.repaint( );
    }

    
    /**
     * This method initializes the order in the panels will be shown.
     * The panels will set in the order given by the panelOrder int array.
     */
      private void initTreeTables( ) 
    {
        hierarchyTree.put( panelOrder[APODIZATION], new JPanel[]{ jPanelApodizationLabel, jPanelApodization } );
        hierarchyTree.put( panelOrder[QUALITY_FILTERING], new JPanel[]{ jPanelQualityLabel, jPanelQuality } );
        hierarchyTree.put( panelOrder[BASELINE_REMOVAL], new JPanel[]{ jPanelBaselineLabel, jPanelBaseline } );
        hierarchyTree.put( panelOrder[WATER_REMOVAL_HLSVD64], new JPanel[]{ jPanelWaterRemovalLabelHLSVD64, jPanelWaterRemovalHLSVD64 } );
        hierarchyTree.put( panelOrder[WATER_REMOVAL_TDFDFIT], new JPanel[]{ jPanelWaterRemovalLabelTDFDFit,jPanelWaterRemovalTDFDFit } );
        hierarchyTree.put( panelOrder[AUTO_PHASING], new JPanel[]{ jPanelAutoPhasingLabel, jPanelAutoPhasing } );
        hierarchyTree.put( panelOrder[FREQUENCY_SHIFT], new JPanel[]{ jPanelFrqShiftLabel, jPanelFrequencyShift } );
        hierarchyTree.put( panelOrder[ZEROFILLING], new JPanel[]{ jPanelZeroFillingLabel, jPanelZeroFilling } );
        hierarchyTree.put( panelOrder[OFFSET_CORRECTION_FD], new JPanel[]{ jPanelRemoveOffsetFDLabel, jPanelRemoveOffsetFD } );
        hierarchyTree.put( panelOrder[OFFSET_CORRECTION_TD], new JPanel[]{ jPanelRemoveOffsetTDLabel, jPanelRemoveOffsetTD } );
        hierarchyTree.put( panelOrder[WATER_REMOVAL_HLSVDPRO], new JPanel[]{ jPanelWaterRemovalLabelHLSVDPro, jPanelWaterRemovalHLSVDPro } );
    }
      
    public void setFreqShiftCorrection( ArrayList<ReferencePeak> refPeaks,
                                        double freqShiftSNR, 
                                        double freqShiftMinPPM, 
                                        double freqShiftMaxPPM, 
                                        double freqShiftMaxLocalShift ) 
    {
        this.freqShiftPeaks = refPeaks;
        this.freqShiftSNR = freqShiftSNR;
        this.freqShiftMinPPM = freqShiftMinPPM;
        this.freqShiftMaxPPM = freqShiftMaxPPM;
        this.freqShiftMaxLocalShift = freqShiftMaxLocalShift;

    }
}
