/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings.processing;

import java.util.ArrayList;
import org.spectrim.settings.ProcessingSettings;

/**
 * This class defines a Quantification Action; Initial it was made for
 * QUEST processing only.
 * @author nunopbarros updated by johannes slotboom 11-2018
 */
public class QuantificationAction 
{

    private String name = "";
    
    public static String QUANTIFICATION_QUEST      = "QUANTIFICATION_QUEST";
    public static String QUANTIFICATION_TDFDFIT    = "QUANTIFICATION_TDFDFIT";
    
    public static int QUANTIFICATION_QUEST_ID      = 1;
    public static int QUANTIFICATION_TDFDFIT_ID    = 2;
    
    
    private String quantification_type              = QUANTIFICATION_TDFDFIT;
    private int    quantification_type_id           = 2;
    
    private String opFile                           = "";
    private String m_tdfdfitQuantificationModelName = "";
    private String m_tdfdfitQuantificationModelPathName = "";
    
    public static final int MANUAL                  = 0;
    public static final int ML                      = 1;
    public static final int MRUI                    = 2;
    private int             mlFileSelectionMode     = MANUAL;

    private String mlFile = "";

    private ArrayList<MruiFile> mruiFiles = new ArrayList<>( );

    
    /**
     * Constructor already indicating type of quantification.
     * @param quantType 
     */
    public QuantificationAction( int quantType ) 
    {
        quantification_type_id = quantType;       
    }

    /**
     * Alternative constructor based on a pre selected quantification method.
     * @param txt 
     */
    public QuantificationAction( String txt ) 
    {
        String[] props      = txt.split( ProcessingSettings.separator1 );
        
        if ( props[0].compareTo( QUANTIFICATION_QUEST ) == 0 )
        {
            name                = props[1];
            opFile              = props[2];
            mlFileSelectionMode = Integer.parseInt( props[3] );
            if ( props.length > 3 ) 
            {
                mlFile          = props[4];
                if ( props.length > 5 ) 
                {
                    String[] mruiFilesTxt = props[5].split(ProcessingSettings.separator2 );
                    for ( int i = 0; i < ( ( mruiFilesTxt.length - 1 ) / 4 ); i++ ) 
                    {
                        mruiFiles.add( new MruiFile( mruiFilesTxt[1 + i * 4],
                                       Integer.parseInt( mruiFilesTxt[2 + i * 4] ), 
                                       mruiFilesTxt[3 + i * 4], 
                                       Integer.parseInt( mruiFilesTxt[4 + i * 4] ) == 1 ) );
                    }
                }
            }
        }
        
        if ( props[0].compareTo( QUANTIFICATION_TDFDFIT ) == 0 )
        {
            name = props[1];
            m_tdfdfitQuantificationModelName = props[1];
            m_tdfdfitQuantificationModelPathName = props[2];
        }
        
    }

    @Override
    public QuantificationAction clone( ) 
    { 
        return new QuantificationAction( this.toString( ) );
    }

    @Override
    public String toString( ) 
    {
        String txt = "";
        
        if ( quantification_type.compareTo( this.QUANTIFICATION_QUEST ) == 0 )
        {
            txt += QUANTIFICATION_QUEST + ProcessingSettings.separator1;
            txt += name + ProcessingSettings.separator1;
            txt += opFile + ProcessingSettings.separator1;
            txt += mlFileSelectionMode + ProcessingSettings.separator1;
            txt += mlFile + ProcessingSettings.separator1;
            for ( MruiFile mrui : mruiFiles ) 
            {
                txt += ProcessingSettings.separator2 + mrui.getName( );
                txt += ProcessingSettings.separator2 + mrui.getSpins( );
                txt += ProcessingSettings.separator2 + mrui.getMruiFile( );
                txt += ProcessingSettings.separator2 + ( mrui.isActive( ) ? 1:0 );
            }
        }
        
        if ( quantification_type.compareTo( this.QUANTIFICATION_TDFDFIT ) == 0 )
        {
            txt += QUANTIFICATION_TDFDFIT;
            txt += ProcessingSettings.separator1 + m_tdfdfitQuantificationModelName;
            txt += ProcessingSettings.separator1 + m_tdfdfitQuantificationModelPathName;
        }
        
        return txt;
    }
    
    /**
     * Sets the name of the TDFDFit model only.
     * @param tdfdfitQuantificationModelName 
     */
    public void setTDFDFitQuantificationModelName( String tdfdfitQuantificationModelName )
    {
        m_tdfdfitQuantificationModelName = tdfdfitQuantificationModelName;
    }
    
    /**
     * Sets the name of the TDFDFit model only. 
     * @return 
     */
    public String getTDFDFitQuantificationModelName( )
    {
        return m_tdfdfitQuantificationModelName;
    }
    
    
     /**
     * Sets the name of the TDFDFit model only.
     * @param tdfdfitQuantificationModelName 
     */
    public void setTDFDFitQuantificationModelPathName( String tdfdfitQuantificationModelPathName )
    {
        m_tdfdfitQuantificationModelPathName = tdfdfitQuantificationModelPathName;
    }
    
    /**
     * Sets the name of the TDFDFit model only. 
     * @return 
     */
    public String getTDFDFitQuantificationModelPathName( )
    {
        return m_tdfdfitQuantificationModelPathName;
    }
    
    public void setQuantificationType( String quantificationType )
    {
        quantification_type = quantificationType;
    }
    
    public String getQuantificationType( )
    {
        return quantification_type;
    }
    
    
    public String getName( ) 
    {
        return name;
    }

    public void setName( String name ) 
    {
        this.name = name;
    }

    public String getOpFile( ) 
    {
        return opFile;
    }

    public void setOpFile( String opFile ) 
    {
        this.opFile = opFile;
    }

    public int getMlFileSelectionMode( ) 
    {
        return mlFileSelectionMode;
    }

    public void setMlFileSelectionMode( int mlFileSelectionMode ) 
    {
        this.mlFileSelectionMode = mlFileSelectionMode;
    }

    public String getMlFile( ) 
    {
        return mlFile;
    }

    public void setMlFile( String mlFile ) 
    {
        this.mlFile = mlFile;
    }

    public ArrayList<MruiFile> getMruiFiles( ) 
    {
        return mruiFiles;
    }

    public void setMruiFiles( ArrayList<MruiFile> mruiFiles ) 
    {
        this.mruiFiles = mruiFiles;
    }

}