package org.spectrim.signalprocessing;


/*
 * File: $Id: FFT.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 04.02.2003  Authors: Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */



public class FFT
{
    
    /**
     * Default constructor
     */
    public FFT( )
    {
    }
    
    /**
     * Calculates a 1D FFT of a input signal 'array' the double precision has
     * been changed to single precision floats
     * @param array
     * @return 
     */
    public float[][] fft_1d(  float[][] array )
    {
        float  u_r, u_i, w_r, w_i, t_r, t_i;
        int     ln, nv2, k, l, le, le1, j, ip, i, n;

        n = array.length;
        ln = (int)( Math.log( (float)n )/Math.log(2) + 0.5 );
        nv2 = n / 2;
        j = 1;
        for ( i = 1; i < n; i++ )
        {
                if ( i < j )
                {
                        t_r = array[i - 1][0];
                        t_i = array[i - 1][1];
                        array[i - 1][0] = array[j - 1][0];
                        array[i - 1][1] = array[j - 1][1];
                        array[j - 1][0] = t_r;
                        array[j - 1][1] = t_i;
                }
                k = nv2;
                while ( k < j )
                {
                        j = j - k;
                        k = k / 2;
                }
                j = j + k;
        }

        for ( l = 1; l <= ln; l++ ) /* loops thru stages */
        {       
                le = (int)(Math.exp( (float)l * Math.log(2) ) + 0.5 );
                le1 = le / 2;
                u_r = 1.0f;
                u_i = 0.0f;
                w_r =  (float)Math.cos( Math.PI / (float)le1 );
                w_i = -(float)Math.sin( Math.PI / (float)le1 );
                for ( j = 1; j <= le1; j++ ) /* loops thru 1/2 twiddle values per stage */
                {
                        for ( i = j; i <= n; i += le ) /* loops thru points per 1/2 twiddle */
                        {
                                ip = i + le1;
                                t_r = array[ip - 1][0] * u_r - u_i * array[ip - 1][1];
                                t_i = array[ip - 1][1] * u_r + u_i * array[ip - 1][0];

                                array[ip - 1][0] = array[i - 1][0] - t_r;
                                array[ip - 1][1] = array[i - 1][1] - t_i; 

                                array[i - 1][0] =  array[i - 1][0] + t_r;
                                array[i - 1][1] =  array[i - 1][1] + t_i;  
                        } 
                        t_r = u_r * w_r - w_i * u_i;
                        u_i = w_r * u_i + w_i * u_r;
                        u_r = t_r;
                } 
        }  
        return array;
    } /* end of FFT_1d */
    
    
    /**
     * Calculates a 1D FFT of a input signal 'array' the double precision has
     * been changed to double precision floats
     * @param array
     * @return 
     */
    public double[][] fft_1d(  double[][] array )
    {
        double  u_r, u_i, w_r, w_i, t_r, t_i;
        int     ln, nv2, k, l, le, le1, j, ip, i, n;

        n = array.length;
        ln = (int)( Math.log( (double)n )/Math.log(2) + 0.5 );
        nv2 = n / 2;
        j = 1;
        for ( i = 1; i < n; i++ )
        {
                if ( i < j )
                {
                    t_r = array[i - 1][0];
                    t_i = array[i - 1][1];
                    array[i - 1][0] = array[j - 1][0];
                    array[i - 1][1] = array[j - 1][1];
                    array[j - 1][0] = t_r;
                    array[j - 1][1] = t_i;
                }
                k = nv2;
                while ( k < j )
                {
                    j = j - k;
                    k = k / 2;
                }
                j = j + k;
        }

        for ( l = 1; l <= ln; l++ ) /* loops thru stages */
        {       
                le = (int)(Math.exp( (double)l * Math.log(2) ) + 0.5 );
                le1 = le / 2;
                u_r = 1.0f;
                u_i = 0.0f;
                w_r =  (double)Math.cos( Math.PI / (double)le1 );
                w_i = -(double)Math.sin( Math.PI / (double)le1 );
                for ( j = 1; j <= le1; j++ ) /* loops thru 1/2 twiddle values per stage */
                {
                        for ( i = j; i <= n; i += le ) /* loops thru points per 1/2 twiddle */
                        {
                                ip = i + le1;
                                t_r = array[ip - 1][0] * u_r - u_i * array[ip - 1][1];
                                t_i = array[ip - 1][1] * u_r + u_i * array[ip - 1][0];

                                array[ip - 1][0] = array[i - 1][0] - t_r;
                                array[ip - 1][1] = array[i - 1][1] - t_i; 

                                array[i - 1][0] =  array[i - 1][0] + t_r;
                                array[i - 1][1] =  array[i - 1][1] + t_i;  
                        } 
                        t_r = u_r * w_r - w_i * u_i;
                        u_i = w_r * u_i + w_i * u_r;
                        u_r = t_r;
                } 
        }  
        return array;
    } /* end of FFT_1d */
    
    
    
    
    /**
     * Some test code
     */ 
    public static void main( String args[] )
    {
        
        float[][] cmplx = new float[1024][2];
        float[][] out = new float[1024][2];
        for ( int i = 0; i < 1024; i++ )
        {
            cmplx[i][0] = (float)i;
            cmplx[i][1] = (float)i;
        }
        
        System.out.println( "Start.." );
        long start = System.currentTimeMillis( );
        
        System.out.println( "Time = " + start );
        FFT myFFT = new FFT( );
        for ( int i = 0 ; i < 10000; i++ )
        {
            out = myFFT.fft_1d( cmplx );
        }
        
        long stop = System.currentTimeMillis( );
        
        System.out.println( "Time = " + stop );
        System.out.println( "Duration = " + ( stop - start ) );
        System.out.println( "Stop." ); 
    }
    
}