/*
 * File: $Id: SignalWriter.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 20.05.2005  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2005 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * SignalWriter.java
 *
 * Created on May 20, 2005, 3:34 PM
 */

package org.spectrim.signalprocessing;

import java.io.RandomAccessFile;
import java.io.File;
import java.io.IOException;


import javax.swing.JFileChooser;

import org.spectrim.lib.Dialogs;
import org.spectrim.lib.MessageBox;

import javax.swing.JFrame;

/**
 *
 * Writes a signal arrays of various type to an unicode-file.
 *
 * @author  hans
 */
public class SignalWriter
{
    
    /** 
     * Creates a new instance of SignalWriter. 
     */
    public SignalWriter( ) 
    {
    }

    /** 
     * Creates a new instance of SignalWriter and writes the signal to 
     * the filesystem.
     * 
     * The signal is a 1D signal of floating points
     *
     */
    public SignalWriter( String dialogTitle, float[] signal, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( new Integer( signal.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }

    
    /** 
     * Creates a new instance of SignalWriter and writes the signal and x-axis 
     * information to the file system.
     * 
     * The signal is a 1D signal of floating points
     *
     */
    public SignalWriter( String dialogTitle, float[] xaxis, float[] signal, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( new Integer( signal.length ).toString( ) );
                writer.writeBytes( "\r\n" );

                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( xaxis[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }

                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }


    /**
     * Creates a new instance of SignalWriter and writes the signal and x-axis
     * information to the file system.
     *
     * The signal is a 1D signal of floating points
     *
     */
    public SignalWriter( JFrame parent, String dialogTitle, float[] xaxis, float[] signal, String path, String fileName )
    {

        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;

        m_parent = parent;

        handleOutputFileNaming( );

        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );

                writer.writeBytes( new Integer( signal.length ).toString( ) );
                writer.writeBytes( "\r\n" );

                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( xaxis[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }

                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) );
            }
        }
    }



    /**
     * Creates a new instance of SignalWriter and writes the signal to 
     * the file system.
     * 
     * The signal is a 1D signal of double type 
     *
     */
    public SignalWriter( String dialogTitle, double[] signal, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( new Integer( signal.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }

    /**
     * Creates a new instance of SignalWriter and writes the signal to 
     * the file system.
     * 
     * The signal is a 1D signal of double type 
     *
     * @param dialogTitle
     * @param xaxis
     * @param signal
     * @param path
     * @param fileName
     */
    public SignalWriter( String dialogTitle, double[] xaxis, double[] signal, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( new Integer( signal.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( xaxis[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                for ( int i = 0; i < signal.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }

    
    
   /**
     * Creates a new instance of SignalWriter and writes the signal to 
     * the file system.
     * 
     * The signal is a 1D complex signal of float type with an absorption and 
     *  dispersion 
     *
     * @param dialogTitle
     * @param path
     * @param fileName
     * @param abs
     * @param disp
     */
    public SignalWriter( String dialogTitle, String path, String fileName, float[] abs, float[] disp )
    {
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        if ( ( abs.length != disp.length ) || ( abs.length == 0 ) )
        {
            System.out.println( "Cannot write complex signal to permanent storage:" );
            System.out.println( "Array size of absorption channel does not match ");
            System.out.println( "array size of displersion channel or length = 0" );
        }
        else
        {
            handleOutputFileNaming( );
        
            if ( m_file != null )
            {
                try
                {
                    RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                    writer.writeBytes( new Integer( abs.length ).toString( ) );
                    writer.writeBytes( "\r\n" );
                
                    for ( int i = 0; i < abs.length; i++ )
                    {
                        writer.writeBytes( ( new Float( abs[i] ) ).toString( ) );
                        writer.writeBytes( ( new Float( disp[i] ) ).toString( ) );
                        writer.writeBytes( "\t" );
                        writer.writeBytes( "\r\n" );
                    }
                    writer.close( );
                }
                catch ( IOException ioe )
                {
                    System.out.println( ioe.getMessage( ) ); 
                }
            }
        }
    }

    
    
    /** 
     * Creates a new instance of SignalWriter and writes the signal to 
     * the file system.
     * 
     * The signal is a 1D complex signal of float type with an absorption and 
     *  dispersion 
     *
     * @param dialogTitle
     * @param path
     * @param fileName
     * @param abs
     * @param disp
     */
    public SignalWriter( String dialogTitle, String path, String fileName, double[] abs, double[] disp )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        if ( ( abs.length != disp.length ) || ( abs.length == 0 ) )
        {
            System.out.println( "Cannot write complex signal to permanent storage:" );
            System.out.println( "Array size of absorption channel does not match ");
            System.out.println( "array size of displersion channel or length = 0" );
        }
        else
        {
            handleOutputFileNaming( );
        
            if ( m_file != null )
            {
                try
                {
                    RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                    writer.writeBytes( new Integer( abs.length ).toString( ) );
                    writer.writeBytes( "\r\n" );
                
                    for ( int i = 0; i < abs.length; i++ )
                    {
                        writer.writeBytes( ( new Float( abs[i] ) ).toString( ) );
                        writer.writeBytes( "\t" );
                        writer.writeBytes( ( new Float( disp[i] ) ).toString( ) );
                        writer.writeBytes( "\r\n" );
                    }
                    writer.close( );
                }
                catch ( IOException ioe )
                {
                    System.out.println( ioe.getMessage( ) ); 
                }
            }
        }
    }
    
    
    /** 
     * Creates a new instance of SignalWriter and writes the signal to 
     * the file system.
     * 
     * The signal is a 1D complex signal of float type with an absorption and 
     *  dispersion 
     *
     * @param dialogTitle
     * @param signal2D
     * @param dim1
     * @param dim2
     * @param fileName
     * @param path
     */
    public SignalWriter( String dialogTitle, float[][] signal2D,
                         int dim1, int dim2, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        if ( ( signal2D.length == 0 ) )
        {
            System.out.println( "Cannot write complex signal to permanent storage:" );
            System.out.println( "Array size of absorption channel does not match ");
            System.out.println( "array size of displersion channel or length = 0" );
        }
        else
        {
            
            handleOutputFileNaming( );
        
            if ( m_file != null )
            {
                try
                {
                    RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                    writer.writeBytes( new Integer( dim1 ).toString( ) );
                    writer.writeBytes( "\r\n" );

                    writer.writeBytes( new Integer( dim2 ).toString( ) );
                    writer.writeBytes( "\r\n" );
                
                    for ( int i = 0; i < dim1; i++ )
                    {
                        for ( int j = 0; j < dim2 ; j++ )
                        {
                            writer.writeBytes( ( new Float( signal2D[i][j] ) ).toString( ) );
                            writer.writeBytes( "\r\n" );
                        }
                    }
                    
                    writer.close( );
                }
                catch ( IOException ioe )
                {
                    System.out.println( ioe.getMessage( ) ); 
                }
            }
        }
    }
    
    /**
     * Handles the file naming and creation in the file system.
     */
    private void handleOutputFileNaming( )
    {
        JFileChooser fileChooser = new JFileChooser( );
        
        String defaultPath = fileChooser.getCurrentDirectory( ).getPath( );
        
        
        if ( ( m_path == null ) && ( m_fileName == null ) )
        {
            int result = fileChooser.showSaveDialog( null );
        
            switch ( result )
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    m_file = fileChooser.getSelectedFile( );
                                                    
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                    break;
            }
        }
        
        if ( ( m_path != null ) && ( m_fileName == null ) )
        {    
            
            fileChooser = new JFileChooser( new File( m_path ) );
            
            if ( m_dialogTitle != null )
            {
                fileChooser.setDialogTitle( m_dialogTitle );
            }
            
            int result = fileChooser.showSaveDialog( null );
        
            switch ( result )
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    m_file = fileChooser.getSelectedFile( );
                                                    
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                    break;
            }
        }

        if ( ( m_path == null ) && ( m_fileName != null ) )
        {
            File constructedFile = new File( defaultPath + System.getProperty( "file.separator" ),  m_fileName );
            
            try
            {
                constructedFile.createNewFile( );
            }
            catch ( IOException e )
            {
                // nothandled
            }
            
            fileChooser = new JFileChooser( constructedFile );
            
            fileChooser.setSelectedFile( constructedFile );
            
            if ( m_dialogTitle != null )
            {
                fileChooser.setDialogTitle( m_dialogTitle );
            }
            
            int result = fileChooser.showSaveDialog( null );
        
            switch ( result )
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    m_file = fileChooser.getSelectedFile( );
                                                    
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    if ( constructedFile.length( ) == 0L )
                                                    {
                                                        try
                                                        {
                                                            constructedFile.delete( );    
                                                        }
                                                        catch ( SecurityException se )
                                                        {
                                                            System.out.println( se.getMessage( ) );
                                                        }
                                                    }
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                    break;
            }
        }
        
        if ( ( m_path != null ) && ( m_fileName != null ) )
        {
            m_file = new File( m_path + System.getProperty( "file.separator" ), m_fileName );
            
            if ( m_file.exists( ) == true )
            {
                m_file.delete( );
            }
            
            try
            {    
                m_file.createNewFile( );
            }
            catch ( IOException e )
            {
                Dialogs.messageBox( m_parent, "Could not create: " + m_file.getName( ), "File Creation Error", MessageBox.MB_OK );
            }
        }
    }
    
    // Attributes
    private File m_file;
    private String m_path;
    private String m_fileName;
    private String m_dialogTitle;
    
    private JFrame m_parent = null;
}
