/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.spectra;

import java.util.Arrays;
import java.util.ArrayList;
import org.spectrim.settings.GlobalSettings;

/**
 *
 * @author nunopbarros
 */
public class Spectrum 
{

//    private final double[] signalFreeIntervalFD = new double[]{8.5, 10.5};
//    private final double[] signalFreeIntervalTD = new double[]{300, 400};//TEMP - prostate data
    final Spectra   spectra;
    private float[] absFID;
    private float[] dispFID;
    private float[] absorptionFD;
    private float[] dispersionFD;
    private float[] absorptionTD;
    private float[] dispersionTD;
    double          ppmshift = 0;
    private int     nrzeros;
    private int     m_truncate_data;
    private double  phase0;
    private double  phase1;
    private int     offset;
    private double  averagePower;
    private float   noiseStdFD;
    private float   noiseStdTD;
    private double  valapodexp;
    private double  valapodgauss;
    private double  reference;
    double          correlationAlignment = 0;
    private int     cluster = -1;
    private boolean isFreqShiftCorrected = false;

    public Spectrum( float[] fidabs, float[] fiddisp, Spectra spectra ) 
    {
        this.spectra = spectra;
        absFID = fidabs.clone( );
        dispFID = fiddisp.clone( );
        absorptionTD = absFID;
        dispersionTD = dispFID;
        absorptionFD = new float[fidabs.length];
        dispersionFD = new float[fiddisp.length];
        
        if ( !spectra.isFftDone ) 
        {
            TD2FD( absorptionTD, dispersionTD );
        } 
        else 
        {
            absorptionFD = fidabs.clone( );
            dispersionFD = fiddisp.clone( );
            absorptionTD = new float[fidabs.length];
            dispersionTD = new float[fiddisp.length];
            FD2TD( absorptionTD, dispersionTD );
        }
        noiseStdFD  = 1;
        noiseStdTD  = 1;
        valapodexp  = 0;
        valapodgauss = 0;
        reference   = 1;
        calcNoiseFD( );
        calcNoiseTD( );
    }

    public int getCluster( ) 
    {
        return cluster;
    }

    public void setCluster( int cluster ) 
    {
        this.cluster = cluster;
    }

    public float getNoiseStdDevFD( ) 
    {
        this.calcNoiseFD( );
        return noiseStdFD;
    }

    public void setNoiseStdFD( float noiseStdFD ) 
    {
        this.noiseStdFD = noiseStdFD;
    }

    public float getNoiseStdDevTD( ) 
    {
        return noiseStdTD;
    }

    public void setNoiseStdTD( float noiseStdTD ) 
    {
        this.noiseStdTD = noiseStdTD;
    }

    public double getReference( ) 
    {
        return reference;
    }

    public void setReference( double reference ) 
    {
        this.reference = reference;
    }

    private void TD2FD( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];

        for ( int m = 0; m < abs.length; m++ ) 
        {
            vaux[m][0] = abs[m];
            vaux[m][1] = disp[m];
        }

        vaux = fft(vaux);
        absorptionFD = new float[abs.length];
        dispersionFD = new float[abs.length];
        
        for ( int t = 0; t < (abs.length); t++ ) 
        {
            int d = (t + (abs.length) / 2) % (abs.length);
            absorptionFD[d] = (float) vaux[t][0];
            dispersionFD[d] = (float) vaux[t][1];
        }
    }

    private void FD2TD( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];

        for ( int m = 0; m < abs.length; m++ ) 
        {
            int d = (m + 1 + abs.length / 2) % abs.length;
            vaux[d][0] = abs[abs.length - 1 - m];
            vaux[d][1] = disp[disp.length - 1 - m];
        }

        vaux = fft(vaux);

        for ( int t = 0; t < (abs.length); t++ )  
        {
            int d = t % (abs.length);
            absorptionTD[d] = vaux[t][0] / abs.length;
            dispersionTD[d] = vaux[t][1] / abs.length;
        }
    }

    /**
     * Calculates a 1D FFT of a input signal 'array' the double precision has
     * been changed to single precision floats
     * @param array
     * @return 
     */
    public float[][] fft( float[][] array ) 
    {
        float u_r, u_i, w_r, w_i, t_r, t_i;
        int ln, nv2, k, l, le, le1, j, ip, i, n;

        n = array.length;
        ln = (int) (Math.log((double) n) / Math.log(2) + 0.5);
        nv2 = n / 2;
        j = 1;
        for (i = 1; i < n; i++) {
            if (i < j) {
                t_r = array[i - 1][0];
                t_i = array[i - 1][1];
                array[i - 1][0] = array[j - 1][0];
                array[i - 1][1] = array[j - 1][1];
                array[j - 1][0] = t_r;
                array[j - 1][1] = t_i;
            }
            k = nv2;
            while (k < j) {
                j = j - k;
                k = k / 2;
            }
            j = j + k;
        }

        for (l = 1; l <= ln; l++) /* loops thru stages */ {
            le = (int) (Math.exp((double) l * Math.log(2)) + 0.5);
            le1 = le / 2;
            u_r = 1.0f;
            u_i = 0.0f;
            w_r = (float) Math.cos(Math.PI / (float) le1);
            w_i = -(float) Math.sin(Math.PI / (float) le1);
            for (j = 1; j <= le1; j++) /* loops thru 1/2 twiddle values per stage */ {
                for (i = j; i <= n; i += le) /* loops thru points per 1/2 twiddle */ {
                    ip = i + le1;
                    t_r = array[ip - 1][0] * u_r - u_i * array[ip - 1][1];
                    t_i = array[ip - 1][1] * u_r + u_i * array[ip - 1][0];

                    array[ip - 1][0] = array[i - 1][0] - t_r;
                    array[ip - 1][1] = array[i - 1][1] - t_i;

                    array[i - 1][0] = array[i - 1][0] + t_r;
                    array[i - 1][1] = array[i - 1][1] + t_i;
                }
                t_r = u_r * w_r - w_i * u_i;
                u_i = w_r * u_i + w_i * u_r;
                u_r = t_r;
            }
        }
        return array;
    } /* end of FFT_1d */


    public void zerofilling( int nzeros ) 
    {
        nrzeros = nzeros;
        updateSpectra( );
    }

    public void truncate_data( int truncate_data ) 
    {
        m_truncate_data = truncate_data;
        truncateSpectra( truncate_data );
    }
    
    // Get TD spectra methods
    // -> Single Values
    public float getAbsorptionTD( int i ) 
    {
        int k = (i + offset) % absorptionTD.length;
        
        // float total_phase = (float)( phase0 + (float)( ( i - absorptionTD.length / 2f ) * phase1 ) );
        
        float res = (absorptionTD[k] * (float) Math.cos(phase0) - dispersionTD[k] * (float) Math.sin(phase0)) / (float) reference;
        if (spectra.isNormalized()) {
            res = res / noiseStdFD;
        }
        return res;
    }

    public float getDispersionTD( int i ) 
    {
        int k = (i + offset) % dispersionTD.length;
        
        // float total_phase = (float)( phase0 + (float)( ( i - absorptionTD.length / 2f ) * phase1 ) );
        
        float res = (dispersionTD[k] * (float) Math.cos(phase0) + absorptionTD[k] * (float) Math.sin(phase0)) / (float) reference;
        if (spectra.isNormalized()) {
            res = res / noiseStdFD;
        }
        return res;
    }

    public float getMagnitudeTD(int i) {
        float res = (float) Math.sqrt(Math.pow((double) getAbsorptionTD(i), 2)
                    + Math.pow((double) getDispersionTD(i), 2));
        return res;
    }

    // -> Whole signal
    public float[] getAbsorptionTD() {
        float[] res = new float[absorptionTD.length];
        for (int i = 0; i < absorptionTD.length; i++) {
            res[i] = getAbsorptionTD(i);
        }
        return res;
    }

    public float[] getDispersionTD() {
        float[] res = new float[dispersionTD.length];
        for (int i = 0; i < dispersionTD.length; i++) {
            res[i] = getDispersionTD(i);
        }
        return res;
    }

    public float[] getMagnitudeTD() {
        float[] res = new float[absorptionTD.length];
        for (int i = 0; i < absorptionFD.length; i++) {
            res[i] = getMagnitudeTD(i);
        }
        return res;
    }

    // -> Range
    public float[] getMagnitudeTD(double mint, double maxt) {
        float[] res = new float[spectra.getIndexScaleTD(maxt) - spectra.getIndexScaleTD(mint) + 1];
        int minppmIndex = spectra.getIndexScaleTD(mint);
        int maxppmIndex = spectra.getIndexScaleTD(maxt);
        for (int i = minppmIndex; i <= maxppmIndex; i++) {
            res[i - minppmIndex] = getMagnitudeFD(i);
        }
        return res;
    }

    // Get FD spectra methods
    // -> Single Values
    public float getAbsorptionFD(int i) {
        float res = 0;
        float total_phase = (float)( phase0 + (float)( ( i - absorptionTD.length / 2f ) * phase1 ) / absorptionTD.length );
        res = (absorptionFD[i] * (float) Math.cos(total_phase) - dispersionFD[i] * (float) Math.sin(total_phase)) / (float) reference;
        if (spectra.isNormalized()) {
            res = res / noiseStdFD;
        }
        return res;
    }

    public float getDispersionFD(int i) {
        float res = 0;
        float total_phase = (float)( phase0 + (float)( ( i - absorptionTD.length / 2f ) * phase1 ) / absorptionTD.length );
        res = (absorptionFD[i] * (float) Math.sin(total_phase) + dispersionFD[i] * (float) Math.cos(total_phase)) / (float) reference;
        if (spectra.isNormalized()) {
            res = res / noiseStdFD;
        }
        return res;
    }

    public float getMagnitudeFD(int i) {
        float res = (float) Math.sqrt(Math.pow((double) getAbsorptionFD(i), 2)
                + Math.pow((double) getDispersionFD(i), 2));
        return res;
    }

    // -> Whole signal
    public float[] getAbsorptionFD() {
        float[] res = new float[absorptionFD.length];
        for (int i = 0; i < absorptionFD.length; i++) {
            res[i] = getAbsorptionFD(i);
        }
        return res;
    }

    public float[] getDispersionFD() {
        float[] res = new float[dispersionTD.length];
        for (int i = 0; i < dispersionTD.length; i++) {
            res[i] = getDispersionFD(i);
        }
        return res;
    }

    public float[] getMagnitudeFD() {
        float[] res = new float[absorptionFD.length];
        for (int i = 0; i < absorptionFD.length; i++) {
            res[i] = getMagnitudeFD(i);
        }
        return res;
    }

    // -> Range
    public float[] getAbsorptionFD(double minppm, double maxppm) {
        float[] res = new float[spectra.getIndexScalePPM(maxppm) - spectra.getIndexScalePPM(minppm) + 1];
        int minppmIndex = spectra.getIndexScalePPM(minppm);
        int maxppmIndex = spectra.getIndexScalePPM(maxppm);
        for (int i = minppmIndex; i <= maxppmIndex; i++) {
            res[i - minppmIndex] = getAbsorptionFD(i);
        }
        return res;
    }

    public float[] getDispersionFD(double minppm, double maxppm) {
        float[] res = new float[spectra.getIndexScalePPM(maxppm) - spectra.getIndexScalePPM(minppm) + 1];
        int minppmIndex = spectra.getIndexScalePPM(minppm);
        int maxppmIndex = spectra.getIndexScalePPM(maxppm);
        for (int i = minppmIndex; i <= maxppmIndex; i++) {
            res[i - minppmIndex] = getDispersionFD(i);
        }
        return res;
    }

    public float[] getMagnitudeFD(double minppm, double maxppm) {
        float[] res = new float[spectra.getIndexScalePPM(maxppm) - spectra.getIndexScalePPM(minppm) + 1];
        int minppmIndex = spectra.getIndexScalePPM(minppm);
        int maxppmIndex = spectra.getIndexScalePPM(maxppm);
        for (int i = minppmIndex; i <= maxppmIndex; i++) {
            res[i - minppmIndex] = getMagnitudeFD(i);
        }
        return res;
    }

    public float[] getMagnitudeGradientFD(double minppm, double maxppm) {
        float[] absolute = new float[spectra.getIndexScalePPM(maxppm) - spectra.getIndexScalePPM(minppm) + 5];
        float[] res = new float[spectra.getIndexScalePPM(maxppm) - spectra.getIndexScalePPM(minppm) + 1];
        int minppmIndex = spectra.getIndexScalePPM(minppm);
        int maxppmIndex = spectra.getIndexScalePPM(maxppm);
        for (int i = minppmIndex - 2; i <= maxppmIndex + 2; i++) {
            absolute[i - minppmIndex + 2] = getMagnitudeFD(i);
        }
        for (int i = 0; i < absolute.length - 4; i++) {
            res[i] = -absolute[i] - 2 * absolute[i + 1] + 2 * absolute[i + 3] + absolute[i + 4];
        }
        return res;
    }

    public void setPhase(double phi0, double phi1) {
         phase0 = phi0 % (Math.PI * 2);
        if (phase0 < -Math.PI) {
            phase0 += (Math.PI * 2);
        } else if (phase0 > Math.PI) {
            phase0 -= (Math.PI * 2);
        }
        phase1 = phi1;

        offset = ((int) phase1) % absorptionTD.length;
        if (offset < 0) {
            offset += absorptionTD.length;
        }
    }

    public void addPhase(double phi0, double phi1) {
        phase0 = (phase0+phi0) % (Math.PI * 2);
        if (phase0 < -Math.PI) {
            phase0 += (Math.PI * 2);
        } else if (phase0 > Math.PI) {
            phase0 -= (Math.PI * 2);
        }
        phase1 = phi1;

        offset = ((int) phase1) % absorptionTD.length;
        if (offset < 0) {
            offset += absorptionTD.length;
        }
    }

    public double[] getPhaseParam() {
        return new double[]{phase0, phase1};
    }

    private void calcNoiseFD() {
        //Limits
        int min2 = Math.max(spectra.getIndexScalePPM(spectra.getXAxis(Spectra.Domain.FD_PPM)[0] + 0.5),spectra.getIndexScalePPM(GlobalSettings.getInstance().getSignalFreeFDMin()));
        int max2 = Math.min(spectra.getIndexScalePPM(spectra.getXAxis(Spectra.Domain.FD_PPM)[spectra.getVectorSize() - 1] - 0.5), spectra.getIndexScalePPM(GlobalSettings.getInstance().getSignalFreeFDMax()));
        
        float mean = 0;
        float var = 0;
        int nr = 0;

        for (int i = min2; i < max2; i++) {
            mean += getMagnitudeFD(i);
            nr++;
        }
        mean = mean / (nr);
        for (int i = min2; i < max2; i++) {
            var += Math.pow(getMagnitudeFD(i) - mean, 2);
        }

        this.noiseStdFD = (float) Math.sqrt(var / nr);

//        double before = noiseStdFD;
//        int l = max2-min2+1;
//        double[][] X = new double[l][2];
//        double[] y = new double[l];
//        for (int i = 0; i < (max2-min2+1); i++) {
//            X[i][0] = 1;
//            X[i][1] = i;
//            y[i] = getMagnitudeFD(i+min2);
//        }
//
//        RealMatrix X2 = new Array2DRowRealMatrix(X);
//        RealVector y2 = new ArrayRealVector(y, false);
//        DecompositionSolver solver = new QRDecomposition(X2).getSolver();
//        RealVector theta = solver.solve(y2);
//        double[] residue = new double[l];
//        for (int i = 0; i < l; i++) {
//            residue[i] = getMagnitudeFD(i+min2) - (theta.getEntry(0) + i * theta.getEntry(1));
//        }
//
//        mean = 0;
//        var = 0;
//        nr = 0;
//
//        for (int i = 0; i < l; i++) {
//            mean += residue[i];
//            nr++;
//        }
//        mean = mean / (nr);
//        for (int i = 0; i < l; i++) {
//            var += Math.pow(residue[i] - mean, 2);
//        }
//
//        this.noiseStdFD = (float) Math.sqrt(var / nr);
//
//        double after = noiseStdFD;
//        
//        System.out.println("Mean:"+mean+" stddevbefore:"+before+" stdafter:"+after+"   rel diference:"+((before-after)/before));
    }

    private void calcNoiseTD() {
        //Limits
        double maxTD = spectra.getXAxis(Spectra.Domain.TD)[spectra.getVectorSize() - 1];
        
        double begin_noise = 0.85 * maxTD;
        double end_noise   = 0.95 * maxTD;
        
        int min = spectra.getIndexScaleTD( begin_noise );
        int max = spectra.getIndexScaleTD( end_noise );

        float mean = 0;
        float var = 0;
        int nr = 0;

        for ( int i = min; i < max; i++ ) 
        {
            mean += getMagnitudeTD(i);
            nr++;
        }

        mean = mean / (nr);
        for (int i = min; i < max; i++) {
            var += Math.pow(getMagnitudeTD(i) - mean, 2);
        }

        this.noiseStdTD = (float) Math.sqrt(var / nr);
    }

    public void printarray(ArrayList<Double> arrayl, String name) {
        String text = name + " = [";
        for (int i = 0; i < arrayl.size() - 1; i++) {
            text = text + arrayl.get(i).toString() + "; ";
        }
        text = text + arrayl.get(arrayl.size() - 1) + "];";
        System.out.println(text);
    }

    
    
    /** 
     * 
     * Performs manual phasing of the dataset.
     * 
     * @param zeroOrder
     * @param firstOrder 
     */
    public void manualPhase( float zeroOrder, float firstOrder ) 
    {
        
        
        phase0 = zeroOrder;
        phase1 = firstOrder;
        
        
//        double maxscore = getAbsorptionSumFD(minppm , maxppm);
//        double phase0max = phase0;
//        double auxscore1;
//        double auxscore2;
//
//        for (int i = 1; i < 15; i++) {
//            phase0 += Math.PI * Math.pow(0.5, i);
//            auxscore1 = getAbsorptionSumFD(minppm, maxppm);
//            phase0 -= Math.PI * 2 * Math.pow(0.5, i);
//            auxscore2 = getAbsorptionSumFD(minppm, maxppm);
//            if (auxscore2 > auxscore1) {
//                if (auxscore2 > maxscore) {
//                    maxscore = auxscore2;
//                    phase0max = phase0;
//                }
//            } else {
//                phase0 += Math.PI * 2 * Math.pow(0.5, i);
//                if (auxscore1 > maxscore) {
//                    maxscore = auxscore1;
//                    phase0max = phase0;
//                }
//            }
//        }
//        phase0 = phase0max % (Math.PI * 2);
//        if (phase0 < -Math.PI) {
//            phase0 += (Math.PI * 2);
//        } else if (phase0 > Math.PI) {
//            phase0 -= (Math.PI * 2);
//        }
    }
    
    
    
    public void autoPhase(float minppm, float maxppm) {
        double maxscore = getAbsorptionSumFD(minppm, maxppm);
        double phase0max = phase0;
        double auxscore1;
        double auxscore2;

        for (int i = 1; i < 15; i++) {
            phase0 += Math.PI * Math.pow(0.5, i);
            auxscore1 = getAbsorptionSumFD(minppm, maxppm);
            phase0 -= Math.PI * 2 * Math.pow(0.5, i);
            auxscore2 = getAbsorptionSumFD(minppm, maxppm);
            if (auxscore2 > auxscore1) {
                if (auxscore2 > maxscore) {
                    maxscore = auxscore2;
                    phase0max = phase0;
                }
            } else {
                phase0 += Math.PI * 2 * Math.pow(0.5, i);
                if (auxscore1 > maxscore) {
                    maxscore = auxscore1;
                    phase0max = phase0;
                }
            }
        }
        phase0 = phase0max % (Math.PI * 2);
        if (phase0 < -Math.PI) {
            phase0 += (Math.PI * 2);
        } else if (phase0 > Math.PI) {
            phase0 -= (Math.PI * 2);
        }
    }

    //Global autophasing using first point FID
    public void autoPhase() {
        if (absorptionTD[0] < 0) {
            if (dispersionTD[0] < 0) {
                phase0 = -(Math.atan(dispersionTD[0] / absorptionTD[0]) - Math.PI);
            } else {
                phase0 = -(Math.atan(dispersionTD[0] / absorptionTD[0]) + Math.PI);
            }
        } else {
            phase0 = -Math.atan(dispersionTD[0] / absorptionTD[0]);
        }
    }

    private double getAbsorptionEntropy(double minppm, double maxppm) {
        double res = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            float val = getAbsorptionFD(i);
            res -= val * Math.log(Math.abs(val));
        }
        return res;
    }

    public double getAbsorptionSumFD(double minppm, double maxppm) {
        double res = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            res += getAbsorptionFD(i);
        }
        return res;
    }

    public double getDispersionSumFD(double minppm, double maxppm) {
        double res = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            res += getDispersionFD(i);
        }
        return res;
    }

    public double getMagnitudeSumFD(double minppm, double maxppm) {
        double res = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            res += getMagnitudeFD(i);
        }
        return res;
    }

    public double getMagnitudeSumTD(double mintime, double maxtime) {
        double res = 0;
        for (int i = spectra.getIndexScaleTD(mintime); i <= spectra.getIndexScaleTD(maxtime); i++) {
            res += Math.sqrt(Math.pow(absorptionTD[i], 2) + Math.pow(dispersionTD[i], 2));
        }
        return res;
    }

    public void setReferenceVal(double ref) {
        reference = ref / 100000;
    }

    public void calcAveragePower() {
        for (int i = 0; i < absorptionTD.length; i++) {
            averagePower += Math.pow(absorptionTD[i], 2) + Math.pow(dispersionTD[i], 2);
        }
        averagePower = averagePower / absorptionTD.length;
    }

    public double getAveragePower() {
        calcAveragePower();
        return averagePower;
    }

    public void apodizeExponential(double a) {
        valapodexp = a;
        valapodgauss = 0;
        updateSpectra();
    }

    public void apodizeGaussian(double a) {
        valapodexp = 0;
        valapodgauss = a;
        updateSpectra();
    }

    private void updateSpectra() {
        //Copy original signal and apply Zerofilling
        absorptionTD = Arrays.copyOf(absFID, absFID.length + nrzeros);
        dispersionTD = Arrays.copyOf(dispFID, absFID.length + nrzeros);

        //Apodization Gaussian+Exponential AND Frequency Shift
        double scale = spectra.getScaleTD() / 1000;//msec to seconds
        double deltaFreq = ppmshift * spectra.resonanceFrequency / 1000000;//ppm to freq
        for (int i = 0; i < absorptionTD.length; i++) {
            //Apodization
            float abs = absorptionTD[i] * ((float) Math.exp(-Math.pow(i * scale * valapodgauss, 2) - i * scale * valapodexp));
            float disp = dispersionTD[i] * ((float) Math.exp(-Math.pow(i * scale * valapodgauss, 2) - i * scale * valapodexp));
            //Frequency Shift
            double t = spectra.getXAxis(Spectra.Domain.TD)[i] / 1000;//seconds
            absorptionTD[i] = abs * (float) Math.cos(2 * Math.PI * deltaFreq * t) - disp * (float) Math.sin(2 * Math.PI * deltaFreq * t);
            dispersionTD[i] = abs * (float) Math.sin(2 * Math.PI * deltaFreq * t) + disp * (float) Math.cos(2 * Math.PI * deltaFreq * t);
        }
        TD2FD(absorptionTD, dispersionTD);
    }

    
    private void truncateSpectra( int truncation_point ) 
    {
        
        // Copy original signal and apply Zerofilling
        absorptionTD = Arrays.copyOf( absFID, truncation_point );
        dispersionTD = Arrays.copyOf( dispFID, truncation_point );
        
        absFID = Arrays.copyOf( absorptionTD , truncation_point);
        dispFID = Arrays.copyOf( dispersionTD , truncation_point);
        
        // Apodization Gaussian+Exponential AND Frequency Shift
        double scale = spectra.getScaleTD( ) / 1000; //msec to seconds
        double deltaFreq = ppmshift * spectra.resonanceFrequency / 1000000; //ppm to freq
        
        for ( int i = 0; i < absorptionTD.length; i++ ) 
        {
            // Apodization
            float abs = absorptionTD[i] * ((float) Math.exp( -Math.pow( i * scale * valapodgauss, 2 ) - i * scale * valapodexp ) );
            float disp = dispersionTD[i] * ((float) Math.exp( -Math.pow( i * scale * valapodgauss, 2 ) - i * scale * valapodexp ) );
            // Frequency Shift 
            double t = spectra.getXAxis( Spectra.Domain.TD )[i] / 1000;//seconds
            absorptionTD[i] = abs * (float) Math.cos( 2 * Math.PI * deltaFreq * t ) - disp * (float) Math.sin( 2 * Math.PI * deltaFreq * t );
            dispersionTD[i] = abs * (float) Math.sin( 2 * Math.PI * deltaFreq * t ) + disp * (float) Math.cos( 2 * Math.PI * deltaFreq * t );
        }
        
        TD2FD( absorptionTD, dispersionTD );
    }
    
    
    public void resetFrequencyShift() {
        isFreqShiftCorrected = false;
        ppmshift = 0;
        updateSpectra();
    }

    public void frequencyShift(double deltaPpm) {
        isFreqShiftCorrected = true;
        ppmshift = deltaPpm;
        updateSpectra();
    }

    public float getMax(double minppm, double maxppm) {
        float max = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            float val = this.getMagnitudeFD(i);
            if (val > max) {
                max = val;
            }
        }
        return max;
    }

    public float getMaxTD(double mintime, double maxtime) {
        float max = 0;
        for (int i = spectra.getIndexScaleTD(mintime); i <= spectra.getIndexScaleTD(maxtime); i++) {
            float val = this.getMagnitudeTD(i);
            if (val > max) {
                max = val;
            }
        }
        return max;
    }

    public float getMean(double minppm, double maxppm) {
        float mean = 0;
        int count = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            mean += this.getMagnitudeFD(i);
            count++;
        }
        return mean / count;
    }

    public float getMeanTD(double minTime, double maxTime) {//ms
        float mean = 0;
        int count = 0;
        for (int i = spectra.getIndexScaleTD(minTime); i <= spectra.getIndexScaleTD(maxTime); i++) {
            mean += this.getMagnitudeTD(i);
            count++;
        }
        return mean / count;
    }

    public double getMaxPos(double minppm, double maxppm) {
        float max = 0;
        double pos = 0;
        for (int i = spectra.getIndexScalePPM(minppm); i <= spectra.getIndexScalePPM(maxppm); i++) {
            float val = this.getMagnitudeFD(i);
            if (val > max) {
                max = val;
                pos = spectra.getXAxis(Spectra.Domain.FD_PPM)[i];
            }
        }
        return pos;
    }

    public double[] getFDMagnitudeStatistics(double minppm, double maxppm) {
        double[] res = this.getArrayStatistics(this.getMagnitudeFD(minppm, maxppm));
        res[1] = spectra.getXAxis(Spectra.Domain.FD_PPM)[((int) res[1]) + spectra.getIndexScalePPM(minppm)];
        res[3] = spectra.getXAxis(Spectra.Domain.FD_PPM)[((int) res[3]) + spectra.getIndexScalePPM(minppm)];
        return res;
    }

    public double[] getFDMagnitudeGradientStatistics(double minppm, double maxppm) {
        double[] res = this.getArrayStatistics(this.getMagnitudeGradientFD(minppm, maxppm));
        res[1] = spectra.getXAxis(Spectra.Domain.FD_PPM)[((int) res[1]) + spectra.getIndexScalePPM(minppm)];
        res[3] = spectra.getXAxis(Spectra.Domain.FD_PPM)[((int) res[3]) + spectra.getIndexScalePPM(minppm)];
        return res;
    }

    public double[] getFDAbsorptionRangeStatistics(double minppm, double maxppm) {
        double[] res = this.getArrayStatistics(this.getAbsorptionFD(minppm, maxppm));
        res[1] = spectra.getXAxis(Spectra.Domain.FD_PPM)[((int) res[1]) + spectra.getIndexScalePPM(minppm)];
        res[3] = spectra.getXAxis(Spectra.Domain.FD_PPM)[((int) res[3]) + spectra.getIndexScalePPM(minppm)];
        return res;
    }

    public double getPeakSNR(double minppm, double maxppm) {
        return (getMax(minppm, maxppm) / noiseStdFD);
    }

    public double getMeanFDSNR(double minppm, double maxppm) {
        return (getMean(minppm, maxppm) / noiseStdFD);
    }

    public double getMeanTDSNR(double minTime, double maxTime) {
        return (getMeanTD(minTime, maxTime) / noiseStdTD);
    }

    public double getTDRelativeChange(double minTime, double maxTime) {
        int mintimep = spectra.getIndexScaleTD(minTime);
        int maxtimep = spectra.getIndexScaleTD(maxTime);
        double vali = (this.getMagnitudeTD(mintimep) + this.getMagnitudeTD(mintimep + 1) + this.getMagnitudeTD(mintimep + 2)) / 3;
        double valf = (this.getMagnitudeTD(maxtimep) + this.getMagnitudeTD(maxtimep + 1) + this.getMagnitudeTD(maxtimep + 2)) / 3;
        double val = (valf - vali) * 100 / (double) this.getMaxTD(0, 100);
        return val;
    }

    public double[] getTDStatistics() {
        double mean = 0;
        double stdDev = 0;
        double m3 = 0;
        double m4 = 0;
        double skewness;
        double kurtosis;
        double tmax = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        double count = 0;
        for (int i = 0; i < absorptionTD.length; i++) {
            double val = this.getMagnitudeTD(i);
            count++;
            mean += val;
            if (val > max) {
                max = val;
                tmax = spectra.getXAxis(Spectra.Domain.TD)[i];
            }
        }
        mean = mean / count;

        for (int i = 0; i < absorptionTD.length; i++) {
            double val = this.getMagnitudeTD(i);
            stdDev += Math.pow((val - mean), 2);
            m3 += Math.pow((val - mean), 3);
            m4 += Math.pow((val - mean), 4);
        }
        skewness = (m3 / count) / Math.pow(Math.sqrt(stdDev / (count - 1)), 3);
        kurtosis = ((m4 / count) / Math.pow(stdDev / count, 2)) - 3;
        stdDev = Math.sqrt(stdDev / count);

        return new double[]{max, tmax, mean, stdDev, skewness, kurtosis};
    }

    public double[] getFDStatistics() {
        double mean = 0;
        double stdDev = 0;
        double m3 = 0;
        double m4 = 0;
        double skewness;
        double kurtosis;
        double ppmmax = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        double count = 0;
        for (int i = 0; i < absorptionFD.length; i++) {
            double val = this.getMagnitudeFD(i);
            count++;
            mean += val;
            if (val > max) {
                max = val;
                ppmmax = spectra.getXAxis(Spectra.Domain.FD_PPM)[i];
            }
        }
        mean = mean / count;

        for (int i = 0; i < absorptionFD.length; i++) {
            double val = this.getMagnitudeFD(i);
            stdDev += Math.pow((val - mean), 2);
            m3 += Math.pow((val - mean), 3);
            m4 += Math.pow((val - mean), 4);
        }
        skewness = (m3 / count) / Math.pow(Math.sqrt(stdDev / (count - 1)), 3);
        kurtosis = ((m4 / count) / Math.pow(stdDev / count, 2)) - 3;
        stdDev = Math.sqrt(stdDev / count);

        return new double[]{max, ppmmax, mean, stdDev, skewness, kurtosis};
    }

    public double getFrequencyShift() {
        return this.ppmshift;
    }

    public boolean isFrequencyShiftCorrected() {
        return isFreqShiftCorrected;
    }

    public void calcLocalPPMShift(float[] refSignal, double minPPM, double maxPPM, double maxLocalShift) {
        spectrumLocalShiftFinder shiftFinder = new spectrumLocalShiftFinder(refSignal, minPPM, maxPPM, maxLocalShift, this);
        shiftFinder.run();
    }

    public double getCorrelationAlignment() {
        return correlationAlignment;
    }

    private double[] getArrayStatistics(float[] array) {
        double mean = 0;
        double stdDev = 0;
        double m3 = 0;
        double m4 = 0;
        double skewness;
        double kurtosis;
        double posmax = 0;
        double posmin = 0;
        double max = Double.NEGATIVE_INFINITY;
        double min = Double.POSITIVE_INFINITY;
        double count = 0;
        for (int i = 0; i < array.length; i++) {
            double val = array[i];
            count++;
            mean += val;
            if (val > max) {
                max = val;
                posmax = i;
            }
            if (val < min) {
                min = val;
                posmin = i;
            }
        }
        mean = mean / count;

        for (int i = 0; i < array.length; i++) {
            double val = array[i];
            stdDev += Math.pow((val - mean), 2);
            m3 += Math.pow((val - mean), 3);
            m4 += Math.pow((val - mean), 4);
        }
        skewness = (m3 / count) / Math.pow(Math.sqrt(stdDev / (count - 1)), 3);
        kurtosis = ((m4 / count) / Math.pow(stdDev / count, 2)) - 3;
        stdDev = Math.sqrt(stdDev / count);

        return new double[]{min, posmin, max, posmax, mean, stdDev, skewness, kurtosis};
    }

    /**
     * computes the difference between this spectrum and the input spectra in
     * the mangitude. returns null if the two vectors do not have the same size
     */
    public float[] getDiff(Spectrum sp) {
        float[] abs1 = this.getAbsorptionFD();
        float[] abs2 = sp.getAbsorptionFD();
        float[] disp1 = this.getDispersionFD();
        float[] disp2 = sp.getDispersionFD();
        if (abs1.length != abs2.length) {
            return null;
        } else {
            float[] d = new float[abs1.length];
            for (int i = 0; i < abs1.length; i++) {
                d[i] = (float) Math.sqrt(Math.pow(abs1[i] - abs2[i], 2) + Math.pow(disp1[i] - disp2[i], 2));
            }
            return d;
        }
    }

    /**
     * computes the magnitude of the complex difference between this spectra and
     * the input spectrum in for the given ppm range. returns null if the two
     * vectors do not have the same size
     */
    public float[] getDiff(Spectrum sp, double minPPM, double maxPPM) {
        float[] abs1 = this.getAbsorptionFD();
        float[] abs2 = sp.getAbsorptionFD();
        float[] disp1 = this.getDispersionFD();
        float[] disp2 = sp.getDispersionFD();
        if (abs1.length != abs2.length) {
            return null;
        } else {
            int minInd = Math.min(spectra.getIndexScalePPM(minPPM), spectra.getIndexScalePPM(maxPPM));
            int maxInd = Math.max(spectra.getIndexScalePPM(minPPM), spectra.getIndexScalePPM(maxPPM));
            float[] d = new float[maxInd - minInd + 1];
            int iter = 0;
            for (int i = 0; i < abs1.length; i++) {
                if (i >= minInd && i <= maxInd) {
                    d[iter++] = (float) Math.sqrt(Math.pow(abs1[i] - abs2[i], 2) + Math.pow(disp1[i] - disp2[i], 2));
                }
            }
            return d;

        }
    }

    /**
     * Performs a complex conjugation on the time domain data in order to flip 
     * the spectra. Operation sometimes if spectra were preprocessed by other 
     * programs.
     */
    void complexConjugate( ) 
    {
        
        for ( int i = 0; i < absorptionTD.length; i++ )
        {
            dispersionTD[i] = -dispersionTD[i];
        }
        
        TD2FD( absorptionTD, dispersionTD );
    }
}

class spectrumLocalShiftFinder implements Runnable {

    private final float[] refSignal;
    private final double minPPM;
    private final double maxPPM;
    private final double maxLocalShift;
    private final Spectrum spectrum;

    public spectrumLocalShiftFinder(float[] refSignal, double minPPM, double maxPPM, double maxLocalShift, Spectrum spectrum) {
        this.refSignal = refSignal;
        this.minPPM = minPPM;
        this.maxPPM = maxPPM;
        this.maxLocalShift = maxLocalShift;
        this.spectrum = spectrum;
    }

    @Override
    public void run() {
        //Reset maximum correlation value
        spectrum.correlationAlignment = Double.NEGATIVE_INFINITY;

        //Calculate mean of the reference signal
        double meanRef = 0;
        for (int j = 0; j < refSignal.length; j++) {
            meanRef += refSignal[j];
        }
        meanRef /= refSignal.length;

        //Calculate standard deviation of the reference signal
        //Note: We don't divide here by N because this will cancel out later
        double stdDevRef = 0;
        for (int j = 0; j < refSignal.length; j++) {
            stdDevRef += Math.pow(refSignal[j] - meanRef, 2);
        }

        //Save the value of the original shift
        double previousShift = spectrum.ppmshift;
        double bestShift = 0;

        //Step between the different shift values that we want to test
        double shiftStep = (spectrum.spectra.getXAxis(Spectra.Domain.FD_PPM)[1] - spectrum.spectra.getXAxis(Spectra.Domain.FD_PPM)[0]) / 2;
        for (double extraShift = -maxLocalShift; extraShift <= maxLocalShift; extraShift += shiftStep) {
            //Pearson's Correlation coefficient
            double corr = 0;
            double meanLocal = 0;
            double shift = previousShift + extraShift;
            spectrum.frequencyShift(shift);
            float[] mag = spectrum.getMagnitudeFD(minPPM, maxPPM);
            for (int j = 0; j < mag.length; j++) {
                meanLocal += mag[j];
            }
            meanLocal /= refSignal.length;

            double stdDevLocal = 0;
            for (int j = 0; j < refSignal.length; j++) {
                stdDevLocal += Math.pow(mag[j] - meanLocal, 2);
            }

            for (int j = 0; j < refSignal.length; j++) {
                corr += (refSignal[j] - meanRef) * (mag[j] - meanLocal);
            }
            corr /= Math.sqrt(stdDevLocal * stdDevRef);

            if (corr > spectrum.correlationAlignment) {
                bestShift = shift;
                spectrum.correlationAlignment = corr;
            }
        }
        spectrum.frequencyShift(bestShift);
        spectrum.spectra.localShiftThreadFinished();
    }

}
