/*
 * File: $Id: FitResultViewerFrame.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 08.10.2003, 09:02  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */ 

package org.spectrim.tdfdfit;

import org.spectrim.lib.StatusPanel;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.BoxLayout;
import javax.swing.KeyStroke;


/**
 *
 * Allows the viewing of the fit result files produced by TDFDFit.
 *
 * @author  hans
 */
public class FitResultViewerFrame extends JFrame 
{
    
    /** 
     * Creates a new instance of FitResultViewer. 
     * @param numberOfSamplePoints
     * @param fittingFileReference
     * @param title
     * @param calledDirectlyAfterFit
     */
    public FitResultViewerFrame( int numberOfSamplePoints, String fittingFileReference, 
                                 String title, boolean calledDirectlyAfterFit )
    {
        this.setSize( 768, 768 );
        setTitle( "TDFDFit result viewer" );
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BoxLayout( m_contentPanel, BoxLayout.Y_AXIS ) );
        m_contentPanel.setMaximumSize( new Dimension( 512, 512 ) );
        
        m_tabbedPane = new JTabbedPane( );
        
        m_fitResultSpectrumViewer = new FitResultSpectrumViewerPanel( title, numberOfSamplePoints, 
                                                                      fittingFileReference, this, calledDirectlyAfterFit );
        m_tabbedPane.addTab( "Spectra", m_fitResultSpectrumViewer );
        
        m_fitResultParameterViewerPanel = new FitResultParameterViewerPanel( fittingFileReference );
        m_tabbedPane.addTab( "Fit parameters", m_fitResultParameterViewerPanel );
        
        m_contentPanel.add( m_tabbedPane );
        
        m_statusPanel = new StatusPanel( this );
        
        m_fitResultSpectrumViewer.setStatusPanel( m_statusPanel );
        
        m_contentPanel.add( m_statusPanel );
        
        this.setContentPane( m_contentPanel );
        
        setVisible( true );
        // pack( );
        
        Action applyAndExit = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e ) 
            {
                dispose( );
            }
        };

        Action exitWithoutChange = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
               dispose( );
            }
        };

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
                                            "applyAndExit" );
        
        getRootPane( ).getActionMap( ).put( "applyAndExit",
                                            applyAndExit  );

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE, 0),
                                           "Cancel and quit" );
        
        getRootPane( ).getActionMap( ).put( "Cancel and quit",
                                            exitWithoutChange );
    }
    
    /**
     * Returns the measured data response.
     * @return 
     */
    public SingleVoxelSpectralData getMeasuredResponse( )
    {
        return m_fitResultSpectrumViewer.getMeasuredDataResponse( );
    }
    
    /**
     * Returns the measured data response.
     * @return 
     */
    public SingleVoxelSpectralData getFittedResponse( )
    {
        return m_fitResultSpectrumViewer.getFittedDataResponse( );
    }
    
    private final JPanel m_contentPanel;
    private final JTabbedPane m_tabbedPane;
    
    private final FitResultSpectrumViewerPanel m_fitResultSpectrumViewer;
    private final FitResultParameterViewerPanel m_fitResultParameterViewerPanel;
    private final StatusPanel m_statusPanel;
    
    // For proper serialization
    private static final long serialVersionUID = 1482255591289663L;
}
