/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

/**
 * Frame to manage default fitting parameter ranges. 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class FittingParameterRangesFrame extends javax.swing.JFrame {

    /**
     * Creates new form DefaultFittingParameterRangesFrame.
     */
    public FittingParameterRangesFrame( ) 
    {
        initComponents( );
        
        setTitle( "Default fitting parameter range editor" );
        
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        String displayUnits = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        
        if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
        {
            m_displayPointUnitsRadioButton.setSelected( true );
        }
        
        if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
        {
            m_displayHzUnitsRadioButton.setSelected( true );
        }
        
        if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            m_displayPPMUnitsRadioButton.setSelected( true );
        }
        
        MRSignalProperties mrSigProp = MRSignalProperties.getInstance( );
        
        switch ( mrSigProp.getNucleusType( ) )
        {
            case MRSignalProperties.PROTON:     
                                                m_nucleusTypeComboBox.setSelectedIndex( 0 );
                                                break;
            case MRSignalProperties.CARBON13: 
                                                m_nucleusTypeComboBox.setSelectedIndex( 1 );
                                                break;
            case MRSignalProperties.SODIUM23:   
                                                m_nucleusTypeComboBox.setSelectedIndex( 2 );
                                                break; 
            case MRSignalProperties.PHOSPHORUS31: 
                                                  m_nucleusTypeComboBox.setSelectedIndex( 3 );
                                                  break;
            
            default: // do nothing
        }
        
        // Lookup the values  
        DefaultFittingParameterRangeValues defVals = DefaultFittingParameterRangeValues.getInstance( );
        
        if  ( defVals.isAreaMaxDeterminedByFactor( ) == true )
        {
            m_areaAbsolutRadioButton.setSelected( false );
            m_areaMaxValDecimalTextField.setEnabled( false );
            m_areaMaxFactorValDecimalTextField.setEnabled( true );
        }
        else
        {
            m_areaAbsolutRadioButton.setSelected( true );
            m_areaMaxValDecimalTextField.setEnabled( true );
            m_areaMaxFactorValDecimalTextField.setEnabled( false );
        }
        
        // Set further properties of the DecimalTextField objects
        m_areaMinValDecimalTextField.setFineAdjustmentValue( 0.01f );
        m_areaMinValDecimalTextField.setMediumAdjustmentValue( 0.1f );
        m_areaMinValDecimalTextField.setCoarseAdjustmentValue( 1.0f );
        m_areaMinValDecimalTextField.enableGUIInteraction( true );
        m_areaMinValDecimalTextField.setFloatValue( defVals.getAreaMinValDecimalValue( ) );
        
        m_offsetMinValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_offsetMinValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_offsetMinValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_offsetMinValDecimalTextField.enableGUIInteraction( true );
        m_offsetMinValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMinValDecimalValue( ) ) );
                
                
        m_phaseMinValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_phaseMinValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_phaseMinValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_phaseMinValDecimalTextField.enableGUIInteraction( true );
        m_phaseMinValDecimalTextField.setFloatValue( defVals.getPhaseMinValDecimalValue( ) );
 
        
        m_lorentzianWidthMinValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_lorentzianWidthMinValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_lorentzianWidthMinValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_lorentzianWidthMinValDecimalTextField.enableGUIInteraction( true );
        m_lorentzianWidthMinValDecimalTextField.setFloatValue( defVals.getLorentzianWidthMinValDecimalValue( ) );
                
                
        m_gaussianWidthMinValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_gaussianWidthMinValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_gaussianWidthMinValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_gaussianWidthMinValDecimalTextField.enableGUIInteraction( true );
        m_gaussianWidthMinValDecimalTextField.setFloatValue( defVals.getGaussianWidthMinValDecimalValue( ) );
        
        // set further properties of the DecimalTextField objects
        m_areaMaxValDecimalTextField.setFineAdjustmentValue( 0.01f );
        m_areaMaxValDecimalTextField.setMediumAdjustmentValue( 0.1f );
        m_areaMaxValDecimalTextField.setCoarseAdjustmentValue( 1.0f );
        m_areaMaxValDecimalTextField.enableGUIInteraction( true );
        m_areaMaxValDecimalTextField.setFloatValue( defVals.getAreaMaxValDecimalValue( ) );
        
        m_offsetMaxValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_offsetMaxValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_offsetMaxValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_offsetMaxValDecimalTextField.enableGUIInteraction( true );
        m_offsetMaxValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMaxValDecimalValue( ) ) );
        
        m_phaseMaxValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_phaseMaxValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_phaseMaxValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_phaseMaxValDecimalTextField.enableGUIInteraction( true );
        m_phaseMaxValDecimalTextField.setFloatValue( defVals.getPhaseMaxValDecimalValue( ) );
                
        m_lorentzianWidthMaxValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_lorentzianWidthMaxValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_lorentzianWidthMaxValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_lorentzianWidthMaxValDecimalTextField.enableGUIInteraction( true );
        m_lorentzianWidthMaxValDecimalTextField.setFloatValue( defVals.getLorentzianWidthMaxValDecimalValue( ) );
                
        m_gaussianWidthMaxValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_gaussianWidthMaxValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_gaussianWidthMaxValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_gaussianWidthMaxValDecimalTextField.enableGUIInteraction( true );
        m_gaussianWidthMaxValDecimalTextField.setFloatValue( defVals.getGaussianWidthMaxValDecimalValue( ) );
                
        // m_areaMaxFactorValDecimalTextField.setFloatValue( 5f );
        m_areaMaxFactorValDecimalTextField.setFineAdjustmentValue( 0.1f );
        m_areaMaxFactorValDecimalTextField.setMediumAdjustmentValue( 1.0f );
        m_areaMaxFactorValDecimalTextField.setCoarseAdjustmentValue( 10.0f );
        m_areaMaxFactorValDecimalTextField.enableGUIInteraction( true );
        
        m_areaMaxFactorValDecimalTextField.setFloatValue( defVals.getAreaMaxFactorDecimalValue( ) );
        
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        this.setVisible( true );
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_areaButtonGroup = new javax.swing.ButtonGroup();
        m_displayUnitButtonGroup = new javax.swing.ButtonGroup();
        m_nucleusTypeSelectionPanel = new javax.swing.JPanel();
        m_nucleusTypeComboBox = new javax.swing.JComboBox();
        m_nucleusTypeLabel = new javax.swing.JLabel();
        m_defaultRangesPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        m_areaLabel = new javax.swing.JLabel();
        m_areaMinValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_areaMaxValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_offsetLabel = new javax.swing.JLabel();
        m_offsetMinValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_offsetMaxValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_phaseLabel = new javax.swing.JLabel();
        m_phaseMinValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_phaseMaxValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_areaAbsolutRadioButton = new javax.swing.JRadioButton();
        m_areaFactorRadioButton = new javax.swing.JRadioButton();
        m_lorentzianWidthMaxValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_lorentzianWidthMinValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_lorentzianWidthLabel = new javax.swing.JLabel();
        m_gaussianWidthMaxValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_gaussianWidthMinValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_lorentzianWidthLabel1 = new javax.swing.JLabel();
        m_areaMaxFactorValDecimalTextField = new org.spectrim.lib.DecimalTextField();
        m_displayUnitsPanel = new javax.swing.JPanel();
        m_displayPointUnitsRadioButton = new javax.swing.JRadioButton();
        m_displayHzUnitsRadioButton = new javax.swing.JRadioButton();
        m_displayPPMUnitsRadioButton = new javax.swing.JRadioButton();
        jPanel1 = new javax.swing.JPanel();
        m_quitButton = new javax.swing.JButton();
        m_saveAndQuitButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        m_nucleusTypeSelectionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Nucleus type selection"));

        m_nucleusTypeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<HTML><SUP>1</SUP><B>H</B></HTML>", "<HTML><SUP>13</SUP><B>C</B></HTML>", "<HTML><SUP>23</SUP><B>Na</B></HTML>", "<HTML><SUP>31</SUP><B>P</B></HTML>" }));

        m_nucleusTypeLabel.setText("Nucleus type");

        javax.swing.GroupLayout m_nucleusTypeSelectionPanelLayout = new javax.swing.GroupLayout(m_nucleusTypeSelectionPanel);
        m_nucleusTypeSelectionPanel.setLayout(m_nucleusTypeSelectionPanelLayout);
        m_nucleusTypeSelectionPanelLayout.setHorizontalGroup(
            m_nucleusTypeSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_nucleusTypeSelectionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_nucleusTypeLabel)
                .addGap(18, 18, 18)
                .addComponent(m_nucleusTypeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 108, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        m_nucleusTypeSelectionPanelLayout.setVerticalGroup(
            m_nucleusTypeSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_nucleusTypeSelectionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_nucleusTypeSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_nucleusTypeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_nucleusTypeLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_defaultRangesPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Parameter range definition"));

        jLabel1.setText("Parameter type");

        jLabel2.setText("Minimum");

        jLabel3.setText("Maximum");

        jLabel4.setText("Absolut");

        jLabel5.setText("Factor");

        jLabel6.setText("Factor value");

        m_areaLabel.setText("Area");

        m_areaMinValDecimalTextField.setText("0.0");

        m_areaMaxValDecimalTextField.setText("0.0");
        m_areaMaxValDecimalTextField.setEnabled(false);

        m_offsetLabel.setText("Offset");

        m_offsetMinValDecimalTextField.setText("0.0");

        m_offsetMaxValDecimalTextField.setText("0.0");

        m_phaseLabel.setText("Phase");

        m_phaseMinValDecimalTextField.setText("0.0");

        m_phaseMaxValDecimalTextField.setText("0.0");

        m_areaButtonGroup.add(m_areaAbsolutRadioButton);
        m_areaAbsolutRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_areaAbsolutRadioButtonActionPerformed(evt);
            }
        });

        m_areaButtonGroup.add(m_areaFactorRadioButton);
        m_areaFactorRadioButton.setSelected(true);
        m_areaFactorRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_areaFactorRadioButtonActionPerformed(evt);
            }
        });

        m_lorentzianWidthMaxValDecimalTextField.setText("0.0");

        m_lorentzianWidthMinValDecimalTextField.setText("0.0");

        m_lorentzianWidthLabel.setText("Lorentzian width");

        m_gaussianWidthMaxValDecimalTextField.setText("0.0");

        m_gaussianWidthMinValDecimalTextField.setText("0.0");

        m_lorentzianWidthLabel1.setText("Gaussian width");

        m_areaMaxFactorValDecimalTextField.setText("5.0");

        javax.swing.GroupLayout m_defaultRangesPanelLayout = new javax.swing.GroupLayout(m_defaultRangesPanel);
        m_defaultRangesPanel.setLayout(m_defaultRangesPanelLayout);
        m_defaultRangesPanelLayout.setHorizontalGroup(
            m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(m_areaLabel)
                    .addComponent(m_offsetLabel)
                    .addComponent(m_phaseLabel)
                    .addComponent(m_lorentzianWidthLabel)
                    .addComponent(m_lorentzianWidthLabel1))
                .addGap(18, 18, 18)
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(jLabel2)
                        .addComponent(m_areaMinValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 110, Short.MAX_VALUE)
                        .addComponent(m_offsetMinValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_phaseMinValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_lorentzianWidthMinValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(m_gaussianWidthMinValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 110, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                        .addGap(18, 18, 18)
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabel3)
                            .addComponent(m_areaMaxValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 110, Short.MAX_VALUE)
                            .addComponent(m_offsetMaxValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_phaseMaxValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_lorentzianWidthMaxValDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_defaultRangesPanelLayout.createSequentialGroup()
                        .addGap(16, 16, 16)
                        .addComponent(m_gaussianWidthMaxValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 10, Short.MAX_VALUE)
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_areaAbsolutRadioButton, javax.swing.GroupLayout.Alignment.TRAILING))
                .addGap(18, 18, 18)
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel5)
                    .addComponent(m_areaFactorRadioButton))
                .addGap(18, 18, 18)
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel6)
                    .addComponent(m_areaMaxFactorValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 110, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        m_defaultRangesPanelLayout.setVerticalGroup(
            m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                .addGap(47, 47, 47)
                .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                            .addComponent(jLabel6)
                            .addGap(43, 43, 43))
                        .addComponent(m_areaMaxFactorValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(jLabel4)
                            .addComponent(jLabel5))
                        .addGap(18, 18, 18)
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(m_areaLabel)
                                .addComponent(m_areaAbsolutRadioButton))
                            .addComponent(m_areaFactorRadioButton)))
                    .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                        .addComponent(jLabel2)
                        .addGap(18, 18, 18)
                        .addComponent(m_areaMinValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_offsetMinValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_offsetLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_phaseMinValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_phaseLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_lorentzianWidthMinValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_lorentzianWidthLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_defaultRangesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_gaussianWidthMinValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_lorentzianWidthLabel1)))
                    .addGroup(m_defaultRangesPanelLayout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addGap(18, 18, 18)
                        .addComponent(m_areaMaxValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_offsetMaxValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_phaseMaxValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_lorentzianWidthMaxValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_gaussianWidthMaxValDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_displayUnitsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Display units"));

        m_displayUnitButtonGroup.add(m_displayPointUnitsRadioButton);
        m_displayPointUnitsRadioButton.setText("Points");
        m_displayPointUnitsRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_displayPointUnitsRadioButtonActionPerformed(evt);
            }
        });

        m_displayUnitButtonGroup.add(m_displayHzUnitsRadioButton);
        m_displayHzUnitsRadioButton.setText("Hertz");
        m_displayHzUnitsRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_displayHzUnitsRadioButtonActionPerformed(evt);
            }
        });

        m_displayUnitButtonGroup.add(m_displayPPMUnitsRadioButton);
        m_displayPPMUnitsRadioButton.setText("ppm");
        m_displayPPMUnitsRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_displayPPMUnitsRadioButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_displayUnitsPanelLayout = new javax.swing.GroupLayout(m_displayUnitsPanel);
        m_displayUnitsPanel.setLayout(m_displayUnitsPanelLayout);
        m_displayUnitsPanelLayout.setHorizontalGroup(
            m_displayUnitsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_displayUnitsPanelLayout.createSequentialGroup()
                .addGap(173, 173, 173)
                .addComponent(m_displayPointUnitsRadioButton)
                .addGap(18, 18, 18)
                .addComponent(m_displayHzUnitsRadioButton)
                .addGap(18, 18, 18)
                .addComponent(m_displayPPMUnitsRadioButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        m_displayUnitsPanelLayout.setVerticalGroup(
            m_displayUnitsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_displayUnitsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_displayUnitsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_displayPointUnitsRadioButton)
                    .addComponent(m_displayHzUnitsRadioButton)
                    .addComponent(m_displayPPMUnitsRadioButton))
                .addContainerGap(23, Short.MAX_VALUE))
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Actions"));

        m_quitButton.setText("Quit");
        m_quitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_quitButtonActionPerformed(evt);
            }
        });

        m_saveAndQuitButton.setText("Save and Quit");
        m_saveAndQuitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_saveAndQuitButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(130, 130, 130)
                .addComponent(m_quitButton, javax.swing.GroupLayout.PREFERRED_SIZE, 129, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(67, 67, 67)
                .addComponent(m_saveAndQuitButton, javax.swing.GroupLayout.PREFERRED_SIZE, 129, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(29, 29, 29)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_quitButton)
                    .addComponent(m_saveAndQuitButton))
                .addContainerGap(33, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_nucleusTypeSelectionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_defaultRangesPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_displayUnitsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(m_nucleusTypeSelectionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_defaultRangesPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_displayUnitsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_areaFactorRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_areaFactorRadioButtonActionPerformed
        m_areaMaxFactorValDecimalTextField.setEnabled( true );
        m_areaMaxValDecimalTextField.setEnabled( false );
    }//GEN-LAST:event_m_areaFactorRadioButtonActionPerformed

    private void m_areaAbsolutRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_areaAbsolutRadioButtonActionPerformed
        m_areaMaxFactorValDecimalTextField.setEnabled( false );
        m_areaMaxValDecimalTextField.setEnabled( true );
    }//GEN-LAST:event_m_areaAbsolutRadioButtonActionPerformed

    private void m_quitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_quitButtonActionPerformed
        dispose( );
    }//GEN-LAST:event_m_quitButtonActionPerformed

    
    /** 
     * Saves the nucleus specific defaults and disposes the frame.
     * @param evt 
     */
    private void m_saveAndQuitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_saveAndQuitButtonActionPerformed
        
        MRSignalProperties mrSigProp = MRSignalProperties.getInstance( );
        
        switch ( mrSigProp.getNucleusType( ) )
        {
            case "N.D.":                            
                                                    handle1HDefaultsStorage( );break;
            case MRSignalProperties.PROTON: 
                                                    handle1HDefaultsStorage( );break;
            case MRSignalProperties.CARBON13: 
                                                    handle13CDefaultsStorage( );break;
            case MRSignalProperties.SODIUM23: 
                                                    handle23NaDefaultsStorage( );break; 
            case MRSignalProperties.PHOSPHORUS31: 
                                                    handle31PDefaultsStorage( );break;
            
            default: // do nothing
        }

        dispose( );
    }//GEN-LAST:event_m_saveAndQuitButtonActionPerformed
    
    public void saveIntermediateSettings( )
    {
        MRSignalProperties mrSigProp = MRSignalProperties.getInstance( );
        
        switch ( mrSigProp.getNucleusType( ) )
        {
            case "N.D.":                        handle1HDefaultsStorage( );break;
            case MRSignalProperties.PROTON:     handle1HDefaultsStorage( );break;
            case MRSignalProperties.CARBON13:   handle13CDefaultsStorage( );break;
            case MRSignalProperties.SODIUM23:   handle23NaDefaultsStorage( );break; 
            case MRSignalProperties.PHOSPHORUS31: handle31PDefaultsStorage( );break;
            
            default: handle1HDefaultsStorage( );
        }
    }
    
    /**
     * Display in point units.
     * @param evt 
     */
    private void m_displayPointUnitsRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_displayPointUnitsRadioButtonActionPerformed
        saveIntermediateSettings( );
        DefaultFittingParameterRangeValues defVals = DefaultFittingParameterRangeValues.getInstance( );
        m_offsetMinValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMinValDecimalValue( ) ) );
        m_offsetMaxValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMaxValDecimalValue( ) ) );
    }//GEN-LAST:event_m_displayPointUnitsRadioButtonActionPerformed

    private void m_displayHzUnitsRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_displayHzUnitsRadioButtonActionPerformed
        saveIntermediateSettings( );
        DefaultFittingParameterRangeValues defVals = DefaultFittingParameterRangeValues.getInstance( );
        m_offsetMinValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMinValDecimalValue( ) ) );
        m_offsetMaxValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMaxValDecimalValue( ) ) );
    }//GEN-LAST:event_m_displayHzUnitsRadioButtonActionPerformed

    private void m_displayPPMUnitsRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_displayPPMUnitsRadioButtonActionPerformed
        saveIntermediateSettings( );
        DefaultFittingParameterRangeValues defVals = DefaultFittingParameterRangeValues.getInstance( );
        m_offsetMinValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMinValDecimalValue( ) ) );
        m_offsetMaxValDecimalTextField.setFloatValue( convertOffsetToDisplayUnit( defVals.getOffsetMaxValDecimalValue( ) ) );
    }//GEN-LAST:event_m_displayPPMUnitsRadioButtonActionPerformed

    /**
     * Converts the displayed offset units to point units.
     * @param value
     * @return 
     */
    private float convertDisplayUnitsToPointUnits( float value )
    {
        float returnValue = 0f;
       
        MRSignalProperties props = MRSignalProperties.getInstance( );
       
        if ( m_displayPointUnitsRadioButton.isSelected( ) == true )
        {
            returnValue = value;
        }
        
        if ( m_displayHzUnitsRadioButton.isSelected( ) == true )
        {
            returnValue = value * props.getNumberOfSamplePoints( ) / props.getAcquistitionBandwidth( );
        }
        
        if ( m_displayPPMUnitsRadioButton.isSelected( ) == true )
        {
            returnValue = ( value * props.getNumberOfSamplePoints( ) / props.getAcquistitionBandwidth( ) ) * props.getResonanceFrequency( ) / 1000000f;
        }
        
        return returnValue; 
    }
    
    
    /**
     * Handles the persistency of 1H fitting range defaults.
     */
    private void handle1HDefaultsStorage( )
    {
        
        // Lookup the values  
        DefaultFittingParameterRangeValues defVals = DefaultFittingParameterRangeValues.getInstance( ); 
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( m_areaFactorRadioButton.isSelected( ) == true )
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H, "yes" );
        }
        else
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H, "no" );
        }
        
        defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H, "" + m_areaMaxFactorValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H, "" + m_areaMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H, "" + m_areaMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H, "" + m_gaussianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H, "" + m_gaussianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H, "" + m_lorentzianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H, "" + m_lorentzianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H, "" + convertDisplayUnitsToPointUnits( m_offsetMaxValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H, "" + convertDisplayUnitsToPointUnits( m_offsetMinValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H, "" + m_phaseMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H, "" + m_phaseMinValDecimalTextField.getFloatValue( ) );
        
        // save the new values into the file!
        defs.save( );
    }
        
     /**
     * Handles the persistency of 1H fitting range defaults.
     */
    private void handle13CDefaultsStorage( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( m_areaFactorRadioButton.isSelected( ) == true )
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_13C, "yes" );
        }
        else
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_13C, "no" );
        }
        
        defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_13C, "" + m_areaMaxFactorValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_13C, "" + m_areaMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_13C, "" + m_areaMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_13C, "" + m_gaussianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_13C, "" + m_gaussianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_13C, "" + m_lorentzianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_13C, "" + m_lorentzianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_13C, "" + convertDisplayUnitsToPointUnits( m_offsetMaxValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_13C, "" + convertDisplayUnitsToPointUnits( m_offsetMinValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_13C, "" + m_phaseMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_13C, "" + m_phaseMinValDecimalTextField.getFloatValue( ) );
        
        // save the new values into the file!
        defs.save( );
    }
    
    /**
     * Converts the offset value to display in the requested display units. 
     */
    private float convertOffsetToDisplayUnit( float value )
    {
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        String displayUnits = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        float returnValue = 0f;
        
        MRSignalProperties props = MRSignalProperties.getInstance( );
        
        if ( m_displayPointUnitsRadioButton.isSelected( ) == true )
        {
            returnValue = value;
        }
        
        if ( m_displayHzUnitsRadioButton.isSelected( ) == true )
        {
            returnValue = value * props.getAcquistitionBandwidth( ) / props.getNumberOfSamplePoints( );
        }
        
        if ( m_displayPPMUnitsRadioButton.isSelected( ) == true )
        {
            returnValue = ( value * props.getAcquistitionBandwidth( ) / props.getNumberOfSamplePoints( ) ) * 1000000f /  props.getResonanceFrequency( );
        }
        
        return returnValue;
    }
    
    
    /**
     * Handles the persistency of 1H fitting range defaults.
     */
    private void handle23NaDefaultsStorage( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( m_areaFactorRadioButton.isSelected( ) == true )
        {
            defs.setProperty(TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_23Na, "yes" );
        }
        else
        {
            defs.setProperty(TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_23Na, "no" );
        }
        
        defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_23Na, "" + m_areaMaxFactorValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_23Na, "" + m_areaMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_23Na, "" + m_areaMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_23Na, "" + m_gaussianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_23Na, "" + m_gaussianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_23Na, "" + m_lorentzianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_23Na, "" + m_lorentzianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_23Na, "" + convertDisplayUnitsToPointUnits( m_offsetMaxValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_23Na, "" + convertDisplayUnitsToPointUnits( m_offsetMinValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_23Na, "" + m_phaseMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_23Na, "" + m_phaseMinValDecimalTextField.getFloatValue( ) );
        
        // save the new values into the file!
        defs.save( );
    }
    
    /**
     * Handles the persistency of 1H fitting range defaults.
     */
    private void handle31PDefaultsStorage( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( m_areaFactorRadioButton.isSelected( ) == true )
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_31P, "yes" );
        }
        else
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_31P, "no" );
        }
        
        defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_31P, "" + m_areaMaxFactorValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_31P, "" + m_areaMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_31P, "" + m_areaMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_31P, "" + m_gaussianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_31P, "" + m_gaussianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_31P, "" + m_lorentzianWidthMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_31P, "" + m_lorentzianWidthMinValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_31P, "" + convertDisplayUnitsToPointUnits( m_offsetMaxValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_31P, "" + convertDisplayUnitsToPointUnits( m_offsetMinValDecimalTextField.getFloatValue( ) ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_31P, "" + m_phaseMaxValDecimalTextField.getFloatValue( ) );
        defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_31P, "" + m_phaseMinValDecimalTextField.getFloatValue( ) );
        
        // save the new values into the file!
        defs.save( );
    }
    /**
     * @param args the command line arguments
     */
    public static void main( String args[] ) 
    {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(FittingParameterRangesFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(FittingParameterRangesFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(FittingParameterRangesFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(FittingParameterRangesFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater( new Runnable( ) 
        {
            @Override
            public void run( ) 
            {
                new FittingParameterRangesFrame( ).setVisible( true );
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JRadioButton m_areaAbsolutRadioButton;
    private javax.swing.ButtonGroup m_areaButtonGroup;
    private javax.swing.JRadioButton m_areaFactorRadioButton;
    private javax.swing.JLabel m_areaLabel;
    private org.spectrim.lib.DecimalTextField m_areaMaxFactorValDecimalTextField;
    private org.spectrim.lib.DecimalTextField m_areaMaxValDecimalTextField;
    private org.spectrim.lib.DecimalTextField m_areaMinValDecimalTextField;
    private javax.swing.JPanel m_defaultRangesPanel;
    private javax.swing.JRadioButton m_displayHzUnitsRadioButton;
    private javax.swing.JRadioButton m_displayPPMUnitsRadioButton;
    private javax.swing.JRadioButton m_displayPointUnitsRadioButton;
    private javax.swing.ButtonGroup m_displayUnitButtonGroup;
    private javax.swing.JPanel m_displayUnitsPanel;
    private org.spectrim.lib.DecimalTextField m_gaussianWidthMaxValDecimalTextField;
    private org.spectrim.lib.DecimalTextField m_gaussianWidthMinValDecimalTextField;
    private javax.swing.JLabel m_lorentzianWidthLabel;
    private javax.swing.JLabel m_lorentzianWidthLabel1;
    private org.spectrim.lib.DecimalTextField m_lorentzianWidthMaxValDecimalTextField;
    private org.spectrim.lib.DecimalTextField m_lorentzianWidthMinValDecimalTextField;
    private javax.swing.JComboBox m_nucleusTypeComboBox;
    private javax.swing.JLabel m_nucleusTypeLabel;
    private javax.swing.JPanel m_nucleusTypeSelectionPanel;
    private javax.swing.JLabel m_offsetLabel;
    private org.spectrim.lib.DecimalTextField m_offsetMaxValDecimalTextField;
    private org.spectrim.lib.DecimalTextField m_offsetMinValDecimalTextField;
    private javax.swing.JLabel m_phaseLabel;
    private org.spectrim.lib.DecimalTextField m_phaseMaxValDecimalTextField;
    private org.spectrim.lib.DecimalTextField m_phaseMinValDecimalTextField;
    private javax.swing.JButton m_quitButton;
    private javax.swing.JButton m_saveAndQuitButton;
    // End of variables declaration//GEN-END:variables
}
