/*
 * File: $Id: MRLineModelParameters.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 27.05.2003, 7:23  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

/**
 * This class contains all attributes that are needed to describe a MR 
 * spectral resonance line for fitting. 
 * The parameters include area, offset, Lorentzian width, phase, 
 * and Gaussian width. 
 * @author  hans
 */
public class MRLineModelParameters extends MRFittingParameters
{
    /**
     * Default Constructor.
     */
    public MRLineModelParameters( )
    {
        super( );
    }
    
    /**
     * Constructor to construct an MRLineModelParameters object.
     * @param name
     * @param modelType
     * @param file_reference
     * @param line_group
     * @param metabolite
     * @param fitlsf
     * @param refline
     * @param area
     * @param offset
     * @param g_width
     * @param phase
     * @param l_width
     * @param area_min
     * @param offset_min
     * @param g_width_min
     * @param phase_min
     * @param l_width_min
     * @param area_max
     * @param offset_max
     * @param g_width_max
     * @param phase_max
     * @param l_width_max
     * @param area_prior
     * @param offset_prior
     * @param g_width_prior
     * @param phase_prior
     * @param l_width_prior
     */
    public MRLineModelParameters( String name, int modelType, String file_reference,
                                  String line_group, String metabolite, boolean fitlsf, boolean refline,
                                  float area, float offset, float g_width, float phase, float l_width,
                                  float area_min, float offset_min, float g_width_min, float phase_min, float l_width_min, 
                                  float area_max, float offset_max, float g_width_max, float phase_max, float l_width_max,
                                  String area_prior, String offset_prior, String g_width_prior, String phase_prior, String l_width_prior )
    {
        super( name, modelType, file_reference,
               line_group, metabolite, fitlsf, refline,
               area, offset, g_width, phase, l_width,
               area_min, offset_min, g_width_min, phase_min, l_width_min, 
               area_max, offset_max, g_width_max, phase_max, l_width_max,
               area_prior, offset_prior, g_width_prior, phase_prior, l_width_prior );
    }
    
    
    /**
     * Copy constructor.
     * @param lineParameters
     */
    public MRLineModelParameters( MRLineModelParameters lineParameters )
    {
        super( lineParameters.getName( ), lineParameters.getModelType( ), 
               lineParameters.getFileReferenceName( ), lineParameters.getLineGroup(), 
               lineParameters.getMetabolite( ), lineParameters.getFitWithArbitraryLineShapeFunction( ), 
               lineParameters.getFitWithReferenceLineShape( ), lineParameters.getArea( ),
               lineParameters.getOffset( ), lineParameters.getGaussianWidth( ),
               lineParameters.getPhase( ), lineParameters.getLorentzianLineWidth( ), 
               lineParameters.getMinimumArea( ), lineParameters.getMinimumOffset( ),
               lineParameters.getMinimumGaussianWidth( ), lineParameters.getMinimumPhase( ),
               lineParameters.getMinimumLorentzianWidth( ), lineParameters.getMaximumArea( ),
               lineParameters.getMaximumOffset( ), lineParameters.getMaximumGaussianWidth( ), 
               lineParameters.getMaximumPhase( ), lineParameters.getMaximumLorentzianWidth( ),
               lineParameters.getAreaReference( ), lineParameters.getOffsetReference( ),
               lineParameters.getGaussianWidthReference( ), lineParameters.getOffsetReference( ),
               lineParameters.getLorentzianLineWidthReference( ) );
    }
    
    /**
     * Sets the fitting parameters of this object after the object is constructed:
     * usage: for updating the object
     * @param fitPars
     */
    public void setFittingParameters( MRFittingParameters fitPars )
    {
        m_name = fitPars.getName( );
        m_modelType = fitPars.getModelType( );
        m_fileReferenceName = fitPars.getFileReferenceName( );
        m_lineGroup = fitPars.getLineGroup( );
        m_metabolite = fitPars.getMetabolite( );
        m_fitWithArbitraryLineShapeFunction = fitPars.getFitWithArbitraryLineShapeFunction( );
        m_fitWithReferenceLineShapeFunction = fitPars.getFitWithReferenceLineShape( );
        
        // Fit parameter values
        m_area = fitPars.getArea( );
        m_offset = fitPars.getOffset( );
        m_gaussianWidth = fitPars.getGaussianWidth( );
        m_phase = fitPars.getPhase( );
        m_lorentzianWidth = fitPars.getLorentzianLineWidth( );
        
        // Minimum parameter values
        m_minimumArea = fitPars.getMinimumArea( );
        m_minimumOffset = fitPars.getMinimumOffset( );
        m_minimumGaussianWidth = fitPars.getMinimumGaussianWidth( );
        m_minimumPhase = fitPars.getMinimumPhase( );
        m_minimumLorentzianWidth = fitPars.getMinimumLorentzianWidth( );
        
        // Maximum parameter values
        m_maximumArea = fitPars.getMaximumArea( );
        m_maximumOffset = fitPars.getMaximumOffset( );
        m_maximumGaussianWidth = fitPars.getMaximumGaussianWidth( );
        m_maximumPhase = fitPars.getMaximumPhase( );
        m_maximumLorentzianWidth = fitPars.getMaximumLorentzianWidth( );
        
        // Prior knowledge references
        m_areaReference = fitPars.getAreaReference( );
        m_offsetReference = fitPars.getOffsetReference( );
        m_gaussianWidthReference = fitPars.getGaussianWidthReference( );
        m_phaseReference = fitPars.getPhaseReference( );
        m_lorentzianWidthReference = fitPars.getLorentzianLineWidthReference( );
    }
    
    /**
     * Returns a reference to the MRFittingParameters object.
     * @return 
     */
    public MRFittingParameters getMRFittingParameters( )
    {
        return (MRFittingParameters)this;
    }

}