/*
 * File: $Id: MRLinearRegression.java,v 1.3 2008-11-12 08:33:44 cvs Exp $
 * History
 * Date: 12.11.2008, 17:58  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2008 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.Color;

/**
 * 
 * Computes the linear regression of a set of points ( x(i), y(i) )
 *  
 * @author hans
 */
public class MRLinearRegression 
{

    /**
     * 
     * @param x : array of observed x-values 
     * @param y : array of observed y-values
     */
    public MRLinearRegression( double[] x, double[] y, boolean plot )
    {
        if ( x.length == y.length )
        {
            m_x = new double[x.length];
            m_y = new double[y.length];
            
            System.arraycopy( x, 0, m_x, 0, x.length );
            System.arraycopy( y, 0, m_y, 0, y.length );
            
            double sumXY = 0.0;
            double sumX = 0.0;
            double sumY = 0.0;
            double sumX2 = 0.0;
            
            for ( int i = 0; i < x.length; i++ )
            {
                sumXY = sumXY + m_x[i] * m_y[i];
                sumX = sumX + m_x[i];
                sumY = sumY + m_y[i];
                sumX2 = sumX2 + m_x[i] * m_x[i];
            }
            
            // estimated slope and intercept parameters
            m_slope = ( m_x.length* sumXY - sumX * sumY ) / ( m_x.length * sumX2 - sumX * sumX );
            m_intercept = ( sumY - m_slope * sumX ) / m_x.length;
            
            if ( plot == true )
            {
                XYPlotFrame xyplotPhase = new XYPlotFrame( "Linear regression", null );
                xyplotPhase.addCurve( new Curve( m_x, m_y, Color.BLUE ), true );
                m_compFuncVal = getComputedFuncVal( );
                xyplotPhase.addCurve( new Curve( m_x, m_compFuncVal, Color.RED ), true );
            }
        }
        else
        {
            // thows exception
        }
        
    }
    /**
     * Returns the value of the fitted line at position x
     * @param x
     * @return
     */
     public double getYvalue( double x )
     {
        return m_slope * x + m_intercept;
     }
     
    /**
     *  Returns the intercept
     * @return
     */
    public double getIntercept( )
    {
        return m_intercept;
    }
     
    /**
     *  Returns the intercept
     * @return
     */
    public double getSlope( )
    {
        return m_slope;
    }

    /**
     *  Returns the residue: difference between input and estimated line
     * @return
     */
    public double[] getResidue( )
    {
        m_residue = new double[ m_x.length ];
        
        for ( int i = 0; i < m_x.length; i++ )
        {
            m_residue[i] = getYvalue( m_x[i] ) - m_y[i];
        }
        
        return m_residue;
        
    }
    
    
    /**
     * 
     */
    private double[] getComputedFuncVal( )
    {
        
        m_compFuncVal = new double[m_x.length];
        
        for ( int i=0; i < m_x.length; i++ )
        {
            m_compFuncVal[i] = m_slope * m_x[i] + m_intercept;
        }
        return m_compFuncVal;
    }
    /**
     * 
     * Returns the variance of the residual function
     *  
     * @return
     */
    public double getVarianceOfResidue( )
    {
        double av = 0.0;
        
        m_residue = getResidue( );
        
        for( int i = 0; i < m_residue.length; i++ )
        {
            av = av + m_residue[i];
        }
        
        av = av/m_residue.length;
        
        double stdev =0.0f;
        for ( int i = 0; i < m_residue.length; i++ )
        {
            stdev = stdev + (m_residue[i]-av)*(m_residue[i]-av);
        }
        
        stdev = Math.sqrt( stdev /( m_residue.length - 1) );
        
        return stdev;
    }
    
    public static void main( String[] args ) 
    {
       
         
       
         double[] x = new double[4];
         double[] y = new double[4];
         
         x[0]=1.0;
         x[1]=2.0;
         x[2]=3.0;
         x[3]=4.0;
         
         y[0]=1.5;
         y[1]=1.6;
         y[2]=2.1;
         y[3]=3.0;
         
         MRLinearRegression linReg = new MRLinearRegression( x, y, true );
         
         System.out.println( "Slope     : " + linReg.getSlope( ) );
         System.out.println( "Intercept : " + linReg.getIntercept( ) );
         
    }
    private double m_slope = 0;
    private double m_intercept = 0;
    private double[] m_x;
    private double[] m_y;
    private double[] m_residue;
    private double[] m_compFuncVal;
}