/*
 * File: $Id: MRMeasuredPatientSpectrumModelViewer.java,v 1.4 2006/10/02 06:13:03 hans Exp $
 * History
 * Date: 31.10.2003, 09:22  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.tdfdfit;


// import com.sun.glass.ui.Cursor;
import org.spectrim.tdfdfit.waterreferencemodelling.TwoCompartmentWaterReferenceModelFittingFrame;
import org.spectrim.tdfdfit.quantitativeresults.QuantitativeSVSContextDataContainer;

import org.spectrim.lib.Dialogs;
import org.spectrim.xyplot.XYPlotRange;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.Rectangle;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.InputEvent;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

import java.awt.print.Book;
import java.awt.print.Printable;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import javax.swing.JOptionPane;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.dicom.DicomHeaderViewer;
import org.spectrim.dicom.DicomSiemensSpecificFileHeaderViewer;
import org.spectrim.lib.DicomStringReformatter;
import org.spectrim.model.GlobalModel;


/**
 * 
 * Class that allows viewing and manipulation of a measured clinical 
 * single voxel spectrum.
 *
 * @author  hans
 */

public class MRMeasuredClinicalSVSSpectrumViewer extends SingleVoxelSpectrumViewer implements ActionListener, MouseListener, MouseMotionListener, Printable
{
    
    
    /** 
     * Creates a new instance of MRMeasuredSpectrumModelViewer. 
     * @param title
     * @param parent
     * @param ddir
     * @param model
     */
    public MRMeasuredClinicalSVSSpectrumViewer( String title, JFrame parent, DicomDirectory ddir, GlobalModel model ) 
    {
        super( title, parent, ddir );
        
        m_parent = parent;
        m_dicomDir = ddir;
        m_defaultCursor = m_parent.getCursor( );
        // addMouseListener( this );
        // addMouseMotionListener( this );
        // This class adds functionality to the popupmenu of the base classes
        addPopupMenuItems( );
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        if ( def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            this.setDisplayXReversed( true ); // Call the base class function to indicate that a ppm scale is drawn reversed
        }
    }
    
    /**
     * This method is needed to pass the GlobalModel to the SCANalyze
     * part of the software.
     * @param model 
     */
    public void setGlobalModel( GlobalModel model )
    {
        m_globalModel = model;
    }
    
    
    /**
     * Calls the constructor of TwoCompartmentWaterReferenceModelFittingFrame, 
     * enabling the user to define the water reference signal(s), necessary to
     * estimate "absolute" concentrations.
     */
    public void defineWaterReferenceSignals( ) 
    {
        m_autoTwoCompartmentModelCalcationFrame = new TwoCompartmentWaterReferenceModelFittingFrame( m_parent, m_dicomDir );
        m_autoTwoCompartmentModelCalcationFrame.addActionListener( this );
        m_autoTwoCompartmentModelCalcationFrame.setAlwaysOnTop( true );
    }
    
    /**
     * Adds entry point for user interactive spectrum model editing.
     */
    private void addPopupMenuItems( )
    {
        
        m_storeDisplayedSignalInFile = new JMenuItem( "Export response" );
        m_storeDisplayedSignalInFile.addActionListener( this );
        m_storeDisplayedSignalInFile.setToolTipText( "Exports currently displayed signal only" );
        
        m_popupmenu.add( m_storeDisplayedSignalInFile );
        
        m_popupmenu.addSeparator( );
        
        m_fitSpectrumMenuItem = new JMenuItem( "Fit spectrum" );
        m_fitSpectrumMenuItem.addActionListener( this );
        m_popupmenu.add( m_fitSpectrumMenuItem );
        
        m_twoCompartmentModelMenu = new JMenu( "Two compartment model" ); 
        
        m_defineSignalsForTwoCompartmentModelCalcationMenuItem = new JMenuItem( "Define signals for 2 compartment model calculation .." );
        m_defineSignalsForTwoCompartmentModelCalcationMenuItem.addActionListener( this );
        m_twoCompartmentModelMenu.add( m_defineSignalsForTwoCompartmentModelCalcationMenuItem );
        
        
        m_performQualityWithReferenceLineMenuItem = new JMenuItem( "Perform quality" );
        m_performQualityWithReferenceLineMenuItem.addActionListener( this );
        m_performQualityWithReferenceLineMenuItem.setEnabled( false );
        
        m_twoCompartmentModelMenu.add( m_performQualityWithReferenceLineMenuItem );
        
        m_popupmenu.add( m_twoCompartmentModelMenu );
        
        m_popupmenu.addSeparator( );
        
        m_displaySignalType = new JMenu( "Display" );
        
        m_displaySpectrum = new JRadioButtonMenuItem( "Patient spectrum" );
        m_displaySpectrum.addActionListener(this);
        m_displaySpectrum.setEnabled( false );
        m_displaySignalType.add( m_displaySpectrum );
        
        m_displayReference = new JRadioButtonMenuItem( "Patient Water reference spectrum" );         
        m_displayReference.addActionListener( this );
        m_displaySignalType.add( m_displayReference );
        m_displayReference.setEnabled( false );
        
        // add to the signal types to button group
        m_signalDisplaySelector.add( m_displaySpectrum );
        m_signalDisplaySelector.add( m_displayReference );
        
        m_printSpectrum = new JMenuItem( "Print" );
        m_printSpectrum.addActionListener( this );
        m_printSpectrum.setEnabled( false );
        m_popupmenu.add( m_printSpectrum );
        
        m_viewDicomHeader = new JMenu( "View DICOM header" );
        
        m_viewDicomHeaderSpectrum = new JMenuItem( "Spectrum" );
        m_viewDicomHeaderSpectrum.setEnabled( false );
        m_viewDicomHeaderSpectrum.addActionListener( this );

        m_viewDicomHeaderReference = new JMenuItem( "Water reference" );
        m_viewDicomHeaderReference.setEnabled( false );
        m_viewDicomHeaderReference.addActionListener( this );
        
        m_viewDicomHeader.add( m_viewDicomHeaderSpectrum );
        m_viewDicomHeader.add( m_viewDicomHeaderReference );
        
        m_viewSiemensSpecificDicomHeader = new JMenu( "View Siemens Specific DICOM header" );
        
        m_viewSiemensSpecificDicomHeaderSpectrum = new JMenuItem( "Spectrum" );
        m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( false );
        m_viewSiemensSpecificDicomHeaderSpectrum.addActionListener( this );

        m_viewSiemensSpecificDicomHeaderReference = new JMenuItem( "Water reference" );
        m_viewSiemensSpecificDicomHeaderReference.setEnabled( false );
        m_viewSiemensSpecificDicomHeaderReference.addActionListener( this );
        
        m_viewSiemensSpecificDicomHeader.add( m_viewSiemensSpecificDicomHeaderSpectrum );
        m_viewSiemensSpecificDicomHeader.add( m_viewSiemensSpecificDicomHeaderReference );
        
        m_importToPatientDatabaseMenuItem = new JMenuItem( "Import spectrum to database .." );
        m_importToPatientDatabaseMenuItem.setEnabled( false );
        m_importToPatientDatabaseMenuItem.addActionListener( this );
        
        m_popupmenu.add( m_importToPatientDatabaseMenuItem );
        
        m_popupmenu.add( m_viewDicomHeader );
        m_popupmenu.add( m_viewSiemensSpecificDicomHeader );
        
        m_popupmenu.add( m_displaySignalType );
    }
    
    /**
     * Called whenever an action event occurs.
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        
        Object sender = e.getSource( );
        
        if ( sender == m_autoTwoCompartmentModelCalcationFrame )
        {
            if ( e.getID( ) == m_autoTwoCompartmentModelCalcationFrame.REFERENCE_WATER_STATUS_OK_ID )
            {
                m_performQualityWithReferenceLineMenuItem.setEnabled( true );
            }
        }
            
        if ( sender == m_performQualityWithReferenceLineMenuItem )
        {
            // The user wants to perform a lineshape correction using the
            // 
            QuantitativeSVSContextDataContainer waterRefData = QuantitativeSVSContextDataContainer.getInstance( );
            double[] waterRefReal = waterRefData.getWaterReferenceSignalReal( );
            double[] waterRefImag = waterRefData.getWaterReferenceSignalImag( );
            
           
            m_measuredPatientSpectrum.applyQuality( true, waterRefReal, waterRefImag );
            
            changeDisplayedSignal( );
            
            System.out.println( "Applying quality lineshape transformation" );
            
        }
        
        if ( sender instanceof MRFitStrategyPanel )
        {
            
            m_fitStrategyPanel = (MRFitStrategyPanel)sender;
            if ( e.getID( ) == MRFitStrategyPanel.DEFINE_GLOBAL_OFFSET_ANCHOR_EVENT )
            { 
                m_definitionMode = IN_GLOBAL_OFFSET_ANCHOR_DEFINITION_MODE;
                m_parent.setCursor( Cursor.getDefaultCursor( ) );            
            }
            if ( e.getID( ) == MRFitStrategyPanel.DEFINE_GLOBAL_OFFSET_EVENT )
            {
                m_definitionMode = IN_GLOBAL_ACTUAL_OFFSET_DEFINITION_MODE;
                m_parent.setCursor( Cursor.getDefaultCursor( ) );              
            }
        }
        
        if ( sender == m_storeDisplayedSignalInFile )
        {
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( ); 
            fdef.load( );
            
            SingleVoxelSpectralData specData = new SingleVoxelSpectralData( );
            
            specData = getNamedResponse( "quality corrected" );
            
            if ( specData.getAbsorptionChannelTD( ) == null )
            {
                specData = getNamedResponse( "measured patient" );
            }
            
            if ( specData.getAbsorptionChannelTD( ) == null )
            {
                specData = getNamedResponse( "1" );
            }
            
            float[] abs = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            SignalWriter swrA = new SignalWriter( "Save Displayed Complex Signal", abs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        }
        
        
        if ( sender == m_defineSignalsForTwoCompartmentModelCalcationMenuItem )
        {
            // m_twoCompartmentModelCalcationFrame = new TwoCompartmentModelFittingFrame( m_parent );

            m_autoTwoCompartmentModelCalcationFrame = new TwoCompartmentWaterReferenceModelFittingFrame( m_parent, m_dicomDir );
            // m_autoTwoCompartmentModelCalcationFrame = new TwoCompartmentWaterReferenceModelFittingFrame( m_parent, this );
            m_autoTwoCompartmentModelCalcationFrame.addActionListener( this );
        }
        
        if ( sender == m_displaySpectrum )
        {
            displaySpectrum( );
        }
        
        if ( sender == m_displayReference )
        {
            displayReferenceWaterSpectrum( );
        }
        
        if ( sender == m_fitSpectrumMenuItem )
        {
            QuantitativeSVSContextDataContainer cont = QuantitativeSVSContextDataContainer.getInstance( );
            if ( ( cont.getTwoCompartmentModelFitted( ) == true )&& 
                 ( cont.getSimpleWaterRefefenceSignalSet( ) == true )  )
            {
                if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).verifyOneToOnePriorKnowledge( true ) == false )
                {
                    MRSpectrumFitter specFitter = new MRSpectrumFitter( m_parent );
                    
                    specFitter.setGlobalModel( m_globalModel );
                    
                    SingleVoxelMeasuredSpectralData specData = 
                                            new SingleVoxelMeasuredSpectralData( );

                    specData = getNamedResponse( "quality corrected" );

                    if ( specData.getAbsorptionChannelTD( ) == null )
                    {
                        specData = getNamedResponse( "measured patient" );
                    }

                    if ( specData.getAbsorptionChannelTD( ) != null )
                    {
                        m_measuredPatientSpectrumFileName = specData.getPatientName( ).replace( ',', '_' ) + "_" +
                                                            specData.getSeriesNumber( ) + "_" +
                                                            specData.getSeriesDescription( ) + "_" +
                                                            specData.getAcquisitionDate( ).replace( '.', '_' );

                        if ( m_measuredPatientSpectrumFileName == null )
                        {
                            specFitter.setSpectrumToFit( "NoNameDefined", "" , specData );
                        }
                        else
                        {
                            String dicomFileRef = ((TDFDFitControlFrame)m_parent).getSignalPatientSpectrumFileReference( );
                            specFitter.setSpectrumToFit( m_dicomDir, dicomFileRef, m_measuredPatientSpectrumFileName, "" , specData );
                        }

                        specFitter.performSpectralFitting( );

                        // Get reference to the model used.
    //                    TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
    //                    QuantitativeSVSContextDataContainer qdc = QuantitativeSVSContextDataContainer.getInstance( );      
    //                    
    //                    // following statement enforces the parsing and loading of 
    //                    // .fitres file.
    //                    String resultsFileName = specFitter.getSpectrumFileName( );
    //                    qdc.readFitresultsToDataStructures( resultsFileName );
    //                    
    //                    QuantitativeFittingResultsOverviewFrame qFrame = new  
    //                    
    //                    QuantitativeFittingResultsOverviewFrame( m_parent,
    //                                                             qdc.get2CompartmentWaterReferenceFileNames( ),
    //                                                             qdc.getTwoCompartmentModelSet( ),
    //                                                             qdc.getTwoCompartmentModelFitted( ),
    //                                                             qdc.getSingleWaterReferenceDefined( ),
    //                                                             qdc.getReferenceWaterDefinedFileName( ),
    //                                                             qdc.getNumberOfComponentsFitted( ),
    //                                                             qdc.getNumberOfMeasuredSignalsFitted( ),
    //                                                         
    //                                                             model.getSpectralComponentsNames( ),
    //                                                             qdc.getAreas( ),
    //                                                             qdc.getAreaErrors( ),
    //                                                             qdc.getResonanceFrequencies( ),
    //                                                             qdc.getResonanceFrequenciesErrors( ),
    //                                                             qdc.getPhase( ),
    //                                                             qdc.getPhasesErrors( ),
    //                                                             qdc.getLorentzWidths( ),
    //                                                             qdc.getLorentzWidthsErrors( ),
    //                                                             qdc.getGaussianWidths( ),
    //                                                             qdc.getGaussianWidthsErrors( ),
    //                                                         
    //                                                             qdc.getNumberOf2CompWaterRefResponses( ),
    //                                                             qdc.getAmp1_TCM( ),
    //                                                             qdc.getAmp2_TCM( ),
    //                                                             qdc.getTC1_TCM( ),
    //                                                             qdc.getTC2_TCM( ),
    //                                                             qdc.getErrAmp1_TCM( ),
    //                                                             qdc.getErrAmp2_TCM( ),
    //                                                             qdc.getErr_TC1_TCM( ),
    //                                                             qdc.getErrAmp2_TCM( ),
    //                                                             qdc.getWaterReferenceSignalReal( ),
    //                                                             qdc.getWaterReferenceSignalImag( ),
    //                                                             qdc.getSVSJMRUIDicomTag( ),
    //                                                    
    //                                                             qdc.getReferenceImage0( ),
    //                                                             qdc.getReferenceImage1( ),
    //                                                             qdc.getReferenceImage2( ),
    //                                                             qdc.getMeasuredReal( ),
    //                                                             qdc.getMeasuredImag( ),
    //                                                             qdc.getFittedReal( ),
    //                                                             qdc.getFittedImag( )
    //                                                   );
    //                            
                    }
                }
                else
                {
                    Dialogs d = new Dialogs( );
                    int messageBox = d.messageBox( m_parent, "<HTML> Cannot start fitting  <BR> Not all required input data is defined </HTML>" );
                }
            }
            else
            {
                // JOptionPane.showMessageDialog( m_parent, "No water reference models defined", "No models", JOptionPane.INFORMATION_MESSAGE );
                
                
                if ( JOptionPane.showConfirmDialog( m_parent, "Fit the spectrum anyhow?", "No water reference models defined", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION) 
                {
                
                
                    if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).verifyOneToOnePriorKnowledge( true ) == false )
                    {
                        MRSpectrumFitter specFitter = new MRSpectrumFitter( m_parent );

                        specFitter.setGlobalModel( m_globalModel );

                        SingleVoxelMeasuredSpectralData specData = 
                                                new SingleVoxelMeasuredSpectralData( );

                        specData = getNamedResponse( "quality corrected" );

                        if ( specData.getAbsorptionChannelTD( ) == null )
                        {
                            specData = getNamedResponse( "measured patient" );
                        }

                        if ( specData.getAbsorptionChannelTD( ) != null )
                        {
                            m_measuredPatientSpectrumFileName = specData.getPatientName( ).replace( ',', '_' ) + "_" +
                                                                specData.getSeriesNumber( ) + "_" +
                                                                specData.getSeriesDescription( ) + "_" +
                                                                specData.getAcquisitionDate( ).replace( '.', '_' );

                            if ( m_measuredPatientSpectrumFileName == null )
                            {
                                specFitter.setSpectrumToFit( "NoNameDefined", "" , specData );
                            }
                            else
                            {
                                String dicomFileRef = ((TDFDFitControlFrame)m_parent).getSignalPatientSpectrumFileReference( );
                                specFitter.setSpectrumToFit( m_dicomDir, dicomFileRef, m_measuredPatientSpectrumFileName, "" , specData );
                            }

                            specFitter.performSpectralFitting( );

                            // Get reference to the model used.
        //                    TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        //                    QuantitativeSVSContextDataContainer qdc = QuantitativeSVSContextDataContainer.getInstance( );      
        //                    
        //                    // following statement enforces the parsing and loading of 
        //                    // .fitres file.
        //                    String resultsFileName = specFitter.getSpectrumFileName( );
        //                    qdc.readFitresultsToDataStructures( resultsFileName );
        //                    
        //                    QuantitativeFittingResultsOverviewFrame qFrame = new  
        //                    
        //                    QuantitativeFittingResultsOverviewFrame( m_parent,
        //                                                             qdc.get2CompartmentWaterReferenceFileNames( ),
        //                                                             qdc.getTwoCompartmentModelSet( ),
        //                                                             qdc.getTwoCompartmentModelFitted( ),
        //                                                             qdc.getSingleWaterReferenceDefined( ),
        //                                                             qdc.getReferenceWaterDefinedFileName( ),
        //                                                             qdc.getNumberOfComponentsFitted( ),
        //                                                             qdc.getNumberOfMeasuredSignalsFitted( ),
        //                                                         
        //                                                             model.getSpectralComponentsNames( ),
        //                                                             qdc.getAreas( ),
        //                                                             qdc.getAreaErrors( ),
        //                                                             qdc.getResonanceFrequencies( ),
        //                                                             qdc.getResonanceFrequenciesErrors( ),
        //                                                             qdc.getPhase( ),
        //                                                             qdc.getPhasesErrors( ),
        //                                                             qdc.getLorentzWidths( ),
        //                                                             qdc.getLorentzWidthsErrors( ),
        //                                                             qdc.getGaussianWidths( ),
        //                                                             qdc.getGaussianWidthsErrors( ),
        //                                                         
        //                                                             qdc.getNumberOf2CompWaterRefResponses( ),
        //                                                             qdc.getAmp1_TCM( ),
        //                                                             qdc.getAmp2_TCM( ),
        //                                                             qdc.getTC1_TCM( ),
        //                                                             qdc.getTC2_TCM( ),
        //                                                             qdc.getErrAmp1_TCM( ),
        //                                                             qdc.getErrAmp2_TCM( ),
        //                                                             qdc.getErr_TC1_TCM( ),
        //                                                             qdc.getErrAmp2_TCM( ),
        //                                                             qdc.getWaterReferenceSignalReal( ),
        //                                                             qdc.getWaterReferenceSignalImag( ),
        //                                                             qdc.getSVSJMRUIDicomTag( ),
        //                                                    
        //                                                             qdc.getReferenceImage0( ),
        //                                                             qdc.getReferenceImage1( ),
        //                                                             qdc.getReferenceImage2( ),
        //                                                             qdc.getMeasuredReal( ),
        //                                                             qdc.getMeasuredImag( ),
        //                                                             qdc.getFittedReal( ),
        //                                                             qdc.getFittedImag( )
        //                                                   );
        //                            
                        }
                    }
                    else
                    {
                        Dialogs d = new Dialogs( );
                        int messageBox = d.messageBox( m_parent, "<HTML> Cannot start fitting  <BR> Not all required input data is defined </HTML>" );
                    }
                }    
            }
        }
        
        if ( sender == m_printSpectrum )
        {
             print( );
        }
        
        if ( sender == m_viewDicomHeaderSpectrum )
        {
            DicomHeaderViewer view = new DicomHeaderViewer( m_parent, m_measuredPatientSpectrum.getDicomHeaderInfo( ) );
        }
        
        if ( sender == m_viewDicomHeaderReference )
        {
            DicomHeaderViewer view = new DicomHeaderViewer( m_parent, m_measuredPatientWaterReferenceSpectrum.getDicomHeaderInfo( ) );
        }
        
        if ( sender == m_viewSiemensSpecificDicomHeaderSpectrum )
        {
            DicomSiemensSpecificFileHeaderViewer view = new DicomSiemensSpecificFileHeaderViewer( m_parent, m_measuredPatientSpectrum.getSiemensSpecificDicomHeaderInfo( ) );
        }
        
        if ( sender == m_viewSiemensSpecificDicomHeaderReference )
        {
            DicomSiemensSpecificFileHeaderViewer view = new DicomSiemensSpecificFileHeaderViewer( m_parent, m_measuredPatientWaterReferenceSpectrum.getSiemensSpecificDicomHeaderInfo( ) );
        }
        
        super.actionPerformed( e );
    }
    
    /**
     * Displays the measured patient spectrum, if defined.
     */
    public void displaySpectrum( )
    {
        if ( this.m_measuredPatientSpectrum != null )
        {
            
            m_displaySpectrum.setSelected( true );
            
            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );
        
            deleteAllCurves( );
        
            m_measuredPatientSpectrum.setCurveColor( Color.black );
        
            addSpectralDataSet( "measured patient spectrum", this.m_measuredPatientSpectrum, true );
        }
        else
        {
            // cannot show measured patient spectrum 
        }
    }
    
    
    /**
     * Displays the reference water spectrum, if defined.
     */
    public void displayReferenceWaterSpectrum( )
    {
        if ( this.m_measuredPatientSpectrum != null )
        {
            m_displayReference.setSelected( true );
            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );
        
            deleteAllCurves( );
        
            m_measuredPatientWaterReferenceSpectrum.setCurveColor( Color.black );
        
            addSpectralDataSet( "measured patient water reference spectrum", m_measuredPatientWaterReferenceSpectrum, true );
        }
        else
        {
            // cannot show measured patient spectrum 
        }
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param specData
     */
    public void setMeasuredSpectrumToPreview( SingleVoxelMeasuredSpectralData specData, boolean refresh )
    {
        if ( refresh == true )
        {
            m_displaySpectrum.setSelected( true );

            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );

            deleteAllCurves( );

            specData.setCurveColor( Color.black );

            addSpectralDataSet( "measured", specData, refresh );
            m_viewDicomHeaderSpectrum.setEnabled( true );
            m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( true );
        }
        else
        {
            specData.setCurveColor( Color.black );
            addSpectralDataSet( "measured", specData, false );
        }
        
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param fileName
     * @param specData
     */
    public void setMeasuredSpectrum( String fileName, 
                                     SingleVoxelMeasuredSpectralData specData )
    {
        
        m_measuredPatientSpectrumFileName = fileName;
        
        m_displaySpectrum.setSelected( true );
        
        m_measuredPatientSpectrum = new SingleVoxelMeasuredSpectralData( );
        m_measuredPatientSpectrum.setSpectralDataPointsTD( specData.getAbsorptionChannelTD( ),
                                                           specData.getDispersionChannelTD( ) );
        
        m_measuredPatientSpectrum.setDicomHeaderInfo( specData.getDicomHeaderInfo( ) );
        m_measuredPatientSpectrum.setSiemensSpecificDicomHeaderInfo( specData.getSiemensSpecificDicomHeaderInfo( ) );
        
        setTitle( specData.getDicomHeaderInfo( ).getPatientName( ) + " (" +
                  specData.getDicomHeaderInfo( ).getPatientBirthDate() + ") - " + 
                  specData.getDicomHeaderInfo( ).getSeriesDescription( ) + " -  Series # : " + 
                  specData.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) + " - " +
                  specData.getDicomHeaderInfo( ).getAcquisitionDate( )
                );
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        deleteAllCurves( );
        
        m_measuredPatientSpectrum.setCurveColor( Color.black );
        
        addSpectralDataSet( "measured patient", m_measuredPatientSpectrum, true );
        
        clearAllLeftUpperInformationLines( );
        
        addLeftLowerInformationLine( "voxel size : " + specData.getVoxelDimensionsAsString( ) );
        
        m_viewDicomHeaderSpectrum.setEnabled( true );
        m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( true );
        m_importToPatientDatabaseMenuItem.setEnabled( true );
        m_displaySpectrum.setEnabled( true );
        m_printSpectrum.setEnabled( true );
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param specData
     */
    public void setMeasuredClinicalSpectrum( SingleVoxelMeasuredSpectralData specData )
    {
        
        m_measuredPatientSpectrumFileName = null;
        
        m_displaySpectrum.setSelected( true );
        
        m_measuredPatientSpectrum = new SingleVoxelMeasuredSpectralData( );
        
        float[] complexConjDisp = new float[ specData.getDispersionChannelTD( ).length ];
        for ( int i=0; i< specData.getDispersionChannelTD( ).length; i++ )
        {
            complexConjDisp[i] = -specData.getDispersionChannelTD( )[i];
        }
        m_measuredPatientSpectrum.setSpectralDataPointsTD( specData.getAbsorptionChannelTD( ),
                                                           complexConjDisp );
        
        
        if ( specData.getDicomHeaderInfo( ) != null )
        {
            m_measuredPatientSpectrum.setDicomHeaderInfo( specData.getDicomHeaderInfo( ) );
            m_measuredPatientSpectrum.setSiemensSpecificDicomHeaderInfo( specData.getSiemensSpecificDicomHeaderInfo( ) );
            DicomStringReformatter dformat = new DicomStringReformatter( );
            
            setTitle( "Patient data" );
            m_leftUpperInfoDisplayLines.clear( );
            m_leftUpperInfoDisplayLines.add( "Name & DOB    : " + dformat.reformatPatientName( specData.getDicomHeaderInfo( ).getPatientName( ) ) + " (*" + dformat.getDateMonthYear( specData.getDicomHeaderInfo( ).getPatientBirthDate( ) ) + ")" );
            m_leftUpperInfoDisplayLines.add( "Description      : " + specData.getDicomHeaderInfo( ).getSeriesDescription( ) );
            m_leftUpperInfoDisplayLines.add( "TE [msec]        : " + specData.getDicomHeaderInfo( ).getEchoTime( ) );
            m_leftUpperInfoDisplayLines.add( "TR [msec]        : " + specData.getDicomHeaderInfo( ).getTR( ) );
            m_leftUpperInfoDisplayLines.add( "Series number    : " + specData.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) );
            m_leftUpperInfoDisplayLines.add( "Instance number  : " + specData.getDicomHeaderInfo( ).getInstanceNumber( ).trim( ) );
            m_leftUpperInfoDisplayLines.add( "Acquisition date : " + specData.getDicomHeaderInfo( ).getSeriesDate( ) );
            m_leftUpperInfoDisplayLines.add( "Voxel size       : " + specData.getExcitedVolumeAsString( ) );
            
            // update the SingleVoxelMeasuredSpectralData instance also
            m_measuredPatientSpectrum.setPatientName( dformat.reformatPatientName( specData.getDicomHeaderInfo( ).getPatientName( ) ) );
            m_measuredPatientSpectrum.setPatientBirthDate( dformat.getDateMonthYear( specData.getDicomHeaderInfo( ).getPatientBirthDate( ) ) );
            m_measuredPatientSpectrum.setTE( new Double( specData.getDicomHeaderInfo( ).getEchoTime( ) ) );
            m_measuredPatientSpectrum.setTR( new Double( specData.getDicomHeaderInfo( ).getTR( ) ) );
            m_measuredPatientSpectrum.setSeriesNumber(  specData.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) );
            m_measuredPatientSpectrum.setAcquisitionDate( dformat.getDateMonthYear( specData.getDicomHeaderInfo( ).getSeriesDate( ) ) );
        }
        else 
        {   

            // In this case the specData attributes themselves should be used
            setTitle( "Patient data" );
            m_leftUpperInfoDisplayLines.clear( );
            m_leftUpperInfoDisplayLines.add( "Name & DOB   : " + specData.getPatientName( ) + " (*" + specData.getPatientBirthDate( ) + ")" );
            m_leftUpperInfoDisplayLines.add( "Description       : " + specData.getSeriesDescription( ) );
            m_leftUpperInfoDisplayLines.add( "TE [msec]        : " + specData.getTE( ) );
            m_leftUpperInfoDisplayLines.add( "TR [msec]        : " + specData.getTR( ) );
            m_leftUpperInfoDisplayLines.add( "Series number  :  " + specData.getSeriesNumber( ).trim( ) );
            m_leftUpperInfoDisplayLines.add( "Instance number  : " + specData.getInstanceNumber( ) );
            m_leftUpperInfoDisplayLines.add( "Acquisition date: " + specData.getAcquisitionDate( ) );
            m_leftUpperInfoDisplayLines.add( "Voxel size        : " + specData.getVoxelDimensionsAsString( ) );
            
            // update the SingleVoxelMeasuredSpectralData instance also
            m_measuredPatientSpectrum.setPatientName( specData.getPatientName( ) );
            m_measuredPatientSpectrum.setPatientBirthDate( specData.getPatientBirthDate( ) );
            m_measuredPatientSpectrum.setTE( specData.getTE( ) );
            m_measuredPatientSpectrum.setTR( specData.getTR( ) );
            m_measuredPatientSpectrum.setSeriesNumber( specData.getSeriesNumber( ) );
            m_measuredPatientSpectrum.setInstanceNumber( specData.getInstanceNumber( ) );
            m_measuredPatientSpectrum.setSeriesDescription( specData.getSeriesDescription( ) );
            m_measuredPatientSpectrum.setAcquisitionDate( specData.getAcquisitionDate( )  );
        }
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        deleteAllCurves( );
        
        m_measuredPatientSpectrum.setCurveColor( Color.black );
        
        addSpectralDataSet( "measured patient", m_measuredPatientSpectrum, true );
        
        // clearAllLeftUpperInformationLines( );
        
        // addLeftLowerInformationLine( "voxel size : " + specData.getVoxelDimensionsAsString( ) );
        
        m_viewDicomHeaderSpectrum.setEnabled( true );
        m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( true );
        m_importToPatientDatabaseMenuItem.setEnabled( true );
        m_displaySpectrum.setEnabled( true );
        m_printSpectrum.setEnabled( true );
    }
    
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param fileName
     * @param specData
     */
    public void setMeasuredPatientWaterReferenceSpectrum( String fileName, 
                                                          SingleVoxelMeasuredSpectralData specData )
    {
        
        m_measuredPatientWaterReferenceSpectrumFileName = fileName;
        
        m_displayReference.setSelected( true );
        
        m_measuredPatientWaterReferenceSpectrum = new SingleVoxelMeasuredSpectralData( );
        m_measuredPatientWaterReferenceSpectrum.setSpectralDataPointsTD( specData.getAbsorptionChannelTD( ),
                                                                         specData.getDispersionChannelTD( ) );
        
        m_measuredPatientWaterReferenceSpectrum.setDicomHeaderInfo( specData.getDicomHeaderInfo( ) );
        m_measuredPatientWaterReferenceSpectrum.setSiemensSpecificDicomHeaderInfo( specData.getSiemensSpecificDicomHeaderInfo( ) );
        if ( m_measuredPatientSpectrum != null )
        {
            m_measuredPatientSpectrum.applyQuality( false, m_measuredPatientWaterReferenceSpectrum );
        }
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        deleteAllCurves( );
        
        addSpectralDataSet( "quality corrected", m_measuredPatientSpectrum, true );
        m_viewDicomHeaderReference.setEnabled( true );
        m_viewSiemensSpecificDicomHeaderReference.setEnabled( true );
        m_displayReference.setEnabled( true );
        
    }
    
    /**
     * Called when the mouse button is pressed.
     * @param mouseEvent 
     */
    @Override
    public void mousePressed( MouseEvent mouseEvent ) 
    {
       
        // is only of interest if left button is pressed
        if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
        {
            m_mouseButtonPressedX = mouseEvent.getX( );
            m_mouseButtonPressedY = mouseEvent.getY( );
        }
        
        super.mousePressed( mouseEvent );
    }
    
    /**
     * Called when the mouse button is pressed.
     * @param mouseEvent
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent )
    {

        if ( m_mouseDragged == true )
        {
            m_mouseDragged = false;
            // is only of interest if left button is released
            if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
            {
            
                m_mouseButtonReleasedX = mouseEvent.getX( );
                m_mouseButtonReleasedY = mouseEvent.getY( );
            
                // continue only if the start and stop position are not the same
                if ( ( Math.abs( m_mouseButtonPressedX - m_mouseButtonReleasedX ) > 5 ) && 
                     ( Math.abs( m_mouseButtonPressedY - m_mouseButtonReleasedY ) > 5 ) )
                {
                    Point2D.Float start = determineValuesAtCursorPosition( new Point( m_mouseButtonPressedX, m_mouseButtonPressedY ) );
                    Point2D.Float stop = determineValuesAtCursorPosition( new Point( m_mouseButtonReleasedX, m_mouseButtonReleasedY ) );
                        
                    if ( ( start.x < stop.x ) && ( start.y < stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange( start.x, stop.x, start.y, stop.y );
                    }
                    if ( ( start.x < stop.x ) && ( start.y > stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange( start.x, stop.x, stop.y, start.y );   
                    }
                    if ( ( start.x > stop.x ) && ( start.y > stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange( stop.x, start.x, stop.y, start.y );
                    }
                    if ( ( start.x > stop.x ) && ( start.y < stop.y ) )
                    {   
                        m_xyForcedPlotRange = new XYPlotRange( stop.x, start.x, start.y, stop.y );   
                    }
                    
                    this.m_XYPlotRangeSelectorMenuItem.setSelected( true );
                    
                    changeDisplayedSignal( );
                    
                }
            }
        }
        
        super.mouseReleased( mouseEvent );
    }
    
    /**
     * Called when the mouse button is clicked.
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
        determineValuesAtCursorPosition( mouseEvent.getPoint( ) );
        m_parent.setCursor( m_defaultCursor );
        if ( m_definitionMode == IN_GLOBAL_OFFSET_ANCHOR_DEFINITION_MODE )
        {
            m_fitStrategyPanel.setExpectedOffsetValue( m_lastSelectedOffsetValue );
        }
        if ( m_definitionMode == IN_GLOBAL_ACTUAL_OFFSET_DEFINITION_MODE )
        {
            m_fitStrategyPanel.setActualOffsetValue( m_lastSelectedOffsetValue );
        }
        this.m_definitionMode = this.IN_DEFAULT_MODE;
        // Set the GUI back to what it was before.
        
        m_parent.setCursor( m_defaultCursor );
        super.mouseClicked( mouseEvent );
    }
    
    /**
     * Called when the mouse is dragged.
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
        m_mouseDragged = true;
        super.mouseDragged( mouseEvent );
    }
    
    /**
     * Called when the mouse enters the control.
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
        super.mouseEntered( mouseEvent );
    }
    
    /**
     * Called when the mouse exits.
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
        super.mouseExited( mouseEvent );
    }
    
    /**
     * Called when the mouse button is released.
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
        createStatusLineMessageMouseLocation( mouseEvent );
        super.mouseMoved( mouseEvent );
    }
    
    /**
     * Creates a status line containing the current location of the 
     * mouse pointer.
     */
    private void createStatusLineMessageMouseLocation( MouseEvent mouseEvent )
    {
        int xPos = mouseEvent.getX( );
        int yPos = mouseEvent.getY( );
        
        if ( ( xPos > this.getSize( ).getWidth( ) / 10 ) && ( xPos < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( yPos > this.getSize( ).getHeight( ) / 10 ) && ( yPos < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)xPos - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)yPos - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                xPosCorrected = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            else
                            {
                                xPosCorrected = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            
                            yPosCorrected = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                xPosCorrected = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                xPosCorrected = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            yPosCorrected = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        
                        m_statusPanel.setLeftMessage( "( x, y ) =  ( " + xPosCorrected + " , " + yPosCorrected + " )" );
                    }
                }
            }
        }
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    private void determineValuesAtCursorPosition( int xPos, int yPos )
    {
        
        if ( ( xPos > this.getSize( ).getWidth( ) / 10 ) && ( xPos < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( yPos > this.getSize( ).getHeight( ) / 10 ) && ( yPos < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)xPos - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)yPos - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        
                        // lorentzian amplitude must be corrected in order to get the correct height in freq. domain
                        m_lastSelectedAmplitudeValue = m_lastSelectedAmplitudeValue * 3.141592f * 2.0f / 1024.0f;
                    }
                }
            }
        }
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    private Point2D.Float determineValuesAtCursorPosition( Point p )
    {
        
        if ( ( p.x > this.getSize( ).getWidth( ) / 10 ) && ( p.x < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( p.y > this.getSize( ).getHeight( ) / 10 ) && ( p.y < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                    }
                }
            }
        }
        
        return new Point2D.Float( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue ); 
    }
    /**
     * Prints this viewer.
     * @param g
     * @param pf
     * @param pageIndex
     * @return
     * @throws PrinterException 
     */
    @Override
    public int print( Graphics g, PageFormat pf, int pageIndex ) throws PrinterException 
    {
        if ( pageIndex > 0 )
        {
            return Printable.NO_SUCH_PAGE;
        }
        Graphics2D g2d = (Graphics2D)g;
        g2d.translate( pf.getImageableX( ), pf.getImageableY( ) );
                
        Rectangle componentBounds = this.getBounds(null);
        double scaleX = pf.getImageableWidth( ) / componentBounds.width;
        double scaleY = pf.getImageableHeight( ) / componentBounds.height;
        /*
        if (scaleX < 1 || scaleY < 1) 
        {
            
            if (scaleX < scaleY) 
            {
                scaleY = scaleX;
            } 
            else 
            {
                scaleX = scaleY;
            }
        }
        */
        g2d.scale( scaleX, scaleY );
        
        paintComponent( g2d );
        
        return Printable.PAGE_EXISTS;  
    }
    
    /**
     * Prints the currently displayed spectrum.
     */
    public void print( )
    {
        //--- Create a new PrinterJob object
        PrinterJob printJob = PrinterJob.getPrinterJob( );
        
        //--- Create a new book to add pages to
        Book book = new Book( );
        
        //--- Add the document page using a landscape page format
        PageFormat documentPageFormat = new PageFormat( );
        documentPageFormat.setOrientation( PageFormat.LANDSCAPE );
        book.append ( this, documentPageFormat );
        
        //--- Tell the printJob to use the Book as the Pageable object
        printJob.setPageable( book );
        
        //--- Show the print dialog box. If the user clicks the
        //--- print button, we then proceed to print, else we cancel
        //--- the process.
        try 
        {
            if ( printJob.printDialog( ) )
            {
                printJob.print( );
            }
        }
        catch ( PrinterException e )
        {
        }  
    }
    
    
    /**
     * Enables printing if "enable" is set to true.
     * @param enable
     */
    public void enablePrinting( boolean enable )
    {
        m_printSpectrum.setEnabled( enable );
    }
    
    
    // Attributes of this class
    private JMenuItem m_fitSpectrumMenuItem;
    
    private JMenu m_twoCompartmentModelMenu;
    private JMenuItem m_defineSignalsForTwoCompartmentModelCalcationMenuItem;
    private JMenuItem m_performQualityWithReferenceLineMenuItem;
    
    private JMenuItem m_printSpectrum;
    
    private JMenu m_viewDicomHeader;
    private JMenuItem m_viewDicomHeaderSpectrum;
    private JMenuItem m_viewDicomHeaderReference;
    
    private JMenu m_viewSiemensSpecificDicomHeader;
    private JMenuItem m_viewSiemensSpecificDicomHeaderSpectrum;
    private JMenuItem m_viewSiemensSpecificDicomHeaderReference;
    
    private JMenuItem m_storeDisplayedSignalInFile;
    
    private JMenu m_displaySignalType;
    
    private JRadioButtonMenuItem m_displaySpectrum;
    private JRadioButtonMenuItem m_displayReference;
    private ButtonGroup m_signalDisplaySelector = new ButtonGroup( );
    
    private JMenuItem m_importToPatientDatabaseMenuItem;
    
//    private String m_measuredPatientSpectrumFileName;
//    private SingleVoxelMeasuredSpectralData m_measuredPatientSpectrum;
//    
//    private String m_measuredPatientWaterReferenceSpectrumFileName;
//    private SingleVoxelMeasuredSpectralData m_measuredPatientWaterReferenceSpectrum;
    
    protected int m_mouseButtonPressedX;
    protected int m_mouseButtonPressedY;
    
    protected int m_mouseButtonReleasedX;
    protected int m_mouseButtonReleasedY;
    
    protected float m_lastSelectedOffsetValue;
    protected float m_lastSelectedAmplitudeValue;
    
    boolean m_mouseDragged = false;
    
    private final JFrame m_parent;
    // private TwoCompartmentModelFittingFrame m_twoCompartmentModelCalcationFrame;
    private TwoCompartmentWaterReferenceModelFittingFrame m_autoTwoCompartmentModelCalcationFrame;

    private final DicomDirectory m_dicomDir;
    
    private final static int IN_DEFAULT_MODE = 0;
    private final static int IN_GLOBAL_OFFSET_ANCHOR_DEFINITION_MODE = 1;
    private final static int IN_GLOBAL_ACTUAL_OFFSET_DEFINITION_MODE = 2;
    
    
    private int m_definitionMode = IN_DEFAULT_MODE;
    
    java.awt.Cursor m_defaultCursor;
    private MRFitStrategyPanel m_fitStrategyPanel;
    
    private GlobalModel m_globalModel;
    
    
    // For proper serialization
    private static final long serialVersionUID = 1482114591289213L;
}
