/*
 * File: $Id: MRSpectrumFitter.java,v 1.2 2006/04/26 14:54:28 hans Exp $
 * History
 * Date: 27.05.2003, 12:03  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * MRSpectrumFitter.java
 *
 * Created on October 30, 2003, 10:41 AM
 */

package org.spectrim.tdfdfit;

import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.util.LinkedHashMap;
import java.util.LinkedList;

import javax.swing.JFrame;
import org.dcm4che2.data.DicomObject;
import org.spectrim.tdfdfit.quantitativeresults.QuantitativeFittingResultsOverviewFrame;
import org.spectrim.tdfdfit.quantitativeresults.QuantitativeSVSContextDataContainer;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.FileInfo;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;

import org.spectrim.lib.Dialogs;
import org.spectrim.lib.SwingWorker;
import org.spectrim.model.GlobalModel;


/**
 *
 * Does the fitting of a spectrum using the spectral model that is defined
 * in the MRModelSpectrum class.
 *
 * @author  hans
 */

public class MRSpectrumFitter 
{
 
    /** 
     * Creates a new instance of MRSpectrumFitter. 
     * @param parent
     */
    public MRSpectrumFitter( JFrame parent )
    {
        m_parent = parent;
    }
    
    /**
     * Sets the global model necessary to make report views.
     * @param model 
     */
    public void setGlobalModel( GlobalModel model )
    {
        m_globalModel = model;
    }
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     * @param specFileName
     * @param instanceNumberDescription
     * @param spectrum
     */
    public void setSpectrumToFit( String specFileName, 
                                  String instanceNumberDescription, 
                                  SingleVoxelSpectralData spectrum )
    {
        m_spectrumFileName = specFileName;
        m_instanceNumberDescription = instanceNumberDescription;
        m_spectrumToQuantify = (SingleVoxelMeasuredSpectralData)spectrum;
    }
    
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     * @param dicomDir
     * @param dicomFileRef
     * @param specFileName
     * @param instanceNumberDescription
     * @param spectrum
     */
    public void setSpectrumToFit( DicomDirectory dicomDir,
                                  String dicomFileRef,
                                  String specFileName, 
                                  String instanceNumberDescription, 
                                  SingleVoxelMeasuredSpectralData spectrum )
    {
        
        String referenceImage0 = null;
        String referenceImage1 = null;
        String referenceImage2 = null;
        
        m_dicomDir = dicomDir;
        m_dicomFileRef = dicomFileRef;
        m_spectrumFileName = specFileName;
        
        // This is manufacturer dependent
        DicomImageData d_ima_data = new DicomImageData( dicomFileRef );
        
        FileInfo fi = d_ima_data.getHeader( );
        
        DicomObject object = DicomHandler.getDicomObject( dicomFileRef );
        DicomHeader header = DicomHandler.getHeader( object );
        
        if ( fi.getManufacturer( ).contains( "Philips" ) == false )
        {
            LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );
            SingleVoxelMeasuredSpectralData specData =  new SingleVoxelMeasuredSpectralData( fi.getNonImageData( ) );
            specData.setDicomHeaderInfo( fi );
            specData.setSiemensSpecificDicomHeaderInfo( d_ima_data.getSiemensSpecificDataFromFile( ) );

            referenceImage0 = dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID( specData.getReferenceFile0_UID( ) );
            referenceImage1 = dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID( specData.getReferenceFile1_UID( ) );
            referenceImage2 = dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID( specData.getReferenceFile2_UID( ) );
        }
        else
        {
           String ref_imageUID1 = readHeaderValue(header, "(0008,1140)[0]->(0008,1155)");
           String ref_imageUID2 = readHeaderValue(header, "(0008,1140)[1]->(0008,1155)");
           String ref_imageUID3 = readHeaderValue(header, "(0008,1140)[2]->(0008,1155)");
           
           referenceImage0 = dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID( ref_imageUID1 );
           referenceImage1 = dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID( ref_imageUID2 );
           referenceImage2 = dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID( ref_imageUID3 );
           
        }
        
        
        QuantitativeSVSContextDataContainer qC = QuantitativeSVSContextDataContainer.getInstance( );
        
        // now set the reference file names in order to be able to draw 
        // a project voxel on top of the reference images.
        qC.setReferenceImage0( referenceImage0 );
        qC.setReferenceImage1( referenceImage1 );
        qC.setReferenceImage2( referenceImage2 );
             
        m_instanceNumberDescription = instanceNumberDescription;
        m_spectrumToQuantify = spectrum;
    }
    
    /**
     * Add hoc solution for reading Dicom Philips Tags
     * @param header
     * @param tag
     * @return 
     */
    protected String readHeaderValue( DicomHeader header, String tag ) 
    {
        DicomHeaderLine headerLine;
        headerLine = (DicomHeaderLine) header.get(tag);
        if ( headerLine != null ) 
        {
            String val = headerLine.Value.replace( "\"", "" );
            return val;
        } 
        else 
        {
            System.out.println( "No value found for tag: " + tag );
            return "0";
        }
    }
    
    
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     */
    public void performSpectralFitting( )
    {   
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        //String spectralDatabasePath = (String)fitDef.get( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        String spectralDatabasePath = (String)fitDef.get( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        
        m_spectrumPath = spectralDatabasePath + System.getProperty( "file.separator" );
        
        m_spectrumFileName = m_spectrumFileName.replaceAll( " ", "" );
        m_spectrumFileName = m_spectrumFileName.replace( '^', '_' );
        
        m_startParameterFileName = spectralDatabasePath.replaceAll( " ", "" ) + System.getProperty( "file.separator" ) + m_spectrumFileName + ".par"; 
        
        m_spectrumFileName = m_spectrumFileName + ".bin";
        
        TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        modSpec.createStartingValueFileToPath( m_startParameterFileName );
        
        m_spectrumToQuantify.writeAsTdfdfitFile( spectralDatabasePath.replaceAll( " ", "" ),  m_spectrumFileName );
        startSpectralFit( ); 
    }
    
    /**
     * Replaces all blanks by "\  " in order to be interpreted 
     * correctly by the operating system
     * @param cmd 
     * @return 
     */
    private String generateProperCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }
    
    
    /**
     * Performs the fit and keeps the interface responsive.
     */
    public void startSpectralFit( )
    {
       
       final SwingWorker worker = new SwingWorker( )
       {
            @Override
            public Object construct( )
            {
                try
                {
                    performFit( );
                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs d = new Dialogs( );
                    d.messageBox( m_parent, "<HTML> An out of memory error occurred  <BR> Close unused windows to free memory </HTML>" );
                }
                return null;
            }
                
            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                moveDeleteTDFDFitRelatedFiles( ); 
                showFitResults( );
            }
 
        };
        
        worker.start( );
    }
    
    public void moveDeleteTDFDFitRelatedFiles( ) 
    {
         // Clean up the bin directory!
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        String path = fitDef.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        int locationOfDot = m_spectrumFileName.lastIndexOf( "." );
        String fileNameBase = m_spectrumFileName.substring( 0, locationOfDot );
        
        File fileDotBin = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin" );
        File fileDotFit = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.fit" );
        File fileDotFit0 = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.fit0" );
        File fileDotFitRes = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.fitres" );
        File fileDotOrg = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.org" );
        File fileDotRes = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.res" );
        File fileDotCorOrg = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.cororg" );
        File fileDotExl = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.exl" );
        File fileDotNln0 = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.nln0" );
        File fileDotMat = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.mat" );
        File fileDotErr = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.err" );
        
        File parameterStartFile = new File( m_startParameterFileName );
        
        String verboseFileName = m_startParameterFileName.replace( ".par", "_verbose.par" );
        File verboseParameterStartFile = new File( verboseFileName );
        
        String outputFileNamePar = parameterStartFile.getName( );
        outputFileNamePar = outputFileNamePar.replace( ".par", "_output.par" );
        
        String outputVerboseParameterStartFileName = verboseParameterStartFile.getName( );
        
        File patDatabasePath = new File( fitDef.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH ) );
        if ( parameterStartFile.exists( ) && patDatabasePath.exists( ) )
        {
            File outputFile = new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputFileNamePar  ); 
            if ( outputFile.exists( ) )
            {
                outputFile.delete( );
            }
            
            if ( new File( outputFileNamePar ).exists( ) )
            {
                System.out.println( "ParameterStartFile exists." );
                parameterStartFile.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR, "no" );
                fitDef.save( );
            }
            
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR ).compareTo( "yes" ) == 0 )
            {
                parameterStartFile.delete( );
            }
            else
            {
                try 
                {
                    Files.move( parameterStartFile.toPath( ), new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputFileNamePar ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved parameter file : " + parameterStartFile.toPath( ) );
                } 
                catch (IOException ex) 
                {
                   System.out.println( "Error moving file " + parameterStartFile.toPath( ) );
                }
                
                //parameterStartFile.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputFileNamePar ) );
            }
            
            // ============================================================================================================================================
            
            File outputVerboseFile = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputVerboseParameterStartFileName );
            if ( outputVerboseFile.exists( ) )
            {
                outputVerboseFile.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR, "no" );
                fitDef.save( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR ).compareTo( "yes" ) == 0 )
            {
                verboseParameterStartFile.delete( );
            }
            else
            {
                
                try 
                {
                    Files.move( verboseParameterStartFile.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputVerboseParameterStartFileName ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + verboseParameterStartFile.toPath( ) );
                } 
                catch (IOException ex) 
                {
                   System.out.println( "Error moving file " + verboseParameterStartFile.toPath( ) );
                }
                
                
//                verboseParameterStartFile.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputVerboseParameterStartFileName ) );
            }
               
        }
        
        if ( fileDotBin.exists( ) && patDatabasePath.exists( ) )
        {
            
            File binfile = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".bin"  );
            if ( binfile.exists( ) )
            {
                binfile.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN, "no" );
                fitDef.save( );
            }        
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN ).compareTo( "yes" ) == 0 )
            {
                fileDotBin.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotBin.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".bin"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotBin.toPath( ) );
                } 
                catch (IOException ex) 
                {
                   System.out.println( "Error moving file " + fileDotBin.toPath( ) );
                } 
                              
//                fileDotBin.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".bin"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File fitfile = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fit"  );
            if ( fitfile.exists( ) )
            {
                fitfile.delete( );
            }
            
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT, "no" );
                fitDef.save( );
            }        
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT ).compareTo( "yes" ) == 0 )
            {
                fileDotFit.delete( );
            }
            else
            {
                
                try 
                {
                    Files.move( fileDotBin.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fit"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotFit.toPath( ) );
                } 
                catch (IOException ex) 
                {
                   System.out.println( "Error moving file " + fileDotFit.toPath( ) );
                } 
                
//                fileDotFit.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fit"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            
            File fitfile0 = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fit0"  );
            if ( fitfile.exists( ) )
            {
                fitfile0.delete( );
            }
                                    
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0 ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0, "no" );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0 ).compareTo( "yes" ) == 0 )
            {
                fileDotFit0.delete( );
            }
            else
            {
                
                try 
                {
                    Files.move( fileDotFit0.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fit0"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotFit0.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotFit0.toPath( ) );
                } 
                
//                fileDotFit0.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fit0"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File filedotfitres = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fitres"  );
            if ( filedotfitres.exists( ) )
            {
                filedotfitres.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES, "no" );
                fitDef.save( );
            }        
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES ).compareTo( "yes" ) == 0 )
            {
                fileDotFitRes.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotFitRes.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fitres"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotFitRes.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotFitRes.toPath( ) );
                } 
                
                
//                fileDotFitRes.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".fitres"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_org =  new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".org"  );
            if ( file_dot_org.exists( ) )
            {
                file_dot_org.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG, "no" );
                fitDef.save( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG ).compareTo( "yes" ) == 0 )
            {
                fileDotOrg.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotOrg.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".org"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotOrg.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotOrg.toPath( ) );
                } 
                
//                fileDotOrg.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".org"  ) );
            }
            
             //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             
            File file_dot_res = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".res"  );
            if ( file_dot_res.exists( ) )
            {
                file_dot_res.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES ).compareTo( "yes" ) == 0 )
            {
                fileDotRes.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotRes.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".res"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotRes.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotRes.toPath( ) );
                }
                
//                fileDotRes.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".res"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_cororg = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".cororg"  );
            if ( file_dot_cororg.exists( ) )
            {
                file_dot_cororg.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG ).compareTo( "yes" ) == 0 )
            {
                fileDotCorOrg.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotCorOrg.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".cororg"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotCorOrg.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotCorOrg.toPath( ) );
                }
                
//                fileDotCorOrg.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".cororg"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_exl = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".exl"  );
            if ( file_dot_exl.exists( ) )
            {
                file_dot_exl.delete( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL, "no" );
                fitDef.save( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL ).compareTo( "yes" ) == 0 )
            {
                fileDotExl.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotExl.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".exl"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotExl.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotExl.toPath( ) );
                }
                
//                fileDotExl.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".exl"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            
            File file_dot_nln_0 = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".nln0"  );
            if ( file_dot_nln_0.exists( ) )
            {
                file_dot_nln_0.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0 ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0 ).compareTo( "yes" ) == 0 )
            {
                fileDotNln0.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotNln0.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".nln0"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotNln0.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotNln0.toPath( ) );
                }
                
                
//                fileDotNln0.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".nln0"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_mat = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".mat"  );
            if ( file_dot_mat.exists( ) )
            {
                file_dot_mat.delete( );
            } 
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT ).compareTo( "yes" ) == 0 )
            {
                fileDotMat.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotMat.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".mat"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotMat.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotMat.toPath( ) );
                }
                
//                fileDotMat.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".mat"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_err = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".err"  );
            if ( file_dot_err.exists( ) )
            {
                file_dot_err.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR ).compareTo( "yes" ) == 0 )
            {
                fileDotErr.delete( );
            }
            else
            {
                
                try 
                {
                    Files.move( fileDotErr.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".mat"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
                    System.out.println( "Moved file : " + fileDotErr.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + fileDotErr.toPath( ) );
                }
                
//                fileDotErr.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + fileNameBase  + ".err"  ) );
            }
           
        }
        
        // renameTo does not move, in fact it is a copy therefore we also need to delete the source files explicitly!
        fileDotBin = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin" );
        if ( fileDotBin.exists( ) )
        {
            fileDotBin.delete( );
        }
        
        fileDotFit = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.fit" );
        if ( fileDotFit.exists( ) )
        {
            fileDotFit.delete( );
        }
        
        fileDotFit0 = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.fit0" );
        if ( fileDotFit0.exists( ) )
        {
            fileDotFit0.delete( );
        }
        
        fileDotFitRes = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.fitres" );
        if ( fileDotFitRes.exists( ) )
        {
            fileDotFitRes.delete( );
        }
        
        fileDotOrg = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.org" );
        if ( fileDotOrg.exists( ) )
        {
            fileDotOrg.delete( );
        }
        
        fileDotRes = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.res" );
        if ( fileDotRes.exists( ) )
        {
            fileDotRes.delete( );
        }
        
        fileDotCorOrg = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.cororg" );
        if ( fileDotCorOrg.exists( ) )
        {
            fileDotCorOrg.delete( );
        }
        
        fileDotExl = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.exl" );
        if ( fileDotExl.exists( ) )
        {
            fileDotExl.delete( );
        }
        
        fileDotNln0 = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.nln0" );
        if ( fileDotNln0.exists( ) )
        {
            fileDotNln0.delete( );
        }
        
        fileDotMat = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.mat" );
        if ( fileDotMat.exists( ) )
        {
            fileDotMat.delete( );
        }
        
        fileDotErr = new File( path + System.getProperty( "file.separator" ) + fileNameBase  + ".bin.err" );
        if ( fileDotErr.exists( ) )
        {
            fileDotErr.delete( );
        }
    }
    
    /**
     * Starts the fit that corresponds to the current status of the model
     * and the loaded measured spectrum.
     */
    private void performFit(  )
    {
        int returnValue;
        String cmd;

        TDFDFittingDefaults fittingDefaults = TDFDFittingDefaults.getInstance( );
        fittingDefaults.load( );
        
        String tdfdfitExecutablePath = fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        String tdfdfitExecutableName = fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_EXECUTABLE_NAME );
        
        cmd = tdfdfitExecutablePath  + System.getProperty( "file.separator" ) 
            + tdfdfitExecutableName + " " + m_spectrumPath + " " 
            + m_spectrumFileName + " " + m_startParameterFileName;

        cmd = generateProperCommand( cmd );
        
        // cmd = "tdfdfit C:\\\\Temp\\\\ FritzMagdalene_20020902_8_svs_se_30_left.bin C:\\Temp\\FritzMagdalene_20020902_8_svs_se_30_left.par";
        
        // System.out.println( "Trying following system command : " + cmd );
        try 
        {
            Process p = Runtime.getRuntime( ).exec( cmd );
            
            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 ) 
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX ) 
        {
            // not handled yet
            System.out.println( "IOException occurred" );
        }
        catch ( InterruptedException intX ) 
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
        }
        
    }
    
    /**
     * Shows the results of the fit in a separate frame.
     */
    private void showFitResults( )
    {
        MRSignalProperties mrSignalProperties = MRSignalProperties.getInstance( );
        
        String title = m_spectrumToQuantify.getPatientName( ) + " (*" +
                       m_spectrumToQuantify.getPatientBirthDate( ) + ") - " + 
                       m_spectrumToQuantify.getSeriesDescription( ) + " -  Series # : " + 
                       m_spectrumToQuantify.getSeriesNumber( ).trim( ) + " - " +
                       m_instanceNumberDescription +
                       m_spectrumToQuantify.getAcquisitionDate( );
        
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String patientPath = defs.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        FitResultViewerFrame fitViewer = new  FitResultViewerFrame( mrSignalProperties.getNumberOfSamplePoints( ), 
                                                                    patientPath + 
                                                                    System.getProperty( "file.separator" ) + 
                                                                    m_spectrumFileName, title, false );
        LinkedList<String> fileList = new LinkedList<>( );
        fileList.add( m_dicomFileRef );
        
        // Get reference to the model used.
        TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        QuantitativeSVSContextDataContainer qdc = QuantitativeSVSContextDataContainer.getInstance( );      
        
        qdc.setFittedSpectrumFileNames( fileList );
        qdc.getNumberOfComponentsFitted( );
        double[][] measuredReal = new double[1][fitViewer.getMeasuredResponse( ).getAbsorptionChannelTD().length];
        double[][] measuredImag = new double[1][fitViewer.getMeasuredResponse( ).getDispersionChannelTD().length];
        double[][] fittedReal   = new double[1][fitViewer.getFittedResponse( ).getAbsorptionChannelTD().length];
        double[][] fittedImag   = new double[1][fitViewer.getFittedResponse( ).getDispersionChannelTD().length];
        
        // currently only 1D signal files for SVS assumed...
        // needs to be refactored to handle also more than 1D cases
        for ( int i = 0; i < fitViewer.getMeasuredResponse( ).getAbsorptionChannelFD( ).length; i++ )
        {
            measuredReal[0][i] = (fitViewer.getMeasuredResponse( ).getAbsorptionChannelFD())[i];
            measuredImag[0][i] = (fitViewer.getMeasuredResponse( ).getDispersionChannelFD())[i];
            fittedReal[0][i] = (fitViewer.getFittedResponse( ).getAbsorptionChannelFD())[i];
            fittedImag[0][i] = (fitViewer.getFittedResponse( ).getDispersionChannelFD())[i];
        }
        
        qdc.setMeasuredReal( measuredReal );
        qdc.setMeasuredImag( measuredReal );
        qdc.setFittedReal( fittedReal );
        qdc.setFittedImag( fittedImag );

        // following statement enforces the parsing and loading of 
        // .fitres file.
        String resultsFileName = getSpectrumFileName( );
        int dotLocation = resultsFileName.lastIndexOf( "." );
        resultsFileName = resultsFileName.substring( 0, dotLocation );
        String resultsFileNameExl = resultsFileName + ".exl";
        resultsFileName = resultsFileName + "_verbose.par";

        qdc.readFitresultsToDataStructures( patientPath + System.getProperty( "file.separator" ) + resultsFileName );

        resultsFileNameExl = patientPath + System.getProperty( "file.separator" ) +  resultsFileNameExl;
        
        // code to read the FQN
        boolean everythingOK;
        resultsFileNameExl = resultsFileNameExl.replace( " ", "_" );
        NumericTDFDFitResults result = new NumericTDFDFitResults( resultsFileNameExl );
        boolean readOK = result.readFile( );
        if ( readOK == false )
        {
            everythingOK = false;
        }
        double FQN = result.getFitQualityNumber( );    
        
        QuantitativeFittingResultsOverviewFrame qFrame = new QuantitativeFittingResultsOverviewFrame( m_parent,
                                                 m_dicomDir,
                                                 qdc.getMeasuredSpectraFileNames( ),
                                                 qdc.getTwoCompartmentModelSet( ),
                                                 qdc.getTwoCompartmentModelFitted( ),
                                                 qdc.getSingleWaterReferenceDefined( ),
                                                 qdc.getReferenceWaterDefinedFileName( ),
                                                 qdc.getNumberOfComponentsFitted( ),
                                                 qdc.getNumberOfMeasuredSignalsFitted( ),

                                                 model.getSpectralComponentsNames( ),
                                                 qdc.getAreas( ),
                                                 qdc.getAreaErrors( ),
                                                 qdc.getResonanceFrequencies( ),
                                                 qdc.getResonanceFrequenciesErrors( ),
                                                 qdc.getPhase( ),
                                                 qdc.getPhasesErrors( ),
                                                 qdc.getLorentzWidths( ),
                                                 qdc.getLorentzWidthsErrors( ),
                                                 qdc.getGaussianWidths( ),
                                                 qdc.getGaussianWidthsErrors( ),

                                                 qdc.getNumberOfFittedResponses( ),
                                                 qdc.getAmp1_TCM( ),
                                                 qdc.getAmp2_TCM( ),
                                                 qdc.getTC1_TCM( ),
                                                 qdc.getTC2_TCM( ),
                                                 qdc.getErrAmp1_TCM( ),
                                                 qdc.getErrAmp2_TCM( ),
                                                 qdc.getErr_TC1_TCM( ),
                                                 qdc.getErrAmp2_TCM( ),
                                                 qdc.getWaterReferenceSignalReal( ),
                                                 qdc.getWaterReferenceSignalImag( ),
                                                 qdc.getSVSJMRUIDicomTag( ),

                                                 qdc.getReferenceImage0( ),
                                                 qdc.getReferenceImage1( ),
                                                 qdc.getReferenceImage2( ),
                
                                                 qdc.getMeasuredReal( ),
                                                 qdc.getMeasuredImag( ),
                                                 qdc.getFittedReal( ),
                                                 qdc.getFittedImag( ),
                                                 m_globalModel,
                                                 FQN
                                                );
    
    
    }
    
    /**
     * The name of the file which was fitted.
     * @return 
     */
    public String getSpectrumFileName( )
    {
        return m_spectrumFileName;
    }
    
    // Attributes of the class
    private final JFrame m_parent;
    
    private String m_dicomFileRef;
    private String m_spectrumFileName;
    private String m_spectrumPath;
    private String m_instanceNumberDescription;
    
    private String m_startParameterFileName;
    private SingleVoxelMeasuredSpectralData  m_spectrumToQuantify;
    
    private DicomDirectory m_dicomDir;
    private GlobalModel m_globalModel;
}
