/*
 * File: $Id: RangeDefinitionFrame.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 11.12.2003 2003, 21:52   Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.tdfdfit;

import org.spectrim.lib.DecimalTextField;
import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JLabel;

import java.util.Iterator;
import java.util.LinkedList;

/**
 *
 * Allows the the user to define the minimum and the maximum numeric value.
 *
 * @author  hans
 */

public class RangeDefinitionFrame extends JFrame implements ActionListener
{
    
    /** 
     * Creates a new instance of RangeDefinitionFrame. 
     */
    public RangeDefinitionFrame( String title, JFrame parent, float minimum, float maximum, String format ) 
    {
        super( );
        m_parent = parent;
        setTitle( title );
        m_format = format;
        
        m_currentMinimum = minimum;
        m_currentMaximum = maximum;
        
        m_absoluteMinimum = minimum;
        m_absoluteMaximum = maximum;
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        createMinimumMaximumDefinitionPanel( );
        
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_minimumMaximumDefinitionPanel, BorderLayout.CENTER );
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        setContentPane( m_contentPanel );
        
         // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        
        setVisible( true );
        
    }
    
    /**
     * Create a panel where the user can enter a minimum and a maximum
     * value.
     */ 
    private void createMinimumMaximumDefinitionPanel( )
    {
        m_minimumMaximumDefinitionPanel = new JPanel( );
        m_minimumMaximumDefinitionPanel.setLayout( new BoxLayout( m_minimumMaximumDefinitionPanel, BoxLayout.Y_AXIS ) );
        m_minimumMaximumDefinitionPanel.setBorder( BorderFactory.createEmptyBorder( 20, 10, 20, 10 ) );
        JLabel minLabel = new JLabel( "Minimum" );
        
        minLabel.setPreferredSize( new Dimension( 100, 25 ) );
        minLabel.setMaximumSize( new Dimension( 100, 25 ) );
        minLabel.setMinimumSize( new Dimension( 100, 25 ) );
        
        m_minimumTextField = new DecimalTextField( m_format, m_currentMinimum, 1, 10, 100 );
        
        m_minimumTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_minimumTextField.setMinimumSize( new Dimension( 100, 25 ) );
        
        JLabel maxLabel = new JLabel( "Maximum" );
        maxLabel.setPreferredSize( new Dimension( 100, 25 ) );
        maxLabel.setMaximumSize( new Dimension( 100, 25 ) );
        maxLabel.setMinimumSize( new Dimension( 100, 25 ) );
        
        m_maximumTextField = new DecimalTextField( m_format, m_currentMaximum, 1, 10, 100 );
        
        m_maximumTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_maximumTextField.setMinimumSize( new Dimension( 100, 25 ) );
        
        JPanel minPanel = new JPanel( );
        minPanel.setLayout( new BoxLayout( minPanel, BoxLayout.X_AXIS ) );
        minPanel.add( minLabel );
        minPanel.add( m_minimumTextField );
        
        JPanel maxPanel = new JPanel( );
        maxPanel.setLayout( new BoxLayout( maxPanel,BoxLayout.X_AXIS ) );
        maxPanel.add( maxLabel );
        maxPanel.add( m_maximumTextField );
        
        m_minimumMaximumDefinitionPanel.add( minPanel );
        m_minimumMaximumDefinitionPanel.add( maxPanel );
    }
    
    /**
     * Is called when ever a registered component raises an ActionEvent.
     */
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
             dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            if ( validateEntries( ) == true )
            {
                
                m_currentMinimum = this.m_minimumTextField.getFloatValue( );
                m_currentMaximum = this.m_maximumTextField.getFloatValue( );
                
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
                }
                dispose( );
            }
            else
            {
                this.setVisible( true );
            }
        }
    }
    
    /**
     * Validates the number the user entered; if they are out of range than the 
     * user is notified via a MessageBox.
     */
    private boolean validateEntries( )
    {
        
        if ( this.m_minimumTextField.getFloatValue( ) < m_absoluteMinimum ) 
        {
            MessageBox mBox = new MessageBox( m_parent );
            mBox.showModal( "Minimum value out of range", "Out of Range", 1 );
            return false;
        }

        else if ( this.m_maximumTextField.getFloatValue( ) > m_absoluteMaximum ) 
        {
            MessageBox mBox = new MessageBox( m_parent );
            mBox.showModal( "Maximum value out of range", "Out of Range", 1 );
            return false;
        }
        
        else if ( this.m_maximumTextField.getFloatValue( ) < m_minimumTextField.getFloatValue( ) )
        {
            MessageBox mBox = new MessageBox( m_parent );
            mBox.showModal( "Maximum value larger than minimum value", "Out of Range", 1 );
            return false;
        }
        else if ( this.m_minimumTextField.getFloatValue( ) > m_absoluteMaximum ) 
        {
            MessageBox mBox = new MessageBox( m_parent );
            mBox.showModal( "Minimum value out of range", "Out of Range", 1 );
            return false;
        }
        else if ( this.m_maximumTextField.getFloatValue( ) < m_absoluteMinimum ) 
        {
            MessageBox mBox = new MessageBox( m_parent );
            mBox.showModal( "Maximum value out of range", "Out of Range", 1 );
            return false;
        }
        return true;
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens within this object.
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns the current maximum.
     */
    public float getCurrentMaximum( )
    {
        return m_currentMaximum;
    }

    /**
     * Returns the current maximum.
     */
    public float getCurrentMinimum( )
    {
        return m_currentMinimum;
    }

    // Attributes of the class
    private JFrame m_parent;
    private JPanel m_contentPanel;
    private JPanel m_selectionPanel;
    private JPanel m_minimumMaximumDefinitionPanel;
    
    private JButton m_applyButton;
    private JButton m_cancelButton;
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // Possible Events
    public final static int CANCEL_EVENT = 0;
    public final static int ACCEPT_EVENT = 1;
    
    DecimalTextField m_minimumTextField;
    DecimalTextField m_maximumTextField;
    String m_format;
    
    float m_currentMinimum;
    float m_currentMaximum;
    float m_absoluteMinimum;
    float m_absoluteMaximum;
    
    // For proper serialization
    private static final long serialVersionUID = 9982232191119213L;
    
}
