/*
 * File: $Id: SingleVoxelFirstSamplePointMultiplicatorControl.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 26.08.2005, 8:48  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2005 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * SingleVoxelFirstSamplePointMultiplicatorControl.java
 *
 * Created on August 26, 2005, 8:48 AM
 */

package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import java.util.LinkedList;
import java.util.Iterator;

import org.spectrim.lib.DecimalTextField;

/**
 * Allows for the setting of the multiplication factor of the first time
 * domain sample point. As a result spectral offset can is removed
 *
 * @author  hans
 */
public class SingleVoxelFirstSamplePointMultiplicatorControl extends JFrame implements ActionListener
{

    /** 
     * Creates a new instance of SingleVoxelFirstSamplePointMultiplicatorControl. 
     * @param title
     * @param parent
     * @param displayMode
     */
    public SingleVoxelFirstSamplePointMultiplicatorControl( String title, 
            JFrame parent, 
            int displayMode )
    {
        super( );
        
        m_displayMode = displayMode;
        
        m_parent = parent;
        setTitle( title );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_controlPanel = new JPanel( );
        m_controlPanel.setLayout( new BoxLayout( m_controlPanel, BoxLayout.X_AXIS ) );
        m_controlPanel.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );
        m_controlPanel.setPreferredSize( new Dimension( 365, 45 ) );
        
        // Filling control panel goes here
        m_decimalTextFieldLabel = new JLabel( "Multiplication factor " );
        m_controlPanel.add( m_decimalTextFieldLabel );
        m_decimalTextField = new DecimalTextField( "###.##", 0.5f, 0.1f, 1.0f, 10.0f );
        
        m_decimalTextField.addActionListener( this );
        
        m_controlPanel.add( m_decimalTextField );
        
        m_contentPanel.add( m_controlPanel, BorderLayout.NORTH );       
        
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the indicated value to the dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the indicated multiplication factor to the dataset" );
        
        if ( m_displayMode == MULTI_RESPONSE_DISPLAY )
        {
            createInstanceSpecificPanel( );
            m_contentPanel.add( m_performActionToCurrentOnlyPanel, BorderLayout.CENTER );
        }
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        setContentPane( m_contentPanel );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        
        setVisible( true );
    }
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
             dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
             dispose( );
        }
        
        if ( sender == m_decimalTextField )
        {
            if ( e.getID( ) == m_decimalTextField.VALUE_CHANGED_EVENT )
            {
                Iterator it = m_listeners.iterator( );
                
                while( it.hasNext( ) )
                {
                    m_shiftValue = m_decimalTextField.getFloatValue( );
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CHANGE_EVENT, "" ) );
                }
            }
        }
    }
    
    /**
     * Returns the current spectral shift value. 
     * @return 
     */
    public float getMultiplicationFactor( )
    {
        return m_shiftValue;
    }
    
    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Creates the panel that allows the user to perform the operation to 
     * the selected instance only.
     */
    public void createInstanceSpecificPanel( )
    {
        m_performActionToCurrentOnlyPanel =  new JPanel( );
        
        m_performActionToCurrentOnlyPanel.setLayout( new BoxLayout( m_performActionToCurrentOnlyPanel, BoxLayout.X_AXIS ) );
        
        m_applyToSelectedInstanceOnly = new JRadioButton( "Selected instance only" );
        m_applyToSelectedInstanceOnly.setSelected( true );
        
        m_applyToAllInstances = new JRadioButton( "All instances" );
        
        m_applyToButtonGroup.add( m_applyToAllInstances );
        m_applyToButtonGroup.add( m_applyToSelectedInstanceOnly );
        
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
        m_performActionToCurrentOnlyPanel.add( m_applyToSelectedInstanceOnly );
        m_applyToSelectedInstanceOnly.setToolTipText( "Applies multiplication only to the selected instance" );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
        m_performActionToCurrentOnlyPanel.add( m_applyToAllInstances );
        m_applyToAllInstances.setToolTipText( "Applies the multiplication to all instances of the spectrum series" );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
    }
    
    
    
    // Attributes of control
    private final JFrame m_parent;
    private final JPanel m_contentPanel;
    private final JPanel m_controlPanel;
    private final JPanel m_selectionPanel;
    
    private final JButton m_applyButton;
    private final JButton m_cancelButton;
    
    private final JLabel m_decimalTextFieldLabel;
    private final DecimalTextField m_decimalTextField;
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( ); 
    
    private JPanel m_performActionToCurrentOnlyPanel;
    private JRadioButton m_applyToSelectedInstanceOnly;
    private JRadioButton m_applyToAllInstances;
    
    private final ButtonGroup m_applyToButtonGroup = new ButtonGroup( );
    
    // Possible Events
    public final static int CANCEL_EVENT = 0;
    public final static int ACCEPT_EVENT = 1;
    public final static int CHANGE_EVENT = 2;
    
    private float m_shiftValue = 0.0f;
    
    private int m_displayMode = SINGLE_RESPONSE_DISPLAY;
    
    public final static int SINGLE_RESPONSE_DISPLAY = 0;
    public final static int MULTI_RESPONSE_DISPLAY = 1;
    // For proper serialization
    private static final long serialVersionUID = 1487495898889327L;
    
}
