/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.FileInfo;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.StatusPanel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import org.spectrim.data.CSIData;
import org.spectrim.patientdatabase.DataBaseConnectionDefinerFrame;

import org.spectrim.patientdatabase.MRPatientSpectrumDataBase;
import org.spectrim.spectra.Spectra;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.TDFDFIT_MODELS_ROOT_PATH;
import org.spectrim.tdfdfit.quantitativeresults.QuantitativeSVSContextDataContainer;
import org.spectrim.tdfdfit.waterreferencemodelling.TwoCompartmentWaterReferenceModelFittingFrame;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.lib.Credits;
import org.spectrim.lib.DicomStringReformatter;
import org.spectrim.main.ClinicalViewer;
import org.spectrim.model.GlobalModel;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME;
import org.spectrim.tdfdfit.mmbaseline.GeneralBaselineSettingsFrame;
import org.spectrim.tdfdfit.mmbaseline.MacroMolecularBaseline;
import org.spectrim.toolbars.TDFDFitFileMenuToolbar;
import org.spectrim.toolbars.TDFDFitModelToolbar;
import org.spectrim.toolbars.TDFDFitSignalPreprocessingToolbar;

/**
 * 
 * This control lets you define and optimize a prior knowledge model 
 * for usage with TDFDFit.
 * 
 * Created: 24.01.2017
 * 
 * @author Johannes Slotboom <johannes.slotboom@insel.ch>
 */
public class TDFDFitControlFrame extends JFrame implements ActionListener 
{
    
    // attributes of this frame
    private JMenuBar    m_menuBar;
    private JMenu       m_TDFDFitModelFileMenu;
    private JMenuItem   m_newTDFDFitModel;
    private JMenuItem   m_openTDFDFitModel;
    private JMenuItem   m_openTDFDFitModelKeepResponse;
    
    private JMenuItem   m_saveTDFDFitModel;
    private JMenuItem   m_closeTDFDFitModel;
    
    private JMenuItem   m_exportPrototypeSignalAsMRUI;
    private JMenuItem   m_openDicomDir;
    private JMenuItem   m_quitApp;
    
    private JMenu       m_analysisMenu;
    private JMenuItem   m_loadFittedDataMenuItem;
    private JMenu       m_propertiesMenu;
    private JMenuItem   m_dataBaseProperties;
    private JMenuItem   m_MRSignalPropertiesMenuItem;
    private JMenuItem   m_tdfdfitDefaultsMenuItem;

    private JMenu       m_helpMenu;
    private JMenuItem   m_aboutFitGuru;
    private JMenuItem   m_addParPatFromFileMenuItem;
    private JMenu       m_addNumericPatternsFromFiles;
    private JMenuItem   m_exportAllNumericPatternsAsMruiFiles;
    
    private JMenu       m_TDFDFitModelMenu;
    
    private JMenuItem   m_defineDefaultTDFDFitModelMenuItem;
    private JMenuItem   m_defineDefaultTDFDFitWaterRemovalModelMenuItem;
    
    private JMenuItem   m_addNumPatFrom_NPF_FileMenuItem;
    private JMenuItem   m_addNumPatFromMRUIFileMenuItem;
    private JMenuItem   m_addNumPatFromBinaryFileMenuItem;
    
    private JMenuItem   m_addMacromolecularBaselineMenuItem;
    private JMenuItem   m_addGeneralBaselineMenuItem;
    
    private JMenuItem   m_addMetaboliteFromFileMenuItem;
    private JMenuItem   m_waterReferenceModellingMenuItem;
    private JMenuItem   m_defaultParameterRangesMenuItem;
    private JMenuItem   m_generateRandomlyVaryingTDFDFitModelSignalsMenuItem;
     
//    private JMenuItem   m_defaultTDFDFitRuntimeParametersMenuItem;
    private JMenuItem   m_launchQuantitativeResultsViewer;
    
    private MRSignalPropertiesPanel m_MRSignalPropertiesPanel;
    private MRSpectrumModellerPanel m_MRSpectrumModellerPanel;
    
    // The one and only reference to the current .tfm model file!
    private File                    m_TDFDfitPrototypeModelFileReference;
    
    private MRSignalProperties      m_MRSignalProperties;
    private MRFitStrategy           m_fitStrategy;
     
    private SingleVoxelMeasuredSpectralData m_singleVoxelMeasuredData;
    private SingleVoxelMeasuredSpectralData m_singleVoxelMeasuredDataLoaded;
    
    private GeneralBaselineSettingsFrame m_BaselineDefinitionFrame;
    
    // Object to store the data to be fitted into.
    private SingleVoxelMeasuredSpectralData m_singleVoxelClinicalData;
    
    private boolean                         m_fitProjectFileSelected = false;
    private String                          m_currentProjectPath;
    private boolean                         m_dicomDirectorySet = false;
    
    // Each spectrum can have a baselinemodel which can be subtracted if necessary 
    private final OrderedBaseLinePointsDataModel m_baseLineModel = OrderedBaseLinePointsDataModel.getInstance( );
    
    private MRPatientSpectrumDataBase       m_mrSpectrumDataBase;
    private final TDFDFittingDefaults       m_fittingDefaults;
    private final StatusPanel               m_statusPanel;
    private final JPanel                    m_overallPanel;
    private DataBaseConnectionDefinerFrame  m_dataBaseConnectionDefinerFrame;
    private int m_samplePoints = 1024;
    
    private TwoCompartmentWaterReferenceModelFittingFrame m_waterReferenceFrame = null;
    private String m_signalPrototypeModelSpectrumFileReference;
    private String m_signalPatientSpectrumFileReference;
    
    private DicomDirectory m_dicomDir;
    
    String m_defaultTDFDFitWaterRemovalModelFileName = null;
    String m_defaultTDFDFitModelFileName = null;
    
    private final JPanel m_toolBarCollectionPanel;
    
    private final TDFDFitFileMenuToolbar m_basicFileToolbar;
    private final TDFDFitModelToolbar    m_TDFDFitModelToolbar;
    private TDFDFitSignalPreprocessingToolbar m_TDFDFitSignalPreprocessingToolbar;
    
    GlobalModel m_globalModel;
    
    boolean m_isModelChanged = false;
    
     // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    
    
    /**
     * Default constructor: Initializes all necessary components correctly and
     * displays the control.
     * @param viewer
     * @param model
     * @param loadMetaboliteTDFDFitModel
     */
    public TDFDFitControlFrame( ClinicalViewer viewer, GlobalModel model, boolean loadMetaboliteTDFDFitModel )
    {
        setTitle( ".::TDFDFit - Control Frame::." );
        m_overallPanel = new JPanel( );
        m_overallPanel.setLayout( new BorderLayout( ) );
        m_globalModel = model;
        m_singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( );
        
        // create the panel on which the toolbars are located
        m_toolBarCollectionPanel = new JPanel( );
        m_toolBarCollectionPanel.setLayout( new FlowLayout( FlowLayout.LEFT ) );
        
        m_basicFileToolbar = new TDFDFitFileMenuToolbar( ); 
        m_basicFileToolbar.addActionListener( this );
        
        m_TDFDFitModelToolbar = new TDFDFitModelToolbar( );
        m_TDFDFitModelToolbar.addActionListener( this );
        
        m_TDFDFitSignalPreprocessingToolbar = new TDFDFitSignalPreprocessingToolbar( );
        m_TDFDFitSignalPreprocessingToolbar.addActionListener(this);
                
                
        m_toolBarCollectionPanel.add( m_basicFileToolbar );
        m_toolBarCollectionPanel.add( m_TDFDFitModelToolbar );
        m_toolBarCollectionPanel.add( m_TDFDFitSignalPreprocessingToolbar );
        
        // Get a reference on the properties class
        m_fittingDefaults = TDFDFittingDefaults.getInstance( );
        m_fittingDefaults.load( );
        
        m_overallPanel.add( m_toolBarCollectionPanel, BorderLayout.NORTH );
        
        
        m_MRSpectrumModellerPanel = new MRSpectrumModellerPanel( this, m_dicomDir, false, m_globalModel );
        
        m_MRSpectrumModellerPanel.listenToChangesInTFDFFittingDefaults( );
        
        m_statusPanel = new StatusPanel( this );
        
        m_MRSpectrumModellerPanel.setStatusPanel( m_statusPanel );       
        
        m_overallPanel.add( m_MRSpectrumModellerPanel, BorderLayout.CENTER );
        m_overallPanel.add( m_statusPanel, BorderLayout.SOUTH );
        
        this.getContentPane( ).add( m_overallPanel );
        
        createMenuBar( );
//        pack( );
//        setExtendedState( JFrame.MAXIMIZED_BOTH );
//        setVisible( true );
        
        m_defaultTDFDFitModelFileName = (String)m_fittingDefaults.get( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
        
        if ( ( m_defaultTDFDFitModelFileName != null ) && ( ( new File( m_defaultTDFDFitModelFileName) ).exists( ) ) )
        {
            m_TDFDfitPrototypeModelFileReference = new File( m_defaultTDFDFitModelFileName );
            if ( loadTDFDFitPrototypeModel( false ) == false )
            {
                JOptionPane.showMessageDialog( rootPane, "Could not load the default model : \n" + m_defaultTDFDFitModelFileName );
            }
        }
        
        if ( loadMetaboliteTDFDFitModel == true )
        {
            TDFDFittingDefaults fdef =  TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            String modelFile = fdef.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
            String modelPath = fdef.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
            
            m_TDFDfitPrototypeModelFileReference = new File( modelPath + System.getProperty( "file.separator" )+ modelFile );
            this.loadTDFDFitPrototypeModelSpectrumFile( );
        }
        
        
        addActionListener( viewer );
        
        pack( );
        setExtendedState( JFrame.MAXIMIZED_BOTH );
        setVisible( true );
        
    }
    
    /**
     * Sets the attribute m_TDFDFitPrototypeModelFileName.
     * @param pathAndFilename 
     */
    public void setTDFDFitPrototypeModelFilereference( String pathAndFilename )
    {
        m_TDFDfitPrototypeModelFileReference = new File( pathAndFilename );
    }
    
    /**
     * Sets the attribute m_TDFDFitPrototypeModelFileName. 
     * @return  
     */
    public String getTDFDFitPrototypeModelFilereference(  )
    {
        if ( m_TDFDfitPrototypeModelFileReference != null )
        {
            return m_TDFDfitPrototypeModelFileReference.getAbsolutePath( );
        }
        else
        {
            return null;
        }
    }
    
    
    /**
     * Necessary to get access to GlobalModel values ...
     * @param model 
     */
    public void setGlobalModel( GlobalModel model )
    {
        m_globalModel = model;
    }
    
    
    /**
     * Quits the TDFDFitControlFrame frame, and performs the clearing related 
     * data.
     */
    private void quitTDFDFitControlFrame( )
    {
        
        if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getNumberOfSpectralComponentsToFit( ) == 0 )
        
        
        if ( m_isModelChanged == true )
        {
            if ( JOptionPane.showConfirmDialog( null, 
                                                "Save changes in TDFDFit-model before closing?", "Are you sure?", 
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE ) == JOptionPane.YES_OPTION )
            {
                if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getNumberOfSpectralComponentsToFit( ) > 0 )
                {
                    saveTDFDFitModel( );
                }
                else
                {
                    if ( JOptionPane.showConfirmDialog( null, 
                                                "Number of parameters to fit in model is zero?", "Are you sure you wat to save empty model?", 
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE ) == JOptionPane.YES_OPTION )
                    {
                        saveTDFDFitModel( );
                    }
                }
            }
            
            
            if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ) != null )
            {
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).clearJTree( );
            }
            MRSignalProperties.getInstance( ).resetMRSignalProperties( );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).resetModel( );
            dispose( );
        }
        else
        {
            if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ) != null )
            {
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).clearJTree( );
            }
            MRSignalProperties.getInstance( ).resetMRSignalProperties( );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).resetModel( );
            dispose( );
        }

    }
            
    /**
     * Constructor: initializes all necessary components correctly and
     * displays the control, but also loads the prototype spectroscopy dataset.
     * @param clinicalSpectrumData
     * @param ddir
     * @param model
     */
    public TDFDFitControlFrame( ClinicalViewer viewer,
                                CSIData clinicalSpectrumData, 
                                DicomDirectory ddir,
                                GlobalModel model )
    {
        setTitle( ".::TDFDFit - Control Frame::." );
        
        m_dicomDir = ddir;
        
        m_overallPanel = new JPanel( );
        m_overallPanel.setLayout( new BorderLayout( ) );
        m_globalModel = model;
        
        m_singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( );
        
        // create the panel on which the toolbars are located
        m_toolBarCollectionPanel = new JPanel( );
        m_toolBarCollectionPanel.setLayout( new BoxLayout( m_toolBarCollectionPanel, BoxLayout.X_AXIS ) );
        
        m_basicFileToolbar = new TDFDFitFileMenuToolbar( ); 
        m_basicFileToolbar.addActionListener( this );
        
        m_TDFDFitModelToolbar = new TDFDFitModelToolbar( );
        m_TDFDFitModelToolbar.addActionListener( this );
        
        m_TDFDFitSignalPreprocessingToolbar = new TDFDFitSignalPreprocessingToolbar( );
        m_TDFDFitSignalPreprocessingToolbar.addActionListener(this);
        
        m_toolBarCollectionPanel.add( m_basicFileToolbar );
        m_toolBarCollectionPanel.add( m_TDFDFitModelToolbar );
        m_toolBarCollectionPanel.add( m_TDFDFitSignalPreprocessingToolbar );
        
        m_overallPanel.add( m_toolBarCollectionPanel, BorderLayout.NORTH );
        
        
        // Get a reference on the properties class
        m_fittingDefaults = TDFDFittingDefaults.getInstance( );
        m_fittingDefaults.load( );
        
        m_MRSpectrumModellerPanel = new MRSpectrumModellerPanel( this, m_dicomDir, 
                                                                 true, m_globalModel );
        
        m_MRSpectrumModellerPanel.listenToChangesInTFDFFittingDefaults( );
        
        m_statusPanel = new StatusPanel( this );
        
        m_MRSpectrumModellerPanel.setStatusPanel( m_statusPanel );       
        
        m_overallPanel.add( m_MRSpectrumModellerPanel, BorderLayout.CENTER );
        m_overallPanel.add( m_statusPanel, BorderLayout.SOUTH );
        
        this.getContentPane( ).add( m_overallPanel );
        
        this.addWindowListener( new java.awt.event.WindowAdapter( ) 
        {
            @Override
            public void windowClosing( java.awt.event.WindowEvent windowEvent ) 
            {
                quitTDFDFitControlFrame( );
            }
        });
        
        // wrong! m_signalPrototypeModelSpectrumFileReference = clinicalSpectrumData.getFile( );
        
        TDFDFittingDefaults fittingDefaults = TDFDFittingDefaults.getInstance( );
        fittingDefaults.load( );
        
        String defaultModelPath = fittingDefaults.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH  );
        String defaultModelName = fittingDefaults.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        
        if ( defaultModelName.endsWith( ".tfm" ) == false )
        {
            defaultModelName = defaultModelName + ".tfm";
        }
        
        m_TDFDfitPrototypeModelFileReference = new File( defaultModelPath + System.getProperty( "file.separator" ) + defaultModelName );
        
        m_signalPatientSpectrumFileReference = clinicalSpectrumData.getFile( );
        
        clinicalSpectrumData.getSpectra( );
        createMenuBar( );
//        pack( );
//        setVisible( true );
        
        // In this case we load the default model
        // m_defaultTDFDFitModelFileName = (String)m_fittingDefaults.get(TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );        
        if ( ( m_TDFDfitPrototypeModelFileReference != null ) && ( m_TDFDfitPrototypeModelFileReference.exists( ) ) )
        {
            // m_TDFDfitPrototypeModelFileReference = new File( m_defaultTDFDFitModelFileName );
            if ( loadTDFDFitPrototypeModel( false ) == false )
            {
                JOptionPane.showMessageDialog( rootPane, "Could not load the default model \n" + m_defaultTDFDFitModelFileName );
            }
        }
        
        // Load the clinical case signal ...
        loadClinicalSVSCaseModelData( clinicalSpectrumData );
        
        addActionListener( viewer );
        
        pack( );
        setVisible( true );
    }        
            
    /**
     * Returns the Dicom file reference to the patient dataset.
     * @return 
     */ 
    public String getSignalPatientSpectrumFileReference( )
    {
        return m_signalPatientSpectrumFileReference;
    }
    
    /**
     * Constructor: initializes all necessary components correctly and
     * displays the control, but also loads the prototype spectroscopy dataset.
     * @param model
     * @param signalPrototypeModelSpectrumFileReference
     * @param ddir
     */
    public TDFDFitControlFrame( ClinicalViewer viewer,
                                GlobalModel model, 
                                String signalPrototypeModelSpectrumFileReference,
                                DicomDirectory ddir,
                                boolean loadMetaboliteTDFDFitModel
                              )
    {
        
        setTitle( ".::TDFDFit - Control Frame::." );
        
        m_dicomDir = ddir;
        
        m_globalModel = model;
        
        m_MRSignalProperties = MRSignalProperties.getInstance( );
        m_MRSignalProperties.setResonanceFrequency( new Double( model.getResonanceFrequency( ) ).longValue( ) );
        m_MRSignalProperties.setFrequencyScale( new Float( model.getBandwidth( ) / model.getVectorSize( ) ) );    
        m_MRSignalProperties.setPPMValueInCenterOfSpectrum( 4.7f ); // assume 1H
        m_MRSignalProperties.setNumberOfSamplePoints( model.getVectorSize( ) );
        
        m_samplePoints =  model.getVectorSize( );
        
        m_overallPanel = new JPanel( );
        m_overallPanel.setLayout( new BorderLayout( ) );
        
        m_singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( );
        m_singleVoxelMeasuredData.setAcquisitionFrequency( (float)model.getBandwidth( ) );
        
        // create the panel on which the toolbars are located
        m_toolBarCollectionPanel = new JPanel( );
        m_toolBarCollectionPanel.setLayout( new BoxLayout( m_toolBarCollectionPanel, BoxLayout.X_AXIS ) );
        
        
        m_basicFileToolbar = new TDFDFitFileMenuToolbar( ); 
        m_basicFileToolbar.addActionListener( this );
        
        m_TDFDFitModelToolbar = new TDFDFitModelToolbar( );
        m_TDFDFitModelToolbar.addActionListener( this );
        
        m_TDFDFitSignalPreprocessingToolbar = new TDFDFitSignalPreprocessingToolbar( );
        m_TDFDFitSignalPreprocessingToolbar.addActionListener(this);
        
        
        m_toolBarCollectionPanel.add( m_basicFileToolbar );
        m_toolBarCollectionPanel.add( m_TDFDFitModelToolbar );
        m_toolBarCollectionPanel.add( m_TDFDFitSignalPreprocessingToolbar );
        
        m_overallPanel.add( m_toolBarCollectionPanel, BorderLayout.NORTH );
        
        // Get a reference on the properties class
        m_fittingDefaults = TDFDFittingDefaults.getInstance( );
        m_fittingDefaults.load( );
        
        m_MRSpectrumModellerPanel = new MRSpectrumModellerPanel( this, m_dicomDir, 
                                                                 false, m_globalModel );

        m_MRSpectrumModellerPanel.listenToChangesInTFDFFittingDefaults( );
        
        m_statusPanel = new StatusPanel( this );
        
        m_MRSpectrumModellerPanel.setStatusPanel( m_statusPanel );       
        
        m_overallPanel.add( m_MRSpectrumModellerPanel, BorderLayout.CENTER );
        m_overallPanel.add( m_statusPanel, BorderLayout.SOUTH );
         
        this.getContentPane( ).add( m_overallPanel );
        m_signalPrototypeModelSpectrumFileReference = signalPrototypeModelSpectrumFileReference;
        // Load the prototype model signal ...
        // old style: loadSpectroscopicPrototypeModelData( signalPrototypeModelSpectrumFileReference );
        // loadSpectrImSpectroscopicPrototypeModelData( signalPrototypeModelSpectrumFileReference );
        loadSpectrImSpectroscopicPrototypeModelData( model.getCSIData( ) );
        createMenuBar( );
                
        this.addWindowListener( new java.awt.event.WindowAdapter( ) 
        {
            @Override
            public void windowClosing( java.awt.event.WindowEvent windowEvent ) 
            {
                quitTDFDFitControlFrame( );
            }
        } );
        
//        pack( );
//        setVisible( true );
        if ( loadMetaboliteTDFDFitModel == false )
        {
            newTDFDFitModel( );
        }
        else
        {
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            String modelFile = fdef.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
            String modelPath = fdef.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
            
            m_TDFDfitPrototypeModelFileReference = new File( modelPath + System.getProperty( "file.separator" )+ modelFile );
            this.loadTDFDFitPrototypeModelSpectrumFile( );
        }
        
        // this.addActionListener( );
        addActionListener( viewer );
        
        pack( );
        setLocationRelativeTo(null);
        setVisible( true );

    }
    
    /**
     * Creates a menu bar for the TDFDFitControlFrame.
     */
    private void createMenuBar( )
    {
        
        m_menuBar = new JMenuBar( );
        
        // Project menu
        m_TDFDFitModelFileMenu = new JMenu( "File" );
        m_TDFDFitModelFileMenu.setMnemonic( KeyEvent.VK_F );
        
        
        m_newTDFDFitModel = new JMenuItem( "New TDFDFit model" );
        m_newTDFDFitModel.addActionListener( this );
        m_TDFDFitModelFileMenu.add( m_newTDFDFitModel );
        
        m_openTDFDFitModel = new JMenuItem( "Open TDFDFit model" );
        m_openTDFDFitModel.addActionListener( this );
        m_TDFDFitModelFileMenu.add( m_openTDFDFitModel );
        
        m_openTDFDFitModelKeepResponse = new JMenuItem( "Open TDFDFit model but keep current response" );
        m_openTDFDFitModelKeepResponse.addActionListener( this );
        m_TDFDFitModelFileMenu.add( m_openTDFDFitModelKeepResponse );
        
        m_saveTDFDFitModel = new JMenuItem( "Save TDFDFit model" );
        m_saveTDFDFitModel.addActionListener( this );
        m_saveTDFDFitModel.setEnabled( false );
        m_TDFDFitModelFileMenu.add( m_saveTDFDFitModel );
        
        m_closeTDFDFitModel = new JMenuItem( "Close TDFDFit model" );
        m_closeTDFDFitModel.addActionListener( this );
        m_TDFDFitModelFileMenu.add( m_closeTDFDFitModel );
         
        m_TDFDFitModelFileMenu.addSeparator( );
        
        m_openDicomDir = new JMenuItem( "Mount Dicom Directory" );
        m_openDicomDir.addActionListener( this );
        
        m_exportPrototypeSignalAsMRUI = new JMenuItem( "Export prototype response as jMRUI-file" );
        m_exportPrototypeSignalAsMRUI.addActionListener( this );
        
        m_quitApp = new JMenuItem( "Quit" );
        m_quitApp.addActionListener( this );
        m_TDFDFitModelFileMenu.add( m_exportPrototypeSignalAsMRUI );
        m_TDFDFitModelFileMenu.addSeparator( );
        
        m_TDFDFitModelFileMenu.add( m_openDicomDir );
        m_TDFDFitModelFileMenu.add( m_quitApp );
        
        m_TDFDFitModelMenu = new JMenu( "TDFDFit-models" );
        m_TDFDFitModelMenu.setMnemonic( KeyEvent.VK_T );
        
        m_defineDefaultTDFDFitModelMenuItem = new JMenuItem( "Set default TDFDFit model .." );
        m_defineDefaultTDFDFitModelMenuItem.setEnabled( true );
        m_defineDefaultTDFDFitModelMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_defineDefaultTDFDFitModelMenuItem );
        
        m_defineDefaultTDFDFitWaterRemovalModelMenuItem = new JMenuItem( "Set default TDFDFit water model .." );
        m_defineDefaultTDFDFitWaterRemovalModelMenuItem.setEnabled( true );
        m_defineDefaultTDFDFitWaterRemovalModelMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_defineDefaultTDFDFitWaterRemovalModelMenuItem );
         
        m_TDFDFitModelMenu.addSeparator( );
        
        m_addParPatFromFileMenuItem = new JMenuItem( "Add parametric pattern from .ppf file .." );
        m_addParPatFromFileMenuItem.setEnabled( false );
        m_addParPatFromFileMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_addParPatFromFileMenuItem );
        
        m_addNumericPatternsFromFiles = new JMenu( "Add numeric pattern from file" );
        
        m_addNumPatFrom_NPF_FileMenuItem = new JMenuItem( "Add numeric pattern from .npf file .." );
        m_addNumPatFrom_NPF_FileMenuItem.setEnabled( false );
        m_addNumPatFrom_NPF_FileMenuItem.addActionListener( this );
        m_addNumericPatternsFromFiles.add( m_addNumPatFrom_NPF_FileMenuItem );
        
        m_addNumPatFromMRUIFileMenuItem = new JMenuItem( "Add numeric pattern from .mrui file .." );
        m_addNumPatFromMRUIFileMenuItem.setEnabled( false );
        m_addNumPatFromMRUIFileMenuItem.addActionListener( this );
        m_addNumericPatternsFromFiles.add( m_addNumPatFromMRUIFileMenuItem );
       
        m_addNumPatFromBinaryFileMenuItem = new JMenuItem( "Add numeric pattern from .bin file .." );
        m_addNumPatFromBinaryFileMenuItem.setEnabled( false );
        m_addNumPatFromBinaryFileMenuItem.addActionListener( this );
        m_addNumericPatternsFromFiles.add( m_addNumPatFromBinaryFileMenuItem );
        
        m_exportAllNumericPatternsAsMruiFiles = new JMenuItem( "Export all numeric patterns in .mrui format .." );
        m_exportAllNumericPatternsAsMruiFiles.setEnabled( true );
        m_exportAllNumericPatternsAsMruiFiles.addActionListener( this );
        m_addNumericPatternsFromFiles.add( m_exportAllNumericPatternsAsMruiFiles );
        
        // Add the submenu ...
        m_TDFDFitModelMenu.add( m_addNumericPatternsFromFiles );
        
        m_addMetaboliteFromFileMenuItem = new JMenuItem( "Add metabolite pattern from .mpf file .." );
        m_addMetaboliteFromFileMenuItem.setEnabled( false );
        m_addMetaboliteFromFileMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_addMetaboliteFromFileMenuItem );
        m_TDFDFitModelMenu.addSeparator( );
        
        m_addMacromolecularBaselineMenuItem = new JMenuItem( "Add macromolecular baseline components .." );
        m_addMacromolecularBaselineMenuItem.setEnabled( false );
        m_addMacromolecularBaselineMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_addMacromolecularBaselineMenuItem );
        
        m_addGeneralBaselineMenuItem = new JMenuItem( "Add general baseline components .." );
        m_addGeneralBaselineMenuItem.setEnabled( true );
        m_addGeneralBaselineMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_addGeneralBaselineMenuItem );
        
        m_TDFDFitModelMenu.addSeparator( );
        m_waterReferenceModellingMenuItem = new JMenuItem( "Water reference modelling .." );
        m_waterReferenceModellingMenuItem.setEnabled( true );
        m_waterReferenceModellingMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_waterReferenceModellingMenuItem );
        
        m_launchQuantitativeResultsViewer = new JMenuItem( "Launch quantative viewer .." );
        m_launchQuantitativeResultsViewer.setEnabled( true );
        m_launchQuantitativeResultsViewer.addActionListener( this );
        m_TDFDFitModelMenu.add( m_launchQuantitativeResultsViewer );
        
        m_TDFDFitModelMenu.addSeparator( );
        
        m_defaultParameterRangesMenuItem = new JMenuItem( "Default parameter ranges .." );
        m_defaultParameterRangesMenuItem.setEnabled( true );
        m_defaultParameterRangesMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_defaultParameterRangesMenuItem );
        
        m_generateRandomlyVaryingTDFDFitModelSignalsMenuItem = new JMenuItem( "Generate Randomly Varying TDFDFit model signals .." );
        m_generateRandomlyVaryingTDFDFitModelSignalsMenuItem.setEnabled( true );
        m_generateRandomlyVaryingTDFDFitModelSignalsMenuItem.addActionListener( this );
        m_TDFDFitModelMenu.add( m_generateRandomlyVaryingTDFDFitModelSignalsMenuItem );
        
//        m_defaultTDFDFitRuntimeParametersMenuItem = new JMenuItem( "Default TDFDFit runtime parameters .." );
//        m_defaultTDFDFitRuntimeParametersMenuItem.setEnabled( true );
//        m_defaultTDFDFitRuntimeParametersMenuItem.addActionListener( this );
//        m_TDFDFitModelMenu.add( m_defaultTDFDFitRuntimeParametersMenuItem );
        
        // Analysis Menu
        m_analysisMenu = new JMenu( "Analysis" );
        m_analysisMenu.setMnemonic( KeyEvent.VK_A );
        
        m_loadFittedDataMenuItem = new JMenuItem( "Load fitted data .." );
        m_loadFittedDataMenuItem.addActionListener( this );
        m_analysisMenu.add( m_loadFittedDataMenuItem );
        
        // Properties menu
        m_propertiesMenu = new JMenu( "Properties" );
        m_propertiesMenu.setMnemonic( KeyEvent.VK_P );
        
        // Item allowing database connection settings
        m_dataBaseProperties = new JMenuItem( "Database settings ..");
        m_dataBaseProperties.addActionListener( this );
        m_propertiesMenu.add( m_dataBaseProperties );
        
        // Item allowing the change of the MRSignalProperties
        m_MRSignalPropertiesMenuItem = new JMenuItem( "MR-signal properties ..");
        m_MRSignalPropertiesMenuItem.addActionListener( this );
        m_propertiesMenu.add(m_MRSignalPropertiesMenuItem );

        m_tdfdfitDefaultsMenuItem = new JMenuItem( "Default TDFDFit runtime parameters .." );
        m_tdfdfitDefaultsMenuItem.addActionListener( this );
        m_propertiesMenu.add( m_tdfdfitDefaultsMenuItem );

       // Properties menu 
        // Help menu
        m_helpMenu = new JMenu( "Help" );
        m_helpMenu.setMnemonic( KeyEvent.VK_H );
        
        m_aboutFitGuru = new JMenuItem( "About TDFDFit" );
        m_aboutFitGuru.addActionListener( this );
        m_helpMenu.add( m_aboutFitGuru );
        
        m_menuBar.add( m_TDFDFitModelFileMenu );
        m_menuBar.add( m_TDFDFitModelMenu );
        m_menuBar.add( m_analysisMenu );
        m_menuBar.add( m_propertiesMenu );
        m_menuBar.add( m_helpMenu );
        
        setJMenuBar( m_menuBar );        
    }
    
    
    /**
     * Is called whenever an ActionEvent is detected.
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        if ( sender == this.m_exportAllNumericPatternsAsMruiFiles )
        {
            jMenuItemExportSelectedSpectraTojMRUIFormatActionPerformed( );
        }
        
        // Handle the toolbar events
        if ( sender == m_generateRandomlyVaryingTDFDFitModelSignalsMenuItem )
        {
            TDFDFitRandomModelSignalGeneratorFrame signalGenerator = new TDFDFitRandomModelSignalGeneratorFrame( this, m_globalModel );
            
        }
        
        if ( sender == m_TDFDFitSignalPreprocessingToolbar )
        {
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_SVD_WATER_REMOVAL_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchSVDInteractiveWaterProcessing( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_HLSVD_WATER_REMOVAL_ID )
            {
                 m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchSVDInteractiveWaterProcessing( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_APODIZATION_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchApodization( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_FIRST_TD_POINT_MULTIPLICATION_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchFirstTDPointMultiplication( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_OFFSET_CORRECTION_FD_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchFDOffsetCorrection( );
            }
            
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_ZERO_FILLING_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchZeroFilling( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_MANUAL_PHASING_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchManualPhasing( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.APPLY_MANUAL_FREQ_SHIFT_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).launchManualFrequencyShift( );
            }
            
            if ( e.getID( ) == TDFDFitSignalPreprocessingToolbar.DEFINE_BASELINE_NODES_SETTINGS_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).defineBaseLineNodes( );
            }
            
        }
        
        if ( sender == m_TDFDFitModelToolbar )
        {
            
            if ( e.getID( ) == TDFDFitModelToolbar.SET_DEFAULT_METABOLITE_MODEL_ID )
            {
                defineDefaultTDFDFitModel( );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.SET_DEFAULT_WATER_MODEL_ID )
            {
                defineDefaultTDFDFitWaterModel( );
            }
            
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_INTERACTIVE_VOIGT_LINE_ID )
            {
                m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).expectMousePressedModeForInteractiveVoigtLineDefinition( );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_PARAMETRIC_PATTERN_ID )
            {
                File[] parametricPatternFileNames = getLoadPatternFileReference( "Load and add parametric pattern" );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addParametricPatternFrom_PPF_File( parametricPatternFileNames );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_NUMERIC_PATTERN_NPF_ID )
            {
                File[] numericPatternFileNames = getLoadPatternFileReference( "Load and add numeric pattern" );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addNumericPatternFrom_NPF_File( numericPatternFileNames );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_NUMERIC_PATTERN_JMRUI_ID )
            {
                File[]  mruiFileNames = getLoadPatternFileReference( "Create and add pattern from mrui file" );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addNumericPatternFrom_MRUI_Files( mruiFileNames );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_NUMERIC_PATTERN_BIN_ID )
            {
                File[] binaryFileNames = getLoadPatternFileReference( "Create and add pattern from binary file" );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addNumericPatternFrom_Binary_Files( binaryFileNames );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_METABOLITE_PATTERN_MPF_ID )
            {
                File[] metabolitePatternFileName = getLoadPatternFileReference( "Load and add metabolite pattern" );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addMetaboliteFrom_MPF_File( metabolitePatternFileName );
                m_MRSpectrumModellerPanel.simulateDecimalTextFieldChangedEvent( );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.ADD_MACRO_MOLECULAR_1H_BRAIN_BASELINE_ID )
            {
                 addMacroMolecularBaseline1HBrain( );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.WATER_REFERENCE_MODELLING_ID )
            {
                if ( m_signalPrototypeModelSpectrumFileReference != null )
                {
                    m_dicomDir = new DicomDirectory( this, 
                                                     new File( m_signalPrototypeModelSpectrumFileReference ).getParentFile( ).getParentFile( ), 
                                                    this );  
                }
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.QUANTITATIVE_VIEWER_ID )
            {
                QuantitativeSVSContextDataContainer dC = QuantitativeSVSContextDataContainer.getInstance( );
                TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
            }
            
            if ( e.getID( ) == TDFDFitModelToolbar.SET_DEFAULT_PARAMETER_RANGES_ID )
            {
                FittingParameterRangesFrame defaultRangesFrame = new FittingParameterRangesFrame( );
            }
            
        }
        
        // Handle the toolbar events
        if ( sender == m_basicFileToolbar )
        {
            if ( e.getID( ) == TDFDFitFileMenuToolbar.NEW_MODEL_ID )
            {
                newTDFDFitModel( );
                m_MRSignalPropertiesPanel.requestFocus( );
            }
            
            if ( e.getID( ) == TDFDFitFileMenuToolbar.OPEN_MODEL_ID )
            {
                loadTDFDFitPrototypeModel( true );
            }
            
            if ( e.getID( ) == TDFDFitFileMenuToolbar.SAVE_MODEL_ID )
            {
                saveTDFDFitModel( );
            }
            
            
            if ( e.getID( ) == TDFDFitFileMenuToolbar.OPEN_MODEL_KEEP_RESPONSE_ID )
            {
                loadTDFDFitPrototypeModelButKeepResponse( true );
            }
            
            if ( e.getID( ) == TDFDFitFileMenuToolbar.CLOSE_MODEL_ID )
            {
                closeTDFDFitModel( );
            }
            
            if ( e.getID( ) == TDFDFitFileMenuToolbar.PRINT_MODEL_ID )
            {
                System.out.println( "Not implemented yet!" );
                JOptionPane.showMessageDialog( this, "Printing is currently not supported!", "Not implemented yet", JOptionPane.INFORMATION_MESSAGE );
            }
        }
        
//        if ( sender == m_defaultTDFDFitRuntimeParametersMenuItem )
//        {
//            TDFDFitDefaultRuntimeSettingsDefinitionFrame defTDFDFitRuntimeSettings = new TDFDFitDefaultRuntimeSettingsDefinitionFrame( this );
//        }
        
        if ( sender == m_defaultParameterRangesMenuItem )
        {
            FittingParameterRangesFrame defaultRangesFrame = new FittingParameterRangesFrame( );
        }     
             
        if ( sender == m_aboutFitGuru )
        {
            Credits credits = new Credits( this, "TDFDFit modelling tool" );
            credits.showModal( );
        }
        
        if ( sender == m_exportPrototypeSignalAsMRUI )
        {
            
            String outputFileRef = m_TDFDfitPrototypeModelFileReference.toString( )+"_prototype_spectrum.mrui";
            System.out.println( outputFileRef );
            writeMeasuredProtoTypeSpectrumAsMruiFile( outputFileRef, 
                                                      MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ),
                                                      1000.0/MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ),
                                                      MRSignalProperties.getInstance( ).getResonanceFrequency( ) );
        }
        
        if ( sender == m_quitApp )
        {
            quitTDFDFitControlFrame( );   
        }
        
        if ( sender == m_addNumPatFromMRUIFileMenuItem )
        {
            File[] mruiFileNames = getLoadPatternFileReference( "Create and add pattern from mrui file" );
                
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addNumericPatternFrom_MRUI_Files( mruiFileNames );
            
        }
        
        if ( sender == m_addNumPatFromBinaryFileMenuItem )
        {
            File[] binaryFileNames = getLoadPatternFileReference( "Create and add pattern from binary file" );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addNumericPatternFrom_Binary_Files( binaryFileNames );
        }
            
        if ( sender == m_addNumPatFrom_NPF_FileMenuItem )
        {
            File[] numericPatternFileNames = getLoadPatternFileReference( "Load and add numeric pattern" );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addNumericPatternFrom_NPF_File( numericPatternFileNames );
        }
        
        if ( sender == m_addParPatFromFileMenuItem )
        {
            File[] parametricPatternFileName = getLoadPatternFileReference( "Load and add parametric pattern" );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addParametricPatternFrom_PPF_File( parametricPatternFileName );
        }
        
        if ( sender == m_defineDefaultTDFDFitWaterRemovalModelMenuItem )
        {
            defineDefaultTDFDFitWaterModel( );
        }
        
        if ( sender == m_defineDefaultTDFDFitModelMenuItem )
        {
             defineDefaultTDFDFitModel( );
        }
        
        if ( sender ==  m_addMacromolecularBaselineMenuItem )
        {
            addMacroMolecularBaseline1HBrain( );            
        }
        
        if ( sender ==  this.m_addGeneralBaselineMenuItem )
        {
            m_BaselineDefinitionFrame = new GeneralBaselineSettingsFrame( );
            m_BaselineDefinitionFrame.addActionListener( this );         
        }
        
        if ( sender == m_BaselineDefinitionFrame )
        {
            List generalBaselineList = m_BaselineDefinitionFrame.getGeneratedGeneralBaselineModelList( );
            m_BaselineDefinitionFrame.dispose( );
            
            for ( int i = 0; i < generalBaselineList.size( ); i++ )
            {   
                MRLineModelParameters par = (MRLineModelParameters)generalBaselineList.get( i );

                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addSimpleLineFromObject( par );     
            }
            // New method call to indicate that a prefab macromolecular baseline
            // was added.
            m_MRSpectrumModellerPanel.setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters.getInstance( ) );
        
            m_isModelChanged = true;          
        }
        
        if ( sender == m_addMetaboliteFromFileMenuItem )
        {
            File[] metabolitePatternFileName = getLoadPatternFileReference( "Load metabolite pattern" );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addMetaboliteFrom_MPF_File( metabolitePatternFileName );
            m_MRSpectrumModellerPanel.simulateDecimalTextFieldChangedEvent( );
        }
        
//        if ( sender == m_waterReferenceModellingMenuItem )
//        {
//            if ( m_signalPrototypeModelSpectrumFileReference != null )
//            {
//                m_dicomDir = new DicomDirectory( this, 
//                                                 new File( m_signalPrototypeModelSpectrumFileReference ).getParentFile( ).getParentFile( ), 
//                                                 this );  
//            }
//        }
        
        if ( sender == m_waterReferenceModellingMenuItem )
        {
            // This program logic works but is not adequat!
            if ( m_signalPrototypeModelSpectrumFileReference != null )
            {
                m_dicomDir = new DicomDirectory( this, 
                                                 new File( m_signalPrototypeModelSpectrumFileReference ).getParentFile( ).getParentFile( ), 
                                                 this );  
            }
        }
        
        if ( sender == m_dicomDir )
        {
            m_waterReferenceFrame = new TwoCompartmentWaterReferenceModelFittingFrame( this, m_dicomDir );
        }
        
        if ( sender == m_launchQuantitativeResultsViewer )
        {
            QuantitativeSVSContextDataContainer dC = QuantitativeSVSContextDataContainer.getInstance( );
            TDFDFitPrototypeModelSpectrumParameters model = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
            
//            QuantitativeFittingResultsOverviewFrame qFROF = new 
//                QuantitativeFittingResultsOverviewFrame( this,
//                                                         dC.get2CompartmentWaterReferenceFileNames( ),
//                                                         dC.getTwoCompartmentModelSet( ),
//                                                         dC.getTwoCompartmentModelFitted( ),
//                                                         dC.getReferenceWaterDefined( ),
//                                                         dC.getReferenceWaterDefinedFileName( ),
//                                                         dC.getNumberOfSpectralComponentsToFit( ),
//                                                         dC.getNumberOfSignalsFitted( ),
//                                                         model.getSpectralComponentsNames( ),
//                                                    
//                                                    double[][] concentrations,
//                                                    double[][] errorConcentrations,
//                                                    double[][] chemShift,
//                                                    double[][] errorsChemShift,
//                                                    double[][] phase,
//                                                    double[][] errorsPhase,
//                                                    double[][] lorentzWidth,
//                                                    double[][] errorsLorentzWidth,
//                                                    double[][] lorentzRWidth,
//                                                    double[][] errorsLorentzRWidth,
//                                                    
//                                                    int numberOf2CompWaterRefResponses,
//                                                    double amp1_TCM,
//                                                    double amp2_TCM,
//                                                    double TC1_TCM,
//                                                    double TC2_TCM,
//                                                    double err_amp1_TCM,
//                                                    double err_amp2_TCM,
//                                                    double err_TC1_TCM,
//                                                    double err_TC2_TCM,
//                                                    double[] waterRefReal,
//                                                    double[] waterRefImag,
//                                                    SVS_JMRUI_DICOM_Tag_FAST SVSJMRUIDicomTag,
//                                                    String referenceImage0,
//                                                    String referenceImage1,
//                                                    String referenceImage2,
//                                                    double[][] measuredReal,
//                                                    double[][] measuredImag,
//                                                    double[][] fittedReal,
//                                                    double[][] fittedImag
//                                                   )
        }
        
        // Handles loading of previously fitted data
        if ( sender == m_loadFittedDataMenuItem )
        {
            String fitFileName = getFilePathAndName( "Choose fit result binary file" );
            FitResultViewerFrame fitResultViewer = new FitResultViewerFrame( m_MRSignalProperties.getNumberOfSamplePoints( ), fitFileName, fitFileName, false );
        }
        
        // Set/change the MR conditions
        if ( sender == m_MRSignalPropertiesMenuItem )
        {
            changeMRSignalProperties( );              
        }
        
        // Set the database properties
        if ( sender == m_dataBaseProperties )
        {
            m_dataBaseConnectionDefinerFrame = new DataBaseConnectionDefinerFrame( this );
            m_dataBaseConnectionDefinerFrame.setVisible( true );
        }
        
        // User wants to view/edit the application defaults
        if ( sender == m_tdfdfitDefaultsMenuItem )
        {
            // MRFittingDefaultsFrame defaults =  new MRFittingDefaultsFrame( this );
            TDFDFitDefaultRuntimeSettingsDefinitionFrame defTDFDFitRuntimeSettings = new TDFDFitDefaultRuntimeSettingsDefinitionFrame( this );
        }
        
        // Generate a new TDFDFit fitting model
        if ( sender == m_newTDFDFitModel )
        {
            newTDFDFitModel( );
            m_MRSignalPropertiesPanel.requestFocus( );
        }
        
        // Close the current TDFDFit fitting model
        if ( sender == m_closeTDFDFitModel )
        {
            closeTDFDFitModel( );
        }
        
        // Handle the loading of the TDFDFit fitting project
        if ( sender == m_openTDFDFitModel )
        {
            loadTDFDFitPrototypeModel( true );
        }
        
        // Handle the loading of the TDFDFit fitting project
        if ( sender == m_openTDFDFitModelKeepResponse )
        {
            loadTDFDFitPrototypeModelButKeepResponse( true );
        }
        
        // Handle the saving of TDFDFitfit project request
        if ( sender == m_saveTDFDFitModel )
        {
            saveTDFDFitModel( );
            // notifyActionListeners( UPDATE_COMPO_BOX_CMD, UPDATE_COMPO_BOX_ID );
            // JOptionPane.showMessageDialog( this, "The default TDFDFit model has changed! \n The new default model is : " + m_TDFDfitPrototypeModelFileReference.getName( ) );
        }
        
        // Process the action events raised by the MRSignalProperties panel
        if ( sender == m_MRSignalPropertiesPanel )
        {
            handleMRSignalPropertiesPanelActionEvents( e );  
        }        
    }
    
    
    public void writeMeasuredProtoTypeSpectrumAsMruiFile( String file_name, 
                                                          int ndp, 
                                                          double step, 
                                                          double larmor ) 
    {

        try 
        {
            double dndp = (double) ndp;
            double[][] data = new double[2][ndp];

            for ( int i = 0; i < ndp; i++ ) 
            {
                data[0][i] = m_singleVoxelMeasuredData.m_absorptionChannelTD[i];
                data[1][i] = -m_singleVoxelMeasuredData.m_dispersionChannelTD[i];
            }

            FileOutputStream fileoutputstream = new FileOutputStream(file_name);
            DataOutputStream dataoutputstream = new DataOutputStream(fileoutputstream);
            dataoutputstream.writeDouble(0.0);					// 0
            dataoutputstream.writeDouble(dndp);					// 1
            dataoutputstream.writeDouble(step);					// 2
            dataoutputstream.writeDouble(0.0);					// 3
            dataoutputstream.writeDouble(0.0);					// 4
            dataoutputstream.writeDouble(larmor);				// 5
            for (int i = 0; i < 64 - 6; i++) 
            {
                dataoutputstream.writeDouble(0.0);	// 64 doubles in fileheader
            }
            for (int i = 0; i < ndp; i++) 
            {
                dataoutputstream.writeDouble(data[0][i]);
                dataoutputstream.writeDouble(data[1][i]);
            }
            dataoutputstream.close();
        }
        catch (IOException e) {}	
    }
    
    private void addMacroMolecularBaseline1HBrain( )
    {
        System.out.println( "Add MM baseline ... " );
        MacroMolecularBaseline mmbaseLineObject = new MacroMolecularBaseline( );

        List baselineList = mmbaseLineObject.getBaseLineComponentList( false, true );

        for ( int i = 0; i < baselineList.size( ); i++ )
        {   
            MRLineModelParameters par = (MRLineModelParameters)baselineList.get( i );

            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).addSimpleLineFromObject( par );     
        }

        // New method call to indicate that a prefab macromolecular baseline
        // was added.
        m_MRSpectrumModellerPanel.setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters.getInstance( ) );
        
        m_isModelChanged = true;
    }
    
    
    public void setModelChanged( boolean changed )
    {
        this.m_isModelChanged = changed;
    }
    
    private void defineDefaultTDFDFitModel( )
    {
        m_defaultTDFDFitModelFileName = getDefaultTDFDFitFileReference( "Setting the default TDFDFit model", 
                                                                        "Define as TDFDFFit default model " ).toString( );
        String parent = new File( m_defaultTDFDFitModelFileName ).getParent( );
        if ( m_defaultTDFDFitModelFileName != null )
        {
            if ( new File( m_defaultTDFDFitModelFileName ).exists( ) )
            {
                m_fittingDefaults.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH,
                                               parent );
                m_fittingDefaults.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME,
                                               ( new File( m_defaultTDFDFitModelFileName ) ).getName( ) );
                m_fittingDefaults.save( );
            }
            
            notifyActionListeners( UPDATE_COMPO_BOX_CMD, UPDATE_COMPO_BOX_ID );
            JOptionPane.showMessageDialog( this, "The default TDFDFit model has changed! \n The new default model is : " + m_defaultTDFDFitModelFileName );   
        }
        
    }
    
    private void defineDefaultTDFDFitWaterModel( )
    {
        m_defaultTDFDFitWaterRemovalModelFileName = getDefaultTDFDFitWaterModelFileReference( "Setting the default TDFDFit water removal model", 
                                                                                              "Define as TDFDFFit water removal model " ).toString( );

        String parent = new File( m_defaultTDFDFitWaterRemovalModelFileName ).getParent( );
        if ( m_defaultTDFDFitWaterRemovalModelFileName != null )
        {
            if ( new File( m_defaultTDFDFitWaterRemovalModelFileName ).exists( ) )
            {
                m_fittingDefaults.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH,
                                               parent );
                m_fittingDefaults.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME,
                                               ( new File( m_defaultTDFDFitWaterRemovalModelFileName ) ).getName( ) );

                m_fittingDefaults.save( );
            }
        }
    }
    
    
    /**
     * Returns the filename and directory selected by the user.
     */
    private String getFilePathAndName( String defaultString )
    {       
        JFileChooser fileChooser = new JFileChooser( );
        fileChooser.setFileHidingEnabled(false);
        
        String selectedFilePathAndName = "";
 
        // Create a File object containing the canonical path of the 
        // desired directory
        File f = new File( System.getProperty( "user.home" ) );
        // Set the current directory
        fileChooser.setCurrentDirectory( f );
            
        fileChooser.setSelectedFile( new File( defaultString ) );
        int result = fileChooser.showOpenDialog( this );
           
        File selected_file = new File( "" );
            
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( ); 
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }
        
        return selectedFilePathAndName; 
    }
    
    
    /**
     * Changes the basic MR-signal properties parameters object via a GUI.
     */
    private void changeMRSignalProperties( )
    {
        if ( m_MRSignalProperties != null )
        {    
            m_MRSignalPropertiesPanel = new MRSignalPropertiesPanel( "MR-signal properties", this, m_samplePoints );
            m_MRSignalPropertiesPanel.addActionListener( this );
            m_MRSignalPropertiesPanel.selectNucleusType( m_MRSignalProperties.getNucleusType( ) );
            m_MRSignalPropertiesPanel.setResonanceFrequency( m_MRSignalProperties.getResonanceFrequency( ) );
            m_MRSignalPropertiesPanel.setFitModelName( m_MRSignalProperties.getFitModelName( ) );
            m_MRSignalPropertiesPanel.setNumberOfSamplePoints( m_MRSignalProperties.getNumberOfSamplePoints( ) );
            m_MRSignalPropertiesPanel.setTruncationPoint( m_MRSignalProperties.getTruncationPoint( ) );
            m_MRSignalPropertiesPanel.setSampleBandwidth( m_MRSignalProperties.getFrequencyScale( ) * m_MRSignalProperties.getNumberOfSamplePoints( ) );
            m_MRSignalPropertiesPanel.setFirstOrderPhase( m_MRSignalProperties.getFirstOrderPhase( ) );
            m_MRSignalPropertiesPanel.setPointsToIgnore( m_MRSignalProperties.getIgnoreNumberOfPoints( ) );
            m_MRSignalPropertiesPanel.setDetermineNonAnalyticLineShapeFuntion( m_MRSignalProperties.getDetermineNonAnalyticLSF( ) );
            m_MRSignalPropertiesPanel.setFitUsesReferenceLine( m_MRSignalProperties.getUseReferenceLine( ) );
            m_MRSignalPropertiesPanel.setMinimumNoiseIntervalPointNumber( m_MRSignalProperties.getMinimumNoiseIntervalPointNumber( ) );
            m_MRSignalPropertiesPanel.setMaximumNoiseIntervalPointNumber( m_MRSignalProperties.getMaximumNoiseIntervalPointNumber( ) );
            
            m_MRSpectrumModellerPanel.updateUI( );
        }       
    }
    
    /**
     * Defines the TDFDFit model directory database directory root.
     */
    private File defineTDFDFitModelDirectoryDatabase( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String defaultPathString = fdef.getProperty( "Fit project database path" );
        
        File defaultRootPath = new File( defaultPathString );
        // check whether previous directory was deleted ; if yes set path back to
        // user.home directory
        if ( !defaultRootPath.exists( ) )
        {
            defaultPathString = System.getProperty( "user.home" );
        }
        
        JFileChooser fileChooser = new JFileChooser( defaultPathString );
        fileChooser.setFileHidingEnabled(false);
        
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        int result = fileChooser.showOpenDialog( this );
        
        File selected_dir = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                
                                                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                                                def.load( );
                                                def.setProperty( "Fit project database path", selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null )
                                                {
                                                    m_dicomDirectorySet = true;
                                                }
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selected_dir;
    }
    
    
    /**
     * Defines the directory root for TDFDFit models.
     */
    private File defineTDFDFitModelRootDirectory( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String defaultPathString = fdef.getProperty( TDFDFIT_MODELS_ROOT_PATH );
        
        File defaultRootPath = new File( defaultPathString );
        // check whether previous directory was deleted ; if yes set path back to
        // user.home directory
        if ( !defaultRootPath.exists( ) )
        {
            defaultPathString = System.getProperty( "user.home" );
        }
        
        JFileChooser fileChooser = new JFileChooser( defaultPathString );
        fileChooser.setFileHidingEnabled(false);
        
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        int result = fileChooser.showOpenDialog( this );
        
        File selected_dir = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                
                                                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                                                def.load( );
                                                def.setProperty( TDFDFIT_MODELS_ROOT_PATH, selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null )
                                                {
                                                    m_dicomDirectorySet = true;
                                                }
            
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selected_dir;
    }
    
    /**
     * Creates a new TDFDFit model!
     * Defines a directory on disk.
     */
    private void newTDFDFitModel( )
    {
        m_MRSignalPropertiesPanel = new MRSignalPropertiesPanel( "TDFDFit signal properties", this, m_samplePoints );
        m_MRSignalPropertiesPanel.addActionListener( this );
        m_MRSpectrumModellerPanel.setTDFDFitModelDefined( true );     
        MRFitStrategy fitStrat = MRFitStrategy.getInstance( );
        fitStrat.setTDTruncation( MRSignalProperties.getInstance( ).getTruncationPoint( ) );
        m_saveTDFDFitModel.setEnabled( true );
        m_addParPatFromFileMenuItem.setEnabled( true );
        m_addNumPatFrom_NPF_FileMenuItem.setEnabled( true );
        m_addNumPatFromBinaryFileMenuItem.setEnabled( true );
        m_addNumPatFromMRUIFileMenuItem.setEnabled( true );
        m_addMetaboliteFromFileMenuItem.setEnabled( true );
        m_addMacromolecularBaselineMenuItem.setEnabled( true );
    }
    
    /**
     * Closes the current TDFDFit model.
     */
    private void closeTDFDFitModel( )
    {
        TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).clearJTree( );
        m_MRSpectrumModellerPanel.getModelSpectrumViewer( ).deleteAllCurves( );
    }
    
    
    /**
     * Loads a TDFDFit prototype model from a file object.
     */
    private boolean loadTDFDFitPrototypeModel( boolean interactive )
    {
        // contains reference to model m_signalPrototypeModelSpectrumFileReference
        if ( interactive == true )
        { // here is some error...
            m_TDFDfitPrototypeModelFileReference = getLoadTDFDFitPrototypeModelFileReference( );
        }
        else
        {
            // m_TDFDfitModelFileReference is already defined in the constructor.
        }
        
        loadTDFDFitPrototypeModelSpectrumFile( );
        // TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( );
        DefaultMutableTreeNode dmt = (DefaultMutableTreeNode)TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).getSelectionPath( ).getLastPathComponent( );           
        ((DefaultTreeModel)TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).getModel( ) ).nodeChanged( dmt );
               
        return true;
    }
    
    /**
     * Loads a TDFDFit prototype model from a file object.
     */
    private boolean loadTDFDFitPrototypeModelButKeepResponse( boolean interactive )
    {
        // contains reference to model m_signalPrototypeModelSpectrumFileReference
        if ( interactive == true )
        { // here is some error...
            m_TDFDfitPrototypeModelFileReference = getLoadTDFDFitPrototypeModelFileReference( );
        }
        else
        {
            // m_TDFDfitModelFileReference is already defined in the constructor.
        }
        
        loadTDFDFitPrototypeModelSpectrumFileButKeepResponse( );
        // TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( );
        DefaultMutableTreeNode dmt = (DefaultMutableTreeNode)TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).getSelectionPath( ).getLastPathComponent( );           
        ((DefaultTreeModel)TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTreeWithPopUp( ).getModel( ) ).nodeChanged( dmt );
               
        return true;
    }
    
    /**
     * Saves a TDFDfit model to a file.
     * @return 
     */
    public boolean saveTDFDFitModel( )
    {

        saveTDFDFitModelToFile( );
        
        m_isModelChanged = false;
        
        return true;
    }
    
    /**
     * Saves the TDFDfit model to a file.
     */
    public void saveTDFDFitModelToFile( )
    {
        if ( m_TDFDfitPrototypeModelFileReference != null )
        {
            try
            {
                RandomAccessFile writer;
                
                if ( m_TDFDfitPrototypeModelFileReference.getName( ).endsWith( ".tfm" ) == false )
                {
                
                    writer = new RandomAccessFile( m_TDFDfitPrototypeModelFileReference + ".tfm", "rw" );
                }
                else
                {
                    writer = new RandomAccessFile( m_TDFDfitPrototypeModelFileReference, "rw" );    
                }
                
                // Write all essential object attritubutes to a file:
                // Write the MRSignalProperties object to file
                m_MRSignalProperties.write( writer );
                
                // Write the fitting strategy; if not defined the 
                // default strategy will be written
                m_fitStrategy = MRFitStrategy.getInstance( );
                m_fitStrategy.write( writer );
                
                // Write the TDFDFitPrototypeModelSpectrumParameters object to file
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).write( writer );
                
                // Write the single voxel spectral data response
                m_singleVoxelMeasuredDataLoaded.write( writer );
                
                // Write the baseline points to the file
                
//                m_baseLineModel.deleteAllPoints( );
//                
//                m_baseLineModel.addPoint( new Point2D.Double( (float)0, (float)50 ) );
//                m_baseLineModel.addPoint( new Point2D.Double( (float)150, (float)175 ) );
//                m_baseLineModel.addPoint( new Point2D.Double( (float)200, (float)250 ) );
//                m_baseLineModel.addPoint( new Point2D.Double( (float)550, (float)575 ) );
                
                m_baseLineModel.write( writer );
                
                try 
                {
                    writer.close( );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFitControlFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                
            }
            catch ( FileNotFoundException e )
            {
                // not handled yet
                e.printStackTrace( );
            }
        }
    }
    
    
    /**
     * Handles the ActionEvents of the MRSignalPropertiesPanel.
     */
    private void handleMRSignalPropertiesPanelActionEvents( ActionEvent e )
    {
        if ( e.getID( ) == MRSignalPropertiesPanel.CANCEL_EVENT )
        {
             m_MRSignalPropertiesPanel.dispose( );
        }
        
        // Called whenever the MR signal properties are changed
        if ( e.getID( ) == MRSignalPropertiesPanel.ACCEPT_EVENT )
        {
            m_MRSignalProperties = MRSignalProperties.getInstance( );
            
            m_MRSignalProperties.setFitModelName( m_MRSignalPropertiesPanel.getFitModelName( ) );   
            m_MRSignalProperties.setNumberOfSamplePoints( m_MRSignalPropertiesPanel.getNumberOfSamplePoints( ) );
            m_MRSignalProperties.setResonanceFrequency( m_MRSignalPropertiesPanel.getResonanceFrequency( ) );
            
            m_MRSignalProperties.setTruncationPoint( m_MRSignalPropertiesPanel.getTruncationPoint( ) );
            float freqScale = m_MRSignalPropertiesPanel.getSampleBandwidth( ) / m_MRSignalPropertiesPanel.getNumberOfSamplePoints( );
            
            m_MRSignalProperties.setPPMValueInCenterOfSpectrum( m_MRSignalPropertiesPanel.getPPMValueInCentreOfSpectrum( ) );
            m_MRSignalProperties.setFrequencyScale( freqScale );
            m_MRSignalProperties.setFirstOrderPhase( m_MRSignalPropertiesPanel.getFirstOrderPhase( ) );
            m_MRSignalProperties.setIgnoreNumberOfPoints( m_MRSignalPropertiesPanel.getPointsToIgnore( ) );
            m_MRSignalProperties.setDetermineNonAnalyticLSF( m_MRSignalPropertiesPanel.getDetermineNonAnalyticLineShapeFuntion( ) );
            m_MRSignalProperties.setUseReferenceLine( m_MRSignalPropertiesPanel.getFitUsesReferenceLine( ) );
            
            MRFitStrategy fitStrat = MRFitStrategy.getInstance( );
            fitStrat.setDefaultFittingRanges( m_MRSignalPropertiesPanel.getNumberOfSamplePoints( ) );
            fitStrat.setTDTruncation( m_MRSignalPropertiesPanel.getTruncationPoint( ) );
            
            m_MRSignalPropertiesPanel.dispose( );
            
            // Set the fitmodel name
            m_MRSpectrumModellerPanel.setFitModelName( m_MRSignalProperties.getFitModelName( ) );
            // m_addParPatFromFileMenuItem.setEnabled( m_dicomDirectorySet );
            m_MRSpectrumModellerPanel.updateUI( );
            
            // Also apply the new fitting conditions to the MRModelSpectrum
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMRSignalProperties( m_MRSignalProperties );   
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( m_MRSignalProperties.getFitModelName( ) );
            
            String modelName = m_MRSignalProperties.getFitModelName( );
            // Create the current project path
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            String currentProjectPath = fdef.getProperty( TDFDFIT_MODELS_BASE_PATH );   
            
            File file = new File( currentProjectPath + System.getProperty( "file.separator" ) + modelName );
            
            m_currentProjectPath = file.getPath( );
            
            if ( file.exists( ) == false )
            {
                // make the model persistent so that it can be found
                defineTDFDFitModelDirectoryStructure( );
                saveTDFDFitModel( );
                
                //fdef.setProperty( "Fit project path",  currentProjectPath + System.getProperty( "file.separator" ) + modelName );
                fdef.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, m_currentProjectPath );
                fdef.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, m_MRSignalPropertiesPanel.getFitModelName( ) + ".tfm" );
                fdef.save( );
                
                notifyActionListeners( UPDATE_COMPO_BOX_CMD, UPDATE_COMPO_BOX_ID );
                JOptionPane.showMessageDialog( this, "The default TDFDFit model has changed! \n The new default model is : " + m_MRSignalPropertiesPanel.getFitModelName( ) );
            }
            else
            {
                //fdef.setProperty( "Fit project path",  currentProjectPath + System.getProperty( "file.separator" ) + modelName );
                fdef.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, m_currentProjectPath );
                fdef.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, m_MRSignalPropertiesPanel.getFitModelName( ) + ".tfm" );
                fdef.save( );
            }
            m_isModelChanged = true;         
            m_mrSpectrumDataBase = MRPatientSpectrumDataBase.getInstance( );
            
        }
    }
    
    /**
     * Returns the file selected by the user.
     */
    private File getLoadTDFDFitPrototypeModelFileReference( )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String modelRef = fdef.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String modelPath = fdef.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
        
        File file = new File( modelPath + System.getProperty( "file.separator" ) + modelRef );
        
        if ( file.exists( ) == false )
        {
            file = new File( fdef.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH ) );
        }
        
        if ( file.exists( ) == false )
        {
            file = new File( System.getProperty( "user.home" ) );
        }
        
        JFileChooser fileChooser = new JFileChooser( file.getAbsolutePath( ) );
        fileChooser.setFileHidingEnabled(false);
        
        int result = fileChooser.showOpenDialog( this );
        
        m_TDFDfitPrototypeModelFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_TDFDfitPrototypeModelFileReference = fileChooser.getSelectedFile( );
//                                                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
//                                                def.load( );
//                                                def.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME,  m_TDFDfitPrototypeModelFileReference.getName( ) );
//                                                def.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, m_TDFDfitPrototypeModelFileReference.getParent( ) );
//                                                def.save( );
                                                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( m_TDFDfitPrototypeModelFileReference.getName( ) );
                                                // m_modelSpectrum.setModelSpectrumName( m_fitProjectFileReference.getName( ) );
                                                
                                                setTitle( m_TDFDfitPrototypeModelFileReference.getParent( ) );
                                                if ( m_TDFDfitPrototypeModelFileReference != null )
                                                {
                                                    m_fitProjectFileSelected = true;
                                                }
                                                
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_TDFDfitPrototypeModelFileReference;
    }
    
    /**
     * Returns the file reference to the model to be loaded and used by default.   
     * @param defaultString
     * @return 
     */
    private String getDefaultTDFDFitFileReference( String dialogTitle, String approveButtonText )
    {   
        
        // Default model is always the currently loaded model!
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String modelBaseName = fdef.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String modelBaseNamePlusExtension = fdef.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        
        if ( modelBaseName.contains( "." ) == true )
        {
            int position = modelBaseName.lastIndexOf(".");
        
            if ( position > 0 )
            {
                modelBaseName = modelBaseName.substring( 0, position );
            }
        }
        else
        {
            modelBaseNamePlusExtension = modelBaseName + ".tfm";
        }
        
        String defaultFilePath = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH ) + 
                                                   System.getProperty( "file.separator" );
        
        JFileChooser fileChooser = new JFileChooser( defaultFilePath );
        fileChooser.setFileHidingEnabled(false);
        
        String selectedFilePathAndName = "";
        fileChooser.setDialogTitle( dialogTitle );
        fileChooser.setApproveButtonText( approveButtonText );
        // Define a file filter to make only file selectable with appropriate
        // file extension.
        FileFilter filter = null;
        filter = new FileNameExtensionFilter( "TDFDFit model file", "tfm" );
        
        // Set the correct file filter options ...
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        // Create a File object containing the canonical path of the 
        // desired directory
        String lastDefinedModel = defaultFilePath + System.getProperty( "file.separator" ) + modelBaseName;    
        if ( ( lastDefinedModel != null ) && ( new File( lastDefinedModel ) ).exists( ) )
        {
            fileChooser.setCurrentDirectory( new File( lastDefinedModel ) );
        }
        else
        {
            fileChooser.setCurrentDirectory( new File( System.getProperty( "user.home" ) ) );
        }
        
        File selected_file = new File( m_fittingDefaults.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH ) + 
                                       System.getProperty( "file.separator" ) + modelBaseNamePlusExtension );
        
        if ( selected_file.exists( ) == true )
        {
            fileChooser.setSelectedFile( selected_file );
        }
        
        int result = fileChooser.showOpenDialog( this );
            
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( ); 
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }
        
        return selectedFilePathAndName; 
    }
    
    
    /**
     * Returns the file reference to the water model to be loaded and used by default.   
     * @param defaultString
     * @return 
     */
    private String getDefaultTDFDFitWaterModelFileReference( String dialogTitle, String approveButtonText )
    {   
        
        // Default model is always the currently loaded model!
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String waterModelBaseName = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME );
        String waterModelBaseNamePlusExtension = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH );
        
        if ( waterModelBaseName.contains( "." ) == true )
        {
            int position = waterModelBaseName.lastIndexOf( "." );
        
            if ( position > 0 )
            {
                waterModelBaseName = waterModelBaseName.substring( 0, position );
            }
        }
        else
        {
            waterModelBaseNamePlusExtension = waterModelBaseName + ".tfm";
        }
        
        String defaultFilePath = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH ) + 
                                                   System.getProperty( "file.separator" );
        
        JFileChooser fileChooser = new JFileChooser( defaultFilePath );
        fileChooser.setFileHidingEnabled(false);
        
        String selectedFilePathAndName = "";
        fileChooser.setDialogTitle( dialogTitle );
        fileChooser.setApproveButtonText( approveButtonText );
        // Define a file filter to make only file selectable with appropriate
        // file extension.
        FileFilter filter = null;
        filter = new FileNameExtensionFilter( "TDFDFit water model file", "tfm" );
        
        // Set the correct file filter options ...
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        // Create a File object containing the canonical path of the 
        // desired directory
        String lastDefinedModel = defaultFilePath + System.getProperty( "file.separator" ) + waterModelBaseName;    
        if ( ( lastDefinedModel != null ) && ( new File( lastDefinedModel ) ).exists( ) )
        {
            fileChooser.setCurrentDirectory( new File( lastDefinedModel ) );
        }
        else
        {
            fileChooser.setCurrentDirectory( new File( System.getProperty( "user.home" ) ) );
        }
        
        File selected_file = new File( m_fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME ) + 
                                       System.getProperty( "file.separator" ) + waterModelBaseNamePlusExtension );
        
        if ( selected_file.exists( ) == true )
        {
            fileChooser.setSelectedFile( selected_file );
        }
        
        int result = fileChooser.showOpenDialog( this );
            
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( ); 
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }
        
        return selectedFilePathAndName; 
    }
    
    /**
     * Returns the file selected by the user.
     */
    private File[] getLoadPatternFileReference( String approveButtonText )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        File directory = null;
        FileFilter filter = null;
        
        if ( approveButtonText.contains( "parametric" ) == true )
        {
            directory = new File( fdef.getProperty( TDFDFittingDefaults.LAST_USED_PARAMETRIC_PATTERN_LOAD_PATH ) + 
                                                    System.getProperty( "file.separator" ) + 
                                                    "Parametric_Patterns" );
            filter = new FileNameExtensionFilter( "Parametric pattern file", "ppf" );
        }
        else if ( approveButtonText.contains( "numeric" ) == true )
        {
            directory = new File( fdef.getProperty( TDFDFittingDefaults.LAST_USED_NUMERIC_PATTERN_LOAD_PATH ) + 
                                                    System.getProperty( "file.separator" ) + 
                                                    "Numeric_Patterns" );
            filter = new FileNameExtensionFilter( "Numeric pattern file", "npf" );
        } 
        else if ( approveButtonText.contains( "metabolite" ) == true )
        {
            directory = new File( fdef.getProperty( TDFDFittingDefaults.LAST_USED_METABOLITE_PATTERN_LOAD_PATH ) + 
                                                    System.getProperty( "file.separator" ) + 
                                                    "Metabolites" );
            filter = new FileNameExtensionFilter( "Metabolite file", "mbf" );
        }
        else if ( approveButtonText.contains( "mrui" ) == true )
        {
            directory = new File( fdef.getProperty( TDFDFittingDefaults.LAST_USED_NUMERIC_PATTERN_LOAD_PATH ) + 
                                                    System.getProperty( "file.separator" ) );
            filter = new FileNameExtensionFilter( "MRUI file", "mrui" );
        }
        else if ( approveButtonText.contains( "binary" ) == true )
        {
            directory = new File( fdef.getProperty( TDFDFittingDefaults.LAST_USED_NUMERIC_PATTERN_LOAD_PATH ) + 
                                                    System.getProperty( "file.separator" ) );
            filter = new FileNameExtensionFilter( "Binary file", "bin" );
        }
        
        JFileChooser fileChooser;
        
        if ( !directory.exists( ) )
        {
            fileChooser = new JFileChooser( System.getProperty( "user.home" ) );
        }
        else
        {
            fileChooser = new JFileChooser( directory );
        }
        fileChooser.setFileHidingEnabled( false );
        fileChooser.setApproveButtonText( approveButtonText );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        boolean multiSelect = false;
        
        if ( ( approveButtonText.contains( "binary" ) == true ) ||
             ( approveButtonText.contains( "mrui" ) == true ) ||
             ( approveButtonText.contains( "parametric" ) == true ) )
        {
            fileChooser.setMultiSelectionEnabled(true);
            multiSelect = true;
        }
        
        File[] selectedFileReferences = null;

        int result = fileChooser.showOpenDialog( this );
        
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   

                                                // Approve Open button was pressed
                                                if ( multiSelect == true )
                                                {
                                                    selectedFileReferences = fileChooser.getSelectedFiles( );
                                                }  
                                                else
                                                {
                                                    selectedFileReferences[0] = fileChooser.getSelectedFile( );
                                                }
                                                
                                                if ( approveButtonText.contains( "mrui" ) || 
                                                     approveButtonText.contains( "binary" ) ||
                                                     approveButtonText.contains( "numeric" ) )
                                                {
                                                    fdef.setProperty( TDFDFittingDefaults.LAST_USED_NUMERIC_PATTERN_LOAD_PATH, 
                                                                      selectedFileReferences[0].getParent( ) );
                                                }
                                                if ( approveButtonText.contains( "parametric" ) )
                                                {
                                                    fdef.setProperty( TDFDFittingDefaults.LAST_USED_PARAMETRIC_PATTERN_LOAD_PATH, 
                                                                      selectedFileReferences[0].getParent( ) );
                                                }
                                                if ( approveButtonText.contains( "metabolite" ) )
                                                {
                                                    fdef.setProperty( TDFDFittingDefaults.LAST_USED_METABOLITE_PATTERN_LOAD_PATH, 
                                                                      selectedFileReferences[0].getParent( ) );
                                                }
                                                
                                                fdef.save( );
                                                
                                                setTitle( directory.getName( ) );

                                                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( m_TDFDfitPrototypeModelFileReference.getName( ) );
                                                // m_modelSpectrum.setModelSpectrumName( m_fitProjectFileReference.getName( ) );
                                                if ( m_TDFDfitPrototypeModelFileReference != null )
                                                {
                                                    m_fitProjectFileSelected = true;
                                                }
                                                
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selectedFileReferences;
    }
    
    
    /**
     * Returns the file selected by the user.
     */
    private File getSaveTDFDFitModelToFileReference( )
    {
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String modelBaseName = fdef.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        int position = modelBaseName.lastIndexOf(".");
        if ( ( position > 0 ) && ( position == modelBaseName.length( ) - 4 ) )
        {
            modelBaseName = modelBaseName.substring( 0, position );
        }

        String defaultFilePath = fdef.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH ) + 
                                                   System.getProperty( "file.separator" );
        
        JFileChooser fileChooser = new JFileChooser( defaultFilePath );
        fileChooser.setFileHidingEnabled(false);
        
        FileFilter filter = new FileNameExtensionFilter( "TDFDFit model", "tfm" );
        fileChooser.addChoosableFileFilter( filter );
        fileChooser.setFileFilter( filter );
        
        String lastModelPath = fdef.getProperty( DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        
        String defaultModelReference = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH ) + System.getProperty( "file.separator" ) + modelBaseName;
        
        if ( defaultModelReference == null )
        {
            defaultModelReference = System.getProperty( "user.home" ) + modelBaseName;
        }
        
        fileChooser.setSelectedFile( new File( defaultModelReference + System.getProperty( "file.separator" ) + modelBaseName ) );
        
        int result = fileChooser.showSaveDialog( this );
        
        m_TDFDfitPrototypeModelFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_TDFDfitPrototypeModelFileReference = fileChooser.getSelectedFile( );
                                                
                                                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                                                def.load( );
                                                
                                                def.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, 
                                                                 m_TDFDfitPrototypeModelFileReference.getParent( ) + 
                                                                 System.getProperty( "file.separator" ) );
                                                def.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, 
                                                                 m_TDFDfitPrototypeModelFileReference.getName( ) );
                                                // save the new settings
                                                def.save( );
                                                if ( m_TDFDfitPrototypeModelFileReference != null )
                                                {
                                                    m_fitProjectFileSelected = true;
                                                }
                                                
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_TDFDfitPrototypeModelFileReference;
    }  
    
    /**
     * Loads the TDFDFit prototype model spectrum from a file which is 
     * measured, and serves as prototype for fitting other signals with.
     */
    private void loadTDFDFitPrototypeModelSpectrumFile( )
    {
        if ( ( m_TDFDfitPrototypeModelFileReference != null ) && 
             ( m_TDFDfitPrototypeModelFileReference.exists( ) ) )
        {
            try
            {
                RandomAccessFile reader = new RandomAccessFile( m_TDFDfitPrototypeModelFileReference, "rw" );
            
                m_MRSignalProperties = MRSignalProperties.getInstance( );
                
                // Load all essential object attritubutes to a file.
                m_MRSignalProperties.read( reader );
                
                // Load the fitting strategy; if not defined you will get the 
                // default strategy.
                m_fitStrategy = MRFitStrategy.getInstance( );
                m_fitStrategy.read( reader );
                
                // Read the TDFDFitPrototypeModelSpectrum object from file.
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).read( reader );
                
                // Read the single voxel spectral data response
                m_singleVoxelMeasuredData.read( reader, false );
                
                if ( m_singleVoxelMeasuredDataLoaded != null )
                {
                    m_singleVoxelMeasuredData.autoSetXaxis( );
                    m_singleVoxelMeasuredDataLoaded = m_singleVoxelMeasuredData;
                    m_singleVoxelMeasuredDataLoaded.autoSetXaxis( );
                    m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredDataLoaded, false );
                }
                else
                {
                    m_singleVoxelMeasuredDataLoaded = m_singleVoxelMeasuredData;
                    m_singleVoxelMeasuredData.autoSetXaxis( );
                    m_singleVoxelMeasuredDataLoaded.autoSetXaxis( );
                    m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredData, false );
                }
                
                // The model spectrum must know its m_MRSignalProperties object
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMRSignalProperties( m_MRSignalProperties );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMeasuredSpectrum( m_singleVoxelMeasuredData );
                m_addParPatFromFileMenuItem.setEnabled( true );
                m_addNumPatFrom_NPF_FileMenuItem.setEnabled( true );
                m_addNumPatFromBinaryFileMenuItem.setEnabled( true );
                m_addNumPatFromMRUIFileMenuItem.setEnabled( true );
                m_addMetaboliteFromFileMenuItem.setEnabled( true );
                m_addMacromolecularBaselineMenuItem.setEnabled( true );
                
                // Set the MRModelSpectrum object
                m_MRSpectrumModellerPanel.setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters.getInstance( ) );
                m_MRSpectrumModellerPanel.setTDFDFitModelDefined( true );
                
                // Reads the baseline points from file
                m_baseLineModel.read( reader );
                
                // Load the patient database, and get a reference to it
                m_mrSpectrumDataBase = MRPatientSpectrumDataBase.getInstance( );
            }
            catch ( FileNotFoundException e )
            {
                // Not handled yet.
            }
            
            if ( m_MRSignalProperties != null )
            {
                m_saveTDFDFitModel.setEnabled( true );
                m_MRSpectrumModellerPanel.setFitModelName( m_MRSignalProperties.getFitModelName( ) );
                m_MRSpectrumModellerPanel.getMRLineModelParametersPanel( ).setCurrentComponentName( m_MRSignalProperties.getFitModelName( ) ); 
            }
        }
    }
    
    /**
     * Loads the TDFDFit prototype model spectrum from a file which is 
     * measured, and serves as prototype for fitting other signals with.
     * The response will be kept though. This is useful in cases you want
     * to fit a response, with an already existent model which you want 
     * to change, and store the resulting new model.
     */
    private void loadTDFDFitPrototypeModelSpectrumFileButKeepResponse( )
    {
        if ( m_TDFDfitPrototypeModelFileReference != null )
        {
            try
            {
                RandomAccessFile reader = new RandomAccessFile( m_TDFDfitPrototypeModelFileReference, "rw" );
            
                m_MRSignalProperties = MRSignalProperties.getInstance( );
                
                // Load all essential object attritubutes to a file.
                m_MRSignalProperties.read( reader );
                
                // Load the fitting strategy; if not defined you will get the 
                // default strategy.
                m_fitStrategy = MRFitStrategy.getInstance( );
                m_fitStrategy.read( reader );
                
                // Read the TDFDFitPrototypeModelSpectrum object from file.
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).read( reader );
                
                // Read the single voxel spectral data response
                
                SingleVoxelMeasuredSpectralData singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( );
                singleVoxelMeasuredData.read( reader, false );
                
                // if ( m_singleVoxelMeasuredDataLoaded != null )
                // {
                //     m_singleVoxelMeasuredData.autoSetXaxis( );
                //     m_singleVoxelMeasuredDataLoaded = m_singleVoxelMeasuredData;
                //     m_singleVoxelMeasuredDataLoaded.autoSetXaxis( );
                //     m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredDataLoaded );
                // }
                // else
                // {
                //     m_singleVoxelMeasuredDataLoaded = m_singleVoxelMeasuredData;
                //     m_singleVoxelMeasuredData.autoSetXaxis( );
                //     m_singleVoxelMeasuredDataLoaded.autoSetXaxis( );
                //     m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredData );
                // }
                
                // The model spectrum must know its m_MRSignalProperties object
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMRSignalProperties( m_MRSignalProperties );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMeasuredSpectrum( m_singleVoxelMeasuredData );
                m_addParPatFromFileMenuItem.setEnabled( true );
                m_addNumPatFrom_NPF_FileMenuItem.setEnabled( true );
                m_addNumPatFromBinaryFileMenuItem.setEnabled( true );
                m_addNumPatFromMRUIFileMenuItem.setEnabled( true );
                m_addMetaboliteFromFileMenuItem.setEnabled( true );
                m_addMacromolecularBaselineMenuItem.setEnabled( true );
                // Set the MRModelSpectrum object
                m_MRSpectrumModellerPanel.setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters.getInstance( ) );
                m_MRSpectrumModellerPanel.setTDFDFitModelDefined( true );
                
                // Reads the baseline points from file
                m_baseLineModel.read( reader );
                
                // Load the patient database, and get a reference to it
                m_mrSpectrumDataBase = MRPatientSpectrumDataBase.getInstance( );
            }
            catch ( FileNotFoundException e )
            {
                // Not handled yet.
            }
            if ( m_MRSignalProperties != null )
            {
                m_saveTDFDFitModel.setEnabled( true );
                m_MRSpectrumModellerPanel.setFitModelName( m_MRSignalProperties.getFitModelName( ) );
                m_MRSpectrumModellerPanel.getMRLineModelParametersPanel( ).setCurrentComponentName( m_MRSignalProperties.getFitModelName( ) ); 
            }
        }
    }
    
    
    /**
     * Defines the new TDFDFit model directory structure of the project.
     */
    private void defineTDFDFitModelDirectoryStructure( )
    {
        // Create the current project path
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        m_currentProjectPath = fdef.getProperty( TDFDFIT_MODELS_BASE_PATH );    
            
        m_currentProjectPath = m_currentProjectPath + System.getProperty( "file.separator" ) + 
                               m_MRSignalPropertiesPanel.getFitModelName( ) + System.getProperty( "file.separator" );
        
        m_TDFDfitPrototypeModelFileReference = new File( m_currentProjectPath + m_MRSignalPropertiesPanel.getFitModelName( ) );
        
        File projectPath = new File( m_currentProjectPath );
        int cntr = 0;
        
        if ( !projectPath.exists( ) )
        {
            // The project does not exist up till now: so create it
            projectPath.mkdir( );
            m_MRSpectrumModellerPanel.setFitModelName( m_MRSignalPropertiesPanel.getFitModelName( ) );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( m_MRSignalPropertiesPanel.getFitModelName( ) );
            fdef.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH,  m_currentProjectPath );
            fdef.save( );
        }
        else
        {
            // The project already exists so try to create a unique project name
            // which does not exist, by numbering.    
            while ( projectPath.exists( ) )
            {
                m_currentProjectPath = fdef.getProperty( "Fit project root path" ) + 
                System.getProperty( "file.separator" ) + 
                m_MRSignalPropertiesPanel.getFitModelName( ) + 
                "_" + (++cntr);
                projectPath = new File( m_currentProjectPath );
            }
            fdef.setProperty( "Fit project path",  m_currentProjectPath );
            projectPath.mkdir( );

            m_MRSpectrumModellerPanel.setFitModelName( m_MRSignalProperties.getFitModelName( ) + "(" + (cntr) + ")" );
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( m_MRSignalPropertiesPanel.getFitModelName( ) + "(" + (cntr) + ")" );
         }   
         m_MRSpectrumModellerPanel.updateUI( );
        
         // Set the title of the application frame object
         this.setTitle( m_currentProjectPath );
         
         // Create sub directory for individual lines
         String singleLinesPathStr = m_currentProjectPath + System.getProperty( "file.separator" ) + "Single_Lines" + System.getProperty( "file.separator" );
         File singleLinesPath = new File( singleLinesPathStr );
         if ( singleLinesPath.exists( ) == false )
         {
            singleLinesPath.mkdir( );
         }
         
        // Create sub directory for numeric patterns
        String numPattternPathStr = m_currentProjectPath + System.getProperty( "file.separator" ) + "Numeric_Patterns" + System.getProperty( "file.separator" );
        File numPattternPath = new File( numPattternPathStr );
        if ( numPattternPath.exists( ) == false )
        {
            numPattternPath.mkdir( );
        }
        // Create sub directory for parametric patterns
        String parPattternPathStr = m_currentProjectPath + System.getProperty( "file.separator" ) + "Parametric_Patterns" + System.getProperty( "file.separator" );
        File parPattternPath = new File( parPattternPathStr );
        if ( parPattternPath.exists( ) == false )
        {
            parPattternPath.mkdir( );
        }
        // Create sub directory for metabolites
         String metabolitePathStr = m_currentProjectPath + System.getProperty( "file.separator" ) + "Metabolites" + System.getProperty( "file.separator" );
         File metabolitePath = new File( metabolitePathStr );
         metabolitePath.mkdir( );
         
        if ( metabolitePath.exists( ) == false )
        {
            metabolitePath.mkdir( );
        }
        // Create sub directory for binary tdfdfit responses
        String tdfdfitResponsePathStr = m_currentProjectPath + System.getProperty( "file.separator" ) + "tdfdfit_responses" + System.getProperty( "file.separator" );
        File tdfdfitPath = new File( tdfdfitResponsePathStr );
        if ( !tdfdfitPath.exists( ) == true )
        {
            tdfdfitPath.mkdir( );
        }
        
        this.saveTDFDFitModel( );
    }
    
    /**
     * Loads spectroscopic data from DICOM files and displays them
     * in appropriate graphic controls, ie determines whether a CSI
     * or an SVS dataset is chosen.
     */
    private void loadSpectroscopicPrototypeModelData( String dicomFileRef )
    {
        // Get the general file characteristics
        DicomImageData d_ima_data = new DicomImageData( dicomFileRef );
        FileInfo fi = d_ima_data.getHeader( );
        LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );
        
        // Decide whether we are dealing with a SVS or CSI
        int npoints, dim1, dim2, dim3;
        npoints = 0;
        dim1 = dim2 = dim3 = 1;
                 
        if ( (String)ssd.get( "sSpecPara.lVectorSize" ) != null )
        {
            npoints = ( new Integer( (String)ssd.get( "sSpecPara.lVectorSize" ) ) ).intValue( );
            m_samplePoints = npoints;
        }
        
        if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
        {
            dim1 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( ); 
        }
                    
        if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
        {
            dim2 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( ); 
        }
                    
        // 3D data set not yet available, so set the value to one
        dim3 = 1;
        
        if ( ( dim1 == 1 ) && ( dim2 == 1 ) && ( dim3 == 1 ) )
        {
            // we are dealing with SVS data
            initializeSVSModelComponent( d_ima_data, fi, ssd );
        }
        
        if ( ( dim1 != 1 ) || ( dim2 != 1 ) || ( dim3 != 1 ) )
        {
            Dialogs.messageBox( this, "Only a 1D prototype model-spectrum type can be handled!", "Not handled spectrum" );
        }
        
    }
    
    /**
     * Loads spectroscopic data from DICOM files and displays them
     * in appropriate graphic controls, ie determines whether a CSI
     * or an SVS dataset is chosen.
     */
    private void loadSpectrImSpectroscopicPrototypeModelData( String dicomFileRef )
    {
        
        SingleVoxelMeasuredSpectralData svsData = new SingleVoxelMeasuredSpectralData( );
        SingleVoxelMeasuredSpectralData dataToLoad = svsData.getSVSData( dicomFileRef );
        // dataToLoad.autoSetXaxis( );
        // We are dealing with SVS data
        initializeSVSModelComponent( dataToLoad );
        
    }
    
    
    /**
     * Loads spectroscopic data from DICOM files and displays them
     * in appropriate graphic controls, ie determines whether a CSI
     * or an SVS dataset is chosen.
     */
    private void loadSpectrImSpectroscopicPrototypeModelData( CSIData modelPrototypeSpectralData )
    {
        
        if ( modelPrototypeSpectralData != null )
        {
            
            Spectra spectra = modelPrototypeSpectralData.getSpectra( );
            
            Iterator it = spectra.getData( ).keySet( ).iterator( );
            String key = spectra.getData( ).keySet( ).iterator( ).next( );
            
            ArrayList<String> ids = this.m_globalModel.getSelectedVoxels( );
            float[] absTD = spectra.getMeanSpectrum( ids, Spectra.Domain.TD, Spectra.Mode.ABS );
            float[] dispTD = spectra.getMeanSpectrum( ids, Spectra.Domain.TD, Spectra.Mode.DISP );
            
            for ( int i = 0; i < dispTD.length; i++ )
            {
                dispTD[i] = -dispTD[i];
            }
            
            
            m_MRSignalProperties = MRSignalProperties.getInstance( );
            
            
            // Read the single voxel spectral data response
            m_singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( absTD, dispTD );
            DicomStringReformatter dformat = new DicomStringReformatter( );
            
            m_singleVoxelMeasuredData.setPatientName( dformat.reformatPatientName( modelPrototypeSpectralData.getPatientName( ).replaceAll( "�","_" ) ) );
            m_singleVoxelMeasuredData.setPatientBirthDate( dformat.getDateMonthYear( modelPrototypeSpectralData.getPatientDOB( ) ) );
            m_singleVoxelMeasuredData.setSeriesDescription( modelPrototypeSpectralData.getSeriesDescription( ) );
            m_singleVoxelMeasuredData.setSeriesNumber( modelPrototypeSpectralData.getSeriesNumber( ) );
            m_singleVoxelMeasuredData.setInstanceNumber( modelPrototypeSpectralData.getInstanceNumber( ) );
            m_singleVoxelMeasuredData.setAcquisitionDate( dformat.getDateMonthYear( modelPrototypeSpectralData.getStudyDate( ) ) );
            m_singleVoxelMeasuredData.setTE( modelPrototypeSpectralData.getTE( ) );
            m_singleVoxelMeasuredData.setTR( modelPrototypeSpectralData.getTR( ) ); 
            
            double[] excVoxelDimensions = modelPrototypeSpectralData.csigrid.getDimsExcitedVolume( );
            
            m_singleVoxelMeasuredData.set_sVOIThickness( excVoxelDimensions[0] );
            m_singleVoxelMeasuredData.set_sVoIPhaseFOV( excVoxelDimensions[1] );
            m_singleVoxelMeasuredData.set_sVoIReadoutFOV( excVoxelDimensions[2] );
            
            // m_MRSpectrumModellerPanel.setMeasuredClinicalSVSSpectrum( modelPrototypeSpectralData.getFile( ), m_singleVoxelMeasuredData );             
           
        }
        else
        {
            JOptionPane.showMessageDialog( rootPane, "No patient data loaded! First load a patient dataset.");
        }
        
        // SingleVoxelMeasuredSpectralData svsData = new SingleVoxelMeasuredSpectralData( );
        // SingleVoxelMeasuredSpectralData dataToLoad = svsData.getSVSData( dicomFileRef );
        // dataToLoad.autoSetXaxis( );
        // We are dealing with SVS data
        initializeSVSModelComponent( m_singleVoxelMeasuredData );
        
    }        
            
            
    
     /**
     * Loads spectroscopic data from DICOM files and displays them
     * in appropriate graphic controls, ie determines whether a CSI
     * or an SVS dataset is chosen.
     */
    private void loadClinicalSVSCaseModelData( String dicomFileRef )
    {
        // Get the general file characteristics
        DicomImageData d_ima_data = new DicomImageData( dicomFileRef );
        FileInfo fi = d_ima_data.getHeader( );
        LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );
        
        // Decide whether we are dealing with a SVS or CSI
        int npoints, dim1, dim2, dim3;
        npoints = 0;
        dim1 = dim2 = dim3 = 1;
                 
        if ( (String)ssd.get( "sSpecPara.lVectorSize" ) != null )
        {
            npoints = ( new Integer( (String)ssd.get( "sSpecPara.lVectorSize" ) ) ).intValue( );
            m_samplePoints = npoints;
        }
        
        if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
        {
            dim1 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( ); 
        }
                    
        if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
        {
            dim2 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( ); 
        }
                    
        // 3D data set not yet available, so set the value to one
        dim3 = 1;
        
        if ( ( dim1 == 1 ) && ( dim2 == 1 ) && ( dim3 == 1 ) )
        {
            // we are dealing with SVS data
            initializeSVSModelComponent( d_ima_data, fi, ssd );
        }
        
        if ( ( dim1 != 1 ) || ( dim2 != 1 ) || ( dim3 != 1 ) )
        {
            Dialogs.messageBox( this, "Only a 1D prototype model-spectrum type can be handled!", "Not handled spectrum" );
        }
        
    }
    
    
    /**
     * Loads spectroscopic data from DICOM files and displays them
     * in appropriate graphic controls, ie determines whether a CSI
     * or an SVS dataset is chosen.
     */
    private void loadSpectroscopicPatientData( String dicomFileRef )
    {
        // Get the general file characteristics
        DicomImageData d_ima_data = new DicomImageData( dicomFileRef );
        FileInfo fi = d_ima_data.getHeader( );
        LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );
        
        // Decide whether we are dealing with a SVS or CSI
        int npoints, dim1, dim2, dim3;
        npoints = 0;
        dim1 = dim2 = dim3 = 1;
                 
        if ( (String)ssd.get( "sSpecPara.lVectorSize" ) != null )
        {
            npoints = ( new Integer( (String)ssd.get( "sSpecPara.lVectorSize" ) ) ).intValue( );
            m_samplePoints = npoints;
        }
        
        if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
        {
            dim1 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( ); 
        }
                    
        if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizeRead" ) != null )
        {
            dim2 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizeRead" ) ) ).intValue( ); 
        }
                    
        // 3D data set not yet available, so set the value to one
        dim3 = 1;
        
        if ( ( dim1 == 1 ) && ( dim2 == 1 ) && (dim3 == 1 ) )
        {
            // we are dealing with SVS data
     //       initializeSVSGraphicComponents( d_ima_data, fi, ssd );
        }
        if ( ( dim1 != 1 ) || ( dim2 != 1 ) || ( dim3 != 1 ) )
        {
            // we are dealing with an CSI dataset
            // initializeCSIGraphicComponents( d_ima_data, fi, ssd, npoints, dim1, dim2, dim3 );
           
        }
    }
    
    /**
     * Initializes the single voxel spectroscopy graphic components for modeling.
     */
    private void initializeSVSModelComponent( DicomImageData d_ima_data, FileInfo fi, LinkedHashMap ssd )
    {
                          
        SingleVoxelMeasuredSpectralData specData =  new SingleVoxelMeasuredSpectralData( fi.getNonImageData( ) );
        specData.setDicomHeaderInfo( fi );
        specData.setSiemensSpecificDicomHeaderInfo( d_ima_data.getSiemensSpecificDataFromFile( ) );
        m_singleVoxelMeasuredDataLoaded = specData;
        specData.autoSetXaxis( );
            
        // Update the MRSignalProperties according to the loaded data.
        MRSignalProperties.getInstance( ).setNumberOfSamplePoints( m_samplePoints );       
        MRSignalProperties.getInstance( ).setFrequencyScale( m_singleVoxelMeasuredDataLoaded.getSampleFrequency( )/ m_samplePoints );
        MRSignalProperties.getInstance( ).setResonanceFrequency( m_singleVoxelMeasuredDataLoaded.getResonanceFrequency( ) );
        MRSignalProperties.getInstance( ).setNucleusType( m_singleVoxelMeasuredDataLoaded.getNucleusType( ) );
         
        m_MRSpectrumModellerPanel.setMeasuredSpectrumToPreview( specData, true );          
        m_MRSpectrumModellerPanel.setMeasuredSpectrum( specData, true );
        TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMeasuredSpectrum( specData );
    }

    /**
     * Initializes the single voxel spectroscopy graphic components for modeling
     * New style, namely adapted for SpectrIm use.
     */
    private void initializeSVSModelComponent( SingleVoxelMeasuredSpectralData svsData )
    {             
        m_singleVoxelMeasuredDataLoaded = new SingleVoxelMeasuredSpectralData( svsData );
        
        m_singleVoxelMeasuredDataLoaded.autoSetXaxis( );
            
        // Update the MRSignalProperties according to the loaded data.
        MRSignalProperties.getInstance( ).setNumberOfSamplePoints( m_samplePoints );       
        MRSignalProperties.getInstance( ).setFrequencyScale( m_singleVoxelMeasuredDataLoaded.getSampleFrequency( )/ m_samplePoints );
        MRSignalProperties.getInstance( ).setResonanceFrequency( m_singleVoxelMeasuredDataLoaded.getResonanceFrequency( ) );
        MRSignalProperties.getInstance( ).setNucleusType( m_singleVoxelMeasuredDataLoaded.getNucleusType( ) );
         
        m_MRSpectrumModellerPanel.setMeasuredSpectrumToPreview( m_singleVoxelMeasuredDataLoaded, false );          
        m_MRSpectrumModellerPanel.setMeasuredSpectrum( m_singleVoxelMeasuredDataLoaded, false );
        TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMeasuredSpectrum( m_singleVoxelMeasuredDataLoaded );
    }
    
    public enum Mode {
        
        ABS, DISP, MAG
    };
    
    public enum Domain {
        
        FD_PPM("FD - ppm"), FD_HZ("FD - MHz"), TD("TD - ms");
        public final String txt;
        
        private Domain(String txt) {
            this.txt = txt;
        }
    };
    
    
    /**
     * Forwards the loaded the clinical SVS dataset to the viewer.
     * @param clinicalSpectrumData 
     */
    private void loadClinicalSVSCaseModelData( CSIData clinicalSpectrumData ) 
    {
        
        if ( clinicalSpectrumData != null )
        {
            
            Spectra spectra = clinicalSpectrumData.getSpectra( );
            
            Iterator it = spectra.getData( ).keySet( ).iterator( );
            String key = spectra.getData( ).keySet( ).iterator( ).next( );
            
            ArrayList<String> ids = this.m_globalModel.getSelectedVoxels( );
            float[] absTD = spectra.getMeanSpectrum( ids, Spectra.Domain.TD, Spectra.Mode.ABS );
            float[] dispTD = spectra.getMeanSpectrum( ids, Spectra.Domain.TD, Spectra.Mode.DISP );
            
            // Read the single voxel spectral data response
            m_singleVoxelClinicalData = new SingleVoxelMeasuredSpectralData( absTD, dispTD );
            DicomStringReformatter dformat = new DicomStringReformatter( );
            
            m_singleVoxelClinicalData.setPatientName( dformat.reformatPatientName( clinicalSpectrumData.getPatientName( ).replaceAll( "�","_" ) ) );
            m_singleVoxelClinicalData.setPatientBirthDate( dformat.getDateMonthYear( clinicalSpectrumData.getPatientDOB( ) ) );
            m_singleVoxelClinicalData.setSeriesDescription( clinicalSpectrumData.getSeriesDescription( ) );
            m_singleVoxelClinicalData.setSeriesNumber( clinicalSpectrumData.getSeriesNumber( ) );
            m_singleVoxelClinicalData.setInstanceNumber( clinicalSpectrumData.getInstanceNumber( ) );
            m_singleVoxelClinicalData.setAcquisitionDate( dformat.getDateMonthYear( clinicalSpectrumData.getStudyDate( ) ) );
            m_singleVoxelClinicalData.setTE( clinicalSpectrumData.getTE( ) );
            m_singleVoxelClinicalData.setTR( clinicalSpectrumData.getTR( ) ); 
            
            double[] excVoxelDimensions = clinicalSpectrumData.csigrid.getDimsExcitedVolume( );
            
            m_singleVoxelClinicalData.set_sVOIThickness( excVoxelDimensions[0] );
            m_singleVoxelClinicalData.set_sVoIPhaseFOV( excVoxelDimensions[1] );
            m_singleVoxelClinicalData.set_sVoIReadoutFOV( excVoxelDimensions[2] );
            
            m_MRSpectrumModellerPanel.setMeasuredClinicalSVSSpectrum( clinicalSpectrumData.getFile( ), m_singleVoxelClinicalData );             
           
        }
        else
        {
            JOptionPane.showMessageDialog( rootPane, "No patient data loaded! First load a patient dataset.");
        }
    }
 
    private void jMenuItemExportSelectedSpectraTojMRUIFormatActionPerformed( ) 
    {  
              
        System.out.println( "jMenuItemExportSelectedSpectraTojMRUIFormatActionPerformed .. " );
        
        JMRUIFile mruiFile = new JMRUIFile( );
        
        mruiFile.saveNumericPatternsOfModelAsMRUITypedData( m_globalModel );
        
    }                                                                              

    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    // sent the cancel event to all listeners!
     private void notifyActionListeners( String cmd, int cmd_id )
     {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, cmd_id, cmd ) );
        }
     }
    
     // Notification of clinical viewer
     public final static String UPDATE_COMPO_BOX_CMD   = "update-combo-box-event";
     public final static int    UPDATE_COMPO_BOX_ID    = 0;
}
