/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.colormap.ColorMap;
import org.spectrim.main.MetaboliteFeatures;
import org.spectrim.main.ResultsStoreInterface;
import org.spectrim.main.VoxelMetabolites;
import org.spectrim.model.GlobalModel;
import org.spectrim.popups.jMRUIResultsMapListener;

/**
 * 
 * Container class that contains the results of the TDFDFit algorithm. 
 * 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class TDFDFitResultsStore implements ResultsStoreInterface, jMRUIResultsMapListener
{
    
    private HashMap<String, VoxelMetabolites> voxels = new HashMap<>( ); // voxel                                                                                                                 // -                                                                                                                       // voxelsmetaboites
    //private ArrayList<String> metabs = new ArrayList<>( );
    private GlobalModel m_globalModel;
    private String[] m_metaboliteNames = null;
    private boolean m_modelNamesDefined = false;
    private HashMap<String, Double> m_allFittedWaterReferenceResonanceFrequencies;
    
    HashMap<String, NumericTDFDFitResults> m_tdfdfitResults;
    
    /**
     * Default constructor.
     * @param globalModel
     */
    public TDFDFitResultsStore( GlobalModel globalModel ) 
    {
        m_globalModel = globalModel;
    }

    /**
     * Adds the results from TDFDfit to the current Hashmap.
     * @param tdfdfitResults
     * @param errorMap
     * @param noiseStdMap
     */
    @Override
    public void addTDFDFitResultsData( HashMap<String, NumericTDFDFitResults> tdfdfitResults,
                                       HashMap<String, float[]> errorMap,
                                       HashMap<String, Float> noiseStdMap ) 
    {
        MRSignalProperties currentMRSignalProperties = MRSignalProperties.getInstance( );
        
        if ( tdfdfitResults != null ) 
        {
                 
//            double step = 1.0/currentMRSignalProperties.getAcquistitionBandwidth( );
//            double tf = currentMRSignalProperties.getResonanceFrequency( );

            for ( String key : tdfdfitResults.keySet( ) ) 
            {
                NumericTDFDFitResults tdfdfitResult = tdfdfitResults.get( key );
                String[] coord = key.split( "_" );
                String id = ( Integer.valueOf( coord[0] ) - 1 ) + "_"
                          + ( Integer.valueOf( coord[1] ) - 1 ) + "_"
                          + ( Integer.valueOf( coord[2] ) - 1 );

                VoxelMetabolites metabolites = new VoxelMetabolites( );
                metabolites.setGlobalError( -1.0 );
                Vector<TDFDFittingParameters> fittingParametersVector = tdfdfitResult.getFittedParametersVector( );
                
                m_metaboliteNames = new String[fittingParametersVector.size( )];
                int cntr = 0;
                Iterator it = fittingParametersVector.iterator( );
                while ( it.hasNext( ) )
                {
                    TDFDFittingParameters fitPar = (TDFDFittingParameters)it.next( );
                    MetaboliteFeatures metab = new MetaboliteFeatures( );
                    metab.setArea( Math.abs( fitPar.getAmplitude( ) ) );
                    metab.setAmplitude( fitPar.getAmplitude( ) );
                    metab.setFrqShift( fitPar.getOffset( ) );
                    metab.setDamping( fitPar.getLwidth( ) );
                    metab.setGaussianDamping( fitPar.getGwidth( ) );
                    metab.setPhase( fitPar.getPhase( ) );
                    metab.setStdDevArea( fitPar.getE_Amplitude( ) );
                    metab.setError( -1.0 );
                    metab.setFitTDFDFitQualityNumber( tdfdfitResult.getFitQualityNumber( ) );
                    metabolites.addMetabolite( fitPar.getComponentName( ), metab );
                    m_metaboliteNames[cntr] = fitPar.getComponentName( );
                    cntr++;
                }
                voxels.put( key, metabolites );
                m_modelNamesDefined = true;
            }
        }
        m_tdfdfitResults = tdfdfitResults;
    }
    
    /**
     * Clears the results map.
     */
    @Override
    public void clearData( ) 
    {
        voxels = new HashMap<>( );
        m_metaboliteNames = null;
    }

    /**
     * Removes the voxels from the ArrayList.
     * @param voxels 
     */
    @Override
    public void removeAllVoxels( ArrayList<String> voxels ) 
    {
        for ( String voxel : voxels )
        {
            removeVoxel( voxel );
        }
    }
    
    /**
     * Removes specific voxel from voxels belonging voxelName. 
     * @param voxelName
     */
    @Override
    public void removeVoxel( String voxelName ) 
    {
        System.out.println( "removing voxel" );
        if ( voxels.containsKey( voxelName ) )
        {
            voxels.remove( voxelName );
        }
    }

    /**
     * Returns the metabolites and their features at the given coordinate.
     * @param coord
     * @return 
     */
    @Override
    public VoxelMetabolites getMetabolitesAt( String coord ) 
    {
        if ( voxels.containsKey( coord ) )
        {
            return voxels.get( coord );
        }
        else
        {
            return null;
        }
    }

    /**
     * Set the metabolites "voxMetabs" of voxel at "coord".
     * @param coord
     * @param voxMetabs 
     */
    public void setMetabolitesAt( String coord, VoxelMetabolites voxMetabs ) 
    {
        voxels.put( coord, voxMetabs );
    }
    

    /**
     * Returns all the names of the available metabolites.
     * @return 
     */
    @Override
    public String[] getAvailableMetabolites( ) 
    {
//        String[] m = new String[metabs.size()];
//        for ( int i = 0; i < m.length; i++ ) 
//        {
//            m[i] = metabs.get( i );
//        }
        return m_metaboliteNames;
    }

    /**
     * Returns the given feature for the given coordinates and the given
     * metabolite; NaN if not existing.
     * @param coord
     * @param metab
     * @param feature
     * @return 
     */
    @Override
    public double getFeature( String coord, String metab, int feature ) 
    {
        // voxels.get( coord ).getFeatures( metab ).getFeature( feature );
        return voxels.get( coord ).getFeatures( metab ).getFeature( feature );
    }

    /**
     * Returns a featureMap containing all the values for the given feature and
     * metabolite.
     * @param metab
     * @param feature
     * @return 
     */
    @Override
    public HashMap<String, Double> getFeatureMap( String metab, int feature ) 
    {
        HashMap<String, Double> map = new HashMap<>( );
        for ( Map.Entry<String, VoxelMetabolites> entrySet : voxels.entrySet( ) ) 
        {
            try 
            {
                map.put( entrySet.getKey( ),
                         entrySet.getValue( ).getFeatures( metab ).getFeature( feature ) );
            } 
            catch ( Exception e ) 
            {
                map.put( entrySet.getKey( ), Double.NaN );
            }
        }
        return map;
    }

    /**
     * Returns a Map containing the global fitting error
     * @return 
     */
    @Override
    public HashMap<String, Double> getGlobalErrorMap( ) 
    {
        HashMap<String, Double> map = new HashMap<>( );
        for ( Map.Entry<String, VoxelMetabolites> entrySet : voxels.entrySet( ) ) 
        {
            try 
            {
                map.put( entrySet.getKey( ), entrySet.getValue( ).getGlobalError( ) );
            } 
            catch ( Exception e ) 
            {
                map.put( entrySet.getKey( ), Double.NaN );
            }
        }
        return map;
    }

    /**
     * Returns the map with metabolites and features.
     * @return 
     */
    @Override
    public HashMap<String, VoxelMetabolites> getVoxelsMetabolites( ) 
    {
        return voxels;
    }

    /**
     * Returns the list of the voxels as ArrayList.
     * @return 
     */
    @Override
    public ArrayList<String> getAvailableVoxels( ) 
    {
        return new ArrayList<String>( voxels.keySet( ) );
    }

    @Override
    public void setColorMapfromjMRUIResults( int metab, 
                                             int feature, 
                                             boolean showJustSelected, 
                                             String name ) 
    {
       if ( m_globalModel != null ) 
       {
            m_globalModel.clearColorMap( );
            HashMap<String, Double> mapValues = this.getFeatureMap(
                            getMetabolitesNames( )[metab], feature );
            m_globalModel.getColorMap( ).setColorMapValues( mapValues,
                            showJustSelected, name );
	}
    }
    
    @Override
    public void setColorMapfromjMRUIResultsRatio( int metab1, int feature1, 
                                                  int metab2, int feature2, 
                                                  boolean showJustSelected, 
                                                  String name ) 
    {
        if ( m_globalModel != null ) 
        {
            m_globalModel.clearColorMap( );
            HashMap<String, Double> mapValues1 = this.getFeatureMap(
                            getMetabolitesNames( )[metab1], feature1 );
            HashMap<String, Double> mapValues2 = this.getFeatureMap(
                            getMetabolitesNames( )[metab2], feature2 );
            HashMap<String, Double> mapValues = new HashMap<>( );
            for ( Map.Entry<String, Double> entrySet : mapValues1.entrySet( ) ) 
            {
                if ( mapValues2.containsKey( entrySet.getKey( ) ) ) 
                {
                    double value = entrySet.getValue( )
                                    / mapValues2.get( entrySet.getKey( ) );
                    mapValues.put( entrySet.getKey( ),
                                    Double.isInfinite( value ) ? Double.NaN : value );
                }
            }

            m_globalModel.getColorMap( ).setColorMapValues( mapValues,
                                                            showJustSelected, 
                                                            name );
        }
    }

    @Override
    public void setColorMapfromjMRUIResultsFormula(
                    ArrayList<Integer> numerator, ArrayList<Integer> denominator,
                    int feature, boolean showJustSelected ) 
    {
        if ( m_globalModel != null && (!numerator.isEmpty( )|| !denominator.isEmpty( ) ) ) 
        {
            m_globalModel.clearColorMap();
            ArrayList<HashMap<String, Double>> numerators = new ArrayList<>();
            ArrayList<HashMap<String, Double>> denominators = new ArrayList<>();
            for (Integer i : numerator) 
            {
                numerators.add( this.getFeatureMap(getMetabolitesNames()[i],
                                feature));
            }
            for (Integer i : denominator) 
            {
                denominators.add( this.getFeatureMap(getMetabolitesNames()[i],
                feature));
            }
            Set<String> coords;
            if (denominators.isEmpty()) 
            {
                coords = ((HashMap<String, Double>) numerators.get(0)).keySet();
            } 
            else 
            {
                coords = ((HashMap<String, Double>) denominators
                                    .get(0)).keySet();
            }
            HashMap<String, Double> mapValues = new HashMap<>();
            for (String coord : coords) 
            {
                double num;
                double denom;

                num = numerators.size() > 0 ? 0 : 1;
                denom = denominators.size() > 0 ? 0 : 1;

                for (int i = 0; i < numerators.size(); i++) 
                {
                    num += numerators.get(i).get(coord);
                }
                for (int i = 0; i < denominators.size(); i++) 
                {
                    denom += denominators.get(i).get(coord);
                }
                mapValues.put(coord, num / denom);
            }

            m_globalModel.getColorMap().setColorMapValues(mapValues,
                            showJustSelected, "formula");
        }

    }

    @Override
    public void setColorMapfromjMRUIResultsTriChannel(int[] metabR,
			int[] featureR, int[] metabG, int[] featureG, int[] metabB,
			int[] featureB, boolean showJustSelected) 
    {
        if ( m_globalModel != null) 
        {
            m_globalModel.clearColorMap();
            LinkedHashMap<String, double[]> mapValues = new LinkedHashMap<>();

            HashMap<String, Double> mapValuesR = this.getFeatureMap(
                            getMetabolitesNames()[metabR[0]], featureR[0]);
            HashMap<String, Double> mapValuesG = this.getFeatureMap(
                            getMetabolitesNames()[metabG[0]], featureG[0]);
            HashMap<String, Double> mapValuesB = this.getFeatureMap(
                            getMetabolitesNames()[metabB[0]], featureB[0]);

            HashMap<String, Double> mapValuesR1 = null;
            HashMap<String, Double> mapValuesG1 = null;
            HashMap<String, Double> mapValuesB1 = null;

            if (metabR[1] != -1)
                mapValuesR1 = this.getFeatureMap(
                                getMetabolitesNames()[metabR[1]], featureR[1]);

            if (metabG[1] != -1)
                mapValuesG1 = this.getFeatureMap(
                                getMetabolitesNames()[metabG[1]], featureG[1]);

            if (metabB[1] != -1)
            {
                mapValuesB1 = this.getFeatureMap(
                                getMetabolitesNames()[metabB[1]], featureB[1]);
            }

            for ( String coord : mapValuesR.keySet( ) ) 
            {
                double[] val = new double[3];
                
                if (!mapValuesG.containsKey(coord)
                                || !mapValuesB.containsKey(coord))
                {
                        break;
                }
                
                val[0] = mapValuesR.get(coord);
                val[1] = mapValuesG.get(coord);
                val[2] = mapValuesB.get(coord);

                if (metabR[1] != -1) 
                {
                    if (!mapValuesR1.containsKey(coord))
                    {
                        break;
                    }
                    val[0] /= mapValuesR1.get(coord);
                }

                if (metabG[1] != -1) 
                {
                    if (!mapValuesG1.containsKey(coord))
                    {
                        break;
                    }
                    val[1] /= mapValuesG1.get(coord);
                }

                if (metabB[1] != -1) 
                {
                    if (!mapValuesB1.containsKey(coord))
                    {
                        break;
                    }
                    val[2] /= mapValuesB1.get(coord);
                }
                mapValues.put(coord, val);
            }
            m_globalModel.getColorMap( ).setColorMapValuesTriChannel( mapValues,
                                                                      showJustSelected );
        }
    }

    public void printTDFDFitResults( ) 
    {
        processTDFDFitResults( false, false );
    }

    public void printTDFDFitResultsSelectedVoxelsToScreen( ) 
    {
        if ( m_globalModel != null ) 
        {
            String txtaux = "\nCoord\tMetabolite";
            MetaboliteFeatures dummy = new MetaboliteFeatures( );
            String[] featNames = dummy.getFeatureNames( );
            for ( String name : featNames ) 
            {
                txtaux = txtaux + "\t" + name;
            }
            
            m_globalModel.printTxt( txtaux );
            String txt = "\n";
            
            for ( Map.Entry<String, VoxelMetabolites> v :
                            this.getVoxelsMetabolites( ).entrySet( ) ) 
            {
                if ( m_globalModel.getSelectedVoxels( ).contains( v.getKey( ) ) ) 
                {
                    for ( Map.Entry<String, MetaboliteFeatures> mf :
                                    v.getValue( ).getMetabFeatures( ).entrySet( ) ) 
                    {
                        txt += v.getKey() + "\t" + mf.getKey( ) + "\t";
                        for ( int i = 0; i < mf.getValue( ).getNbFeatures( ); i++ ) 
                        {
                            Double featVal = mf.getValue( ).getFeature( i );
                            if ( featVal == null || featVal.isNaN( ) )
                            {
                                txt += "-" + "\t";
                            }
                            else if( featVal.isInfinite( ) )
                            {
                                txt += "Inf" + "\t";
                            }
                            else
                            {
                                txt += featVal + "\t";
                            }
                        }
                        txt += mf.getValue( ).getFitTDFDFitQualityNumber( );
                        txt += "\n";
                    }
                }
            }
            m_globalModel.printNumeric( txt, true );
        }
    }
    
    /**
     * Writes the TDFDFitting results to a text file.
     */
    public void writeTDFDFitResultsSelectedVoxelsToTextFile( ) 
    {
        
        if ( m_globalModel.getCSIData( ) != null )
        {
            this.writeTDFDFitResults( );
        } 
        else
        {
            JOptionPane.showMessageDialog( null, "Cannot write TDFDFit fitted data to text file: \nFirst load an MRS(I) dataset, and quantify it!" );
        }       
    }
    
    public void writeTDFDFitAverageResultsToFile( String file )
    {
        if ( m_globalModel != null ) 
        {
            try 
            {
                TextFileWriter fileWriter;
                
                fileWriter = new TextFileWriter( new FileWriter( file ) );
                                
                String txtaux = "\nCoord\tMetabolite";
                MetaboliteFeatures dummy = new MetaboliteFeatures();
                String[] featNames = dummy.getFeatureNames();
                for (String name : featNames) 
                {
                    txtaux = txtaux + "\t" + name;
                } 

                fileWriter.writeStringLn( txtaux );
                
                HashMap<String, MetaboliteFeatures> averageMap = new HashMap<>( );
                
                int counter = 0;
                HashMap<String,MetaboliteFeatures> sumOfAllFeaturesMap = new HashMap<>( );
                
                for ( Map.Entry<String, VoxelMetabolites> v :
                        this.getVoxelsMetabolites( ).entrySet( ) )
                {
                    
                    if ( m_globalModel.getSelectedVoxels( ).contains( v.getKey( ) ) )
                    {
                        
                        for ( Map.Entry<String, MetaboliteFeatures> mf : 
                              v.getValue( ).getMetabFeatures( ).entrySet( ) )
                        {
                            
                            if ( counter == 0 )
                            {
                                // First simply fill the map, no average needed the first time
                                sumOfAllFeaturesMap.put( mf.getKey( ), mf.getValue( ) );
                            }
                            else
                            {
                                 MetaboliteFeatures avFeat = sumOfAllFeaturesMap.get( mf.getKey( ) );
                                 avFeat.add( mf.getValue( ) );
                                 sumOfAllFeaturesMap.put( mf.getKey( ), avFeat );
                            }
                        }
                        counter++;
                    }                   
                }
                
                // Now write the averages to a file.
                
                Iterator it = sumOfAllFeaturesMap.keySet( ).iterator( );
                
                String one_text_line;
                
                while ( it.hasNext( ) )
                {    
                    String key = (String)it.next( );
                    MetaboliteFeatures metabFeat = sumOfAllFeaturesMap.get( key ).getAverageFeatures( );
                    one_text_line = "av." + counter + "voxels\t" + key + "\t\t";
                    
                    for ( int i = 0; i < metabFeat.getNbFeatures( ); i++ )
                    {
                        Double featVal = metabFeat.getFeature( i );
                        
                        if ( featVal == null || featVal.isNaN( ) )
                        {
                            one_text_line += "-" + "\t";
                        }
                        else if( featVal.isInfinite( ) )
                        {
                            one_text_line += "Inf" + "\t";
                        }
                        else
                        {
                            one_text_line += featVal + "\t";
                        }
                    }

                    one_text_line += "\n";
                    fileWriter.writeStringLn( one_text_line );
                }
                
                fileWriter.writeStringLn( "" );

                fileWriter.close( );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        else
        {
            JOptionPane.showMessageDialog( null, "Cannot write TDFDFit fitted data to text file: \nFirst load an MRS(I) dataset, and quantify it!" );
        } 
    }
    
    /**
     * This method writes the TDFDFitReesultsStore to a file, which can later on
     * being be loaded and analyzed again.
     * @param file 
     */
    public void writeTDFDFitResultsToFile( String file )
    {
        if ( m_globalModel != null ) 
        {
            
            try 
            {
                TextFileWriter fileWriter = new TextFileWriter( new FileWriter( file ) );
                
                String txtaux = "Coord\tMetabolite";
                MetaboliteFeatures dummy = new MetaboliteFeatures( );
                String[] featNames = dummy.getFeatureNames();
                for (String name : featNames) 
                {
                    txtaux = txtaux + "\t" + name;
                } 
                
                // m_globalModel.printTxt( txtaux );
                
                // fileWriter.writeStringLn( txtaux );
                    
                for ( Map.Entry<String, VoxelMetabolites> v :
                      this.getVoxelsMetabolites( ).entrySet( ) )
                {
                    
                    fileWriter.writeStringLn( txtaux );
                    
                    if ( m_globalModel.getSelectedVoxels( ).contains( v.getKey( ) ) )
                    {
                        for ( Map.Entry<String, MetaboliteFeatures> mf : 
                              v.getValue( ).getMetabFeatures( ).entrySet( ) )
                        {
                           
                            String one_text_line = "";

                            one_text_line += v.getKey( ) + "\t" + mf.getKey( ) + "\t";
                            for (int i = 0; i < mf.getValue().getNbFeatures(); i++)
                            {
                                Double featVal = mf.getValue().getFeature(i);
                                if ( featVal == null || featVal.isNaN( ) )
                                {
                                    one_text_line += "-" + "\t";
                                }
                                else if( featVal.isInfinite( ) )
                                {
                                    one_text_line += "Inf" + "\t";
                                }
                                else
                                {
                                    one_text_line += featVal + "\t";
                                }
                            }

                            // one_text_line += "\n";
                            fileWriter.writeStringLn( one_text_line );
                        }
                        fileWriter.writeStringLn( "###" );
                    }
                }
                // m_globalModel.printNumeric( txt, true );
                fileWriter.close( );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        else
        {
            JOptionPane.showMessageDialog( null, "Cannot write TDFDFit fitted data to text file: \nFirst load an MRS(I) dataset, and quantify it!" );
        }
    }
    
     /**
     * This method reads the TDFDFitReesultsStore from a large results file.
     * The file is NOT an .exl file!
     * @param file 
     */
    public void readTDFDFitResultsFromFile( String file )
    {
        if ( m_globalModel != null ) 
        {
            
            BufferedReader bfr =  null;
            String first_line;
            String txtaux = "\nCoord\tMetabolite";
            try 
            {
                bfr = new BufferedReader( new FileReader( file ) );
                
                try 
                {
                    first_line = bfr.readLine( );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                
                
                MetaboliteFeatures dummy = new MetaboliteFeatures( );
                String[] featNames = dummy.getFeatureNames( );
                for ( String name : featNames )
                {
                    txtaux = txtaux + "\t" + name;
                }   boolean retVal = true;
                
                while ( retVal == true )
                {
                    NumericTDFDFitResults result = new NumericTDFDFitResults( );
                    try 
                    {
                        retVal = result.readNumericTDFDFitResultsFromBufferedReader( bfr );
                    } 
                    catch ( IOException ex ) 
                    {
                        Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                }
                        
                // m_globalModel.printTxt( txtaux );
                
                // fileWriter.writeStringLn( txtaux );
                    
//                for ( Map.Entry<String, VoxelMetabolites> v :
//                      this.getVoxelsMetabolites( ).entrySet( ) )
//                {
//                    
//                    fileWriter.writeStringLn( txtaux );
//                    
//                    if ( m_globalModel.getSelectedVoxels( ).contains( v.getKey( ) ) )
//                    {
//                        for ( Map.Entry<String, MetaboliteFeatures> mf : 
//                              v.getValue( ).getMetabFeatures( ).entrySet( ) )
//                        {
//                           
//                            String one_text_line = "";
//
//                            one_text_line += v.getKey( ) + "\t" + mf.getKey( ) + "\t";
//                            for (int i = 0; i < mf.getValue().getNbFeatures(); i++)
//                            {
//                                Double featVal = mf.getValue().getFeature(i);
//                                if ( featVal == null || featVal.isNaN( ) )
//                                {
//                                    one_text_line += "-" + "\t";
//                                }
//                                else if( featVal.isInfinite( ) )
//                                {
//                                    one_text_line += "Inf" + "\t";
//                                }
//                                else
//                                {
//                                    one_text_line += featVal + "\t";
//                                }
//                            }
//
//                            one_text_line += "\n";
//                            fileWriter.writeStringLn( one_text_line );
//                        }
//                        fileWriter.writeStringLn( "" );
//                        
//                    }
//                }
//                // m_globalModel.printNumeric( txt, true );
//                fileWriter.close( );
//            }
//            catch ( IOException ex ) 
//            {
//                Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
//            }
            } 
            catch ( FileNotFoundException ex ) 
            {
                Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
            } 
            finally 
            {
                try 
                {
                    bfr.close( );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
        }
    }
    
    
    
    @Override
    public void printJMRUIResultsSelectedVoxels( ) 
    {
        if ( m_globalModel != null ) 
        {
            String txtaux = "\nCoord\tMetabolite";
            MetaboliteFeatures dummy = new MetaboliteFeatures();
            String[] featNames = dummy.getFeatureNames();
            for (String name : featNames) 
            {
                txtaux = txtaux + "\t" + name;
            }
            
            m_globalModel.printTxt( txtaux );
            String txt = "\n";
            
            for ( Map.Entry<String, VoxelMetabolites> v :
                            this.getVoxelsMetabolites( ).entrySet( ) ) 
            {
                if ( m_globalModel.getSelectedVoxels( ).contains( v.getKey() ) ) 
                {
                    for ( Map.Entry<String, MetaboliteFeatures> mf :
                                    v.getValue( ).getMetabFeatures( ).entrySet( ) ) 
                    {
                        txt += v.getKey() + "\t" + mf.getKey() + "\t";
                        for (int i = 0; i < mf.getValue().getNbFeatures(); i++) 
                        {
                            Double featVal = mf.getValue().getFeature(i);
                            if ( featVal == null || featVal.isNaN( ) )
                            {
                                txt += "-" + "\t";
                            }
                            else if( featVal.isInfinite( ) )
                            {
                                txt += "Inf" + "\t";
                            }
                            else
                            {
                                txt += featVal + "\t";
                            }
                        }
                        txt += "\n";
                    }
                }
            }
            m_globalModel.printNumeric( txt, true );
        }
    }

    @Override
    public String[] getMetabolitesNames( ) 
    {       
        return m_metaboliteNames;
    }
    
    /**
     * Returns the names of the metabolite features quantified by the algorithm.
     * @return 
     */
    @Override
    public String[] getFeatureNames( ) 
    {
        MetaboliteFeatures dummy = new MetaboliteFeatures( );
        String[] featNames = dummy.getFeatureNames( );
        return featNames;
    }

    /**
     * returns the area, or variance, for each metabolite for the voxel
     * selection
     * @param voxels
     * @param var
     * @return 
     */
    @Override
    public LinkedHashMap<String, Double> getMetabolitesPerSelection(
                                 ArrayList<String> voxels, boolean... var ) 
    {
        // if var.length > 0 returns the variance
        LinkedHashMap<String, Double> mapValues = new LinkedHashMap<>();
        for ( String m : getMetabolitesNames( ) ) 
        {
            double val = 0;
            int count = 0;
            for ( Map.Entry<String, Double> entrySet : this.getFeatureMap( m,
                            MetaboliteFeatures.AMPLITUDE).entrySet( ) ) 
            {
                if (voxels.contains(entrySet.getKey())
                                && !Double.isNaN(entrySet.getValue())) {
                    val += entrySet.getValue();
                    count++;
                }
            }
            val /= (double) count;

            if (var.length != 0) 
            {
                double std = 0.0;
                count = 0;
                for ( Map.Entry<String, Double> entrySet : this.getFeatureMap( m,
                                     MetaboliteFeatures.AMPLITUDE).entrySet( ) ) 
                {
                    if ( voxels.contains( entrySet.getKey( ) )
                    &&   !Double.isNaN(entrySet.getValue( ) ) ) 
                    {
                        std += Math.pow((entrySet.getValue() - val), 2);
                        count++;
                    }
                }
                val = std / (double) count;
            }
            mapValues.put( m, val );
        }
        return mapValues;
    }

    @Override
    public ColorMap getColorMap( ) 
    {
       return m_globalModel.getColorMap( );
    }

    @Override
    public GlobalModel getModel( ) 
    {
        return m_globalModel;
    }

    public void setModel( GlobalModel model ) 
    {
        m_globalModel = model;
    }
    
    @Override
    public String getISPOutputFileName( ) 
    {
        return null;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // non overwrite methods
    ////////////////////////////////////////////////////////////////////////////
    private String processTDFDFitResults( boolean isWritingModeActive,
                                          boolean appendMetabolicFeatures ) 
    {
        // Data data = mrui.getData();
        String txt = "";
        String header = "";
        int nrMetabs = this.getAvailableMetabolites( ).length;
        if ( !isWritingModeActive ) 
        {
            String txtaux = "Coord\tMetabolite";
            MetaboliteFeatures dummy = new MetaboliteFeatures( );
            String[] featNames = dummy.getFeatureNames( );
            for ( String name : featNames ) 
            {
                txtaux += "\t" + name;
            }
            m_globalModel.printTxt(txtaux);
        }
        else 
        {
            header = "Patient\tStudy\tCoord\t";
            for ( int k = 0; k < nrMetabs; k++ ) 
            {
                String metabName = this.getAvailableMetabolites()[k];
                MetaboliteFeatures dummy = new MetaboliteFeatures( );
                String[] featNames = dummy.getFeatureNames();
                for ( String name : featNames ) 
                {
                    header = header + "\t" + metabName + "-" + name;
                }
                if ( k < nrMetabs - 1 || appendMetabolicFeatures ) 
                {
                    header += "\t";
                }
            }
            if ( appendMetabolicFeatures ) 
            {
                ArrayList<String> featnames = m_globalModel.getSpectra( )
                                             .getMetabolicFeaturesNames( );
                for ( int j = 0; j < featnames.size( ); j++ ) 
                {
                    header += featnames.get( j ) + "\t";
                }
            }
        }
        for ( Map.Entry<String, VoxelMetabolites> v : this.getVoxelsMetabolites( ).entrySet( ) ) 
        {
            if ( isWritingModeActive ) 
            {
                txt += m_globalModel.getPatientName( ) + "\t"
                    + m_globalModel.getStudyDate( ) + "\t"
                    + v.getKey( ) + "\t";
            }
            for ( Map.Entry<String, MetaboliteFeatures> mf : v.getValue( ).getMetabFeatures( ).entrySet( ) ) 
            {
                if ( !isWritingModeActive ) 
                {
                    txt += v.getKey( ) + "\t"+ mf.getKey( ) + "\t";
                }
                for ( int i = 0; i < mf.getValue( ).getNbFeatures( ); i++ ) 
                {
                    Double featVal = mf.getValue( ).getFeature( i );
                    if ( featVal == null || featVal.isNaN( ) )
                    {
                        txt += "-" + "\t";
                    }
                    else if ( featVal.isInfinite( ) )
                    {
                        txt += "Inf" + "\t";
                    }
                    else
                    {
                        txt += featVal + "\t";
                    }
                }
                if ( !isWritingModeActive ) 
                {
                    if ( appendMetabolicFeatures ) 
                    {
                        ArrayList<Double> featvals = m_globalModel
                                          .getSpectra( )
                                          .getMetabolicFeatures( getSpectrumID( v.getKey( ) ) );
                        for ( int j = 0; j < featvals.size( ) - 1; j++ ) 
                        {
                            txt += featvals.get( j ) + "\t";
                        }
                        txt += featvals.get( featvals.size( ) - 1) + "\n";
                    } 
                    else 
                    {
                        txt += "\n";
                    }
                }
            }
            if ( isWritingModeActive )
            {
                txt += "\n";
            }
        }
        if ( !isWritingModeActive ) 
        {
            m_globalModel.printNumeric( txt, true );
        } 
        else 
        {
            txt = header + "\n" + txt;
        }
        return txt;
    }
    
    /**
     * The spectrum ID is at the same time also the coordinate of 
     * the voxel as string.
     * @param name
     * @return 
     */
    private String getSpectrumID( String name ) 
    {
        String[] coord = name.split("_");
        String indexes = ( Integer.valueOf( coord[0] ) - 1 ) + "_"
                       + ( Integer.valueOf( coord[1] ) - 1 ) + "_"
                       + ( Integer.valueOf( coord[2] ) - 1 );
        return indexes;
    }

    
    /**
     * 
     * Sets the list of water reference line offsets (aiming to compute
     * temperature map.
     * 
     * @param allFittedWaterReferenceResonanceFrequencies 
     */
    public void setWaterReferenceOffsets( HashMap<String, Double> allFittedWaterReferenceResonanceFrequencies ) 
    {
        m_allFittedWaterReferenceResonanceFrequencies = allFittedWaterReferenceResonanceFrequencies;
    }
  // why did I implement this?
    /**
     * Utility function to get area parameters of current store as 3D array.
     * @param metaboliteName
     * @param feature
     * @return 
     */
    public double[][][] getFittingParameters( String metaboliteName, 
                                              int feature )
    {
        int nd1; 
        int nd2; 
        int nd3;
        
        nd1 = m_globalModel.getGridDim1( );
        nd2 = m_globalModel.getGridDim2( );
        nd3 = m_globalModel.getGridDim3( );
        
        double[][][] fittingParameters = new double[nd1][nd2][nd3];
           
        HashMap<String, Double> map = new HashMap<>( );
        for ( Map.Entry<String, VoxelMetabolites> entrySet : voxels.entrySet( ) ) 
        {
            try 
            {
                String[] coord = entrySet.getKey( ).split( "_" );
                
                int coord_1 = new Integer( coord[0] );
                int coord_2 = new Integer( coord[1] );
                int coord_3 = new Integer( coord[2] );
            
                fittingParameters[coord_1][coord_2][coord_3] = getFeature( entrySet.getKey( ), metaboliteName , feature );
            } 
            catch ( Exception e ) 
            {
                String[] coord = entrySet.getKey( ).split( "_" );
                
                int coord_1 = new Integer( coord[0] );
                int coord_2 = new Integer( coord[1] );
                int coord_3 = new Integer( coord[2] );
                
                fittingParameters[coord_1][coord_2][coord_3] = Double.NaN;
            }
            
            
        }
        return fittingParameters;  
    }

    public void writeTDFDFitResults( ) 
    {
        JFileChooser fileChooser = new JFileChooser( );
        fileChooser.setDialogTitle( "Set file" );
        File currentFolderFeats = new File( System.getProperty( "user.home" ) );
        fileChooser.setCurrentDirectory(currentFolderFeats);
        fileChooser.setSelectedFile( new File( m_globalModel.getCSIData( ).getPatientName( ).replaceAll( "�","_" ) + "_" + 
                                               m_globalModel.getCSIData( ).getStudyDate( ) + "_" +
                                               m_globalModel.getCSIData( ).getProtocolName( ) + "_" + 
                                               m_globalModel.getCSIData( ).getSeriesNumber( ) + ".txt" ) );
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "text", "txt" );
        fileChooser.setFileFilter(filter);
        int result = fileChooser.showOpenDialog( null );
        File selected_file;
        switch ( result ) 
        {
            case JFileChooser.APPROVE_OPTION: // Approve Open button was pressed
                                    selected_file = fileChooser.getSelectedFile();
                                    currentFolderFeats = fileChooser.getCurrentDirectory( );
                                    if ( selected_file != null ) 
                                    {
                                        writeTDFDFitResultsToFile( selected_file.getAbsolutePath( ) );
                                    }
                                    break;
            case JFileChooser.CANCEL_OPTION: // Cancel or close dialog was selected
                break;
            case JFileChooser.ERROR_OPTION: // the selected process did not
                                        // terminate properly
                break;
        }
    }   

    public void writeTDFDFitAverageResults( ) 
    {
        m_globalModel.getPatientName();
        JFileChooser fileChooser = new JFileChooser( );
        fileChooser.setDialogTitle( "Define file name average TDFDFit results" );
        File currentFolderFeats = new File( System.getProperty( "user.home" ) );
        fileChooser.setCurrentDirectory( currentFolderFeats );
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "text", "txt" );
        fileChooser.setFileFilter( filter );
        //if ( m_globalModel.getCSIData( ).getManufacturer( ).contains("Siemens") )
        fileChooser.setSelectedFile( new File( m_globalModel.getCSIData( ).getPatientName( ).replaceAll( "�","_" ) + "_" + 
                                               m_globalModel.getCSIData( ).getStudyDate( ) + "_" +
                                               m_globalModel.getCSIData( ).getProtocolName( ) + "_" + 
                                               m_globalModel.getCSIData( ).getSeriesNumber( ) + ".txt" ) );
        
        int result = fileChooser.showOpenDialog( null );
        File selected_file;
        switch ( result ) 
        {
            case JFileChooser.APPROVE_OPTION: // Approve Open button was pressed
                                    selected_file = fileChooser.getSelectedFile();
                                    currentFolderFeats = fileChooser.getCurrentDirectory( );
                                    if ( selected_file != null ) 
                                    {
                                        writeTDFDFitAverageResultsToFile( selected_file.getAbsolutePath( ) );
                                    }
                                    break;
            case JFileChooser.CANCEL_OPTION: // Cancel or close dialog was selected
                break;
            case JFileChooser.ERROR_OPTION: // the selected process did not
                                        // terminate properly
                break;
        }
    }
    
    /**
     * Write to one huge file that makes fitting data persistent.
     * @param dos
     * @throws IOException 
     */
   private void writeAllParametricData( DataOutputStream dos ) throws IOException
    {
        
        NumericTDFDFitResults nullResult = null;
        
        // Strategy is to write model parameters for all voxels
        // Step 1. find a non empty and fitted NumericTDFDFitResults object.
        for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1 ; i++ )
        {
            for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2 ; j++ )
            {
                for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3 ; k++ )
                {
                    String key = "" + i + "_" + j + "_" + k;
                    NumericTDFDFitResults result = m_tdfdfitResults.get( key );
                    
                    if ( ( result != null ) && ( nullResult == null ) )
                    {
                        nullResult = new NumericTDFDFitResults( result );
                        nullResult.clearData( );
                    }
                }
            } 
        }
        
        for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1 ; i++ )
        {
            for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2 ; j++ )
            {
                for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3 ; k++ )
                {
                    String key = "" + i + "_" + j + "_" + k;
                    
                    dos.writeUTF( key );
                    
                    NumericTDFDFitResults result = m_tdfdfitResults.get( key );
                    
                    if ( result == null )
                    {
                        result = nullResult;                        
                    }
                                      
                    result.writeToDataOutputStream( dos );
                    
                }
            }
        }
    }
    
    
    /**
     * Write to one huge file that makes fitting data persistent.
     * @param dos
     * @throws IOException 
     */
    private void readAllParametricData( DataInputStream dis ) throws IOException
    {
        for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1 ; i++ )
        {
            for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2 ; j++ )
            {
                for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3 ; k++ )
                {
                    String key = "" + i + "_" + j + "_" + k;
                    
                    String keyFromFile = dis.readUTF( );
                    
                    if ( key.compareTo( keyFromFile ) != 0 )
                    {
                        System.out.println( "Warning:  Unexpected signal key object : " + keyFromFile  + "  (expected key : " + key );
                    }
                    NumericTDFDFitResults result = new NumericTDFDFitResults( );
                    result.readFromDataInputStream( dis );
                    m_tdfdfitResults.put( keyFromFile, result );
                }
            }
        }
    }
    
    
    /**
     * Method returning all signal files from NumericTDFDFitResults as single 
     * float array. 
     * @return 
     */
    private float[] getAllFittedModelSignals( )
    {
        
        int size = m_globalModel.getCSIGrid( ).m_gridSizeDir1 *
                   m_globalModel.getCSIGrid( ).m_gridSizeDir2 *
                   m_globalModel.getCSIGrid( ).m_gridSizeDir3 *
                   2 * MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
        
        float[] allComplexDotFitSignals = new float[size];
        int cntr = 0;
        
        for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1 ; i++ )
        {
            for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2 ; j++ )
            {
                for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3 ; k++ )
                {
                    String key = "" + i + "_" + j + "_" + k;
                    NumericTDFDFitResults result = m_tdfdfitResults.get( key );
                    if ( result != null )
                    {
                        float[] abs = result.getFittedSpectrumRealTD( );
                        float[] disp = result.getFittedSpectrumImagTD( );

                        // Copy the TD data into the global data array.
                        for ( int ii = 0; ii < MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ); ii++ )
                        {
                            allComplexDotFitSignals[cntr] = abs[ii];
                            cntr++;
                            allComplexDotFitSignals[cntr] = -disp[ii];
                            cntr++;
                        }
                    }
                    else
                    {
                        for ( int ii = 0; ii < MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ); ii++ )
                        {
                            allComplexDotFitSignals[cntr] = 0f;
                            cntr++;
                            allComplexDotFitSignals[cntr] = 0f;
                            cntr++;
                        }
                    }
                }
            }
        }
        return allComplexDotFitSignals;
    }

   /**
     * Method returning all signal files from NumericTDFDFitResults as single 
     * float array. 
     * @return 
     */
    private float[] getAllPreProcessedMeasuredSignals( )
    {
        
        int size = m_globalModel.getCSIGrid( ).m_gridSizeDir1 *
                   m_globalModel.getCSIGrid( ).m_gridSizeDir2 *
                   m_globalModel.getCSIGrid( ).m_gridSizeDir3 *
                   2 * MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
        
        float[] allComplexDotBinSignals = new float[size];
        int cntr = 0;
        
        for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1 ; i++ )
        {
            for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2 ; j++ )
            {
                for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3 ; k++ )
                {
                    String key = "" + i + "_" + j + "_" + k;
                    
                    NumericTDFDFitResults result = m_tdfdfitResults.get( key );
                    
                    if ( ( result != null ) && ( result.getPreprocessedInputSignalRealTD( ) != null ) )
                    {
                        
                        float[] abs = result.getPreprocessedInputSignalRealTD( );
                        float[] disp = result.getPreprocessedInputSignalImagTD( );

                        // Copy the TD data into the global data array.
                        for ( int ii = 0; ii < MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ); ii++ )
                        {
                            allComplexDotBinSignals[cntr] = abs[ii];
                            cntr++;
                            allComplexDotBinSignals[cntr] = -disp[ii];
                            cntr++;
                        }
                    }
                    else
                    {
                        for ( int ii = 0; ii < MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ); ii++ )
                        {
                            allComplexDotBinSignals[cntr] = 0f;
                            cntr++;
                            allComplexDotBinSignals[cntr] = 0f;
                            cntr++;
                        }
                    }
                }
            }
        }
        return allComplexDotBinSignals;
    }

    
    /**
     * Reads the TDFDFitResultsStore from the files system.
     */
    public boolean readTDFDFitResultsStore( )
    {
        String currentDCMFileRef = m_globalModel.getCSIData( ).getFile( );
        String inputOverallResultFile = null;
        
        int numberOfSpectra = m_globalModel.getCSIGrid( ).m_gridSizeDir1 *
                              m_globalModel.getCSIGrid( ).m_gridSizeDir2 *
                              m_globalModel.getCSIGrid( ).m_gridSizeDir3;
        
        // Declare a data container object
        m_tdfdfitResults = new HashMap<String, NumericTDFDFitResults>( );
        
        int floatSize = numberOfSpectra *
                        2 * m_globalModel.getSpectraVectorSize( );
        
        float[] allSignalFloats = new float[floatSize];
        float[] allPreprocessedSignalFloats = new float[floatSize];
        
        if ( currentDCMFileRef.length( ) > 0 )
        {
            int index = currentDCMFileRef.lastIndexOf( "." );
            if ( ( index > 0 ) && ( index <= currentDCMFileRef.length( ) ) )
            {
                inputOverallResultFile = currentDCMFileRef.substring( 0, index );
            }
            else
            {
                inputOverallResultFile = currentDCMFileRef;
            }
            inputOverallResultFile = inputOverallResultFile + ".tfr";
        }
        
        try ( DataInputStream dis = new DataInputStream( new FileInputStream( inputOverallResultFile ) ) ) 
        {
            
            byte[] buffer = new byte[4*floatSize];
            
            // first read all fitted model signal data
            dis.read( buffer );
            for ( int i = 0; i < floatSize; i++ )
            {
                byte[] floatAsByteArray = new byte[4];
                
                floatAsByteArray[0] = buffer[4*i];
                floatAsByteArray[1] = buffer[4*i+1];
                floatAsByteArray[2] = buffer[4*i+2];
                floatAsByteArray[3] = buffer[4*i+3];
                
                allSignalFloats[i] = byteArrayToFloat( floatAsByteArray );
            }
            
            // Read the preprocessed data
            dis.read( buffer );
            
            for ( int i = 0; i < floatSize; i++ )
            {
                byte[] preprocessedfloatAsByteArray = new byte[4];
                
                preprocessedfloatAsByteArray[0] = buffer[4*i];
                preprocessedfloatAsByteArray[1] = buffer[4*i+1];
                preprocessedfloatAsByteArray[2] = buffer[4*i+2];
                preprocessedfloatAsByteArray[3] = buffer[4*i+3];
                
                allPreprocessedSignalFloats[i] = byteArrayToFloat( preprocessedfloatAsByteArray );
            }
            // Create the NumericTDFDFitResults objects and fill them with
            // the just read signal data.
            int cntr = 0;
            for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1; i++ )
            {
                for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2; j++ )
                {
                    for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3; k++ )
                    {
                        
                        String key = "" + i + "_" + j + "_" + k;
                        
                        float[] abs  = new float[ m_globalModel.getSpectraVectorSize( ) ];
                        float[] disp = new float[ m_globalModel.getSpectraVectorSize( ) ];
            
                        float[] absPrepro  = new float[ m_globalModel.getSpectraVectorSize( ) ];
                        float[] dispPrepro = new float[ m_globalModel.getSpectraVectorSize( ) ];
                        
                        
                        NumericTDFDFitResults tdfdfitResult = new NumericTDFDFitResults( );
                        
                        for ( int kk = 0; kk < m_globalModel.getSpectraVectorSize( ); kk++ )
                        {
                            abs[kk]  = allSignalFloats[ ( cntr * 2 * m_globalModel.getSpectraVectorSize( ) ) + 2 * kk];
                            disp[kk] = allSignalFloats[ ( cntr * 2 * m_globalModel.getSpectraVectorSize( ) ) + 2 * kk + 1];
                            
                            absPrepro[kk] = allPreprocessedSignalFloats[ ( cntr * 2 * m_globalModel.getSpectraVectorSize( ) ) + 2 * kk];
                            dispPrepro[kk] = allPreprocessedSignalFloats[ ( cntr * 2 * m_globalModel.getSpectraVectorSize( ) ) + 2 * kk + 1];   
                        }
                
                        tdfdfitResult.setFittedSpectrumRealTD( abs );
                        tdfdfitResult.setFittedSpectrumImagTD( disp );
                        
                        tdfdfitResult.setPreprocessedInputSignalRealTD( absPrepro );
                        tdfdfitResult.setPreprocessedInputSignalImagTD( dispPrepro );
                        
                        m_tdfdfitResults.put( key, tdfdfitResult );
                        
                        cntr++; 
                    }
                }
            }
            
            // Now read the fitted values from the file system.
            for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1; i++ )
            {
                for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2; j++ )
                {
                    for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3; k++ )
                    {
                        String key = "" + i + "_" + j + "_" + k;
                        
                        NumericTDFDFitResults tdfdfitResult = m_tdfdfitResults.get( key );
                        String keyFromFile = dis.readUTF( );
                        tdfdfitResult.readFromDataInputStream( dis );
                    }
                }
            }
        } 
        catch ( FileNotFoundException ex ) 
        {        
             JOptionPane.showMessageDialog( null, "Cannot load previously stored TDFDFit-ted!\n No .tfr TDFDFit-results file found!" );
             return false;
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
            return false;
        }
        return true;
    }
    
    public HashMap<String, NumericTDFDFitResults> getNumericTDFDFitResultsHashMap( )
    {
        return m_tdfdfitResults;
    }
    
    
    /**
     * This methods stores the whole object into a binary file.
     * The fitting parameters as well as the .bin and .fit files are stored,
     * such that the user gets access to previously quantified and stored 
     * fitting data.
     */
    public void writeTDFDFitResultsStore( ) 
    {
        String currentDCMFileRef = m_globalModel.getCSIData( ).getFile( );
        String outputOverallResultFile;
        
        if ( currentDCMFileRef.length( ) > 0 )
        {
            int index = currentDCMFileRef.lastIndexOf( "." );
            if ( ( index > 0 ) && ( index <= currentDCMFileRef.length( ) ) )
            {
                outputOverallResultFile = currentDCMFileRef.substring( 0, index );
            }
            else
            {         
                outputOverallResultFile = currentDCMFileRef;
            }
            
            outputOverallResultFile = outputOverallResultFile + ".tfr";
            
            float[] allOrderedOutputFitSignals = getAllFittedModelSignals( ); 
            float[] allOrderedOutputMeasuredAndPreprocessedSignals = getAllPreProcessedMeasuredSignals( );
            
            try ( DataOutputStream dos = new DataOutputStream( new FileOutputStream( outputOverallResultFile ) ) ) 
            {    
                byte[] buffer = new byte[4*allOrderedOutputFitSignals.length];
                
                for ( int i = 0; i < allOrderedOutputFitSignals.length; ++i )
                {
                    byte[] convertedFloatAsByteArray = floatToByteArray( allOrderedOutputFitSignals[i] );
                    buffer[4 * i]     = convertedFloatAsByteArray[0];
                    buffer[4 * i + 1] = convertedFloatAsByteArray[1];
                    buffer[4 * i + 2] = convertedFloatAsByteArray[2];
                    buffer[4 * i + 3] = convertedFloatAsByteArray[3];
                }
                dos.write( buffer );
                dos.flush( );
                
                byte[] buffer2 = new byte[4*allOrderedOutputFitSignals.length];
                for ( int i = 0; i < allOrderedOutputMeasuredAndPreprocessedSignals.length; ++i )
                {
                    byte[] convertedFloatAsByteArray = floatToByteArray( allOrderedOutputMeasuredAndPreprocessedSignals[i] );
                    buffer2[4 * i]     = convertedFloatAsByteArray[0];
                    buffer2[4 * i + 1] = convertedFloatAsByteArray[1];
                    buffer2[4 * i + 2] = convertedFloatAsByteArray[2];
                    buffer2[4 * i + 3] = convertedFloatAsByteArray[3];
                }
                
                dos.write( buffer2 );
                dos.flush( );
                 
                writeAllParametricData( dos );
                dos.flush( );
                // finally close the data output stream
                dos.close( );    
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( TDFDFitResultsStore.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        else
        {
            System.out.println( "Warning: Cannot make TDFDFit results persistent!" );
        }
    }
    
    
    
    public static byte[] floatToByteArray( float value ) 
    {
        int intBits =  Float.floatToIntBits(value);
        return new byte[] { (byte) (intBits >> 24), (byte) (intBits >> 16), (byte) (intBits >> 8), (byte) (intBits) };
    }
    
    
    public static float byteArrayToFloat( byte[] bytes ) 
    {
        int intBits = bytes[0] << 24 | (bytes[1] & 0xFF) << 16 | (bytes[2] & 0xFF) << 8 | (bytes[3] & 0xFF);
        return Float.intBitsToFloat(intBits);  
    }
    
    @Override
    public void printJMRUIResults( ) 
    {
        throw new UnsupportedOperationException( "Not supported yet." ); //To change body of generated methods, choose Tools | Templates.
    }

}