/*
 * File: $Id: MRFittingDefaults.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 16.09.2003, 10:58  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.main.SpectrImClinicalQMRSBoot;
import org.spectrim.settings.GlobalSettings;
import org.spectrim.tdfdfit.quantitativeresults.MetaboliteRelaxationTimesDatabase;

/**
 * Singleton class that contains all user preferred TDFDFit related settings.
 *
 * @author  hans
 */
public final class TDFDFittingDefaults extends Properties
{
   
    /** 
     * Creates a new instance of TDFDFittingDefaults. 
     */
    private TDFDFittingDefaults( )
    {
        GlobalSettings globSettings = GlobalSettings.getInstance( );
        String TDFDFittingDefaultsPath = globSettings.getSpectrimRootDir( );
        
        String userPreferenceFileName = TDFDFittingDefaultsPath + 
                                        System.getProperty( "file.separator" ) +
                                        "TDFDFit" +
                                        System.getProperty( "file.separator" ) +
                                        "TDFDfit.ini";
        
        File testExistenceOfFile = new File( userPreferenceFileName );
        
        if ( testExistenceOfFile.exists( ) == false )
        {
            String sep =  System.getProperty( "file.separator" );
            SpectrImClinicalQMRSBoot instance = SpectrImClinicalQMRSBoot.getInstance( );
            String spectrimRootDir = instance.getProperty(  SpectrImClinicalQMRSBoot.SPECTRIM_CLINICAL_QMRS_BASE_PATH );

            setProperty( TDFDFIT_MODELS_BASE_PATH, spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFit_FitModelsDir"  + sep );    

            setProperty( DEFAULT_TDFDFIT_FIT_MODEL_NAME, "Undefined" );

            setProperty( TDFDFIT_WATER_MODEL_PATH, spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFit_FitModelsDir"  + sep );

            setProperty( TDFDFIT_WATER_MODEL_NAME, "Undefined" );

            setProperty( FIT_PROJECT_DATABASE_PATH, System.getProperty( "user.home" ) );

            setProperty( DICOM_MOUNT_PATH, System.getProperty( "user.home" ) );

            setProperty( TDFDFIT_NORMAL_VALUES_CONCENTRATIONS_PATH, spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFit_NormalConcentrationsDBaseDir" + sep );

            setProperty( TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE, spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFit_NormalConcentrationsDBaseDir" + sep + "TDFDFitMetaboliteRelaxationTimesDatabase.txt" );

            setProperty( TDFDFIT_INSTALLATION_PATH, spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFitExecutableDir" + sep );

            setProperty( BASELINE_SPLINE_MODEL_PATH, spectrimRootDir + sep + "TDFDFit" + sep + "BaselineSplineModels" + sep );
            
            setProperty( TDFDFIT_EXECUTABLE_NAME, "tdfdfit2017.exe" );

            setProperty( LIB_HLSVD_PATH_AND_DLL_NAME, System.getProperty( "user.home" ) );
            setProperty( CALL_FORTRAN_PATH_AND_DLL_NAME, System.getProperty( "user.home" ) );

            setProperty( BASELINE_SPLINE_REMOVAL_MODE, MAGNITUDE );
            
            setProperty( DISPLAY_DOMAIN, FREQ_DOMAIN );
            setProperty( DISPLAY_CHANNEL, ABSORPTION );
            setProperty( DISPLAY_UNIT, FREQUENCY_DOMAIN_PPM );

            // export model as pattern
            setProperty( EXPORT_MODEL_AS_PATTERN_APPLY_FILTER, YES );
            setProperty( PATTERN_EXPORT_CUTOFF_FREQ, "5.000" );

            setProperty( APPLY_QUALITY, "false" );
            setProperty( QUALITY_TYPE, QUALITY_TYPE_PHASE_ONLY );

            // patient database root path
            setProperty( PATIENT_DATABASE_ROOT_PATH, spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFitPatientDatabaseDir" + sep );

            setProperty( HLSVD_NUMBER_OF_DATAPOINTS, "1024" );
            setProperty( HLSVD_TD_SAMPLE_STEP_SIZE, "1" );
            setProperty( HLSVD_HANKEL_MATRIX_SIZE, "120" );
            setProperty( HLSVD_NUMBER_OF_SINUSOIDS, "30" );
            setProperty( HLSVD_MINUMUM_FILTER_FREQUENCY, "4.4" );
            setProperty( HLSVD_MAXIMUM_FILTER_FREQUENCY, "5.4" );
            setProperty( HLSVD_RANGES_FILTER_FREQUENCY, "4.4#5.4");
            setProperty( HLSVDPRO_VERSION, HLSVDPRO_VERSION_SERIAL );
            setProperty( HLSVD_FILTER_TYPE, HLSVD_FILTER_WATER_ONLY );
            setProperty( HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD,  "50" );
            setProperty( HLSVDPRO_BATCH_SIZE,  "32" );

            
            // Values added to define segmentation pixel thressholds for image based s
            // releative volume segmentation
            setProperty( IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE, "0" );
            setProperty( IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE, "100" );
            setProperty( IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE, "101" );
            setProperty( IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE, "200" );
            setProperty( IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE, "201" );
            setProperty( IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE, "800" );

            // Values related to generating groundtruth for RFregression
            setProperty( LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY, "not defined yet!" );
            save( );
        }
    }
    
    /**
     * Returns the one and only instance of MRFittingDefaults.
     * @return 
     */
    public static TDFDFittingDefaults getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new TDFDFittingDefaults( );
            theInstance.load( );
            
            String sep =  System.getProperty( "file.separator" );
            String userDirectory = System.getProperty("user.dir");
            // System.out.println( "userDirectory = " + userDirectory );
            SpectrImClinicalQMRSBoot spectrImClinicalQMRSBootInstance = SpectrImClinicalQMRSBoot.getInstance( );
            String spectrimRootDir = spectrImClinicalQMRSBootInstance.getProperty( SpectrImClinicalQMRSBoot.SPECTRIM_CLINICAL_QMRS_BASE_PATH );
            
            // since this is the first time that this program is lauched we copy now the dll and .exe files
            File tdfdfitFile = new File( userDirectory + sep + "bin" + sep + "tdfdfit2017.exe" );
            File toTDFDFitExeFile =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFitExecutableDir" + sep + "tdfdfit2017.exe" );
            
            if ( tdfdfitFile.exists( ) && !toTDFDFitExeFile.exists( ) )
            {   
                
                try 
                {
                    // toTDFDFitExeFile =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "TDFDFitExecutableDir" + sep + "tdfdfit2017.exe" );
                    
                    // boolean retVal = tdfdfitFile.renameTo( new File( toExeFileName ) );
                    
                    Files.copy( tdfdfitFile.toPath( ), 
                                toTDFDFitExeFile.toPath( ), 
                                StandardCopyOption.REPLACE_EXISTING );
                    
                    System.out.println( "tdfdfit2017.exe copied to : " + toTDFDFitExeFile.getAbsolutePath( ) );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
            
            File callfortran_hlsvd_arr = new File( userDirectory + sep + "bin" + sep + "callfortran_hlsvd_arr.dll" );
            File toDLLFileName =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "callfortran_hlsvd_arr.dll" );
            
            if ( callfortran_hlsvd_arr.exists( ) && !toDLLFileName.exists( ) )
            {   
                
                try 
                {
                    // File toDLLFileName =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "callfortran_hlsvd_arr.dll" );
                    
                    //boolean retVal = callfortran_hlsvd_arr.renameTo( new File( toDLLFileName ) );
                    
                    Files.copy( callfortran_hlsvd_arr.toPath( ), 
                                toDLLFileName.toPath( ),
                                StandardCopyOption.REPLACE_EXISTING );
                    
                    System.out.println( "callfortran_hlsvd_arr.dll copied to : " + toDLLFileName.getAbsolutePath( ) );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
            
            File libgcc_s_seh_1 = new File( userDirectory + sep + "bin" + sep + "libgcc_s_seh-1.dll" );
            File toDLLFileName2 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libgcc_s_seh-1.dll" );
            if ( libgcc_s_seh_1.exists( ) && !toDLLFileName2.exists( ) )
            {   
                
                // File toDLLFileName2 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libgcc_s_seh-1.dll" );
                
                try 
                {
                    // boolean retVal = libgcc_s_seh_1.renameTo( new File( toDLLFileName2 ) );
                    Files.copy( libgcc_s_seh_1.toPath( ),
                                toDLLFileName2.toPath( ),
                                StandardCopyOption.REPLACE_EXISTING );
                    System.out.println( "libgcc_s_seh-1.dll copied to : " + toDLLFileName2.getAbsolutePath( ) );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                
                
            }
            
            File libgfortran_4 = new File( userDirectory + sep + "bin" + sep + "libgfortran-4.dll" );
            File toDLLFileName3 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libgfortran-4.dll" );
            if ( libgfortran_4.exists( ) && !toDLLFileName3.exists( ) )
            {   
                
                try 
                {
                    // File toDLLFileName3 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libgfortran-4.dll" );
                    
                    // boolean retVal = libgfortran_4.renameTo( new File( toDLLFileName3 ) );
                    Files.copy( libgfortran_4.toPath( ),
                                toDLLFileName3.toPath( ),
                                StandardCopyOption.REPLACE_EXISTING );
                    System.out.println( "libgfortran-4.dll copied to : " + toDLLFileName3.getAbsolutePath( ) );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
            
            File libhlsvd = new File( userDirectory + sep + "bin" + sep + "libhlsvd.dll" );
            File toDLLFileName4 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libhlsvd.dll" );
            if ( libhlsvd.exists( ) && !toDLLFileName4.exists( ) )
            {   
                
                try 
                {
                    // File toDLLFileName4 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libhlsvd.dll" );
                    
                    // boolean retVal = libhlsvd.renameTo( new File( toDLLFileName4 ) );
                    Files.copy( libhlsvd.toPath( ),
                                toDLLFileName4.toPath( ),
                                StandardCopyOption.REPLACE_EXISTING );
                    System.out.println( "libhlsvd.dll copied to : " + toDLLFileName4.getAbsolutePath( ) );
                } catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
            
            File libquadmath_0 = new File( userDirectory + sep + "bin" + sep + "libquadmath-0.dll" );
            File toDLLFileName5 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libquadmath-0.dll" );
                    
            if ( libquadmath_0.exists( ) && !toDLLFileName5.exists( ) )
            {   
                
                try 
                {
                    // File toDLLFileName5 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libquadmath-0.dll" );
                    
                    Files.copy( libquadmath_0.toPath( ),
                                toDLLFileName5.toPath( ),
                                StandardCopyOption.REPLACE_EXISTING );
                    
                    // boolean retVal = libquadmath_0.renameTo( new File( toDLLFileName5 ) );

                    System.out.println( "libquadmath-0.dll copied to : " + toDLLFileName5 );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
            
            File libwinpthread_1 = new File( userDirectory + sep + "bin" + sep + "libwinpthread-1.dll" );
            File toDLLFileName6 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libwinpthread-1.dll" );
            
            if ( libwinpthread_1.exists( ) && !toDLLFileName6.exists() )
            {
                try 
                {
                    // File toDLLFileName6 =  new File( spectrimRootDir + sep + "TDFDFit" + sep + "HLSVD-64" + sep + "libwinpthread-1.dll" );
                    
                    Files.copy( libwinpthread_1.toPath( ),
                                toDLLFileName6.toPath( ),
                                StandardCopyOption.REPLACE_EXISTING );
                    
                    // boolean retVal = libwinpthread_1.renameTo( new File( toDLLFileName6 ) );
                    
                    System.out.println( "libwinpthread-1.dll copied to: " + toDLLFileName6.getAbsolutePath( ) );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFittingDefaults.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }
        }
        return theInstance;
    }

    /**
     * Loads the user preferences file.
     */ 
    public void load( )
    {
        GlobalSettings globSet = GlobalSettings.getInstance( );
        String TDFDFittingDefaultsPath = globSet.getSpectrimRootDir( );
        
        String userPreferenceFileName = TDFDFittingDefaultsPath + 
                                        System.getProperty( "file.separator" ) +
                                        "TDFDFit" +
                                        System.getProperty( "file.separator" ) +
                                        "TDFDfit.ini";
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
        
            // now the ini file must exist with default contents, so load it
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            try
            {
                File f = new File( userPreferenceFileName );
                if ( f.exists( ) == false )
                {
                    File TDFDDir = new File( f.getParent( ) );
                    if ( TDFDDir.exists( ) == false )
                    {
                        TDFDDir.mkdir( );
                        if ( TDFDDir.exists( ) )
                        {
                            // Make all the expected subdirectories of TDFDFit
                            String TDFDFitInstallationDir = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "TDFDFitExecutableDir";
                            File TFDFitInstallationDirFile = new File( TDFDFitInstallationDir );
                            if ( TFDFitInstallationDirFile.exists( ) == false )
                            {
                                TFDFitInstallationDirFile.mkdir( );
                                if ( TFDFitInstallationDirFile.exists( ) )
                                {
                                    setProperty( TDFDFIT_INSTALLATION_PATH, TFDFitInstallationDirFile.getPath( ) );
                                }
                            }
                            
                            String TDFDFitPatientDataBaseDir = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "TDFDFitPatientDatabaseDir";
                            File TDFDFitPatientDataBaseDirFile = new File( TDFDFitPatientDataBaseDir );
                            if ( TDFDFitPatientDataBaseDirFile.exists( ) == false )
                            {
                                TDFDFitPatientDataBaseDirFile.mkdir( );
                                setProperty( PATIENT_DATABASE_ROOT_PATH, TDFDFitPatientDataBaseDirFile.getPath( ) );
                            }
                            
                            String TDFDFit_FitModelsDir = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "TDFDFit_FitModelsDir";
                            File TDFDFit_FitModelsDirFile = new File( TDFDFit_FitModelsDir );
                            if ( TDFDFit_FitModelsDirFile.exists( ) == false )
                            {
                                TDFDFit_FitModelsDirFile.mkdir( );
                                setProperty( TDFDFIT_MODELS_BASE_PATH, TDFDFit_FitModelsDirFile.getPath( ) );
                            }
                            
                            String TDFDFit_RelaxationDBaseDir = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "TDFDFit_RelaxationTimesDBaseDir";
                            File TDFDFit_RelaxationDBaseDirFile = new File( TDFDFit_RelaxationDBaseDir );
                            if ( TDFDFit_RelaxationDBaseDirFile.exists( ) == false )
                            {
                                TDFDFit_RelaxationDBaseDirFile.mkdir( );
                                
                                setProperty( TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE, 
                                             TDFDFit_RelaxationDBaseDir + 
                                             System.getProperty( "file.separator" ) + 
                                             "TDFDFitMetaboliteRelaxationTimesDatabase.txt" );
                                // Get the singleton instance: if the default database does not exists it gets created.
                                MetaboliteRelaxationTimesDatabase dbase = MetaboliteRelaxationTimesDatabase.getInstance( );
                            }
                            
                            String TDFDFit_NormalConcentrationsDBase = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "TDFDFit_NormalConcentrationsDBaseDir";
                            File TDFDFit_NormalConcentrationsDBaseFile = new File( TDFDFit_NormalConcentrationsDBase );
                            if ( TDFDFit_NormalConcentrationsDBaseFile.exists( ) == false )
                            {
                                TDFDFit_NormalConcentrationsDBaseFile.mkdir( );
                                setProperty( TDFDFIT_NORMAL_VALUES_CONCENTRATIONS_PATH, TDFDFit_NormalConcentrationsDBaseFile.getPath( ) );
                            }
                            
                            String HLSVD64Path = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "HLSVD-64";
                            File HLSVD64FILE = new File( HLSVD64Path );
                            if ( HLSVD64FILE.exists( ) == false )
                            {
                                HLSVD64FILE.mkdir( );
                                setProperty( LIBHLSVD_DLL_PATH, HLSVD64FILE.getPath( ) );
                            }
                             
                            String HLSVD_64__call_fortran_str = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "HLSVD-64" + System.getProperty( "file.separator" ) + "callfortran_hlsvd_arr.dll"; 
                            setProperty( CALL_FORTRAN_PATH_AND_DLL_NAME, HLSVD_64__call_fortran_str );
                            
                            String HLSVD_64__libhslvd_str = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "HLSVD-64" + System.getProperty( "file.separator" ) + "libhlsvd.dll"; 
                            setProperty( LIB_HLSVD_PATH_AND_DLL_NAME, HLSVD_64__libhslvd_str );
                            
                            // Baseline spline path
                            String BaselineSplineNodeModelsDir = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "BaselineSplineNodeModels";
                            File BaselineSplineNodeModelsDirFile = new File( BaselineSplineNodeModelsDir );
                            if ( BaselineSplineNodeModelsDirFile.exists( ) == false )
                            {
                                BaselineSplineNodeModelsDirFile.mkdir( );
                                setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH, BaselineSplineNodeModelsDirFile.getPath( ) );
                            }
                        }
                        
                        // example: setx PATH "C:\myfolder;%PATH%"
                        
                        String HLSVD64Path = TDFDDir.getPath( ) + System.getProperty( "file.separator" ) + "HLSVD-64";
                        
                        String systemAddItemToPATH = "setx PATH \"" + HLSVD64Path + ";%PATH%\"";
                        
                        String systemAddItemToPATHCall = generateProperCommand( systemAddItemToPATH );
                        
                        System.out.println( "Trying to extend the Windows search PATH environment variable : " + systemAddItemToPATH );

                        try
                        {       
                            Process p = Runtime.getRuntime( ).exec( systemAddItemToPATH ); 

                            InputStream in = p.getInputStream( );
                            int c;
                            while ( ( c = in.read( ) ) != -1 )
                            {
                                System.out.print( (char)c ) ;
                            }
                            int returnValue = p.waitFor( );
                            in.close( );
                        }
                        catch ( IOException ioX )
                        {
                            // not handled yet
                            System.out.println( "IOException occurred in : " +  systemAddItemToPATH );
                        }
                        catch ( InterruptedException intX )
                        {
                            // not handled yet
                            System.out.println( "InterruptedException occurred" );
                        }
                        
                        this.setProperty( TDFDFittingDefaults.FittingTolerance , "1.0E-007" );
                        this.setProperty( TDFDFittingDefaults.NumberOfLineShapeFourierCoefficientsToFit , "" + 0 );
                        this.setProperty( TDFDFittingDefaults.FittingMode, "" +  1 );
                        this.setProperty( TDFDFittingDefaults.TDFDFitVerbosity, "" + 0 );
                        this.setProperty( TDFDFittingDefaults.ShowInfoOfComponent , "" + 1 );
                        this.setProperty( TDFDFittingDefaults.CalculateErrorsAfterFit, "no" );
                        this.setProperty( TDFDFittingDefaults.NumberOfLineShapeFourierCoefficientsToFit , "0" );
                        
                    }
                } 
                
                OutputStream os = new BufferedOutputStream( new FileOutputStream(  userPreferenceFileName ) );
                this.save( os, "// This is a generated file: do not edit!!" );
                os.close( );
            }
            catch ( IOException ioe )
            { 
                System.out.println( "IO-error occured while attempting to write TFDFFit.ini" );
                // should hopefully never occur
            }
        }
        
        // now the ini file must exist with default contents, so load it
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            // should never occur
        }
    }
    
    private String generateProperCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }
    
    
    
    
   /**
     * Saves the user preferences file.
     */ 
    public void save( )
    {
        try
        {
            GlobalSettings globSet = GlobalSettings.getInstance( );
            String TDFDFittingDefaultsPath = globSet.getSpectrimRootDir( );
            String userPreferenceFileName = TDFDFittingDefaultsPath + 
                                        System.getProperty( "file.separator" ) +
                                        "TDFDFit" +
                                        System.getProperty( "file.separator" ) +
                                        "TDFDfit.ini";
            
            OutputStream out = new BufferedOutputStream( new FileOutputStream( userPreferenceFileName ) );
            this.save( out, "//This is a generated file : do not edit" );
        }
        catch ( IOException e )
        {
            // should not occur
        }
    }
    
    /**
     * Overrides the base class; this is necessary since we want to implement
     * to notify changes to the settings to ActionListeners.
     * @param key
     * @param value
     * @return 
     */
    @Override
    public synchronized Object setProperty( String key, String value ) 
    {
        
        Object retObj = super.setProperty( key, value );
        
        // notify ActionListeners that there is a change in properties!
        if ( key.compareTo( "display unit" ) == 0 )
        {
            save();
            notifyActionListeners( this, PROPERTY_CHANGED_EVENT, PROPERTY_CHANGED_CMD );
        }
        return retObj;
    }
    
    /**
     * Adds an ActionListener to "this"
     * @param listener 
     */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );   
    }
    
    /**
     * 
     * Method called to notify registered classes of certain events.
     * 
     * @param obj
     * @param eventType
     * @param command 
     */
    private void notifyActionListeners( Object obj, int eventType,  String command )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, command ) );
        }
    }
    
    // static private attribute is the instance itself (Singleton!)
    private static  TDFDFittingDefaults theInstance = null;
    
    // Static and non static member variables
    // public final static String DEFAULT_TDFDFIT_MODEL_PATH_REFERENCE = "Default TDFDFit model file reference";
    
    public final static String TDFDFIT_MODELS_ROOT_PATH         = "TDFDFit models root path";
    public final static String DEFAULT_TDFDFIT_FIT_MODEL_NAME   = "Default used TDFDFit model name";
    public final static String DEFAULT_USED_TDFDFIT_MODEL_PATH  = "Default used TDFDFit model path";
      
    public final static String TDFDFIT_WATER_MODEL_NAME         = "Default used TDFDFit water model name";
    public final static String TDFDFIT_WATER_MODEL_PATH         = "Default used TDFDFit water model path";
    
    public final static String LAST_USED_NUMERIC_PATTERN_LOAD_PATH      = "Last used numeric pattern path";
    public final static String LAST_USED_PARAMETRIC_PATTERN_LOAD_PATH   = "Last used parametric pattern path";
    public final static String LAST_USED_METABOLITE_PATTERN_LOAD_PATH   = "Last used metabolite pattern path";
    
    public final static String FIT_PROJECT_DATABASE_PATH    = "Fit project database path";
    public final static String DICOM_MOUNT_PATH             = "Dicom mount path";
    
    // Indicator to the location of the patient database files
    public final static String PATIENT_DATABASE_ROOT_PATH   = "Patient database root path";
    public final static String TDFDFIT_INSTALLATION_PATH    = "TDFDFit installation path";
    public final static String TDFDFIT_EXECUTABLE_NAME      = "TDFDFit executable name";
    public final static String TDFDFIT_MODELS_BASE_PATH     = "TDFDFit models base path";
    public final static String TDFDFIT_NORMAL_VALUES_CONCENTRATIONS_PATH = "TDFDFit normal metabolite concentration path";
    
    public final static String BASELINE_SPLINE_MODEL_PATH = "Baseline spline model path";
    public final static String BASELINE_SPLINE_MODEL_NAME = "Baseline spline model name";
    public final static String BASELINE_SPLINE_REMOVAL_MODE = "Baseline spline model mode";
    
    public final static String HLSVD_64_CALL_FORTRAN_DLL_FILE_REF = "HLSVD callfortran_hlsvd_arr.dll path";
    public final static String HLSVD_64_LIBHLSVD_DLL_FILE_REF = "HLSVD libhlsvd.dll path";
    public final static String PYTHON_EXECUTABLE_FILENAME = "python.exe path";
    
    public final static String TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE = "TDFDFit relaxation times data base path and file";
    public final static String DISPLAY_CHANNEL  = "display channel";
    public final static String DISPLAY_DOMAIN   = "display domain";
    public final static String DISPLAY_UNIT     = "display unit";
    
    // Variables needed to identify display domain types
    public final static String FREQ_DOMAIN      = "freq-domain";
    public final static String TIME_DOMAIN      = "time-domain";
    
    // Variables needed to identify display channel types
    public final static String ABSORPTION = "absorption";
    public final static String DISPERSION = "dispersion";
    public final static String MAGNITUDE  = "magnitude";
    public final static String POWER = "power";
    public final static String PHASE = "phase";
    

    public final static String TIME_DOMAIN_POINTS = "td_points";
    public final static String TIME_DOMAIN_SECONDS = "td_seconds";
    
    public final static String FREQUENCY_DOMAIN_POINTS = "fd_points";
    public final static String FREQUENCY_DOMAIN_HZ = "fd_hertz";
    public final static String FREQUENCY_DOMAIN_PPM = "fd_ppm";
    
    public final static String EXPORT_MODEL_AS_PATTERN_APPLY_FILTER = "When model exported as filter apply filter";
    
    public final static String YES = "yes";
    public final static String NO = "no";
    
    public final static String PATTERN_EXPORT_CUTOFF_FREQ = "Pattern cut-off frequency";
    
    public final static String APPLY_QUALITY = "Apply QUALITY";
    public final static String QUALITY_TYPE = "QUALITY Type";
    
    // Quality type can either be complete (amplitude and phase) or phase only
    public final static String QUALITY_TYPE_PHASE_ONLY = "Phase only";
    public final static String QUALITY_TYPE_AMPLITUDE_AND_PHASE = "Amplitude and Phase";
    
    // Related to action listeners
    LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );
    public final static int     PROPERTY_CHANGED_EVENT = 1000;
    public final static String  PROPERTY_CHANGED_CMD = "PROPERTY_CHANGED";
    
    // Protons
    public final static String AreaMaxFactorDecimalValue_1H = "AreaMaxFactorDecimalValue_1H";
    
    public final static String AreaMaxValDecimalValue_1H = "AreaMaxValDecimalValue_1H";
    public final static String AreaMinValDecimalValue_1H = "AreaMinValDecimalValue_1H";
    
    public final static String GaussianWidthMaxValDecimalValue_1H = "GaussianWidthMaxValDecimalValue_1H";
    public final static String GaussianWidthMinValDecimalValue_1H = "GaussianWidthMinValDecimalValue_1H";
    
    public final static String LorentzianWidthMaxValDecimalValue_1H = "LorentzianWidthMaxValDecimalValue_1H";
    public final static String LorentzianWidthMinValDecimalValue_1H = "LorentzianWidthMinValDecimalValue_1H";
    
    public final static String OffsetMaxValDecimalValue_1H =  "OffsetMaxValDecimalValue_1H";
    public final static String OffsetMinValDecimalValue_1H = "OffsetMinValDecimalValue_1H";
    
    public final static String PhaseMaxValDecimalValue_1H = "PhaseMaxValDecimalValue_1H";
    public final static String PhaseMinValDecimalValue_1H = "PhaseMinValDecimalValue_1H";
    
    public final static String IsAreaMaxDeterminedByFactor_1H = "IsAreaMaxDeterminedByFactor_1H";
    /////////////////////////////////////////////
    
    // Phosphorus
    public final static String AreaMaxFactorDecimalValue_31P = "AreaMaxFactorDecimalValue_31P";
    
    public final static String AreaMaxValDecimalValue_31P = "AreaMaxValDecimalValue_31P";
    public final static String AreaMinValDecimalValue_31P = "AreaMinValDecimalValue_31P";
    
    public final static String GaussianWidthMaxValDecimalValue_31P = "GaussianWidthMaxValDecimalValue_31P";
    public final static String GaussianWidthMinValDecimalValue_31P = "GaussianWidthMinValDecimalValue_31P";
    
    public final static String LorentzianWidthMaxValDecimalValue_31P = "LorentzianWidthMaxValDecimalValue_31P";
    public final static String LorentzianWidthMinValDecimalValue_31P = "LorentzianWidthMinValDecimalValue_31P";
    
    public final static String OffsetMaxValDecimalValue_31P = "OffsetMaxValDecimalValue_31P";
    public final static String OffsetMinValDecimalValue_31P = "OffsetMinValDecimalValue_31P";
    
    public final static String PhaseMaxValDecimalValue_31P = "PhaseMaxValDecimalValue_31P";
    public final static String PhaseMinValDecimalValue_31P = "PhaseMinValDecimalValue_31P";
    
    public final static String IsAreaMaxDeterminedByFactor_31P = "IsAreaMaxDeterminedByFactor_31P";
    
    /////////////////////////////////////////////

    // Carbon 13 
    public final static String AreaMaxFactorDecimalValue_13C = "AreaMaxFactorDecimalValue_13C";
    
    public final static String AreaMaxValDecimalValue_13C = "AreaMaxValDecimalValue_13C";
    public final static String AreaMinValDecimalValue_13C = "AreaMinValDecimalValue_13C";
    
    public final static String GaussianWidthMaxValDecimalValue_13C = "GaussianWidthMaxValDecimalValue_13C";
    public final static String GaussianWidthMinValDecimalValue_13C = "GaussianWidthMinValDecimalValue_13C";
    
    public final static String LorentzianWidthMaxValDecimalValue_13C = "LorentzianWidthMaxValDecimalValue_13C";
    public final static String LorentzianWidthMinValDecimalValue_13C = "LorentzianWidthMinValDecimalValue_13C";
    
    public final static String OffsetMaxValDecimalValue_13C = "OffsetMaxValDecimalValue_13C";
    public final static String OffsetMinValDecimalValue_13C = "OffsetMinValDecimalValue_13C";
    
    public final static String PhaseMaxValDecimalValue_13C = "PhaseMaxValDecimalValue_13C";
    public final static String PhaseMinValDecimalValue_13C = "PhaseMinValDecimalValue_13C";
    
    public final static String IsAreaMaxDeterminedByFactor_13C = "IsAreaMaxDeterminedByFactor_13C";
    
/////////////////////////////////////////////
    
    // Sodium 23
    public final static String AreaMaxFactorDecimalValue_23Na = "AreaMaxFactorDecimalValue_23Na";
    
    public final static String AreaMaxValDecimalValue_23Na = "AreaMaxValDecimalValue_23Na";
    public final static String AreaMinValDecimalValue_23Na = "AreaMinValDecimalValue_23Na";
    
    public final static String GaussianWidthMaxValDecimalValue_23Na = "GaussianWidthMaxValDecimalValue_23Na";
    public final static String GaussianWidthMinValDecimalValue_23Na = "GaussianWidthMinValDecimalValue_23Na";
    
    public final static String LorentzianWidthMaxValDecimalValue_23Na = "LorentzianWidthMaxValDecimalValue_23Na";
    public final static String LorentzianWidthMinValDecimalValue_23Na = "LorentzianWidthMinValDecimalValue_23Na";
    
    public final static String OffsetMaxValDecimalValue_23Na = "OffsetMaxValDecimalValue_23Na";
    public final static String OffsetMinValDecimalValue_23Na = "OffsetMinValDecimalValue_23Na";
    
    public final static String PhaseMaxValDecimalValue_23Na = "PhaseMaxValDecimalValue_23Na";
    public final static String PhaseMinValDecimalValue_23Na = "PhaseMinValDecimalValue_23Na";

    public final static String IsAreaMaxDeterminedByFactor_23Na = "IsAreaMaxDeterminedByFactor_23Na";
    
    // Setting that determine TDFDFit runtime behaviour 
    public final static String FittingTolerance = "FittingTolenancce";
    public final static String FittingMode      = "TDFDFitFittingMode";
    public final static String TDFDFitVerbosity = "Verbosity";
    public final static String ShowInfoOfComponent = "ShowComponentInfoDuringFit";
    public final static String CalculateErrorsAfterFit = "CalculateErrorsAfterFit";
    public final static String NumberOfLineShapeFourierCoefficientsToFit = "NumberOfLineShapeFourierCoefficientsToFit";
    
    // HLSVD defaults
    public final static String HLSVD_NUMBER_OF_DATAPOINTS          = "Number of HLSVD data points";
    public final static String HLSVD_TD_SAMPLE_STEP_SIZE           = "HLSVD time domain sample step size";
    public final static String HLSVD_HANKEL_MATRIX_SIZE            = "HLSVD Hankel matrix size";
    public final static String HLSVD_NUMBER_OF_SINUSOIDS           = "HLSVD number of sinusoids";
    public final static String HLSVD_MINUMUM_FILTER_FREQUENCY      = "HLSVD minimum filter frequency";
    public final static String HLSVD_MAXIMUM_FILTER_FREQUENCY      = "HLSVD maximum filter frequency";
    public final static String HLSVD_RANGES_FILTER_FREQUENCY       = "HLSVD ranges filter frequency";
    
    public final static String HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD = "HLSVD extreme broad line width limit";
    public final static String HLSVD_PPM_REFERENCE_VALUE           = "HLSVD assume reference frequency";
    public final static String HLSVDPRO_BATCH_SIZE                 = "HLSVDPro batch size";

    // some EPSI related variables
    public final static String EPSI_MIDAS_LAST_LOADED_RESONANCE_FREQUENCY  = "EPSI MIDAS LAST RESONANCE FREQUENCY";
    public final static String EPSI_MIDAS_LAST_LOADED_SAMPLING_FREQUENCY   = "EPSI MIDAS LAST SAMPLING FREQUENCY";
    public final static String EPSI_MIDAS_LAST_PPM_REFERENCE_VALUE         = "EPSI MIDAS LAST PPM REFERENCE VALUE";
    // Pixel based 
    public final static String IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE = "Image segmentation GM minimal pixel value";
    public final static String IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE = "Image segmentation GM maximal pixel value";
    public final static String IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE = "Image segmentation WM minimal pixel value";
    public final static String IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE = "Image segmentation WM maximal pixel value";
    public final static String IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE = "Image segmentation CSF minimal pixel value";
    public final static String IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE = "Image segmentation CSF maximal pixel value";
    
    public final static String LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY = "Last Random Forest regression data load directory";
    
    public final static String IMAGE_SEGMENTATION_DISPLAY_TRANSPARENCY  = "Image segmentation transparency";
    
    public final static String IMAGE_SEGMENTATION_GM_RED_COMPONENT      = "Image segmentation GM red";
    public final static String IMAGE_SEGMENTATION_GM_GREEN_COMPONENT    = "Image segmentation GM green";
    public final static String IMAGE_SEGMENTATION_GM_BLUE_COMPONENT     = "Image segmentation GM blue";
    
    public final static String IMAGE_SEGMENTATION_WM_RED_COMPONENT      = "Image segmentation WM red";
    public final static String IMAGE_SEGMENTATION_WM_GREEN_COMPONENT    = "Image segmentation WM green";
    public final static String IMAGE_SEGMENTATION_WM_BLUE_COMPONENT     = "Image segmentation WM blue";
    
    public final static String IMAGE_SEGMENTATION_CSF_RED_COMPONENT     = "Image segmentation CSF red";
    public final static String IMAGE_SEGMENTATION_CSF_GREEN_COMPONENT   = "Image segmentation CSF green";
    public final static String IMAGE_SEGMENTATION_CSF_BLUE_COMPONENT    = "Image segmentation CSF blue";
    public final static String TISSUE_TYPE_ORDER                        = "Tissue type ordering";
    
    
    public final static String HLSVD_FILTER_TYPE                = "HLSVD filter type";
    
    public final static String LIBHLSVD_DLL_PATH                = "HLSVD installation path";
    
    public final static String CALL_FORTRAN_PATH_AND_DLL_NAME   = "Call fortran dll path and name";
    public final static String LIB_HLSVD_PATH_AND_DLL_NAME      = "Libhlslvd dll path and name";
    
    public final static String HLSVD_FILTER_WATER_ONLY                     = "Water only";
    public final static String HLSVD_FILTER_EXTREME_BROAD_LINES_ONLY       = "Extreme broad components";
    public final static String HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES  = "Water and extreme broad components";
    
    public final static String HLSVDPRO_VERSION                            = "HLSVDPro version"; 
    public final static String HLSVDPRO_VERSION_PARALLEL                   = "HLSVDPro multithreading version";
    public final static String HLSVDPRO_VERSION_SERIAL                     = "HLSVDPro serial version";
    
    // Attributes needed to handle output file elimination 
    // after TDFDFit has been applied
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN              = "Delete TDFDFit .bin file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT          = "Delete TDFDFit .bin.fit file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0         = "Delete TDFDFit .bin.fit0 file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG          = "Delete TDFDFit .bin.org file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES       = "Delete TDFDFit .bin.fitres file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_RES          = "Delete TDFDFit .bin.res file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG       = "Delete TDFDFit .bin.cororg file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL          = "Delete TDFDFit .bin.exl file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0         = "Delete TDFDFit .bin.nln0 file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT          = "Delete TDFDFit .bin.mat file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR          = "Delete TDFDFit .bin.err file";
    
    public final static String DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR          = "Delete TDFDFit .par file";
    public final static String DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR  = "Delete TDFDFit verbose .par file";
    
    // Attributes needed to handle storage into one single file 
    // after TDFDFit has been applied
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN        = "Store TDFDFit .bin file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_FIT    = "Store TDFDFit .bin.fit file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_FIT0   = "Store TDFDFit .bin.fit0 file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_ORG    = "Store TDFDFit .bin.org file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_FITRES = "Store TDFDFit .bin.fitres file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_RES    = "Store TDFDFit .bin.res file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_CORORG = "Store TDFDFit .bin.cororg file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_EXL    = "Store TDFDFit .bin.exl file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_NLN0   = "Store TDFDFit .bin.nln0 file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_MAT    = "Store TDFDFit .bin.mat file in one single file";
    public final static String STORE_SINGLE_TDFDFIT_OUTPUTFILE_BIN_ERR    = "Store TDFDFit .bin.err file in one single file";
    
    public final static String  SELECTED_EPSI_BINARY_FILE_REFERENCE_FIRST  = "Selected EPSI binary file reference first";
    public final static String  SELECTED_EPSI_BINARY_FILE_REFERENCE_SECOND = "Selected EPSI binary file reference second";
    
// For proper serialization
    private static final long serialVersionUID = 8882232191119213L;
}
