/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import org.spectrim.lib.StringComparator;
import org.spectrim.lib.TextFileWriter;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 *
 * This class stores the values for the T1 and T2 relaxation times
 *
 * Currently only 1.5T and 3.0T are implemented
 * Currently 1H (protons) only.
 *
 * @author slotboom
 */
public class MetaboliteRelaxationTimesDatabase
{

    /**
     * Default constructor of MetaboliteRelaxationTimesDatabase.
     */
    private MetaboliteRelaxationTimesDatabase( )
    {
        m_fieldstrength = ONE_POINT_FIVE_TESLA;

        addDefaultZeroPointFiveTeslaDatabase( );

        addDefaultOnePointZeroTeslaDatabase( );

        addDefaultOnePointFiveTeslaDatabase( );

        addDefaultTwoTeslaDatabase( );

        addDefaultThreeTeslaDatabase( );

        addDefaultFourTeslaDatabase( );

        addDefaultSixPointThreeTeslaDatabase( );

        addDefaultSevenTeslaDatabase( );

        // System.out.println( "End of private constructor \n Number of databases: " + m_allAvailableDataBases.keySet( ).size( ) );
    }

    /**
     * Returns the one and only instance of MetaboliteRelaxationTimesDatabase.
     * @return theInstance
     */
    public static MetaboliteRelaxationTimesDatabase getInstance( )
    {

        if ( theInstance == null )
        {
            theInstance = new MetaboliteRelaxationTimesDatabase( );

            TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            String dBaseName = defaults.getProperty( TDFDFittingDefaults.TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE );
            
            if ( new File( dBaseName ).exists( ) == false )
            {
                // Write the default data base into the working directory
                saveRelaxationTimeDataBase( );
            }
            else
            {
                loadDefaultRelaxationTimeDataBase( );
            }
        }

        return theInstance;
    }

    /**
     *
     * Returns a vector of metabolite names.
     *
     * @return
     */
    private Vector<String> getProtonMetaboliteNameList( )
    {
        m_metaboliteNameList = new Vector<>( );
        m_metaboliteNameList.add( ACE );
        m_metaboliteNameList.add( ATP );
        m_metaboliteNameList.add( ALA );
        m_metaboliteNameList.add( ASP );
        m_metaboliteNameList.add( CHO );
        m_metaboliteNameList.add( CR );
        m_metaboliteNameList.add( CYS );
        m_metaboliteNameList.add( GABA );
        m_metaboliteNameList.add( GLC );
        m_metaboliteNameList.add( GLU );
        m_metaboliteNameList.add( GLN );
        m_metaboliteNameList.add( GSH );
        m_metaboliteNameList.add( GLYCEROL );
        m_metaboliteNameList.add( GLY );
        m_metaboliteNameList.add( GPC );
        m_metaboliteNameList.add( ILEU );
        m_metaboliteNameList.add( ETA );
        m_metaboliteNameList.add( ETH );
        m_metaboliteNameList.add( LEU );
        m_metaboliteNameList.add( MA );
        m_metaboliteNameList.add( NAA );
        m_metaboliteNameList.add( NAAG );
        m_metaboliteNameList.add( PE );
        m_metaboliteNameList.add( MINS );
        m_metaboliteNameList.add( HISTAMINE );
        m_metaboliteNameList.add( HIS );
        m_metaboliteNameList.add( HOMOCARSINE );
        m_metaboliteNameList.add( PCR );
        m_metaboliteNameList.add( PC );
        m_metaboliteNameList.add( SINS );
        m_metaboliteNameList.add( LAC );
        m_metaboliteNameList.add( PHE );
        m_metaboliteNameList.add( PYRUVATE );
        m_metaboliteNameList.add( SER );
        m_metaboliteNameList.add( SUC );
        m_metaboliteNameList.add( TAU );
        m_metaboliteNameList.add( THR );
        m_metaboliteNameList.add( TRP );
        m_metaboliteNameList.add( TYR );
        m_metaboliteNameList.add( VAL );
        
        Collections.sort( m_metaboliteNameList, new StringComparator( ) );
        return m_metaboliteNameList;
    }

    /**
     * Adds the 0.5T metabolite database to the list of databases.
     */
    private void addDefaultZeroPointFiveTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 2.000 , 0.300, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 2.000 , 0.300, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 2.000 , 0.300, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 2.000 , 0.300, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, -1.000 , 0.300, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 2.000 , 0.300, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 2.000 , 0.300, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 2.000 , 0.300, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 2.000 , 0.300, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 2.000 , 0.300, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 2.000 , 0.300, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 2.000 , 0.300, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 2.000 , 0.300, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 2.000 , 0.300, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 2.000 , 0.300, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 2.000 , 0.300, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 2.000 , 0.300, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 2.000 , 0.300, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 2.000 , 0.300, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 2.000 , 0.300, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 2.000 , 0.300, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 2.000 , 0.300, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 2.000 , 0.300, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 2.000 , 0.300, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 2.000 , 0.275, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 2.000 , 0.300, 1) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 2.000 , 0.300, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 2.000 , 0.300, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 2.000 , 0.300, 1 ) );

        m_allAvailableDataBases.put( HALVE_TESLA_STRING, newDataBase );
    }

    /**
     * Adds a 1.5T data relaxation time base to the list of databases.
     */
    private void addDefaultOnePointZeroTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 2.000 , 0.300, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 2.000 , 0.300, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 2.000 , 0.300, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 2.000 , 0.300, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 2.000 , 0.300, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 2.000 , 0.300, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 2.000 , 0.300, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 2.000 , 0.300, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 2.000 , 0.300, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 2.000 , 0.300, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 2.000 , 0.300, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 2.000 , 0.300, 1 ) );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 2.000 , 0.300, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 2.000 , 0.300, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 2.000 , 0.300, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 2.000 , 0.300, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 2.000 , 0.300, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 2.000 , 0.300, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 2.000 , 0.300, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 2.000 , 0.300, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 2.000 , 0.300, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 2.000 , 0.300, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 2.000 , 0.300, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 2.000 , 0.300, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 2.000 , 0.300, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 2.000 , 0.300, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 2.000 , 0.300, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 2.000 , 0.300, 1) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 2.000 , 0.300, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 2.000 , 0.300, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 2.000 , 0.300, 1 ) );

        m_allAvailableDataBases.put( ONE_TESLA_STRING, newDataBase );
    }

    /**
     * Adds a 1.5T data relaxation time base to the list of databases.
     */
    private void addDefaultOnePointFiveTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 1.321 , 0.275, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 2.00 , 0.300, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 1.321 , 0.275, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 1.321 , 0.275, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 1.369 , 0.346, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 1.424 , 0.217, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 2.000 , 0.300, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 1.321 , 0.275, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 1.321 , 0.275, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 1.321 , 0.275, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 1.321 , 0.275, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 1.321 , 0.275, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 1.369 , 0.346, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 1.321 , 0.275, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 2.000 , 0.300, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 1.368 , 0.376, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 1.368 , 0.376, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 1.368 , 0.376, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 1.368 , 0.376, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 1.321 , 0.275, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 1.124 , 0.162, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 2.000 , 0.300, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 2.000 , 0.3000, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 1.424 , 0.217, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 1.369 , 0.346, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 1.124 , 0.162, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 1.321 , 0.275, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 2.000 , 0.300, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 1.321 , 0.275, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 2.000 , 0.300, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 1.321 , 0.275, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 1.321 , 0.275, 1 ) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 2.000 , 0.300, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 2.000 , 0.300, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 1.321 , 0.275, 1 ) );

        m_allAvailableDataBases.put( ONE_POINT_FIVE_TESLA_STRING, newDataBase );
    }

    /**
     * Adds a 2T data relaxation time base to the list of databases.
     */
    private void addDefaultTwoTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 1.321 , 0.275, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 2.00 , 0.300, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 1.321 , 0.275, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 1.321 , 0.275, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 1.369 , 0.346, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 1.424 , 0.217, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 2.000 , 0.300, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 1.321 , 0.275, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 1.321 , 0.275, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 1.321 , 0.275, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 1.321 , 0.275, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 2.000 , 0.300, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 1.321 , 0.275, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 1.369 , 0.346, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 2.000 , 0.300, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 2.000 , 0.300, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 2.000 , 0.300, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 2.000 , 0.300, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 1.368 , 0.376, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 1.368 , 0.376, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 1.368 , 0.376, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 1.368 , 0.376, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 1.321 , 0.275, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 1.124 , 0.162, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 2.000 , 0.300, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 1.424 , 0.217, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 1.369 , 0.346, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 1.124 , 0.162, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 1.321 , 0.275, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 2.000 , 0.300, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 2.000 , 0.300, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 1.321 , 0.275, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 2.000 , 0.300, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 1.321 , 0.275, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 1.321 , 0.275, 1 ) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 2.000 , 0.300, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 2.000 , 0.300, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 1.321 , 0.275, 1 ) );

        m_allAvailableDataBases.put( TWO_TESLA_STRING, newDataBase );
    }

    /**
     * Adds a 3T data relaxation time base to the list of databases.
     */
    private void addDefaultThreeTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 1.100 , 0.219, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 1.100 , 0.219, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 1.100 , 0.219, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 1.100 , 0.219, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 0.810 , 0.250, 1  ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 0.800 , 0.155, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 1.100 , 0.180, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 1.100 , 0.219, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 1.100 , 0.219, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 0.505 , 0.380, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 0.505 , 0.380, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 1.100 , 0.219, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 1.100 , 0.219, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 1.100 , 0.219, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 1.100 , 0.219, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 1.100 , 0.219, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 1.100 , 0.219, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 1.100 , 0.219, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 1.100 , 0.219, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 1.100 , 0.219, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 1.365 , 0.292, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 2.00 , 0.300, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 3.00 , 0.300, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 1.100 , 0.219, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 1.100 , 0.219, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 0.750 , 0.150, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 1.100 , 0.219, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 1.100 , 0.219, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 1.100 , 0.219, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 1.100 , 0.219, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 1.100 , 0.219, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 1.100 , 0.219, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 0.700 , 0.205, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 1.100 , 0.219, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 1.100 , 0.219, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 1.100 , 0.219, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 1.100 , 0.219, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 1.100 , 0.219, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 1.100 , 0.275, 1 ) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 1.100 , 0.219, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 1.100 , 0.219, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 1.100 , 0.219, 1 ) );

        m_allAvailableDataBases.put( THREE_TESLA_STRING, newDataBase );
    }

    /**
     * Adds a 4T data relaxation time base to the list of databases.
     */
    private void addDefaultFourTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 2.500 , 0.250, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 2.500 , 0.250, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 2.500 , 0.250, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 2.500 , 0.250, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 2.500 , 0.250, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 2.500 , 0.250, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 2.500 , 0.250, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 2.500 , 0.250, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 2.500 , 0.250, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 2.500 , 0.250, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 2.500 , 0.250, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 2.500 , 0.250, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 2.500 , 0.250, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 2.500 , 0.250, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 2.500 , 0.250, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 2.500 , 0.250, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 2.500 , 0.250, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 2.500 , 0.250, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 2.500 , 0.250, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 2.500 , 0.250, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 2.500 , 0.250, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 2.500 , 0.250, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 2.500 , 0.250, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 2.500 , 0.250, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 2.500 , 0.250, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 2.500 , 0.250, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 2.500 , 0.250, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 2.500 , 0.250, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 2.500 , 0.250, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 2.500 , 0.250, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 2.500 , 0.250, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 2.500 , 0.250, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 2.500 , 0.250, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 2.500 , 0.250, 1) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 2.500 , 0.250, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 2.500 , 0.250, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 2.500 , 0.250, 1 ) );

        m_allAvailableDataBases.put( FOUR_TESLA_STRING, newDataBase );
    }

    /**
     * Adds a 6.3T data relaxation time base to the list of databases.
     */
    private void addDefaultSixPointThreeTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 2.500 , 0.250, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 2.500 , 0.250, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 2.500 , 0.250, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 2.500 , 0.250, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 2.500 , 0.250, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 2.500 , 0.250, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 2.500 , 0.250, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 2.500 , 0.250, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 2.500 , 0.250, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 2.500 , 0.250, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 2.500 , 0.250, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 2.500 , 0.250, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 2.500 , 0.250, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 2.500 , 0.250, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 2.500 , 0.250, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 2.500 , 0.250, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 2.500 , 0.250, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 2.500 , 0.250, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 2.500 , 0.250, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 2.500 , 0.250, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 2.500 , 0.250, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 2.500 , 0.250, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 2.500 , 0.250, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 2.500 , 0.250, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 2.500 , 0.250, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 2.500 , 0.250, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 2.500 , 0.250, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 2.500 , 0.250, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 2.500 , 0.250, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 2.500 , 0.250, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 2.500 , 0.250, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 2.500 , 0.250, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 2.500 , 0.250, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 2.500 , 0.250, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 2.500 , 0.250, 1) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 2.500 , 0.250, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 2.500 , 0.250, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 2.500 , 0.250, 1 ) );

        m_allAvailableDataBases.put( SIX_POINT_THREE_TESLA_STRING, newDataBase );
    }


    /**
     * Adds a 7T data relaxation time base to the list of databases.
     */
    private void addDefaultSevenTeslaDatabase( )
    {

        TreeMap<String, MetaboliteRelaxationTimeProperties> newDataBase = new TreeMap<>( );

        newDataBase.put( ACE, new MetaboliteRelaxationTimeProperties( ACE, 3.000 , 0.250, 1 ) );
        newDataBase.put( ATP, new MetaboliteRelaxationTimeProperties( ATP, 3.000 , 0.250, 1 ) );
        newDataBase.put( ALA, new MetaboliteRelaxationTimeProperties( ALA, 3.000 , 0.250, 1 ) );
        newDataBase.put( ASP, new MetaboliteRelaxationTimeProperties( ASP, 3.000 , 0.250, 1 ) );
        newDataBase.put( CHO, new MetaboliteRelaxationTimeProperties( CHO, 3.000 , 0.250, 1 ) );
        newDataBase.put( CR,  new MetaboliteRelaxationTimeProperties( CR , 3.000 , 0.250, 1 ) );
        newDataBase.put( CYS, new MetaboliteRelaxationTimeProperties( CYS, 3.000 , 0.250, 1 ) );
        newDataBase.put( GABA, new MetaboliteRelaxationTimeProperties( GABA, 3.000 , 0.250, 1 ) );
        newDataBase.put( GLC, new MetaboliteRelaxationTimeProperties( GLC, 3.000 , 0.250, 1 ) );
        newDataBase.put( GLU, new MetaboliteRelaxationTimeProperties( GLU, 3.000 , 0.250, 1 ) );
        newDataBase.put( GLN, new MetaboliteRelaxationTimeProperties( GLN, 3.000 , 0.250, 1 ) );
        newDataBase.put( GSH, new MetaboliteRelaxationTimeProperties( GSH, 3.000 , 0.250, 1 ) );
        newDataBase.put( GLYCEROL,  new MetaboliteRelaxationTimeProperties( GLYCEROL, 3.000 , 0.250, 1 ) );
        newDataBase.put( GLY,  new MetaboliteRelaxationTimeProperties( GLY, 3.000 , 0.250, 1 ) );
        newDataBase.put( GPC,  new MetaboliteRelaxationTimeProperties( GPC, 3.000 , 0.250, 1 ) );
        newDataBase.put( ILEU,  new MetaboliteRelaxationTimeProperties( ILEU, 3.000 , 0.250, 1 ) );
        newDataBase.put( ETA,  new MetaboliteRelaxationTimeProperties( ETA, 3.000 , 0.250, 1 ) );
        newDataBase.put( ETH,  new MetaboliteRelaxationTimeProperties( ETH, 3.000 , 0.250, 1 ) );
        newDataBase.put( LEU,  new MetaboliteRelaxationTimeProperties( LEU, 3.000 , 0.250, 1 ) );
        newDataBase.put( MA,  new MetaboliteRelaxationTimeProperties( MA, 3.000 , 0.250, 1 )  );
        newDataBase.put( NAA,  new MetaboliteRelaxationTimeProperties( NAA, 3.000 , 0.250, 1 ) );
        newDataBase.put( NAA_Ace,  new MetaboliteRelaxationTimeProperties( NAA_Ace, 3.000 , 0.250, 1 ) );
        newDataBase.put( NAA_Asp,  new MetaboliteRelaxationTimeProperties( NAA_Asp, 3.000 , 0.250, 1 ) );
        newDataBase.put( NAAG,  new MetaboliteRelaxationTimeProperties( NAAG, 3.000 , 0.250, 1 ) );
        newDataBase.put( PE,  new MetaboliteRelaxationTimeProperties( PE, 3.000 , 0.250, 1 ) );
        newDataBase.put( MINS,  new MetaboliteRelaxationTimeProperties( MINS, 3.000 , 0.250, 1 ) );
        newDataBase.put( HISTAMINE,  new MetaboliteRelaxationTimeProperties( HISTAMINE, 3.000 , 0.250, 1 ) );
        newDataBase.put( HIS,  new MetaboliteRelaxationTimeProperties( HIS, 3.000 , 0.250, 1 ) );
        newDataBase.put( HOMOCARSINE,  new MetaboliteRelaxationTimeProperties( HOMOCARSINE, 3.000 , 0.250, 1 ) );
        newDataBase.put( PCR,  new MetaboliteRelaxationTimeProperties( PCR, 3.000 , 0.250, 1 ) );
        newDataBase.put( PC,  new MetaboliteRelaxationTimeProperties( PC, 3.000 , 0.250, 1 ) );
        newDataBase.put( SINS,  new MetaboliteRelaxationTimeProperties( SINS, 3.000 , 0.250, 1 ) );
        newDataBase.put( LAC,  new MetaboliteRelaxationTimeProperties( LAC, 3.000 , 0.250, 1 ) );
        newDataBase.put( PHE,  new MetaboliteRelaxationTimeProperties( PHE, 3.000 , 0.250, 1 ) );
        newDataBase.put( PYRUVATE,  new MetaboliteRelaxationTimeProperties( PYRUVATE, 3.000 , 0.250, 1 ) );
        newDataBase.put( SER,  new MetaboliteRelaxationTimeProperties( SER, 3.000 , 0.250, 1 ) );
        newDataBase.put( SUC,  new MetaboliteRelaxationTimeProperties( SUC, 3.000 , 0.250, 1 ) );
        newDataBase.put( TAU,  new MetaboliteRelaxationTimeProperties( TAU, 3.000 , 0.250, 1 ) );
        newDataBase.put( THR,  new MetaboliteRelaxationTimeProperties( THR, 3.000 , 0.250, 1) );
        newDataBase.put( TRP,  new MetaboliteRelaxationTimeProperties( TRP, 3.000 , 0.250, 1 ) );
        newDataBase.put( TYR,  new MetaboliteRelaxationTimeProperties( TYR, 3.000 , 0.250, 1 ) );
        newDataBase.put( VAL,  new MetaboliteRelaxationTimeProperties( VAL, 3.000 , 0.250, 1 ) );

        m_allAvailableDataBases.put( SEVEN_TESLA_STRING, newDataBase );
    }

    /**
     * Return the relaxation database for the metabolites belonging to the
     * given field strength string
     * @param dataBaseIDString
     * @return
     */
    public TreeMap<String, MetaboliteRelaxationTimeProperties> getDataBase( String dataBaseIDString )
    {

        //System.out.println( "dataBaseIDString : " + dataBaseIDString );

        Collection c = m_allAvailableDataBases.keySet( );
        Iterator it = c.iterator( );

        while ( it.hasNext( ) )
        {
            String key = (String)it.next( );
            if ( dataBaseIDString.compareTo( key ) == 0 )
            {
                // System.out.println( "Found data base : " + key );
                return m_allAvailableDataBases.get( key );
            }
        }

        return null;

    }

    /**
     * Saves the relaxation time database to a file name relaxationTimedBaseName.
     */
    public static void saveRelaxationTimeDataBase( )
    {
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String dBaseFileName = defaults.getProperty( TDFDFittingDefaults.TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE );
//
//      String dBaseFileName = System.getProperty( "user.home" ) + 
//                               System.getProperty( "file.separator" ) + 
//                               "TDFDFitMetaboliteRelaxationTimesDatabase.txt";
//        
//        defaults.setProperty( TDFDFittingDefaults.TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE, dBaseFileName );
        
        System.out.println( "Writing : " + dBaseFileName );

        TextFileWriter writer;
        try
        {
            writer = new TextFileWriter( new FileWriter( dBaseFileName ) );

            // System.out.println( "Writing to : " + dBaseName );

            writer.writeStringLn( "**************************************************");
            writer.writeStringLn( "*  T1, T2 Relaxation time data base.             *");
            writer.writeStringLn( "*  File generated by SpectrIm/TDFDFit.           *");
            writer.writeStringLn( "*  Do not edit!!.                                *");
            writer.writeStringLn( "**************************************************");

            Date today = new Date( );
            SimpleDateFormat formatter = new SimpleDateFormat( "dd-MMM-yyyy" );
            String dateStr = formatter.format( today );
            writer.writeStringLn( "* Date: " + dateStr + "                              *" );
            writer.writeStringLn( "**************************************************");
            writer.writeStringLn( "" );

            for ( String key : m_allAvailableDataBases.keySet( ) ) 
            {
                TreeMap<String, MetaboliteRelaxationTimeProperties> selectedDataBase = m_allAvailableDataBases.get( key );

                //System.out.println( "Writing db for fieldstrength : " + key );

                writer.writeStringLn( "Database_for_field_strength_: "  + key );
                writer.writeStringLn( "===================================" );

                Iterator dBaseIterator = selectedDataBase.keySet( ).iterator( );
                while( dBaseIterator.hasNext( ) )
                {
                    String metab_key = (String) dBaseIterator.next( );
                    MetaboliteRelaxationTimeProperties props = selectedDataBase.get( metab_key );
                    writer.write( metab_key + "\t" + props.getT1( ) + "\t" + props.getT2( ) + "\t" + props.getMultiplicity( ) + "\n" );
                }

                writer.writeStringLn( "==========================================" );
                writer.writeStringLn( "" );
            }
            writer.writeStringLn( "end_of_data_base" );
            writer.close( );

        }
        catch ( IOException ex )
        {
            Logger.getLogger( MetaboliteRelaxationTimesDatabase.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

    }

    /**
     * This routine loads the default database which can be altered by the
     * user and saved again.
     */
    public static void loadDefaultRelaxationTimeDataBase( )
    {

        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String dBaseName = defaults.getProperty( TDFDFittingDefaults.TDFDFIT_RELAXATION_TIMES_DATABASE_PATH_AND_FILE );
            
        System.out.println( "Loading T1, T2 database : " + dBaseName );

        if ( new File( dBaseName ).exists( ) == true )
        {

            InputStream ist = null;
            try
            {
                
                // it is loaded, so lets instantiate a new data containing object..
                m_allAvailableDataBases = new TreeMap<String, TreeMap<String, MetaboliteRelaxationTimeProperties>>( );
                TreeMap<String, MetaboliteRelaxationTimeProperties> currentDatabase = null;

                ist = new FileInputStream( dBaseName );
                BufferedReader istream = new BufferedReader( new InputStreamReader( ist ) );
                StringTokenizer st;

                String text = istream.readLine( );
                String dBaseKey = "";

                //currentDatabase = new TreeMap<String, MetaboliteRelaxationTimeProperties>( );
                
                do
                {
                    if ( text.contains( "Database_for_field_strength_:" ) == true )
                    {
                        // Start a new database for belonging to this field strength
                        st = new StringTokenizer( text );
                        String dummy = st.nextToken( );
                        dBaseKey = st.nextToken( );
                        currentDatabase = new TreeMap<String, MetaboliteRelaxationTimeProperties>( );
                        //System.out.println( "Read dBaseKey : " + dBaseKey );
                    }
                    else
                    {
                        if ( ( text.contains( "*" ) == false ) &&
                             ( text.contains( "=" ) == false ) &&
                             ( text.length( ) > 0 )            )
                        {
                            st = new StringTokenizer( text );
                            int length = st.countTokens( );
                            int cntr = 1;

                            String metabName = "";
                            double T1           = 0.0;
                            double T2           = 0.0;
                            int    multiplicity = 1;
                            
                            while ( st.hasMoreTokens( ) == true )
                            {
                                if ( cntr < length - 2 )
                                {
                                    metabName = metabName + " " + st.nextToken( );
                                }

                                if ( cntr == ( length - 2 ) )
                                {
                                    T1 = new Double( st.nextToken( ) ).doubleValue( );
                                }

                                if ( cntr == length -1 )
                                {
                                    T2 = new Double( st.nextToken( ) ).doubleValue( );
                                }
                                
                                if ( cntr == length  )
                                {
                                    multiplicity = new Integer( st.nextToken( ) ).intValue( );
                                }

                                cntr++;
                            }
                            
                            // System.out.println( "Name : \t" + metabName + " \t " + T1 +  " \t " + T2 );

                            MetaboliteRelaxationTimeProperties metabProp =  new MetaboliteRelaxationTimeProperties( metabName.trim( ), T1, T2, multiplicity );
                            currentDatabase.put( metabName.trim( ), metabProp );
                        }
                    }

                    if ( dBaseKey.length( ) > 0 )
                    {
                        m_allAvailableDataBases.put( dBaseKey, currentDatabase );
                    }

                    text = istream.readLine( );

                } while ( text.compareTo( "end_of_data_base" ) != 0 );
            }
            catch ( IOException ex )
            {
                Logger.getLogger( MetaboliteRelaxationTimesDatabase.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
            finally
            {
                try
                {
                    ist.close( );
                }
                catch ( IOException ex )
                {
                    Logger.getLogger( MetaboliteRelaxationTimesDatabase.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
            }

            // write complete database for checking to stdio...

            // System.out.println( "Checking loaded data bases from file .." );
            // System.out.println( "=======================================" );

            Iterator it = m_allAvailableDataBases.keySet( ).iterator( );
            while ( it.hasNext( ) )
            {
                String dataBaseKey2 = (String) it.next( );
                //System.out.println( "Database key = " + dataBaseKey2 );
                TreeMap<String, MetaboliteRelaxationTimeProperties> testDatabase = m_allAvailableDataBases.get( dataBaseKey2 );
                Iterator it2 = testDatabase.keySet( ).iterator( );
                while ( it2.hasNext( ) )
                {
                    String metabKey = (String)it2.next( );
                    MetaboliteRelaxationTimeProperties props = testDatabase.get( metabKey );
                    //System.out.println( props.getMetaboliteName( ) + "\t\t" + props.getT1( ) + "\t\t" + props.getT2( ) );
                }
            }
        }
        else
        {
            saveRelaxationTimeDataBase( );
        }
    }
    
    // Known metabolites of jMRUI
    private static final String ACE = "Acetate_Ace)";
    private static final String ATP = "Adenosine_triphosphate_(ATP)";
    private static final String ALA = "Alanine_(Ala)";
    private static final String ASP = "Aspartate_(Asp)";
    private static final String CHO = "Choline_(Cho)";
    private static final String CR = "Creatine_(Cr)";
    private static final String CYS = "Cysteine_(Cys)";
    private static final String ETA = "Ethanol_(EtOH)";
    private static final String ETH = "Ethanolamine_(Eta)";
    private static final String GABA = "GammaAminobutyricAcid_(GABA)";
    private static final String GLC = "Glucose_(Glc)";
    private static final String GLU = "Glutamate_(Glu)";
    private static final String GLN = "Glutamine_(Gln)";
    private static final String GSH = "Glutathione_(GSH)";
    private static final String GLYCEROL = "Glycerol";
    private static final String GLY = "Glycine_(Gly)";
    private static final String GPC = "Glycerophosphorylcholine_(GPC)";
    private static final String ILEU = "Isoleucine_(I-Leu)";
    private static final String LEU = "Leucine_(Leu)";
    private static final String MA = "Malate_(Ma)";
    private static final String NAA = "NAcetylAspartate_(NAA)";
    private static final String NAA_Ace = "NAcetylAspartate_Acetyl_(NAA_Ace)";
    private static final String NAA_Asp = "NAcetylAspartate_Aspartate_(NAA_Asp)";
    private static final String NAAG = "NAcetylAspartylGlutamate_(NAAG)";
    private static final String PE = "Phosphoethanolamine_(PE)";
    private static final String MINS = "MyoInositol_(m-Ins)";
    private static final String HISTAMINE = "Histamine";
    private static final String HIS = "Histidine (His)";
    private static final String HOMOCARSINE = "Homocarnosine";
    private static final String PCR = "Phosphocreatine_(PCr)";
    private static final String PC = "Phosphorylcholine_(PC)";
    private static final String SINS = "ScylloInositol_(s-Ins)";
    private static final String LAC = "Lactate_(Lac)";
    private static final String PHE = "Phenylalanine_(Phe)";
    private static final String PYRUVATE = "Pyruvate";
    private static final String SER = "Serine_(Ser)";
    private static final String SUC = "Succinate_(Suc)";
    private static final String TAU = "Taurine_(Tau)";
    private static final String THR = "Threonine_(Thr)";
    private static final String TRP = "Tryptophan_(Trp)";
    private static final String TYR = "Tyrosine_(Tyr)";
    private static final String VAL = "Valine_(Val)";

    //private static final String MA = "Malate (Ma)";
    //private static final String LEU = "Leucine (Leu)";
    //private static final String ILEU = "Isoleucine (I-Leu)";


    // Attributes of the database class
    public int m_nucleusType =  PROTON;

    // predefined nuclei types
    public final static int PROTON                  = 0;
    public final static int PHOSPORUS31             = 1;
    public final static int CARBON13                = 2;
    public final static int SODIUM23                = 3;

    // Relaxation times are field strength dependent
    private int m_fieldstrength = ONE_POINT_FIVE_TESLA;

    public final static int HALVE_TESLA             = 0;
    public final static int ONE_TESLA               = 1;
    public final static int ONE_POINT_FIVE_TESLA    = 2;
    public final static int TWO_TESLA               = 3;
    public final static int THREE_TESLA             = 4;
    public final static int FOUR_TESLA              = 5;
    public final static int SIX_POINT_THREE_TESLA   = 6;
    public final static int SEVEN_TESLA             = 7;


    public final static String HALVE_TESLA_STRING             = "0.5T";
    public final static String ONE_TESLA_STRING               = "1.0T";
    public final static String ONE_POINT_FIVE_TESLA_STRING    = "1.5T";
    public final static String TWO_TESLA_STRING               = "2.0T";
    public final static String THREE_TESLA_STRING             = "3.0T";
    public final static String FOUR_TESLA_STRING              = "4.0T";
    public final static String SIX_POINT_THREE_TESLA_STRING   = "6.3T";
    public final static String SEVEN_TESLA_STRING             = "7.0T";

    private Vector<String> m_metaboliteNameList = new Vector<>( );
    
    private static TreeMap<String, TreeMap<String, MetaboliteRelaxationTimeProperties> > m_allAvailableDataBases = new TreeMap< >( );
    
    private static MetaboliteRelaxationTimesDatabase theInstance;

}
