/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.FileInfo;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.spectrim.model.GlobalModel;

/**
 * Class that handles the creation of a spectroscopy report file: displays the
 * results dynamically in a JPanel and puts all patient information and metabolite
 * report on the panel.
 *
 * @author slotboom
 */
public class SVSMetaboliteTextReport extends JPanel
{
    /**
     * Default constructor of SVSMetaboliteTextReport.
     */
    public SVSMetaboliteTextReport( )
    {
        this.setPreferredSize( new Dimension( 1024, 800 ) );
        this.getGraphics( ).setColor( Color.BLACK );
    }

    /**
     * Alternative constructor defining the size of the panel.
     * @param d the dimensions of the Panel
     */
    public SVSMetaboliteTextReport( Dimension d )
    {
        this.setPreferredSize( d );
        this.getGraphics( ).setColor( Color.BLACK );
    }
 

    /**
     * Alternative constructor defining the size of the panel.
     * @param d the dimensions of the Panel
     * @param rawSpectrosopyDicomFileName
     * @param waterRefernceFileName
     * @param numberOfMetabolites
     * @param metaboliteNames
     * @param concentrations
     * @param errorBars
     * @param normValues
     * @param errorsNormValues
     * @param outputStyle
     */
    public SVSMetaboliteTextReport( Dimension d, String rawSpectrosopyDicomFileName,
                                    String waterRefernceFileName, int numberOfMetabolites,
                                    LinkedList<String> metaboliteNames, 
                                    double[] concentrations, double[] errorBars,
                                    double[] normValues, double[] errorsNormValues,
                                    int outputStyle,
                                    GlobalModel globalModel
                                  )
    {

        m_outputStyle = outputStyle;
        m_dimensionOfControl = d;

        setPreferredSize( d );
        

        m_metaboliteNames = metaboliteNames;

        m_metabConcentrations = concentrations;
        m_metaboliteConcentrationErrorBars = errorBars;
        m_metaboliteNormConcentrations = normValues;
        m_metaboliteNormConcentrationsErrorBar = errorsNormValues;

        m_numberOfMetabolites = concentrations.length;

        m_rawSpectroscopyFileReference = rawSpectrosopyDicomFileName;
        m_waterReferenceFileName = waterRefernceFileName;
        m_globalModel = globalModel;
    }

    /**
     * Sets the size of the
     * @param dim
     */
    public void setDimension( Dimension dim )
    {
        m_dimensionOfControl = dim;
    }

    /**
     * Returns the Dimension of the control.
     *
     * @return 
     */
    public Dimension getDimension( )
    {
        return m_dimensionOfControl;

    }

    /**
     * Sets the concentration array elements.
     * @param concentrations
     */
    public void setMetaboliteConcentrations( double[] concentrations )
    {
        m_numberOfMetabolites = concentrations.length;
        m_metabConcentrations = new double[concentrations.length];

        System.arraycopy( concentrations, concentrations.length, m_metabConcentrations, 0, concentrations.length);
    }

    /**
     * Returns the metabolite concentrations
     * @return
     */
    public double[] getMetaboliteConcentrations( )
    {
        return m_metabConcentrations;
    }


    /**
     * Sets the concentration array error values.
     * @param errBar
     */
    public void setMetaboliteErrorBars( double[] errBar )
    {
        m_numberOfMetabolites = errBar.length;
        m_metaboliteConcentrationErrorBars = new double[errBar.length];

        System.arraycopy( errBar, errBar.length, m_metaboliteConcentrationErrorBars, 0, errBar.length );
    }

    /**
     * Returns the metabolite concentrations.
     * @return
     */
    public double[] getMetaboliteErrorBars( )
    {
        return m_metaboliteConcentrationErrorBars;
    }

     /**
     * Sets the concentration array elements.
     * @param normConcentration
     */
    public void setMetaboliteNormalConcentrations( double[] normConcentration )
    {
        m_numberOfMetabolites = normConcentration.length;
        m_metaboliteNormConcentrations  = new double[normConcentration.length];

        System.arraycopy( normConcentration, normConcentration.length,
                          m_metaboliteConcentrationErrorBars, 0, normConcentration.length );
    }

    /**
     * Returns the metabolite concentrations.
     * @return
     */
    public double[] getMetaboliteNormalConcentrations( )
    {
        return m_metaboliteNormConcentrations;
    }

    /**
     * Sets the reference to the spectroscopy Dicom file.
     * @param fileReference
     */
    public void setRawSpectroscopyFileReference( String fileReference )
    {
        m_rawSpectroscopyFileReference = fileReference;
    }

    /**
     * Returns the file system reference to the raw spectroscopy Dicom file.
     * @return
     */
    public String getRawSpectroscopyFileReference( )
    {
        return m_rawSpectroscopyFileReference;
    }

    /**
     * Sets the reference to the water reference spectroscopy Dicom file.
     * @param waterReferenceFileName
     */
    public void setWaterFileReference( String waterReferenceFileName )
    {
        m_waterReferenceFileName = waterReferenceFileName;
    }

    /**
     * Returns the file system reference to the water reference Dicom file.
     * @return
     */
    public String getWaterFileReference( )
    {
        return m_waterReferenceFileName;
    }

    /**
     * Investigates the given reference file for additional attributes.
     */
    private void determineAdditionalAtttributes( )
    {

        // Investigate the given raw reference file for missing attributes

        DicomImageData data = new DicomImageData( m_rawSpectroscopyFileReference );

        FileInfo finfo = data.getHeader( );

        m_patientName = m_globalModel.getPatientName( );
        m_patientNameDisplayString = m_patientName;

        m_patientBirthDate = m_globalModel.getPatientBirthDate( );
        m_patientSex = m_globalModel.getPatientSex( );
        m_patientNumber = m_globalModel.getPatientID( );
        m_patientNumberDisplayString = m_patientNumber;

        if ( m_patientNumberDisplayString != null )
        {
            if ( m_patientNumberDisplayString.length( ) > 15 )
            {
                // if patient ID is too long, take the last 15 characters
                m_patientNumberDisplayString = ".." + m_patientNumberDisplayString.substring( m_patientNumberDisplayString.length( ) - 15, m_patientNumberDisplayString.length( ) );
            }
        }

        if ( m_patientNameDisplayString != null )
        {
            if ( m_patientNameDisplayString.length( ) > 30 )
            {
                // if patient ID is too long, take the last 15 characters
                m_patientNameDisplayString = m_patientNameDisplayString.substring( 0, 30 ) + "..";
            }
        }

        m_accessionNumber = m_globalModel.getAccessionNumber( );

        m_studyDescription = m_globalModel.getStudyDescription( );
        m_studyDate = m_globalModel.getStudyDate( );
        m_studyTime = m_globalModel.getStudyTime( );
        
        m_seriesDescription = m_globalModel.getSeriesDescription( );
        m_seriesNumber = m_globalModel.getSeriesNumber( );
        m_seriesTime = m_globalModel.getSeriesTime( );

        // now determine the Siemens specific attributes
//        LinkedHashMap siemensSpecific = data.getSiemensSpecificDataFromFile( );
//        m_resonanceFrequency = ( new Long( (String)( siemensSpecific.get( "sTXSPEC.asNucleusInfo[0].lFrequency" ) ) ) );
        m_resonanceFrequency = new Double( m_globalModel.getResonanceFrequency( ) ).longValue( );
        
        double[] ecxVoxDims = m_globalModel.getCSIData().csigrid.getDimsExcitedVolume();
        
        m_voxelDimensions = "[" +  m_format.format( ecxVoxDims[0] ) + ",";
        m_voxelDimensions += m_format.format( ecxVoxDims[1] ) + ",";
        m_voxelDimensions += m_format.format( ecxVoxDims[2] ) + "]";

        m_TE = "" + m_globalModel.getTE( );
        m_TR = "" + m_globalModel.getTR( );

        m_scannerModelName = m_globalModel.getScannerModelName( );
    }


    /**
     * Does the actual painting of the report.
     */
    @Override
    public synchronized void paintComponent( Graphics g )
    {

        String drawString = "";
        int maxLength = 0;
        int length = 0;
        FontMetrics fm = g.getFontMetrics( );

        int startx = m_dimensionOfControl.width / 40;
        int starty = m_dimensionOfControl.height / 40;


        super.paintComponent( g );

        // determine the additional attributes to be drawn
        determineAdditionalAtttributes( );

        // draw the background black
        g.setColor( Color.BLACK );
        g.fillRect( 0, 0, getWidth( ), getHeight( ) );

        // Set the font color to white
        g.setColor( Color.WHITE );

        // =====================================================================

        drawString = "Patient name  : " + m_patientNameDisplayString;
        length = fm.stringWidth( drawString );
        if (length > maxLength )
        {
            maxLength = length;
        }

        g.drawString( drawString , startx + 10, starty + 15 );

        //=====================================================================

        drawString = "Date of Birth : " + m_patientBirthDate;

        length = fm.stringWidth( drawString );
        if (length > maxLength )
        {
            maxLength = length;
        }

        g.drawString( drawString, startx + 10, starty + 30 );

        // =====================================================================

        drawString = "Patient sex   : " + m_patientSex;

        length = fm.stringWidth( drawString );
        if ( length > maxLength )
        {
            maxLength = length;
        }

        g.drawString( drawString, startx + 10, starty + 45 );

        // =====================================================================

        drawString = "Patient ID    : " + m_patientNumberDisplayString;

        length = fm.stringWidth( drawString );
        if (length > maxLength )
        {
            maxLength = length;
        }

        g.drawString( drawString, startx + 10, starty + 60 );

        // draw a rectangle aroud the info box
        g.drawRect( startx, starty, maxLength + 10, 80 );

        // =====================================================================
        // Now the Exam info box
        // =====================================================================

        int maxLength1 = 0;
        int startx1 = maxLength;
        int starty1 = starty;

        drawString = "Accession number  : " + m_accessionNumber;

        length = fm.stringWidth( drawString );

        if ( length > maxLength1 )
        {
            maxLength1 = length;
        }

        g.drawString( drawString, startx + startx1 + 30, starty1 + 15 );

        //=====================================================================

        drawString = "Study description  : " + m_studyDescription;

        length = fm.stringWidth( drawString );

        if ( length > maxLength1 )
        {
            maxLength1 = length;
        }

        g.drawString( drawString, startx + startx1 + 30, starty1 + 30 );

        //=====================================================================

        drawString = "Study date  : " + m_studyDate;

        length = fm.stringWidth( drawString );

        if ( length > maxLength1 )
        {
            maxLength1 = length;
        }

        g.drawString( drawString, startx + startx1 + 30, starty1 + 45 );

        g.drawRect( startx + maxLength + 10, starty1, maxLength1 + 30, 80 );
        
        //=====================================================================
        // Now the Series info box
        //=====================================================================

        int maxLength2 = 0;
        int startx2 = maxLength + maxLength1;

        drawString = "Series description  : " + m_seriesDescription;

        length = fm.stringWidth( drawString );

        if ( length > maxLength2 )
        {
            maxLength2 = length;
        }

        g.drawString( drawString, startx + maxLength + maxLength1 + 50, starty + 15 );

        //=====================================================================

        drawString = "Series number  : " + m_seriesNumber;

        length = fm.stringWidth( drawString );

        if ( length > maxLength2 )
        {
            maxLength2 = length;
        }

        g.drawString( drawString, startx + maxLength + maxLength1 + 50, starty + 30 );

        //=====================================================================

        drawString = "Series time  : " + m_seriesTime;

        length = fm.stringWidth( drawString );

        if ( length > maxLength2 )
        {
            maxLength2 = length;
        }


        g.drawString( drawString, startx + maxLength + maxLength1 + 50, starty + 45 );

        g.drawRect( startx + maxLength + maxLength1 + 40, starty1, maxLength2 + 10, 80 );

        
        //=====================================================================
        // Now the Spectroscopy info box
        //=====================================================================

        int maxLength3 = 0;
        int startx3 = maxLength + maxLength1 + maxLength2;


        drawString = "Scanner type  : " + m_scannerModelName;

        length = fm.stringWidth( drawString );

        if ( length > maxLength3 )
        {
            maxLength3 = length;
        }

        g.drawString( drawString, startx + maxLength + maxLength1 + maxLength2 + 65, starty + 15 );

        //=====================================================================


        drawString = "Echo time TE  : " + m_TE;

        length = fm.stringWidth( drawString );

        if ( length > maxLength3 )
        {
            maxLength3 = length;
        }
    
        g.drawString( drawString, startx + maxLength + maxLength1 + maxLength2 + 65, starty + 30 );

        //=====================================================================

        drawString = "Repetition time TR  : " + m_TR;

        length = fm.stringWidth( drawString );

        if ( length > maxLength3 )
        {
            maxLength3 = length;
        }

        g.drawString( drawString, startx + maxLength + maxLength1 + maxLength2 + 65, starty + 45 );

        //=====================================================================

        drawString = "Voxel dimension  : " + m_voxelDimensions;

        length = fm.stringWidth( drawString );

        if ( length > maxLength3 )
        {
            maxLength3 = length;
        }

        g.drawString( drawString, startx + maxLength + maxLength1 + maxLength2 + 65, starty + 60 );

        g.drawRect( startx + maxLength + maxLength1 + maxLength2 + 50, starty1, maxLength3 + 20, 80 );
        
        g.setColor( Color.WHITE );

        drawMetaboliteTable( g );
    }

    /**
     * Does the actual drawing of the metabolite table.
     * @param g
     */
    private void drawMetaboliteTable( Graphics g )
    {
        int startx = m_dimensionOfControl.width / 10;
        int starty = m_dimensionOfControl.height / 10;

        int columnwidth = 8 * m_dimensionOfControl.width / ( 3 * 10 );

        String drawString = "Metabolite Name";

        FontMetrics fm = g.getFontMetrics( );

        int length = fm.stringWidth( drawString );

        g.drawString( drawString, startx + (columnwidth/2) - (length/2) , starty + 120 );

        // 2nd column
        String drawString2 = "not defined";
        if ( m_outputStyle == RAW_VALUES )
        {
            drawString2 = "Peak areas (raw)";
        }
        if ( m_outputStyle == RAW_CORRECTED_VALUES )
        {
            drawString2 = "Corrected peak areas";
        }
        if ( m_outputStyle == JMRUI_STYLE_VALUES )
        {
            drawString2 = "Concentrations (jMRUI) [mmol/kg ww]";
        }
        if ( m_outputStyle == TRUE_CONCENTRATIONS_VALUES )
        {
            drawString2 = "Concentrations (2 comp) [mmol/kg ww]";
        }

        int length2 = fm.stringWidth( drawString2 );

        g.drawString( drawString2, startx + (columnwidth) + (columnwidth/2) - (length2/2) , starty + 120 );

        // 3rd column

        String drawString3 = "Normal value [mmol/kg ww]";

        if ( m_outputStyle == RAW_VALUES )
        {
            drawString3 = "Normal values (raw)";
        }
        if ( m_outputStyle == RAW_CORRECTED_VALUES )
        {
            drawString3 = "Normal values (raw corrected)";
        }
        if ( m_outputStyle == JMRUI_STYLE_VALUES )
        {
            drawString3 = "Normal (jMRUI) [mmol/kg ww]";
        }
        if ( m_outputStyle == TRUE_CONCENTRATIONS_VALUES )
        {
            drawString3 = "Normal (2 comp) [mmol/kg ww]";
        }
        
        int length3 = fm.stringWidth( drawString3 );

        g.drawString( drawString3, startx + (2*columnwidth) + (columnwidth/2) - (length3/2) , starty + 120 );


        for ( int i = 0; i < m_numberOfMetabolites; i++ )
        {
            // System.out.println( "Painting " + m_metaboliteNames[i] );
            int length4 = fm.stringWidth( m_metaboliteNames.get(i) );
            g.drawString( m_metaboliteNames.get(i), startx + (columnwidth/2) - (length4/2), starty + 165 + i * 25 );
            g.drawString( "" + m_format.format( m_metabConcentrations[i] ) + " ± " + m_format.format( m_metaboliteConcentrationErrorBars[i] ),
                          startx + columnwidth + columnwidth/2,
                          starty + 165 + i * 25 );
            g.drawString( "" + m_format.format( m_metaboliteNormConcentrations[i] ) + " ± " + m_format.format( m_metaboliteNormConcentrationsErrorBar[i] ),
                          startx + 2 * columnwidth + columnwidth/2,
                          starty + 165 + i * 25 );
        }
   
        // draw first column
        g.drawRect( startx, starty + 100, columnwidth, 40 );

        // draw second column
        g.drawRect( startx, starty + 100,  2 * columnwidth, 40 );

        // draw third column
        g.drawRect( startx, starty + 100,  3 * columnwidth, 40 );

         // draw first column
        g.drawRect( startx, starty + 140, columnwidth,  20 + 25 * m_numberOfMetabolites );

        // draw second column
        g.drawRect( startx, starty + 140,  2 * columnwidth,  20 + 25 * m_numberOfMetabolites );

        // draw third column
        g.drawRect( startx, starty + 140,  3 * columnwidth,  20 + 25 * m_numberOfMetabolites );


        g.drawString( "For scientific purposes only!", startx, starty + 175 + 25 * m_numberOfMetabolites  );
        //g.drawString( "Copyright © - Institute for Diagnostic and Interventional Neuroradiology - Bern - Switzerland", startx,  starty + 190 + 25 * m_numberOfMetabolites );
        g.drawString( "Report created with jMRUI.", startx,  starty + 190 + 25 * m_numberOfMetabolites );
    }

    /**
     * Test the code.
     * @param args
     */
    public static void main( String[] args )
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater(new Runnable( )
        {
            @Override
            public void run( )
            {
                JFrame frm = new JFrame( );


                 // Handle window closing
                frm.addWindowListener(
                    new WindowAdapter( )
                    {
                        @Override
                        public void windowClosing( WindowEvent event )
                        {
                            System.exit( 0 );
                        }
                    }
                );


                int numberOfMetabolites = 10;

                Dimension d = new Dimension( 1024, 200 + numberOfMetabolites * 25 );

                LinkedList<String> metabNames = new LinkedList<String>();
                

                double[] concentrations = new double[numberOfMetabolites];
                double[] errorBars = new double[numberOfMetabolites];
                double[] normValues = new double[numberOfMetabolites];
                double[] errorsNormValues = new double[numberOfMetabolites];
                
                String refImage0 = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/Becher_Sandro_t2_fl2d_tra_hemo_5_35.dcm";
                String refImage1 = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/Becher_Sandro_t2_fl2d_tra_hemo_5_35.dcm";
                String refImage2 = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/Becher_Sandro_t2_fl2d_tra_hemo_5_35.dcm";

                for ( int i = 0; i < numberOfMetabolites; i++ )
                {
                    metabNames.add( "Metab" + i);
                    concentrations[i] = 20.0 * (double)(i);
                    errorBars[i] = (double)(i);
                    normValues[i] = 25.0 * (double)(i);
                    errorsNormValues[i] = (double)(i);
                }
                
                frm.setVisible( true );
                
                frm.getGraphics( ).setColor( Color.BLACK );

                SVSMetaboliteTextReport report =
                        new SVSMetaboliteTextReport( d, "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/svs_12ch_135_nws_CS/Becher_Sandro_svs_12ch_135_nws_CS_13_1.dcm",
                                    "testWaterRefernceFileName", 5,
                                    metabNames,
                                    concentrations, errorBars,
                                    normValues, errorsNormValues, 0,
                                    null
                                  );

               frm.add( report );
               
               frm.pack( );
               
            }
        });
    }

    // Attributes of this class
    private double[] m_metabConcentrations;          // concentrations in mmol/kg ww
    private double[] m_metaboliteConcentrationErrorBars;            // half error bar size

    private double[] m_metaboliteNormConcentrations;                // normal values for the metabolite
    private double[] m_metaboliteNormConcentrationsErrorBar;        // normal values for the metabolite

    private LinkedList<String> m_metaboliteNames;

    private String   m_rawSpectroscopyFileReference; // complete path and filename
    private String   m_waterReferenceFileName;       // filename of the water reference filename

    private Dimension m_dimensionOfControl;          // defines the dimension of the control
    private int       m_numberOfMetabolites;         // number of metabolites


    // attributes from given DICOM reference filename
    private String  m_patientName;
    private String  m_patientNameDisplayString;
    private String  m_patientBirthDate;
    private String  m_patientSex;
    private String  m_patientNumber;
    private String  m_patientNumberDisplayString;

    private String  m_accessionNumber;

    private String  m_studyDescription;
    private String  m_studyDate;
    private String  m_studyTime;

    private String  m_seriesDescription;
    private String  m_seriesNumber;
    private String  m_seriesTime;

    private long    m_resonanceFrequency;

    private String  m_voxelDimensions;

    private String  m_TE; // the pulse sequence TE
    private String  m_TR; // the pulse sequence TR

    private NumberFormat m_format = new DecimalFormat( "###.#" );

    private String m_scannerModelName;

    private String m_referenceImage1FileReference;
    private String m_referenceImage2FileReference;
    private String m_referenceImage3FileReference;

    private JPanel m_textPanel;

    public final static int RAW_VALUES                      = 0;
    public final static int RAW_CORRECTED_VALUES            = 1;
    public final static int JMRUI_STYLE_VALUES              = 2;
    public final static int TRUE_CONCENTRATIONS_VALUES      = 3;

    private int m_outputStyle = 0;
    private GlobalModel m_globalModel;
}
