/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;


import java.util.Vector;

/**
 *
 * This class handles the creation, reading and writing of a jMRUI DICOM tag;
 * This class intermediates between the SCANalyze code and the jMRUI code.
 *
 * @author slotboom
 */
public class SVS_JMRUI_DICOM_Tag_FAST
{
    /**
     * Default constructor.
     */
    public SVS_JMRUI_DICOM_Tag_FAST( )
    {

        //m_dicomTag                      = null;
        m_numberOfSignals               = 0;
        m_numberOfSampledRawDataPoints  = -1;
        m_sampledRawDataPointsVector    = new Vector< double[][] >( );

        // members necessary to contruct a valid jMRUI -> Data object
        m_stepTime          = 0.0;
        m_B0                = 0.0;
        m_transmitterFreq   = 0.0;
        m_nucleus           = 0.0;
        m_areEchoes         = false;
        m_hzRef             = 0.0;
        m_ppmRef            = 0.0;

    }

    /**
     * Returns the jMRUI DICOM tag as a String.
     * @return
     */
    public String getJMRUIDICOMTag( )
    {
        if ( m_stringBuilder.toString( ) != null )
        {
            createCompleteJMRUIDICOMTAG( );
        }

        return m_stringBuilder.toString( );
    }

    /**
     * Returns the jMRUI DICOM tag as a String.
     * @param dicomTag
     */
    public void setJMRUIDICOMTag( String dicomTag )
    {
        m_stringBuilder = new StringBuilder( dicomTag );
    }

    /**
     * Sets the number of acquired data points.
     * @param sampledTimeDomainPoints
     */
    public void setNumberOfSampledTDDataPoints( int sampledTimeDomainPoints )
    {
        m_numberOfSampledRawDataPoints = sampledTimeDomainPoints;
    }

    /**
     * Sets the number of acquired data points.
     * @param sampledTimeDomainPoints
     * @return 
     */
    public int getNumberOfSampledTDDataPoints( int sampledTimeDomainPoints )
    {
        return m_numberOfSampledRawDataPoints;
    }

    /**
     * Sets the raw time domain sampled data points.
     * @param sampledRawTDData
     */
    public void addMeasuredRawSVSResponse( double[][] sampledRawTDData )
    {
        double[][] sampledRawDataPointsToAdd = new double[2][m_numberOfSampledRawDataPoints];

        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
        {
            sampledRawDataPointsToAdd[0][i] = sampledRawTDData[0][i];
            sampledRawDataPointsToAdd[1][i] = sampledRawTDData[1][i];
        }

        m_sampledRawDataPointsVector.add( sampledRawDataPointsToAdd );
        m_numberOfSignals = m_numberOfSignals + 1;
    }

    /**
     * Sets the fitted time domain sampled data points.
     * @param fittedTDData
     */
    public void addFittedModelSVSResponse( double[][] fittedTDData )
    {
        double[][] modelDataPointsToAdd = new double[2][m_numberOfSampledRawDataPoints];

        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
        {
            modelDataPointsToAdd[0][i] = fittedTDData[0][i];
            modelDataPointsToAdd[1][i] = fittedTDData[1][i];
        }

        m_fittedModelDataPointsVector.add( modelDataPointsToAdd );
        m_numberOfFittedSignals = m_numberOfFittedSignals + 1;
    }

    /**
     *
     * Returns the complex time domain response at position "responseNumber"
     *
     * @param responseNumber
     * @return
     */
    public double[][] getMeasuredRawSVSResponse( int responseNumber )
    {
        return m_sampledRawDataPointsVector.get( responseNumber );
    }

    /**
     *
     * Returns the complex time domain fitted response at 
     * position "responseNumber".
     *
     * @param responseNumber
     * @return
     */
    public double[][] getFittedModelSVSResponse( int responseNumber )
    {
        return m_fittedModelDataPointsVector.get( responseNumber );
    }

    /**
     * Returns the time step.
     * @return
     */
    public double getStepTime( )
    {
        return m_stepTime;
    }

    /**
     * Returns the B0 field strength.
     * @return
     */
    public double getB0( )
    {
        return m_B0;
    }

    /**
     * Returns the transmitter frequency.
     * @return
     */
    public double getTransmitterFreq( )
    {
        return m_transmitterFreq;
    }

    /**
     * Returns the nucleus type.
     * @return
     */
    public double getNucleus( )
    {
        return m_nucleus;
    }

    /**
     * Returns whether the data contains echo signals.
     * @return
     */
    public boolean areEchoes( )
    {
        return m_areEchoes;
    }

    /**
     *
     * Returns the reference frequency.
     * @return
     */
    public double getHzref( )
    {
        return m_hzRef;
    }

    /**
     * Returns the reference ppm point.
     * @return
     */
    public double getPpmRef( )
    {
        return m_ppmRef;
    }

    /**
     * Returns the time step.
     * @param stepTime
     */
    public void setStepTime( double stepTime )
    {
        m_stepTime = stepTime;
    }

    /**
     * Returns the B0 field strength.
     * @param B0
     */
    public void setB0( double B0 )
    {
        m_B0 = B0;
    }

    /**
     * Returns the transmitter frequency.
     * @param transmitterFreq
     */
    public void setTransmitterFreq( double transmitterFreq )
    {
        m_transmitterFreq = transmitterFreq;
    }

    /**
     * Returns the nucleus type
     * @param nucleus
     */
    public void setNucleus( double nucleus )
    {
        m_nucleus = nucleus;
    }

    /**
     * Returns whether the data contains echo signals.
     * @param areEchoes
     */
    public void setAreEchoes( boolean areEchoes )
    {
        m_areEchoes = areEchoes;
    }

    /**
     * Returns the reference frequency.
     * @param hzRef
     */
    public void setHzref( double hzRef )
    {
        m_hzRef = hzRef;
    }

    /**
     * Returns the reference ppm point.
     * @param ppmRef
     */
    public void setPpmRef( double ppmRef )
    {
        m_ppmRef = ppmRef;
    }

    /**
     * Resets the jMRUI DICOM tag to null object.
     */
    public void resetJMRUIDicomTag( )
    {
        m_stringBuilder = new StringBuilder( );
    }

    /**
     * Sets the number of metabolites fitted.
     * @param numberOfMetabolitesFitted
     */
    public void setNumberOfMetabolitesFitted( int numberOfMetabolitesFitted )
    {
        m_numberOfMetabolitesFitted = numberOfMetabolitesFitted;
    }

    /**
     * Sets the number of metabolites fitted.
     * @return 
     */
    public int getNumberOfMetabolitesFitted( )
    {
        return m_numberOfMetabolitesFitted;
    }

    /**
     * This method generates the complete DICOM tag.
     * @return
     */
    public String createCompleteJMRUIDICOMTAG( )
    {

        m_stringBuilder = new StringBuilder( );


        // m_dicomTag = "<jMRUIDICOMTAG>" + "\t";
        m_stringBuilder.append( "<jMRUIDICOMTAG>" + "\t" );

        // Write the header attributes
        writejMRUIDataHeaderAttributes( );
        writeMetaboliteNames( );

        for ( int i = 0; i < m_numberOfSignals; i++ )
        {

            writeMetaboliteAmplitudes( i );
            writeMetaboliteAmplitudeErrors( i );

            writeMetaboliteDampings( i );
            writeMetaboliteDampingErrors( i );

            writeMetaboliteDampings2( i);
            writeMetaboliteDamping2Errors( i );


            writeMetaboliteFrequencies( i );
            writeMetaboliteFrequencyErrors( i );

            writeMetabolitePhases( i );
            writeMetabolitePhaseErrors( i );

        }

        writeMetaboliteSignals( );
        writeModelType( );

        for ( int i = 0; i < m_numberOfSignals; i++ )
        {
            writeComplexRawSignal( i );
        }


        for ( int i = 0; i < m_numberOfSignals; i++ )
        {
            writeComplexModelSignal( i );
        }


         for ( int i = 0; i < m_numberOfSignals; i++ )
        {
             writeBackSignal( i );
        }

        m_stringBuilder.append( "\t" + "</jMRUIDICOMTAG>" );

        //m_dicomTag += "\t" + "</jMRUIDICOMTAG>";

        //System.out.println( "Size of old string : " + m_dicomTag.length( ) );
        //System.out.println( "Size of new string : " + m_stringBuilder.toString( ).length( ) );

        // return m_dicomTag;
        return m_stringBuilder.toString( );
        
    }

    /**
     * Writes the jMRUI header file to the TAG.
     */
    protected void writejMRUIDataHeaderAttributes( )
    {

        // append it to string builder
        m_stringBuilder.append( "<JMRUIDataHeader>" + "\t");
        m_stringBuilder.append( m_numberOfSignals   + "\t");
        m_stringBuilder.append( m_numberOfSampledRawDataPoints + "\t");
        m_stringBuilder.append( m_stepTime          + "\t" );
        m_stringBuilder.append( m_B0                + "\t" );
        m_stringBuilder.append( m_transmitterFreq   + "\t" );
        m_stringBuilder.append( m_nucleus           + "\t" );
        m_stringBuilder.append( m_areEchoes         + "\t" );
        m_stringBuilder.append( m_hzRef             + "\t" );
        m_stringBuilder.append( m_ppmRef            + "\t" );
        m_stringBuilder.append( m_numberOfMetabolitesFitted + "\t" );
        m_stringBuilder.append( m_nbIterations      + "\t" );
        m_stringBuilder.append( m_echoesInt         + "\t" );
        m_stringBuilder.append( m_zeroOrderPhase    + "\t" );
        m_stringBuilder.append( m_nameOfAlgorithm   + "\t" );
        m_stringBuilder.append( "</JMRUIDataHeader>" );


//        m_dicomTag += "<JMRUIDataHeader>" + "\t";
//        m_dicomTag += m_numberOfSignals + "\t";
//        m_dicomTag += m_numberOfSampledRawDataPoints + "\t";
//        m_dicomTag += m_stepTime        + "\t";
//        m_dicomTag += m_B0              + "\t";
//        m_dicomTag += m_transmitterFreq + "\t";
//        m_dicomTag += m_nucleus         + "\t";
//        m_dicomTag += m_areEchoes       + "\t";
//        m_dicomTag += m_hzRef           + "\t";
//        m_dicomTag += m_ppmRef          + "\t";
//        m_dicomTag += m_numberOfMetabolitesFitted + "\t";
//        m_dicomTag += m_nbIterations    + "\t";
//        m_dicomTag += m_echoesInt       + "\t";
//        m_dicomTag += m_zeroOrderPhase  + "\t";
//        m_dicomTag += m_nameOfAlgorithm  + "\t";
//
//        m_dicomTag += "</JMRUIDataHeader>";

    }

    /**
     * Writes the signal number "signalNumber" to the jMRUIDICOMTAG.
     *
     * @param signalNumber
     */
    protected void writeComplexModelSignal( int signalNumber )
    {

        m_stringBuilder.append( "<ModelSignal>" + "\t" );
        m_stringBuilder.append(signalNumber  + "\t" );
        m_stringBuilder.append( m_numberOfSampledRawDataPoints  + "\t" );
        double[][] signalToWrite = m_fittedModelDataPointsVector.get( signalNumber );
        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
        {
            m_stringBuilder.append( signalToWrite[0][i] + "\t" + signalToWrite[1][i] + "\t" );
        }

        m_stringBuilder.append( "</ModelSignal>" );

        // old code

//        m_dicomTag += "<ModelSignal>" + "\t";
//        m_dicomTag += signalNumber  + "\t";
//        m_dicomTag += m_numberOfSampledRawDataPoints  + "\t";
//        // double[][] signalToWrite = m_fittedModelDataPointsVector.get( signalNumber );
//        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
//        {
//            m_dicomTag += signalToWrite[0][i] + "\t" + signalToWrite[1][i] + "\t";
//        }
//
//        m_dicomTag += "</ModelSignal>";
    }


    /**
     * Writes the background signal to the string.
     * @param signalNumber
     */
    protected void writeBackSignal( int signalNumber )
    {

        m_stringBuilder.append( "<BackSignal>" + "\t" );
        m_stringBuilder.append( signalNumber  + "\t" );
        m_stringBuilder.append( m_numberOfSampledRawDataPoints  + "\t" );

        double[][] signalToWrite = null;

        if ( m_fittedBackModelDataPointsVector.size() > 0 )
        {
            signalToWrite = m_fittedBackModelDataPointsVector.get( signalNumber );
        }

        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
        {
            if ( ( signalToWrite != null ) && ( signalToWrite.length != 0 ) )
            {
                m_stringBuilder.append( signalToWrite[0][i] + "\t" + signalToWrite[1][i] + "\t" );
            }
            else
            {
                // if the back ground signal is not set, than simply set it to zero
                m_stringBuilder.append( "0.0" + "\t" + "0.0" + "\t" );
            }
        }

        m_stringBuilder.append( "</BackSignal>" );


        // old code..

//        m_dicomTag += "<BackSignal>" + "\t";
//        m_dicomTag += signalNumber  + "\t";
//        m_dicomTag += m_numberOfSampledRawDataPoints  + "\t";
//
//        signalToWrite = null;
//
//        if ( m_fittedBackModelDataPointsVector.size( ) > 0 )
//        {
//            signalToWrite = m_fittedBackModelDataPointsVector.get( signalNumber );
//        }
//
//        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
//        {
//            if ( ( signalToWrite != null ) && ( signalToWrite.length != 0 ) )
//            {
//                m_dicomTag += signalToWrite[0][i] + "\t" + signalToWrite[1][i] + "\t";
//            }
//            else
//            {
//                // if the back ground signal is not set, than simply set it to zero
//                m_dicomTag += "0.0" + "\t" + "0.0" + "\t";
//            }
//        }
//
//        m_dicomTag += "</BackSignal>";
    }


    /**
     *
     * Writes the signal number "signalNumber" to the jMRUIDICOMTAG.
     *
     * @param signalNumber
     */
    protected void writeComplexRawSignal( int signalNumber )
    {

        m_stringBuilder.append( "<RawSignal>" + "\t" );
        m_stringBuilder.append( signalNumber  + "\t" );
        m_stringBuilder.append( m_numberOfSampledRawDataPoints  + "\t" );
        double[][] signalToWrite = m_sampledRawDataPointsVector.get( signalNumber );
        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
        {
            m_stringBuilder.append( signalToWrite[0][i] + "\t" + signalToWrite[1][i] + "\t" );
        }

        m_stringBuilder.append( "</RawSignal>" );

        // old code
//        m_dicomTag += "<RawSignal>" + "\t";
//        m_dicomTag += signalNumber  + "\t";
//        m_dicomTag += m_numberOfSampledRawDataPoints  + "\t";
//        // double[][] signalToWrite = m_sampledRawDataPointsVector.get( signalNumber );
//        for ( int i = 0; i < m_numberOfSampledRawDataPoints; i++ )
//        {
//            m_dicomTag += signalToWrite[0][i] + "\t" + signalToWrite[1][i] + "\t";
//        }
//
//        m_dicomTag += "</RawSignal>";
    }

    /**
     * Writes the names of the fitted metabolites to the Dicom file tag.
     */
    protected void writeMetaboliteNames( )
    {

        m_stringBuilder.append( "<MetaboliteNames>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( m_metaboliteNames.get( i ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteNames>" + "\t" );

//        m_dicomTag += "<MetaboliteNames>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += m_metaboliteNames.get( i ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteNames>" + "\t";
    }

    /**
     * Writes the metabolite amplitudes into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteAmplitudes( int responseNumber )
    {
        Vector<Double> amps = m_ALL_peakAreas.get( responseNumber );


        m_stringBuilder.append( "<MetaboliteAmplitudes>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( amps.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteAmplitudes>" + "\t" );

        // old code
//        m_dicomTag += "<MetaboliteAmplitudes>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += amps.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteAmplitudes>" + "\t";
    }

    /**
     * Writes the metabolite amplitude errors into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteAmplitudeErrors( int responseNumber )
    {
        Vector<Double> ampErrs = m_ALL_peakAreaErrors.get( responseNumber );

        m_stringBuilder.append( "<MetaboliteAmplitudeErrors>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( ampErrs.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteAmplitudeErrors>" + "\t" );

        // old code
//        m_dicomTag += "<MetaboliteAmplitudeErrors>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += ampErrs.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteAmplitudeErrors>" + "\t";
    }

    /**
     * Writes the metabolite damping parameters into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteDampings( int responseNumber )
    {
        Vector<Double> damps = m_ALL_dampings.get( responseNumber );

        m_stringBuilder.append( "<MetaboliteDampings>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( damps.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteDampings>" + "\t" );

        // old code
//        m_dicomTag += "<MetaboliteDampings>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += damps.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteDampings>" + "\t";
    }

    /**
     * Writes the metabolite damping errors into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteDampingErrors( int responseNumber )
    {
        Vector<Double> damperrs = m_ALL_dampingErrors.get( responseNumber );


         m_stringBuilder.append( "<MetaboliteDampingErrors>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( damperrs.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteDampingErrors>" + "\t" );


        // old code

//        m_dicomTag += "<MetaboliteDampingErrors>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += damperrs.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteDampingErrors>" + "\t";
    }



    /**
     * Writes the metabolite damping constants (2) into the jmrui Dicom tag
     * @param responseNumber
     */
    protected void writeMetaboliteDampings2( int responseNumber )
    {
        Vector<Double> damps2 = m_ALL_dampings2.get( responseNumber );
        
        m_stringBuilder.append( "<MetaboliteDampings2>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append(damps2.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteDampings2>" + "\t" );

        // old code
//        m_dicomTag += "<MetaboliteDampings2>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += damps2.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteDampings2>" + "\t";

    }

    /**
     * Writes the metabolite damping errors (2) into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteDamping2Errors( int responseNumber )
    {
        Vector<Double> damps2errs = m_ALL_dampings2Errors.get( responseNumber );


        m_stringBuilder.append( "<MetaboliteDampings2Errors>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( damps2errs.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteDampings2Errors>" + "\t" );

        // old code

//        m_dicomTag += "<MetaboliteDampings2Errors>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += damps2errs.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteDampings2Errors>" + "\t";
    }



    /**
     * Writes the metabolite frequencies into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteFrequencies( int responseNumber )
    {
        Vector<Double> freqs = m_ALL_frequencies.get( responseNumber );


        m_stringBuilder.append( "<MetaboliteFrequencies>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( freqs.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteFrequencies>" + "\t" );


        // old code
//        m_dicomTag += "<MetaboliteFrequencies>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += freqs.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteFrequencies>" + "\t";
    }

    /**
     * Writes the metabolite frequencies errors into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetaboliteFrequencyErrors( int responseNumber )
    {
        Vector<Double> freqErrs = this.m_ALL_frequencyErrors.get( responseNumber );


        m_stringBuilder.append( "<MetaboliteFrequencyErrors>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( freqErrs.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetaboliteFrequencyErrors>" + "\t" );

        // old code
//        m_dicomTag += "<MetaboliteFrequencyErrors>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += freqErrs.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetaboliteFrequencyErrors>" + "\t";
    }

    /**
     * Writes the metabolite phases into the jmrui Dicom tag.
     * @param responseNumber
     */
    protected void writeMetabolitePhases( int responseNumber )
    {
        Vector<Double> phases = m_ALL_phases.get( responseNumber );

        m_stringBuilder.append( "<MetabolitePhases>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( phases.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetabolitePhases>" + "\t" );

        // old code
//        m_dicomTag += "<MetabolitePhases>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += phases.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetabolitePhases>" + "\t";
    }

    /**
     * Writes the errors in the phase to the Dicom tag.
     * @param responseNumber 
     */
    protected void writeMetabolitePhaseErrors( int responseNumber )
    {
        Vector<Double> phaseErrs = m_ALL_phaseErrors.get( responseNumber );

        m_stringBuilder.append( "<MetabolitePhaseErrors>" + "\t" );

        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( phaseErrs.get( i ).doubleValue( ) + "\t" );
        }

        m_stringBuilder.append( "</MetabolitePhaseErrors>" + "\t" );


        // old code
//        m_dicomTag += "<MetabolitePhaseErrors>" + "\t";
//
//        for ( int i = 0; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += phaseErrs.get( i ).doubleValue( ) + "\t";
//        }
//
//        m_dicomTag += "</MetabolitePhaseErrors>" + "\t";

    }

    /**
     * Write the metabolite signals into DICOM tag to be archived.
     */
    protected void writeMetaboliteSignals( )
    {

        m_stringBuilder.append( "<MetaboliteSignals>" + "\t" );

        for( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
        {
            for ( int j = 0; j < m_numberOfSampledRawDataPoints; j++ )
            {
                m_stringBuilder.append( m_metaboliteSignals[i][0][j] + "\t" );
                m_stringBuilder.append( m_metaboliteSignals[i][1][j] + "\t" );
            }
        }

        m_stringBuilder.append( "</MetaboliteSignals>" + "\t" );

        // old code

//        m_dicomTag += "<MetaboliteSignals>" + "\t";
//        for( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            for ( int j = 0; j < m_numberOfSampledRawDataPoints; j++ )
//            {
//                m_dicomTag += m_metaboliteSignals[i][0][j] + "\t";
//                m_dicomTag += m_metaboliteSignals[i][1][j] + "\t";
//            }
//        }
//        m_dicomTag += "</MetaboliteSignals>" + "\t";
    }

    /**
     * Writes the QUEST model type into the jMRUI Dicom tag string.
     */
    protected void writeModelType( )
    {

        m_stringBuilder.append( "<ModelType>" + "\t" );
        for( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
        {
            m_stringBuilder.append( m_modelType.get( i ).intValue( )  + "\t" );
        }
        m_stringBuilder.append( "</ModelType>" + "\t" );


        // old code
//        m_dicomTag += "<ModelType>" + "\t";
//        for( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
//        {
//            m_dicomTag += m_modelType.get( i ).intValue( )  + "\t";
//        }
//        m_dicomTag += "</ModelType>" + "\t";

    }

    /**
     *
     * Sets the model type attribute.
     *
     * @param modelType
     */
    public void setModelType( Vector<Integer> modelType )
    {
        m_modelType = modelType;
    }

    /**
     * Returns the model type array: length is number of metabolites
     * @return
     */
    public Vector<Integer> getModelType( )
    {
        return m_modelType;
    }

   /**
    * Sets the number of iterations.
    * @param nbIterations
    */
    public void setNbIterations( int nbIterations )
    {
        m_nbIterations = nbIterations;
    }

    /**
    * Returns the number of iterations.
     * @return 
    */
    public int getNbIterations( )
    {
        return m_nbIterations;
    }


    /**
     * Sets the number of echoes.
     * @param echoesInt
    */
    public void setEchoesInt( int echoesInt )
    {
        m_echoesInt = echoesInt;
    }

    /**
    * Returns the number of echoes.
     * @return 
    */
    public int getEchoesInt( )
    {
        return m_echoesInt;
    }

    /**
     * Based on the given DICOM tag in the String m_dicomTag set by
     * method setJMRUIDICOMTag( )
     * @return
     */
    public boolean decodeCompleteJMRUIDICOMTag( )
    {

        String localDicomTag = m_stringBuilder.toString( );

        if ( ( localDicomTag.indexOf( "<jMRUIDICOMTAG>" ) != -1 ) &&
             ( localDicomTag.indexOf( "</jMRUIDICOMTAG>" ) != -1 ) )
        {
            // tag has valid start and stop xml-tag: strip off the start and stop tag
            localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "\t" ), localDicomTag.lastIndexOf("\t") ) + "\t";

            // 1. Read the Data object header information
            int startHeader = localDicomTag.indexOf( "<JMRUIDataHeader>" );
            int stopHeader  = localDicomTag.indexOf( "</JMRUIDataHeader>" );

            if ( ( startHeader != -1 ) && ( stopHeader != -1 ) )
            {
                String header =  localDicomTag.substring( startHeader + 18, stopHeader );

                m_numberOfSignals = new Integer( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).intValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_numberOfSampledRawDataPoints = new Integer( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).intValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_stepTime = new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_B0 = new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_transmitterFreq = new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_nucleus= new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_areEchoes = new Boolean( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).booleanValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_hzRef= new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_ppmRef= new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_numberOfMetabolitesFitted = new Integer( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).intValue( );
                // strip of the first part of the header and continue

                // strip of the first part of the header and continue
                header = header.substring( header.indexOf( "\t" ) + 1 );
                m_nbIterations = new Integer( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).intValue( );
                // strip of the first part of the header and continue

                header = header.substring( header.indexOf( "\t" ) + 1 );

                m_echoesInt = new Integer( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).intValue( );

               header = header.substring( header.indexOf( "\t" ) + 1 );

                m_zeroOrderPhase = new Double( header.substring( 0, header.indexOf( "\t" ) ).trim( ) ).doubleValue( );

                header = header.substring( header.indexOf( "\t" ) + 1 );

                m_nameOfAlgorithm = header.trim( );

                // now parse the signals from the string

                // first strip off the header
                localDicomTag = localDicomTag.substring( stopHeader + 18 );

                // 2. Parse the metabolite names
                m_metaboliteNames = new Vector<String>( );

                // strip off the first signal
                String metaboliteNamesString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteNames>" ) + 18,
                                                                     localDicomTag.indexOf( "</MetaboliteNames>" ) );

                for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                {
                    String metaboliteNamesStringToAdd = metaboliteNamesString.substring( 0, metaboliteNamesString.indexOf( "\t" ) + 1 ).trim( );
                    m_metaboliteNames.add( metaboliteNamesStringToAdd );
                    metaboliteNamesString = metaboliteNamesString.substring( metaboliteNamesString.indexOf( "\t" ) + 1 );
                }

                // reduce the m_dicomTag: strip off the metabolite names
                localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteNames>" ) + 19 );

                // Repeatedly read the parameters from the string!
                // repeat m_numberOfFittedSignals times!
                for ( int counter = 0; counter < m_numberOfSignals; counter++ )
                {

                    ////////////////////////////////////////////////////////////////
                    // 3. Parse the Amplitude values

                    m_peakAreas        = new Vector<Double>( );

                    // strip off the amplitude parameter string
                    String metaboliteAmplitudesString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteAmplitudes>" ) + 23,
                                                                              localDicomTag.indexOf( "</MetaboliteAmplitudes>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteAmplitudeStringToAdd = metaboliteAmplitudesString.substring( 0, metaboliteAmplitudesString.indexOf( "\t" ) );
                        m_peakAreas.addElement( new Double( metaboliteAmplitudeStringToAdd.trim( ) ) );
                        metaboliteAmplitudesString = metaboliteAmplitudesString.substring( metaboliteAmplitudesString.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals..
                    m_ALL_peakAreas.add( m_peakAreas );

                    // reduce the m_dicomTag: strip off the metabolite amplitudes
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteAmplitudes>" ) + 24 );


                    // 4. Parse the Amplitude error values

                    m_peakAreaErrors        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteAmplitudeErrorsString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteAmplitudeErrors>" ) + 28,
                                                                                   localDicomTag.indexOf( "</MetaboliteAmplitudeErrors>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteAmplitudeErrorsStringToAdd = metaboliteAmplitudeErrorsString.substring( 0, metaboliteAmplitudeErrorsString.indexOf( "\t" ) );
                        m_peakAreaErrors.addElement( new Double( metaboliteAmplitudeErrorsStringToAdd.trim( ) ) );
                        metaboliteAmplitudeErrorsString = metaboliteAmplitudeErrorsString.substring( metaboliteAmplitudeErrorsString.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals..
                    m_ALL_peakAreaErrors.add( m_peakAreaErrors );


                    // reduce the m_dicomTag: strip off the metabolite amplitudes
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteAmplitudeErrors>" ) + 29 );

                    // 5. Parse the damping values

                    m_dampings        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteDampingsString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteDampings>" ) + 21,
                                                                            localDicomTag.indexOf( "</MetaboliteDampings>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteDampingsStringToAdd = metaboliteDampingsString.substring( 0, metaboliteDampingsString.indexOf( "\t" ) );
                        m_dampings.addElement( new Double( metaboliteDampingsStringToAdd.trim( ) ) );
                        metaboliteDampingsString = metaboliteDampingsString.substring( metaboliteDampingsString.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals..
                    m_ALL_dampings.add( m_dampings );


                    // reduce the m_dicomTag: strip off the metabolite dampings
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteDampings>" ) + 22 );


                    // 6. Parse the damping error values

                    m_dampingErrors        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteDampingsErrorsString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteDampingErrors>" ) + 26,
                                                                                  localDicomTag.indexOf( "</MetaboliteDampingErrors>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteDampingsErrorsStringToAdd = metaboliteDampingsErrorsString.substring( 0, metaboliteDampingsErrorsString.indexOf( "\t" ) );
                        m_dampingErrors.addElement( new Double( metaboliteDampingsErrorsStringToAdd.trim( ) ) );
                        metaboliteDampingsErrorsString = metaboliteDampingsErrorsString.substring( metaboliteDampingsErrorsString.indexOf( "\t" ) + 1 );
                    }

                     // add this vector to the total vector for all signals..
                    m_ALL_dampingErrors.add( m_dampingErrors );


                    // reduce the m_dicomTag: strip off the metabolite amplitudes
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteDampingErrors>" ) + 27 );



                    // 7. Parse the damping values

                    m_dampings2        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteDampings2String = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteDampings2>" ) + 22,
                                                                             localDicomTag.indexOf( "</MetaboliteDampings2>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteDampingsStringToAdd = metaboliteDampings2String.substring( 0, metaboliteDampings2String.indexOf( "\t" ) );
                        m_dampings2.addElement( new Double( metaboliteDampingsStringToAdd.trim( ) ) );
                        metaboliteDampings2String = metaboliteDampings2String.substring( metaboliteDampings2String.indexOf( "\t" ) + 1 );
                    }


                    // add this vector to the total vector for all signals..
                    m_ALL_dampings2.add( m_dampings2 );



                    // reduce the m_dicomTag: strip off the metabolite dampings
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteDampings2>" ) + 23 );


                    // 8. Parse the damping error values

                    m_dampings2Errors        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteDampingsErrors2String = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteDampings2Errors>" ) + 28,
                                                                                   localDicomTag.indexOf( "</MetaboliteDampings2Errors>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteDampingsErrors2StringToAdd = metaboliteDampingsErrors2String.substring( 0, metaboliteDampingsErrors2String.indexOf( "\t" ) );
                        m_dampings2Errors.addElement( new Double( metaboliteDampingsErrors2StringToAdd.trim( ) ) );
                        metaboliteDampingsErrors2String = metaboliteDampingsErrors2String.substring( metaboliteDampingsErrors2String.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals ..
                    m_ALL_dampings2Errors.add( m_dampings2Errors );


                    // reduce the m_dicomTag: strip off the metabolite amplitudes
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteDampings2Errors>" ) + 27 );

                    // 7. Parse the frequencies values
                    m_frequencies        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteFrequenciesString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteFrequencies>" ) + 24,
                                                                               localDicomTag.indexOf( "</MetaboliteFrequencies>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteFrequenciesStringToAdd = metaboliteFrequenciesString.substring( 0, metaboliteFrequenciesString.indexOf( "\t" ) );
                        m_frequencies.addElement( new Double( metaboliteFrequenciesStringToAdd.trim( ) ) );
                        metaboliteFrequenciesString = metaboliteFrequenciesString.substring( metaboliteFrequenciesString.indexOf( "\t" ) + 1 );
                    }


                    // add this vector to the total vector for all signals ..
                    m_ALL_frequencies.add( m_frequencies );

                    // reduce the m_dicomTag: strip off the metabolite frequencies
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteFrequencies>" ) + 25 );


                    // 8. Parse the frequencies error values
                    m_frequencyErrors        = new Vector<Double>( );

                    // strip off the first signal
                    String metaboliteFrequencyErrorsString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteFrequencyErrors>" ) + 28,
                                                                                   localDicomTag.indexOf( "</MetaboliteFrequencyErrors>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metaboliteFrequencyErrorsStringToAdd = metaboliteFrequencyErrorsString.substring( 0, metaboliteFrequencyErrorsString.indexOf( "\t" ) );
                        m_frequencyErrors.addElement( new Double( metaboliteFrequencyErrorsStringToAdd.trim( ) ) );
                        metaboliteFrequencyErrorsString = metaboliteFrequencyErrorsString.substring( metaboliteFrequencyErrorsString.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals ..
                    m_ALL_frequencyErrors.add( m_frequencyErrors );



                    // reduce the m_dicomTag: strip off the metabolite amplitudes
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetaboliteFrequencyErrors>" ) + 29 );


                    // 9. Parse the phases values
                    m_phases      = new Vector<Double>( );

                    // strip off the first signal
                    String metabolitePhasesString = localDicomTag.substring( localDicomTag.indexOf( "<MetabolitePhases>"  ) + 19,
                                                                          localDicomTag.indexOf( "</MetabolitePhases>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metabolitePhasesStringToAdd = metabolitePhasesString.substring( 0, metabolitePhasesString.indexOf( "\t" ) );
                        m_phases.addElement( new Double( metabolitePhasesStringToAdd.trim( ) ) );
                        metabolitePhasesString = metabolitePhasesString.substring( metabolitePhasesString.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals ..
                    m_ALL_phases.add( m_phases );


                    // reduce the m_dicomTag: strip off the metabolite frequencies
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetabolitePhases>" ) + 20 );


                    // 10. Parse the phase error values
                    m_phaseErrors        = new Vector<Double>( );

                    // strip off the first signal
                    String metabolitePhaseErrorsString = localDicomTag.substring( localDicomTag.indexOf( "<MetabolitePhaseErrors>" ) + 24,
                                                                               localDicomTag.indexOf( "</MetabolitePhaseErrors>" ) );

                    for ( int k = 0; k < m_numberOfMetabolitesFitted; k++ )
                    {
                        String metabolitePhaseErrorsStringToAdd = metabolitePhaseErrorsString.substring( 0, metabolitePhaseErrorsString.indexOf( "\t" ) );
                        m_phaseErrors.addElement( new Double( metabolitePhaseErrorsStringToAdd ) );
                        metabolitePhaseErrorsString = metabolitePhaseErrorsString.substring( metabolitePhaseErrorsString.indexOf( "\t" ) + 1 );
                    }

                    // add this vector to the total vector for all signals ..
                    m_ALL_phaseErrors.add( m_phaseErrors );

                    // reduce the m_dicomTag: strip off the metabolite amplitudes
                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</MetabolitePhaseErrors>" ) + 25 );
                }

                // read the metabolite signal base set

                String metaboliteBasicSetSignalString = localDicomTag.substring( localDicomTag.indexOf( "<MetaboliteSignals>" ) + 20,
                                                                              localDicomTag.indexOf( "</MetaboliteSignals>" ) );

                // declare a metabolite base signal set!
                m_metaboliteSignals = new double[m_numberOfMetabolitesFitted][2][m_numberOfSampledRawDataPoints];

                for( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
                {
                    for ( int j = 0; j < m_numberOfSampledRawDataPoints; j++ )
                    {
                       m_metaboliteSignals[i][0][j] = new Double( metaboliteBasicSetSignalString.substring( 0, metaboliteBasicSetSignalString.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                       metaboliteBasicSetSignalString = metaboliteBasicSetSignalString.substring( metaboliteBasicSetSignalString.indexOf( "\t" ) + 1 );


                        m_metaboliteSignals[i][1][j] = new Double( metaboliteBasicSetSignalString.substring( 0, metaboliteBasicSetSignalString.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                        metaboliteBasicSetSignalString = metaboliteBasicSetSignalString.substring( metaboliteBasicSetSignalString.indexOf( "\t" ) + 1 );
                    }
                }

                String modelTypesString = localDicomTag.substring( localDicomTag.indexOf( "<ModelType>" ) + 12,
                                                                localDicomTag.indexOf( "</ModelType>" ) );

                // 11 declare a metabolite base signal set!
                m_modelType = new Vector<Integer>( );

                for( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
                {
                    m_modelType.add( new Integer( modelTypesString.substring( 0, modelTypesString.indexOf( "\t" ) ).trim( ) ) );
                    modelTypesString = modelTypesString.substring( modelTypesString.indexOf( "\t" ) + 1 );
                }

                // 12. read the raw signals from the tag
                for ( int j = 0; j < m_numberOfSignals; j++ )
                {
                    // strip off the first signal
                    String signal = localDicomTag.substring( localDicomTag.indexOf( "<RawSignal>" ) + 12,
                                                          localDicomTag.indexOf( "</RawSignal>" ) );

                    double sigRe, sigIma;
                    double[][] complexSignal = new double[2][m_numberOfSampledRawDataPoints];

                    int signalNumber = new Integer( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).intValue( );
                    signal = signal.substring( signal.indexOf( "\t" ) + 1 );

                    m_numberOfSampledRawDataPoints  = new Integer( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).intValue( );
                    signal = signal.substring( signal.indexOf( "\t" ) + 1 );

                    for ( int k = 0; k < m_numberOfSampledRawDataPoints; k++ )
                    {
                        complexSignal[0][k] = new Double( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                        signal = signal.substring( signal.indexOf( "\t" ) + 1 );


                        complexSignal[1][k] = new Double( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                        signal = signal.substring( signal.indexOf( "\t" ) + 1 );
                    }

                    m_sampledRawDataPointsVector.add( complexSignal  );

                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</RawSignal>" ) + 12 );
                }

                // 13. read the model signals from the tag
                for ( int j = 0; j < m_numberOfSignals; j++ )
                {
                    // strip off the first signal
                    String signal = localDicomTag.substring( localDicomTag.indexOf( "<ModelSignal>" ) + 14,
                                                          localDicomTag.indexOf( "</ModelSignal>" ) );

                    double sigRe, sigIma;
                    double[][] complexSignal = new double[2][m_numberOfSampledRawDataPoints];

                    int signalNumber = new Integer( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).intValue( );
                    signal = signal.substring( signal.indexOf( "\t" ) + 1 );

                    m_numberOfSampledRawDataPoints  = new Integer( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).intValue( );
                    signal = signal.substring( signal.indexOf( "\t" ) + 1 );

                    for ( int k = 0; k < m_numberOfSampledRawDataPoints; k++ )
                    {

                        complexSignal[0][k] = new Double( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                        signal = signal.substring( signal.indexOf( "\t" ) + 1 );
                        if ( k < m_numberOfSampledRawDataPoints - 1 )
                        {
                            complexSignal[1][k] = new Double( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                            signal = signal.substring( signal.indexOf( "\t" ) + 1 );
                        }
                        else
                        {
                            complexSignal[1][k] = new Double( signal.trim( ) );
                        }
                    }

                    m_fittedModelDataPointsVector.add( complexSignal );

                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</ModelSignal>" ) + 14 );
                }

                // 14. read the model signals from the tag
                for ( int j = 0; j < m_numberOfSignals; j++ )
                {
                    // strip off the first signal
                    String signal = localDicomTag.substring( localDicomTag.indexOf( "<BackSignal>" ) + 13,
                                                          localDicomTag.indexOf( "</BackSignal>" ) );

                    double sigRe, sigIma;
                    double[][] complexSignal = new double[2][m_numberOfSampledRawDataPoints];

                    int signalNumber = new Integer( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).intValue( );
                    signal = signal.substring( signal.indexOf( "\t" ) + 1 );

                    m_numberOfSampledRawDataPoints  = new Integer( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).intValue( );
                    signal = signal.substring( signal.indexOf( "\t" ) + 1 );

                    for ( int k = 0; k < m_numberOfSampledRawDataPoints; k++ )
                    {

                        complexSignal[0][k] = new Double( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                        signal = signal.substring( signal.indexOf( "\t" ) + 1 );
                        if ( k < m_numberOfSampledRawDataPoints - 1 )
                        {
                            complexSignal[1][k] = new Double( signal.substring( 0, signal.indexOf( "\t" ) ).trim( ) ).doubleValue( );
                            signal = signal.substring( signal.indexOf( "\t" ) + 1 );
                        }
                        else
                        {
                            complexSignal[1][k] = new Double( signal.trim( ) );
                            //System.out.println( "OK from back model decoding .." );
                        }
                    }

                    // add the signal to the correct vector of signals
                    m_fittedBackModelDataPointsVector.add( complexSignal );

                    localDicomTag = localDicomTag.substring( localDicomTag.indexOf( "</BackSignal>" ) + 13 );
                }



                // if everything is parsed well the last string should be </jMRUIDICOMTAG>
                // so test for this ..
                if ( localDicomTag.trim( ).compareTo( "" ) != 0 )
                {
                    System.out.println( "Error in parsing JMRUI DICOM TAG!" );
                    return false;
                }
                else
                {
                    //System.out.println( "Successfully parsed JMRUI DICOM TAG!" );
                    return true;
                }
            }
            else
            {
                System.out.println( "jMRUIDICOMTAG has no valid header!" );
                return false;
            }
        }
        else
        {
            System.out.println( "No valid jMRUIDICOMTAG : missing <jMRUIDICOMTAG> or </jMRUIDICOMTAG> !" );
            return false;
        }
    }

    /**
     * Allows the user to set the list of metabolite names fitted.
     * @param metabNames
     */
    public void setMetaboliteNames( Vector<String> metabNames )
    {
        m_metaboliteNames = metabNames;
        m_numberOfMetabolitesFitted = m_metaboliteNames.size( );
    }

    /**
     * Returns the array with metabolite names.
     * @return
     */
    public String[] getMetaboliteNames( )
    {

        String[] metaboliteNames = new String[m_numberOfMetabolitesFitted];

        for ( int i = 0 ; i < m_numberOfMetabolitesFitted; i++ )
        {
            metaboliteNames[i] = m_metaboliteNames.get( i );
        }

        return metaboliteNames;

    }

    /**
     * Sets the vector of peak areas attribute.
     * @param peakAreas
     */
     public void addPeakAreaVector( Vector<Double> peakAreas )
     {
         m_ALL_peakAreas.add( peakAreas );
         m_numberOfMetabolitesFitted = peakAreas.size( );
     }

    /**
     * Returns the peak area double precision array belonging to 
     * 'responseNumber'.
     * @param responseNumber
     * @return
     */
     public double[] getPeakAreaArray( int responseNumber )
     {
         double[] amps = new double[m_ALL_peakAreas.get( responseNumber ).size( )];
         for ( int i = 0; i < m_ALL_peakAreas.get( responseNumber ).size(); i++ )
         {
             amps[i] = m_ALL_peakAreas.get( responseNumber ).get( i ).doubleValue( );
         }
         return amps;
     }

    /**
     * Sets the peak area Vector attribute.
     * @param peakAreaErrors
     */
     public void addPeakAreaErrorVector( Vector<Double> peakAreaErrors )
     {
         m_ALL_peakAreaErrors.add( peakAreaErrors );
         m_numberOfMetabolitesFitted = peakAreaErrors.size( );
     }


    /**
     * Returns the peak area error double precision array.
     * @param responseNumber
     * @return
     */
     public double[] getPeakAreaErrorArray( int responseNumber )
     {
         double[] ampsError = new double[m_ALL_peakAreaErrors.get( responseNumber ).size( )];
         for ( int i = 0; i < m_ALL_peakAreaErrors.get( responseNumber ).size( ); i++ )
         {
             ampsError[i] = m_ALL_peakAreaErrors.get( responseNumber ).get( i ).doubleValue( );
         }
         return ampsError;
     }


    /**
     * Sets the vector of damping parameters attribute.
     * @param dampings
     */
     public void addDampingsVector( Vector<Double> dampings )
     {
         m_ALL_dampings.add( dampings );
         m_numberOfMetabolitesFitted = dampings.size( );
     }

    /**
     * Returns the dampings parameters double precision array.
     * @param responseNumber
     * @return
     */
     public double[] getDampingsArray( int responseNumber )
     {
         double[] dampings = new double[m_ALL_dampings.get( responseNumber ).size( )];
         for ( int i = 0; i < m_ALL_dampings.get( responseNumber ).size( ); i++ )
         {
             dampings[i] = m_ALL_dampings.get( responseNumber ).get( i ).doubleValue( );
         }
         return dampings;
     }

    /**
     * Sets the vector of damping parameters (2) attribute.
     * @param dampings2
     */
     public void addDampings2Vector( Vector<Double> dampings2 )
     {
         m_ALL_dampings2.add( dampings2 );
         m_numberOfMetabolitesFitted = dampings2.size( );
     }

     /**
      * Returns the damping parameters (2) double precision array.
     * @param responseNumber
      * @return
      */
     public double[] getDampings2Array( int responseNumber )
     {
         double[] dampings2 = new double[m_ALL_dampings2.get( responseNumber ).size( )];
         for ( int i = 0; i < m_ALL_dampings2.get( responseNumber ).size( ); i++ )
         {
             dampings2[i] = m_ALL_dampings2.get( responseNumber ).get( i ).doubleValue( );
         }
         return dampings2;
     }

     /**
     * Sets the vector of damping errors.
     * @param dampingErrors
     */
     public void addDampingsErrorVector( Vector<Double> dampingErrors )
     {
         m_ALL_dampingErrors.add( dampingErrors );
         m_numberOfMetabolitesFitted = dampingErrors.size( );
     }


    /**
     * Returns the damping parameter error double precision array.
     * @param responseNumber
     * @return
     */
     public double[] getDampingErrorsArray(  int responseNumber )
     {
         double[] dampingErrors = new double[m_ALL_dampingErrors.get( responseNumber ).size( )];
         for ( int i = 0; i < m_ALL_dampingErrors.get( responseNumber ).size( ); i++ )
         {
             dampingErrors[i] = m_ALL_dampingErrors.get( responseNumber ).get( i ).doubleValue( );
         }
         return dampingErrors;
     }


     /**
     * Sets the damping parameters (2) error.
     * @param dampingErrors2
     */
     public void addDampings2ErrorVector( Vector<Double> dampingErrors2 )
     {
         m_ALL_dampings2Errors.add( dampingErrors2 );
         m_numberOfMetabolitesFitted = dampingErrors2.size( );
     }


     /**
      * Returns the damping parameter (2) errors as double precision array.
     * @param responseNumber
      * @return
      */
     public double[] getDampings2ErrorsArray( int responseNumber )
     {
         double[] dampings2Errors = new double[ m_ALL_dampings2Errors.get( responseNumber ).size( ) ];
         for ( int i = 0; i < m_ALL_dampings2Errors.get( responseNumber ).size( ); i++ )
         {
             dampings2Errors[i] = m_ALL_dampings2Errors.get( responseNumber ).get( i ).doubleValue( );
         }
         return dampings2Errors;
     }

    /**
     * Sets the frequencies Vector attribute.
     * @param frequencies
     */
     public void addFrequenciesVector( Vector<Double> frequencies )
     {
         m_ALL_frequencies.add( frequencies );
         m_numberOfMetabolitesFitted = frequencies.size( );
     }


    /**
     * Returns the frequencies double precision array.
     * @param responseNumber
     * @return
     */
    public double[] getFrequenciesArray( int responseNumber )
    {
        double[] frequencies = new double[m_ALL_frequencies.get( responseNumber ).size( )];
        for ( int i = 0; i < m_ALL_frequencies.get( responseNumber ).size( ); i++ )
        {
            frequencies[i] = m_ALL_frequencies.get( responseNumber ).get( i ).doubleValue( );
        }
        return frequencies;
    }

    /**
     * Sets the frequency parameter error vector attribute.
     *
     * @param frequencyErrors
     * */
     public void addFrequencyErrorVector( Vector<Double> frequencyErrors )
     {
         m_ALL_frequencyErrors.add( frequencyErrors );
         m_numberOfMetabolitesFitted = frequencyErrors.size( );
     }


    /**
     * Returns the frequency parameter errors double precision array.
     * @param responseNumber
     * @return
     */
    public double[] getFrequencyErrorsArray( int responseNumber )
    {
        double[] frequencyErrors = new double[m_ALL_frequencyErrors.get( responseNumber ).size( )];
        for ( int i = 0; i < m_ALL_frequencyErrors.get( responseNumber ).size( ); i++ )
        {
            frequencyErrors[i] = m_ALL_frequencyErrors.get( responseNumber ).get( i ).doubleValue( );
        }
        return frequencyErrors;
    }


    /**
     * Sets the vector of phase parameters attribute.
     * @param phases
     */
     public void addPhasesVector( Vector<Double> phases )
     {
         m_ALL_phases.add( phases );
         m_numberOfMetabolitesFitted = phases.size( );
     }

    /**
     * Returns the phase double precision array.
     * @param responseNumber
     * @return
     */
    public double[] getPhasesArray( int responseNumber )
    {
        double[] phases = new double[m_ALL_phases.get( responseNumber ).size( )];
        for ( int i = 0; i < m_ALL_phases.get( responseNumber ).size( ); i++ )
        {
            phases[i] = m_ALL_phases.get( responseNumber ).get( i ).doubleValue( );
        }
        return phases;
    }

    /**
     * Adds the phase parameter error vector.
     * @param phaseErrors
     */
     public void addPhaseErrorVector( Vector<Double> phaseErrors )
     {
         m_ALL_phaseErrors.add( phaseErrors );
         m_numberOfMetabolitesFitted = phaseErrors.size( );
     }

    /**
     * Returns the phase error parameter as double precision array.
     * @param responseNumber
     * @return
     */
    public double[] getPhaseErrorsArray( int responseNumber )
    {
        double[] phaseErrors = new double[m_ALL_phaseErrors.get( responseNumber ).size( )];
        for ( int i = 0; i < m_ALL_phaseErrors.get( responseNumber ).size( ); i++ )
        {
            phaseErrors[i] = m_ALL_phaseErrors.get( responseNumber ).get( i ).doubleValue( );
        }
        return phaseErrors;
    }

    /**
     * Metabolite signals.
     * @param metabSignals
     */
    public void setMetaboliteSignals( double[][][] metabSignals )
    {
        m_metaboliteSignals = metabSignals;
    }

    /**
     * Returns the metabolite signals as 3D array.
     * @return 
     */
    public double[][][] getMetaboliteSignals( )
    {
        return m_metaboliteSignals;
    }

    /**
     * Returns the vector containing the number of background signals.
     * @return
     */
    public Vector< double[][] > getBackModelDataPointsVector( )
    {
        return m_fittedBackModelDataPointsVector;
    }

    /**
     * Sets the zero order phase parameter.
     * @param phase0
     */
    public void setZeroOrderPhase( double phase0 )
    {
        m_zeroOrderPhase = phase0;
    }

    /**
     * Returns the zero order phase.
     * @return 
     */
    public double getZeroOrderPhase( )
    {
        return m_zeroOrderPhase;
    }

    /**
     * Returns the name of the used algorithm to fit the signal with.
     * @return
     */
    public String getNameOfAlgorithm( )
    {
        return m_nameOfAlgorithm;
    }

    /**
     * Sets the name of the used algorithm to fit the signal with.
     * @param nameOfAlgorithm
     */
    public void setNameOfAlgorithm( String nameOfAlgorithm )
    {
        m_nameOfAlgorithm = nameOfAlgorithm;
    }

    /**
     * Returns the number of fitted signals.
     * @return
     */
    public int getNumberOfSignals( )
    {
        return m_numberOfSignals;
    }


    //Atttributes of this class
    //protected String m_dicomTag                     = null;

    protected StringBuilder m_stringBuilder;

    protected int m_numberOfSignals                 = 0;
    protected int m_numberOfFittedSignals           = 0;

    protected int m_numberOfSampledRawDataPoints    = -1;

    protected Vector< double[][] > m_sampledRawDataPointsVector = new Vector<  >( );
    protected Vector< double[][] > m_fittedModelDataPointsVector = new Vector<  >( );
    protected Vector< double[][] > m_fittedBackModelDataPointsVector = new Vector<  >( );

    // members necessary to contruct a valid jMRUI -> Data object
    protected double m_stepTime;
    protected double m_B0;
    protected double m_transmitterFreq;
    protected double m_nucleus;
    protected boolean m_areEchoes;
    protected double m_hzRef;
    protected double m_ppmRef;

    // FitResults related tags
    protected int m_numberOfMetabolitesFitted = -1;
    protected Vector<String> m_metaboliteNames  = new Vector<>( );

    protected Vector<Double> m_peakAreas        = new Vector<>( );
    protected Vector<Double> m_peakAreaErrors   = new Vector<>( );

    protected Vector<Double> m_dampings         = new Vector<>( );
    protected Vector<Double> m_dampingErrors    = new Vector<>( );

    protected Vector<Double> m_dampings2        = new Vector<>( );
    protected Vector<Double> m_dampings2Errors  = new Vector<>( );


    protected Vector<Double> m_frequencies      = new Vector<>( );
    protected Vector<Double> m_frequencyErrors  = new Vector<>( );

    protected Vector<Double> m_phases           = new Vector<>( );
    protected Vector<Double> m_phaseErrors      = new Vector<>( );

    // Data elemenents for all
    protected Vector< Vector<Double> > m_ALL_peakAreas        = new Vector<  >( );
    protected Vector< Vector<Double> > m_ALL_peakAreaErrors   = new Vector<  >( );

    protected Vector< Vector<Double> > m_ALL_dampings         = new Vector<  >( );
    protected Vector< Vector<Double> > m_ALL_dampingErrors    = new Vector<  >( );

    protected Vector< Vector<Double> > m_ALL_dampings2        = new Vector<  >( );
    protected Vector< Vector<Double> > m_ALL_dampings2Errors  = new Vector<  >( );


    protected Vector< Vector<Double> > m_ALL_frequencies      = new Vector<  >( );
    protected Vector< Vector<Double> > m_ALL_frequencyErrors  = new Vector<  >( );

    protected Vector< Vector<Double> > m_ALL_phases           = new Vector<  >( );
    protected Vector< Vector<Double> > m_ALL_phaseErrors      = new Vector<  >( );

    protected double[][][]      m_metaboliteSignals;
    protected  Vector<Integer>  m_modelType;
    protected  int              m_nbIterations;
    protected  int              m_echoesInt;
    protected double            m_zeroOrderPhase;
    protected String            m_nameOfAlgorithm;
}
