/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.texturemaps;

import org.spectrim.signalprocessing.SignalTools;

/**
 * 
 * This class lets the user define a cooccurence matrix based on input image.
 * 
 * @author slotboom
 */
public class CooccurenceMatrix 
{
    
    /**
     * Default constructor.
     */
    public CooccurenceMatrix( )
    {
        
    }
    
    /**
     * Constructor 1: defines the input image and dimensions
     * A window width of 5 x 5 is assumed which corresponds to a
     * im_width = 2 im_height = 2: [-2,-1,0,1,2],[-2,-1,0,1,2].
     * @param im_width
     * @param im_height
     * @param inputImage
     */
    public CooccurenceMatrix( int im_width, int im_height, int[] inputImage )
    {
        
        m_image_width   = im_width;
        m_image_height  = im_height;
        
        m_window_width  = 2;
        m_window_height = 2;
        
        m_grey_levels    = 16;
        
        m_image = new int[inputImage.length];
        
        // copy image data
        System.arraycopy( inputImage, 0, m_image, 0, inputImage.length );
    }  
    
    /**
     * Constructor 2: defines the input image and dimensions
     * The window width can be set as well.
     * @param im_width
     * @param im_height
     * @param inputImage
     * @param wi_width
     * @param wi_height
     */
    public CooccurenceMatrix( int im_width, 
                              int im_height, 
                              int[] inputImage,
                              int wi_width, 
                              int wi_height )
    {
        m_image_width = im_width;
        m_image_height = im_height;
        
        m_window_width = wi_width;
        m_window_height = wi_height;
        
        m_grey_levels    = 16;
        
        m_image = new int[inputImage.length];
        // copy image data
        System.arraycopy( inputImage, 0, m_image, 0, inputImage.length );
    }
    
    /**
     * Constructor 2: defines the input image and dimensions
     * The window width can be set as well.
     * @param im_width
     * @param im_height
     * @param inputImage
     * @param wi_width
     * @param wi_height
     * @param grey_levels
     */
    public CooccurenceMatrix( int im_width, int im_height, int[] inputImage,
                              int wi_width, int wi_height, int grey_levels )
    {
        m_image_width   = im_width;
        m_image_height  = im_height;
        
        m_window_width  = wi_width;
        m_window_height = wi_height;
        
        m_grey_levels   = grey_levels;
        
        m_image = new int[inputImage.length];
        // copy image data
        System.arraycopy( inputImage, 0, m_image, 0, inputImage.length );
    }
    
    /**
     * Constructor 3: defines the input image and dimensions
     * The window width can be set as well.
     * @param numberOfImages
     * @param im_width
     * @param im_height
     * @param inputImage
     * @param wi_width
     * @param wi_height
     * @param grey_levels
     */
    public CooccurenceMatrix( int numberOfImages, 
                              int im_width, int im_height, int[] inputImage,
                              int wi_width, int wi_height, int grey_levels )
    {
        m_numberOfImages = numberOfImages;
        m_image_width   = im_width;
        m_image_height  = im_height;
        
        m_window_width  = wi_width;
        m_window_height = wi_height;
        
        m_grey_levels   = grey_levels;
        
        m_image = new int[inputImage.length];
        
        // copy image data
        System.arraycopy( inputImage, 0, m_image, 0, inputImage.length );
        
    }
    
    
    /**
     * Prints the complete input image.
     */
    public void printInputImage( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "Input Image" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_image[i*m_image_width + j] + " " );
                }   
                System.out.println( );   
            }
        }
    }

        
    /**
     * Prints the local Grey Level Co-occurrence Matrix (GLCM) Variance Image.
     */
    public void print_GLCM_variance_Image( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance Image (GLCM_var)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_GLCM_variance[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );
            }
        }
    }
 
    
    /**
     * Prints the local Grey Level Co-occurrence Matrix (GLCM) 
     * Correlation Image.
     */
    public void print_GLCM_correlation_Image( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Correlation Image (GLCM_Corr)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_GLCMCorrelation[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );
            }
        }
    }
    

    /**
     * Prints the local Grey Level Co-occurrence Matrix (GLCM) Mean value Image.
     */
    public void print_GLCM_mean_Image( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean Image (GLCM_Mean)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_GLCMMean[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );
            }
        }
    }
    
    
    
    /**
     * Prints the local homogeneity image (MAX).
     */
    public void printMaximumProbabilityImage_MAX( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "Local Maximum Probability Image (MAX)" );
        
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_maximumProbability_MAX[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );
            }
        }
    }
    
    /**
     * Prints the local angular second moment image (ASM).
     */
    public void printAngularSecondMomentImage_ASM( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "Local Angular Second Moment Image (ASM)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {
                    System.out.print( m_angularSecondMoment_ASM[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );
            }
        }
    }
    
    
    /**
     * Prints the local homogeneity image (HOM).
     */
    public void printHomogeneityImage_HOM( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "Local Homogeneity Image (HOM)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_homogeneity_HOM[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );   
            }
        }
    }

    
    /**
     * Prints the complete input image.
     */
    public void printDissimilarityImage_DIS( )
    {
        if ( m_verbose == true )
        {
            System.out.println( "Local Dissimilarity Image (DIS)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_dissimilarity_DIS[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );
            }
        }
    }

    
    /**
     * Prints the complete input image.
     */
    public void printContrast_CON( )
    {
        if ( m_verbose == true )
        {

            System.out.println( "Local Contrast Image (CON)" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_contrast_CON[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );   
            }
        }
    }
    
    /**
     * Prints the local entropy image ENT.
     */
    public void printEntropy_ENT( )
    {
        if ( m_verbose == true )
        {

            System.out.println( "Local Entropy Image" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_entropy_ENT[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );   
            }
        }
    }
    
    /**
     * Prints the local energy image ENT.
     */
    public void printEnergy_ENGY( )
    {
        if ( m_verbose == true )
        {

            System.out.println( "Local Energy Image" );
            for ( int i = 0; i < m_image_height; i++ )
            {
                for ( int j = 0; j < m_image_width; j++ )
                {   
                    System.out.print( m_energy[i*m_image_width + j] + "\t" );
                }   
                System.out.println( );   
            }
        }
    }
    
    
    /**
     * Prints the local image around (col,row).
     * @param col
     * @param row
     */
    public void printLocalImage( int col, int row )
    {
        if ( m_verbose == true )
        {
            System.out.println( );
            System.out.println( "Local image (" + col + "," + row + ")" );
        
            for ( int i = row - m_window_height; i <= row + m_window_height; i++ )
            {
                for ( int j = col - m_window_width; j <= col + m_window_width; j++ )
                {   
                    System.out.print( m_image[i*m_image_width + j] + " " );
                }   
                System.out.println( );   
            }
        }
    }
    
   
    /**
     * Sets the window width/height.
     * @param wi_width 
     * @param wi_height 
     */
    public void setWindowDimensions( int wi_width, int wi_height )
    {
        m_window_width = wi_width;
        m_window_height = wi_height;
    }

    /**
     * Sets the number of grey levels.
     * @param grey_levels 
     */
    public void setGreyLevels( int grey_levels )
    {
        m_grey_levels = grey_levels;
    }

    /**
     * Set the image
     * @param im_width
     * @param im_height 
     * @param image 
     */
    public void setImage( int im_width, int im_height, int[] image )
    {
        m_image_width  = im_width;
        m_image_height = im_height;
    }
    
    /**
     * Computes the local cooccurrence matrix for the required directions
     * @param imageNumber
     * @param col
     * @param row
     * @param type
     * @return 
     */
    public float[][] computeLocalCooccurrenceMatrix( int imageNumber, int col, 
                                                     int row, int type )
    {
        if ( m_reducedGreyLevelImage == null )
        {
            computeReducedImageBasedOnMaximum( );
        }
        
        if ( type == HORIZONTAL )
        {
            return computeLocalHorizontalCooccurrenceMatrix( imageNumber, col, row );
        }
        
        if  ( type == VERTICAL )
        {
            return computeLocalVerticalCooccurrenceMatrix( imageNumber, col, row );
        }
        
        if  ( type == NORTHEAST )
        {
            return computeLocalNorthEastCooccurrenceMatrix( imageNumber, col, row );
        }

        if  ( type == SOUTHEAST )
        {
            return computeLocalSouthEastCooccurrenceMatrix( imageNumber, col, row );
        }
        
        return null;
    }
    
    
    
    /**
     * 
     * @param col
     * @param row
     * @return 
     */
    private float[][] computeLocalHorizontalCooccurrenceMatrix( int imageNumber, 
                                                                int col, 
                                                                int row )
    {
        // set the cooccurence matrix to null
        m_coocMatrixHor = null;
        
        if ( ( col >= m_window_width ) && ( col < m_image_width - m_window_width ) && 
             ( row >= m_window_height ) && ( row < m_image_height - m_window_height ) )
        {
            printLocalImage( col, row );
            
            m_coocMatrixHor = new float[m_grey_levels][m_grey_levels];
            
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    int count = 0;
                                    
                    for ( int x = col - m_window_width; x <= col + m_window_width - 1; x++ )
                    {
                        for ( int y = row - m_window_height; y <= row + m_window_height ; y++ )
                        {
                            if ( ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + y * m_image_width + x ]     == i ) &&
                                 ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + y * m_image_width + x + 1 ] == j ) )
                            {                            
                                count++;
                            }
                        }
                    }
                
                    m_coocMatrixHor[i][j] = count;
                }
            }
        
            float[][] sub_res = new float[m_grey_levels][m_grey_levels];
            
            // make matrix symmetric 
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    if ( i==j )
                    {
                        sub_res[i][j] = 2 * m_coocMatrixHor[i][j];
                    }
                    else
                    {
                        sub_res[i][j] =  m_coocMatrixHor[j][i] + m_coocMatrixHor[i][j];                    
                    }
                }
            }
        
            // determine normalization constant
            int norm = 0;
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    norm += sub_res[i][j];
                }
            }
        
            // normalize cooccurrence matrix
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    m_coocMatrixHor[i][j] = sub_res[i][j] / norm;
//                  m_coocMatrixHor[i][j] = sub_res[i][j];                    
                }
            }
        }
        
        return m_coocMatrixHor;
    }
     
    
    /**
     * Computes the co-occurrence matrix in the north east direction
     * @param col
     * @param row
     * @return 
     */
    private float[][] computeLocalNorthEastCooccurrenceMatrix( int imageNumber, 
                                                               int col, 
                                                               int row ) 
    {
        
        // set the cooccurence matrix to null
        m_coocMatrixNE = null;
        
        if ( ( col >= m_window_width ) && ( col < m_image_width - m_window_width ) && 
             ( row >= m_window_height ) && ( row < m_image_height - m_window_height ) )
        {
            printLocalImage( col, row );
        
            m_coocMatrixNE = new float[m_grey_levels][m_grey_levels];
        
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    int count = 0;
                    // System.out.println( "i = " + i + "  j = " + j );
                
                    for ( int x = col - m_window_width; x <= col + m_window_width - 1; x++ )
                    {
                        for ( int y = row - m_window_height + 1; y <= row + m_window_height; y++ )
                        {
                            if ( ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + y * m_image_width + x ]            == i ) &&
                                 ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + ( y - 1 ) * m_image_width + x + 1] == j ) )
                            {
                                count++;
                                // System.out.println( "Count = " + count );
                            }
                        }
                    }
                
                    m_coocMatrixNE[i][j] = count;
                }
            }
        
            float[][] sub_res = new float[m_grey_levels][m_grey_levels];
            
            // make matrix symmetric
            
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    if ( i==j )
                    {
                        sub_res[i][j] = 2 * m_coocMatrixNE[i][j];
                    }
                    else
                    {
                        sub_res[i][j] =  m_coocMatrixNE[j][i] + m_coocMatrixNE[i][j];
                    }
                }
            }
        
            // determine normalization constant
            int norm = 0;
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    norm += sub_res[i][j];
                }
            }
        
            // normalize cooccurrence matrix
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    m_coocMatrixNE[i][j] = sub_res[i][j] / norm;
//                  m_coocMatrixNE[i][j] = sub_res[i][j];                    
                }
            }
        
        }
        
        return m_coocMatrixNE;    
    }
       
    /**
     * Computes the co-occurrence matrix in the south east direction
     * @param col
     * @param row
     * @return 
     */
    private float[][] computeLocalSouthEastCooccurrenceMatrix( int imageNumber, 
                                                               int col, 
                                                               int row ) 
    {
        // set the cooccurence matrix to null
        m_coocMatrixSE = null;
        
        if ( ( col >= m_window_width ) && ( col < m_image_width - m_window_width ) && 
             ( row >= m_window_height ) && ( row < m_image_height - m_window_height ) )
        {
            printLocalImage( col, row );
        
            m_coocMatrixSE = new float[m_grey_levels][m_grey_levels];
        
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    int count = 0;
                    // System.out.println( "i = " + i + "  j = " + j );
                
                    for ( int x = col - m_window_width; x <= col + m_window_width - 1; x++ )
                    {
                        for ( int y = row - m_window_height; y <= row + m_window_height - 1; y++ )
                        {
                            if ( ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + y * m_image_width + x ]            == i ) &&
                                 ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + ( y + 1 ) * m_image_width + x + 1] == j ) )
                            {
                                count++;
                                // System.out.println( "Count = " + count );
                            }
                        }
                    }
                
                    m_coocMatrixSE[i][j] = count;
                }
            }
        
            float[][] sub_res = new float[m_grey_levels][m_grey_levels];
            
            // make matrix symmetric
            
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    if ( i==j )
                    {
                        sub_res[i][j] = 2 * m_coocMatrixSE[i][j];
                    }
                    else
                    {
                        sub_res[i][j] =  m_coocMatrixSE[j][i] + m_coocMatrixSE[i][j];
                    }
                }
            }
        
            // determine normalization constant
            int norm = 0;
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    norm += sub_res[i][j];
                }
            }
        
            // normalize cooccurrence matrix
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    m_coocMatrixSE[i][j] = sub_res[i][j] / norm;
//                  m_coocMatrixSE[i][j] = sub_res[i][j];                    
                }
            }
        
        }
        
        return m_coocMatrixSE;
    }
    
     
    /**
     * Computes the local cooccurrence matrix of the input image 2 (col,row)
     * @param col
     * @param row
     * @return cooccurrence matrix in vertical direction
     */
    private float[][] computeLocalVerticalCooccurrenceMatrix( int imageNumber, 
                                                              int col, 
                                                              int row )
    {
        // set the cooccurence matrix to null
        m_coocMatrixVert = null;
        
        if ( ( col >= m_window_width ) && ( col < m_image_width - m_window_width ) && 
             ( row >= m_window_height ) && ( row < m_image_height - m_window_height ) )
        {
            printLocalImage( col, row );
        
            m_coocMatrixVert = new float[m_grey_levels][m_grey_levels];
        
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    int count = 0;
                    // System.out.println( "i = " + i + "  j = " + j );
                    for ( int x = col - m_window_width; x <= col + m_window_width; x++ )
                    {
                        for ( int y = row - m_window_height; y <= row + m_window_height - 1 ; y++ )
                        {
                            if ( ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + y * m_image_width + x ]         == i ) &&
                                 ( m_reducedGreyLevelImage[ imageNumber * m_image_width * m_image_height + ( y + 1 ) * m_image_width + x ] == j ) )
                            {
                                count++;
                                // System.out.println( "Count = " + count );
                            }
                        }
                    }
                
                    m_coocMatrixVert[i][j] = count;
                }
            }
        
            float[][] sub_res = new float[m_grey_levels][m_grey_levels];
            
            // make matrix symmetric 
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    if ( i==j )
                    {
                        sub_res[i][j] = 2 * m_coocMatrixVert[i][j];
                    }
                    else
                    {
                        sub_res[i][j] =  m_coocMatrixVert[j][i] + m_coocMatrixVert[i][j];                    
                    }
                }
            }
        
            // determine normalization constant
            int norm = 0;
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                    norm += sub_res[i][j];
                }
            }
        
            // normalize cooccurrence matrix
            for ( int i = 0; i < m_grey_levels; i++ )
            {
                for ( int j = 0; j < m_grey_levels; j++ )
                {
                  m_coocMatrixVert[i][j] = sub_res[i][j] / norm;
//                    m_coocMatrixVert[i][j] = sub_res[i][j];                    
                }
            }
        }
        
        return m_coocMatrixVert;
        
    }

    
    /**
     * Returns the version of the image. 
     * @return 
     */
    public int[] computeReducedImageBasedOnMaximum( )
    {
        
        int maxOfImage = SignalTools.getMaximum( m_image );
        
        if ( maxOfImage == 0 )
        {
            return m_image;
        }
        
        m_reducedGreyLevelImage = new int[ m_image.length ];
        
        for ( int i = 0; i < m_image.length; i++ )
        {
            m_reducedGreyLevelImage[i] = ( m_image[i] * ( m_grey_levels - 1) ) / maxOfImage;
        }
        
        return m_reducedGreyLevelImage;
    }
    
    /**
     * Prints the vertical co occurrence matrix to the stdout.
     */
    public void printCoocMatrixVert( )
    {
        if ( m_verbose == true )
        {
            System.out.println( );
            System.out.println( "Vertical cooccurrence matrix:" );
        }
        
        if ( m_coocMatrixVert != null )
        {
            if ( m_verbose == true )
            {
                for ( int i = 0; i < m_grey_levels; i++ )
                {
                    for ( int j = 0; j < m_grey_levels; j++ )
                    {
                        System.out.print( m_coocMatrixVert[i][j] + " " );
                    }   
                    System.out.println( );
                }
            }
        }
        else
        {
            if ( m_verbose == true )
            {
                System.out.println( "Not defined for this coordinate" );
            }
        }
        
    }
    
    
    /**
     * Prints the vertical co occurrence matrix to the stdout.
     */
    public void printReducedGrayLevelMatrix( )
    {
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                System.out.print( m_reducedGreyLevelImage[i* m_grey_levels + j] + " " );
            }   
            System.out.println( );
        }
    }
    
    /**
     * Prints the SE co occurrence matrix to the stdout.
     */
    public void printCoocMatrixSE( )
    {
        if ( m_verbose == true )
        {
            if ( m_verbose == true )
            {
                System.out.println( );
                System.out.println( "South east cooccurrence matrix:" );
            }
        }
        
        if ( m_coocMatrixSE != null )
        {
            if ( m_verbose == true )
            {
                for ( int i = 0; i < m_grey_levels; i++ )
                {
                    for ( int j = 0; j < m_grey_levels; j++ )
                    {
                        System.out.print( m_coocMatrixSE[i][j] + " " );
                    }   
                    System.out.println( );
                }
            }
        }
        else
        {
            if ( m_verbose == true )
            {
                System.out.println( "Not defined for this coordinate" );
            }
        }
    }
    
    /**
     * Prints the NE co-occurrence matrix to the stdout.
     */
    public void printCoocMatrixNE( )
    {
        if ( m_verbose == true )
        {
            System.out.println( );
            System.out.println( "North east cooccurrence matrix:" );
        }
        
        if ( m_coocMatrixNE != null )
        {
            if ( m_verbose == true )
            {
                for ( int i = 0; i < m_grey_levels; i++ )
                {
                    for ( int j = 0; j < m_grey_levels; j++ )
                    {
                        System.out.print( m_coocMatrixNE[i][j] + " " );
                    }   
                    System.out.println( );
                }
            }
        }
        else
        {
            if ( m_verbose == true )
            {
                System.out.println( "Not defined for this coordinate" );
            }
        }
    }
    
    public void printCoocMatrixHor( )
    {
        if ( m_verbose == true )
        {
            System.out.println( );
            System.out.println( "Horizontal cooccurrence matrix:" );
        }
        
        if ( m_coocMatrixHor != null )
        {
            if ( m_verbose == true )
            {
                for ( int i = 0; i < m_grey_levels; i++ )
                {    
                    for ( int j = 0; j < m_grey_levels; j++ )
                    {
                        System.out.print( m_coocMatrixHor[i][j] + " " );
                    }
            
                    System.out.println( );
                }
            }
        }
        else
        {
            if ( m_verbose == true )
            {
                System.out.println( "Not defined for this coordinate" );
            }
        }
    }
    
    
    /**
     * Compute all cooccurrence matrices.
     */
    public void computeAllCooccurrenceMatrices( )
    {
        for ( int k = 0; k < m_numberOfImages; k++ )
        {
            
            System.out.println( "Processing image number." );
            
            for ( int i = 0; i < m_image_width; i++ )
            {
                for ( int j = 0; j < m_image_height; j++ )    
                {
                    if ( m_verbose == true )
                    {
                        System.out.println( "(col,row) = (" + i + "," + j + ")"  );
                    }    
                
                    computeLocalCooccurrenceMatrix( k, i, j, CooccurenceMatrix.HORIZONTAL );
                    printCoocMatrixHor( );
                
                    computeLocalCooccurrenceMatrix( k, i, j, CooccurenceMatrix.VERTICAL );
                    printCoocMatrixVert( );
                
                    computeLocalCooccurrenceMatrix( k, i, j, CooccurenceMatrix.NORTHEAST );
                    printCoocMatrixNE( );

                    computeLocalCooccurrenceMatrix( k, i, j, CooccurenceMatrix.SOUTHEAST );
                    printCoocMatrixSE( );
                
                    computeTextureParameters( k, i, j );
                }
            }
        }
    }
    
    /**
     * 
     * Computes all texture maps for the computed cooccurence matrices.
     * 
     * @param col
     * @param row 
     */
    private void computeTextureParameters( int imageNumber, int col, int row )
    {
        int imageStackPixelNumber = imageNumber *m_image_width * m_image_height + row * m_image_width + col;
        
        if ( m_contrast_CON == null )
        {
           m_contrast_CON = new float[this.m_image.length];
        }
       
        if ( m_dissimilarity_DIS == null )
        {
           m_dissimilarity_DIS = new float[this.m_image.length];
        }
        
        if ( m_homogeneity_HOM == null )
        {
           m_homogeneity_HOM = new float[this.m_image.length];
        }
       
        if ( m_angularSecondMoment_ASM == null )
        {
           m_angularSecondMoment_ASM = new float[this.m_image.length];
        }
       
        if ( m_energy == null )
        {
            m_energy = new float[this.m_image.length];
        }
        
        if ( m_maximumProbability_MAX == null )
        {
            m_maximumProbability_MAX = new float[this.m_image.length];
        }
        
        if ( m_entropy_ENT == null )
        {
            m_entropy_ENT = new float[this.m_image.length];
        }
        
        if ( m_GLCMMean == null )
        {
            m_GLCMMean = new float[this.m_image.length];
        }    
     
        if ( m_GLCMCorrelation == null )
        {
            m_GLCMCorrelation = new float[this.m_image.length];
        }
        
        if ( m_GLCM_variance == null )
        {
            m_GLCM_variance = new float[this.m_image.length];
        }
        
        // account for horizontal direction ..
        if ( this.m_coocMatrixHor != null )
        {
            m_contrast_CON[ imageStackPixelNumber ]            = this.computeContrastHorizontal( );
            m_dissimilarity_DIS[ imageStackPixelNumber ]       = this.computeDissimilarityHorizontal( );
            m_homogeneity_HOM[ imageStackPixelNumber ]         = this.computeHomogeneityHorizontal( );
            m_angularSecondMoment_ASM[ imageStackPixelNumber ] = this.computeAngularSecondMomentHorizontal( );
            m_energy[ imageStackPixelNumber ]                  = this.computeEnergyHorizontal( );
            m_maximumProbability_MAX[ imageStackPixelNumber ]  = this.computeMaximumProbabilityHorizontal( );
            m_entropy_ENT[ imageStackPixelNumber ]             = this.computeEntropyHorizontal( );
            m_GLCMMean[ imageStackPixelNumber ]                = this.computeGLCMMeanHorizontal_i( );
            m_GLCMCorrelation[ imageStackPixelNumber ]         = this.computeGLCMCorrelationHorizontal( );
            m_GLCM_variance[ imageStackPixelNumber ]           = this.computeGLCMVarianceHorizontal_i( );
        }
        else
        {
            m_contrast_CON[ imageStackPixelNumber ]            = 0f;
            m_dissimilarity_DIS[ imageStackPixelNumber ]       = 0f;
            m_homogeneity_HOM[ imageStackPixelNumber ]         = 0f;
            m_angularSecondMoment_ASM[ imageStackPixelNumber ] = 0f;
            m_energy[ imageStackPixelNumber ]                  = 0f;
            m_maximumProbability_MAX[ imageStackPixelNumber ]  = 0f;
            m_entropy_ENT[ imageStackPixelNumber ]             = 0f;
            m_GLCMMean[ imageStackPixelNumber ]                = 0f;
            m_GLCMCorrelation[ imageStackPixelNumber ]         = 0f;
            m_GLCM_variance[ imageStackPixelNumber ]           = 0f;
        }
        
        // account for vertical direction ..
        if ( this.m_coocMatrixVert != null )
        {
            m_contrast_CON[ imageStackPixelNumber ]            += this.computeContrastVertical( );
            m_dissimilarity_DIS[ imageStackPixelNumber ]       += this.computeDissimilarityVertical( );
            m_homogeneity_HOM[ imageStackPixelNumber ]         += this.computeHomogeneityVertical( );
            m_angularSecondMoment_ASM[ imageStackPixelNumber ] += this.computeAngularSecondMomentVertical( );
            m_energy[ imageStackPixelNumber ]                  += this.computeEnergyVertical( );
            m_maximumProbability_MAX[ imageStackPixelNumber ]  += this.computeMaximumProbabilityVertical( );
            m_entropy_ENT[ imageStackPixelNumber ]             += this.computeEntropyVertical( );
            m_GLCMMean[ imageStackPixelNumber ]                += this.computeGLCMMeanVertical_i( );
            m_GLCMCorrelation[ imageStackPixelNumber ]         += this.computeGLCMCorrelationVertical( );
            m_GLCM_variance[ imageStackPixelNumber ]           += this.computeGLCMVarianceVertical_i( );
        }
        
        // account for NE direction ..
        if ( this.m_coocMatrixNE != null )
        {
            m_contrast_CON[ imageStackPixelNumber ]            += this.computeContrastNE( );
            m_dissimilarity_DIS[ imageStackPixelNumber ]       += this.computeDissimilarityNE( );
            m_homogeneity_HOM[ imageStackPixelNumber ]         += this.computeHomogeneityNE( );
            m_angularSecondMoment_ASM[ imageStackPixelNumber ] += this.computeAngularSecondMomentNE( );
            m_energy[ imageStackPixelNumber ]                  += this.computeEnergyNE( );
            m_maximumProbability_MAX[ imageStackPixelNumber ]  += this.computeMaximumProbabilityNE( );
            m_entropy_ENT[ imageStackPixelNumber ]             += this.computeEntropyNE( );
            m_GLCMMean[ imageStackPixelNumber ]                += this.computeGLCMMeanNE_i( );
            m_GLCMCorrelation[ imageStackPixelNumber ]         += this.computeGLCMCorrelationNE( );
            m_GLCM_variance[ imageStackPixelNumber ]           += this.computeGLCMVarianceNE_i( );
        }
        
        // account for SE direction ..
        if ( this.m_coocMatrixSE != null )
        {
            m_contrast_CON[ imageStackPixelNumber ]            += this.computeContrastSE( );
            m_dissimilarity_DIS[ imageStackPixelNumber ]       += this.computeDissimilaritySE( );
            m_homogeneity_HOM[ imageStackPixelNumber ]         += this.computeHomogeneitySE( );
            m_angularSecondMoment_ASM[ imageStackPixelNumber ] += this.computeAngularSecondMomentSE( );
            m_energy[ imageStackPixelNumber ]                  += this.computeEnergySE( );
            m_maximumProbability_MAX[ imageStackPixelNumber ]  += this.computeMaximumProbabilitySE( );
            m_entropy_ENT[ imageStackPixelNumber ]             += this.computeEntropySE( );
            m_GLCMMean[ imageStackPixelNumber ]                += this.computeGLCMMeanSE( );
            m_GLCMCorrelation[ imageStackPixelNumber ]         += this.computeGLCMCorrelationSE( );
            m_GLCM_variance[ imageStackPixelNumber ]           += this.computeGLCMVarianceSE( );
        }
        
        m_contrast_CON[ imageStackPixelNumber ]                = m_contrast_CON[ imageStackPixelNumber ]/4;
        m_dissimilarity_DIS[ imageStackPixelNumber ]           = m_dissimilarity_DIS[ imageStackPixelNumber ]/4;
        m_homogeneity_HOM[ imageStackPixelNumber ]             = m_homogeneity_HOM[ imageStackPixelNumber ]/4;
        m_angularSecondMoment_ASM[ imageStackPixelNumber ]     = m_angularSecondMoment_ASM[ imageStackPixelNumber ]/4;
        m_energy[ imageStackPixelNumber ]                      = m_energy[ imageStackPixelNumber ]/4;
        m_maximumProbability_MAX[ imageStackPixelNumber ]      = m_maximumProbability_MAX[ imageStackPixelNumber ]/4;
        m_entropy_ENT[ imageStackPixelNumber ]                 = m_entropy_ENT[ imageStackPixelNumber ]/4;
        m_GLCMMean[ imageStackPixelNumber ]                    = m_GLCMMean[ imageStackPixelNumber ]/4;
        m_GLCMCorrelation[ imageStackPixelNumber ]             = m_GLCMCorrelation[ imageStackPixelNumber ]/4;
        m_GLCM_variance[ imageStackPixelNumber ]               = m_GLCM_variance[ imageStackPixelNumber ]/4;
        
    }
            
    private native float [][] winNativeCooccurenceMatrix( int[] image, int imageWidth, int imageHeight, int numberOfImages, int windowWidth, int windowheight, int greyLevels);
//    private native float [][] nativeCudaCooccurenceMatrix( int[] image, int imageWidth, int imageHeight, int numberOfImages, int windowWidth, int windowheight, int greyLevels);
    
//    static {
//        System.load("/home/user/development/C/parallel/CooccurenceMatrixLib/dist/CoocurenceMatrixLib.so");
//    }
    // Windows
//    static {
//        System.load("C:\\Users\\slotboom\\Documents\\NetBeansProjects\\winNativeCooccurenceMatrixLib\\build\\Release\\Cygwin-Windows\\libwinNativeCooccurenceMatrixLib.dll");
//    }
//    
//    static {
//        System.load("/home/user/development/C/parallel/cudaCooccurenceMatrix/dist/cudaCooccurenceMatrix.so");
//    }
    static 
    {
        try
        {
            
            
            System.load("C:\\Users\\slotboom\\Documents\\NetBeansProjects\\winNativeCooccurenceMatrixLib\\dist\\Release\\Cygwin-Windows\\libwinNativeCooccurenceMatrixLib.dll");
            
            //System.load("C:\\Users\\slotboom\\Documents\\NetBeansProjects\\winNativeCooccurenceMatrixLib\\build\\Release\\Cygwin-Windows\\libwinNativeCooccurenceMatrixLib");
            //System.load("/home/slotboom/development/C/parallel/nativeCooccurenceMatrix/dist/nativeCooccurenceMatrix.so");
            //System.load("/home/guest/Desktop/SCANalyzeRegio/nativeCooccurenceMatrix.so");
        }
        catch (Exception e){System.out.println("loading of static library failed");}
    }
    
    public void computeCooccurrenceMatrices( )
    {
        try
        {
            
            launchNativeComputeAllCooccurrenceMatrices( );
            m_isSuccessfullyComputed = true;
            
        }
        catch(Exception e)
        {
            System.out.println("Native coocurence matrix computation failed, computing map in java");
            computeAllCooccurrenceMatrices( );
            m_isSuccessfullyComputed = true;
        }
    }
    
    private void launchNativeComputeAllCooccurrenceMatrices( ) throws Exception
    {
        if ( m_reducedGreyLevelImage == null )
        {
            computeReducedImageBasedOnMaximum( );
        }
        float[][] textureMaps = null;
        
        try
        {
            textureMaps = winNativeCooccurenceMatrix( m_reducedGreyLevelImage, m_image_width , m_image_height, m_numberOfImages, m_window_width, m_window_height, m_grey_levels);
        }
        catch( java.lang.UnsatisfiedLinkError e )
        {
            throw new Exception( );
        }
        
        if ( m_contrast_CON == null )
        {
           m_contrast_CON = new float[this.m_image.length];
        }

        if ( m_dissimilarity_DIS == null )
        {
           m_dissimilarity_DIS = new float[this.m_image.length];
        }

        if ( m_homogeneity_HOM == null )
        {
           m_homogeneity_HOM = new float[this.m_image.length];
        }

        if ( m_angularSecondMoment_ASM == null )
        {
           m_angularSecondMoment_ASM = new float[this.m_image.length];
        }

        if ( m_energy == null )
        {
            m_energy = new float[this.m_image.length];
        }

        if ( m_maximumProbability_MAX == null )
        {
            m_maximumProbability_MAX = new float[this.m_image.length];
        }

        if ( m_entropy_ENT == null )
        {
            m_entropy_ENT = new float[this.m_image.length];
        }

        if ( m_GLCMMean == null )
        {
            m_GLCMMean = new float[this.m_image.length];
        }

        if ( m_GLCMCorrelation == null )
        {
            m_GLCMCorrelation = new float[this.m_image.length];
        }

        if ( m_GLCM_variance == null )
        {
            m_GLCM_variance = new float[this.m_image.length];
        }
        
        m_contrast_CON            = textureMaps[0];
        m_dissimilarity_DIS       = textureMaps[1];
        m_homogeneity_HOM         = textureMaps[2];
        m_angularSecondMoment_ASM = textureMaps[3];
        m_energy                  = textureMaps[4];
        m_maximumProbability_MAX  = textureMaps[5];
        m_entropy_ENT             = textureMaps[6];
        m_GLCMMean                = textureMaps[7];
        m_GLCMCorrelation         = textureMaps[8];
        m_GLCM_variance           = textureMaps[9];
        
    }
    
    // some test code
    public static void main( String args[] )
    {
        int[] testImage = { 3, 0, 1, 1,
                            0, 3, 1, 1,
                            3, 2, 2, 2,
                            2, 2, 3, 3 };
        

//        CooccurenceMatrix cooc = new CooccurenceMatrix( 4, 4, testImage, 1, 1, 4 );

        
        int[] testImage2 = { 0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 2, 0, 1, 1, 1, 0,
                             0, 0, 0, 3, 1, 1, 1, 0,
                             0, 0, 3, 2, 2, 2, 1, 0,
                             0, 0, 2, 2, 3, 3, 2, 0,
                             0, 0, 1, 2, 3, 1, 2, 0,
                             0, 0, 0, 0, 0, 0, 0, 0 };
        
        
        int[] testImage3 = { 0, 3, 3, 3, 2, 1, 0, 1,
                             0, 1, 0, 0, 0, 0, 0, 2,
                             0, 2, 2, 0, 1, 1, 1, 3,
                             0, 3, 0, 3, 1, 1, 1, 3,
                             0, 3, 3, 2, 2, 2, 1, 3,
                             0, 3, 2, 2, 3, 3, 2, 2,
                             0, 1, 1, 2, 3, 1, 2, 1,
                             0, 2, 3, 3, 3, 2, 1, 0 };
        
//     public CooccurenceMatrix( int im_width, int im_height, int[] inputImage,
//                               int wi_width, int wi_height, int grey_levels )
                
                
        CooccurenceMatrix cooc = new CooccurenceMatrix( 8, 8, testImage3, 2, 2, 4 );
        
        cooc.printInputImage( );
        
        
//        System.out.println( "Horizontal cooccurrence matrix" );

//        cooc.computeLocalCooccurrenceMatrix( 5, 5, CooccurenceMatrix.HORIZONTAL );
        
        cooc.computeAllCooccurrenceMatrices( );
        //cooc.launchNativeComputeAllCooccurrenceMatrices( );
        cooc.setVerbosity( true );
        
        cooc.printAngularSecondMomentImage_ASM( );
        cooc.printContrast_CON( );
        cooc.printDissimilarityImage_DIS( );
        cooc.printEntropy_ENT( );
        cooc.printEnergy_ENGY( );
        cooc.printHomogeneityImage_HOM( );
        cooc.printMaximumProbabilityImage_MAX( );
        cooc.print_GLCM_mean_Image( );
        cooc.print_GLCM_variance_Image( );
        cooc.print_GLCM_correlation_Image( );
        cooc.setVerbosity( false );
        
//        cooc.computeCooccurrenceMatrix( 4, 4, CooccurenceMatrix.VERTICAL );

        

//        cooc.printCoocMatrixHor( );

//        cooc.printCoocMatrixVert( );

        
//        cooc.computeContrastHorizontal( );
//        cooc.computeDissimilarityHorizontal( );
//        cooc.computeHomogeneityHorizontal( );
//        cooc.computeAngularSecondMomentHorizontal( );
//        cooc.computeMaximumProbabilityHorizontal( );
//        cooc.computeEntropyHorizontal( );
//        cooc.computeGLCMMeanHorizontal( );
//        cooc.computeGLCMVarianceHorizontal( );
//        cooc.computeGLCMCorrelationHorizontal( );
        
//        System.out.println( "\nVertical cooccurrence matrix" );
//        cooc.computeCooccurrenceMatrix( 1, 1, CooccurenceMatrix.VERTICAL );
//        cooc.printCoocMatrixVert( );
        
//        cooc.computeContrastVertical( );
//        cooc.computeDissimilarityVertical( );
//        cooc.computeHomogeneityVertical( );
//        cooc.computeAngularSecondMomentVertical( );
//        cooc.computeMaximumProbabilityVertical( );
//        cooc.computeEntropyVertical( );
//        cooc.computeGLCMMeanVertical( );
//        cooc.computeGLCMVarianceVertical( );
//        cooc.computeGLCMCorrelationVertical( );
//        cooc.printCoocMatrixVert( );
        
//        cooc.computeCooccurrenceMatrix( 1, 1, CooccurenceMatrix.SOUTHEAST );
//        
//        cooc.printReducedGrayLevelMatrix( );
//        cooc.printCoocMatrixSE( );
        
//        cooc.computeCooccurrenceMatrix( 1, 1, CooccurenceMatrix.NORTHEAST );
//        
//        cooc.printReducedGrayLevelMatrix( );
//        cooc.printCoocMatrixNE( );
        
        //return 0;
    }
    
    
    // Computes the texture parameters in the horizontal direction
    
    /**
     * Computes the contrast for horizontal normalized cooc
     */
    public float computeContrastHorizontal( )
    {
        
        float contrastH = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                contrastH += m_coocMatrixHor[i][j] * ( i - j ) * ( i - j );
            }  
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "CON H : " + contrastH );
        }
        
        return contrastH;
    }
    
    
    /**
     * Computes the dissimilarity (DIS) for horizontal normalized cooc
     */
    public float computeDissimilarityHorizontal( )
    {
        float dissimilarityH = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                dissimilarityH += m_coocMatrixHor[i][j] * Math.abs( i - j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "DIS H : " + dissimilarityH );
        }
        
        return dissimilarityH;
    }
    
    /**
     * Computes the homogeneity HOM (inverse difference moment) 
     * for horizontal normalized cooc.
     * @return 
     */
    public float computeHomogeneityHorizontal( )
    {
        float homogeneityH = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                homogeneityH += m_coocMatrixHor[i][j] / ( 1 + ( i - j ) * ( i - j ) );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "HOM H : " + homogeneityH );
        }
        
        return homogeneityH;
    }
    
    /*
     * Angular Second Moment (ASM) and Energy (also called Uniformity). 
     */    
    public float computeAngularSecondMomentHorizontal( )
    {
        float computeAngularSecondMomentH = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeAngularSecondMomentH += m_coocMatrixHor[i][j] * m_coocMatrixHor[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ASM H : " + computeAngularSecondMomentH );
        }
        
        return computeAngularSecondMomentH;
    }
    
    
    /*
     * Energy horizontal (also called Uniformity). 
     */    
    public float computeEnergyHorizontal( )
    {
        float computeEnergyH = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeEnergyH += m_coocMatrixHor[i][j] * m_coocMatrixHor[i][j];
            }
        }
        
        computeEnergyH = (float)Math.sqrt( computeEnergyH );
        
        if ( m_verbose == true )
        {
            System.out.println( "Energy H : " + computeEnergyH );
        }
        
        return computeEnergyH;
    }
    
    
     /**
      * Maximum probability (MAX H) Horizontal:
      * High MAX values occur if one combination of pixels 
      * dominates the pixel pairs in the window. 
     * @return 
      */    
    public float computeMaximumProbabilityHorizontal( )
    {
        float maximumProbabilityH = -Float.MAX_VALUE;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixHor[i][j] > maximumProbabilityH ) 
                {
                    maximumProbabilityH = m_coocMatrixHor[i][j];
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "MAX H : " + maximumProbabilityH );
        }
         
        return maximumProbabilityH;
    }
    
    /**
     * Computes the entropy (ENT) in the horizontal direction. 
     * @return 
     */    
    public float computeEntropyHorizontal( )
    {
        float computeEntropyH = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixHor[i][j] != 0f )
                {
                    computeEntropyH += - m_coocMatrixHor[i][j] * Math.log( m_coocMatrixHor[i][j] );
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ENT H : " + computeEntropyH );
        }
        return computeEntropyH;
    }
    
    
    /**
     * Computes the horizontal GLCM Mean i in the horizontal direction. 
     * @return 
     */    
    public float computeGLCMMeanHorizontal_i( )
    {
        float GLCM_mean_H_i = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_H_i += i * m_coocMatrixHor[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean H : " + GLCM_mean_H_i );
        }
        
        return GLCM_mean_H_i;
    }

    /**
     * Computes the horizontal GLCM Mean i in the horizontal direction. 
     * @return 
     */    
    public float computeGLCMMeanHorizontal_j( )
    {
        float GLCM_mean_H_j = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_H_j += j * m_coocMatrixHor[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean H : " + GLCM_mean_H_j );
        }
        
        return GLCM_mean_H_j;
    }
    
    /**
     * Computes the horizontal GLCM Correlation in the horizontal direction. 
     * @return 
     */    
    public float computeGLCMCorrelationHorizontal( )
    {
        float GLCM_correlation_H = 0f;
        
        float GLCM_variance_H_i = computeGLCMVarianceHorizontal_i( );
        float GLCM_variance_H_j = computeGLCMVarianceHorizontal_j( );
        
        float GLCM_mean_H_i = computeGLCMMeanHorizontal_i( );
        float GLCM_mean_H_j = computeGLCMMeanHorizontal_j( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_correlation_H += ( i - GLCM_mean_H_i ) * ( j - GLCM_mean_H_j ) * m_coocMatrixHor[i][j] / Math.sqrt( GLCM_variance_H_i * GLCM_variance_H_j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Correlation H : " + GLCM_correlation_H );
        }
        
        return GLCM_correlation_H;
    }
    
     
    /**
     * Computes the horizontal GLCM Mean i in the horizontal direction. 
     * @return 
     */    
    public float computeGLCMVarianceHorizontal_i( )
    {
        float GLCM_variance_H_i = 0f;
        
        float GLCM_mean_H_i = computeGLCMMeanHorizontal_i( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_H_i += ( i - GLCM_mean_H_i ) * ( i - GLCM_mean_H_i ) * m_coocMatrixHor[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance H_i : " + GLCM_variance_H_i );
        }
        
        return GLCM_variance_H_i;
    }
    
    /**
     * Computes the horizontal GLCM Mean j in the horizontal direction. 
     * @return 
     */    
    public float computeGLCMVarianceHorizontal_j( )
    {
        float GLCM_variance_H_j = 0f;
        
        float GLCM_mean_H_j = computeGLCMMeanHorizontal_j( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_H_j += ( j - GLCM_mean_H_j ) * ( j - GLCM_mean_H_j ) * m_coocMatrixHor[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance H_j: " + GLCM_variance_H_j );
        }
        
        return GLCM_variance_H_j;
    }
    
    
    ///////////////////////////////////////////////////////////////////////////
    // Computes the texture parameters in the vertical direction
    ///////////////////////////////////////////////////////////////////////////
    
    
    /**
     * Computes the contrast for vertical normalized cooc.
     * @return 
     */
    public float computeContrastVertical( )
    {
        
        float contrastV = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                contrastV += m_coocMatrixVert[i][j] * ( i - j ) * ( i - j );
            }   
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "CON V : " + contrastV );
        }
        
        return contrastV;
    }
    
    
    /**
     * Computes the dissimilarity (DIS) for vertical normalized cooc.
     * @return 
     */
    public float computeDissimilarityVertical( )
    {
        float dissimilarityV = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                dissimilarityV += m_coocMatrixVert[i][j] * Math.abs( i - j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "DIS V : " + dissimilarityV );
        }
        
        return dissimilarityV;
    }
    
    /**
     * Computes the homogeneity HOM V (inverse difference moment) 
     * for vertical normalized cooc.
     * @return 
     */
    public float computeHomogeneityVertical( )
    {
        float homogeneityV = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                homogeneityV += m_coocMatrixVert[i][j] / ( 1 + ( i - j ) * ( i - j ) );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "HOM V : " + homogeneityV );
        }
        
        return homogeneityV;
    }
    
     /**
      * Angular Second Moment vertical (ASM) and 
      * Energy (also called Uniformity). 
     * @return 
      */    
    public float computeAngularSecondMomentVertical( )
    {
        float computeAngularSecondMomentV = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeAngularSecondMomentV += m_coocMatrixVert[i][j] * m_coocMatrixVert[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ASM V : " + computeAngularSecondMomentV );
        }
        
        return computeAngularSecondMomentV;
    }
    
     /**
      * Computes Energy vertical (also called Uniformity). 
     * @return 
      */    
    public float computeEnergyVertical( )
    {
        float computeEnergyV = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeEnergyV += m_coocMatrixVert[i][j] * m_coocMatrixVert[i][j];
            }
        }
        
        computeEnergyV = (float)Math.sqrt( computeEnergyV );
        
        if ( m_verbose == true )
        {
            System.out.println( "Energy V : " + computeEnergyV );
        }
        
        return computeEnergyV;
    }
    
    
     /**
      * Maximum probability (MAX V) Vertical :
      * High MAX values occur if one combination of pixels dominates 
      * the pixel pairs in the window. 
     * @return 
      */    
    public float computeMaximumProbabilityVertical( )
    {
        float maximumProbabilityV = -Float.MAX_VALUE;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixVert[i][j] > maximumProbabilityV ) 
                {
                    maximumProbabilityV = m_coocMatrixVert[i][j];
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "MAX V : " + maximumProbabilityV );
        }
        
        return maximumProbabilityV;
    }
    
    /**
     * Computes the entropy (ENT V) in the vertical direction. 
     * @return 
     */    
    public float computeEntropyVertical( )
    {
        float computeEntropyV = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixVert[i][j] != 0f )
                {
                    computeEntropyV += - m_coocMatrixVert[i][j] * Math.log( m_coocMatrixVert[i][j] );
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ENT V : " + computeEntropyV );
        }
        
        return computeEntropyV;
    }
    
    
    /**
     * Computes the GLCM Mean i in the vertical direction. 
     * @return 
     */    
    public float computeGLCMMeanVertical_i( )
    {
        float GLCM_mean_V_i = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_V_i += i * m_coocMatrixVert[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean V : " + GLCM_mean_V_i );
        }
        
        return GLCM_mean_V_i;
    }

    /**
     * Computes the GLCM Mean i in the vertical direction. 
     * @return 
     */    
    public float computeGLCMMeanVertical_j( )
    {
        float GLCM_mean_V_j = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_V_j += j * m_coocMatrixVert[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean V : " + GLCM_mean_V_j );
        }
        
        return GLCM_mean_V_j;
    }
    
    
    /**
     * Computes the GLCM Correlation in the vertical direction. 
     * @return 
     */    
    public float computeGLCMCorrelationVertical( )
    {
        float GLCM_correlation_V = 0f;
        
        float GLCM_variance_V_i = computeGLCMVarianceVertical_i( );
        float GLCM_variance_V_j = computeGLCMVarianceVertical_j( );
        
        float GLCM_mean_V_i = computeGLCMMeanVertical_i( );
        float GLCM_mean_V_j = computeGLCMMeanVertical_j( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_correlation_V += ( i - GLCM_mean_V_i ) * ( j - GLCM_mean_V_j ) * m_coocMatrixVert[i][j] / Math.sqrt( GLCM_variance_V_i * GLCM_variance_V_j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Correlation V : " + GLCM_correlation_V );
        }
        
        return GLCM_correlation_V;
    }
    
     
    /**
     * Computes the GLCM Mean i in the vertical direction. 
     * @return 
     */    
    public float computeGLCMVarianceVertical_i( )
    {
        float GLCM_variance_V_i = 0f;
        
        float GLCM_mean_V_i = computeGLCMMeanVertical_i( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_V_i += ( i - GLCM_mean_V_i ) * ( i - GLCM_mean_V_i ) * m_coocMatrixVert[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance V i : " + GLCM_variance_V_i );
        }
        
        return GLCM_variance_V_i;
    }

    /**
     * Computes the GLCM Mean i in the vertical direction. 
     * @return 
     */    
    public float computeGLCMVarianceVertical_j( )
    {
        float GLCM_variance_V_j = 0f;
        
        float GLCM_mean_V_j = computeGLCMMeanVertical_j( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_V_j += ( j - GLCM_mean_V_j ) * ( j - GLCM_mean_V_j ) * m_coocMatrixVert[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance V j : " + GLCM_variance_V_j );
        }
        
        return GLCM_variance_V_j;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // Computes the texture parameters in the south east direction
    ///////////////////////////////////////////////////////////////////////////
    
    
    /**
     * Computes the contrast for SE normalized cooc.
     * @return 
     */
    public float computeContrastSE( )
    {
        
        float contrastSE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                contrastSE += m_coocMatrixSE[i][j] * ( i - j ) * ( i - j );
            }   
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "CON SE : " + contrastSE );
        }
        
        return contrastSE;
    }
    
    
    /**
     * Computes the dissimilarity (DIS) for SE normalized cooc.
     * @return 
     */
    public float computeDissimilaritySE( )
    {
        float dissimilaritySE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                dissimilaritySE += m_coocMatrixSE[i][j] * Math.abs( i - j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "DIS SE : " + dissimilaritySE );
        }
        
        return dissimilaritySE;
    }
    
    /**
     * Computes the homogeneity HOM SE (inverse difference moment)
     * for SE normalized cooc.
     * @return 
     */
    public float computeHomogeneitySE( )
    {
        float homogeneitySE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                homogeneitySE += m_coocMatrixSE[i][j] / ( 1 + ( i - j ) * ( i - j ) );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "HOM SE : " + homogeneitySE );
        }
        
        return homogeneitySE;
    }
    
     /**
      * Angular Second Moment SE (ASM) and Energy (also called Uniformity). 
     * @return 
      */    
    public float computeAngularSecondMomentSE( )
    {
        float computeAngularSecondMomentSE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeAngularSecondMomentSE += m_coocMatrixSE[i][j] * m_coocMatrixSE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ASM SE : " + computeAngularSecondMomentSE );
        }
        
        return computeAngularSecondMomentSE;
    }
    
    /**
      * Computes Energy SE (also called Uniformity). 
     * @return 
      */    
    public float computeEnergySE( )
    {
        float computeEnergySE = 0f;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeEnergySE += m_coocMatrixSE[i][j] * m_coocMatrixSE[i][j];
            }
        }
        
        computeEnergySE = (float)Math.sqrt( computeEnergySE );
        
        if ( m_verbose == true )
        {
            System.out.println( "Energy SE : " + computeEnergySE );
        }
        
        return computeEnergySE;
    }
    
    
     /**
      * Maximum probability (MAX SE) SE :
      * High MAX values occur if one combination of pixels 
      * dominates the pixel pairs in the window. 
     * @return 
      */    
    public float computeMaximumProbabilitySE( )
    {
        float maximumProbabilitySE = -Float.MAX_VALUE;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixSE[i][j] > maximumProbabilitySE ) 
                {
                    maximumProbabilitySE = m_coocMatrixSE[i][j];
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "MAX SE : " + maximumProbabilitySE );
        }
        
        return maximumProbabilitySE;
    }
    
    /**
     * Computes the entropy (ENT SE) in the SE direction. 
     * @return 
     */    
    public float computeEntropySE( )
    {
        float computeEntropySE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixSE[i][j] != 0f )
                {
                    computeEntropySE += - m_coocMatrixSE[i][j] * Math.log( m_coocMatrixSE[i][j] );
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ENT SE : " + computeEntropySE );
        }
        
        return computeEntropySE;
    }
    
    
    /**
     * Computes the SE GLCM Mean i in the SE direction. 
     * @return 
     */    
    public float computeGLCMMeanSE( )
    {
        float GLCM_mean_SE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_SE += j * m_coocMatrixSE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean SE : " + GLCM_mean_SE );
        }
        
        return GLCM_mean_SE;
    }
    
    /**
     * Computes the GLCM Correlation in the SE direction. 
     * @return 
     */    
    public float computeGLCMCorrelationSE( )
    {
        float GLCM_correlation_SE = 0f;
        
        float GLCM_variance_SE = computeGLCMVarianceSE( );
        
        float GLCM_mean_SE = computeGLCMMeanSE( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_correlation_SE += ( i - GLCM_mean_SE ) * ( j - GLCM_mean_SE ) * m_coocMatrixSE[i][j] / Math.sqrt( GLCM_variance_SE * GLCM_variance_SE );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Correlation SE : " + GLCM_correlation_SE );
        }
        
        return GLCM_correlation_SE;
        
    }
    
     
    /**
     * Computes the horizontal GLCM Mean in the SE direction. 
     * @return 
     */    
    public float computeGLCMVarianceSE( )
    {
        float GLCM_variance_SE = 0f;
        
        float GLCM_mean_SE = computeGLCMMeanSE( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_SE += ( j - GLCM_mean_SE ) * ( j - GLCM_mean_SE ) * m_coocMatrixSE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance SE : " + GLCM_variance_SE );
        }
        
        return GLCM_variance_SE;
    }

    
    ///////////////////////////////////////////////////////////////////////////
    // Computes the texture parameters in the south east direction
    ///////////////////////////////////////////////////////////////////////////
    
    
    /**
     * Computes the contrast for NE normalized cooc.
     * @return 
     */
    public float computeContrastNE( )
    {
        
        float contrastNE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                contrastNE += m_coocMatrixNE[i][j] * ( i - j ) * ( i - j );
            }   
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "CON NE : " + contrastNE );
        }
        
        return contrastNE;
    }
    
    
    /**
     * Computes the dissimilarity (DIS) for NE normalized cooc.
     * @return 
     */
    public float computeDissimilarityNE( )
    {
        float dissimilarityNE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                dissimilarityNE += m_coocMatrixNE[i][j] * Math.abs( i - j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "DIS NE : " + dissimilarityNE );
        }
        
        return dissimilarityNE;
    }
    
    /**
     * Computes the homogeneity HOM NE (inverse difference moment) 
     * for SE normalized cooc.
     * @return 
     */
    public float computeHomogeneityNE( )
    {
        float homogeneityNE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                homogeneityNE += m_coocMatrixNE[i][j] / ( 1 + ( i - j ) * ( i - j ) );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "HOM NE : " + homogeneityNE );
        }
        
        return homogeneityNE;
    }
    
     /**
      * Angular Second Moment NE (ASM) and Energy (also called Uniformity). 
     * @return 
      */    
    public float computeAngularSecondMomentNE( )
    {
        float computeAngularSecondMomentNE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeAngularSecondMomentNE += m_coocMatrixNE[i][j] * m_coocMatrixNE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ASM NE : " + computeAngularSecondMomentNE );
        }
        
        return computeAngularSecondMomentNE;
    }
    
     /**
      * Computes Energy NE ((also called Uniformity). 
     *  @return 
      */    
    public float computeEnergyNE( )
    {
        float computeEnergyNE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                computeEnergyNE += m_coocMatrixNE[i][j] * m_coocMatrixNE[i][j];
            }
        }
        
        computeEnergyNE = (float)Math.sqrt( computeEnergyNE );
        
        if ( m_verbose == true )
        {
            System.out.println( "Energy NE : " + computeEnergyNE );
        }
        
        return computeEnergyNE;
    }
    
    
     /**
      * Maximum probability (MAX PROB NE):
      * High MAX values occur if one combination of pixels 
      * dominates the pixel pairs in the window. 
     * @return 
      */    
    public float computeMaximumProbabilityNE( )
    {
        float maximumProbabilityNE = -Float.MAX_VALUE;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixNE[i][j] > maximumProbabilityNE ) 
                {
                    maximumProbabilityNE = m_coocMatrixNE[i][j];
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "MAX NE : " + maximumProbabilityNE );
        }
        
        return maximumProbabilityNE;
    }
    
    /**
     * Computes the entropy (ENT NE) in the SE direction. 
     * @return 
     */    
    public float computeEntropyNE( )
    {
        float computeEntropyNE = 0f;
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                if ( m_coocMatrixNE[i][j] != 0f )
                {
                    computeEntropyNE += - m_coocMatrixNE[i][j] * Math.log( m_coocMatrixNE[i][j] );
                }
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "ENT NE : " + computeEntropyNE );
        }
        
        return computeEntropyNE;
    }
    
    
    /**
     * Computes the SE GLCM Mean i in the NE direction. 
     * @return 
     */    
    public float computeGLCMMeanNE_i( )
    {
        float GLCM_mean_NE_i = 0f;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_NE_i += i * m_coocMatrixNE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean NE i : " + GLCM_mean_NE_i );
        }
        
        return GLCM_mean_NE_i;
    }
    
     /**
     * Computes the SE GLCM Mean i in the NE direction. 
     * @return 
     */    
    public float computeGLCMMeanNE_j( )
    {
        float GLCM_mean_NE_j = 0f;
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_mean_NE_j += j * m_coocMatrixNE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Mean NE j : " + GLCM_mean_NE_j );
        }
        
        return GLCM_mean_NE_j;
    }
    
    
    /**
     * Computes the GLCM Correlation in the NE direction. 
     * @return 
     */    
    public float computeGLCMCorrelationNE( )
    {
        float GLCM_correlation_NE = 0f;
        
        float GLCM_variance_NE_i = computeGLCMVarianceNE_i( );
        float GLCM_variance_NE_j = computeGLCMVarianceNE_j( );
        
        float GLCM_mean_NE_i = computeGLCMMeanNE_i( );
        float GLCM_mean_NE_j = computeGLCMMeanNE_j( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_correlation_NE += ( i - GLCM_mean_NE_i ) * ( j - GLCM_mean_NE_j ) * m_coocMatrixNE[i][j] / Math.sqrt( GLCM_variance_NE_i * GLCM_variance_NE_j );
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Correlation NE : " + GLCM_correlation_NE );
        }
        
        return GLCM_correlation_NE;
        
    }
    
     
    /**
     * Computes the horizontal GLCM Mean in the NE direction. 
     * @return 
     */    
    public float computeGLCMVarianceNE_i( )
    {
        float GLCM_variance_NE_i = 0f;
        
        float GLCM_mean_NE_i = computeGLCMMeanNE_i( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_NE_i += ( i - GLCM_mean_NE_i ) * ( i - GLCM_mean_NE_i ) * m_coocMatrixNE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance NE i : " + GLCM_variance_NE_i );
        }
        
        return GLCM_variance_NE_i;
    }
    
    /**
     * Computes the horizontal GLCM Mean in the NE direction. 
     * @return 
     */    
    public float computeGLCMVarianceNE_j( )
    {
        float GLCM_variance_NE_j = 0f;
        
        float GLCM_mean_NE_j = computeGLCMMeanNE_j( );
        
        for ( int i = 0; i < m_grey_levels; i++ )
        {
            for ( int j = 0; j < m_grey_levels; j++ )
            {
                GLCM_variance_NE_j += ( j - GLCM_mean_NE_j ) * ( j - GLCM_mean_NE_j ) * m_coocMatrixNE[i][j];
            }
        }
        
        if ( m_verbose == true )
        {
            System.out.println( "GLCM Variance NE : " + GLCM_variance_NE_j );
        }
        
        return GLCM_variance_NE_j;
    }
    
    /** 
     * Set verbosity option.
     * @param verbose
     */
    public void setVerbosity( boolean verbose )
    {
        m_verbose = verbose;
    }
    
    /**
     * Returns the contrast (CON) of the GLCM
     * @return 
     */
    public float[] getGLCM_CONtrast( )
    {
        return m_contrast_CON;
    }
    /**
     * Returns the dissimilarity (DIS) of the GLCM
     * @return 
     */
    public float[] getGLCM_DISsimilarity( )
    {
        return m_dissimilarity_DIS;
    }
    
    /**
     * Returns the homogeneity (HOM) of the GLCM
     * @return 
     */
    public float[] getGLCM_HOMogeneity( )
    {
        return m_homogeneity_HOM;
    }
    
    /**
     * Returns the Angular Second Moment (ASM) of the GLCM
     * @return 
     */
    public float[] getGLCM_AngularSecondMoment_ASM( )
    {
        return m_angularSecondMoment_ASM;
    }
    
    /**
     * Returns the Energy (ENE) of the GLCM
     * @return 
     */
    public float[] getGLCM_Energy( )
    {
        return m_energy;
    }
    
    /**
     * Returns the Maximum probability (MAX) of the GLCM
     * @return 
     */
    public float[] getGLCM_MAXimumProbability( )
    {
        return m_maximumProbability_MAX;
    }
        
    /**
     * Returns the Entropy (ENT) of the GLCM
     * @return 
     */
    public float[] getGLCM_ENTropy( )
    {
        return m_entropy_ENT;
    }

    /**
     * Returns the GLCM_MEAN of the GLCM
     * @return 
     */
    public float[] getGLCM_Mean( )
    {
        return m_GLCMMean;
    }

    /**
     * Returns the GLCM_CORR of the GLCM
     * @return 
     */
    public float[] getGLCM_Correlation( )
    {
        return m_GLCMCorrelation;
    }
    
    /**
     * Returns the GLCM_VAR of the GLCM
     * @return 
     */
    public float[] getGLCM_Variance( )
    {
        return m_GLCM_variance;
    }        
    
    ///////////////////////////////////////////////////////////////////////////
    
    /**
     * Returns the contrast (CON) of the GLCM
     * @return 
     */
    public int[] getGLCM_CONtrastAsScaledIntArray( )
    {
        int[] CON = new int[m_contrast_CON.length];
        
        for ( int i = 0; i < m_contrast_CON.length; i++ )
        {
            CON[i] = (int)( 1000f * m_contrast_CON[i] );
        }
        
        return CON;
    }
    
    /**
     * Returns the dissimilarity (DIS) of the GLCM
     * @return 
     */
    public int[] getGLCM_DISsimilarityAsScaledIntArray( )
    {
        int[] DIS = new int[m_dissimilarity_DIS.length];
        
        for ( int i = 0; i < m_dissimilarity_DIS.length; i++ )
        {
            DIS[i] = (int)( 1000f * m_dissimilarity_DIS[i] );
        }
        return DIS;
    }
    
    /**
     * Returns the homogeneity (HOM) of the GLCM
     * @return 
     */
    public int[] getGLCM_HOMogeneityAsScaledIntArray( )
    {
        int[] HOM = new int[m_homogeneity_HOM.length];
        
        for ( int i = 0; i < m_homogeneity_HOM.length; i++ )
        {
            HOM[i] = (int)( 1000f * m_homogeneity_HOM[i] );
        }
        
        return HOM;
    }
    
    /**
     * Returns the Angular Second Moment (ASM) of the GLCM
     * @return 
     */
    public int[] getGLCM_AngularSecondMoment_ASM_AsScaledIntArray( )
    {
        int[] ASM = new int[m_angularSecondMoment_ASM.length];
        
        for ( int i = 0; i < m_angularSecondMoment_ASM.length; i++ )
        {
            ASM[i] = (int)( 1000f * m_angularSecondMoment_ASM[i] );
        }
        
        return ASM;
    }
    
    
    /**
     * Returns the Energy (ENE) of the GLCM
     * @return 
     */
    public int[] getGLCM_Energy_AsScaledIntArray( )
    {
        int[] ENE = new int[m_energy.length];
        
        for ( int i = 0; i < m_energy.length; i++ )
        {
            ENE[i] = (int)( 1000f * m_energy[i] );
        }
        
        return ENE;
    }
    
    /**
     * Returns the Maximum probability (MAX) of the GLCM
     * @return 
     */
    public int[] getGLCM_MAXimumProbability_AsScaledIntArray( )
    {
        int[] MAX = new int[m_maximumProbability_MAX.length];
        
        for ( int i = 0; i < m_maximumProbability_MAX.length; i++ )
        {
            MAX[i] = (int)( 1000f * m_maximumProbability_MAX[i] );
        }
        
        return MAX;
    }
        
    /**
     * Returns the Entropy (ENT) of the GLCM
     * @return 
     */
    public int[] getGLCM_ENTropy_AsScaledIntArray( )
    {
        int[] ENT = new int[m_entropy_ENT.length];
        
        for ( int i = 0; i < m_entropy_ENT.length; i++ )
        {
            ENT[i] = (int)( 1000f * m_entropy_ENT[i] );
        }
        
        return ENT;
    }

    /**
     * Returns the GLCM_MEAN of the GLCM
     * @return 
     */
    public int[] getGLCM_Mean_AsScaledIntArray( )
    {
        int[] MEAN = new int[m_GLCMMean.length];
        
        for ( int i = 0; i < m_GLCMMean.length; i++ )
        {
            MEAN[i] = (int)( 1000f * m_GLCMMean[i] );
        }
        
        return MEAN;
    }

    /**
     * Returns the GLCM_CORR of the GLCM
     * @return 
     */
    public int[] getGLCM_Correlation_AsScaledIntArray( )
    {
        int[] CORR = new int[m_GLCMCorrelation.length];
        
        for ( int i = 0; i < m_GLCMCorrelation.length; i++ )
        {
            CORR[i] = (int)( 1000f * m_GLCMCorrelation[i] );
        }
        
        return CORR;
    }
    
    /**
     * Returns the GLCM_VAR of the GLCM
     * @return 
     */
    public int[] getGLCM_Variance_AsScaledIntArray( )
    {
        int[] VAR = new int[m_GLCM_variance.length];
        
        for ( int i = 0; i < m_GLCM_variance.length; i++ )
        {
            VAR[i] = (int)( 1000f * m_GLCM_variance[i] );
        }
        
        return VAR;
    }
    
    /**
     * retuns true if the computation ended with success
     * @return 
     */
    public boolean isSuccessfullyComputed( )
    {
        return m_isSuccessfullyComputed;
    }
    
    // basic member attributes
    private int m_numberOfImages            = 1;
    private int m_image_width               = 256;
    private int m_image_height              = 256;
    private int m_window_width              = 2; // total # of pixels 2 * m_window_width + 1
    private int m_window_height             = 2;
    private int m_grey_levels               = 16;
    
    private int[] m_image                   = null;
    private int[] m_reducedGreyLevelImage   = null;
    
    private float[][] m_coocMatrixHor       = null;
    private float[][] m_coocMatrixVert      = null;
    private float[][] m_coocMatrixNE        = null;
    private float[][] m_coocMatrixSE        = null;
    
    public final static int HORIZONTAL      = 0;
    public final static int VERTICAL        = 1;
    public final static int NORTHEAST       = 2;
    public final static int SOUTHEAST       = 3;
    
    private boolean m_verbose = false;
    
    // data structures to store texture maps into
    private float[] m_contrast_CON;
    private float[] m_dissimilarity_DIS;
    private float[] m_homogeneity_HOM;
    private float[] m_angularSecondMoment_ASM;
    private float[] m_energy;
    private float[] m_maximumProbability_MAX;
    private float[] m_entropy_ENT;
    private float[] m_GLCMMean;
    private float[] m_GLCMCorrelation;
    private float[] m_GLCM_variance;
    
    private boolean m_isSuccessfullyComputed = false;
    
}
