/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.texturemaps;

import java.util.Iterator;
import java.util.LinkedList;

/**
 *  Container class containing the first four moments about the mean and their
 *  variances.
 * 
 *      - This data object knows to which image it belongs (dicom instance uid & file reference)
 *      - This data object knows to which ROI object it belongs: reference to ROI file object
 *      - This data object knows which normalization factor was used for computation
 * 
 * @author slotboom
 */
public class MomentsAboutTheMean
{
    
    /**
     * Default constructor
     */
    public MomentsAboutTheMean( )
    {

    }
    
    public void setNumberOfROIPixels( int numberOfPixels ) 
    {
       m_numberOfPixels = numberOfPixels;
    }
    
    public int getNumberOfROIPixels( ) 
    {
       return m_numberOfPixels;
    }
    
    
    
    /**
     * Sets the patient ID 
     * @param patientID 
     */
    public void setPatientID( String patientID )
    {
        m_patientID = patientID;
    }
    
    public String getPatientID( )
    {
        return m_patientID;
    }
    
    /**
     * Sets the type of texture parameter map to which the numerical data belongs
     * @param tpmType 
     */
    public void setTPMTypeNumber( int tpmTypeNumber )
    {
        m_tpmTypeNumber = tpmTypeNumber;
        
        switch ( tpmTypeNumber )
        {
            case TPM_TYPE_ORIG: m_tpmTypeString = TPM_TYPE_STRING_ORIG; break;
            case TPM_TYPE_AVER: m_tpmTypeString = TPM_TYPE_STRING_AVER; break;
            case TPM_TYPE_VARI: m_tpmTypeString = TPM_TYPE_STRING_VARI; break;
            case TPM_TYPE_STDE: m_tpmTypeString = TPM_TYPE_STRING_STDE; break;
            case TPM_TYPE_VAVA: m_tpmTypeString = TPM_TYPE_STRING_VAVA; break;
            case TPM_TYPE_KURT: m_tpmTypeString = TPM_TYPE_STRING_KURT; break;
            case TPM_TYPE_SKEW: m_tpmTypeString = TPM_TYPE_STRING_SKEW; break;
    
            // cooccurence based tpm parameters 
            case TPM_TYPE_CMEA: m_tpmTypeString = TPM_TYPE_STRING_CMEA; break;
            case TPM_TYPE_CCON: m_tpmTypeString = TPM_TYPE_STRING_CCON; break;
            case TPM_TYPE_CDIS: m_tpmTypeString = TPM_TYPE_STRING_CDIS; break;
            case TPM_TYPE_CHOM: m_tpmTypeString = TPM_TYPE_STRING_CHOM; break;
            case TPM_TYPE_CASM: m_tpmTypeString = TPM_TYPE_STRING_CASM; break;
            case TPM_TYPE_CENE: m_tpmTypeString = TPM_TYPE_STRING_CENE; break;
            case TPM_TYPE_CMAX: m_tpmTypeString = TPM_TYPE_STRING_CMAX; break;
            case TPM_TYPE_CENT: m_tpmTypeString = TPM_TYPE_STRING_CENT; break;
            case TPM_TYPE_CVAR: m_tpmTypeString = TPM_TYPE_STRING_CVAR; break;
            case TPM_TYPE_CCOR: m_tpmTypeString = TPM_TYPE_STRING_CCOR; break;
                
            case TPM_TYPE_DIFF: m_tpmTypeString = TPM_TYPE_STRING_DIFF; break;
                
        }
    
    }
    
    
    /**
     * Returns the tmp type number
     * @return 
     */
    public int getTPMTypeNumber( )
    {
        return m_tpmTypeNumber;
    }
    
    /**
     * Returns the tmp type number
     * @return 
     */
    public String getTPMTypeString( )
    {
        return m_tpmTypeString;
    }
    
    /**
     * Defines the normalization factor which was used to compute the values 
     * contained in the object of this class
     * 
     * @param normalizationFactor 
     */
    public void setNormalizationConstant( double normalizationFactor )
    {
        m_normalizationFactor = normalizationFactor;
    }
    
    public double getNormalizationConstant( )
    {
        return m_normalizationFactor;
    }
    
    /**
     * Sets the reference to the image Dicom file
     * @param dicomImageFileReference 
     */
    public void setDicomImageFileReference( String dicomFileLocation )
    {
        m_dicomFileLocation = dicomFileLocation;
    }
    
    /**
     * Returns the reference to the image Dicom file
     * @return m_dicomFileLocation 
     */
    public String getDicomImageFileReference( )
    {
        return m_dicomFileLocation;
    }
    
    /**
     * Dicom related information
     * 
     * @return Dicom instance uid
     */
    public String getInstanceSeriesUID( )
    {
        return m_instanceUID;
    }
    
    /**
     * Set Dicom image instance uid
     * @param instanceUID 
     */
    public void setInstanceSeriesUID( String instanceUID )
    {
        m_instanceUID = instanceUID;
    }
    
    /**
     * Dicom file location
     * 
     * @return m_dicomFileLocation
     */
    public String getDicomFileLocation( )
    {
        return m_dicomFileLocation;
    }
    
    /**
     * Set Dicom image file location
     * @param dicomFileLocation 
     */
    public void setDicomFileLocation( String dicomFileLocation )
    {
        m_dicomFileLocation = dicomFileLocation;
    }       

    /**
     * Sets the mean value attribute
     * @param mean
     */
    public void setMean( double mean )
    {
        m_mean = mean;
    }

    /**
     * Sets the stdev value attribute
     * @param stdev
     */
    public void setStDev( double stdev )
    {
        m_stdev = stdev;
    }
    /**
     *
     * Sets the variance value attribute
     * @param variance
     */
    public void setVariance( double variance )
    {
        m_variance = variance;
    }

    /**
     * Sets the variance of the variance value attribute
     * @param varianceOfVariance
     */
    public void setVarianceOfVariance( double varianceOfVariance )
    {
        m_varianceOfVariance = varianceOfVariance;
    }

    /**this
     * Sets the skewness value attribute
     * @param skewness
     */
    public void setSkewness( double skewness )
    {
        m_skewness = skewness;
    }

    /**
     * Sets the variance of the skewness value attribute
     * @param varianceOfSkewness
     */
    public void setVarianceOfSkewness( double varianceOfSkewness )
    {
        m_varianceOfSkewness = varianceOfSkewness;
    }

    /**
     * Sets the kurtosis value attribute
     * @param kurtosis
     */
    public void setKurtosis( double kurtosis )
    {
        m_kurtosis = kurtosis;
    }


    /**
     * Sets the variance of the kurtosis attribute
     * @param varianceOfKurtosis
     */
    public void setVarianceOfKurtosis( double varianceOfKurtosis )
    {
        m_varianceOfKurtosis = varianceOfKurtosis;
    }

    /**
     * Returns the mean value attribute
     */
    public double getMean( )
    {
        return m_mean;
    }

    /**
     * Returns the stdev value attribute
     */
    public double getStDev( )
    {
        return m_stdev;
    }

    /**
     * Returns the variance value attribute
     */
    public double getVariance( )
    {
        return m_variance;
    }

    /**
     * Returns the variance of the variance value attribute
     */
    public double getLogVarianceOfVariance( )
    {
        if ( m_varianceOfVariance == 0 )
        {
            return 0;
        }
        else
        {
            return Math.log( m_varianceOfVariance );
        }
    }

    /**
     * Returns the variance of the variance value attribute
     */
    public double getVarianceOfVariance( )
    {
        return m_varianceOfVariance;
    }
    
    
    /**
     * Returns the skewness value attribute
     */
    public double getSkewness( )
    {
        return m_skewness;
    }

    /**
     * Returns the variance of the skewness value attribute
     */
    public double getVarianceOfSkewness( )
    {
        return m_varianceOfSkewness;
    }

    /**
     * Returns the kurtosis value attribute
     */
    public double getKurtosis( )
    {
        return m_kurtosis;
    }


    /**
     * Returns the variance of the kurtosis attribute
     */
    public double getVarianceOfKurtosis( )
    {
        return m_varianceOfKurtosis;
    }

    /**
     * Set slice number value
     * @param sliceNumber 
     */
    public void setSliceNumber(int sliceNumber) 
    {
        m_sliceNumber = sliceNumber;
    }
    /**
     * Return slice number value
     * @return 
     */
    public int getSliceNumber( ) 
    {
        return m_sliceNumber;
    }
    
    public void setPolygonName( String polygonName ) 
    {
        m_polygonName = polygonName;
    }
    
    public String getPolygonName( ) 
    {
        return m_polygonName;
    }
    
    public void setExaminedVolumeSize( float volumeSize ) 
    {
        m_volumeSize = volumeSize;
    }
    
    public float getExaminedVolumeSize( ) 
    {
        return m_volumeSize;
    }
    
    /**
     * Returns the number of pixels on which computation is based
     * @return m_numberOfPixels
     */
    public int getNumberOfPixels( )
    {
        return m_numberOfPixels;
    }
    
    /**
     * Set the number of pixels on which computation is based
     */
    public void setNumberOfPixels( int pixel_number )
    {
        m_numberOfPixels = pixel_number;
    }
    
    /**
     * Returns the a MomentsAboutTheMean object the contains
     * @param list
     * @return 
     */   
    static public MomentsAboutTheMean ComputeWeightedAverages( LinkedList<MomentsAboutTheMean> list,
                                                               float volumePerPixel,
                                                               int TPMTypeNumber,
                                                               String TPMTypeString)
    {
        
        MomentsAboutTheMean returnObj= new MomentsAboutTheMean( );
        int sum_of_pixels = 0;
        
        returnObj.setDicomFileLocation( list.getFirst( ).getDicomFileLocation( ) );
        returnObj.setDicomImageFileReference( list.getFirst( ).getDicomImageFileReference( ) );
        returnObj.setInstanceSeriesUID( list.getFirst( ).getInstanceSeriesUID( ) );
        returnObj.setNormalizationConstant( list.getFirst( ).getNormalizationConstant( ) );
        returnObj.setPatientID( list.getFirst( ).getPatientID( ) );
        returnObj.setPolygonName( "sum of polygons" );
        returnObj.setSliceNumber( -1 );
        returnObj.setTPMTypeNumber( TPMTypeNumber );
        returnObj.setTPMTypeString( TPMTypeString );
        
        
        // first create the weighted sum
        for ( MomentsAboutTheMean mam : list )
        {
            
            returnObj.setNumberOfROIPixels( returnObj.getNumberOfROIPixels( ) + mam.getNumberOfROIPixels( ) );
            returnObj.setMean( returnObj.getMean( ) + mam.getMean( ) * mam.getNumberOfROIPixels( ) );
            returnObj.setStDev( returnObj.getStDev( ) + mam.getStDev( )* mam.getNumberOfROIPixels( ) );
            returnObj.setVariance( returnObj.getVariance( ) + mam.getVariance( ) * mam.getNumberOfROIPixels( ) ); 
            returnObj.setVarianceOfVariance( returnObj.getVarianceOfVariance( ) + mam.getVarianceOfVariance( ) * mam.getNumberOfROIPixels( ) );
            returnObj.setSkewness(  returnObj.getSkewness( ) + mam.getSkewness( ) * mam.getNumberOfROIPixels( ) );
            returnObj.setVarianceOfSkewness(  returnObj.getVarianceOfSkewness( ) + mam.getVarianceOfSkewness( ) * mam.getNumberOfROIPixels() ); 
            returnObj.setKurtosis( returnObj.getKurtosis( ) + mam.getKurtosis( ) * mam.getNumberOfROIPixels( ) );
            returnObj.setVarianceOfKurtosis( returnObj.getVarianceOfKurtosis( ) + mam.getVarianceOfKurtosis( ) * mam.getNumberOfROIPixels( ) );
        }
        
        returnObj.setNumberOfPixels( returnObj.getNumberOfROIPixels( ) );
        returnObj.setExaminedVolumeSize( returnObj.getNumberOfROIPixels( ) * volumePerPixel );
        returnObj.setMean( returnObj.getMean( ) / returnObj.getNumberOfROIPixels( ) );
        returnObj.setStDev( returnObj.getStDev( ) / returnObj.getNumberOfROIPixels( ) );
        returnObj.setVariance( returnObj.getVariance( ) / returnObj.getNumberOfROIPixels( ) ); 
        returnObj.setVarianceOfVariance( returnObj.getVarianceOfVariance( ) / returnObj.getNumberOfROIPixels() ); 
        returnObj.setSkewness( returnObj.getSkewness( ) / returnObj.getNumberOfROIPixels( ) );
        returnObj.setVarianceOfSkewness( returnObj.getVarianceOfSkewness( ) * returnObj.getNumberOfROIPixels( ) ); 
        returnObj.setKurtosis( returnObj.getKurtosis( ) / returnObj.getNumberOfROIPixels() );
        returnObj.setVarianceOfKurtosis( returnObj.getVarianceOfKurtosis( ) / returnObj.getNumberOfROIPixels( ) );
    
        return returnObj;
    }
      
    
    // attributes than contain the numeric values of the computation
    private double        m_mean                 = 0.0;
    private double        m_stdev                = 0.0;

    private double        m_variance             = 0.0;
    private double        m_varianceOfVariance   = 0.0;

    private double        m_skewness             = 0.0;
    private double        m_varianceOfSkewness   = 0.0;

    private double        m_kurtosis             = 0.0;
    private double        m_varianceOfKurtosis   = 0.0;
    
    private double        m_normalizationFactor  = 1.0;
    
    private String        m_dicomFileLocation    = null;
    private String        m_regioPolyonFileName  = null;
    
    private String        m_instanceUID = null;
    private String        m_patientID   = null;
    
    private int           m_tpmTypeNumber;
    private String        m_tpmTypeString;
    
    private int           m_numberOfPixels;
    private int           m_sliceNumber;
    
    private String        m_polygonName;
    private float         m_volumeSize = 0f;
    private float         m_volumePerPixel = 0f;
    
    // histogram parameters
    public final static int TPM_TYPE_ORIG = 0;
    public final static int TPM_TYPE_AVER = 1;
    public final static int TPM_TYPE_VARI = 2;
    public final static int TPM_TYPE_STDE = 3;
    public final static int TPM_TYPE_VAVA = 4;
    public final static int TPM_TYPE_KURT = 5;
    public final static int TPM_TYPE_SKEW = 6;
    
    // cooccurence based parameters 
    public final static int TPM_TYPE_CMEA = 7;
    public final static int TPM_TYPE_CCON = 8;
    public final static int TPM_TYPE_CDIS = 9;
    public final static int TPM_TYPE_CHOM = 10;
    public final static int TPM_TYPE_CASM = 11;
    public final static int TPM_TYPE_CENE = 12;
    public final static int TPM_TYPE_CMAX = 13;
    public final static int TPM_TYPE_CENT = 14;
    public final static int TPM_TYPE_CVAR = 15;
    public final static int TPM_TYPE_CCOR = 16;
    
    public final static int TPM_TYPE_DIFF = 17;
    
    // histogram parameters
    public final static String TPM_TYPE_STRING_ORIG = "ORIG";
    public final static String TPM_TYPE_STRING_AVER = "AVER";
    public final static String TPM_TYPE_STRING_VARI = "VARI";
    public final static String TPM_TYPE_STRING_STDE = "STDE";
    public final static String TPM_TYPE_STRING_VAVA = "VAVA";
    public final static String TPM_TYPE_STRING_KURT = "KURT";
    public final static String TPM_TYPE_STRING_SKEW = "SKEW";
    
    // cooccurence based parameters 
    public final static String TPM_TYPE_STRING_CMEA = "CMEA";
    public final static String TPM_TYPE_STRING_CCON = "CCON";
    public final static String TPM_TYPE_STRING_CDIS = "CDIS";
    public final static String TPM_TYPE_STRING_CHOM = "CHOM";
    public final static String TPM_TYPE_STRING_CASM = "CASM";
    public final static String TPM_TYPE_STRING_CENE = "CENE";
    public final static String TPM_TYPE_STRING_CMAX = "CMAX";
    public final static String TPM_TYPE_STRING_CENT = "CENT";
    public final static String TPM_TYPE_STRING_CVAR = "CVAR";
    public final static String TPM_TYPE_STRING_CCOR = "CCOR";
    
    public final static String TPM_TYPE_STRING_DIFF = "DIFF";
    
    public void setTPMTypeString( String tpmType ) 
    {
        switch( tpmType )
        {
            case "ORIG": m_tpmTypeNumber = TPM_TYPE_ORIG; 
                         m_tpmTypeString = TPM_TYPE_STRING_ORIG;
                         break;
            case "AVER": m_tpmTypeNumber = TPM_TYPE_AVER; 
                         m_tpmTypeString = TPM_TYPE_STRING_AVER;
                         break;
            case "DIFF": m_tpmTypeNumber = TPM_TYPE_DIFF; 
                         m_tpmTypeString = TPM_TYPE_STRING_DIFF;
                         break;
            case "VARI": m_tpmTypeNumber = TPM_TYPE_VARI; 
                         m_tpmTypeString = TPM_TYPE_STRING_VARI;
                         break;
            case "STDE": m_tpmTypeNumber = TPM_TYPE_STDE; 
                         m_tpmTypeString = TPM_TYPE_STRING_STDE;
                         break;
            case "VAVA": m_tpmTypeNumber = TPM_TYPE_VAVA;
                         m_tpmTypeString = TPM_TYPE_STRING_VAVA; 
                         break;
            case "KURT": m_tpmTypeNumber = TPM_TYPE_KURT;
                         m_tpmTypeString = TPM_TYPE_STRING_KURT;
                         break;   
            case "SKEW": m_tpmTypeNumber = TPM_TYPE_SKEW;
                         m_tpmTypeString = TPM_TYPE_STRING_SKEW;
                         break;
            case "CMEA": m_tpmTypeNumber = TPM_TYPE_CMEA;
                         m_tpmTypeString = TPM_TYPE_STRING_CMEA;
                         break;
            case "CCON": m_tpmTypeNumber = TPM_TYPE_CCON;
                         m_tpmTypeString = TPM_TYPE_STRING_CCON;
                         break;
            case "CDIS": m_tpmTypeNumber = TPM_TYPE_CDIS;
                         m_tpmTypeString = TPM_TYPE_STRING_CDIS;
                         break;
            case "CHOM": m_tpmTypeNumber = TPM_TYPE_CHOM;
                         m_tpmTypeString = TPM_TYPE_STRING_CHOM;
                         break;
            case "CASM": m_tpmTypeNumber = TPM_TYPE_CASM;
                         m_tpmTypeString = TPM_TYPE_STRING_CASM;
                         break;
            case "CENE": m_tpmTypeNumber = TPM_TYPE_CENE;
                         m_tpmTypeString = TPM_TYPE_STRING_CENE;
                         break;
            case "CMAX": m_tpmTypeNumber = TPM_TYPE_CMAX;
                         m_tpmTypeString = TPM_TYPE_STRING_CMAX;
                         break;
            case "CENT": m_tpmTypeNumber = TPM_TYPE_CENT;
                         m_tpmTypeString = TPM_TYPE_STRING_CENT;
                         break;
            case "CVAR": m_tpmTypeNumber = TPM_TYPE_CVAR;
                         m_tpmTypeString = TPM_TYPE_STRING_CVAR;
                         break;
            case "CCOR": m_tpmTypeNumber = TPM_TYPE_CCOR;
                         m_tpmTypeString = TPM_TYPE_STRING_CCOR;
                         break;   
        }
    } 

}