/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.toolbars;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 * JToolBar to enable quick and user friendly preprocessing steps.
 * @author Johannes Slotboom 
 */
public final class SpectrImMainSignalPreprocessingToolbar extends JPanel implements ActionListener, ItemListener
{
    
    public SpectrImMainSignalPreprocessingToolbar ( )
    {
        super( new BorderLayout( ) );
        // Create the toolbar.
        JToolBar toolBar = new JToolBar( "TDFDFit model toolbar " );
        addButtons( toolBar );
        
        add( toolBar, BorderLayout.PAGE_START );
        
    
        fillComboboxes( );
        setVisible( true );
        
    }

    public void fillComboboxes( )
    {
        
        File currentTDFDFitModelBasePathFiles   = null;
        String currentWaterModel                = null;
        String currentMetaboliteModel           = null;
        File[] currentModeToplDirs              = null;
        
        m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.removeItemListener( this );
        m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.removeItemListener( this );
        m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.removeItemListener( this );
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String TDFDFitModelBasePath = defs.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH );
        
        
        if ( TDFDFitModelBasePath != null )
        {
            currentTDFDFitModelBasePathFiles = new File( TDFDFitModelBasePath );
        
            currentWaterModel = defs.getProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME );
        
            currentMetaboliteModel = defs.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        }
        
        m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.removeAllItems( );   
        m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.removeAllItems( );
        m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.removeAllItems( ); 
        
        m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.addItem( "No model defined yet!" );
        m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.addItem( "No model defined yet!" );
        m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.addItem( "No model defined yet!" );
        int selectedIndexWaterModelIndex    = 0;
        int selectedIndexMetaboliteModel    = 0;
        int modelCounter = 0;
        
        m_allDetectedTFMFiles.clear( );
        
        if (  TDFDFitModelBasePath != null )
        {
            currentModeToplDirs = currentTDFDFitModelBasePathFiles.listFiles( );
            for ( int i = 0; i < currentModeToplDirs.length; i++ )
            {
                // We search only one subdirectory deep! 
                if ( currentModeToplDirs[i].isDirectory( ) )
                {
                    File[] filesInModelDir = currentModeToplDirs[i].listFiles( );
                    for ( int j = 0; j < filesInModelDir.length; j++ )
                    {
                        if ( filesInModelDir[j].getName( ).endsWith( ".tfm" ) == true )
                        {
                        
                            m_allDetectedTFMFiles.add( filesInModelDir[j] );
                            // System.out.println( filesInModelDir[j] );
                        
                            m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.addItem( filesInModelDir[j].getName( ) );
                            m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.addItem( filesInModelDir[j].getName( ) );
                        
                            if ( filesInModelDir[j].getName( ).compareTo( currentWaterModel ) == 0 )
                            {
                                selectedIndexWaterModelIndex = modelCounter + 1;
                            }
                        
                            if ( filesInModelDir[j].getName( ).compareTo( currentMetaboliteModel ) == 0 )
                            {
                                selectedIndexMetaboliteModel = modelCounter + 1;
                            }
                            modelCounter++;
                        }
                    }
                }   
            }
        }
         
        if ( modelCounter > 0 )
        {
            m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.setSelectedIndex( selectedIndexWaterModelIndex );
            m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.setSelectedIndex( selectedIndexMetaboliteModel );
        }
        else
        {
            m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.setSelectedIndex( 0 ); // displays No model defined yet!
            m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.setSelectedIndex( 0 ); // displays No model defined yet!
        }
        
        String splineNodeModelPath = TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH );
        String splineNodeModelName = TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME );
        
        if ( splineNodeModelPath != null )
        {
            File f = new File( splineNodeModelPath );

            // Populates the array with names of files and directories
            String[] pathnames = f.list( );
        
            // For each pathname in the pathnames array
            for ( String pathname : pathnames ) 
            {
                // Print the names of files and directories
                // System.out.println( pathname ); 
                if ( pathname.contains( ".snm" ) == true )
                {
                    m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.addItem( pathname );
                }
            }
        
            if ( splineNodeModelName != null )
            {
                m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.setSelectedItem( splineNodeModelName );
            }
        }
        
        // Finally add Item listeners! 
        m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.addItemListener( this );
        m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.addItemListener( this );
        
    }
    
    
    
    /**
     * Add the buttons to the JToolBar.
     * @param toolBar
     */
    protected void addButtons( JToolBar toolBar )
    {  
         // first item
        m_refreshToolbarComboboxesButton = makeBasicFileToolBarButton( "Refresh24", 
                                                                       APPLY_REFRESH_TOOLBAR_COMBOBOXES,
                                                                       APPLY_REFRESH_TOOLBAR_COMBOBOXES,
                                                                       "RefreshComboboxes" );
        toolBar.add( m_refreshToolbarComboboxesButton );
        
        // second item
        m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox = new JComboBox( new String[] { "item 1", "item 2", "item 3" } );
        toolBar.add( m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox );
       
        // third item
        m_applyWaterRemovalTDFDFitButton = makeBasicFileToolBarButton( "ApplyWatRemTDFDfit24", APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL,
                                                                       APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL,
                                                                       "ApplyWRTDFDfit" );
        toolBar.add( m_applyWaterRemovalTDFDFitButton );
        
        
        // fourth item
        m_applyInteractiveHLSVDButton = makeBasicFileToolBarButton( "ApplyIntHLSVD24", APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL,
                                                                    APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL,
                                                                    "ApplySVD" );
        toolBar.add( m_applyInteractiveHLSVDButton );
        
        // fifth item
        m_applyHLSVDButton = makeBasicFileToolBarButton("ApplyHLSVD24", APPLY_HLSVD_PRO_WATER_REMOVAL,
                                                         APPLY_HLSVD_PRO_WATER_REMOVAL,
                                                         "ApplyHLSVD" );
        toolBar.add( m_applyHLSVDButton );
        
        // sixth item
        m_applyApodizationButton = makeBasicFileToolBarButton( "ApplyApodization24", APPLY_APODIZATION,
                                                               APPLY_APODIZATION,
                                                               "ApplyApodization" );
        toolBar.add( m_applyApodizationButton );

        // seventh item
        m_multiplyFirstTimePointButton = makeBasicFileToolBarButton( "FirstTDPointMult24", APPLY_FIRST_TD_POINT_MULTIPLICATION,
                                                                     APPLY_FIRST_TD_POINT_MULTIPLICATION,
                                                                     "FirstTDPointMult" );
        m_multiplyFirstTimePointButton.setEnabled( false );
        toolBar.add( m_multiplyFirstTimePointButton );

        // eighth item
        m_offsetCorrectionFDButton = makeBasicFileToolBarButton( "OffsetCorrectionFD24", APPLY_OFFSET_CORRECTION_FD,
                                                                 APPLY_OFFSET_CORRECTION_FD,
                                                                 "OffsetCorrectionFD" );
        toolBar.add( m_offsetCorrectionFDButton );
        
        // nineth item
        m_offsetCorrectionTDButton = makeBasicFileToolBarButton( "OffsetCorrectionTD24", APPLY_OFFSET_CORRECTION_TD,
                                                                 APPLY_OFFSET_CORRECTION_TD,
                                                                 "OffsetCorrectionTD" );
        toolBar.add( m_offsetCorrectionTDButton );
        
        // tenth item
        m_applyZeroFillingButton = makeBasicFileToolBarButton( "ApplyZeroFilling24", APPLY_ZERO_FILLING,
                                                               APPLY_ZERO_FILLING,
                                                               "ApplyZeroFilling" );
        
        toolBar.add( m_applyZeroFillingButton );

        // eleventh item
        m_applyManualPhasingButton = makeBasicFileToolBarButton( "ManualPhasing24", APPLY_MANUAL_PHASING,
                                                                 APPLY_MANUAL_PHASING,
                                                                 "ManualPhasing" );
        toolBar.add( m_applyManualPhasingButton );
        
        // twelfth item
        m_applyAutomaticPhasingButton = makeBasicFileToolBarButton( "AutoPhasing24", APPLY_AUTOMATIC_PHASING,
                                                                    APPLY_AUTOMATIC_PHASING,
                                                                    "ManualPhasing" );
        toolBar.add( m_applyAutomaticPhasingButton );
        
        // thirtheenth item
        m_applyFrequencyShiftButton = makeBasicFileToolBarButton( "ApplyManualFrequencyShift24", APPLY_INTERACTIVE_FREQ_SHIFT,
                                                                  APPLY_INTERACTIVE_FREQ_SHIFT,
                                                                  "ApplyManualFreqShift" );
        toolBar.add( m_applyFrequencyShiftButton );
        
        // eightteenth item
        m_applyComplexConjugationButton = makeBasicFileToolBarButton( "ApplySpectrumFlipping24", APPLY_COMPLEX_CONJUGATION,
                                                                      APPLY_COMPLEX_CONJUGATION,
                                                                      "ApplyComplexConjugation" );
        toolBar.add( m_applyComplexConjugationButton );
        
        
        
        // fourteenth item
        m_manualBaseLineRemovalButton = makeBasicFileToolBarButton("ApplyManualBaselineRemoval24", APPLY_NON_INTERACTIVE_BASELINE_REMOVAL,
                                                                    APPLY_NON_INTERACTIVE_BASELINE_REMOVAL,
                                                                    "ApplyManualBaselineRemoval" );
        toolBar.add( m_manualBaseLineRemovalButton );
        
        m_manualBaseLineRemovalButton.setEnabled( true );
        
        m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox = new JComboBox( new String[] { "item 1", "item 2", "item 3" } );
        toolBar.add( m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox );
        
        m_defineManualBaseLineRemovalSettingsButton = makeBasicFileToolBarButton( "DefineManualBaselineRemovalSettings24", 
                                                                                  DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS,
                                                                                  DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS,
                                                                                  "ApplyManualBaselineRemoval" );
        toolBar.add( m_defineManualBaseLineRemovalSettingsButton );
        
        m_defineManualBaseLineRemovalSettingsButton.setEnabled( true );
        
        // fifteenth item
        m_quantifySpectraWithTDFDFitButton = makeBasicFileToolBarButton( "QuantifyWithTDFDFit24", APPLY_TDFDFIT_QUANTIFICATION,
                                                                         APPLY_TDFDFIT_QUANTIFICATION,
                                                                         "ApplyTDFDFitQuantification" );
        toolBar.add( m_quantifySpectraWithTDFDFitButton );
             
        // sixteenth item
        m_quantifySpectraWithTDFDFitModelNameSelectionComboBox = new JComboBox( new String[] { "item 1", "item 2", "item 3" } );
        toolBar.add( m_quantifySpectraWithTDFDFitModelNameSelectionComboBox );
        
        
        // seventeenth button
        m_lauchTDFDFitMetaboliteModelEditorButton = makeBasicFileToolBarButton( "EditTDFDFit24", EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL,
                                                                                EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL,
                                                                                "EditTDFDFitMetabModel" );
        toolBar.add( m_lauchTDFDFitMetaboliteModelEditorButton );
        
        // first button
        m_createNewModelButton = makeBasicFileToolBarButton( "New24", CREATE_AND_ADD_NEW_MODEL,
                                                             CREATE_AND_ADD_NEW_MODEL,
                                                             "Create model" );
        toolBar.add( m_createNewModelButton );
        
        
        // first button
        m_editTDFDFitDefaultsButton = makeBasicFileToolBarButton( "TDFDFit_Preferences24", EDIT_TDFDFIT_DEFAULTS,
                                                                  EDIT_TDFDFIT_DEFAULTS,
                                                                  "Edit TDFDFIT model defaults" );
        toolBar.add( m_editTDFDFitDefaultsButton );
//        // tenth button
//        m_launchQuantitativeViewerButton = makeBasicFileToolBarButton( "QuantitativeEvaluationSVS24", QUANTITATIVE_VIEWER,
//                                                                       QUANTITATIVE_VIEWER,
//                                                                       "Launch-quant-viewer" );
//        toolBar.add( m_launchQuantitativeViewerButton );
//        m_launchQuantitativeViewerButton.setEnabled(false);
//        
//        // eleventh button
//        m_setDefaultParameterRangesButton = makeBasicFileToolBarButton( "RangeDefine24", SET_DEFAULT_PARAMETER_RANGES,
//                                                                        SET_DEFAULT_PARAMETER_RANGES,
//                                                                        "Launch-param-range-definer" );
//        toolBar.add( m_setDefaultParameterRangesButton );
        
    }


    /**
     *
     * Creates all buttons of the JToolBar.
     *
     * @param imageName
     * @param actionCommand
     * @param toolTipText
     * @param altText
     * @return the button to add
     */
    protected JButton makeBasicFileToolBarButton( String imageName,
                                                  String actionCommand,
                                                  String toolTipText,
                                                  String altText )
    {
        //Look for the image.
        String workingDir = System.getProperty( "user.dir" );
        String imgLocation = workingDir + System.getProperty( "file.separator" )
                                        + "images"
                                        + System.getProperty( "file.separator" )
                                        + imageName
                                        + ".gif";
        File imageFile = new File( imgLocation );

        //Create and initialize the button.
        JButton button = new JButton( );
        button.setActionCommand( actionCommand );
        button.setToolTipText( toolTipText );
        button.addActionListener( this );

        if ( imageFile.exists( ) )
        {                      //image found
            button.setIcon( new ImageIcon( imgLocation ) );
        } 
        else
        {                                     //no image found
            button.setText( altText );
            System.err.println( "Resource not found: "
                               + imgLocation );
        }

        return button;
    }
    
    /**
     *
     * Is called whenever a registered component raises an ActionEvent.
     *
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        String cmd = e.getActionCommand( );
        Object sender = e.getSource( );
        String description = null;
        
        
        if ( sender == m_refreshToolbarComboboxesButton )
        {
            this.fillComboboxes( );
        }
        
        // Handle each button.
        if ( APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL.equals( cmd ) )
        {   // first button clicked
            notifyActionListeners( APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL, APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL_ID );
        }
        else if ( APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL.equals( cmd ) )
        {   // second button clicked
            notifyActionListeners( APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL, APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL_ID );
        }
        else if ( APPLY_HLSVD_PRO_WATER_REMOVAL.equals( cmd ) )
        {   // third button clicked
            notifyActionListeners(APPLY_HLSVD_PRO_WATER_REMOVAL, APPLY_HLSVD_PRO_WATER_REMOVAL_ID );
        } 
        else if ( APPLY_APODIZATION.equals( cmd ) )
        {   // fourth button clicked
            notifyActionListeners( APPLY_APODIZATION, APPLY_APODIZATION_ID );
        }
        else if ( APPLY_FIRST_TD_POINT_MULTIPLICATION.equals( cmd ) )
        {   // fifth button clicked
            notifyActionListeners( APPLY_FIRST_TD_POINT_MULTIPLICATION, APPLY_FIRST_TD_POINT_MULTIPLICATION_ID );
        }
        else if ( APPLY_OFFSET_CORRECTION_FD.equals( cmd ) )
        {   // sixth button clicked
            notifyActionListeners( APPLY_OFFSET_CORRECTION_FD, APPLY_OFFSET_CORRECTION_FD_ID );
        }
        else if ( APPLY_OFFSET_CORRECTION_TD.equals( cmd ) )
        {   // seventh button clicked
            notifyActionListeners( APPLY_OFFSET_CORRECTION_TD, APPLY_OFFSET_CORRECTION_TD_ID );
        } 
        else if ( APPLY_ZERO_FILLING.equals( cmd ) )
        {   // eigths button clicked
            notifyActionListeners( APPLY_ZERO_FILLING, APPLY_ZERO_FILLING_ID );
        }
        else if ( APPLY_MANUAL_PHASING.equals( cmd ) )
        {   // nineth button clicked
            notifyActionListeners( APPLY_MANUAL_PHASING, APPLY_MANUAL_PHASING_ID );
        }
        else if ( APPLY_AUTOMATIC_PHASING.equals( cmd ) )
        {   // tenth button clicked
            notifyActionListeners( APPLY_AUTOMATIC_PHASING, APPLY_AUTOMATIC_PHASING_ID );
        }
        else if ( APPLY_INTERACTIVE_FREQ_SHIFT.equals( cmd ) )
        {   //eleventh button clicked
            notifyActionListeners( APPLY_INTERACTIVE_FREQ_SHIFT, APPLY_INTERACTIVE_FREQ_SHIFT_ID );
        } 
        else if ( APPLY_NON_INTERACTIVE_BASELINE_REMOVAL.equals( cmd ) )
        {   // twelfth button clicked
            notifyActionListeners( APPLY_NON_INTERACTIVE_BASELINE_REMOVAL, APPLY_MANUAL_BASELINE_REMOVAL_ID );
        }
        else if ( DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS.equals( cmd ) )
        {   // twelfth button clicked
            notifyActionListeners( DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS, DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS_ID );
        }
        else if ( APPLY_TDFDFIT_QUANTIFICATION.equals( cmd ) )
        {   // thirteenth button clicked
            notifyActionListeners( APPLY_TDFDFIT_QUANTIFICATION, APPLY_TDFDFIT_QUANTIFICATION_ID );
        }
        else if ( EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL.equals( cmd ) )
        {   // fourteenth button clicked
            notifyActionListeners( EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL, EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL_ID );
        }
        else if ( CREATE_AND_ADD_NEW_MODEL.equals( cmd ) )
        {   // fifteenth button clicked
            notifyActionListeners( CREATE_AND_ADD_NEW_MODEL, CREATE_AND_ADD_NEW_MODEL_ID );
        }
        else if ( EDIT_TDFDFIT_DEFAULTS.equals( cmd ) )
        {   // fifteenth button clicked
            notifyActionListeners( EDIT_TDFDFIT_DEFAULTS, EDIT_TDFDFIT_DEFAULTS_ID );
        }
        else if ( APPLY_COMPLEX_CONJUGATION.equals( cmd ) )
        {
            notifyActionListeners( APPLY_COMPLEX_CONJUGATION, APPLY_COMPLEX_CONJUGATION_ID );
        }
                           
        if ( sender ==  m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox )
        {
            int selWaterModelIndex      = m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.getSelectedIndex( );
            String selWaterModelName    = m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.getName( );
        }
        
//        if ( sender ==  m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox )
//        {
//            int selMetaboliteModelIndex        = m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.getSelectedIndex( );
//            String selMetaboliteModelIndexName = m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.getName( );
//            File fullPath = m_allDetectedTFMFiles.get( selMetaboliteModelIndex );
//            
//            TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
//            defs.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME, selMetaboliteModelIndexName );
//            defs.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH, fullPath.getParent( ) );
//            defs.save( );
//        }
//        
//        if ( sender ==  this.m_quantifySpectraWithTDFDFitModelNameSelectionComboBox )
//        {
//            
//            int selMetaboliteModelIndex = m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.getSelectedIndex( );
//            String currentMetaboliteModel = m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.getName( );
//            File fullPath = m_allDetectedTFMFiles.get( selMetaboliteModelIndex );
//             
//            TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );          
//            defs.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, currentMetaboliteModel );
//            defs.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, fullPath.getParent( ) );
//            defs.save( );
//            
//        }
        
    }

     /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( String actionCommand, int actionID )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionID, actionCommand ) );
        }
    }

    /**
     * Adds an ActionListener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    private static void createAndShowGUI( )
    {
        // Create and set up the window.
        JFrame frame = new JFrame( "BasicFileMenuToolbar" );
        frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

        // Add content to the window.
        frame.add(new TDFDFitFileMenuToolbar( ) );

        // Display the window.
        frame.pack( );
        frame.setVisible( true );
    }
    
    public void m_applySVDButtonEnabled( boolean enable )
    {
        m_applyInteractiveHLSVDButton.setEnabled( enable );
    }
    
    public void m_applyHLSVDButton( boolean enable )
    {
        m_applyHLSVDButton.setEnabled( enable );
    }        
            
    public void m_applyApodizationButtonEnabled( boolean enable )
    {
        m_applyApodizationButton.setEnabled( enable );
    }
    
    public void m_multiplyFirstTimePointButtonEnabled( boolean enable )
    {
        m_multiplyFirstTimePointButton.setEnabled( enable );
    }
    
    public void m_offsetCorrectionTDButtonEnabled( boolean enable )
    {
        m_offsetCorrectionTDButton.setEnabled( enable );
    }
    
    public void m_applyZeroFillingButtonEnabled( boolean enable )
    {
        m_applyZeroFillingButton.setEnabled( enable );
    }
    
    public void m_applyManualPhasingButtonEnabled( boolean enable )
    {
        m_applyManualPhasingButton.setEnabled( enable );
    }
    
    public void m_applyFrequencyShiftButtonEnabled( boolean enable )
    {
        m_applyFrequencyShiftButton.setEnabled( enable );
    }
    
    public void m_manualBaseLineRemovalButtonEnabled( boolean enable )
    {
        m_manualBaseLineRemovalButton.setEnabled( enable );
    }
    
    public void m_applyComplexConjugationButton( boolean enable )
    {
        m_applyComplexConjugationButton.setEnabled( enable );
    }
    
    public static void main( String[] args )
    {
        // Schedule a job for the event dispatch thread:
        // creating and showing this application's GUI.
        SwingUtilities.invokeLater( new Runnable( )
        {
            @Override
            public void run( )
            {
                //Turn off metal's use of bold fonts
                UIManager.put( "swing.boldMetal", Boolean.FALSE );
                createAndShowGUI( );
            }
        });
    }

    protected JTextArea textArea;
    protected String newline = "\n";
    
    JButton     m_refreshToolbarComboboxesButton;
    JButton     m_applyWaterRemovalTDFDFitButton;
    JComboBox   m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox;
    
    JComboBox   m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox;
    
    JButton m_applyInteractiveHLSVDButton;
    JButton m_applyHLSVDButton;
    JButton m_applyApodizationButton;
    JButton m_multiplyFirstTimePointButton;
    JButton m_offsetCorrectionTDButton;
    JButton m_offsetCorrectionFDButton;
    JButton m_applyZeroFillingButton;
    JButton m_applyManualPhasingButton;
    JButton m_applyAutomaticPhasingButton;
    JButton m_applyFrequencyShiftButton;
    JButton m_manualBaseLineRemovalButton;
    JButton m_defineManualBaseLineRemovalSettingsButton;
    JButton m_applyComplexConjugationButton;
    
    JButton m_quantifySpectraWithTDFDFitButton;
    JComboBox m_quantifySpectraWithTDFDFitModelNameSelectionComboBox;
    
    JButton m_lauchTDFDFitMetaboliteModelEditorButton;
    JButton m_createNewModelButton;
    JButton m_editTDFDFitDefaultsButton;
    
    
    static final public String APPLY_REFRESH_TOOLBAR_COMBOBOXES         = "Refresh both default model and water model selectioon toolbar comboboxes (apply after adding new models!).";
    static final public String APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL  = "Apply non-interactive TDFDFit water removal.";
    static final public String APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL    = "Settings HLSVD water removal parameters.";
    static final public String APPLY_HLSVD_PRO_WATER_REMOVAL            = "Apply non-interactive HLSVDPro water removal using default values.";
    static final public String APPLY_APODIZATION                        = "Apply interactive apodization.";
    static final public String APPLY_FIRST_TD_POINT_MULTIPLICATION      = "Apply interactive 1st TD-point scaling.";
    static final public String APPLY_OFFSET_CORRECTION_FD               = "Apply intractive offset correction in FD.";
    static final public String APPLY_OFFSET_CORRECTION_TD               = "Apply intractive offset correction in TD.";
    static final public String APPLY_ZERO_FILLING                       = "Apply intractive zero filling.";
    static final public String APPLY_MANUAL_PHASING                     = "Apply intractive spectrum phasing.";
    static final public String APPLY_AUTOMATIC_PHASING                  = "Apply automatic spectrum phasing.";
    static final public String APPLY_INTERACTIVE_FREQ_SHIFT             = "Apply intractive frequency shift.";
    static final public String APPLY_NON_INTERACTIVE_BASELINE_REMOVAL   = "Apply non-intractive baseline removal.";
    static final public String DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS  = "Define manual baseline removal settings.";
    static final public String APPLY_COMPLEX_CONJUGATION                = "Apply complex conjugation (flip spectra).";
    
    static final public String APPLY_TDFDFIT_QUANTIFICATION            = "Apply TDFDFit quantification on selected spectra.";
    static final public String EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL  = "Edit TDFDFit selected metabolite model of combobox.";
    static final public String CREATE_AND_ADD_NEW_MODEL                = "Create and edit a new TDFDFit model";
    static final public String EDIT_TDFDFIT_DEFAULTS                   = "Edit TDFDFit default settings";
    
    static final public int APPLY_REFRESH_TOOLBAR_COMBOBOXES_ID         = 0;
    static final public int APPLY_INTERACTIVE_TDFDFIT_WATER_REMOVAL_ID  = 1;
    static final public int APPLY_INTERACTIVE_HLSVD_WATER_REMOVAL_ID    = 2;
    static final public int APPLY_HLSVD_PRO_WATER_REMOVAL_ID            = 3;
    static final public int APPLY_APODIZATION_ID                        = 4;
    static final public int APPLY_FIRST_TD_POINT_MULTIPLICATION_ID      = 5;
    static final public int APPLY_OFFSET_CORRECTION_FD_ID               = 6;
    static final public int APPLY_OFFSET_CORRECTION_TD_ID               = 7;
    static final public int APPLY_ZERO_FILLING_ID                       = 8;
    static final public int APPLY_MANUAL_PHASING_ID                     = 9;
    static final public int APPLY_AUTOMATIC_PHASING_ID                  = 10;
    static final public int APPLY_INTERACTIVE_FREQ_SHIFT_ID             = 11;
    static final public int APPLY_MANUAL_BASELINE_REMOVAL_ID            = 12;
    static final public int DEFINE_MANUAL_BASELINE_REMOVAL_SETTINGS_ID  = 13;
    static final public int APPLY_TDFDFIT_QUANTIFICATION_ID             = 14;
    static final public int EDIT_SELECTED_TDFDFIT_METABOLITE_MODEL_ID   = 15;
    static final public int CREATE_AND_ADD_NEW_MODEL_ID                 = 16;
    static final public int EDIT_TDFDFIT_DEFAULTS_ID                    = 17;
    static final public int APPLY_COMPLEX_CONJUGATION_ID                = 18;
    
    LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    LinkedList<File> m_allDetectedTFMFiles      = new LinkedList<File>( );
    LinkedList<File> m_allDetectedBaselineFiles = new LinkedList<File>( );
    
    @Override
    public void itemStateChanged( ItemEvent e ) 
    {
        
        if ( e.getSource( ) == m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox )
        {
            int waterModelIndex             = m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.getSelectedIndex( );
            String selWatereModelIndexName  = (String)m_applyWaterRemovalTDFDFitWaterModelNameSelectionComboBox.getSelectedItem ( );
            
            
            if ( selWatereModelIndexName.compareTo( "No model defined yet!" ) != 0 )
            {
                File fullPath = m_allDetectedTFMFiles.get( waterModelIndex - 1 );
            
                TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
                defs.load( );
                
                defs.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_NAME, selWatereModelIndexName );
                defs.setProperty( TDFDFittingDefaults.TDFDFIT_WATER_MODEL_PATH, fullPath.getParent( ) );
                
                defs.save( );
            }
        }
        
        if ( e.getSource( ) == m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox )
        {
            int selBaselineModelIndex = m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.getSelectedIndex( );
            String currentBaselineModel = (String)m_defaultSplineBasedBaselineRemovalModelNameSelectionComboBox.getSelectedItem ( );
            
            if ( currentBaselineModel.compareTo( "No model defined yet!" ) != 0 )
            {
                File fullPath = m_allDetectedBaselineFiles.get( selBaselineModelIndex - 1 );
                TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
                defs.load( );
                
                defs.setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME, currentBaselineModel );
                defs.setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH, fullPath.getParent( ) );
                
                defs.save( );
            }
        }   
        
        if ( e.getSource( ) == m_quantifySpectraWithTDFDFitModelNameSelectionComboBox )
        {
            int selMetaboliteModelIndex = m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.getSelectedIndex( );
            String currentMetaboliteModel = (String)m_quantifySpectraWithTDFDFitModelNameSelectionComboBox.getSelectedItem ( );
            // File fullPath = m_allDetectedTFMFiles.get( selMetaboliteModelIndex - 1 );
            if ( currentMetaboliteModel.compareTo( "No model defined yet!" ) != 0 )
            {
                File fullPath = m_allDetectedTFMFiles.get( selMetaboliteModelIndex - 1 );
                TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
                defs.load( );
                
                defs.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, currentMetaboliteModel );
                defs.setProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH, fullPath.getParent( ) );
                
                defs.save( );
            }
        } 
            
    }
}