/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.toolbars;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * Toolbar to enable quick and user friendly preprocessing steps.
 * @author Johannes Slotboom 
 */
public final class TDFDFitSignalPreprocessingToolbar extends JPanel implements ActionListener
{
    public TDFDFitSignalPreprocessingToolbar ( )
    {
        super( new BorderLayout( ) );

        // Create the toolbar.
        JToolBar toolBar = new JToolBar( "TDFDFit model toolbar " );
        addButtons( toolBar );

        add( toolBar, BorderLayout.PAGE_START );
        
        setVisible( true );
    }

    /**
     * Add the buttons to the toolbar.
     * @param toolBar
     */
    protected void addButtons( JToolBar toolBar )
    {
        
        // first button
        m_applySVDButton = makeBasicFileToolBarButton( "ApplySVD24", APPLY_SVD_WATER_REMOVAL,
                                                       APPLY_SVD_WATER_REMOVAL,
                                                       "ApplySVD" );
        toolBar.add( m_applySVDButton );
        m_applySVDButton.setEnabled( false );
        // second button
        m_applyHLSVDButton = makeBasicFileToolBarButton( "ApplyHLSVD24", APPLY_HLSVD_WATER_REMOVAL,
                                                         APPLY_HLSVD_WATER_REMOVAL,
                                                         "ApplyHLSVD" );
        toolBar.add( m_applyHLSVDButton );
        
        // third button
        m_applyApodizationButton = makeBasicFileToolBarButton( "ApplyApodization24", APPLY_APODIZATION,
                                                               APPLY_APODIZATION,
                                                               "ApplyApodization" );
        toolBar.add( m_applyApodizationButton );

        // fourth button
        m_multiplyFirstTimePointButton = makeBasicFileToolBarButton( "FirstTDPointMult24", APPLY_FIRST_TD_POINT_MULTIPLICATION,
                                                                     APPLY_FIRST_TD_POINT_MULTIPLICATION,
                                                                     "FirstTDPointMult" );
        toolBar.add( m_multiplyFirstTimePointButton );

        // fifth button
        m_offsetCorrectionTDButton = makeBasicFileToolBarButton( "OffsetCorrectionFD24", APPLY_OFFSET_CORRECTION_FD,
                                                                 APPLY_OFFSET_CORRECTION_FD,
                                                                 "OffsetCorrectionFD" );
        toolBar.add( m_offsetCorrectionTDButton );
        
        // sixth button
        m_applyZeroFillingButton = makeBasicFileToolBarButton( "ApplyZeroFilling24", APPLY_ZERO_FILLING,
                                                               APPLY_ZERO_FILLING,
                                                               "ApplyZeroFilling" );
        
        toolBar.add( m_applyZeroFillingButton );

        // seventh button
        m_applyManualPhasingButton = makeBasicFileToolBarButton( "ManualPhasing24", APPLY_MANUAL_PHASING,
                                                                 APPLY_MANUAL_PHASING,
                                                                 "ManualPhasing" );
        toolBar.add( m_applyManualPhasingButton );
        
        // eigths button
        m_applyFrequencyShiftButton = makeBasicFileToolBarButton( "ApplyManualFrequencyShift24", APPLY_MANUAL_FREQ_SHIFT,
                                                                  APPLY_MANUAL_FREQ_SHIFT,
                                                                  "ApplyManualFreqShift" );
        toolBar.add( m_applyFrequencyShiftButton );
        
        // nineth button
        m_manualBaseLineRemovalButton = makeBasicFileToolBarButton("ApplyManualBaselineRemoval24", 
                                                                    APPLY_NON_INTERACTIVE_BASELINE_REMOVAL,
                                                                    APPLY_NON_INTERACTIVE_BASELINE_REMOVAL,
                                                                    "ApplyManualBaselineRemoval" );
        toolBar.add( m_manualBaseLineRemovalButton );
        m_manualBaseLineRemovalButton.setEnabled( true );
                
        // Define manual baseline spline node definition
        m_defineManualBaseLineRemovalSettingsButton = makeBasicFileToolBarButton( "DefineManualBaselineRemovalSettings24", 
                                                                                  DEFINE_BASELINE_NODES_SETTINGS,
                                                                                  DEFINE_BASELINE_NODES_SETTINGS,
                                                                                  "DefineBaselineNodes" );
        toolBar.add( m_defineManualBaseLineRemovalSettingsButton );
        
        m_defineManualBaseLineRemovalSettingsButton.setEnabled( true );
        
//        // tenth button
//        m_launchQuantitativeViewerButton = makeBasicFileToolBarButton( "QuantitativeEvaluationSVS24", QUANTITATIVE_VIEWER,
//                                                                 QUANTITATIVE_VIEWER,
//                                                                 "Launch-quant-viewer" );
//        toolBar.add( m_launchQuantitativeViewerButton );
//        m_launchQuantitativeViewerButton.setEnabled(false);
//        
//        // eleventh button
//        m_setDefaultParameterRangesButton = makeBasicFileToolBarButton( "RangeDefine24", SET_DEFAULT_PARAMETER_RANGES,
//                                                                        SET_DEFAULT_PARAMETER_RANGES,
//                                                                        "Launch-param-range-definer" );
//        toolBar.add( m_setDefaultParameterRangesButton );
        
    }


    /**
     *
     * Creates all buttons of the Toolbar
     *
     * @param imageName
     * @param actionCommand
     * @param toolTipText
     * @param altText
     * @return the button to add
     */
    protected JButton makeBasicFileToolBarButton( String imageName,
                                                  String actionCommand,
                                                  String toolTipText,
                                                  String altText )
    {
        //Look for the image.
        String workingDir = System.getProperty( "user.dir" );
        String imgLocation = workingDir + System.getProperty( "file.separator" )
                                        + "images"
                                        + System.getProperty( "file.separator" )
                                        + imageName
                                        + ".gif";
        File imageFile = new File( imgLocation );

        //Create and initialize the button.
        JButton button = new JButton( );
        button.setActionCommand( actionCommand );
        button.setToolTipText( toolTipText );
        button.addActionListener( this );

        if ( imageFile.exists( ) )
        {                      //image found
            button.setIcon( new ImageIcon( imgLocation ) );
        } 
        else
        {                                     //no image found
            button.setText( altText );
            System.err.println( "Resource not found: "
                               + imgLocation );
        }

        return button;
    }
    
    /**
     *
     * Is called whenever a registered component raises an ActionEvent.
     *
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        String cmd = e.getActionCommand( );
        String description = null;

        // Handle each button.
        if ( APPLY_SVD_WATER_REMOVAL.equals( cmd ) )
        {   //first button clicked
            notifyActionListeners( APPLY_SVD_WATER_REMOVAL, APPLY_SVD_WATER_REMOVAL_ID );
        }
        else if ( APPLY_HLSVD_WATER_REMOVAL.equals( cmd ) )
        {   //second button clicked
            notifyActionListeners( APPLY_HLSVD_WATER_REMOVAL, APPLY_HLSVD_WATER_REMOVAL_ID );
        } 
        else if ( APPLY_APODIZATION.equals( cmd ) )
        {   // third button clicked
            notifyActionListeners( APPLY_APODIZATION, APPLY_APODIZATION_ID );
        }
        else if ( APPLY_FIRST_TD_POINT_MULTIPLICATION.equals( cmd ) )
        {   // fourth button clicked
            notifyActionListeners( APPLY_FIRST_TD_POINT_MULTIPLICATION, APPLY_FIRST_TD_POINT_MULTIPLICATION_ID );
        }
        else if ( APPLY_OFFSET_CORRECTION_FD.equals( cmd ) )
        {   //first button clicked
            notifyActionListeners( APPLY_OFFSET_CORRECTION_FD, APPLY_OFFSET_CORRECTION_FD_ID );
        } 
        else if ( APPLY_ZERO_FILLING.equals( cmd ) )
        {   // fifth button clicked
            notifyActionListeners( APPLY_ZERO_FILLING, APPLY_ZERO_FILLING_ID );
        }
        else if ( APPLY_MANUAL_PHASING.equals( cmd ) )
        {   // sixth button clicked
            notifyActionListeners( APPLY_MANUAL_PHASING, APPLY_MANUAL_PHASING_ID );
        }
        else if ( APPLY_MANUAL_FREQ_SHIFT.equals( cmd ) )
        {   //seventh button clicked
            notifyActionListeners( APPLY_MANUAL_FREQ_SHIFT, APPLY_MANUAL_FREQ_SHIFT_ID );
        } 
        else if ( APPLY_NON_INTERACTIVE_BASELINE_REMOVAL.equals( cmd ) )
        {   // eights button clicked
            notifyActionListeners(APPLY_NON_INTERACTIVE_BASELINE_REMOVAL, APPLY_NON_INTERACTIVE_BASELINE_REMOVAL_ID );
        }
        else if ( DEFINE_BASELINE_NODES_SETTINGS.equals( cmd ) )
        {   // eights button clicked
            notifyActionListeners( DEFINE_BASELINE_NODES_SETTINGS, DEFINE_BASELINE_NODES_SETTINGS_ID );
        }
    }

     /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( String actionCommand, int actionID )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionID, actionCommand ) );
        }
    }

    /**
     * Adds an Actionlistener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    private static void createAndShowGUI( )
    {
        //Create and set up the window.
        JFrame frame = new JFrame( "BasicFileMenuToolbar" );
        frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

        //Add content to the window.
        frame.add(new TDFDFitFileMenuToolbar( ) );

        //Display the window.
        frame.pack( );
        frame.setVisible( true );
    }
    
    public void m_applySVDButtonEnabled( boolean enable )
    {
        m_applySVDButton.setEnabled( enable );
    }
    
    public void m_applyHLSVDButton( boolean enable )
    {
        m_applyHLSVDButton.setEnabled( enable );
    }        
            
    public void m_applyApodizationButtonEnabled( boolean enable )
    {
        m_applyApodizationButton.setEnabled( enable );
    }
    
    public void m_multiplyFirstTimePointButtonEnabled( boolean enable )
    {
        m_multiplyFirstTimePointButton.setEnabled( enable );
    }
    
    public void m_offsetCorrectionTDButtonEnabled( boolean enable )
    {
        m_offsetCorrectionTDButton.setEnabled( enable );
    }
    
    public void m_applyZeroFillingButtonEnabled( boolean enable )
    {
        m_applyZeroFillingButton.setEnabled( enable );
    }
    
    public void m_applyManualPhasingButtonEnabled( boolean enable )
    {
        m_applyManualPhasingButton.setEnabled( enable );
    }
    
    public void m_applyFrequencyShiftButtonEnabled( boolean enable )
    {
        m_applyFrequencyShiftButton.setEnabled( enable );
    }
    
    public void m_manualBaseLineRemovalButtonEnabled( boolean enable )
    {
        m_manualBaseLineRemovalButton.setEnabled( enable );
    }
    
    
    public static void main( String[] args )
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater( new Runnable( )
        {
            @Override
            public void run( )
            {
                //Turn off metal's use of bold fonts
                UIManager.put( "swing.boldMetal", Boolean.FALSE );
                createAndShowGUI( );
            }
        });
    }

    protected JTextArea textArea;
    protected String newline = "\n";
    
    JButton m_applySVDButton;
    JButton m_applyHLSVDButton;
    JButton m_applyApodizationButton;
    JButton m_multiplyFirstTimePointButton;
    JButton m_offsetCorrectionTDButton;
    JButton m_applyZeroFillingButton;
    JButton m_applyManualPhasingButton;
    JButton m_applyFrequencyShiftButton;
    JButton m_manualBaseLineRemovalButton;
    JButton m_defineManualBaseLineRemovalSettingsButton;
    
//  JButton m_launchQuantitativeViewerButton;
//  JButton m_setDefaultParameterRangesButton;
    
    static final public String APPLY_SVD_WATER_REMOVAL      = "Apply inter-active SVD water removal";
    static final public String APPLY_HLSVD_WATER_REMOVAL    = "Apply inter-active HLSVD water removal";
    static final public String APPLY_APODIZATION            = "Apply inter-active apodization";
    static final public String APPLY_FIRST_TD_POINT_MULTIPLICATION      = "Apply interactive 1st TD-point scaling";
    static final public String APPLY_OFFSET_CORRECTION_FD   = "Apply inter-active offset correction in FD";
    static final public String APPLY_ZERO_FILLING           = "Apply inter-active zero filling";
    static final public String APPLY_MANUAL_PHASING         = "Apply inter-active spectrum phasing";
    static final public String APPLY_MANUAL_FREQ_SHIFT      = "Apply inter-active frequency";
    static final public String APPLY_NON_INTERACTIVE_BASELINE_REMOVAL   = "Apply non inter-active baseline removal";
    static final public String DEFINE_BASELINE_NODES_SETTINGS  = "Define baseline spline nodes";
    
    static final public int APPLY_SVD_WATER_REMOVAL_ID                  = 0;
    static final public int APPLY_HLSVD_WATER_REMOVAL_ID                = 1;
    static final public int APPLY_APODIZATION_ID                        = 2;
    static final public int APPLY_FIRST_TD_POINT_MULTIPLICATION_ID      = 3;
    static final public int APPLY_OFFSET_CORRECTION_FD_ID               = 4;
    static final public int APPLY_ZERO_FILLING_ID                       = 5;
    static final public int APPLY_MANUAL_PHASING_ID                     = 6;
    static final public int APPLY_MANUAL_FREQ_SHIFT_ID                  = 7;
    static final public int APPLY_NON_INTERACTIVE_BASELINE_REMOVAL_ID   = 8;
    static final public int DEFINE_BASELINE_NODES_SETTINGS_ID           = 9;
    
    LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
}