/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import java.awt.image.IndexColorModel;

/**
 *
 * @author user
 */
public class ColorModelGenerator {

    public static final int COLMODE_GRAY = 0;
    public static final int COLMODE_RGB = 1;
    public static final int COLMODE_BGR = 2;
    public static final int COLMODE_RBG = 3;
    public static final int COLMODE_BRG = 4;
    public static final int COLMODE_GBR = 5;
    public static final int COLMODE_GRB = 6;

    public static IndexColorModel createColorModel(int window, float level, int maxval, int colormode) {
        if (colormode == ColorModelGenerator.COLMODE_GRAY) {
            return ColorModelGenerator.grayColorModel(window, level, maxval);
        }
        return ColorModelGenerator.coloredColorModel(window, level, maxval, colormode);
    }

    private static IndexColorModel grayColorModel(int window, float level, int maxval) {
        int length = window;
        if (maxval > window) {
            length = maxval;
        }
        length++;
        byte[] r = new byte[length];
        byte[] g = new byte[length];
        byte[] b = new byte[length];

        for (int i = 0; i < length; i++) {
            int val = Math.round((255 / (float) window) * ((float) i - level + (float) window * 0.5f));
            if (val > 255) {
                val = 255;
            }
            if (val < 0) {
                val = 0;
            }
            r[ i] = (byte) val;
            g[ i] = (byte) val;
            b[ i] = (byte) val;
        }
        return (new IndexColorModel(16, length, r, g, b));
    }

    private static IndexColorModel coloredColorModel(int window, float level, int maxval, int colormode) {
        int length = window;
        if (maxval > window) {
            length = maxval;
        }
        length++;
        byte[] c1 = new byte[length];
        byte[] c2 = new byte[length];
        byte[] c3 = new byte[length];
        float val2 = 0;

        for (int i = 0; i < length; i++) {
            float val = (255 / (float) window) * ((float) i - level + (float) window * 0.5f);

            if (val < 0) {
                c1[i] = (byte) 255;

            } else if (val >= 0 && val < 255f / 4f) {
                c1[i] = (byte) 255;
                val2 = Math.max(0, Math.min(255, Math.round(val * 4)));
                c2[i] = (byte) val2;

            } else if (val >= 255f / 4f && val < 255f / 2f) {
                c2[i] = (byte) 255;
                val2 = Math.max(0, Math.min(255, Math.round(255 - (val - 255 / 4) * 4)));
                c1[i] = (byte) val2;

            } else if (val >= 255f / 2f && val < 255f * 0.75f) {
                c2[i] = (byte) 255;
                val2 = Math.max(0, Math.min(255, Math.round((val - 255 / 2) * 4)));
                c3[i] = (byte) val2;

            } else if (val >= 255f * 0.75f && val < 255) {
                c3[i] = (byte) 255;
                val2 = Math.max(0, Math.min(255, Math.round(255 - (val - 255 * 0.75) * 4)));
                c2[i] = (byte) val2;
            } else if (val >= 255) {
                c3[i] = (byte) 255;
            }
        }

        switch (colormode) {
            case COLMODE_RGB:
                return (new IndexColorModel(16, length, c1, c2, c3));
            case COLMODE_BGR:
                return (new IndexColorModel(16, length, c3, c2, c1));
            case COLMODE_RBG:
                return (new IndexColorModel(16, length, c1, c3, c2));
            case COLMODE_BRG:
                return (new IndexColorModel(16, length, c2, c3, c1));
            case COLMODE_GBR:
                return (new IndexColorModel(16, length, c3, c1, c2));
            case COLMODE_GRB:
                return (new IndexColorModel(16, length, c2, c1, c3));
            default:
                return (new IndexColorModel(16, length, c1, c2, c3));
        }

    }
}
