/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import org.spectrim.model.GlobalModel;
import org.spectrim.geometricobjects.ImagePlane;
import org.spectrim.geometricobjects.IntersectionLine;
import org.spectrim.imagegraphicobjects.Point3D;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;
import java.util.ArrayList;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;

/**
 *
 * @author user
 */
public class LocalizerPanel extends JFractionPanel implements ComponentListener, MouseMotionListener, MouseListener {

    private GlobalModel model;
    private int window;
    private int level;
    private int[] pressedCoord = new int[2];
    private ImageSlice imageInfo;
    private double[] intersectionLine;
    private ArrayList<double[]> gridProjection;
    private ArrayList<double[]> selectedBlueVoxelsProjection;
    private ArrayList<double[]> selectedRedVoxelsProjection;
    private ArrayList<double[]> excitationVoxelProjection;
    private Image image;
    private Color windowColor = Color.gray;
    private ArrayList<ImagePlane> referenceImages;
    private ArrayList<Color> referenceColors;
    private ArrayList<IntersectionLine> referenceLines;
    private int colmode = 0;
    private IndexColorModel colModel;
    private double colRowRatio = 1;

    public LocalizerPanel() {
        addComponentListener(this);
        addMouseMotionListener(this);
        addMouseListener(this);
        this.window = 255;
        this.level = 128;
        colModel = ColorModelGenerator.createColorModel(255, 128, 256, colmode);
    }

    public void setColorMode(int mode) {
        colmode = mode;
        autoWindowing();
    }

    public void setModel(GlobalModel model) {
        this.model = model;
    }

    public void setColor(Color c) {
        if (c != null) {
            windowColor = c;
        }
    }

    public Color getColor() {
        return windowColor;
    }

    public void setImageInfo(ImageSlice imageInfo) {
        this.imageInfo = imageInfo;
        this.setVisible(true);
        referenceImages = new ArrayList<>();
        referenceColors = new ArrayList<>();
        referenceLines = new ArrayList<>();
        colRowRatio = (double) imageInfo.sizeX / (double) imageInfo.sizeY;
        autoWindowing();
    }

    public ImageSlice getImageInfo() {
        return imageInfo;
    }

    public void autoWindowing() {
        if (imageInfo != null) {
            window = (int) (imageInfo.getMaxVal()[0] - imageInfo.getMinVal()[0] + 1);
            level = (int) (imageInfo.getMinVal()[0] + (imageInfo.getMaxVal()[0] - imageInfo.getMinVal()[0]) / 2);
            if (window < 1) {
                window = 1;
            }
            if (level < 1) {
                level = 1;
            }
            colModel = ColorModelGenerator.createColorModel(window, level, (int) imageInfo.getMaxVal()[0], colmode);
            setView();
            repaint();
        }
    }

    public void clear() {
        this.imageInfo = null;
        colRowRatio = 1;
        referenceImages.clear();
        referenceColors.clear();
        referenceLines.clear();
        this.intersectionLine = null;
        this.selectedBlueVoxelsProjection.clear();
        this.selectedRedVoxelsProjection.clear();
        this.excitationVoxelProjection.clear();
        this.gridProjection.clear();
        this.setVisible(false);
    }

    public void clearGridProjections() {
        this.selectedBlueVoxelsProjection.clear();
        this.selectedRedVoxelsProjection.clear();
        this.excitationVoxelProjection.clear();
        this.gridProjection.clear();
    }

    public void computeProjections() {
        gridProjection = model.getGridLinerProjection(this.imageInfo.imagePosition,
                this.imageInfo.imageOrientationX,
                this.imageInfo.imageOrientationY,
                this.imageInfo.sizeX,
                this.imageInfo.sizeY,
                this.imageInfo.pixelSpacingX,
                this.imageInfo.pixelSpacingY);
        excitationVoxelProjection = model.getExcitationVoxelProjection(this.imageInfo.imagePosition,
                this.imageInfo.imageOrientationX,
                this.imageInfo.imageOrientationY,
                this.imageInfo.sizeX,
                this.imageInfo.sizeY,
                this.imageInfo.pixelSpacingX,
                this.imageInfo.pixelSpacingY);
        selectedBlueVoxelsProjection = model.getSelectedVoxelProjection(this.imageInfo.imagePosition,
                this.imageInfo.imageOrientationX,
                this.imageInfo.imageOrientationY,
                this.imageInfo.sizeX,
                this.imageInfo.sizeY,
                this.imageInfo.pixelSpacingX,
                this.imageInfo.pixelSpacingY,
                GlobalModel.SelectionGroup.BLUE);
        selectedRedVoxelsProjection = model.getSelectedVoxelProjection(this.imageInfo.imagePosition,
                this.imageInfo.imageOrientationX,
                this.imageInfo.imageOrientationY,
                this.imageInfo.sizeX,
                this.imageInfo.sizeY,
                this.imageInfo.pixelSpacingX,
                this.imageInfo.pixelSpacingY,
                GlobalModel.SelectionGroup.RED);
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (this.imageInfo != null) {
            Graphics2D g2D = (Graphics2D) g;
            AffineTransform transfOrig = (AffineTransform) g2D.getTransform().clone();
            AffineTransform transf = g2D.getTransform();
            transf.translate(((double) this.getWidth() / 2 * (1 - colRowRatio)), 0);
            transf.scale(colRowRatio, 1);
            g2D.setTransform(transf);
            g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g2D.setRenderingHint(RenderingHints.KEY_RENDERING,
                    RenderingHints.VALUE_RENDER_SPEED);
            g2D.drawImage(image, 0, 0, this);


            if (model != null) {

                if (model.getGridSize() > 1) {
                    drawProjection(g2D, new Color(0f, 0.45f, 0f, 1f), this.gridProjection);
                    drawProjection(g2D, new Color(0.06f, 0.34f, 1f, 1f), this.selectedBlueVoxelsProjection);
                    drawProjection(g2D, new Color(1f, 0.34f, 0.06f, 1f), this.selectedRedVoxelsProjection);
                }
                drawProjection(g2D, Color.red, this.excitationVoxelProjection);
            }
            g2D.setStroke(new BasicStroke(3));
            if (intersectionLine != null) {
                g2D.setColor(Color.blue);
                g2D.drawLine((int) Math.round(intersectionLine[0]), (int) Math.round(intersectionLine[1]), (int) Math.round(intersectionLine[2]), (int) Math.round(intersectionLine[3]));
            }
            if (referenceLines.size() > 0) {
                for (int i = 0; i < referenceLines.size(); i++) { 
                    referenceLines.get(i).draw(g2D);
                }
            }
            g2D.setTransform(transfOrig);
        }
    }

    public void setView() {
        if (this.imageInfo != null && model != null) {
            if (imageInfo.image != null && this.getWidth() > 0 && this.getHeight() > 0) {
                ImageSlice mainImage = model.getCurrentImage();
                intersectionLine = getIntersection(mainImage.imagePosition, mainImage.imageOrientationX, mainImage.imageOrientationY);
                BufferedImage buffered = new BufferedImage(colModel, (WritableRaster) imageInfo.image, colModel.isAlphaPremultiplied(), null);
                Image img = buffered.getScaledInstance(this.getWidth(), this.getHeight(), BufferedImage.SCALE_FAST);
                image = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_ARGB);
                image.getGraphics().drawImage(img, 0, 0, null);
//                image = this.imageInfo.Image.getScaledInstance(this.getWidth(), this.getHeight(), Image.SCALE_FAST);
            }
            if (model != null && model.getGridSize() > 1) {
                selectedBlueVoxelsProjection = model.getSelectedVoxelProjection(this.imageInfo.imagePosition,
                        this.imageInfo.imageOrientationX,
                        this.imageInfo.imageOrientationY,
                        this.imageInfo.sizeX,
                        this.imageInfo.sizeY,
                        this.imageInfo.pixelSpacingX,
                        this.imageInfo.pixelSpacingY,GlobalModel.SelectionGroup.BLUE);
                selectedRedVoxelsProjection = model.getSelectedVoxelProjection(this.imageInfo.imagePosition,
                        this.imageInfo.imageOrientationX,
                        this.imageInfo.imageOrientationY,
                        this.imageInfo.sizeX,
                        this.imageInfo.sizeY,
                        this.imageInfo.pixelSpacingX,
                        this.imageInfo.pixelSpacingY,GlobalModel.SelectionGroup.RED);
            }
            if (referenceImages.size() > 0) {
                calcReferenceIntersections();
            }
        }
    }

    public void drawProjection(Graphics2D g, Color c, ArrayList<double[]> lines) {
        double[] aux;
        for (int i = 0; i < lines.size(); i++) {
            aux = lines.get(i);
            g.setColor(c);
            g.drawLine((int) Math.round(aux[0] * this.getWidth()), (int) Math.round(aux[1] * this.getHeight()), (int) Math.round(aux[2] * this.getWidth()), (int) Math.round(aux[3] * this.getHeight()));
        }
    }

    public double[] getIntersection(Point3D sliceP, Point3D sliceV1, Point3D sliceV2) {
        ArrayList<Point3D> intersections = new ArrayList<>();
        Point3D p1 = this.imageInfo.imagePosition;
        Point3D p2 = this.imageInfo.imageOrientationX.scalarMultiplication(imageInfo.sizeX * imageInfo.pixelSpacingX).translate(p1);
        Point3D p3 = this.imageInfo.imageOrientationY.scalarMultiplication(imageInfo.sizeY * imageInfo.pixelSpacingY).translate(p1);
        Point3D p4 = p3.Clone().translate(p2.subtractPoint(p1));
        PlaneLineIntersection(p1, p2, intersections, sliceP, sliceV1, sliceV2);
        PlaneLineIntersection(p2, p4, intersections, sliceP, sliceV1, sliceV2);
        PlaneLineIntersection(p4, p3, intersections, sliceP, sliceV1, sliceV2);
        PlaneLineIntersection(p3, p1, intersections, sliceP, sliceV1, sliceV2);

        double[] point2Daux1;
        double[] point2Daux2;
        double[] res = new double[4];
        if (intersections.size() == 2) {
            point2Daux1 = coordTransform(intersections.get(0), this.imageInfo.imagePosition, this.imageInfo.imageOrientationX, this.imageInfo.imageOrientationY);
            point2Daux2 = coordTransform(intersections.get(1), this.imageInfo.imagePosition, this.imageInfo.imageOrientationX, this.imageInfo.imageOrientationY);
            res[0] = point2Daux1[0];
            res[1] = point2Daux1[1];
            res[2] = point2Daux2[0];
            res[3] = point2Daux2[1];
        }
        return res;
    }

    //Intersection of segment AB with the Plane defined in the class constructor 
    public void PlaneLineIntersection(Point3D pA, Point3D pB, ArrayList<Point3D> intersections, Point3D p, Point3D r1, Point3D r2) {
        Point3D res = new Point3D();
        double[][] matrixData = {
            {r1.m_x1, r2.m_x1, pA.m_x1 - pB.m_x1},
            {r1.m_x2, r2.m_x2, pA.m_x2 - pB.m_x2},
            {r1.m_x3, r2.m_x3, pA.m_x3 - pB.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{pA.m_x1 - p.m_x1, pA.m_x2 - p.m_x2, pA.m_x3 - p.m_x3}, false);

        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new LUDecomposition(n).getSolver();

        try {
            RealVector solution = solver.solve(constants);
            double[] sol;
            sol = solution.toArray();
            if (sol[2] >= 0d && sol[2] <= 1d) {
                res.m_x1 = pA.m_x1 + sol[2] * (pB.m_x1 - pA.m_x1);
                res.m_x2 = pA.m_x2 + sol[2] * (pB.m_x2 - pA.m_x2);
                res.m_x3 = pA.m_x3 + sol[2] * (pB.m_x3 - pA.m_x3);
                intersections.add(res);
            }
        } catch (Exception ex) {
        }
    }

    public double[] coordTransform(Point3D c1, Point3D p, Point3D r1, Point3D r2) {
        Point3D point = new Point3D();
        point.m_x1 = (c1.m_x1 - p.m_x1);
        point.m_x2 = (c1.m_x2 - p.m_x2);
        point.m_x3 = (c1.m_x3 - p.m_x3);
        double[][] matrixData = {
            {r1.m_x1, r2.m_x1},
            {r1.m_x2, r2.m_x2},
            {r1.m_x3, r2.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{point.m_x1, point.m_x2, point.m_x3}, false);
        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new QRDecomposition(n).getSolver();
        RealVector solution = solver.solve(constants);
        double[] res = new double[2];
        res[0] = solution.getEntry(0) * this.getWidth() / (this.imageInfo.sizeX * this.imageInfo.pixelSpacingX);
        res[1] = solution.getEntry(1) * this.getHeight() / (this.imageInfo.sizeY * this.imageInfo.pixelSpacingY);
        return res;
    }

    public void addReferencePlane(Color c, ImagePlane plane) {
        this.referenceImages.add(plane);
        this.referenceColors.add(c);
    }

    public void calcReferenceIntersections() {
        this.referenceLines.clear();
        double[] aux;
        ImagePlane plane;
        IntersectionLine line;
        for (int i = 0; i < referenceImages.size(); i++) {
            plane = referenceImages.get(i);
            aux = getIntersection(plane.p, plane.v1, plane.v2);
            line = new IntersectionLine(referenceColors.get(i), (int) Math.round(aux[0]), (int) Math.round(aux[1]), (int) Math.round(aux[2]), (int) Math.round(aux[3]));
            this.referenceLines.add(line);
        }
    }

    public ImagePlane getImagePlane() {
        ImagePlane res = new ImagePlane(this.imageInfo.imagePosition, this.imageInfo.imageOrientationX, this.imageInfo.imageOrientationY);
        return res;
    }

    @Override
    public void componentResized(ComponentEvent ce) {
        setView();
        repaint();
    }

    @Override
    public void componentMoved(ComponentEvent ce) {
    }

    @Override
    public void componentShown(ComponentEvent ce) {
    }

    @Override
    public void componentHidden(ComponentEvent ce) {
    }

    @Override
    public void mouseDragged(MouseEvent me) {
        if (imageInfo != null && me.isAltDown()) { //Alt and Middle mouse button are equivalent
            this.window = this.window + (me.getX() - this.pressedCoord[0]);
            this.level = this.level + (me.getY() - this.pressedCoord[1]);
            if (window < 1) {
                window = 1;
            }
            if (level < 1) {
                level = 1;
            }
            colModel = ColorModelGenerator.createColorModel(window, level, (int) imageInfo.getMaxVal()[0], colmode);
            this.pressedCoord = new int[]{me.getX(), me.getY()};
        }
        setView();
        repaint();        
    }

    @Override
    public void mouseMoved(MouseEvent me) {
    }

    @Override
    public void mouseClicked(MouseEvent me) {
    }

    @Override
    public void mousePressed(MouseEvent me) {
        this.pressedCoord = new int[]{me.getX(), me.getY()};
    }

    @Override
    public void mouseReleased(MouseEvent me) {
    }

    @Override
    public void mouseEntered(MouseEvent me) {
    }

    @Override
    public void mouseExited(MouseEvent me) {
    }
}
