/*
 * File: $Id: Curve.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 02.04.2003, 11:47   Martin Buerki & Hans Slotboom  
 * 
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.xyplot;

/**
 *
 * @author  hans
 */
import java.awt.Color;

import java.io.IOException;
import java.io.RandomAccessFile;


/** 
 * This class encapsulates a curve. It is a container class
 */
public class Curve 
{     
    /**
     * Constructs a new Curve object
     * @param x
     * @param y
     * @param col
     * @param label  */
    public Curve( float[] x, float[] y, Color col, String label ) 
    {
        m_x = x.clone( );
        m_y = y.clone( );
        m_lineColor = col;
        m_label = new String( label );
        setRange( );
    }

    /**
     * Constructs a new Curve object
     * @param x
     * @param y
     * @param col
     * @param label  */
    public Curve( double[] x, double[] y, Color col, String label )
    {

        m_x = new float[x.length];
        m_y = new float[y.length];
        
        for (int i=0;i< x.length;i++)
        {
            m_x[i] = (float)x[i];
            m_y[i] = (float)y[i];
        }
        
        m_lineColor = col;
        m_label = new String( label );
        setRange( );
    }


    /** Constructs a new Curve object
     * @param x
     * @param y
     * @param col
     */    
    public Curve( float[] x, float[] y, Color col ) 
    {
        m_x = x.clone( );
        m_y = y.clone( );
        m_lineColor = col;
        setRange( );
    }

    /** Constructs a new Curve object
     * @param x
     * @param y
     * @param col
     * @param lineStyle
     */
    public Curve( float[] x, float[] y, Color col, int lineStyle )
    {
        m_x = x.clone( );
        m_y = y.clone( );
        m_lineColor = col;
        m_lineStyle = lineStyle;
        setRange( );
    }


    /** 
     * Constructs a new Curve object.
     * @param x
     * @param y
     * @param col
     * @param lineStyle
     * @param curveLabel
     */
    public Curve( float[] x, float[] y, Color col, 
                  int lineStyle, String curveLabel )
    {

        m_x = x.clone( );
        m_y = y.clone( );
        m_lineColor = col;
        m_lineStyle = lineStyle;
        this.m_label = curveLabel;

        setRange( );
    }

    /** 
     * Constructs a new Curve object.
     * @param x
     * @param y
     * @param label  
     */    
    public Curve( float[] x, float[] y, String label ) 
    {
        m_x = x.clone( );
	m_y = y.clone( );
	m_label = label;
	setRange( );
    }

    /** 
     * Constructs a new Curve object.
     * @param x
     * @param y
     */    
    public Curve( float[] x, float[] y ) 
    {
        m_x = x.clone( );
        m_y = y.clone( );
        setRange( );
    }

    
    /** 
     * Constructs a new Curve object.
     * @param x
     * @param y
     */    
    public Curve( double[] x, double[] y ) 
    {
        m_x = new float[ x.length ];
        m_y = new float[ y.length ];
        
        for ( int i = 0 ; i < x.length ; i++ )
        {
            m_x[i] = (float)x[i];
            m_y[i] = (float)y[i];
        }
        setRange( );
    }
    
    /** 
     * Constructs a new Curve object.
     * @param x
     * @param y
     * @param col
     */    
    public Curve( double[] x, double[] y, Color col ) 
    {
        m_x = new float[ x.length ];
        m_y = new float[ y.length ];
        
        for ( int i = 0 ; i < x.length ; i++ )
        {
            m_x[i] = (float)x[i];
            m_y[i] = (float)y[i];
        }
        m_lineColor = col;
        setRange( );
    }
    
    /** 
     * Constructs a new Curve object.
     * @param x
     * @param y
     * @param col
     */    
    public Curve( int[] x, int[] y, Color col ) 
    {
        int N = Math.min( x.length, y.length );
        this.m_x = new float[N];
        this.m_y = new float[N];
        for ( int k = 0; k < N; k++ ) 
        {
            m_x[k] = (float)x[k];
            m_y[k] = (float)y[k];
        }
        m_lineColor = col;
        setRange( );
    }
    
    /**
     * Set the horizontal x-values.
     * @param x 
     */
    public void setXValues( float[] x ) 
    {
        if ( x.length != this.m_x.length ) 
        {
            this.m_x = new float[x.length];
            float[] newY = new float[x.length];
            System.arraycopy( this.m_y, 0, newY, 0, x.length );
            this.m_y = newY;
        }
        System.arraycopy( x, 0, this.m_x, 0, x.length );
        setRange( );
    }
    
    /**
     * Set the vertical y-values.
     * @param y 
     */
    public void setYValues( float[] y ) 
    {
        if ( y.length != this.m_y.length ) 
        {
            this.m_y = new float[y.length];
            float[] newX = new float[y.length];
            System.arraycopy( this.m_x, 0, newX, 0, y.length );
            this.m_x = newX;
        }
        System.arraycopy( y, 0, this.m_y, 0, y.length );
        setRange( );
    }

    /**
     * Sets the range based on the x and th y array values.
     */
    private void setRange( ) 
    {

        m_xmin = m_xmax = m_x[0];
        for ( int k = 1; k < m_x.length; k++ ) 
        {
            if ( m_x[k] < m_xmin ) 
            {
                m_xmin = m_x[k];
            }
            else if ( m_x[k] > m_xmax ) 
            {
                m_xmax = m_x[k];
            }
        }
        
        m_ymin = m_ymax = m_y[0];
        for ( int k = 1; k < m_y.length; k++ ) 
        {
            if ( m_y[k] < m_ymin ) 
            {
                m_ymin = m_y[k];
            }
            else if ( m_y[k] > m_ymax ) 
            {
                m_ymax = m_y[k];
            }
        }	    
    }
    
    /**
     * Sets the color of the curve.
     * @param col 
     */
    public void setColor( Color col ) 
    { 
        m_lineColor = col; 
    }

    /**
     * Returns the color of the curve.
     * @return 
     */
    public Color getColor( ) 
    { 
        return m_lineColor; 
    }

    /**
     * Sets the text label of the curve. 
     * @param label 
     */
    public void setLabel( String label ) 
    {
        m_label = new String( label );
    }

    /**
     * Sets the curve drawing style.
     * @param style
     */
    public void setStyle( int style ) 
    {
        if ( ( style == NORMAL_LINE  )       ||
             ( style == DOTTED_LINE )       ||
             ( style == POINTS_ONLY )       ||
             ( style == THICK_SOLID_LINE )  ||
             ( style == FAT_SOLID_LINE )    ||
             ( style == FAT_POINTS_ONLY ) )
        {
            m_lineStyle = style;
        }
        else
        {
            m_lineStyle = NORMAL_LINE;
        }
    }
    
    /**
     * Returns the style of the curve.  
     * @return 
     */    
    public int getStyle( ) 
    {
        return m_lineStyle;
    }
    
    
    /**
     * Returns the size of the curve, ie the number of points.
     * @return 
     */
    public int getSize( )
    {
        return m_x.length;
    }
    
    /**
     * Returns the x-coordinate array.
     * @return 
     */
    public float[] getXvalues( )
    {
        return m_x;
    }
    
    /**
     * Returns the y-coordinate array.
     * @return 
     */
    public float[] getYvalues( )
    {
        return m_y;
    }

    
    /**
     * Returns the scaled x-coordinate array.
     * @return 
     */
    public float[] getScaledXvalues( )
    {
        return m_scaled_x;
    }
    
    /**
     * Returns the scaled y-coordinate array.
     * @return 
     */
    public float[] getScaledYvalues( )
    {
        return m_scaled_y;
    }
    
    /**
     * Returns the minimum x-value.
     * @return 
     */
    public float getMinXValue( )
    {
        return m_xmin;
    }
    
    /**
     * Returns the maximum x-value.
     * @return 
     */
    public float getMaxXValue( )
    {
        return m_xmax;
    }

    /**
     * Returns the minimum y-value.
     * @return 
     */
    public float getMinYValue( )
    {
        return m_ymin;
    }

    /**
     * Returns the maximum y-value.
     * @return 
     */
    public float getMaxYValue( )
    {
        return m_ymax;
    }
    
    /**
     * Returns the label string.
     * @return 
     */
    public String getLabel( )
    {
        return m_label;
    }
    
    
    /**
     * Scales the curve in x and y direction according to the scaling factors
     * xscale and yscale.
     * @param x_min
     * @param xscale
     * @param y_min
     * @param yscale
     */
    public void setScaledCurve( float x_min, float xscale, float y_min, float yscale )
    {
        
        m_scaled_x = new float[m_x.length];
        m_scaled_y = new float[m_y.length];
        
        for ( int i = 0 ; i < m_x.length; i++ )
        {
            m_scaled_x[i] = ( m_x[i] - x_min ) / xscale;
            m_scaled_y[i] = ( m_y[i] - y_min ) / yscale;
        }
    }
    
    
    /**
     * Writes the current curve to a text file.
     * @param pathAndFileName
     */
    public void writeToTextFile( String pathAndFileName )
    {
        RandomAccessFile f;
        try
        {
            f = new RandomAccessFile( pathAndFileName, "rw" );
                
            for ( int i = 0; i < m_x.length; i++ ) 
            {
                f.writeBytes( ""  + m_x[i] );
                f.writeBytes( "\t" );
                f.writeBytes( "" + m_y[i] );
                f.writeBytes( "\n" );
            }
            f.close( );    
        }
        catch ( IOException e )
        {
            System.out.println( e.getMessage( ) );
        }
    }
    
    // Attributes for the class
    protected float[] m_x;
    protected float[] m_y;
    
    // Declaration of arrays that contain scaled versions of m_x and m_y
    // that can be drawn on the xyplot
    
    protected float[] m_scaled_x;
    protected float[] m_scaled_y;
    
    protected float m_xmin;
    protected float m_xmax;
    protected float m_ymin;
    protected float m_ymax;
    private Color   m_lineColor = Color.black;
    private String  m_label     = null;
    private int     m_lineStyle = NORMAL_LINE;
     
    public final static int NORMAL_LINE          = 0;
    public final static int DOTTED_LINE         = 1;
    public final static int POINTS_ONLY         = 2;
    public final static int THICK_SOLID_LINE    = 3;
    public final static int FAT_SOLID_LINE      = 4;
    public final static int FAT_POINTS_ONLY     = 5;
    public final static int THIN_SOLID_LINE     = 6;

}
