/*
 * File: $Id: MRFittingDefaults.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 16.09.2003, 10:58  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

 
package org.spectrim.xyplot;

import java.util.Properties;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;

import java.io.IOException;

/**
 * Singleton class that contains all user preferred settings
 *
 * @author  hans
 */
public class MRFittingDefaults extends Properties
{

    /** 
     * Creates a new instance of MRFittingDefaults 
     */
    private MRFittingDefaults( ) 
    {
        setProperty( FIT_PROJECT_ROOT_PATH, System.getProperty( "user.home" ) );
        setProperty( FIT_PROJECT_PATH, System.getProperty( "user.home" ) );
        setProperty( FIT_PROJECT_DATABASE_PATH, System.getProperty( "user.home" ) );
        setProperty( DICOM_MOUNT_PATH, System.getProperty( "user.home" ) );
        setProperty( DISPLAY_DOMAIN, FREQ_DOMAIN );
        setProperty( DISPLAY_CHANNEL, ABSORPTION );
        setProperty( DISPLAY_UNIT, FREQUENCY_DOMAIN_HZ );
        
        // export model as pattern
        setProperty( EXPORT_MODEL_AS_PATTERN_APPLY_FILTER, YES );
        setProperty( PATTERN_EXPORT_CUTOFF_FREQ, "5.000" );
        
        setProperty( APPLY_QUALITY, "true" );
        setProperty( QUALITY_TYPE, QUALITY_TYPE_PHASE_ONLY );
        
        // patient database root path
        setProperty( PATIENT_DATABASE_ROOT_PATH, System.getProperty( "user.home" ) );
        
    }
    
    /**
     * Returns the one and onnly instance of MRFittingDefaults
     */
    public static MRFittingDefaults getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new MRFittingDefaults( );
        }
        return theInstance;
    }

    /**
     * Loads the user preferences file
     */ 
    public void load( )
    {
        String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "FitGuru.ini";
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
        
            // now the ini file must exist with default contents, so load it
            
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            try
            {
                OutputStream os = new BufferedOutputStream( new FileOutputStream(  userPreferenceFileName ) );
                this.save( os, "// This is a generated file: do not edit!!" );
                os.close( );
            }
            catch ( IOException ioe )
            { 
                // should hopefully never occur
            }
        }
        
        // now the ini file must exist with default contents, so load it
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            // should never occur
        }
    }
    
   /**
     * Saves the user preferences file
     */ 
    public void save( )
    {
        try
        {
            String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "FitGuru.ini";
            OutputStream out = new BufferedOutputStream( new FileOutputStream( userPreferenceFileName ) );
            this.save( out, "//This is a generated file : do not edit" );
        }
        catch ( IOException e )
        {
            // should not occur
        }
    }
    // static private attribute is the instance itself (Singleton!)
    private static  MRFittingDefaults theInstance = null;
    
    // Member variables
    public final static String FIT_PROJECT_ROOT_PATH = "Fit project root path";
    public final static String FIT_PROJECT_PATH = "Fit project path";
    public final static String FIT_PROJECT_DATABASE_PATH = "Fit project database path";
    public final static String DICOM_MOUNT_PATH = "Dicom mount path";
    
    // Indicator to the location of the patient database files
    public final static String PATIENT_DATABASE_ROOT_PATH = "Patient database root path";
    
    public final static String DISPLAY_CHANNEL = "display channel";
    public final static String DISPLAY_DOMAIN = "display domain";
    public final static String DISPLAY_UNIT = "display unit";
    
    // Variables needed to identify display domain types
    public final static String FREQ_DOMAIN = "freq-domain";
    public final static String TIME_DOMAIN = "time-domain";
    
    // Variables needed to identify display channel types
    public final static String ABSORPTION = "absorption";
    public final static String DISPERSION = "dispersion";
    public final static String MAGNITUDE  = "magnitude";
    public final static String POWER = "power";
    public final static String PHASE = "phase";
    

    public final static String TIME_DOMAIN_POINTS = "td_points";
    public final static String TIME_DOMAIN_SECONDS = "td_seconds";
    
    public final static String FREQUENCY_DOMAIN_POINTS = "fd_points";
    public final static String FREQUENCY_DOMAIN_HZ = "fd_hertz";
    public final static String FREQUENCY_DOMAIN_PPM = "fd_ppm";
    
    public final static String EXPORT_MODEL_AS_PATTERN_APPLY_FILTER = "When model exported as filter apply filter";
    
    public final static String YES = "yes";
    public final static String NO = "no";
    
    public final static String PATTERN_EXPORT_CUTOFF_FREQ = "Pattern cut-off frequency";
    
    public final static String APPLY_QUALITY = "Apply QUALITY";
    public final static String QUALITY_TYPE = "QUALITY Type";
    
    // Quality type can either be complete (amplitude and phase) or phase only
    public final static String QUALITY_TYPE_PHASE_ONLY = "Phase only";
    public final static String QUALITY_TYPE_AMPLITUDE_AND_PHASE = "Amplitude and Phase";
    
    // For proper serialization
    private static final long serialVersionUID = 8882232191119213L;
}
