/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.xyplot;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.data.CSIData;
import org.spectrim.model.GlobalModel;
import org.spectrim.report.Report;
import org.spectrim.spectra.Spectra;
import org.spectrim.tdfdfit.MRParameterSpectrumModelViewer;
import org.spectrim.tdfdfit.MRSignalProperties;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.SingleVoxelSpectrumViewer;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import static org.spectrim.xyplot.MultiXYPlotFrame.FRAME_RESIZED_EVENT_CMD;

/**
 * This class handles the display of a grid of XYPlot views.
 * 
 * @author johan
 */
public class MultiXYPlotView  extends JPanel implements ActionListener 
{
    // Attributes
    private Vector<MRParameterSpectrumModelViewer> m_XYPlotViews = new Vector<>( );
    private Vector<MRParameterSpectrumModelViewer> m_selectedXYPlotViews = new Vector<>( );
    
    // Alternative way to store the XYPlot 
    private MRParameterSpectrumModelViewer[][] m_XYPlotViews2D;
    
    private XYPlot m_selectedView; 
    private int m_parentFrameWidth  = 512;
    private int m_parentFrameHeight = 512;
        
    private int m_currentXYPlotWidth  = 256;
    private int m_currentXYPlotHeight = 256;
    
    
    private float m_globalZoomFactor = 1.0f;
    private JFrame m_parent;
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    private int m_numberOfPlotsInXdirection = 0;
    private int m_numberOfPlotsInYdirection = 0;
    
    JScrollPane m_scrollPane;

    /**
     * Constructor MultiXYPlotView. Component displays a grid of XYPlot objects.
     * @param parent
     * @param numberOfPlotsInXdirection
     * @param numberOfPlotsInYdirection
     */
    public MultiXYPlotView( JFrame parent, int numberOfPlotsInXdirection, int numberOfPlotsInYdirection ) 
    {
        JPanel collectionPanel = new JPanel( ); 
        collectionPanel.setLayout( null );
        
        m_scrollPane = new JScrollPane( collectionPanel );
        
        m_parent = parent;
        
        int ndx = 5;
        int ndy = 5;
        
        float[] xaxis  = new float[1024];
        float[] absTD  = new float[1024];
        float[] dispTD = new float[1024];
        
        // Fill the arrays with some values
        for ( int i = 0; i < 1024;i++ )
        {
            xaxis[i]  = (float)i;
            absTD[i]  = (float)Math.cos( 2f * Math.PI * i / 256f );
            dispTD[i] = (float)Math.sin( 2f * Math.PI * i / 256f ); 
        }
        
        Curve testCurve = new Curve( xaxis, absTD, Color.BLUE );
        
        for ( int i = 0; i < ndx; i++ )
        {
            for ( int j = 0; j < ndy; j++ )
            {
                MRParameterSpectrumModelViewer plt = new MRParameterSpectrumModelViewer( "dummy", m_parent, null );
                plt.setTextFontSize( 14 - ndx );
                plt.setbackgroundFillColor( Color.LIGHT_GRAY, 255 );
                
                plt.setSize( m_parent.getWidth( ) / ndx , m_parent.getHeight( ) / ndy );
                plt.addSpectralDataSet( "Spectral_ID", new SingleVoxelMeasuredSpectralData( absTD, dispTD ), true );
                m_XYPlotViews.add( plt );
            }
        }
        
        collectionPanel.setPreferredSize( new Dimension( m_parent.getWidth( ), m_parent.getHeight( ) ) );
        collectionPanel.setSize( new Dimension( m_parent.getWidth( ), m_parent.getHeight( ) ) );
        
        m_scrollPane.setSize( new Dimension( m_parent.getWidth( ), m_parent.getHeight( ) ) );
        
        this.add( m_scrollPane, BorderLayout.CENTER );
        
        setupDefaultLayout( );
        
        m_numberOfPlotsInXdirection = numberOfPlotsInXdirection;
        m_numberOfPlotsInYdirection = numberOfPlotsInYdirection;
    }
    
    /**
     * Constructor to make a MultiXYPlot. 
     * @param parent
     * @param model 
     */
    public MultiXYPlotView( JFrame parent, GlobalModel model ) 
    {
        m_parent = parent;
        
        ArrayList<String> ids = model.getSelectedVoxels( );
        // determine number of voxels in the x-direction
        ArrayList<Integer> uniqueXs = new ArrayList<>( );
        ArrayList<Integer> uniqueYs = new ArrayList<>( );
        ArrayList<Integer> uniqueZs = new ArrayList<>( );
        
        Iterator voxIt = ids.iterator( );
        
        while ( voxIt.hasNext( ) )
        {
            StringTokenizer strtok = new StringTokenizer( (String) voxIt.next( ), "_" );
            int x = new Integer( strtok.nextToken( ) );
            if ( !uniqueXs.contains( x ) == true )
            {
                uniqueXs.add( x );
            }
            
            int y = new Integer( strtok.nextToken( ) );
            
            if ( !uniqueYs.contains( y ) == true )
            {
                uniqueYs.add( y );
            }
            
            int z = new Integer( strtok.nextToken( ) );
            
            if ( !uniqueZs.contains( z ) == true )
            {
                uniqueZs.add( z );
            }
            
        }
        
        float[] absTD;
        float[] dispTD;
        CSIData csiData = model.getCSIData( );
        Spectra spectra = csiData.getSpectra( );
        
        int cntrX = 0;
        int cntrY = 0;
        
        // Set the MRSignalProperties to be sure that it are set
        MRSignalProperties.getInstance( ).setNumberOfSamplePoints( model.getVectorSize( ) );
        MRSignalProperties.getInstance( ).setHz_pp( (float)( model.getBandwidth( ) / model.getVectorSize( ) ) );
        MRSignalProperties.getInstance( ).setResonanceFrequency( new Double( "" + model.getResonanceFrequency( ) ).longValue( ) );
        MRSignalProperties.getInstance( ).setPPMValueInCenterOfSpectrum( (float)model.getPPMReference( ) );
         
        // we restrict the display only to selected voxels in one slice!
        if ( uniqueZs.size( ) == 1 )
        {
            
            m_numberOfPlotsInXdirection = uniqueXs.size( );
            m_numberOfPlotsInYdirection = uniqueYs.size( );

            // If the selected voxels are contiguous with m_numberOfPlotsInXdirection
            // and m_numberOfPlotsInYdirection number of unique values the
            // selected spectra are a packed block
            if ( ids.size( ) == m_numberOfPlotsInXdirection * m_numberOfPlotsInYdirection )
            {
                
                m_XYPlotViews2D = new MRParameterSpectrumModelViewer[m_numberOfPlotsInXdirection][m_numberOfPlotsInYdirection];
                
                for ( int i = 0; i < model.getGridDim1( ); i++ )
                {
                    for ( int j = 0; j < model.getGridDim2( ); j++ )
                    {
                        String keyIt = "" + i + "_" + j + "_" + uniqueZs.get(0);
                        
                        if ( ids.contains( keyIt ) == true )
                        {
                            
//                          System.out.println( "keyIt = " + keyIt );
                            
                            absTD  = spectra.getSpectrumData( keyIt, Spectra.Domain.TD, Spectra.Mode.ABS );
                            dispTD = spectra.getSpectrumData( keyIt, Spectra.Domain.TD, Spectra.Mode.DISP );

                            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_DOMAIN, TDFDFittingDefaults.FREQ_DOMAIN );
                            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.ABSORPTION );
                            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM );
                            TDFDFittingDefaults.getInstance( ).save( );

                            SingleVoxelMeasuredSpectralData singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( absTD, dispTD );
                            singleVoxelMeasuredData.doComplexConjugation( );

                            MRParameterSpectrumModelViewer plt = new MRParameterSpectrumModelViewer( keyIt, m_parent, null );
                            plt.setTextFontSize( 11 );
                            plt.setXTitle( "ppm" );
                            plt.setbackgroundFillColor( Color.LIGHT_GRAY, 255 );

                            plt.setSize( m_parent.getWidth( ) / 2 , m_parent.getHeight( ) / 2 );
                            plt.addSpectralDataSet( keyIt, singleVoxelMeasuredData, true );

                            // plt and the multiview should listen to one another!
                            plt.addActionListener( this );
                            this.addActionListener( plt );
                                                       
                            m_XYPlotViews2D[cntrX][cntrY] = plt;
                            
                            // compute the position in the 2D array
                            cntrX++;
                            if ( cntrX > m_numberOfPlotsInXdirection - 1 )
                            {
                                cntrX = 0;
                                cntrY++;
                            }
                        }
                    }
                }
            }
            else
            {
//                 JOptionPane.showMessageDialog( this, "<html>Cannot make a 2D grid of selected XYplots:" + 
//                                                "<br><br>Only voxel selections of one contiguous area can be plotted!</html>");
            
                Iterator it = ids.iterator( );

                int numberOfSelectedVoxels = ids.size( );

                m_numberOfPlotsInYdirection = (int)Math.ceil( Math.sqrt( numberOfSelectedVoxels ) );
                m_numberOfPlotsInXdirection = (int)Math.ceil( numberOfSelectedVoxels / (float)m_numberOfPlotsInYdirection );
                
                m_XYPlotViews2D = new MRParameterSpectrumModelViewer[m_numberOfPlotsInXdirection][m_numberOfPlotsInYdirection];
                
                for ( int i = 0; i < m_numberOfPlotsInXdirection; i++ )
                {
                    for ( int j = 0; j < m_numberOfPlotsInYdirection; j++ )
                    {
                        if ( it.hasNext( ) )
                        {
                            String key = (String)it.next( );
                            absTD  = spectra.getSpectrumData( key, Spectra.Domain.TD, Spectra.Mode.ABS );
                            dispTD = spectra.getSpectrumData( key, Spectra.Domain.TD, Spectra.Mode.DISP );

                            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_DOMAIN, TDFDFittingDefaults.FREQ_DOMAIN );
                            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.ABSORPTION );
                            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM );
                            TDFDFittingDefaults.getInstance( ).save( );

                            SingleVoxelMeasuredSpectralData singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( absTD, dispTD );
                            singleVoxelMeasuredData.doComplexConjugation( );

                            MRParameterSpectrumModelViewer plt = new MRParameterSpectrumModelViewer( key, m_parent, null );
                            plt.setTextFontSize( 14  );
                            plt.setXTitle( "ppm" );
                            plt.setbackgroundFillColor( Color.LIGHT_GRAY, 255 );

                            plt.setSize( m_parent.getWidth( ) / 2 , m_parent.getHeight( ) / 2 );
                            plt.addSpectralDataSet( key, singleVoxelMeasuredData, true );

                            // plt and the multiview should listen to one another!
                            plt.addActionListener( this );
                            this.addActionListener( plt );

                            // m_XYPlotViews.add( plt ); 
                            m_XYPlotViews2D[i][j] = plt;
                        }
                    }
                }       
            }            
        }
        else
        {    
            JOptionPane.showMessageDialog( this, "<html>Cannot make a 2D grid of selected XYplots:" + 
                                           "<br><br>All selected voxels must be in one slice!</html>");
        }
    }
    
    /**
     * Returns the top, left coordinate of the XYPlot object.
     * @param viewNumber
     * @return 
     */    
    public Point getTopLeftPosition( int viewNumber ) 
    {
        int gap = 0;	/* gap between images */
        float n = (float)m_XYPlotViews.size( );
        if ( n == 0 ) return new Point( 0, 0 );
        
        int cols = (int)Math.ceil( Math.sqrt( n ) );
        int rows = (int)Math.ceil( n / (float)cols );
        
        int cntr = 0;
        
        for ( int i = 0; i < cols; i++ )
        {
            for ( int j = 0; j < rows; j++ )
            {
                if ( cntr == viewNumber )
                {
                    int leftPosition = (int)( i * m_currentXYPlotWidth  );
                    int topPosition  = (int)( j * m_currentXYPlotHeight  );
                    return new Point(  leftPosition, topPosition );
                }
                cntr++;
            }
        }
        
        return null;
    }
    
    
    /**
     * Handles the layout of the MultiXYPlotView.
     */    
    public void setupDefaultLayout( ) 
    {
        // System.out.println( "In setupLayout !" );
        this.removeAll( );
        
        int gap = 5;	// gap between images
        float n = (float)m_XYPlotViews.size( );
        if ( n == 0 ) return;
	
        int cols = (int)Math.ceil( Math.sqrt( n ) );
        int rows = (int)Math.ceil( n / (float)cols );
	
        this.setLayout( null );
        
        m_currentXYPlotWidth  = ( m_parent.getWidth( )  / cols ) - 0;
        m_currentXYPlotHeight = ( m_parent.getHeight( ) / rows ) - 20;
        
        Iterator it = m_XYPlotViews.iterator( );
        
        int xyplot_cntr = 0;
        
        while ( it.hasNext( ) )
        {
            SingleVoxelSpectrumViewer xyplot = (SingleVoxelSpectrumViewer)it.next( );
            
            // canvas.setRedrawImage( true );
            
            this.add( xyplot );
            
            Point p = getTopLeftPosition( xyplot_cntr );
            xyplot.setSize( m_currentXYPlotWidth, m_currentXYPlotHeight );
            xyplot.setLocation( p );
            
            xyplot_cntr++;
        }
         
        // doLayout( ); 
    }
    
    /**
     * Handles the layout of the MultiXYPlotView.
     */    
    public void setupLayoutGrid( ) 
    {
        // System.out.println( "In setupLayout !" );
        this.removeAll( );
        
        this.setLayout( null );
        
        m_currentXYPlotWidth  = ( m_parent.getContentPane( ).getWidth( )  / m_numberOfPlotsInXdirection );
        m_currentXYPlotHeight = ( m_parent.getContentPane( ).getHeight( ) / m_numberOfPlotsInYdirection );
               
        for ( int i = 0; i < m_numberOfPlotsInXdirection; i++ )
        {
            for ( int j = 0; j < m_numberOfPlotsInYdirection; j++ )
            {
                SingleVoxelSpectrumViewer xyplot = m_XYPlotViews2D[i][j];
                if ( xyplot != null )
                {    
                    this.add( xyplot );
                    int leftPosition = (int)( i * m_currentXYPlotWidth  );
                    int topPosition  = (int)( j * m_currentXYPlotHeight  );

                    Point p = new Point( leftPosition, topPosition );
                    xyplot.setSize( m_currentXYPlotWidth, m_currentXYPlotHeight );
                    xyplot.setLocation( p );
                }
            }
        }
    }
    
     /**
     * Adds an ActionListener to the list of action listeners who are notified
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
 
    /**
     * Notify all registered listeners that something in the object has changed.
     */
    private void notifyActionListeners( int eventType, String eventCmd )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, eventCmd ) );
        }
    }

    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        if ( e.getActionCommand( ).compareTo( FRAME_RESIZED_EVENT_CMD ) == 0 )
        {
            if ( m_XYPlotViews2D != null )
            {
                System.out.println( "User change frame size!" );
                Dimension  dim = ( (MultiXYPlotFrame)e.getSource( ) ).getContentPane( ).getSize( );
                setSize( dim );
                setupLayoutGrid( );
            }
            else
            {
                m_parent.dispose( );
            }
        }
        
        if ( e.getActionCommand( ).compareTo( "USER_REQUESTED_XYPLOT_ZOOM_EVENT_CMD" ) == 0 )
        {
            SingleVoxelSpectrumViewer sender = (SingleVoxelSpectrumViewer)e.getSource( );
            
            System.out.println( "User wants to zoom one of the plots! \n Zooming event detected in object : " + sender.getTitle( ) );
            XYPlotRange pltRange = sender.getForcedPlotRange( );
            
            for ( int i = 0; i < m_numberOfPlotsInXdirection; i++ )
            {
                for ( int j = 0; j < m_numberOfPlotsInYdirection; j++ )
                {
                    SingleVoxelSpectrumViewer xyplot = m_XYPlotViews2D[i][j];
                    if ( xyplot.getTitle( ).compareTo( sender.getTitle( ) ) != 0 )
                    {
                        xyplot.setForcedPlotRange( pltRange );
                    }    
                }
            }
            
        }   
    }

    
    /**
     * Handles the printing of the currently displayed view.
     */
    void print( ) 
    {
        PrinterJob pjob = PrinterJob.getPrinterJob();
        PageFormat preformat = pjob.defaultPage();
        preformat.setOrientation(PageFormat.LANDSCAPE);
        PageFormat postformat = pjob.pageDialog(preformat);
        //If user does not hit cancel then print.
        if ( preformat != postformat ) 
        {
            //Set print component
            pjob.setPrintable( new JPanelPrinter( this ), postformat );
            if ( pjob.printDialog( ) ) 
            {
                try 
                {
                    pjob.print( );
                } catch ( PrinterException e ) 
                {
                    // TODO Auto-generated catch block
                    e.printStackTrace( );
                }
            }
        }
    }

    
    /**
     * Creates a png file of the spectral grid of XYPlots. 
     */
    void createPNGFile( ) 
    {
        JFileChooser jFileChooserSavePNG = new JFileChooser( );
        
        String defaultFileReference = System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "Pictures" + System.getProperty( "file.separator" ) + "screenshot.png";
        jFileChooserSavePNG.setSelectedFile( new File( defaultFileReference ) );
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "PNG FILES", "png", "png" );
        jFileChooserSavePNG.setFileFilter( filter );
        int returnVal = jFileChooserSavePNG.showSaveDialog( this );
        if ( returnVal == JFileChooser.APPROVE_OPTION ) 
        {
            File imageToSave = jFileChooserSavePNG.getSelectedFile( );
            BufferedImage bi = new BufferedImage( this.getWidth( ), this.getHeight( ), BufferedImage.TYPE_INT_RGB );
            Graphics g = bi.createGraphics( );
            this.print( g );
            g.dispose( );
            try 
            {
                ImageIO.write( bi, "PNG", imageToSave );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( Report.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        } 
        else 
        {
            //log.append("Open command cancelled by user." + newline);
        }
    }


}
