/*
 * File: $Id: XYPlot.java,v 1.2 2006/04/20 09:29:39 hans Exp $
 * History
 * Date: 01.04.2003, 17:09 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
package org.spectrim.xyplot;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.JPopupMenu;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.BorderFactory;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ButtonGroup;

import java.util.LinkedList;


import java.text.NumberFormat;
import java.text.DecimalFormat;

import java.util.Vector;
import java.util.Iterator;


import org.spectrim.lib.StatusPanel;

/**
 * Class to draw simple xy-plots.
 * Based on an Modula2 implementation of 1990-tees.
 * @author  hans
 */

public class XYPlot extends JPanel implements ActionListener, MouseListener, MouseMotionListener
{

    public XYPlot( )
    {
        // set the reference to the parent frame object
        m_parent = null;
        
        setLayout( null );
        // setMinimumSize( new Dimension( 256, 256 ) );
        // setPreferredSize( new Dimension( 256, 256 ) );
        // setMaximumSize( new Dimension( 1024, 1024 ) );
        setBorder( BorderFactory.createLoweredBevelBorder( ) );

        setDoubleBuffered( true );        
        
        // Determine some important attributes
        m_plotAreaWidth = this.getWidth( );
        m_plotAreaHeight = this.getHeight( );
        m_title = "No title defined";
        
        m_popupmenu = new JPopupMenu( );
        
        m_xyScaleMenu = new JMenu( "XY scaling" );
        
        m_XYDefaultRangeMenuItem = new JRadioButtonMenuItem( "Default XY Plot ranges" );
        m_XYDefaultRangeMenuItem.setToolTipText( "Applies default ranges to the plot" );
        m_XYDefaultRangeMenuItem.addActionListener( this );
        m_XYDefaultRangeMenuItem.setSelected( true );
        m_xyScaleMenu.add( m_XYDefaultRangeMenuItem );
        
        m_XYPlotRangeSelectorMenuItem = new JRadioButtonMenuItem( "XY Plot ranges .." );
        m_XYPlotRangeSelectorMenuItem.setToolTipText( "Allows the definition of horizontal and vertical plot ranges" );
        m_XYPlotRangeSelectorMenuItem.addActionListener( this );
        m_xyScaleMenu.add( m_XYPlotRangeSelectorMenuItem );
        
        m_popupmenu.add( m_xyScaleMenu );
        
        m_XYRangeButtongroup = new ButtonGroup( );
        m_XYRangeButtongroup.add( m_XYDefaultRangeMenuItem );
        m_XYRangeButtongroup.add( m_XYPlotRangeSelectorMenuItem );
        
        // Instantiate a plot auto xy plot range object
        m_xyAutoPlotRange = new XYPlotRange( );
        
        // Instantiate a plot auto xy plot range object
        m_xyForcedPlotRange = new XYPlotRange( );  
    }
    
    
    /**
     * Constructor of a simple xy-plot.
     * @param title
     * @param parent
     */
    public XYPlot( String title, JFrame parent ) 
    {   
        // set the reference to the parent frame object
        m_parent = parent;
        
        setLayout( null );
//        setMinimumSize( new Dimension( 256, 256 ) );
//        setPreferredSize( new Dimension( 512, 512 ) );
//        setMaximumSize( new Dimension( 1024, 1024 ) );
        setBorder( BorderFactory.createLineBorder( Color.BLACK ) );

        setDoubleBuffered( true );        
        
        // Determine some important attributes
        m_plotAreaWidth = this.getWidth( );
        m_plotAreaHeight = this.getHeight( );
       
        if ( title != null )
        {
            m_title = title;
        }
        else
        {
            m_title = "no title defined";
        }
        
        m_popupmenu = new JPopupMenu( );
        
        m_xyScaleMenu = new JMenu( "XY scaling" );
        
        m_XYDefaultRangeMenuItem = new JRadioButtonMenuItem( "Default XY Plot ranges" );
        m_XYDefaultRangeMenuItem.setToolTipText( "Applies default ranges to the plot" );
        m_XYDefaultRangeMenuItem.addActionListener( this );
        m_XYDefaultRangeMenuItem.setSelected( true );
        m_xyScaleMenu.add( m_XYDefaultRangeMenuItem );
        
        m_XYPlotRangeSelectorMenuItem = new JRadioButtonMenuItem( "XY Plot ranges .." );
        m_XYPlotRangeSelectorMenuItem.setToolTipText( "Allows the definition of horizontal and vertical plot ranges" );
        m_XYPlotRangeSelectorMenuItem.addActionListener( this );
        m_xyScaleMenu.add( m_XYPlotRangeSelectorMenuItem );
        
        m_popupmenu.add( m_xyScaleMenu );
        
        m_XYRangeButtongroup = new ButtonGroup( );
        m_XYRangeButtongroup.add( m_XYDefaultRangeMenuItem );
        m_XYRangeButtongroup.add( m_XYPlotRangeSelectorMenuItem );
        
        // Instantiate a plot auto xy plot range object
        m_xyAutoPlotRange = new XYPlotRange( );
        
        // Instantiate a plot auto xy plot range object
        m_xyForcedPlotRange = new XYPlotRange( );   
    }

    protected void drawSplineNodes( Graphics g )
    {
        Color col = g.getColor( );
        if ( m_splineNodes != null )
        {
            g.setColor( Color.BLACK );
            for ( int i = 0; i < m_splineNodes.size( ); i++ )
            {
                float xVal = (float)m_splineNodes.get( i ).x;
                float yVal = (float)m_splineNodes.get( i ).y;
                int xPos = (int)( ( getWidth( )/10 ) + ( m_xyAutoPlotRange.getMaximumX( ) - xVal ) * ( 8 * getWidth( )/10f ) / this.m_xyAutoPlotRange.getXRange( ) );
                int yPos = (int)( ( getHeight( )/10 ) + ( m_xyAutoPlotRange.getMaximumY( ) - yVal ) * ( 8 * getHeight( )/10f ) / this.m_xyAutoPlotRange.getYRange( ) );
                g.drawOval( xPos - 4, 
                            yPos - 4 , 
                            8, 
                            8 );
            }
        }
        g.setColor( col );
    }        
    
    public void setSplineNodes( Vector<Point2D.Double> splineNodesIntPointValues )
    {
        m_splineNodes = splineNodesIntPointValues;
    }
    
    
    /**
     * Calculates the offset.
     */
    private void offset( float[] x, float q )
    {
        int j;

        for ( j = 0; j < x.length; j++ )
        {
            x[j] = x[j] - q;
        }
    }
    /* end offset */

    
    /**
     * Finds the maximum value of a float array x.
     */
    private float getMaximum( float[] x )
    {
        float max = Float.MIN_VALUE;
        
        for ( int j = 1; j < x.length; j++ )
        {
            if ( x[j] > max )
            {
		max = x[j];
            }
        }
        return max;
    }
    
    /**
     * Finds the minimum value of a float array x.
     */
    private float getMinimum( float[] x )
    {

        float min = Float.MAX_VALUE;
        
        for ( int j = 1; j < x.length; j++ )
        {
            if ( x[j] < min )
            {
                min = x[j];
            }
        }
        return min;
    }

    public XYPlotRange getForcedPlotRange( )
    {
        return m_xyForcedPlotRange;
    }
    
    public void setForcedPlotRange( XYPlotRange plotRange )
    {
        m_xyForcedPlotRange = plotRange;
        m_XYPlotRangeSelectorMenuItem.setSelected( true );
        refresh( );
    }
    
    /**
     * Does the actual over plotting!
     * @param g
     */
    private void overPlotRange( Graphics g )
    {
        
        // System.out.println( "XYPlot -> Over plotting!" );

        float xscale, yscale, xrange, yrange;
            
        int labelStartX = (int)( 9f * m_plotAreaWidth / 10f ) - computeCurveLabelStartPosition( g ) - 5;

        if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
        {
            xrange = Math.abs( m_xyAutoPlotRange.getMaximumX( ) - m_xyAutoPlotRange.getMinimumX( ) );
            yrange = Math.abs( m_xyAutoPlotRange.getMaximumY( ) - m_xyAutoPlotRange.getMinimumY( ) );
        }
        else
        {
            xrange = Math.abs( m_xyForcedPlotRange.getMaximumX( ) - m_xyForcedPlotRange.getMinimumX( ) );
            yrange = Math.abs( m_xyForcedPlotRange.getMaximumY( ) - m_xyForcedPlotRange.getMinimumY( ) );
        }

        xscale = ( 10 * xrange ) / ( 8 * m_plotAreaWidth );
        yscale = ( 10 * yrange ) / ( 8 * m_plotAreaHeight );



        m_minCurve.setScaledCurve( m_xyForcedPlotRange.getMaximumX( ), -xscale, m_xyForcedPlotRange.getMinimumY( ), yscale );
        m_maxCurve.setScaledCurve( m_xyForcedPlotRange.getMaximumX( ), -xscale, m_xyForcedPlotRange.getMinimumY( ), yscale );

        plotRange( g, m_minCurve.getScaledXvalues( ), m_minCurve.getScaledYvalues( ), m_maxCurve.getScaledYvalues( ), m_backgroundFillColor );

        // System.out.println( "XYPlot -> end of over plotting!" );
        
    }

    /**
     * Prepares the screen.
     */
    private void screenready( Graphics g )
    {
        int j, u, w;
        
        g.setColor( m_frameColor );
  
        // draw plot frame takes 0.8 of total screen
        
        g.drawLine( m_plotAreaWidth / 10, m_plotAreaHeight / 10, 
                    m_plotAreaWidth / 10, m_plotAreaHeight / 10 );
        g.drawLine( m_plotAreaWidth / 10, m_plotAreaHeight / 10, 
                    9 * m_plotAreaWidth / 10, m_plotAreaHeight / 10 );
        g.drawLine( 9 * m_plotAreaWidth / 10, m_plotAreaHeight / 10, 
                    9 * m_plotAreaWidth / 10, 9 * m_plotAreaHeight / 10 );
        g.drawLine( m_plotAreaWidth / 10, 9 * m_plotAreaHeight / 10, 
                    m_plotAreaWidth / 10, m_plotAreaHeight / 10 );
  
        for ( j = 1; j <= 9; j++ )
        {
            u = j * 8 * m_plotAreaWidth / 100 + m_plotAreaWidth / 10;
            w = j * 8 * m_plotAreaHeight / 100 + m_plotAreaHeight / 10;
            
            g.drawLine( u, m_plotAreaHeight / 10, u, m_plotAreaHeight / 10 + 5 );
            g.drawLine( u, 9 * m_plotAreaHeight / 10, u, 9 * m_plotAreaHeight / 10 - 5 );
            g.drawLine( m_plotAreaWidth / 10 , w, m_plotAreaWidth / 10 + 5, w );
            g.drawLine( 9 * m_plotAreaWidth / 10, w, 9 * m_plotAreaWidth / 10 - 5, w );
        }
    }
    /* screenready; */

    /**
     * Plots the range
     * @param g
     * @param x
     * @param ymin
     * @param ymax
     * @param lineColor
     */
    private void plotRange( Graphics g, float[] x, float[] ymin,
                            float[] ymax, Color rangeColor )
    {
        int j;

        int startx, startYmin, startYmax;
        int endx, endYmin, endYmax;

        // set the color of the line to be drawn
        g.setColor( rangeColor );


        // Starting condition x-axis
        if ( (int)x[0] < 0 )
        {
            startx = m_plotAreaWidth / 10;
        }
        else
        {
            if ( (int)x[0] > ( 8 * m_plotAreaWidth / 10 ) )
            {
                startx = 9 * m_plotAreaWidth / 10;
            }
            else
            {
                startx = ( m_plotAreaWidth / 10 ) + (int)x[0];
            }
        }

        // Starting condition y-axis
        if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymin[0] < ( m_plotAreaHeight / 10 ) )
        {
            startYmin = m_plotAreaHeight / 10;
        }
        else
        {
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymin[0] > ( 9 * m_plotAreaHeight / 10 ) )
            {
                startYmin = 9 * m_plotAreaHeight / 10;
            }
            else
            {
                startYmin = ( 9 * m_plotAreaHeight / 10 ) - (int)ymin[0];
            }
        }

        if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymax[0] < ( m_plotAreaHeight / 10 ) )
        {
            startYmax = m_plotAreaHeight / 10;
        }
        else
        {
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymax[0] > ( 9 * m_plotAreaHeight / 10 ) )
            {
                startYmax = 9 * m_plotAreaHeight / 10;
            }
            else
            {
                startYmax = ( 9 * m_plotAreaHeight / 10 ) - (int)ymax[0];
            }
        }

        // rest of the points
        for ( j = 1; j <= x.length - 1; j++ )
        {
            if ( (int)x[j] < 0 )
            {
                endx = m_plotAreaWidth / 10;
            }
            else
            {
                if ( (int)x[j] > ( 8 * m_plotAreaWidth / 10 ) )
                {
                    endx = 9 * m_plotAreaWidth / 10;
                }
                else
                {
                    endx = ( m_plotAreaWidth / 10 ) + (int)x[j];
                }
            }


            // ymin
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymin[j] < ( m_plotAreaHeight / 10 ) )
            {
                endYmin = m_plotAreaHeight / 10;
            }
            else
            {
                if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymin[j] > ( 9 * m_plotAreaHeight / 10 ) )
                {
                    endYmin = 9 * m_plotAreaHeight / 10;
                }
                else
                {
                    endYmin = 9 * m_plotAreaHeight / 10 - (int)ymin[j];
                }
            }

            // ymax
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymax[j] < ( m_plotAreaHeight / 10 ) )
            {
                endYmax = m_plotAreaHeight / 10;
            }
            else
            {
                if ( ( 9 * m_plotAreaHeight / 10 ) - (int)ymax[j] > ( 9 * m_plotAreaHeight / 10 ) )
                {
                    endYmax = 9 * m_plotAreaHeight / 10;
                }
                else
                {
                    endYmax = 9 * m_plotAreaHeight / 10 - (int)ymax[j];
                }
            }

            // now we cans do some some drawing looks solid in case there
            // are enough points, else some kind of rastered
            g.drawLine( startx, startYmin, startx, startYmax );

            g.drawLine( startx, startYmin, endx, endYmax );

            g.drawLine( startx, startYmax, endx, endYmin );

            g.drawLine( endx, endYmin, endx, endYmax );
            
            // System.out.println( "Drawing range!" );

            startx = endx;
            startYmin = endYmin;
            startYmax = endYmax;
        }
    }

    /**
     *  Draws the curve into the graphic context.
     */
    private void solidCurve( Graphics g, float[] x, float[] y, Color lineColor, float lineThickness )
    {
        int j;
        
        int startx, starty;
        int endx, endy;
        Graphics2D g2d = (Graphics2D)g;
        
        // set the color of the line to be drawn
        g2d.setColor( lineColor );
        // creates a solid stroke with line width is 2
        Stroke stroke = new BasicStroke( lineThickness );
        g2d.setStroke( stroke );
        
        // Starting condition x-axis
        if ( (int)x[0] < 0 )
        {
            startx = m_plotAreaWidth / 10;
        }
        else
        {
            if ( (int)x[0] > ( 8 * m_plotAreaWidth / 10 ) )
            {
                startx = 9 * m_plotAreaWidth / 10; 
            }
            else
            {
                startx = ( m_plotAreaWidth / 10 ) + (int)x[0];                   
            }
        }
        
        // Starting condition y-axis
        if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[0] < ( m_plotAreaHeight / 10 ) )
        {
            starty = m_plotAreaHeight / 10;
        }
        else
        {
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[0] > ( 9 * m_plotAreaHeight / 10 ) )
            { 
                starty = 9 * m_plotAreaHeight / 10;
            }
            else
            { 
                starty = ( 9 * m_plotAreaHeight / 10 ) - (int)y[0];
            }
        }

        // rest of the points
        for ( j = 1; j <= x.length - 1; j++ )
        {
            if ( (int)x[j] < 0 )
            {        
                endx = m_plotAreaWidth / 10;
            }
            else
            {
                if ( (int)x[j] > ( 8 * m_plotAreaWidth / 10 ) )
                {
                    endx = 9 * m_plotAreaWidth / 10;          
                }
                else
                {
                    endx = ( m_plotAreaWidth / 10 ) + (int)x[j];   
                }
            }
            
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[j] < ( m_plotAreaHeight / 10 ) )
            {
                endy = m_plotAreaHeight / 10;                
            }
            else
            {
                if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[j] > ( 9 * m_plotAreaHeight / 10 ) )
                {   
                    endy = 9 * m_plotAreaHeight / 10;   
                }
                else
                {   
                    endy = 9 * m_plotAreaHeight / 10 - (int)y[j];
                }
            }
                        
            g.drawLine( startx, starty, endx, endy );
            
            startx = endx;
            starty = endy;
        }
    }
    /* end curve */
    

    /**
     *  Draws the curve into the graphic context.
     */
    private void thickSolidCurve( Graphics g, float[] x, float[] y, Color lineColor )
    {
        int j;

        int startx, starty;
        int endx, endy;

        // set the color of the line to be drawn
        g.setColor( lineColor );

        // Starting condition x-axis
        if ( (int)x[0] < 0 )
        {
            startx = m_plotAreaWidth / 10;
        }
        else
        {
            if ( (int)x[0] > ( 8 * m_plotAreaWidth / 10 ) )
            {
                startx = 9 * m_plotAreaWidth / 10;
            }
            else
            {
                startx = ( m_plotAreaWidth / 10 ) + (int)x[0];
            }
        }

        // Starting condition y-axis
        if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[0] < ( m_plotAreaHeight / 10 ) )
        {
            starty = m_plotAreaHeight / 10;
        }
        else
        {
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[0] > ( 9 * m_plotAreaHeight / 10 ) )
            {
                starty = 9 * m_plotAreaHeight / 10;
            }
            else
            {
                starty = ( 9 * m_plotAreaHeight / 10 ) - (int)y[0];
            }
        }

        // Rest of the points
        for ( j = 1; j <= x.length - 1; j++ )
        {
            if ( (int)x[j] < 0 )
            {
                endx = m_plotAreaWidth / 10;
            }
            else
            {
                if ( (int)x[j] > ( 8 * m_plotAreaWidth / 10 ) )
                {
                    endx = 9 * m_plotAreaWidth / 10;
                }
                else
                {
                    endx = ( m_plotAreaWidth / 10 ) + (int)x[j];
                }
            }

            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[j] < ( m_plotAreaHeight / 10 ) )
            {
                endy = m_plotAreaHeight / 10;
            }
            else
            {
                if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[j] > ( 9 * m_plotAreaHeight / 10 ) )
                {
                    endy = 9 * m_plotAreaHeight / 10;
                }
                else
                {
                    endy = 9 * m_plotAreaHeight / 10 - (int)y[j];
                }
            }

            g.drawLine( startx, starty, endx, endy );
            g.drawLine( startx, starty+1, endx, endy+1 );

            startx = endx;
            starty = endy;
        }
    }
    /* end curve */



    /**
     *  Draws a fat solid curve into the graphic context.
     */
    private void fatSolidCurve( Graphics g, float[] x, float[] y, Color lineColor )
    {
        int j;

        int startx, starty;
        int endx, endy;

        // set the color of the line to be drawn
        g.setColor( lineColor );

        // Starting condition x-axis
        if ( (int)x[0] < 0 )
        {
            startx = m_plotAreaWidth / 10;
        }
        else
        {
            if ( (int)x[0] > ( 8 * m_plotAreaWidth / 10 ) )
            {
                startx = 9 * m_plotAreaWidth / 10;
            }
            else
            {
                startx = ( m_plotAreaWidth / 10 ) + (int)x[0];
            }
        }

        // Starting condition y-axis
        if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[0] < ( m_plotAreaHeight / 10 ) )
        {
            starty = m_plotAreaHeight / 10;
        }
        else
        {
            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[0] > ( 9 * m_plotAreaHeight / 10 ) )
            {
                starty = 9 * m_plotAreaHeight / 10;
            }
            else
            {
                starty = ( 9 * m_plotAreaHeight / 10 ) - (int)y[0];
            }
        }

        // rest of the points
        for ( j = 1; j <= x.length - 1; j++ )
        {
            if ( (int)x[j] < 0 )
            {
                endx = m_plotAreaWidth / 10;
            }
            else
            {
                if ( (int)x[j] > ( 8 * m_plotAreaWidth / 10 ) )
                {
                    endx = 9 * m_plotAreaWidth / 10;
                }
                else
                {
                    endx = ( m_plotAreaWidth / 10 ) + (int)x[j];
                }
            }

            if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[j] < ( m_plotAreaHeight / 10 ) )
            {
                endy = m_plotAreaHeight / 10;
            }
            else
            {
                if ( ( 9 * m_plotAreaHeight / 10 ) - (int)y[j] > ( 9 * m_plotAreaHeight / 10 ) )
                {
                    endy = 9 * m_plotAreaHeight / 10;
                }
                else
                {
                    endy = 9 * m_plotAreaHeight / 10 - (int)y[j];
                }
            }

            // The fat line:
            g.drawLine( startx-1, starty, endx-1, endy );
            g.drawLine( startx+1, starty, endx+1, endy );
            g.drawLine( startx, starty, endx, endy );
            g.drawLine( startx, starty + 1, endx, endy + 1 );
            g.drawLine( startx, starty - 1, endx, endy - 1 );
            
            startx = endx;
            starty = endy;
        }
    }
    /* end curve */

    /**
     * Draws a dotted line on the canvas.
     */ 
    private void dottedCurve( Graphics g, float[] x, float[] y, Color lineColor )
    {
        int j;
        int startx, starty;
        
        // Set the color of the dotted line to be drawn 
        g.setColor( lineColor );
  
        for ( j = 0; j <= x.length - 1; j++ )
        {   
            g.drawLine( m_plotAreaWidth / 10 + (int)x[j], 
                        9 * m_plotAreaHeight / 10 - (int)y[j],
                        m_plotAreaWidth / 10 + (int)x[j], 
                        9 * m_plotAreaHeight / 10 - (int)y[j] );
        }
    }

    /**
     * Draws the points only on the canvas.
     */ 
    private void pointsOnlyCurve( Graphics g, float[] x, float[] y, Color lineColor )
    {
        int j;
        int startx, starty;
        
        // Set the color of the dotted line to be drawn 
        g.setColor( lineColor );
  
        for ( j = 0; j <= x.length - 1; j++ )
        {   
            g.drawLine( m_plotAreaWidth / 10 + (int)x[j], 
                        9 * m_plotAreaHeight / 10 - (int)y[j],
                        m_plotAreaWidth / 10 + (int)x[j], 
                        9 * m_plotAreaHeight / 10 - (int)y[j] );
        }
    }

    /**
     * Draws the points only on the canvas.
     */
    private void fatPointsOnlyCurve( Graphics g, float[] x, float[] y, Color lineColor )
    {
        int j;
        int startx, starty;

        // Set the color of the dotted line to be drawn
        g.setColor( lineColor );

        for ( j = 0; j <= x.length - 1; j++ )
        {
            g.drawLine( m_plotAreaWidth / 10 + (int)x[j],
                        9 * m_plotAreaHeight / 10 - (int)y[j],
                        m_plotAreaWidth / 10 + (int)x[j],
                        9 * m_plotAreaHeight / 10 - (int)y[j] );

            g.drawLine( m_plotAreaWidth / 10 + (int)x[j]+1,
                        9 * m_plotAreaHeight / 10 - (int)y[j],
                        m_plotAreaWidth / 10 + (int)x[j]+1,
                        9 * m_plotAreaHeight / 10 - (int)y[j] );

            g.drawLine( m_plotAreaWidth / 10 + (int)x[j],
                        9 * m_plotAreaHeight / 10 - (int)y[j] + 1,
                        m_plotAreaWidth / 10 + (int)x[j],
                        9 * m_plotAreaHeight / 10 - (int)y[j] + 1 );

            g.drawLine( m_plotAreaWidth / 10 + (int)x[j] + 1,
                        9 * m_plotAreaHeight / 10 - (int)y[j] + 1,
                        m_plotAreaWidth / 10 + (int)x[j] + 1,
                        9 * m_plotAreaHeight / 10 - (int)y[j] + 1);
        }
    }

    /**
     * Draws the frame of the xy plot including scale and ticks.
     */
    private void draw_xy_frame( Graphics g, Color lineColor,
                                float xrange, float yrange )
    {
 
        Graphics2D g2d = (Graphics2D)g;
        
        // set the color of the line to be drawn
        g2d.setColor( lineColor );
        // creates a solid stroke with line width is 2
        Stroke stroke = new BasicStroke( 3f );
        g2d.setStroke( stroke );
        
        // Get current font
        Font currentFont = g2d.getFont( );
        String oldFontName = currentFont.getFontName( );
        int oldFontStyle = currentFont.getStyle( );
        int oldFontSize = currentFont.getSize( );
        
        // Set current font to be
        g2d.setFont( new Font( oldFontName, oldFontStyle, m_textFontSize ) );
        
        FontMetrics fm = g2d.getFontMetrics( );
        int stringHeight = fm.getHeight( );
        
        int slen = fm.stringWidth( m_title );
        
        g2d.drawString( m_title, (m_plotAreaWidth / 2) - (slen / 2 ), ( m_plotAreaHeight / 10 ) - 10 );
        // draw plot frame takes 0.8 of total screen
        g2d.setColor( m_backgroundFillColor );
        g2d.fillRect( m_plotAreaWidth / 10, m_plotAreaHeight / 10, 8 * m_plotAreaWidth / 10, 8 * m_plotAreaHeight / 10 );
        
        g2d.setColor( lineColor );
        // set pen to starting point
        int startx, starty;
        
        startx = m_plotAreaWidth / 10;
        starty = m_plotAreaHeight / 10;
        
        // draw x,y rectangle
        g2d.drawLine( startx, starty, m_plotAreaWidth / 10, m_plotAreaHeight / 10 );
        g2d.drawLine( m_plotAreaWidth / 10, m_plotAreaHeight / 10, 9 * m_plotAreaWidth / 10, m_plotAreaHeight / 10 );
        g2d.drawLine( 9 * m_plotAreaWidth / 10, m_plotAreaHeight / 10, 9 * m_plotAreaWidth / 10, 9 * m_plotAreaHeight / 10 );
        g2d.drawLine( 9 * m_plotAreaWidth / 10, 9 * m_plotAreaHeight / 10, m_plotAreaWidth / 10, 9 * m_plotAreaHeight / 10 );
        g2d.drawLine( m_plotAreaWidth / 10, 9 * m_plotAreaHeight / 10, m_plotAreaWidth / 10, m_plotAreaHeight / 10 );
        
        if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
        {
            // draw the x-ticks    
            drawxticks( m_frameColor, g, m_xyAutoPlotRange.getMinimumX( ), xrange );

            // draw the y-ticks
            drawyticks( m_frameColor, g, m_xyAutoPlotRange.getMinimumY( ), yrange );
        }
        else
        {

            // draw the x-ticks    
            drawxticks( m_frameColor, g, m_xyForcedPlotRange.getMinimumX( ), xrange );

            // draw the y-ticks
            drawyticks( m_frameColor, g, m_xyForcedPlotRange.getMinimumY( ), yrange );

        }
        
        // set current font size back
        super.setFont( new Font( oldFontName, oldFontStyle, oldFontSize ) );
        
        
    }
    /* screenready; */
    
    
    /**
     * Draws the linked list containing n lines of formatted String data
     * onto the plot.
     */
    private void drawLeftUpperInfoLines( Graphics g )
    {
        
        if ( m_curves != null )
        {
            Curve y = m_curves.get( 0 );
            float[] fy = y.getYvalues( );
        /*
            g.drawString( "val[0]  = " + fy[0], 
                          20 + ( m_plotAreaWidth / 10 ), 
                          ( m_plotAreaHeight / 10 ) + 20 );
         */
        }
        
        if ( m_leftUpperInfoDisplayLines.size( ) > 0 )
        {
            Font curFont = g.getFont( );
            String name = curFont.getFontName( );
            int oldStyle = curFont.getStyle( );
            int oldSize = curFont.getSize( );
            
            g.setFont( new Font( name, Font.BOLD, 14 ) );
            
            Graphics2D g2 = (Graphics2D) g;
            FontRenderContext frc = g2.getFontRenderContext();
            
            Iterator it = m_leftUpperInfoDisplayLines.iterator( );
            
            int ycntr = 0;
            int totalHeight = 0;
            int maxWidth = 0;
            
            while ( it.hasNext( ) )
            {
                String drawString = (String)it.next( );
                Rectangle2D bounds = g2.getFont( ).getStringBounds( drawString, frc );
                Rectangle ibounds = bounds.getBounds( );
                if ( bounds.getWidth( ) > maxWidth )
                {
                    maxWidth = ibounds.width;
                }
                totalHeight += ibounds.height;
            }
            
            Color curColor = g.getColor( );
            
            Color textBackGroudColor = new Color( 128, 128, 128, 50 );
            g.setColor( textBackGroudColor );
            
            g.fillRect( 15 + ( m_plotAreaWidth / 10 ), 
                        10 + ( m_plotAreaHeight / 10 ), 
                        maxWidth + 15, 
                        totalHeight + 8 );
            
            g.setColor( curColor );
            
            g.drawRect( 15 + ( m_plotAreaWidth / 10 ), 
                        10 + ( m_plotAreaHeight / 10 ), 
                        maxWidth + 15, 
                        totalHeight + 8 );
            
            Iterator it2 = m_leftUpperInfoDisplayLines.iterator( );
            
            while ( it2.hasNext( ) )
            {
                String drawString = (String)it2.next( );
                
                g.drawString( drawString, 
                              25 + ( m_plotAreaWidth / 10 ), 
                              ( m_plotAreaHeight / 10 ) + ( ycntr * 20 ) + 25 );
                
                ycntr++;
            }
            
            // Set font back to default size.
            g.setFont( new Font( name, oldStyle, oldSize ) );
            
        }
    }
    
    /**
     * Draws the linked list containing n lines of formatted String data
     * onto the plot.
     */
    private void drawLeftLowerInfoLines( Graphics g )
    {
        if ( m_leftLowerInfoDisplayLines.size( ) > 0 )
        {
            Iterator it = m_leftLowerInfoDisplayLines.iterator( );
            int ycntr = 0;
            while ( it.hasNext( ) )
            {
                String drawString = (String)it.next( );
                g.drawString( drawString, 
                              20 + ( m_plotAreaWidth / 10 ), 
                              ( 7 * m_plotAreaHeight / 10 ) + ( ycntr * 15 ) + 20 );
                ycntr++;
            }
        }
    }
    
    
    /**
     * Plots the vector of curves in one plot.
     * @param g
     */
    public synchronized void plot( Graphics g )
    {
        
        int   i, j, noplot = 1;
        float xscale, yscale, xrange, yrange;
        
        Color lineColor = Color.black;
        
        m_plotAreaWidth = this.getWidth( );
        m_plotAreaHeight = this.getHeight( );
        
        if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
        {
            autoDeterminePlotRanges( );
        }
        else
        {
            autoDeterminePlotRanges( );
            forcedPlotRanges(0,5);
        }
        
        if ( m_xyAutoPlotRange.getMinimumY( ) == m_xyAutoPlotRange.getMaximumY( ) )
        {
            g.drawString( "Ymin = Ymax, no plot", m_plotAreaWidth / 2, m_plotAreaHeight / 2 );
            noplot = 0;
        }
        
        if ( m_xyAutoPlotRange.getMinimumX( ) == m_xyAutoPlotRange.getMaximumX( ) )
        {
            g.drawString( "Xmin = Xmax, no plot", m_plotAreaWidth / 2, m_plotAreaHeight / 2 );
            noplot = 0;
        }
        
        if ( noplot == 1 )
        {
            
            int labelStartX = (int)( 9f * m_plotAreaWidth / 10f ) - computeCurveLabelStartPosition( g ) - 5;
            
            if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
            {
                xrange = Math.abs( m_xyAutoPlotRange.getMaximumX( ) - m_xyAutoPlotRange.getMinimumX( ) );
                yrange = Math.abs( m_xyAutoPlotRange.getMaximumY( ) - m_xyAutoPlotRange.getMinimumY( ) );
            }
            else
            {
                xrange = Math.abs( m_xyForcedPlotRange.getMaximumX( ) - m_xyForcedPlotRange.getMinimumX( ) );
                yrange = Math.abs( m_xyForcedPlotRange.getMaximumY( ) - m_xyForcedPlotRange.getMinimumY( ) );                
            }
            
            xscale = ( 10 * xrange ) / ( 8 * m_plotAreaWidth );
            yscale = ( 10 * yrange ) / ( 8 * m_plotAreaHeight );
            
            Iterator iter = m_curves.iterator( );
            int cntr = 0;
            
            Color backup = g.getColor( );
            
            while ( iter.hasNext( ) )
            {    
                Curve c = (Curve)iter.next( );
                
                if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
                {
                    if ( m_displayXReversed == false )
                    {
                        c.setScaledCurve( m_xyAutoPlotRange.getMinimumX( ), xscale, m_xyAutoPlotRange.getMinimumY( ), yscale );   
                        if ( c.getLabel( ) != null )
                        {
                            g.setColor( c.getColor( ) );
                            g.drawString( c.getLabel( ), labelStartX , (int)( 0.15 * m_plotAreaHeight + cntr * 15 ) );
                            cntr++;
                        }
                    }
                    else
                    {
                        c.setScaledCurve( m_xyAutoPlotRange.getMaximumX( ), -xscale, m_xyAutoPlotRange.getMinimumY( ), yscale );   
                        if ( c.getLabel( ) != null )
                        {
                            g.setColor( c.getColor( ) );
                            g.drawString( c.getLabel( ), labelStartX , (int)( 0.15 * m_plotAreaHeight + cntr * 15 ) );
                            cntr++;
                        }
                    }
                }
                else
                {  
                    // here is the bug somewhere !!!
                    if ( m_displayXReversed == false )
                    {
                        c.setScaledCurve( m_xyForcedPlotRange.getMinimumX( ), xscale, m_xyForcedPlotRange.getMinimumY( ), yscale );
                        if ( c.getLabel( ) != null )
                        {
                            g.setColor( c.getColor( ) );
                            g.drawString( c.getLabel( ), labelStartX , (int)( 0.15 * m_plotAreaHeight + cntr * 15 ) );
                            cntr++;
                        }
                    }
                    else
                    {
                        c.setScaledCurve( m_xyForcedPlotRange.getMaximumX( ), -xscale, m_xyForcedPlotRange.getMinimumY( ), yscale );
                        if ( c.getLabel( ) != null )
                        {
                            g.setColor( c.getColor( ) );
                            g.drawString( c.getLabel( ), labelStartX, (int)( 0.15 * m_plotAreaHeight + cntr * 15 ) );
                            cntr++;
                        }
                    }
                }
            }
        
            g.setColor( backup );
            
            draw_xy_frame( g, lineColor, xrange, yrange );
            
            draw_x_title( g );
            
            draw_y_title( g );
            
            drawLeftUpperInfoLines( g );
            
            drawLeftLowerInfoLines( g );
            
            Iterator iterator = m_curves.iterator( );
            
            while ( iterator.hasNext( ) )
            {
                Curve c = (Curve)iterator.next( );
                
                if ( c.getStyle( ) == Curve.THIN_SOLID_LINE )
                {
                    solidCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ), 1f );
                }
                
                if ( c.getStyle( ) == Curve.NORMAL_LINE )
                {
                    solidCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ), 2f );
                }
                
                if ( c.getStyle( ) == Curve.THICK_SOLID_LINE )
                {
                    solidCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ), 4f );
                }
                
                if ( c.getStyle( ) == Curve.DOTTED_LINE )
                {
                    dottedCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ) );
                }
                if ( c.getStyle( ) == Curve.POINTS_ONLY )
                {
                    pointsOnlyCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ) );
                }
                
                if ( c.getStyle( ) == Curve.FAT_SOLID_LINE )
                {
                    fatSolidCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ) );
                }
                if ( c.getStyle( ) == Curve.FAT_POINTS_ONLY )
                {
                    fatPointsOnlyCurve( g, c.getScaledXvalues( ), c.getScaledYvalues( ), c.getColor( ) );
                }
            }
        }
    }
    
    /**
     * Computes the size of the largest label string.
     */
    private int computeCurveLabelStartPosition( Graphics g )
    {
        
        FontMetrics fm = g.getFontMetrics( );
        
        Iterator iterator = m_curves.iterator( );
        int maxLabelLength = 0;
        while ( iterator.hasNext( ) )
        {
            int labelLength;
            Curve c = (Curve)iterator.next( );
            if ( c.getLabel( ) != null  )
            {
                labelLength = fm.stringWidth( c.getLabel( ) );
                if ( labelLength > maxLabelLength )
                {
                    maxLabelLength = labelLength;
                }
            }
            
        }
        return maxLabelLength;
    }
    
    /**
     * Determines the minimum and maximum x,y values for the vector
     * of curves to be displayed.
     */
    protected void autoDeterminePlotRanges( )
    {
        m_xyAutoPlotRange.reset( );
        autoDetermineRanges( );
    }
    
    /**
     * Determines the default ranges which are used in auto range setting.
     */
    private void autoDetermineRanges( )
    {
        Iterator it = m_curves.iterator( );
        
        float min_x = Float.MAX_VALUE;
        float max_x = -Float.MAX_VALUE;
        float min_y = Float.MAX_VALUE;
        float max_y = -Float.MAX_VALUE;
        
        while ( it.hasNext( ) )
        {
            Curve c = (Curve)it.next( );
            min_x = c.getMinXValue( );
            max_x = c.getMaxXValue( );       
            min_y = c.getMinYValue( );
            max_y = c.getMaxYValue( );
            
            if ( min_x < m_xyAutoPlotRange.getMinimumX( ) )
            {
                m_xyAutoPlotRange.setMinimumX( min_x );
            }
            if ( min_y < m_xyAutoPlotRange.getMinimumY( ) )
            {
                m_xyAutoPlotRange.setMinimumY( min_y );
            }

            if ( max_x > m_xyAutoPlotRange.getMaximumX( ) )
            {
                m_xyAutoPlotRange.setMaximumX( max_x );
            }

            if ( max_y > m_xyAutoPlotRange.getMaximumY( ) )
            {
                m_xyAutoPlotRange.setMaximumY( max_y );
            }
        }
    }
    
    /**
     * If the user wants to set a non default plotting range 
     * this method is called.
     * @param min
     * @param max
     */
    public void forcedPlotRanges( int min, int max )
    {

        if ( m_useNiceProtonDefaults == true )
        {
            // In this we should force the horizontal range to 0 .. 5 ppm range
            m_xyForcedPlotRange.setMinimumX( min );
            m_xyForcedPlotRange.setMaximumX( max );
            m_xyForcedPlotRange.setMinimumY( m_xyAutoPlotRange.getMinimumY( ) );
            m_xyForcedPlotRange.setMaximumY( m_xyAutoPlotRange.getMaximumY( ) );
        }
    }
    
    /**
     * Defines the font size for the x and y axis. 
     * @param fontsize 
     */
    public void setTextFontSize( int fontsize )
    {
        m_textFontSize = fontsize;
    }
    
    /**
     * Defines the backgound color of the plottable area of the xyplot.
     * @param color
     * @param translucency
     */
    public void setbackgroundFillColor( Color color, int translucency )
    {
        m_backgroundFillColor = new Color( color.getRed( ), color.getGreen( ), 
                                           color.getBlue( ), translucency );
    }
    
    
    /**
     * Draws the large x-ticks and values.
     */
    private void drawxticks( Color color, Graphics g, float xmin, float xrange )
    {
        String	strbuf;
        float	xlargeticksize, currentx;
        short	k, jumpout;
        int	xcoord;

        Graphics2D g2d = (Graphics2D)g;
        
        // Set the color of the line to be drawn
        g2d.setColor( color );
        // Creates a solid stroke with line width is 2
        Stroke stroke = new BasicStroke( 2f );
        g2d.setStroke( stroke ); 
        
        Font currentFont = g2d.getFont( );
        String oldFontName = currentFont.getFontName( );
        int oldFontStyle = currentFont.getStyle( );
        int oldFontSize = currentFont.getSize( );
        // set current font to be
        g2d.setFont( new Font( oldFontName, oldFontStyle, m_textFontSize ) );
           
        jumpout = 0;
        
        float power = (float)Math.floor( (float)Math.log( xrange ) / Math.log( 10 ) );

        xlargeticksize = (float)Math.pow( 10.0, power )/2;
	
        int numberOfStringsToDraw = (int)(xrange/xlargeticksize);
        
        if ( numberOfStringsToDraw > 10 )
        {
            xlargeticksize = (float)Math.pow( 10.0, power );
        }
        
        k = 1;

        currentx = 0;
	
        String strbuffer;

        while ( jumpout == 0 )
        {
            currentx = (float)Math.floor( ( xmin + k * xlargeticksize ) / xlargeticksize ) * xlargeticksize;
            g2d.setStroke( stroke );
            if ( currentx <= xmin + xrange )
            {
                
                if ( m_displayXReversed == false )
                {
                    xcoord = m_plotAreaWidth / 10 + (int)( Math.abs( ( currentx - xmin ) / xrange ) * ( 8 * m_plotAreaWidth / 10 ) );
                }
                else
                {
                    xcoord = 9 * m_plotAreaWidth / 10 - (int)( Math.abs( ( currentx - xmin ) / xrange ) * ( 8 * m_plotAreaWidth / 10 ) );
                }
                
                NumberFormat format;
                
                if ( ( xrange > 1f ) && ( xrange < 9999f ) )
                {
                    format = new DecimalFormat( "#.##" );
                    strbuffer = format.format( currentx  );
                }
                else
                {
                   format = new DecimalFormat( "0.0E0" );
                   strbuffer = format.format( currentx  );
                }
                    
                FontMetrics fm = g2d.getFontMetrics( );
                int slen = fm.stringWidth( strbuffer );
                
                // CSize mysize = m_graphics->GetOutputTextExtent(strbuffer, strbuffer.GetLength());
                
                g2d.drawString( strbuffer, xcoord - slen / 2, ( 9 * m_plotAreaHeight / 10 ) + 15 );

                g2d.drawLine( xcoord, m_plotAreaHeight / 10, xcoord, m_plotAreaHeight / 10 + 7 );
                g2d.drawLine( xcoord, 9 * m_plotAreaHeight / 10, xcoord, 9 * m_plotAreaHeight / 10 - 7 );
 
                float[] dashingPattern1 = {1f, 1f};
                Stroke stroke1 = new BasicStroke( 2f, BasicStroke.CAP_BUTT,
                                                 BasicStroke.JOIN_MITER, 1.0f, dashingPattern1, 2.0f );
                g2d.setStroke( stroke1 );
                g2d.setColor( Color.WHITE );
                g2d.drawLine( xcoord, (m_plotAreaHeight / 10) + 7, xcoord, (9 * m_plotAreaHeight / 10 - 7) );
                g2d.setColor( color );              
                k++;
            }
            else	
            {
                jumpout = 1;
            }
        }
                
        switch ( k - 1 )
        {
            case 1:	drawsmallxticks( color, g, xmin, xrange, 10, xlargeticksize );
			break;
            case 2:	drawsmallxticks( color, g, xmin, xrange,  5, xlargeticksize );
			break;
            case 3:	drawsmallxticks( color, g, xmin, xrange,  5, xlargeticksize );
			break;
            case 4:	drawsmallxticks( color, g, xmin, xrange,  2, xlargeticksize );
			break;
            case 5:	drawsmallxticks( color, g, xmin, xrange,  2, xlargeticksize );
			break;
        }
        
        super.setFont( new Font( oldFontName, oldFontStyle, oldFontSize ) );
        
    }

    /**
     * Draws the small x-ticks.
     */
    private void drawsmallxticks( Color color, Graphics g, float xmin, float xrange, 
				  int nsmall,
				  float xlargeticksize )
    {
        int	xcoord, k;
        float	currentx;
        short	jumpout;

        jumpout = 0;
        k = 1;
        
        Graphics2D g2d = (Graphics2D)g;
        
        // Set the color of the line to be drawn
        g2d.setColor( color );
        // Creates a solid stroke with line width is 2
        Stroke stroke = new BasicStroke( 1f );
        g2d.setStroke( stroke ); 
        
        while ( jumpout == 0 )
        {
            currentx = (float)Math.floor( nsmall * (xmin + k * xlargeticksize/nsmall) / xlargeticksize ) * xlargeticksize / nsmall;
            if ( currentx <= xmin + xrange )
            {
                
                if ( m_displayXReversed == false )
                {
                    xcoord = m_plotAreaWidth / 10 + (int)(Math.abs( ( currentx - xmin ) / xrange ) * ( 8 * m_plotAreaWidth / 10 ) );
                }
                else
                {
                     xcoord = 9 * m_plotAreaWidth / 10 - (int)(Math.abs( ( currentx - xmin ) / xrange ) * ( 8 * m_plotAreaWidth / 10 ) );
                }
                
                g2d.drawLine( xcoord, m_plotAreaHeight / 10, xcoord, m_plotAreaHeight / 10 + 5 );
                g2d.drawLine( xcoord, 9 * m_plotAreaHeight / 10, xcoord, 9 * m_plotAreaHeight / 10 - 5 );
                k++;
            }
            else
            {
            	jumpout = 1;
            }
        }
        
    }

    /**
     * Draws the large y-ticks and values.
     */
    private void drawyticks( Color color, Graphics g, float ymin, float yrange )
    {
        String	strbuf;
        float   ylargeticksize, currenty;
        short	k, jumpout;
        int	ycoord;

        jumpout = 0;
	
               
        Graphics2D g2d = (Graphics2D)g;
        
        // Set the color of the line to be drawn
        g2d.setColor( color );
        // Creates a solid stroke with line width is 2
        Stroke stroke = new BasicStroke( 2f );
        g2d.setStroke( stroke ); 
        
        Font currentFont = g2d.getFont( );
        String oldFontName = currentFont.getFontName( );
        int oldFontStyle = currentFont.getStyle( );
        int oldFontSize = currentFont.getSize( );
        // set current font to be
        g2d.setFont( new Font( oldFontName, oldFontStyle, m_textFontSize ) );
     
        
        float power = (float)Math.floor( Math.log( yrange ) / Math.log( 10 ) );

        ylargeticksize = (float)Math.pow( 10.0, power )/2;

        int numberOfStringsToDraw = (int)( yrange/ylargeticksize );
        
        if ( numberOfStringsToDraw > 10 )
        {
            ylargeticksize = (float)Math.pow( 10.0, power );
        }
            
        k = 1;
        currenty = 0;
        String strbuffer;

        while ( jumpout == 0 )
        {
            
            currenty = (float)Math.floor( ( ymin + k * ylargeticksize ) / ylargeticksize ) * ylargeticksize;
            
            if ( currenty <= ymin + yrange )
            {
                ycoord = 9 * m_plotAreaHeight / 10 -  (int)( Math.abs( ( currenty - ymin ) / yrange ) * ( 8 * m_plotAreaHeight / 10 ) );
                
                NumberFormat format;
                
                if ( ( yrange > 1f ) && ( yrange < 9999f ) )
                {
                    format = new DecimalFormat( "#.##" );
                    strbuffer = format.format( currenty  );
                }
                else
                {
                   format = new DecimalFormat( "0.0E0" );
                   strbuffer = format.format( currenty  );
                }
                
                strbuffer = format.format( currenty  );
                FontMetrics fm = g2d.getFontMetrics( );
                int stringHeight = fm.getHeight( );
                int slen = fm.stringWidth( strbuffer );
                
                // CSize mysize = m_graphics->GetOutputTextExtent(strbuffer, strbuffer.GetLength());
                g2d.drawString( strbuffer, (m_plotAreaWidth / 10 ) - slen - 5, ycoord + 5 );

                g2d.drawLine( m_plotAreaWidth / 10, ycoord, m_plotAreaWidth / 10   + 10, ycoord );
                g2d.drawLine( 9 * m_plotAreaWidth / 10, ycoord, 9 * m_plotAreaWidth / 10 - 10, ycoord );
                
                float[] dashingPattern1 = {1f, 1f};
                Stroke stroke1 = new BasicStroke( 2f, BasicStroke.CAP_BUTT,
                                                  BasicStroke.JOIN_MITER, 
                                                  1.0f, dashingPattern1, 2.0f );
                g2d.setStroke( stroke1 );
                g2d.setColor( Color.WHITE );
                g2d.drawLine( (m_plotAreaWidth / 10) + 10, ycoord,  (9 * m_plotAreaWidth / 10) - 10, ycoord );
                g2d.setColor( color );    
                
                g2d.setStroke( stroke );
                
                k++;
            }
            else
            {
                jumpout = 1;
            }
        }
        
        switch ( k - 1 )
        {
            case 1:	drawsmallyticks( color, g, ymin, yrange, 10, ylargeticksize );
			break;
            case 2:	drawsmallyticks( color, g, ymin, yrange,  5, ylargeticksize );
			break;
            case 3:	drawsmallyticks( color, g, ymin, yrange, 5, ylargeticksize );
			break;
            case 4:	drawsmallyticks( color, g, ymin, yrange, 2, ylargeticksize );
			break;
            case 5:	drawsmallyticks( color, g, ymin, yrange, 2, ylargeticksize );
			break;
        }
        g2d.setFont( new Font( oldFontName, oldFontStyle, oldFontSize ) ); 
    }
    
    /**
     * Draws the small y-ticks.
     */
    private void drawsmallyticks( Color color, Graphics g, float ymin, float yrange, 
                                  int nsmall,
                                  float ylargeticksize )
    {
        int ycoord, k;
        float currenty;
        short jumpout;

        jumpout = 0;
        k = 1;
                     
        Graphics2D g2d = (Graphics2D)g;
        
        // Set the color of the line to be drawn
        g2d.setColor( color );
        // Creates a solid stroke with line width is 2
        Stroke stroke = new BasicStroke( 1f );
        g2d.setStroke( stroke );   
        
        while ( jumpout == 0 )
        {
            currenty = (float)Math.floor( nsmall * (ymin + k * ylargeticksize/nsmall) / ylargeticksize ) * ( ylargeticksize / nsmall );
            if ( currenty <= ymin + yrange )
            {
                // here is something wrong
                ycoord = ( 9 * m_plotAreaHeight / 10 ) - (int)(Math.abs( ( currenty - ymin ) / yrange ) * ( 8 * m_plotAreaHeight / 10 ) );
                g2d.drawLine( m_plotAreaWidth / 10, ycoord, m_plotAreaWidth / 10 + 7, ycoord );
                g2d.drawLine( 9 * m_plotAreaWidth / 10, ycoord, 9 * m_plotAreaWidth / 10 - 7, ycoord );
                k++;             
            }
            else
            {
                jumpout = 1;
            }
        }
    }
    
    /**
     * Allows the redefinition of the background color of the component.
     * @param background
     */
    public void setBackgroundColor( Color background )
    {
        m_backgroundColor = background;
    }

    /**
     * Adds a min max mean range to drawn.
     * @param min
     * @param max
     * @param mean
     * @param color
     * @param refresh
     * @return
     */
    public boolean addMinMaxMeanRange( Curve min, Curve max, Curve mean, Color color, boolean refresh )
    {
        m_minCurve      = min;
        m_maxCurve      = max;
        m_meanCurve     = mean;
        
        m_rangeColor    = color;

        m_curves.add( min );
        m_curves.add( max );
        m_curves.add( mean );

        return true;
    }


    /**
     * Adds a curve to the list of curves to be drawn:
     * The curve can only be added if the dimension of the curves
     * are the same, and the x-range is the same.
     * @param curve
     * @param refresh
     * @return 
     */ 
    public synchronized boolean addCurve( Curve curve, boolean refresh )
    {
        if ( m_curves.isEmpty( ) ) 
        {
            if ( curve.getSize( ) > 1 )
            {
                m_curveSize = curve.getSize( );
                m_curves.add( curve );
                if ( refresh == true )
                {
                    refresh( );
                }
            }
            else
            {
                return false;
            }
        }
        else
        {
            //if ( m_curveSize == curve.getSize( ) )
//            {
                m_curves.add( curve );
                if ( refresh == true )
                {
                    refresh( );
                }
//            }
//            else
//            {
//                return false;
//            }
        }
        return true;
    }
    
    /**
     * Deletes a curve from the list of the vector of curves to be drawn.
     * @param curveNumber
     * @return 
     */ 
    public synchronized boolean deleteCurve( int curveNumber )
    {
        if ( m_curves.size( ) > 0 ) 
        {
            if ( curveNumber <= m_curves.size( ) )
            {
                m_curves.removeElementAt( curveNumber );
                if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
                {
                    autoDeterminePlotRanges( );
                }
                else
                {
                    forcedPlotRanges( 0, 5 );
                }
                refresh( );
                return true;
            }
        }
        return false;
    }
    
    /**
     * Deletes all curves from the list of curves to be drawn.
     */ 
    public synchronized void deleteAllCurves( )
    {
        m_curves.clear( );
        if ( m_XYDefaultRangeMenuItem.isSelected( ) == true )
        {
            autoDeterminePlotRanges( );
        }
        else
        {
            forcedPlotRanges(0,5);
        }
        refresh( );
    }
    
    /**
     * Does the painting of this component.
     * @param g
     */
    @Override
    public synchronized void paintComponent( Graphics g )
    {
        super.paintComponent( g );


        g.setColor( Color.white );
        g.fillRect( 0, 0, getWidth( ), getHeight( ) );
        g.setColor( Color.black );
        
        if ( m_curves.size( ) > 0 )
        {
            if ( g != null )
            {
                g.setColor( Color.white );
                g.fillRect( 0, 0, getWidth( ), getHeight( ) );
                g.setColor( Color.black );
                plot( g );
                drawInfoString( g );
            }
        }
        else
        {
            if ( g != null )
            {
                String str = "No curves to draw";
                FontMetrics fm = g.getFontMetrics( );
                int slen = fm.stringWidth( str ); 
                g.drawString( str , ( getWidth( ) - slen ) / 2, getHeight( ) / 2 );
            }
        }
        
        if ( this.m_plotSliderList.size( ) > 0 )
        {
            Iterator it = m_plotSliderList.iterator( );
            
            while ( it.hasNext( ) )
            {
                ((XYPlotSlider)it.next( ) ).drawSlider( g );
            }
        }

        if ( overPlotWithRangeAsWell( ) == true )
        {

            overPlotRange( g );

            // force another plot to ensure visibility of curves over range
//            g.setColor( Color.white );
//            g.fillRect( 0, 0, getWidth( ), getHeight( ) );
//            g.setColor( Color.black );

            if ( m_curves.size( ) > 0 )
            {
                if ( g != null )
                {
//                    g.setColor( Color.white );
//                    g.fillRect( 0, 0, getWidth( ), getHeight( ) );
                    g.setColor( Color.black );
                    plot( g );
                    drawInfoString( g );
                }
            }
            else
            {
                if ( g != null )
                {
                    String str = "No curves to draw";
                    FontMetrics fm = g.getFontMetrics( );
                    int slen = fm.stringWidth( str );
                    g.drawString( str , ( getWidth( ) - slen ) / 2, getHeight( ) / 2 );
                }
            }

            if ( this.m_plotSliderList.size( ) > 0 )
            {
                Iterator it = m_plotSliderList.iterator( );

                while ( it.hasNext( ) )
                {
                    ((XYPlotSlider)it.next( ) ).drawSlider( g );
                }
            }
        }
        
        drawSplineNodes( g );
        
        
    }

    /**
     * Over plots the range.
     * @return
     */
    private boolean overPlotWithRangeAsWell( )
    {
        boolean minimumRangeDefined = false;
        boolean maximumRangeDefined = false;

        for ( int i = 0; i < m_curves.size( ); i++ )
        {
            Curve crv = m_curves.get( i );

            // System.out.println( "Curve label : " + crv.getLabel( ) );
            if ( crv.getLabel( ) != null )
            {
                if ( crv.getLabel( ).compareTo( "minimum range" ) == 0 )
                {
                    minimumRangeDefined = true;
                    m_minCurve = crv;
                }
                if ( crv.getLabel( ).compareTo( "maximum range" ) == 0 )
                {
                    maximumRangeDefined = true;
                    m_maxCurve = crv;
                }
            }
        }

        if ( ( minimumRangeDefined == true ) && ( maximumRangeDefined == true ) )
        {
            // System.out.println( "§§§§§§§§§§§§§§§§§§§§§§§§   XYPlot::overPlotWithRangeAsWell( ) return  true! " );
            return true;
        }
        else
        {
            // System.out.println( "§§§§§§§§§§§§§§§§§§§§§§§§§  XYPlot::overPlotWithRangeAsWell( ) return false! " );
            return false;
        }
    }


    /**
     * Allows an update of the component.
     */
    public void refresh( )
    {
        if ( getGraphics( ) != null )
        {
            paintComponent( getGraphics( ) );
            updateUI( );
        }
    }
        
    /**
     * Draws the info string on the XY plot.
     */
    private void drawInfoString( Graphics g )
    {
        Color curColor = g.getColor( );

        g.setColor( m_infoStringColor );
        g.drawString( m_infoString, getWidth( ) / 10 + 20, getHeight( ) / 10 + 30 );
        g.setColor( curColor );
 
    }
    
    /**
     * Sets the information string.
     * @param info
     */
    public void setInfoString( String info )
    {
        m_infoString = info;
    }


    /**
     * Sets the information string.
     * @param info
     * @param color
     */
    public void setInfoString( String info, Color color )
    {
        m_infoString      = info;
        m_infoStringColor = color;
    }

    /**
     * Returns the info string.
     * @return 
     */
    public String getInfoString( )
    {
        return m_infoString;
    }
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        // Handle responses if range selection menu item was selected
        if ( sender == m_XYPlotRangeSelectorMenuItem )
        {

            m_xyForcedPlotRange = new XYPlotRange( m_xyAutoPlotRange.getMinimumX( ), m_xyAutoPlotRange.getMaximumX( ),
                                                   m_xyAutoPlotRange.getMinimumY( ), m_xyAutoPlotRange.getMaximumY( ) );

            m_plotRangeSelector = new XYPlotRangeSelector( "Set Horizontal and Vertical Plot Ranges", m_parent );
            m_plotRangeSelector.addActionListener( this );
            refresh( );
        }
        
        if ( sender == m_plotRangeSelector )
        {
            if ( e.getID( ) == XYPlotRangeSelector.HORIZONTAL_HALF_RANGE_EVENT )
            {
                m_xyForcedPlotRange.setXRange( m_xyForcedPlotRange.getXRange( ) / 2.0f );
                refresh( );
            }
            
            if ( e.getID( ) == XYPlotRangeSelector.HORIZONTAL_DOUBLE_RANGE_EVENT )
            {
                m_xyForcedPlotRange.setXRange( m_xyForcedPlotRange.getXRange( ) * 2.0f );
                refresh( );
            }

            if ( e.getID( ) == XYPlotRangeSelector.VERTICAL_HALF_RANGE_EVENT )
            {
                m_xyForcedPlotRange.setYRange( m_xyForcedPlotRange.getYRange( ) / 2.0f );
                refresh( );
            }

            if ( e.getID( ) == XYPlotRangeSelector.VERTICAL_DOUBLE_RANGE_EVENT )
            {
                m_xyForcedPlotRange.setYRange( m_xyForcedPlotRange.getYRange( ) * 2.0f );
                refresh( );
            }
            
            if ( e.getID( ) == XYPlotRangeSelector.HORIZONTAL_CENTER_EVENT )
            {   
                m_xyForcedPlotRange.setXCenter( m_plotRangeSelector.getXCenter( ) );
                refresh( );
            }
            
            if ( e.getID( ) == XYPlotRangeSelector.VERTICAL_CENTER_EVENT )
            {
                m_xyForcedPlotRange.setYCenter( m_plotRangeSelector.getYCenter( ) );
                refresh( );
            }
            
            if ( e.getID( ) == XYPlotRangeSelector.CANCEL_EVENT )
            {
                m_XYDefaultRangeMenuItem.setSelected( true );
                refresh( );
            }
        }
        
        if ( sender == m_XYDefaultRangeMenuItem )
        {
            m_xyForcedPlotRange = new XYPlotRange( m_xyAutoPlotRange.getMinimumX( ), m_xyAutoPlotRange.getMaximumX( ),
                                                   m_xyAutoPlotRange.getMinimumY( ), m_xyAutoPlotRange.getMaximumY( ) );
            refresh( );
        }
    }
    
    /**
     * Set the reference to the StatusPanel of the application.
     * @param statusPanel
     */
    public void setStatusPanel( StatusPanel statusPanel )
    {
        m_statusPanel = statusPanel;   
    }
    
    /**
     * Sets the x-axis display reverse (true) or normal (false).
     * @param doReverse
     */
    public void setDisplayXReversed( boolean doReverse )
    {
        m_displayXReversed = doReverse;
    }

    /**
     * Sets the x-axis display reverse (true) or normal (false).
     * @return 
     */
    public boolean isDisplayXReversed(  )
    {
        return m_displayXReversed;
    }
    
    /**
     * Sets the title of the xyplot.
     * @param title
     */
    public void setTitle( String title )
    {
        m_title = title;
    }
    
    /**
     * Returns the title of the xyplot.
     * @return 
     */
    public String getTitle( )
    {
        return m_title;
    }
    
    /**
     * Adds one information line to the list of information lines.
     * @param infoLine
     */
    public void addLeftUpperInformationLine( String infoLine )
    {
        m_leftUpperInfoDisplayLines.add( infoLine );
    }
    
    /**
     * Adds one information line to the list of information lines.
     * @param infoLine
     */
    public void addLeftLowerInformationLine( String infoLine )
    {
        m_leftLowerInfoDisplayLines.add( infoLine );
    }
    
    /**
     * Adds one information line to the list of information lines.
     */
    public void clearAllLeftUpperInformationLines( )
    {
        m_leftUpperInfoDisplayLines.clear( );
    }

    /**
     * Adds one information line to the list of information lines.
     */
    public void clearAllLeftLowerInformationLines( )
    {
        m_leftUpperInfoDisplayLines.clear( );
    }

    /**
     * Adds a horizontal or vertical XYPlotSlider to the list of sliders
     * and displays it.
     * @param value
     * @param orientation
     */
    public void addXYPlotSlider( int value, int orientation )
    {
        Graphics g =  getGraphics( );
        
        XYPlotSlider slider = new XYPlotSlider( this, value, orientation  );
        
        m_plotSliderList.add( slider );
    }
    

    /**
     * Adds a horizontal or vertical XYPlotSlider to the list of sliders
     * and displays it.
     */
    public void addTwoHorizontalXYPlotSliders( )
    {
        Graphics g =  getGraphics( );
        
        int startValue1 = ( 4 * getWidth( ) / 10 );
        int startValue2 = ( 6 * getWidth( ) / 10 );
        
        XYPlotSlider slider1 = new XYPlotSlider( this, startValue1, XYPlotSlider.VERTICAL_SLIDER );
        
        m_plotSliderList.add( slider1 );

        XYPlotSlider slider2 = new XYPlotSlider( this, startValue2, XYPlotSlider.VERTICAL_SLIDER );
        
        m_plotSliderList.add( slider2 );
    }
    
    
    /**
     * Returns a array with slider positions
     * Slider position varies from 0 till this.getWidth().
     * @return 
     */
    public int[] getSliderPositions( )
    {
        
        int[] positions = new int[ m_plotSliderList.size( ) ];
        
        Iterator it = this.m_plotSliderList.iterator( );
        int i = 0;
        while ( it.hasNext( ) )
        {
            XYPlotSlider slider = (XYPlotSlider)it.next( );
            positions[i++] = slider.getCurrentPosition( );
        }
        
        return positions;        
    }
    
    
    /**
     * Returns the selected slider of the control.
     * @return 
     */
    public XYPlotSlider getSelectedSlider( )
    {
        XYPlotSlider slider;
        
        Iterator it = this.m_plotSliderList.iterator( );
        while ( it.hasNext( ) )
        {
            slider = (XYPlotSlider)it.next( );
            
            if ( slider.isSelected( )  == true )
            {
                return slider;
            }
        }
        
        return null;
    }

    /**
     * Returns the selected slider of the control.
     * @param xCoord
     * @param yCoord
     */
    protected void setSelectedSlider( int xCoord, int yCoord )
    {
        XYPlotSlider slider;
        
        m_sliderIsSelected = false;
        
        Iterator it = this.m_plotSliderList.iterator( );
        while ( it.hasNext( ) )
        {
            slider = (XYPlotSlider)it.next( );
            if ( slider.isSliderSelected( xCoord, yCoord )  == true )
            {
                slider.setSelected( true );
                m_sliderIsSelected = true;
            }
        }
    }

    /**
     * Returns the selected slider of the control.
     * @param xCoord
     * @param yCoord
     */
    protected void moveSelectedSlider( int xCoord, int yCoord )
    {
       XYPlotSlider slider;
        
        Iterator it = this.m_plotSliderList.iterator( );
        while ( it.hasNext( ) )
        {
            slider = (XYPlotSlider)it.next( );
            if ( slider.isSelected( )  == true )
            {
                slider.setNewSliderPosition( xCoord, yCoord );
            }
        }
        
        //refresh( );
    }
    
   /**
    * Called when the mouse button is clicked.
     * @param mouseEvent
    */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
        // sets a slider selected is mouse pointer is on appropriate postion
        setSelectedSlider( mouseEvent.getX( ), mouseEvent.getY( ) );
        refresh( );
    }
    
    /**
     * Called when the mouse is dragged.
     * @param mouseEvent
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
        moveSelectedSlider( mouseEvent.getX( ), mouseEvent.getY( ) );
        
        // only repaint where the line was moved
        repaint( mouseEvent.getX( ) - 10, 0, 20, this.getHeight( ) );
    }
    
    /**
     * Called when the mouse enters the control.
     * @param mouseEvent
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse exits.
     * @param mouseEvent
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse button is released.
     * @param mouseEvent
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse button is pressed.
     * @param mouseEvent
     */    
    @Override
    public void mousePressed( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse button is released.
     * @param mouseEvent
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Sets the ForcedPlotRange to true and defines a nice ppm display range
     * for clinical in vivo spectra (method used for jMRUI).
     */
    public void set_Default_1H_ForcedPlotRange( )
    {
        m_XYPlotRangeSelectorMenuItem.setSelected( true );
        m_useNiceProtonDefaults = true;
    }
    
    /**
     * 
     * Returns the set title of the x-axis.
     * 
     * @return 
     */
    public String getXTitle( )
    {
        return m_xtitle;
    }
    /**
     * 
     * Defines the title to be set at the x-axis.
     * 
     * @param xtitle 
     */
    public void setXTitle( String xtitle) 
    {
        m_xtitle = xtitle;
    }

     /**
     * 
     * Returns the set title of the y-axis.
     * 
     * @return 
     */
    public String getYTitle( ) 
    {
        return m_ytitle;
    }

    /**
     * 
     * Defines the title to be set at the y-axis.
     * 
     * @param ytitle 
     */
    public void setYTitle( String ytitle ) 
    {
        m_ytitle = ytitle;
    }
    
    /**
     * Does the actual drawing of the y title string. 
     * @param g 
     */
    private void draw_y_title( Graphics g ) 
    {
        if ( m_ytitle != null )
        {
            Graphics2D g2d = (Graphics2D)g;
            
            AffineTransform defaultAt = g2d.getTransform( );
         
            // rotates the coordinate by 90 degree counterclockwise
            AffineTransform at = new AffineTransform( );
            at.rotate( -Math.PI / 2 );
            g2d.setTransform( at );
            
            FontMetrics fm = g2d.getFontMetrics( );
            int slen = fm.stringWidth( m_ytitle );
            
            g2d.drawString( m_ytitle, 
                            - ( 5 * m_plotAreaHeight / 10 ) - ( slen / 2 ), 
                            ( m_plotAreaWidth / 10 ) - 70 );
            
            g2d.setTransform( defaultAt );
        }
    }
    
    /**
     * Does the actual drawing of the x title string. 
     * @param g 
     */
    private void draw_x_title( Graphics g ) 
    {
        if ( m_xtitle != null )
        {
            Graphics2D g2d = (Graphics2D)g;
                   
            FontMetrics fm = g2d.getFontMetrics( );
            int slen = fm.stringWidth( m_xtitle );
            g2d.drawString( m_xtitle, 5 * m_plotAreaWidth / 10 - (slen/2), ( 9 * m_plotAreaHeight / 10 ) + 25 );
        }
    }
       
    //  Attributes 
    JFrame m_parent;
        
    Color m_frameColor = Color.black;
    Color m_backgroundColor = Color.white;
    
    protected int m_plotAreaWidth;
    protected int m_plotAreaHeight;

    protected Vector<Curve> m_curves = new Vector<>( );
    int m_curveSize;
    
    protected XYPlotRange m_xyAutoPlotRange;
    protected XYPlotRange m_xyForcedPlotRange;
    
    protected String m_title = "no title";
    
    // Popup menu : derived classes should be able to extend the popup menu
    protected JPopupMenu m_popupmenu;
    protected JMenu m_xyScaleMenu;
    protected JRadioButtonMenuItem m_XYPlotRangeSelectorMenuItem;
    protected JRadioButtonMenuItem m_XYDefaultRangeMenuItem;
    protected XYPlotRangeSelector m_plotRangeSelector;
    private final ButtonGroup m_XYRangeButtongroup;
    
    protected StatusPanel m_statusPanel;
    
    // needed to indicate whether horizontal axis should be displayed reverse
    protected boolean m_displayXReversed = false;
    
    protected LinkedList<String> m_leftUpperInfoDisplayLines = new LinkedList<>( );
    protected LinkedList<String> m_leftLowerInfoDisplayLines = new LinkedList<>( );
    
    protected LinkedList<XYPlotSlider> m_plotSliderList = new LinkedList<>( );
    protected boolean m_sliderIsSelected = false;
    
    private String m_infoString      = "";
    private Color  m_infoStringColor = Color.BLACK;

    private boolean m_useNiceProtonDefaults = false;

    // For proper serialization
    private static final long serialVersionUID = 34874958988126393L;
    
    // new attributes for drawing (normal) ranges
    private Curve m_minCurve;
    private Curve m_maxCurve;
    private Curve m_meanCurve;

    private Color m_rangeColor;
    
    private String m_xtitle = null;
    private String m_ytitle = null;
    
    private Vector<Point2D.Double> m_splineNodes;
    
    // Novel settable properties
    private int m_textFontSize = 20;
    private Color m_backgroundFillColor = Color.LIGHT_GRAY;
}